@@ -15,14 +15,22 @@ limitations under the License.
 
 #include "tensorflow/cc/saved_model/loader.h"
 
+#include <string>
 #include <unordered_set>
 
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
 #include "tensorflow/cc/saved_model/constants.h"
+#include "tensorflow/cc/saved_model/fingerprinting.h"
 #include "tensorflow/cc/saved_model/loader_util.h"
+#include "tensorflow/cc/saved_model/metrics.h"
 #include "tensorflow/cc/saved_model/reader.h"
+#include "tensorflow/cc/saved_model/util.h"
 #include "tensorflow/core/framework/attr_value.pb.h"
 #include "tensorflow/core/framework/function.pb.h"
+#include "tensorflow/core/framework/graph_debug_info.pb.h"
 #include "tensorflow/core/framework/node_def.pb.h"
+#include "tensorflow/core/framework/op_def.pb.h"
 #include "tensorflow/core/framework/tensor.pb.h"
 #include "tensorflow/core/lib/io/path.h"
 #include "tensorflow/core/lib/monitoring/counter.h"
@@ -31,7 +39,9 @@ limitations under the License.
 #include "tensorflow/core/lib/strings/strcat.h"
 #include "tensorflow/core/platform/env.h"
 #include "tensorflow/core/platform/errors.h"
-#include "tensorflow/core/protobuf/graph_debug_info.pb.h"
+#include "tensorflow/core/platform/file_system_helper.h"
+#include "tensorflow/core/platform/statusor.h"
+#include "tensorflow/core/protobuf/meta_graph.pb.h"
 #include "tensorflow/core/protobuf/saver.pb.h"
 #include "tensorflow/core/public/session.h"
 #include "tensorflow/core/public/session_options.h"
@@ -57,11 +67,13 @@ auto* load_latency_by_stage = monitoring::Sampler<2>::New(
         "model_path",
         "stage",
     },
-    // Scale of 10, power of 1.8 with bucket count 33 (~20 minutes).
-    monitoring::Buckets::Exponential(10, 1.8, 33));
+    // Scale of 10, power of 1.8 with bucket count 37 (~258 minutes).
+    monitoring::Buckets::Exponential(10, 1.8, 37));
 
 constexpr char kLoadAttemptFail[] = "fail";
 constexpr char kLoadAttemptSuccess[] = "success";
+// `tensorflow::LoadSavedModel` API label.
+constexpr char kCCLoadLabel[] = "cc_load";
 
 uint64 GetLatencyMicroseconds(const uint64 start_microseconds) {
   const uint64 end_microseconds = EnvTime::NowMicros();
@@ -80,18 +92,31 @@ static Status ValidateNode(const NodeDef& node) {
     if (node_value.has_tensor()) {
       const PartialTensorShape node_shape(node_value.tensor().tensor_shape());
       if (node_shape.num_elements() < 0) {
-        return errors::FailedPrecondition(
+        return absl::FailedPreconditionError(absl::StrCat(
             "Saved model contains node \"", node.name(), "\" (op \"", node.op(),
             "\") which initializes from a tensor with ",
-            node_shape.num_elements(), " elements");
+            node_shape.num_elements(), " elements"));
       }
     }
   } else if (node.op() == "Const") {
-    return errors::FailedPrecondition(
+    return absl::FailedPreconditionError(absl::StrCat(
         "Saved model contains node \"", node.name(),
-        "\" which is a constant tensor but no value has been provided");
+        "\" which is a constant tensor but no value has been provided"));
   }
-  return Status::OK();
+  return OkStatus();
+}
+
+static Status ValidateFunctionNotRecursive(const FunctionDef& function) {
+  const auto& function_name = function.signature().name();
+  for (const auto& node : function.node_def()) {
+    if (node.op() == function_name) {
+      return absl::FailedPreconditionError(absl::StrCat(
+          "Function ", function_name,
+          " is self recursive and TensorFlow does not support this scenario."));
+    }
+  }
+
+  return OkStatus();
 }
 
 static Status ValidateSavedTensors(const GraphDef& graph_def) {
@@ -103,22 +128,15 @@ static Status ValidateSavedTensors(const GraphDef& graph_def) {
     const FunctionDefLibrary& library = graph_def.library();
     for (const auto& function : library.function()) {
       for (const auto& node : function.node_def()) {
-	TF_RETURN_IF_ERROR(ValidateNode(node));
+        TF_RETURN_IF_ERROR(ValidateNode(node));
       }
+
+      // Also check that there is no recursivity in the library
+      TF_RETURN_IF_ERROR(ValidateFunctionNotRecursive(function));
     }
   }
 
-  return Status::OK();
-}
-
-Status LoadMetaGraphIntoSession(const MetaGraphDef& meta_graph_def,
-                                const SessionOptions& session_options,
-                                std::unique_ptr<Session>* session) {
-  Session* session_p = nullptr;
-  TF_RETURN_IF_ERROR(NewSession(session_options, &session_p));
-  session->reset(session_p);
-  TF_RETURN_IF_ERROR(ValidateSavedTensors(meta_graph_def.graph_def()));
-  return (*session)->Create(meta_graph_def.graph_def());
+  return OkStatus();
 }
 
 Tensor CreateStringTensor(const string& value) {
@@ -204,7 +222,7 @@ Status RunInitOp(const RunOptions& run_options, const string& export_dir,
     return RunOnce(run_options, inputs, {}, {init_op_name},
                    nullptr /* outputs */, &run_metadata, session);
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status RunRestore(const RunOptions& run_options, const string& export_dir,
@@ -221,11 +239,14 @@ Status RunRestore(const RunOptions& run_options, const string& export_dir,
   // variables are stored in the variables.data-?????-of-????? files.
   const string variables_index_path = io::JoinPath(
       variables_directory, MetaFilename(kSavedModelVariablesFilename));
-  if (!Env::Default()->FileExists(variables_index_path).ok()) {
+  TF_ASSIGN_OR_RETURN(
+      bool variables_index_exists,
+      internal::FileExists(Env::Default(), variables_index_path));
+  if (!variables_index_exists) {
     LOG(INFO) << "The specified SavedModel has no variables; no checkpoints "
                  "were restored. File does not exist: "
               << variables_index_path;
-    return Status::OK();
+    return OkStatus();
   }
   const string variables_path =
       io::JoinPath(variables_directory, kSavedModelVariablesFilename);
@@ -244,22 +265,18 @@ Status RunRestore(const RunOptions& run_options, const string& export_dir,
                  nullptr /* outputs */, &run_metadata, session);
 }
 
-Status ReadSavedModelDebugInfoIfPresent(
-    const string& export_dir,
-    std::unique_ptr<GraphDebugInfo>* debug_info_proto) {
-  LOG(INFO) << "Reading SavedModel debug info (if present) from: "
-            << export_dir;
+}  // namespace
 
-  const string debug_info_pb_path =
-      io::JoinPath(export_dir, "debug", "saved_model_debug_info.pb");
-  if (Env::Default()->FileExists(debug_info_pb_path).ok()) {
-    GraphDebugInfo debug_info;
-    TF_RETURN_IF_ERROR(
-        ReadBinaryProto(Env::Default(), debug_info_pb_path, &debug_info));
-    *debug_info_proto =
-        absl::make_unique<GraphDebugInfo>(std::move(debug_info));
-  }
-  return Status::OK();
+SavedModelBundleInterface::~SavedModelBundleInterface() {}
+
+Status LoadMetagraphIntoSession(const SessionOptions& session_options,
+                                const MetaGraphDef& meta_graph,
+                                std::unique_ptr<Session>* session) {
+  Session* session_p = nullptr;
+  TF_RETURN_IF_ERROR(NewSession(session_options, &session_p));
+  session->reset(session_p);
+  TF_RETURN_IF_ERROR(ValidateSavedTensors(meta_graph.graph_def()));
+  return (*session)->Create(meta_graph.graph_def());
 }
 
 Status LoadSavedModelInternal(const SessionOptions& session_options,
@@ -267,50 +284,30 @@ Status LoadSavedModelInternal(const SessionOptions& session_options,
                               const string& export_dir,
                               const std::unordered_set<string>& tags,
                               SavedModelBundle* const bundle) {
-  const uint64 read_start_microseconds = Env::Default()->NowMicros();
   TF_RETURN_IF_ERROR(ReadMetaGraphDefFromSavedModel(export_dir, tags,
                                                     &bundle->meta_graph_def));
   TF_RETURN_IF_ERROR(
       ReadSavedModelDebugInfoIfPresent(export_dir, &bundle->debug_info));
-  TF_RETURN_IF_ERROR(LoadMetaGraphIntoSession(
-      bundle->meta_graph_def, session_options, &bundle->session));
-
-  std::vector<AssetFileDef> asset_file_defs;
-  TF_RETURN_IF_ERROR(
-      internal::GetAssetFileDefs(bundle->meta_graph_def, &asset_file_defs));
-  TF_RETURN_IF_ERROR(
-      RunRestore(run_options, export_dir,
-                 bundle->meta_graph_def.saver_def().restore_op_name(),
-                 bundle->meta_graph_def.saver_def().filename_tensor_name(),
-                 asset_file_defs, bundle->session.get()));
-  // Record walltime spent in restoring graph from disk, but postpone metric
-  // increments until graph init finishes.
-  const uint64 restore_graph_walltime =
-      GetLatencyMicroseconds(read_start_microseconds);
-
-  const uint64 graph_init_start_microseconds = Env::Default()->NowMicros();
-  string init_op_name;
-  TF_RETURN_IF_ERROR(
-      internal::GetInitOp(export_dir, bundle->meta_graph_def, &init_op_name));
-  TF_RETURN_IF_ERROR(RunInitOp(run_options, export_dir, bundle->meta_graph_def,
-                               asset_file_defs, bundle->session.get(),
-                               init_op_name));
-  load_latency_by_stage->GetCell(export_dir, "restore_graph")
-      ->Add(restore_graph_walltime);
-  // Record wall time spent in init op.
-  load_latency_by_stage->GetCell(export_dir, "init_graph")
-      ->Add(GetLatencyMicroseconds(graph_init_start_microseconds));
-  return Status::OK();
+  TF_RETURN_IF_ERROR(LoadMetagraphIntoSession(
+      session_options, bundle->meta_graph_def, &bundle->session));
+  TF_RETURN_IF_ERROR(RestoreSession(run_options, bundle->meta_graph_def,
+                                    export_dir, &bundle->session));
+  return OkStatus();
 }
 
-}  // namespace
-
-SavedModelBundleInterface::~SavedModelBundleInterface() {}
-
 Status LoadSavedModel(const SessionOptions& session_options,
                       const RunOptions& run_options, const string& export_dir,
                       const std::unordered_set<string>& tags,
                       SavedModelBundle* const bundle) {
+  metrics::SavedModelReadApi(kCCLoadLabel).IncrementBy(1);
+  auto fingerprint_proto =
+      saved_model::fingerprinting::ReadSavedModelFingerprint(export_dir);
+  if (fingerprint_proto.ok()) {
+    // Set gauge cell with saved_model_checksum.
+    metrics::SavedModelReadFingerprint().Set(
+        std::to_string(fingerprint_proto->saved_model_checksum()));
+  }
+
   // TODO(robson): Add tests for the counters.
   const uint64 start_microseconds = Env::Default()->NowMicros();
   const Status status = LoadSavedModelInternal(session_options, run_options,
@@ -323,6 +320,7 @@ Status LoadSavedModel(const SessionOptions& session_options,
   };
   if (status.ok()) {
     log_and_count(kLoadAttemptSuccess);
+    metrics::SavedModelReadPath().Set(export_dir);
   } else {
     log_and_count(kLoadAttemptFail);
   }
@@ -344,17 +342,17 @@ class LiteSessionWrapper : public Session {
       : wrapped_(std::move(wrapped)) {}
 
   Status Create(const GraphDef& graph) override {
-    return errors::Unimplemented("Session::Create()");
+    return absl::UnimplementedError("Session::Create()");
   }
   Status Create(GraphDef&& graph) override {
-    return errors::Unimplemented("Session::Create()");
+    return absl::UnimplementedError("Session::Create()");
   }
 
   Status Extend(const GraphDef& graph) override {
-    return errors::Unimplemented("Session::Extend()");
+    return absl::UnimplementedError("Session::Extend()");
   }
   Status Extend(GraphDef&& graph) override {
-    return errors::Unimplemented("Session::Extend()");
+    return absl::UnimplementedError("Session::Extend()");
   }
 
   Status Run(const std::vector<std::pair<string, Tensor>>& inputs,
@@ -366,16 +364,16 @@ class LiteSessionWrapper : public Session {
   }
 
   Status Create(const RunOptions& run_options, const GraphDef& graph) override {
-    return errors::Unimplemented("Session::Create()");
+    return absl::UnimplementedError("Session::Create()");
   }
   Status Extend(const RunOptions& run_options, const GraphDef& graph) override {
-    return errors::Unimplemented("Session::Extend()");
+    return absl::UnimplementedError("Session::Extend()");
   }
   Status Create(const RunOptions& run_options, GraphDef&& graph) override {
-    return errors::Unimplemented("Session::Create()");
+    return absl::UnimplementedError("Session::Create()");
   }
   Status Extend(const RunOptions& run_options, GraphDef&& graph) override {
-    return errors::Unimplemented("Session::Extend()");
+    return absl::UnimplementedError("Session::Extend()");
   }
   Status Close(const RunOptions& run_options) override {
     return wrapped_->Close(run_options);
@@ -394,14 +392,14 @@ class LiteSessionWrapper : public Session {
                    const std::vector<string>& output_names,
                    const std::vector<string>& target_nodes,
                    string* handle) override {
-    return errors::Unimplemented("Session::PRunSetup()");
+    return absl::UnimplementedError("Session::PRunSetup()");
   }
 
   Status PRun(const string& handle,
               const std::vector<std::pair<string, Tensor>>& inputs,
               const std::vector<string>& output_names,
               std::vector<Tensor>* outputs) override {
-    return errors::Unimplemented("Session::PRun()");
+    return absl::UnimplementedError("Session::PRun()");
   }
 
   Status ListDevices(std::vector<DeviceAttributes>* response) override {
@@ -435,11 +433,44 @@ class LiteSessionWrapper : public Session {
     return wrapped_->ReleaseCallable(handle);
   }
 
+  Status Finalize() override { return wrapped_->Finalize(); }
+
  private:
   const std::unique_ptr<Session> wrapped_;
 };
 }  // namespace
 
+Status RestoreSession(const RunOptions& run_options,
+                      const MetaGraphDef& meta_graph, const string& export_dir,
+                      std::unique_ptr<Session>* session) {
+  const uint64 read_start_microseconds = Env::Default()->NowMicros();
+  std::vector<AssetFileDef> asset_file_defs;
+  TF_RETURN_IF_ERROR(internal::GetAssetFileDefs(meta_graph, &asset_file_defs));
+  if (meta_graph.has_saver_def()) {
+    TF_RETURN_IF_ERROR(RunRestore(run_options, export_dir,
+                                  meta_graph.saver_def().restore_op_name(),
+                                  meta_graph.saver_def().filename_tensor_name(),
+                                  asset_file_defs, session->get()));
+  }
+  // Record walltime spent in restoring graph from disk, but postpone metric
+  // increments until graph init finishes.
+  const uint64 restore_graph_walltime =
+      GetLatencyMicroseconds(read_start_microseconds);
+
+  const uint64 graph_init_start_microseconds = Env::Default()->NowMicros();
+  string init_op_name;
+  TF_RETURN_IF_ERROR(
+      internal::GetInitOp(export_dir, meta_graph, &init_op_name));
+  TF_RETURN_IF_ERROR(RunInitOp(run_options, export_dir, meta_graph,
+                               asset_file_defs, session->get(), init_op_name));
+  load_latency_by_stage->GetCell(export_dir, "restore_graph")
+      ->Add(restore_graph_walltime);
+  // Record wall time spent in init op.
+  load_latency_by_stage->GetCell(export_dir, "init_graph")
+      ->Add(GetLatencyMicroseconds(graph_init_start_microseconds));
+  return OkStatus();
+}
+
 Status LoadSavedModel(const SessionOptions& session_options,
                       const RunOptions& run_options, const string& export_dir,
                       const std::unordered_set<string>& tags,
@@ -462,15 +493,18 @@ Status LoadSavedModel(const SessionOptions& session_options,
   *bundle = SavedModelBundleLite(
       absl::make_unique<LiteSessionWrapper>(std::move(legacy_bundle.session)),
       std::move(*legacy_bundle.meta_graph_def.mutable_signature_def()));
-  return Status::OK();
+  return OkStatus();
 }
 
 bool MaybeSavedModelDirectory(const string& export_dir) {
   const string saved_model_pb_path =
       io::JoinPath(export_dir, kSavedModelFilenamePb);
+  const string saved_model_cpb_path =
+      io::JoinPath(export_dir, kSavedModelFilenameCpb);
   const string saved_model_pbtxt_path =
       io::JoinPath(export_dir, kSavedModelFilenamePbTxt);
   return Env::Default()->FileExists(saved_model_pb_path).ok() ||
+         Env::Default()->FileExists(saved_model_cpb_path).ok() ||
          Env::Default()->FileExists(saved_model_pbtxt_path).ok();
 }
 
