@@ -15,10 +15,6 @@
 
 """Tests for special math operations."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
 
 from absl import flags
@@ -26,7 +22,6 @@ from absl.testing import parameterized
 
 import numpy as np
 import scipy.special as sps
-import six
 
 from tensorflow.compiler.tests import xla_test
 from tensorflow.python.eager import def_function
@@ -44,16 +39,26 @@ flags.DEFINE_bool('vary_seed', False,
 NUM_SAMPLES = int(1e3)
 
 
-@def_function.function(experimental_compile=True)
+@def_function.function(jit_compile=True)
 def _igamma(a, x):
   return math_ops.igamma(a, x)
 
 
-@def_function.function(experimental_compile=True)
+@def_function.function(jit_compile=True)
 def _igammac(a, x):
   return math_ops.igammac(a, x)
 
 
+@def_function.function(jit_compile=True)
+def _polygamma(n, x):
+  return math_ops.polygamma(n, x)
+
+
+@def_function.function(jit_compile=True)
+def _zeta(a, q):
+  return math_ops.zeta(a, q)
+
+
 # This is df/da / df/dx, where f = igamma.
 def implicit_reparameterization_grad(a, x):
   log_prob = math_ops.xlogy(a - 1., x) - math_ops.lgamma(a) - x
@@ -61,7 +66,7 @@ def implicit_reparameterization_grad(a, x):
   return -gen_math_ops.igamma_grad_a(a, x) / prob
 
 
-@def_function.function(experimental_compile=True)
+@def_function.function(jit_compile=True)
 def _log1p(x):
   return math_ops.log1p(x)
 
@@ -71,10 +76,7 @@ class Log1pTest(xla_test.XLATestCase, parameterized.TestCase):
   def setUp(self):
     if flags.FLAGS.vary_seed:
       entropy = os.urandom(64)
-      if six.PY2:
-        answer = int(entropy.encode('hex'), 16)
-      else:
-        answer = int.from_bytes(entropy, 'big')
+      answer = int.from_bytes(entropy, 'big')
       np.random.seed(answer % (2**32 - 1))
     super(Log1pTest, self).setUp()
 
@@ -136,15 +138,220 @@ class Log1pTest(xla_test.XLATestCase, parameterized.TestCase):
     self._test_range(0., 3., dtype, rtol, atol, is_negative=False)
 
 
+class ZetaTest(xla_test.XLATestCase, parameterized.TestCase):
+
+  def setUp(self):
+    if flags.FLAGS.vary_seed:
+      entropy = os.urandom(64)
+      answer = int.from_bytes(entropy, 'big')
+      np.random.seed(answer % (2**32 - 1))
+    super(ZetaTest, self).setUp()
+
+  def adjust_tolerance_for_tpu(self, dtype, rtol, atol):
+    if self.device not in ['TPU']:
+      return rtol, atol
+
+    if dtype == np.float32:
+      return 2e-2, 1e-7
+    return 2e-4, 1e-20
+
+  def testBadValues(self):
+    q = np.random.uniform(low=0.3, high=20., size=[10])
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta(np.float64(1.), q)
+      actual = sess.run(y)
+    # When x == 1, this is the Harmonic series.
+    self.assertTrue(np.all(np.isinf(actual)))
+
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta(np.float64(0.1), q)
+      actual = sess.run(y)
+    # When x < 1, this is undefined.
+    self.assertTrue(np.all(np.isnan(actual)))
+
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta([1.1, 1.2, 2.1, 2.2, 3.1], [-2.0, -1.1, -1.0, -0.5, -0.1])
+      actual = sess.run(y)
+    # For q <= 0, x must be an integer.
+    self.assertTrue(np.all(np.isnan(actual)))
+
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta([2.0, 4.0, 6.0], [0.0, -1.0, -2.0])
+      actual = sess.run(y)
+    # For integer q <= 0, zeta has poles with a defined limit of +inf where x is
+    # an even integer.
+    self.assertTrue(np.all(np.isinf(actual)))
+
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta([3.0, 5.0, 7.0], [0.0, -1.0, -2.0])
+      actual = sess.run(y)
+    # For non-positive integer q, zeta has poles with an undefined limit where x
+    # is an odd integer.
+    self.assertTrue(np.all(np.isnan(actual)))
+
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta([1.1, 2.2, 3.3], [-1.1, -1.0, 0.0])
+      actual = sess.run(y)
+    # For non-positive q, zeta is not defined if x is not an integer.
+    self.assertTrue(np.all(np.isnan(actual)))
+
+  @parameterized.parameters((np.float32, 1e-2, 1e-11),
+                            (np.float64, 1e-4, 1e-30))
+  def testLargeXSmallQ(self, dtype, rtol, atol):
+    rtol, atol = self.adjust_tolerance_for_tpu(dtype, rtol, atol)
+    if self.device not in ['XLA_GPU', 'XLA_CPU'] and dtype == np.float64:
+      # TODO(b/165739664): Figure out why on TPU F64 Zeta sometimes returns
+      # infs.
+      self.skipTest(
+          'Skipping test because some F64 operations are numerically '
+          'unstable on TPU.')
+
+    x = np.random.uniform(low=100., high=200., size=[NUM_SAMPLES]).astype(dtype)
+    q = np.random.uniform(low=0.3, high=1., size=[NUM_SAMPLES]).astype(dtype)
+
+    expected_values = sps.zeta(x, q)
+    with self.session() as sess:
+      with self.test_scope():
+        y = _zeta(x, q)
+      actual = sess.run(y)
+
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+  @parameterized.parameters((np.float32, 1e-2, 1e-11),
+                            (np.float64, 1e-4, 1e-30))
+  def testSmallValues(self, dtype, rtol, atol):
+    rtol, atol = self.adjust_tolerance_for_tpu(dtype, rtol, atol)
+    # Test values near zero.
+    x = np.random.uniform(low=1.1, high=10., size=[NUM_SAMPLES]).astype(dtype)
+    q = np.random.uniform(
+        low=np.finfo(dtype).tiny, high=1., size=[NUM_SAMPLES]).astype(dtype)
+
+    expected_values = sps.zeta(x, q)
+    with self.session() as sess:
+      with self.test_scope():
+        actual = sess.run(_zeta(x, q))
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+  @parameterized.parameters((np.float32, 1e-2, 1e-11),
+                            (np.float64, 1e-4, 1e-30))
+  def testMediumValues(self, dtype, rtol, atol):
+    rtol, atol = self.adjust_tolerance_for_tpu(dtype, rtol, atol)
+    x = np.random.uniform(low=1.1, high=100., size=[NUM_SAMPLES]).astype(dtype)
+    q = np.random.uniform(low=1., high=1e1, size=[NUM_SAMPLES]).astype(dtype)
+
+    expected_values = sps.zeta(x, q)
+    with self.session() as sess:
+      with self.test_scope():
+        actual = sess.run(_zeta(x, q))
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+  @parameterized.parameters((np.float32, 2e-2, 1e-5), (np.float64, 1e-4, 1e-30))
+  def testLargeValues(self, dtype, rtol, atol):
+    x = np.random.uniform(
+        low=100., high=int(1e3), size=[NUM_SAMPLES]).astype(dtype)
+    q = np.random.uniform(
+        low=1., high=int(1e1), size=[NUM_SAMPLES]).astype(dtype)
+
+    expected_values = sps.zeta(x, q)
+    with self.session() as sess:
+      with self.test_scope():
+        actual = sess.run(_zeta(x, q))
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+
+class PolygammaTest(xla_test.XLATestCase, parameterized.TestCase):
+
+  def setUp(self):
+    if flags.FLAGS.vary_seed:
+      entropy = os.urandom(64)
+      answer = int.from_bytes(entropy, 'big')
+      np.random.seed(answer % (2**32 - 1))
+    super(PolygammaTest, self).setUp()
+
+  def adjust_tolerance_for_tpu(self, dtype, rtol, atol):
+    if self.device not in ['TPU']:
+      return rtol, atol
+
+    if dtype == np.float32:
+      return 2e-2, 1e-7
+    return 2e-4, 1e-20
+
+  def testBadValues(self):
+    x = np.random.uniform(low=0.3, high=20., size=[10])
+    with self.session() as sess:
+      with self.test_scope():
+        y = _polygamma(np.float64(-1.), x)
+      actual = sess.run(y)
+    # Not defined for negative numbers.
+    self.assertTrue(np.all(np.isnan(actual)))
+
+    with self.session() as sess:
+      with self.test_scope():
+        y = _polygamma(np.float64(0.1), x)
+      actual = sess.run(y)
+    # Not defined for non-integers.
+    self.assertTrue(np.all(np.isnan(actual)))
+
+  @parameterized.parameters((np.float32, 1e-2, 1e-11),
+                            (np.float64, 1e-4, 1e-30))
+  def testRecoverDigamma(self, dtype, rtol, atol):
+    rtol, atol = self.adjust_tolerance_for_tpu(dtype, rtol, atol)
+    if self.device not in ['XLA_GPU', 'XLA_CPU'] and dtype == np.float64:
+      self.skipTest(
+          'Skipping test because some F64 operations are '
+          'numerically unstable on TPU.'
+      )
+
+    x = np.random.uniform(low=0.1, high=50., size=[NUM_SAMPLES]).astype(dtype)
+    expected_values = sps.digamma(x)
+    with self.session() as sess:
+      with self.test_scope():
+        y = _polygamma(dtype(0.), x)
+      actual = sess.run(y)
+
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+  @parameterized.parameters((np.float32, 1e-2, 1e-11),
+                            (np.float64, 1e-4, 1e-30))
+  def testSmallN(self, dtype, rtol, atol):
+    rtol, atol = self.adjust_tolerance_for_tpu(dtype, rtol, atol)
+    # Test values near zero.
+    n = np.random.randint(low=1, high=5, size=[NUM_SAMPLES]).astype(dtype)
+    x = np.random.uniform(
+        low=np.finfo(dtype).tiny, high=1., size=[NUM_SAMPLES]).astype(dtype)
+
+    expected_values = sps.polygamma(n, x)
+    with self.session() as sess:
+      with self.test_scope():
+        actual = sess.run(_polygamma(n, x))
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+  @parameterized.parameters((np.float32, 1e-2, 1e-11),
+                            (np.float64, 1e-4, 1e-30))
+  def testMediumLargeN(self, dtype, rtol, atol):
+    rtol, atol = self.adjust_tolerance_for_tpu(dtype, rtol, atol)
+    n = np.random.randint(low=5, high=10, size=[NUM_SAMPLES]).astype(dtype)
+    x = np.random.uniform(low=1., high=1e1, size=[NUM_SAMPLES]).astype(dtype)
+
+    expected_values = sps.polygamma(n, x)
+    with self.session() as sess:
+      with self.test_scope():
+        actual = sess.run(_polygamma(n, x))
+    self.assertAllClose(expected_values, actual, atol=atol, rtol=rtol)
+
+
 class IgammaTest(xla_test.XLATestCase, parameterized.TestCase):
 
   def setUp(self):
     if flags.FLAGS.vary_seed:
       entropy = os.urandom(64)
-      if six.PY2:
-        answer = int(entropy.encode('hex'), 16)
-      else:
-        answer = int.from_bytes(entropy, 'big')
+      answer = int.from_bytes(entropy, 'big')
       np.random.seed(answer % (2**32 - 1))
     super(IgammaTest, self).setUp()
 
@@ -326,10 +533,7 @@ class IgammacTest(xla_test.XLATestCase, parameterized.TestCase):
   def setUp(self):
     if flags.FLAGS.vary_seed:
       entropy = os.urandom(64)
-      if six.PY2:
-        answer = int(entropy.encode('hex'), 16)
-      else:
-        answer = int.from_bytes(entropy, 'big')
+      answer = int.from_bytes(entropy, 'big')
       np.random.seed(answer % (2**32 - 1))
     super(IgammacTest, self).setUp()
 
