@@ -26,8 +26,7 @@ limitations under the License.
 #include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/public/session.h"
 
-#if GOOGLE_CUDA
-#if GOOGLE_TENSORRT
+#if GOOGLE_CUDA && GOOGLE_TENSORRT
 
 namespace tensorflow {
 namespace tensorrt {
@@ -40,7 +39,7 @@ class SegmentTest : public ::testing::Test {
       const std::set<string>& node_names) {
     return [node_names](const Node* node) -> Status {
       if (node_names.find(node->name()) != node_names.end()) {
-        return Status::OK();
+        return OkStatus();
       }
       return errors::NotFound("Not a user specified candidate");
     };
@@ -66,7 +65,7 @@ class SegmentTest : public ::testing::Test {
                const std::set<string>& input_candidates,
                const std::set<string>& output_candidates,
                const std::vector<std::set<string>>& expected_segments) {
-    SegmentNodesVector segments;
+    SegmentVector segments;
     TF_EXPECT_OK(SegmentGraph(graph, graph_properties,
                               MakeCandidateFn(candidates),
                               MakeInputEdgeCandidateFn(input_candidates),
@@ -83,12 +82,12 @@ class SegmentTest : public ::testing::Test {
             expected_segments);
   }
 
-  void ValidateSegment(const SegmentNodesVector& segments,
+  void ValidateSegment(const SegmentVector& segments,
                        const std::vector<std::set<string>>& expected_segments) {
     EXPECT_EQ(expected_segments.size(), segments.size());
     for (int i = 0; i < segments.size(); ++i) {
       std::set<string> segment_node_names;
-      for (const Node* node : segments[i]) {
+      for (const Node* node : segments[i].nodes) {
         segment_node_names.insert(node->name());
       }
       const auto& expected = expected_segments[i];
@@ -109,8 +108,9 @@ class SegmentTest : public ::testing::Test {
     segment_options_.allow_dynamic_non_batch_dim = true;
   }
 
-  void EnableImplicitBatchModeForStaticEngine() {
+  void EnableImplicitBatchModeForStaticEngine(int maximum_batch_size = 1000) {
     segment_options_.use_implicit_batch = true;
+    segment_options_.maximum_batch_size = maximum_batch_size;
     segment_options_.allow_dynamic_non_batch_dim = false;
   }
 
@@ -180,6 +180,69 @@ TEST_F(SegmentTest, Simple) {
   RunTest(&g, all_adds, all_adds, without_add3, {all_adds});
 }
 
+TEST_F(SegmentTest, WithDeviceAssignments) {
+  //           feed
+  //          //  \\
+  //       add0    add1
+  //        | \    /
+  //        |  add2
+  //        | /   \\
+  //       add3    add4
+  //          \    /
+  //          <sink>
+  Scope s = Scope::NewRootScope();
+  auto feed = ops::Placeholder(s.WithOpName("feed"), DT_FLOAT);
+  auto add0 = ops::Add(s.WithOpName("add0"), feed, feed);
+  auto add1 = ops::Add(s.WithOpName("add1"), feed, feed);
+  auto add2 = ops::Add(s.WithOpName("add2"), add0, add1);
+  auto add3 = ops::Add(s.WithOpName("add3"), add0, add2);
+  auto add4 = ops::Add(s.WithOpName("add4"), add2, add2);
+
+  const std::set<string> all_adds = {"add0", "add1", "add2", "add3", "add4"};
+  DisableImplicitBatchMode();
+
+  {
+    Graph g(OpRegistry::Global());
+    TF_EXPECT_OK(s.ToGraph(&g));
+    RunTest(&g, all_adds, all_adds, all_adds, {all_adds});
+  }
+
+  {
+    // Assigning add1 to CPU to exclude it from the cluster.
+    add1.node()->set_assigned_device_name("/device:CPU:0");
+    Graph g(OpRegistry::Global());
+    TF_EXPECT_OK(s.ToGraph(&g));
+    RunTest(&g, all_adds, all_adds, all_adds, {all_adds - "add1"});
+    add1.node()->set_assigned_device_name("");
+  }
+
+  {
+    // Assigning operations add3 and add4 to another GPU to exclude the
+    // operation from the cluster.
+    constexpr char kGpu0[] = "/device:GPU:0";
+    add0.node()->set_assigned_device_name(kGpu0);
+    add1.node()->set_assigned_device_name(kGpu0);
+    add2.node()->set_assigned_device_name(kGpu0);
+    constexpr char kGpu1[] = "/device:GPU:1";
+    add3.node()->set_assigned_device_name(kGpu1);
+    add4.node()->set_assigned_device_name(kGpu1);
+    Graph g(OpRegistry::Global());
+    TF_EXPECT_OK(s.ToGraph(&g));
+    RunTest(&g, all_adds, all_adds, all_adds, {{"add0", "add1", "add2"}});
+  }
+
+  {
+    // Assigning the operations to two compatibile GPU devices resulting in
+    // one cluster with all operations.
+    constexpr char kGpuAny[] = "/device:GPU:*";
+    add3.node()->set_assigned_device_name(kGpuAny);
+    add4.node()->set_assigned_device_name(kGpuAny);
+    Graph g(OpRegistry::Global());
+    TF_EXPECT_OK(s.ToGraph(&g));
+    RunTest(&g, all_adds, all_adds, all_adds, {all_adds});
+  }
+}
+
 TEST_F(SegmentTest, AvoidCycle) {
   //           feed
   //          //  \\
@@ -425,7 +488,12 @@ TEST_F(SegmentTest, TwoChainsDiffBatchSizes) {
   const std::set<string> all_nodes = {"const-scalar", "output-0", "output-1"};
   EnableImplicitBatchModeForStaticEngine();
   RunTest(&g, &static_graph_properties, all_nodes, all_nodes, all_nodes,
-          {{"output-0", "const-scalar"}});
+          /*expected_segments=*/{{"output-0", "const-scalar"}});
+
+  // Converter will create engines based on the static batch size
+  EnableImplicitBatchModeForStaticEngine(1);
+  RunTest(&g, &static_graph_properties, all_nodes, all_nodes, all_nodes,
+          /*expected_segments=*/{{"output-0", "const-scalar"}});
 }
 
 TEST_F(SegmentTest, SameRankImplicitBroadcastingStaticBatchSize) {
@@ -522,5 +590,4 @@ TEST_F(SegmentTest, IncompatibleBatchSizes) {
 }  // namespace tensorrt
 }  // namespace tensorflow
 
-#endif  // GOOGLE_TENSORRT
-#endif  // GOOGLE_CUDA
+#endif  // GOOGLE_CUDA && GOOGLE_TENSORRT
