@@ -16,6 +16,9 @@ limitations under the License.
 #include "tensorflow/core/kernels/stateful_random_ops.h"
 
 #include <cmath>
+#include <functional>
+#include <tuple>
+#include <utility>
 
 #include "tensorflow/compiler/tf2xla/kernels/random_ops_util.h"
 #include "tensorflow/compiler/tf2xla/lib/random.h"
@@ -30,39 +33,42 @@ limitations under the License.
 #include "tensorflow/compiler/xla/client/xla_builder.h"
 #include "tensorflow/compiler/xla/xla_data.pb.h"
 #include "tensorflow/core/framework/op_kernel.h"
+#include "tensorflow/core/framework/rng_alg.h"
 #include "tensorflow/core/framework/tensor.h"
 #include "tensorflow/core/framework/tensor_shape.h"
 #include "tensorflow/core/lib/math/math_util.h"
+#include "tensorflow/core/platform/statusor.h"
 
 namespace tensorflow {
 namespace {
 
-xla::BitGeneratorTy BitGen(Algorithm alg) {
-  if (alg == RNG_ALG_PHILOX) {
-    return [=](xla::XlaOp key, xla::XlaOp state, const xla::Shape& shape) {
-      state =
-          xla::ConcatInDim(key.builder(), {xla::Reshape(key, {1}), state}, 0);
-      xla::XlaOp result =
-          xla::RngBitGenerator(xla::RandomAlgorithm::RNG_PHILOX, state, shape);
-      xla::XlaOp data = xla::GetTupleElement(result, 1);
-      xla::XlaOp new_state =
-          xla::Slice(xla::GetTupleElement(result, 0), {1}, {3}, {1});
-      return xla::RngOutput{data, new_state};
-    };
-  } else {
-    return [=](xla::XlaOp key, xla::XlaOp state, const xla::Shape& shape) {
-      state = xla::ConcatScalars(key.builder(), {key, state});
-      xla::XlaOp result = xla::RngBitGenerator(
-          xla::RandomAlgorithm::RNG_THREE_FRY, state, shape);
-      xla::XlaOp data = xla::GetTupleElement(result, 1);
-      xla::XlaOp new_state = xla::Reshape(
-          xla::Slice(xla::GetTupleElement(result, 0), {1}, {2}, {1}), {});
-      return xla::RngOutput{data, new_state};
-    };
+xla::BitGeneratorTy BitGen(xla::RandomAlgorithm alg) {
+  switch (alg) {
+    case xla::RandomAlgorithm::RNG_PHILOX:
+      return [=](xla::XlaOp key, xla::XlaOp state, const xla::Shape& shape) {
+        state =
+            xla::ConcatInDim(key.builder(), {xla::Reshape(key, {1}), state}, 0);
+        xla::XlaOp result = xla::RngBitGenerator(alg, state, shape);
+        xla::XlaOp data = xla::GetTupleElement(result, 1);
+        xla::XlaOp new_state =
+            xla::Slice(xla::GetTupleElement(result, 0), {1}, {3}, {1});
+        return xla::RngOutput{data, new_state};
+      };
+    case xla::RandomAlgorithm::RNG_THREE_FRY:  // fall through
+    case xla::RandomAlgorithm::RNG_DEFAULT:    // fall through
+    default:
+      return [=](xla::XlaOp key, xla::XlaOp state, const xla::Shape& shape) {
+        state = xla::ConcatScalars(key.builder(), {key, state});
+        xla::XlaOp result = xla::RngBitGenerator(alg, state, shape);
+        xla::XlaOp data = xla::GetTupleElement(result, 1);
+        xla::XlaOp new_state = xla::Reshape(
+            xla::Slice(xla::GetTupleElement(result, 0), {1}, {2}, {1}), {});
+        return xla::RngOutput{data, new_state};
+      };
   }
 }
 
-xla::RngOutput StatefulRngUniform(Algorithm alg, xla::XlaOp key,
+xla::RngOutput StatefulRngUniform(xla::RandomAlgorithm alg, xla::XlaOp key,
                                   xla::XlaOp initial_state,
                                   const xla::Shape& shape, xla::XlaOp minval,
                                   xla::XlaOp maxval) {
@@ -88,7 +94,8 @@ xla::RngOutput StatefulRngUniform(Algorithm alg, xla::XlaOp key,
   }
 }
 
-xla::RngOutput StatefulRngUniformFullInt(Algorithm alg, xla::XlaOp key,
+xla::RngOutput StatefulRngUniformFullInt(xla::RandomAlgorithm alg,
+                                         xla::XlaOp key,
                                          xla::XlaOp initial_state,
                                          const xla::Shape& shape) {
   xla::PrimitiveType type = shape.element_type();
@@ -111,16 +118,20 @@ xla::RngOutput StatefulRngUniformFullInt(Algorithm alg, xla::XlaOp key,
   }
 }
 
-using SamplerReturnType = xla::StatusOr<xla::RngOutput>;
+using SamplerReturnType = StatusOr<xla::RngOutput>;
 
-int64 GetMinStateSize(Algorithm alg) {
-  if (alg == RNG_ALG_PHILOX) {
-    return PHILOX_MIN_STATE_SIZE;
+int64_t GetMinStateSize(xla::RandomAlgorithm alg) {
+  switch (alg) {
+    case xla::RandomAlgorithm::RNG_PHILOX:
+      return PHILOX_MIN_STATE_SIZE;
+    case xla::RandomAlgorithm::RNG_THREE_FRY:  // fall through
+    case xla::RandomAlgorithm::RNG_DEFAULT:    // fall through
+    default:
+      return THREEFRY_MIN_STATE_SIZE;
   }
-  return THREEFRY_MIN_STATE_SIZE;
 }
 
-Status CheckStateShape(Algorithm alg, const TensorShape& shape) {
+Status CheckStateShape(xla::RandomAlgorithm alg, const TensorShape& shape) {
   if (shape.dims() != 1) {
     return errors::InvalidArgument(
         "RNG state must have one and only one dimension, not ", shape.dims());
@@ -131,38 +142,55 @@ Status CheckStateShape(Algorithm alg, const TensorShape& shape) {
     return errors::InvalidArgument("The size of the state must be at least ",
                                    min_state_size, "; got ", state_size);
   }
-  return Status::OK();
+  return OkStatus();
 }
 
-std::pair<xla::XlaOp, xla::XlaOp> StateAndKeyFromVariable(Algorithm alg,
-                                                          xla::XlaOp var) {
-  if (alg == RNG_ALG_THREEFRY) {
-    static constexpr int kStateSize = 1;
-    auto state = BitcastConvertType(
-        xla::Reshape(xla::Slice(var, {0}, {kStateSize}, {1}), {}), xla::U64);
-    auto key = BitcastConvertType(
-        xla::Reshape(xla::Slice(var, {kStateSize}, {kStateSize + 1}, {1}), {}),
-        xla::U64);
-    return std::make_pair(state, key);
-  } else {
-    static constexpr int kStateSize = 2;
-    auto state =
-        BitcastConvertType(xla::Slice(var, {0}, {kStateSize}, {1}), xla::U64);
-    auto key = xla::Reshape(
-        BitcastConvertType(xla::Slice(var, {kStateSize}, {kStateSize + 1}, {1}),
-                           xla::U64),
-        {});
-    return std::make_pair(state, key);
+StatusOr<xla::RandomAlgorithm> ResolveAlg(int alg_id) {
+  switch (alg_id) {
+    case RNG_ALG_PHILOX:
+      return xla::RandomAlgorithm::RNG_PHILOX;
+    case RNG_ALG_THREEFRY:
+      return xla::RandomAlgorithm::RNG_THREE_FRY;
+    case RNG_ALG_AUTO_SELECT:
+      // For AUTO_SELECT, we'll manage the counter as if it's for Philox.
+      return xla::RandomAlgorithm::RNG_PHILOX;
+    default:
+      return errors::InvalidArgument("Unsupported algorithm id: ", alg_id);
   }
 }
 
-xla::XlaOp StateAndKeyToVariable(Algorithm alg, xla::XlaOp state,
-                                 xla::XlaOp key) {
+StatusOr<xla::RandomAlgorithm> GetAlg(XlaOpKernelContext* ctx,
+                                      int alg_input_idx) {
+  TF_ASSIGN_OR_RETURN(auto alg_id, GetAlgId(ctx, alg_input_idx));
+  return ResolveAlg(alg_id);
+}
+
+std::pair<xla::XlaOp, xla::XlaOp> CounterAndKeyFromVariable(
+    xla::RandomAlgorithm alg, xla::XlaOp var) {
+  auto counter_size = GetCounterSize(alg);
+  auto counter =
+      BitcastConvertType(xla::Slice(var, {0}, {counter_size}, {1}), xla::U64);
+  if (counter_size == 1) {
+    counter = xla::Reshape(counter, {});
+  }
+  // Slicing by [-1:] may be slightly better than by
+  // [counter_size:counter_size+1], but -1 requires var_size.
+  auto key = BitcastConvertType(
+      xla::Slice(var, {counter_size}, {counter_size + 1}, {1}), xla::U64);
+  key = xla::Reshape(key, {});
+  return std::make_pair(counter, key);
+}
+
+xla::XlaOp CounterAndKeyToVariable(xla::RandomAlgorithm alg, xla::XlaOp state,
+                                   xla::XlaOp key) {
   auto builder = state.builder();
-  if (alg == RNG_ALG_THREEFRY) {
-    return ConcatScalars(builder, {state, key});
-  } else {
-    return ConcatInDim(builder, {state, xla::Reshape(key, {1})}, 0);
+  switch (alg) {
+    case xla::RandomAlgorithm::RNG_PHILOX:
+      return ConcatInDim(builder, {state, xla::Reshape(key, {1})}, 0);
+    case xla::RandomAlgorithm::RNG_THREE_FRY:  // fall through
+    case xla::RandomAlgorithm::RNG_DEFAULT:    // fall through
+    default:
+      return ConcatScalars(builder, {state, key});
   }
 }
 
@@ -171,19 +199,9 @@ xla::XlaOp StateAndKeyToVariable(Algorithm alg, xla::XlaOp state,
 Status CompileImpl(
     XlaOpKernelContext* ctx, int state_input_idx, int alg_input_idx,
     int shape_input_idx,
-    std::function<SamplerReturnType(Algorithm, xla::XlaOp, xla::XlaOp,
-                                    TensorShape)> const& sampler) {
-  auto alg_shape = ctx->InputShape(alg_input_idx);
-  if (alg_shape.dims() != 0) {
-    return errors::InvalidArgument("algorithm must be of shape [], not ",
-                                   alg_shape.DebugString());
-  }
-  xla::Literal alg_literal;
-  TF_RETURN_IF_ERROR(ctx->ConstantInput(alg_input_idx, &alg_literal));
-  auto alg = alg_literal.Get<Algorithm>({});
-  if (!(alg == RNG_ALG_THREEFRY || alg == RNG_ALG_PHILOX)) {
-    return errors::InvalidArgument("Unsupported algorithm id: ", alg);
-  }
+    std::function<SamplerReturnType(xla::RandomAlgorithm, xla::XlaOp,
+                                    xla::XlaOp, TensorShape)> const& sampler) {
+  TF_ASSIGN_OR_RETURN(auto alg, GetAlg(ctx, alg_input_idx));
 
   xla::XlaOp var;
   TensorShape var_shape;
@@ -194,22 +212,22 @@ Status CompileImpl(
   TF_RETURN_IF_ERROR(ctx->ConstantInputAsShape(shape_input_idx, &shape));
   xla::XlaOp state;
   xla::XlaOp key;
-  std::tie(state, key) = StateAndKeyFromVariable(alg, var);
+  std::tie(state, key) = CounterAndKeyFromVariable(alg, var);
   auto status_or_value = sampler(alg, state, key, shape);
   if (!status_or_value.ok()) {
     return status_or_value.status();
   }
-  xla::RngOutput value_state = status_or_value.ConsumeValueOrDie();
+  xla::RngOutput value_state = std::move(status_or_value).value();
   state = value_state.state;
   ctx->SetOutput(0, value_state.value);
-  var = StateAndKeyToVariable(alg, state, key);
+  var = CounterAndKeyToVariable(alg, state, key);
   xla::PrimitiveType state_element_type;
   TF_RETURN_IF_ERROR(
       DataTypeToPrimitiveType(STATE_ELEMENT_DTYPE, &state_element_type));
   var = BitcastConvertType(var, state_element_type);
   TF_RETURN_IF_ERROR(
       ctx->AssignVariable(state_input_idx, STATE_ELEMENT_DTYPE, var));
-  return Status::OK();
+  return OkStatus();
 }
 
 class StatefulUniformOp : public XlaOpKernel {
@@ -220,11 +238,11 @@ class StatefulUniformOp : public XlaOpKernel {
 
   void Compile(XlaOpKernelContext* ctx) override {
     xla::XlaBuilder* builder = ctx->builder();
-    auto sampler = [builder, this](Algorithm alg, xla::XlaOp state,
+    auto sampler = [builder, this](xla::RandomAlgorithm alg, xla::XlaOp state,
                                    xla::XlaOp key,
                                    TensorShape shape) -> SamplerReturnType {
+      auto rng_dtype = MaybeConvertBF16ToF32(dtype_);
       xla::Shape xla_shape;
-      DataType rng_dtype = dtype_ == DT_DOUBLE ? DT_DOUBLE : DT_FLOAT;
       TF_RETURN_IF_ERROR(TensorShapeToXLAShape(rng_dtype, shape, &xla_shape));
       xla::PrimitiveType rng_primitive_type = xla_shape.element_type();
       xla::RngOutput uniform_state = StatefulRngUniform(
@@ -251,8 +269,8 @@ class StatefulUniformOp : public XlaOpKernel {
 REGISTER_XLA_OP(Name("StatefulUniform")
                     .CompileTimeConstantInput("algorithm")
                     .CompileTimeConstantInput("shape")
-                    .TypeConstraint("dtype",
-                                    {DT_DOUBLE, DT_FLOAT, DT_BFLOAT16}),
+                    .TypeConstraint("dtype", {DT_DOUBLE, DT_FLOAT, DT_HALF,
+                                              DT_BFLOAT16}),
                 StatefulUniformOp);
 
 class StatefulStandardNormalOp : public XlaOpKernel {
@@ -265,10 +283,10 @@ class StatefulStandardNormalOp : public XlaOpKernel {
   void Compile(XlaOpKernelContext* ctx) override {
     auto sampler =
         // Needs explicit lambda return type because it fails to be inferred.
-        [this](Algorithm alg, xla::XlaOp state, xla::XlaOp key,
+        [this](xla::RandomAlgorithm alg, xla::XlaOp state, xla::XlaOp key,
                TensorShape shape) -> SamplerReturnType {
+      auto rng_dtype = MaybeConvertBF16ToF32(dtype_);
       xla::Shape xla_shape;
-      DataType rng_dtype = dtype_ == DT_DOUBLE ? DT_DOUBLE : DT_FLOAT;
       TF_RETURN_IF_ERROR(TensorShapeToXLAShape(rng_dtype, shape, &xla_shape));
       xla::RngOutput value_state = xla::NormalFloatingPointDistribution(
           key, state, BitGen(alg), xla_shape);
@@ -290,8 +308,8 @@ class StatefulStandardNormalOp : public XlaOpKernel {
 REGISTER_XLA_OP(Name("StatefulStandardNormalV2")
                     .CompileTimeConstantInput("algorithm")
                     .CompileTimeConstantInput("shape")
-                    .TypeConstraint("dtype",
-                                    {DT_DOUBLE, DT_FLOAT, DT_BFLOAT16}),
+                    .TypeConstraint("dtype", {DT_DOUBLE, DT_FLOAT, DT_HALF,
+                                              DT_BFLOAT16}),
                 StatefulStandardNormalOp);
 
 class StatefulTruncatedNormalOp : public XlaOpKernel {
@@ -305,10 +323,11 @@ class StatefulTruncatedNormalOp : public XlaOpKernel {
     xla::XlaBuilder* builder = ctx->builder();
     auto sampler =
         // Needs explicit lambda return type because it fails to be inferred.
-        [builder, this](Algorithm alg, xla::XlaOp state, xla::XlaOp key,
+        [builder, this](xla::RandomAlgorithm alg, xla::XlaOp state,
+                        xla::XlaOp key,
                         TensorShape shape) -> SamplerReturnType {
+      auto rng_dtype = MaybeConvertBF16ToF32(dtype_);
       xla::Shape xla_shape;
-      DataType rng_dtype = dtype_ == DT_DOUBLE ? DT_DOUBLE : DT_FLOAT;
       TF_RETURN_IF_ERROR(TensorShapeToXLAShape(rng_dtype, shape, &xla_shape));
 
       xla::RngOutput uniform_result = StatefulRngUniform(
@@ -336,8 +355,8 @@ class StatefulTruncatedNormalOp : public XlaOpKernel {
 REGISTER_XLA_OP(Name("StatefulTruncatedNormal")
                     .CompileTimeConstantInput("algorithm")
                     .CompileTimeConstantInput("shape")
-                    .TypeConstraint("dtype",
-                                    {DT_DOUBLE, DT_FLOAT, DT_BFLOAT16}),
+                    .TypeConstraint("dtype", {DT_DOUBLE, DT_FLOAT, DT_HALF,
+                                              DT_BFLOAT16}),
                 StatefulTruncatedNormalOp);
 
 class StatefulUniformIntOp : public XlaOpKernel {
@@ -349,9 +368,10 @@ class StatefulUniformIntOp : public XlaOpKernel {
   void Compile(XlaOpKernelContext* ctx) override {
     xla::XlaOp minval = ctx->Input(3);
     xla::XlaOp maxval = ctx->Input(4);
-    auto sample_with_threefry =
-        [minval, maxval, this](Algorithm alg, xla::XlaOp state, xla::XlaOp key,
-                               TensorShape shape) -> SamplerReturnType {
+    auto sample_with_threefry = [minval, maxval, this](
+                                    xla::RandomAlgorithm alg, xla::XlaOp state,
+                                    xla::XlaOp key,
+                                    TensorShape shape) -> SamplerReturnType {
       xla::Shape xla_shape;
       TF_RETURN_IF_ERROR(TensorShapeToXLAShape(dtype_, shape, &xla_shape));
       return StatefulRngUniform(alg, key, state, xla_shape, minval, maxval);
@@ -382,8 +402,8 @@ class StatefulUniformFullIntOp : public XlaOpKernel {
   }
 
   void Compile(XlaOpKernelContext* ctx) override {
-    auto sample_with_threefry = [this](Algorithm alg, xla::XlaOp state,
-                                       xla::XlaOp key,
+    auto sample_with_threefry = [this](xla::RandomAlgorithm alg,
+                                       xla::XlaOp state, xla::XlaOp key,
                                        TensorShape shape) -> SamplerReturnType {
       xla::Shape xla_shape;
       TF_RETURN_IF_ERROR(TensorShapeToXLAShape(dtype_, shape, &xla_shape));
@@ -407,5 +427,80 @@ REGISTER_XLA_OP(Name("StatefulUniformFullInt")
                                     {DT_INT32, DT_UINT32, DT_INT64, DT_UINT64}),
                 StatefulUniformFullIntOp);
 
+xla::XlaOp IncreaseCounter(xla::RandomAlgorithm const& alg, xla::XlaOp counter,
+                           xla::XlaOp delta) {
+  // Multiplying 256 to be consistent with the CPU/GPU kernels
+  delta = delta * ConstantR0WithType(delta.builder(), xla::U64, 256);
+  switch (alg) {
+    case xla::RandomAlgorithm::RNG_PHILOX:
+      return xla::PhiloxIncreaseCounter(counter, delta);
+    case xla::RandomAlgorithm::RNG_THREE_FRY:  // fall through
+    case xla::RandomAlgorithm::RNG_DEFAULT:    // fall through
+    default:
+      return counter + delta;
+  }
+}
+
+xla::XlaOp PadRight(xla::XlaOp a, int n) {
+  return xla::Pad(a, xla::ScalarLike(a, 0),
+                  xla::MakeEdgePaddingConfig({{0, n}}));
+}
+
+template <typename AlgEnumType = int64_t, bool read_old_value = false>
+class RngSkipOp : public XlaOpKernel {
+ public:
+  explicit RngSkipOp(OpKernelConstruction* ctx) : XlaOpKernel(ctx) {}
+
+  void Compile(XlaOpKernelContext* ctx) override {
+    const int state_input_idx = 0;
+    const int alg_input_idx = 1;
+    const int delta_input_idx = 2;
+    xla::XlaOp var;
+    TensorShape var_shape;
+    OP_REQUIRES_OK(ctx,
+                   ctx->ReadVariableInput(state_input_idx, STATE_ELEMENT_DTYPE,
+                                          &var_shape, &var));
+    // GetAlg will treat RNG_ALG_AUTO_SELECT as RNG_ALG_PHILOX.
+    OP_REQUIRES_VALUE(auto alg, ctx, GetAlg(ctx, alg_input_idx));
+    OP_REQUIRES_OK(ctx, CheckStateShape(alg, var_shape));
+    if (read_old_value) {
+      auto counter_size = GetCounterSize(alg);
+      xla::XlaOp output = var;
+      if (RNG_MAX_COUNTER_SIZE > counter_size) {
+        // Because the size of `var` depends on the algorithm while we want the
+        // output to have a fixed size (to help shape inference), we fix the
+        // output size to be the maximal state size among algorithms, and right-
+        // pad it with zeros if var's size is smaller than that.
+        output = PadRight(output, RNG_MAX_COUNTER_SIZE - counter_size);
+      }
+      ctx->SetOutput(0, output);
+    }
+    xla::XlaOp counter;
+    xla::XlaOp key;
+    std::tie(counter, key) = CounterAndKeyFromVariable(alg, var);
+    xla::XlaOp delta = ctx->Input(delta_input_idx);
+    delta = BitcastConvertType(delta, xla::U64);
+    auto new_counter = IncreaseCounter(alg, counter, delta);
+    var = CounterAndKeyToVariable(alg, new_counter, key);
+    xla::PrimitiveType state_element_type;
+    OP_REQUIRES_OK(
+        ctx, DataTypeToPrimitiveType(STATE_ELEMENT_DTYPE, &state_element_type));
+    var = BitcastConvertType(var, state_element_type);
+    OP_REQUIRES_OK(
+        ctx, ctx->AssignVariable(state_input_idx, STATE_ELEMENT_DTYPE, var));
+  }
+
+ private:
+  TF_DISALLOW_COPY_AND_ASSIGN(RngSkipOp);
+};
+
+REGISTER_XLA_OP(Name("RngSkip").CompileTimeConstantInput("algorithm"),
+                RngSkipOp<>);
+
+using RngReadAndSkipOp = RngSkipOp<int32, true>;
+
+REGISTER_XLA_OP(Name("RngReadAndSkip").CompileTimeConstantInput("alg"),
+                RngReadAndSkipOp);
+
 }  // namespace
 }  // namespace tensorflow
