@@ -15,7 +15,14 @@ limitations under the License.
 
 #include "tensorflow/compiler/xla/client/lib/math.h"
 
+#include <cmath>
+#include <complex>
+#include <functional>
 #include <limits>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
 #include "tensorflow/compiler/xla/client/lib/constants.h"
 #include "tensorflow/compiler/xla/client/xla_builder.h"
@@ -26,6 +33,7 @@ limitations under the License.
 #include "tensorflow/compiler/xla/tests/test_macros.h"
 #include "tensorflow/compiler/xla/types.h"
 #include "tensorflow/compiler/xla/xla_data.pb.h"
+#include "tensorflow/tsl/lib/core/status_test_util.h"
 
 namespace xla {
 namespace {
@@ -119,10 +127,6 @@ class MathTypedTest : public MathTest {
   //
   // For good measure, we also check pow with an exponent other than 0.5.
   void TestSqrtPowInequivalence() {
-    // TODO(b/145798892): test fails on GPU for double values.
-    if (std::is_same<T, double>::value) {
-      return;
-    }
     SetFastMathDisabled(true);
 
     // Tests disable constant folding by default, but this test needs it
@@ -143,7 +147,7 @@ class MathTypedTest : public MathTest {
     ComputeAndCompareR1<T>(&b, expected, {}, error_spec_);
   }
 
-  void TestErfEdgeCases() {
+  void TestErfInvEdgeCases() {
     SetFastMathDisabled(true);
 
     XlaBuilder b(TestName());
@@ -155,6 +159,23 @@ class MathTypedTest : public MathTest {
 
     ComputeAndCompareR1<T>(&b, expected, {}, error_spec_);
   }
+
+  void TestErfEdgeCases() {
+    SetFastMathDisabled(true);
+    const T kErfInvOneMinusHalfULP = T(3.832506856900711);
+    const T inf(std::numeric_limits<float>::infinity());
+
+    XlaBuilder b(TestName());
+    auto x = AddParam(LiteralUtil::CreateR1<T>({T{-inf}, T{inf}, T{-0}, T{0},
+                                                T{-kErfInvOneMinusHalfULP},
+                                                T{kErfInvOneMinusHalfULP}}),
+                      &b);
+    Erf(x);
+
+    std::vector<T> expected = {T(-1), T(1), T(-0), T(0), T(-1), T(1)};
+
+    ComputeAndCompareR1<T>(&b, expected, {}, error_spec_);
+  }
 };
 
 // TODO(b/123355973): Add bfloat16 to TestTypes once it's working.
@@ -178,16 +199,21 @@ XLA_TYPED_TEST(MathTypedTest, IsNegZero) { this->TestIsNegZero(); }
 XLA_TYPED_TEST(MathTypedTest, SqrtPowInequivalence) {
   this->TestSqrtPowInequivalence();
 }
-XLA_TYPED_TEST(MathTypedTest, ErfInvEdgeCases) { this->TestErfEdgeCases(); }
+XLA_TYPED_TEST(MathTypedTest, ErfInvEdgeCases) { this->TestErfInvEdgeCases(); }
+XLA_TYPED_TEST(MathTypedTest, ErfEdgeCases) { this->TestErfEdgeCases(); }
 
 // Check that certain ops only support real, floating-point inputs.
 //
 // TODO(jlebar): Expand this test to cover more ops.
 XLA_TEST_F(MathTest, RealFpOnlyOps) {
-  for (int64 i = PrimitiveType_MIN; i <= PrimitiveType_MAX; ++i) {
+  for (int64_t i = PrimitiveType_MIN; i <= PrimitiveType_MAX; ++i) {
     auto ty = static_cast<PrimitiveType>(i);
     SCOPED_TRACE(PrimitiveType_Name(ty));
     Shape shape;
+    if (ty == U4 || ty == S4) {
+      // TODO(b/259306620): breaking MakeShape()
+      continue;
+    }
     if (primitive_util::IsArrayType(ty)) {
       shape = ShapeUtil::MakeShape(ty, {42});
     } else if (ty == PrimitiveType::TUPLE) {
@@ -201,7 +227,7 @@ XLA_TEST_F(MathTest, RealFpOnlyOps) {
     }
 
     for (const auto& test :
-         std::vector<std::pair<std::function<XlaOp(XlaOp)>, string>>({
+         std::vector<std::pair<std::function<XlaOp(XlaOp)>, std::string>>({
              {IsFinite, "is_finite"},
              {IsInf, "is_inf"},
              {IsPosInf, "is_pos_inf"},
@@ -218,7 +244,11 @@ XLA_TEST_F(MathTest, RealFpOnlyOps) {
       XlaOp p = Parameter(&b, 0, shape, "p0");
       test.first(p);
 
-      EXPECT_EQ(b.first_error().ok(), primitive_util::IsFloatingPointType(ty));
+      if (primitive_util::IsFloatingPointType(ty)) {
+        TF_EXPECT_OK(b.first_error());
+      } else {
+        EXPECT_FALSE(b.first_error().ok());
+      }
     }
   }
 }
@@ -228,7 +258,7 @@ XLA_TEST_F(MathTest, SqrtF32) {
   Literal zero_literal = LiteralUtil::Zero(PrimitiveType::F32);
 
   std::unique_ptr<GlobalData> zero_data =
-      client_->TransferToServer(zero_literal).ConsumeValueOrDie();
+      client_->TransferToServer(zero_literal).value();
 
   XlaOp zero = Parameter(&builder, 0, zero_literal.shape(), "zero");
   Sqrt(zero);
@@ -241,7 +271,7 @@ XLA_TEST_F(MathTest, SqrtF64) {
   Literal zero_literal = LiteralUtil::Zero(PrimitiveType::F64);
 
   std::unique_ptr<GlobalData> zero_data =
-      client_->TransferToServer(zero_literal).ConsumeValueOrDie();
+      client_->TransferToServer(zero_literal).value();
 
   XlaOp zero = Parameter(&builder, 0, zero_literal.shape(), "zero");
   Sqrt(zero);
@@ -311,7 +341,7 @@ XLA_TEST_F(MathTest, SqrtSixValues) {
   ComputeAndCompareR1<float>(&builder, expected, {}, error_spec_);
 }
 
-XLA_TEST_F(MathTest, CbrtSixValues) {
+XLA_TEST_F(MathTest, CbrtSixF32Values) {
   XlaBuilder builder(TestName());
   auto x = ConstantR1<float>(&builder, {8.0, 1.0, 4096.0, -64.0, 1.728, 1331});
   Cbrt(x);
@@ -320,6 +350,15 @@ XLA_TEST_F(MathTest, CbrtSixValues) {
   ComputeAndCompareR1<float>(&builder, expected, {}, ErrorSpec(0.001));
 }
 
+XLA_TEST_F(MathTest, CbrtSixF64Values) {
+  XlaBuilder builder(TestName());
+  auto x = ConstantR1<double>(&builder, {8.0, 1.0, 4096.0, -64.0, 1.728, 1331});
+  Cbrt(x);
+
+  std::vector<double> expected = {2, 1, 16, -4, 1.2, 11};
+  ComputeAndCompareR1<double>(&builder, expected, {}, ErrorSpec(0.001));
+}
+
 XLA_TEST_F(MathTest, SinhSmallValues) {
   XlaBuilder builder(TestName());
   auto x = ConstantR1<float>(&builder, {1e-3, 1e-5, 1e-7, 1e-9, 1e-11});
@@ -457,6 +496,29 @@ XLA_TEST_F(MathTest, IgammaSpecialValues) {
   ComputeAndCompareR1<float>(&builder, expected, {}, error_spec_);
 }
 
+#if !defined(XLA_BACKEND_DOES_NOT_SUPPORT_FLOAT16)
+XLA_TEST_F(MathTest, IgammaF16) {
+  SetFastMathDisabled(true);
+
+  XlaBuilder builder(TestName());
+
+  auto a = ConstantR3FromArray3D<half>(
+      &builder,
+      {{{half(0.37603), half(1.6268), half(0.53327), half(1.5111)},
+        {half(1.79378), half(1.05317), half(0.85049), half(1.3995)},
+        {half(1.17725), half(0.90727), half(1.32418), half(1.5323)}}});
+
+  Igamma(a, a);
+
+  // Golden values generated by scipy.special.gammainc
+  Array3D<half> expected = {
+      {{half(0.7068214), half(0.6041154), half(0.67748886), half(0.60799426)},
+       {half(0.599202), half(0.6288743), half(0.64280254), half(0.6121421)},
+       {half(0.6220287), half(0.6384635), half(0.6152258), half(0.6072449)}}};
+  ComputeAndCompareR3<half>(&builder, expected, {}, ErrorSpec{1e-3});
+}
+#endif
+
 XLA_TEST_F(MathTest, Igammac) {
   XlaBuilder builder(TestName());
   auto a = ConstantR3FromArray3D<float>(
@@ -481,6 +543,30 @@ XLA_TEST_F(MathTest, Igammac) {
   ComputeAndCompareR3<float>(&builder, expected, {}, error_spec_);
 }
 
+#if !defined(XLA_BACKEND_DOES_NOT_SUPPORT_FLOAT16)
+XLA_TEST_F(MathTest, IgammacF16) {
+  SetFastMathDisabled(true);
+
+  XlaBuilder builder(TestName());
+
+  auto a = ConstantR3FromArray3D<half>(
+      &builder,
+      {{{half(0.37603), half(1.6268), half(0.53327), half(1.5111)},
+        {half(1.79378), half(1.05317), half(0.85049), half(1.3995)},
+        {half(1.17725), half(0.90727), half(1.32418), half(1.5323)}}});
+
+  Igammac(a, a);
+
+  // Golden values generated by scipy.special.gammaincc
+  Array3D<half> expected = {
+      {{half(0.29317862), half(0.39588454), half(0.32251117), half(0.39200574)},
+       {half(0.40079802), half(0.37112573), half(0.35719746), half(0.3878579)},
+       {half(0.3779713), half(0.36153653), half(0.38477424),
+        half(0.39275512)}}};
+  ComputeAndCompareR3<half>(&builder, expected, {}, ErrorSpec{1e-4});
+}
+#endif
+
 XLA_TEST_F(MathTest, RoundToEven) {
   XlaBuilder builder(TestName());
   auto x = ConstantR1<float>(
@@ -660,5 +746,19 @@ XLA_TEST_F(MathTest, BesselI1eDouble) {
   ComputeAndCompareR1<double>(&builder, expected, {}, error_spec_);
 }
 
+XLA_TEST_F(MathTest, AcosComplexValues) {
+  XlaBuilder builder(TestName());
+  auto x = ConstantR1<std::complex<float>>(
+      &builder, {{0, 0}, {0, 1}, {1, 1}, {0.8, 0.2}});
+
+  Acos(x);
+  std::vector<std::complex<float>> expected = {
+      {1.5707963267948966, 0},
+      {1.5707963267948966, -0.881373587019543},
+      {0.9045568943023814, -1.0612750619050357},
+      {0.7011246914497526, -0.30527648462436596}};
+  ComputeAndCompareR1<std::complex<float>>(&builder, expected, {}, error_spec_);
+}
+
 }  // namespace
 }  // namespace xla
