@@ -15,48 +15,60 @@ limitations under the License.
 
 #include "tensorflow/compiler/xla/python/tpu_driver/client/tpu_client.h"
 
+#include <algorithm>
+#include <array>
 #include <memory>
+#include <optional>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "absl/memory/memory.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/time/time.h"
 #include "absl/types/span.h"
+#include "mlir/IR/BuiltinOps.h"  // from @llvm-project
 #include "tensorflow/compiler/xla/literal.h"
+#include "tensorflow/compiler/xla/pjrt/mlir_to_hlo.h"
 #include "tensorflow/compiler/xla/pjrt/semaphore.h"
 #include "tensorflow/compiler/xla/python/tpu_driver/tpu_driver.h"
 #include "tensorflow/compiler/xla/service/computation_placer.h"
 #include "tensorflow/compiler/xla/shape_util.h"
 #include "tensorflow/compiler/xla/util.h"
 #include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/core/platform/threadpool.h"
-#include "tensorflow/core/profiler/lib/traceme.h"
+#include "tensorflow/tsl/platform/threadpool.h"
+#include "tensorflow/tsl/profiler/lib/traceme.h"
 
 namespace xla {
 
-TpuDevice::TpuDevice(int id, int host_id, const std::array<int, 3>& coords,
-                     int core_on_chip)
-    : xla::Device(id, /*local_device_state=*/nullptr, kTpuPlatform,
-                  /*device_kind=*/"Cloud TPU", host_id),
+TpuDeviceDescription::TpuDeviceDescription(int id, int process_index,
+                                           const std::array<int, 3>& coords,
+                                           int core_on_chip)
+    : id_(id),
+      process_index_(process_index),
       coords_(coords),
-      core_on_chip_(core_on_chip) {}
-
-std::string TpuDevice::DebugString() const {
-  return absl::StrFormat("TPU_%i(host=%i,(%i,%i,%i,%i))", id(), host_id(),
-                         coords_[0], coords_[1], coords_[2], core_on_chip_);
+      core_on_chip_(core_on_chip) {
+  debug_string_ =
+      absl::StrFormat("TPU_%i(host=%i,(%i,%i,%i,%i))", id_, process_index_,
+                      coords_[0], coords_[1], coords_[2], core_on_chip_);
+  to_string_ = absl::StrFormat(
+      "TpuDevice(id=%i, process_index=%i, coords=(%s), core_on_chip=%i)", id_,
+      process_index_, absl::StrJoin(coords_, ","), core_on_chip_);
 }
 
-xla::StatusOr<std::vector<std::shared_ptr<xla::Device>>>
+TpuDevice::TpuDevice(int id, int process_index,
+                     const std::array<int, 3>& coords, int core_on_chip)
+    : description_(id, process_index, coords, core_on_chip) {}
+
+xla::StatusOr<std::vector<std::shared_ptr<xla::PjRtDevice>>>
 TpuDevice::GetTpuDevices(const tpu_driver::SystemInfo& system_info) {
-  std::vector<std::shared_ptr<Device>> devices;
+  std::vector<std::shared_ptr<PjRtDevice>> devices;
   for (const auto& chip : system_info.tpu_chip()) {
     auto& coord = chip.chip_coord();
     std::array<int, 3> coords_array = {coord.x(), coord.y(), coord.z()};
-    int host_id = chip.host_id();
+    int process_index = chip.host_id();
     for (const auto& core : chip.core()) {
       auto device = std::make_shared<TpuDevice>(
-          core.id(), host_id, coords_array, core.core_on_chip_index());
+          core.id(), process_index, coords_array, core.core_on_chip_index());
       devices.push_back(device);
     }
   }
@@ -73,33 +85,35 @@ StatusOr<std::shared_ptr<PyTpuClient>> PyTpuClient::Get(
     return client_status.status();
   }
 
-  auto client = client_status.ConsumeValueOrDie();
+  auto client = std::move(client_status).value();
 
   tpu_driver::SystemInfo system_info;
   client->QuerySystemInfo(&system_info);
 
-  TF_ASSIGN_OR_RETURN(std::vector<std::shared_ptr<Device>> devices,
+  TF_ASSIGN_OR_RETURN(std::vector<std::shared_ptr<PjRtDevice>> devices,
                       TpuDevice::GetTpuDevices(system_info));
 
-  return std::make_shared<PyTpuClient>(kTpuPlatform, std::move(client),
+  return std::make_shared<PyTpuClient>(TpuPlatform(), std::move(client),
                                        std::move(devices),
                                        system_info.host_id());
 }
 
 PyTpuClient::PyTpuClient(std::string platform_name,
                          std::unique_ptr<tpu_driver::TpuDriver> driver,
-                         std::vector<std::shared_ptr<Device>> devices,
-                         int host_id)
+                         std::vector<std::shared_ptr<PjRtDevice>> devices,
+                         int process_index)
     : platform_name_(std::move(platform_name)),
+      platform_version_("tpu_driver (deprecated)"),
       driver_(std::move(driver)),
       devices_(std::move(devices)),
-      host_id_(host_id) {
-  for (const std::shared_ptr<Device>& device : devices_) {
+      process_index_(process_index) {
+  for (const std::shared_ptr<PjRtDevice>& device : devices_) {
+    tensorflow::down_cast<TpuDevice*>(device.get())->set_tpu_client(this);
     CHECK(id_to_device_.insert({device->id(), device}).second)
         << "Duplicate device id: " << device->id();
 
-    if (device->host_id() == host_id_) {
-      LOG(INFO) << "Detected local device, host id: " << host_id_
+    if (device->process_index() == process_index_) {
+      LOG(INFO) << "Detected local device, host id: " << process_index_
                 << ". device id: " << device->id();
       local_devices_.push_back(device);
     } else {
@@ -116,8 +130,8 @@ PyTpuClient::PyTpuClient(std::string platform_name,
   // TODO(frankchn): Check if thread pool size needs to be adjusted (perhaps
   // something like min(cores, devices_.size()) might be appropriate depending
   // on the number of devices.
-  pool_ = std::make_unique<tensorflow::thread::ThreadPool>(
-      tensorflow::Env::Default(), "PyTpuClient", devices_.size());
+  pool_ = std::make_unique<tsl::thread::ThreadPool>(
+      tsl::Env::Default(), "PyTpuClient", devices_.size());
 }
 
 Status PyTpuClient::TransferToInfeed(const LiteralSlice& literal,
@@ -156,7 +170,7 @@ Status PyTpuClient::CheckDeviceId(int device_id,
     return InvalidArgument("%s got bad device_id: %d (num_devices=%d).",
                            caller_name, device_id, device_count());
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 static Status CheckDataType(xla::PrimitiveType dtype) {
@@ -164,17 +178,17 @@ static Status CheckDataType(xla::PrimitiveType dtype) {
       dtype == xla::PrimitiveType::U64) {
     return InvalidArgument(
         "64-bit data types are not yet supported on the TPU driver API. "
-        "Convert inputs to float32/int32 before using.");
+        "Convert inputs to float32/int32_t before using.");
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 /* static */
 StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::FromLiterals(
     std::vector<BorrowingLiteral> leaves, const Shape& tuple_shape,
     std::shared_ptr<void> leaves_references,
-    std::shared_ptr<PyTpuClient> client, std::shared_ptr<Device> device) {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::FromLiterals");
+    std::shared_ptr<PyTpuClient> client, std::shared_ptr<PjRtDevice> device) {
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::FromLiterals");
   VLOG(1) << "PyTpuBuffer::FromLiterals: shape: " << tuple_shape.DebugString()
           << " device: " << device->DebugString();
   TF_RETURN_IF_ERROR(
@@ -229,7 +243,7 @@ StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::FromLiterals(
 /* static */
 StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::MakeTuple(
     absl::Span<PyTpuBuffer* const> buffers, std::shared_ptr<PyTpuClient> client,
-    std::shared_ptr<Device> device) {
+    std::shared_ptr<PjRtDevice> device) {
   std::vector<Shape> child_shapes;
   std::vector<std::shared_ptr<TpuSharedBuffer>> child_device_buffers;
   std::vector<tpu_driver::BufferHandle*> child_handle_ptrs;
@@ -257,7 +271,7 @@ StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::MakeTuple(
   auto tuple_device_buffer = std::make_shared<TpuSharedBuffer>(
       client->driver(), std::move(tuple_handle), std::move(child_events),
       std::move(device));
-  return absl::make_unique<PyTpuBuffer>(
+  return std::make_unique<PyTpuBuffer>(
       tuple_shape, std::move(tuple_device_buffer),
       std::move(child_device_buffers), std::move(client));
 }
@@ -291,7 +305,7 @@ Status PyTpuBuffer::CopyToHostAsync() {
 
     if (host_value_) {
       // The host value has already been requested or is available.
-      return Status::OK();
+      return OkStatus();
     }
 
     host_value->value = std::make_shared<Literal>(on_host_shape_);
@@ -325,9 +339,8 @@ Status PyTpuBuffer::CopyToHostAsync() {
     t->AddCallback([host_value](const xla::Status& status) {
       VLOG(1) << "Device to host transfer finished.";
       if (!status.ok()) {
-        host_value->status =
-            Status(static_cast<tensorflow::error::Code>(status.code()),
-                   status.error_message());
+        host_value->status = Status(
+            static_cast<absl::StatusCode>(status.code()), status.message());
       }
 
       absl::MutexLock m(&host_value->mutex);
@@ -338,11 +351,11 @@ Status PyTpuBuffer::CopyToHostAsync() {
       }
     });
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 StatusOr<std::shared_ptr<Literal>> PyTpuBuffer::ToLiteral() {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::ToLiteral");
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::ToLiteral");
   TF_RETURN_IF_ERROR(CopyToHostAsync());
 
   mu_.Lock();
@@ -364,7 +377,7 @@ std::shared_ptr<TpuSharedBuffer> PyTpuBuffer::DeviceBuffer() const {
 
 StatusOr<std::vector<std::unique_ptr<PyTpuBuffer>>>
 PyTpuBuffer::DestructureTuple() {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::DestructureTuple");
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::DestructureTuple");
   if (!on_host_shape_.IsTuple()) {
     return InvalidArgument(
         "Attempted to destructure a PyTpuBuffer that did not have a tuple "
@@ -380,7 +393,7 @@ PyTpuBuffer::DestructureTuple() {
   std::vector<std::unique_ptr<PyTpuBuffer>> results;
   results.reserve(num_children);
   for (int i = 0; i < num_children; ++i) {
-    results.push_back(absl::make_unique<PyTpuBuffer>(
+    results.push_back(std::make_unique<PyTpuBuffer>(
         on_host_shape_.tuple_shapes(i), child_buffers_.at(i),
         std::vector<std::shared_ptr<TpuSharedBuffer>>(), client_));
   }
@@ -388,15 +401,15 @@ PyTpuBuffer::DestructureTuple() {
 }
 
 StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::CopyToDevice(
-    std::shared_ptr<Device> dst_device) {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::CopyToDevice");
+    std::shared_ptr<PjRtDevice> dst_device) {
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::CopyToDevice");
   if (on_host_shape_.IsTuple()) {
     return Unimplemented("CopyToDevice for tuples is not supported.");
   }
 
   std::shared_ptr<TpuSharedBuffer> src_device_buffer = DeviceBuffer();
   if (dst_device->id() == device_->id()) {
-    return absl::make_unique<PyTpuBuffer>(
+    return std::make_unique<PyTpuBuffer>(
         on_host_shape_, src_device_buffer,
         std::vector<std::shared_ptr<TpuSharedBuffer>>(), client_);
   }
@@ -426,20 +439,25 @@ StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::CopyToDevice(
 }
 
 Status PyTpuBuffer::BlockHostUntilReady() {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::BlockHostUntilReady");
-  return DeviceBuffer()->handle->OnReady()->Await();
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::BlockHostUntilReady");
+  std::shared_ptr<TpuSharedBuffer> device_buffer = DeviceBuffer();
+  if (!device_buffer) {
+    return InvalidArgument(
+        "BlockHostUntilReady() called on deleted or donated buffer");
+  }
+  return device_buffer->handle->OnReady()->Await();
 }
 
 /* static */
 StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::AllocateBuffer(
     const Shape& shape, std::shared_ptr<PyTpuClient> client,
-    std::shared_ptr<Device> device) {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::AllocateBuffer");
+    std::shared_ptr<PjRtDevice> device) {
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::AllocateBuffer");
   VLOG(1) << "PyTpuBuffer::AllocateBuffer: shape: " << shape.DebugString()
           << " device: " << device->DebugString();
 
   if (!shape.IsTuple()) {
-    return CreateBuffer(shape, absl::nullopt, std::move(client),
+    return CreateBuffer(shape, std::nullopt, std::move(client),
                         std::move(device));
   }
 
@@ -464,9 +482,9 @@ StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::AllocateBuffer(
 
 /*static*/
 StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::CreateBuffer(
-    const Shape& non_tuple_shape, absl::optional<BufferInitializer> initializer,
-    std::shared_ptr<PyTpuClient> client, std::shared_ptr<Device> device) {
-  tensorflow::profiler::TraceMe traceme("PyTpuBuffer::CreateBuffer");
+    const Shape& non_tuple_shape, std::optional<BufferInitializer> initializer,
+    std::shared_ptr<PyTpuClient> client, std::shared_ptr<PjRtDevice> device) {
+  tsl::profiler::TraceMe traceme("PyTpuBuffer::CreateBuffer");
   VLOG(1) << "PyTpuBuffer::CreateBuffer: shape: "
           << non_tuple_shape.DebugString()
           << " device: " << device->DebugString();
@@ -488,13 +506,13 @@ StatusOr<std::unique_ptr<PyTpuBuffer>> PyTpuBuffer::CreateBuffer(
       client->driver(), std::move(handle), std::move(wait_for_use),
       std::move(device));
 
-  return absl::make_unique<PyTpuBuffer>(
+  return std::make_unique<PyTpuBuffer>(
       non_tuple_shape, std::move(device_buffer),
       std::vector<std::shared_ptr<TpuSharedBuffer>>(), client);
 }
 
-static std::shared_ptr<Device> LookupDevice(const PyTpuClient& client,
-                                            int device_id) {
+static std::shared_ptr<PjRtDevice> LookupDevice(const PyTpuClient& client,
+                                                int device_id) {
   auto it = client.id_to_device().find(device_id);
   CHECK(it != client.id_to_device().end())
       << "Unknown device id: " << device_id;
@@ -516,8 +534,8 @@ PyTpuExecutable::PyTpuExecutable(
   for (int replica = 0; replica < num_replicas; ++replica) {
     for (int partition = 0; partition < num_partitions; ++partition) {
       int device_id = device_assignment_(replica, partition);
-      std::shared_ptr<Device> device = LookupDevice(*client_, device_id);
-      if (device->host_id() != client_->host_id()) {
+      std::shared_ptr<PjRtDevice> device = LookupDevice(*client_, device_id);
+      if (device->process_index() != client_->process_index()) {
         VLOG(3) << "Non-local device: " << device_id;
         continue;
       }
@@ -537,20 +555,20 @@ PyTpuExecutable::PyTpuExecutable(
 }
 
 PyTpuExecutable::ExecuteResult PyTpuExecutable::ExecuteHelper(
-    absl::Span<const std::vector<PyTpuBuffer*>> all_core_arguments,
+    absl::Span<const std::vector<PyTpuBuffer*>> maybe_tupled_args,
     absl::Span<PyTpuBuffer* const> this_core_arguments, int replica,
     int partition, const RunId& run_id) {
   const int device_id = device_assignment_(replica, partition);
-  std::shared_ptr<Device> device = LookupDevice(*client_, device_id);
-  CHECK_EQ(device->host_id(), client_->host_id());
-  tensorflow::profiler::TraceMe traceme("PyTpuExecutable::Execute");
+  std::shared_ptr<PjRtDevice> device = LookupDevice(*client_, device_id);
+  CHECK_EQ(device->process_index(), client_->process_index());
+  tsl::profiler::TraceMe traceme("PyTpuExecutable::Execute");
   VLOG(3) << "Replica " << replica << ", partition " << partition
           << " mapped to device id for execution: " << device_id;
 
   std::unique_ptr<::xla::PyTpuBuffer> output_buffer =
       ::xla::PyTpuBuffer::AllocateBuffer(result_shape_, client_,
                                          std::move(device))
-          .ValueOrDie();
+          .value();
   VLOG(1) << "Created output buffer: " << result_shape_.DebugString();
 
   std::vector<tpu_driver::BufferHandle*> inputs;
@@ -565,7 +583,7 @@ PyTpuExecutable::ExecuteResult PyTpuExecutable::ExecuteHelper(
     inputs.push_back(input_handle->DeviceBuffer()->handle.get());
   }
 
-  for (const auto& core_args : all_core_arguments) {
+  for (const auto& core_args : maybe_tupled_args) {
     for (const auto* handle : core_args) {
       for (const auto& pending_event : handle->DeviceBuffer()->wait_for_use) {
         ready_to_execute.push_back(pending_event.get());
@@ -574,7 +592,7 @@ PyTpuExecutable::ExecuteResult PyTpuExecutable::ExecuteHelper(
   }
 
   xla::DeviceAssignmentProto device_assignment;
-  CHECK(device_assignment_.Serialize(&device_assignment).ok());
+  TF_CHECK_OK(device_assignment_.Serialize(&device_assignment));
   std::shared_ptr<tpu_driver::Event> on_execute_finished =
       client_->driver()->ExecuteProgram(
           executables_.find(replica)->second.get(), inputs,
@@ -588,10 +606,10 @@ PyTpuExecutable::ExecuteResult PyTpuExecutable::ExecuteHelper(
 static const absl::Duration kWarnExecutionDelay = absl::Seconds(10);
 
 // Delay before terminating a stalled execute call.
-static const absl::Duration kMaxExecutionDelay = absl::Seconds(120);
+static const absl::Duration kMaxExecutionDelay = absl::Minutes(60);
 
 Status WaitForExecuteEvent(tpu_driver::Event* event) {
-  absl::optional<Status> opt_status;
+  std::optional<Status> opt_status;
   auto start_time = absl::Now();
 
   while (!opt_status.has_value() &&
@@ -605,7 +623,7 @@ Status WaitForExecuteEvent(tpu_driver::Event* event) {
   }
 
   if (!opt_status.has_value()) {
-    return tensorflow::errors::DeadlineExceeded(
+    return tsl::errors::DeadlineExceeded(
         absl::StrFormat("TPU program took more than %d seconds to complete.",
                         absl::ToInt64Seconds(kMaxExecutionDelay)));
   }
@@ -626,20 +644,19 @@ StatusOr<std::vector<std::unique_ptr<PyTpuBuffer>>> PyTpuExecutable::Execute(
         num_partitions());
   }
 
-  std::vector<PyTpuBuffer*> all_core_arguments;
-
+  std::vector<PyTpuBuffer*> maybe_tupled_args;
   std::unique_ptr<PyTpuBuffer> tupled_arguments;
   if (tuple_arguments_) {
     TF_ASSIGN_OR_RETURN(tupled_arguments,
                         PyTpuBuffer::MakeTuple(argument_handles, client_,
                                                local_devices_.front()));
-    all_core_arguments = {tupled_arguments.get()};
+    maybe_tupled_args = {tupled_arguments.get()};
   } else {
-    all_core_arguments = std::vector<PyTpuBuffer*>(argument_handles.begin(),
-                                                   argument_handles.end());
+    maybe_tupled_args = std::vector<PyTpuBuffer*>(argument_handles.begin(),
+                                                  argument_handles.end());
   }
   ExecuteResult result =
-      ExecuteHelper(absl::MakeSpan(&all_core_arguments, 1), argument_handles,
+      ExecuteHelper(absl::MakeSpan(&maybe_tupled_args, 1), maybe_tupled_args,
                     /*replica=*/0, /*partition=*/0, RunId());
 
   Status status = WaitForExecuteEvent(result.on_execute_finished.get());
@@ -661,8 +678,7 @@ StatusOr<std::vector<std::unique_ptr<PyTpuBuffer>>> PyTpuExecutable::Execute(
 StatusOr<std::vector<std::vector<std::unique_ptr<PyTpuBuffer>>>>
 PyTpuExecutable::ExecuteOnLocalDevices(
     absl::Span<const std::vector<PyTpuBuffer*>> argument_handles) {
-  tensorflow::profiler::TraceMe traceme(
-      "PyTpuExecutable::ExecuteOnLocalDevices");
+  tsl::profiler::TraceMe traceme("PyTpuExecutable::ExecuteOnLocalDevices");
 
   const int num_local_devices = local_devices_.size();
 
@@ -722,8 +738,8 @@ PyTpuExecutable::ExecuteOnLocalDevices(
     auto s = WaitForExecuteEvent(results[i].on_execute_finished.get());
     if (!s.ok()) {
       if (failed == 0) {
-        first_failure_status = Status(
-            static_cast<tensorflow::error::Code>(s.code()), s.error_message());
+        first_failure_status =
+            Status(static_cast<absl::StatusCode>(s.code()), s.message());
       }
       ++failed;
     }
@@ -746,15 +762,54 @@ PyTpuExecutable::ExecuteOnLocalDevices(
   return wrapped_results;
 }
 
+StatusOr<std::vector<std::vector<std::unique_ptr<PyTpuBuffer>>>>
+PyTpuExecutable::ExecuteShardedOnLocalDevices(
+    absl::Span<const std::vector<PyTpuBuffer*>> args) {
+  std::vector<std::vector<std::unique_ptr<PyTpuBuffer>>> output_buffers;
+  TF_RET_CHECK(!args.empty());
+  int num_computations = args.front().size();
+  for (const auto& arg : args) {
+    if (arg.size() != num_computations) {
+      return xla::InvalidArgument(
+          "Expected args to execute_sharded_on_local_devices to have %d "
+          "shards, got: [%s]",
+          num_computations,
+          absl::StrJoin(
+              args, ", ",
+              [](std::string* out, const std::vector<PyTpuBuffer*>& arg) {
+                out->append(std::to_string(arg.size()));
+              }));
+    }
+  }
+  std::vector<std::vector<PyTpuBuffer*>> arg_buffers(num_computations);
+  for (int computation = 0; computation < num_computations; ++computation) {
+    arg_buffers[computation].resize(args.size());
+    absl::c_transform(
+        args, arg_buffers[computation].begin(),
+        [&](const std::vector<PyTpuBuffer*>& arg) { return arg[computation]; });
+  }
+  TF_ASSIGN_OR_RETURN(output_buffers, ExecuteOnLocalDevices(arg_buffers));
+  int num_output_buffers = output_buffers[0].size();
+  std::vector<std::vector<std::unique_ptr<PyTpuBuffer>>> outputs;
+  outputs.resize(num_output_buffers);
+  for (int buffer_id = 0; buffer_id < num_output_buffers; ++buffer_id) {
+    outputs[buffer_id].reserve(num_computations);
+    for (int computation = 0; computation < num_computations; ++computation) {
+      outputs[buffer_id].push_back(
+          std::move(output_buffers[computation][buffer_id]));
+    }
+  }
+  return outputs;
+}
+
 /*static*/ StatusOr<std::unique_ptr<PyTpuExecutable>> PyTpuExecutable::Compile(
     const XlaComputation& computation,
-    absl::optional<std::vector<Shape>> argument_layouts,
+    std::optional<std::vector<Shape>> argument_layouts,
     const ExecutableBuildOptions* build_options,
     std::shared_ptr<PyTpuClient> client, bool tuple_arguments) {
-  tensorflow::profiler::TraceMe traceme("PyTpuExecutable::Compile");
+  tsl::profiler::TraceMe traceme("PyTpuExecutable::Compile");
 
-  VLOG(1) << "Compile: "
-          << computation.GetProgramShape().ValueOrDie().DebugString();
+  VLOG(1) << "Compile: " << computation.GetProgramShape().value().DebugString();
 
   // TODO(power) -- handle argument layouts
   // TODO(power) -- handle build options
@@ -762,7 +817,7 @@ PyTpuExecutable::ExecuteOnLocalDevices(
   if (build_options != nullptr) {
     options = *build_options;
   }
-  absl::optional<xla::DeviceAssignment> device_assignment;
+  std::optional<xla::DeviceAssignment> device_assignment;
 
   // For POD use case, the device_assignment.num_replicas() may be greater than
   // the number of available local devices, where applicable the non-local
@@ -795,7 +850,8 @@ PyTpuExecutable::ExecuteOnLocalDevices(
   // TODO(henrytan): in the future, we want to consider argument Layout
   // information e.g. for linearization.
   std::unique_ptr<tpu_driver::CompiledProgramHandle> compiled_program =
-      client->driver()->CompileProgram(hlo_proto, options.num_replicas(), {});
+      client->driver()->CompileProgram(hlo_proto, options.num_replicas(), {},
+                                       options.debug_options());
 
   ::xla::Shape result_layout;
   if (options.result_layout()) {
@@ -806,17 +862,30 @@ PyTpuExecutable::ExecuteOnLocalDevices(
         compiled_program->program_shape(&program_shape_proto);
 
     if (!fetch_metadata_status.ok()) {
-      return Status(
-          static_cast<tensorflow::error::Code>(fetch_metadata_status.code()),
-          fetch_metadata_status.error_message());
+      return Status(static_cast<absl::StatusCode>(fetch_metadata_status.code()),
+                    fetch_metadata_status.message());
     }
     result_layout = ::xla::Shape(program_shape_proto.result());
   }
   VLOG(1) << "Got result shape: " << result_layout.DebugString();
 
-  return absl::make_unique<PyTpuExecutable>(
+  return std::make_unique<PyTpuExecutable>(
       std::move(compiled_program), std::move(*device_assignment),
       std::move(client), std::move(result_layout), tuple_arguments);
 }
 
+/*static*/ StatusOr<std::unique_ptr<PyTpuExecutable>>
+PyTpuExecutable::CompileMlir(mlir::ModuleOp module,
+                             std::optional<std::vector<Shape>> argument_layouts,
+                             const ExecutableBuildOptions* build_options,
+                             std::shared_ptr<PyTpuClient> client,
+                             bool tuple_arguments) {
+  XlaComputation xla_computation;
+  TF_RETURN_IF_ERROR(MlirToXlaComputation(module, xla_computation,
+                                          /*use_tuple_args=*/tuple_arguments,
+                                          /*return_tuple=*/false));
+  return Compile(xla_computation, argument_layouts, build_options, client,
+                 tuple_arguments);
+}
+
 }  // namespace xla
