@@ -16,25 +16,26 @@ limitations under the License.
 #include "tensorflow/compiler/xla/service/bfloat16_propagation.h"
 
 #include "absl/algorithm/container.h"
+#include "absl/cleanup/cleanup.h"
 #include "absl/container/flat_hash_set.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_computation.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_instruction.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_module.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_opcode.h"
 #include "tensorflow/compiler/xla/literal.h"
 #include "tensorflow/compiler/xla/map_util.h"
-#include "tensorflow/compiler/xla/service/hlo_computation.h"
 #include "tensorflow/compiler/xla/service/hlo_dce.h"
-#include "tensorflow/compiler/xla/service/hlo_instruction.h"
-#include "tensorflow/compiler/xla/service/hlo_module.h"
-#include "tensorflow/compiler/xla/service/hlo_opcode.h"
 #include "tensorflow/compiler/xla/service/tuple_simplifier.h"
 #include "tensorflow/compiler/xla/shape_tree.h"
 #include "tensorflow/compiler/xla/shape_util.h"
-#include "tensorflow/core/lib/gtl/cleanup.h"
-#include "tensorflow/core/platform/logging.h"
+#include "tensorflow/tsl/platform/logging.h"
 
 namespace xla {
 
-BFloat16Propagation::BFloat16Propagation(
-    const BFloat16Support* bfloat16_support)
-    : bfloat16_support_(bfloat16_support) {}
+BFloat16Propagation::BFloat16Propagation(const FloatSupport* bfloat16_support)
+    : bfloat16_support_(bfloat16_support) {
+  DCHECK_EQ(bfloat16_support->LowPrecisionType(), BF16);
+}
 
 void BFloat16Propagation::DetermineFusionComputationPrecision(
     HloInstruction* fusion) {
@@ -96,30 +97,26 @@ void BFloat16Propagation::RevertIfFusionInternalBF16Changes(
     }
   }
 
-  auto aliases_changed_root_buffer =
-      [this, &changed_root_buffers](const HloInstruction* inst) {
-        bool aliasing = false;
-        ShapeUtil::ForEachSubshape(
-            inst->shape(), [&](const Shape& subshape, const ShapeIndex& index) {
-              if (aliasing) {
-                // Skip if aliasing is already found.
-                return;
-              }
-              // Only F32 buffers are considered for changing to BF16 in this
-              // pass.
-              if (subshape.element_type() != F32) {
-                return;
-              }
-              for (const HloValue* value :
-                   dataflow_->GetValueSet(inst, index).values()) {
-                if (ContainsKey(changed_root_buffers, value)) {
-                  aliasing = true;
-                  break;
-                }
-              }
-            });
-        return aliasing;
-      };
+  auto aliases_changed_root_buffer = [this, &changed_root_buffers](
+                                         const HloInstruction* inst) {
+    bool aliasing = false;
+    ShapeUtil::ForEachSubshape(inst->shape(), [&](const Shape& subshape,
+                                                  const ShapeIndex& index) {
+      if (aliasing) {
+        // Skip if aliasing is already found.
+        return;
+      }
+      // Only F32 buffers are considered for changing to BF16 in this
+      // pass.
+      if (subshape.element_type() != F32) {
+        return;
+      }
+
+      aliasing = absl::c_any_of(dataflow_->GetValueSet(inst, index).values(),
+                                IsValueIn(changed_root_buffers));
+    });
+    return aliasing;
+  };
 
   for (auto inst :
        fusion->fused_instructions_computation()->MakeInstructionPostOrder()) {
@@ -131,7 +128,7 @@ void BFloat16Propagation::RevertIfFusionInternalBF16Changes(
     }
     if (inst->opcode() == HloOpcode::kFusion) {
       bool parameter_reverted = false;
-      for (int64 i = 0; i < inst->operand_count(); ++i) {
+      for (int64_t i = 0; i < inst->operand_count(); ++i) {
         if (has_changes(inst->mutable_operand(i))) {
           // Changes on the operand have not been reverted.
           continue;
@@ -208,7 +205,7 @@ void BFloat16Propagation::DetermineWhileComputationsPrecision(
 void BFloat16Propagation::DetermineConditionalComputationsPrecision(
     HloInstruction* cond) {
   CHECK_EQ(cond->opcode(), HloOpcode::kConditional);
-  for (int64 i = 0; i < cond->branch_count(); ++i) {
+  for (int64_t i = 0; i < cond->branch_count(); ++i) {
     auto branch = cond->branch_computation(i);
     auto root = branch->root_instruction();
     ShapeUtil::ForEachSubshape(
@@ -253,7 +250,7 @@ bool BFloat16Propagation::AllUsersConsumeBF16(const HloInstruction& hlo,
     if (value->shape().element_type() == BF16) {
       continue;
     }
-    for (const HloUse& use : value->uses()) {
+    for (const HloUse& use : value->GetUses()) {
       if (!ContainsKey(instructions_visited_in_backward_pass_,
                        use.instruction)) {
         // We don't know yet whether use.instruction will consume BF16 since it
@@ -303,7 +300,7 @@ bool BFloat16Propagation::AllUsersConsumeBF16(const HloInstruction& hlo,
         }
         continue;
       }
-      if (bfloat16_support_->EffectiveOperandPrecisionIsBF16(
+      if (bfloat16_support_->EffectiveOperandPrecisionIsLowPrecision(
               *use.instruction, use.operand_number)) {
         continue;
       }
@@ -316,7 +313,7 @@ bool BFloat16Propagation::AllUsersConsumeBF16(const HloInstruction& hlo,
             (use.instruction->opcode() == HloOpcode::kAllReduce &&
              use.instruction->shape().IsTuple())) {
           ShapeIndex use_output_index{use.operand_number};
-          for (int64 i : use.operand_index) {
+          for (int64_t i : use.operand_index) {
             use_output_index.push_back(i);
           }
           if (OutputTypeAfterChange(use.instruction, use_output_index) ==
@@ -325,7 +322,7 @@ bool BFloat16Propagation::AllUsersConsumeBF16(const HloInstruction& hlo,
           }
         } else if (use.instruction->opcode() == HloOpcode::kGetTupleElement) {
           ShapeIndex use_output_index;
-          for (int64 i = 1; i < use.operand_index.size(); ++i) {
+          for (int64_t i = 1; i < use.operand_index.size(); ++i) {
             use_output_index.push_back(use.operand_index[i]);
           }
           if (OutputTypeAfterChange(use.instruction, use_output_index) ==
@@ -345,11 +342,8 @@ bool BFloat16Propagation::AllUsersConsumeBF16(const HloInstruction& hlo,
   return true;
 }
 
-namespace {
-
-// Returns whether we should avoid changing the precision of inst regardless of
-// the producers and users.
-bool ShouldKeepPrecisionUnchanged(const HloInstruction* inst) {
+bool BFloat16Propagation::ShouldKeepPrecisionUnchanged(
+    const HloInstruction* inst) {
   if (inst->opcode() == HloOpcode::kFusion &&
       inst->fusion_kind() == HloInstruction::FusionKind::kCustom) {
     return ShouldKeepPrecisionUnchanged(
@@ -358,14 +352,12 @@ bool ShouldKeepPrecisionUnchanged(const HloInstruction* inst) {
   // Do not change precision for side-effecting instructions, control flow, and
   // bitcast-convert, because this pass might break the interfaces or
   // assumptions for them.
-  return inst->opcode() == HloOpcode::kCustomCall ||      //
-         inst->opcode() == HloOpcode::kCall ||            //
-         inst->opcode() == HloOpcode::kBitcastConvert ||  //
+  return inst->opcode() == HloOpcode::kCustomCall ||
+         inst->opcode() == HloOpcode::kCall ||
+         inst->opcode() == HloOpcode::kBitcastConvert ||
          inst->HasSideEffectNoRecurse();
 }
 
-}  // namespace
-
 void BFloat16Propagation::DetermineInstructionPrecision(HloInstruction* hlo,
                                                         bool skip_parameters) {
   // We handle any fusion computation, while body/condition or conditional
@@ -373,19 +365,19 @@ void BFloat16Propagation::DetermineInstructionPrecision(HloInstruction* hlo,
   // output shape of a fusion or while before propagating inside its
   // computations.
   bool postpone_processing_called_computations = false;
-  auto cleaner = tensorflow::gtl::MakeCleanup(
-      [this, hlo, &postpone_processing_called_computations] {
-        if (!postpone_processing_called_computations) {
-          if (hlo->opcode() == HloOpcode::kFusion) {
-            DetermineFusionComputationPrecision(hlo);
-          } else if (hlo->opcode() == HloOpcode::kWhile) {
-            DetermineWhileComputationsPrecision(hlo);
-          } else if (hlo->opcode() == HloOpcode::kConditional) {
-            DetermineConditionalComputationsPrecision(hlo);
-          }
-        }
-        instructions_visited_in_backward_pass_.insert(hlo);
-      });
+  absl::Cleanup cleaner = [this, hlo,
+                           &postpone_processing_called_computations] {
+    if (!postpone_processing_called_computations) {
+      if (hlo->opcode() == HloOpcode::kFusion) {
+        DetermineFusionComputationPrecision(hlo);
+      } else if (hlo->opcode() == HloOpcode::kWhile) {
+        DetermineWhileComputationsPrecision(hlo);
+      } else if (hlo->opcode() == HloOpcode::kConditional) {
+        DetermineConditionalComputationsPrecision(hlo);
+      }
+    }
+    instructions_visited_in_backward_pass_.insert(hlo);
+  };
 
   if (hlo->opcode() == HloOpcode::kWhile &&
       (caller_counts_[hlo->while_condition()] > 1 ||
@@ -432,7 +424,7 @@ void BFloat16Propagation::DetermineInstructionPrecision(HloInstruction* hlo,
     return;
   }
 
-  if (!bfloat16_support_->SupportsBF16Output(*hlo)) {
+  if (!bfloat16_support_->SupportsLowPrecisionOutput(*hlo)) {
     return;
   }
 
@@ -455,7 +447,7 @@ bool BFloat16Propagation::InstructionIsCandidateForBF16Output(
       hlo->opcode() != HloOpcode::kGetTupleElement &&
       hlo->opcode() != HloOpcode::kDomain &&
       hlo->shape().element_type() != BF16) {
-    for (int64 i = 0; i < hlo->operand_count(); ++i) {
+    for (int64_t i = 0; i < hlo->operand_count(); ++i) {
       if (!bfloat16_support_->EffectiveOperandPrecisionIsOutputPrecision(*hlo,
                                                                          i) ||
           !ContainsKey(consider_using_bfloat16_, hlo->operand(i))) {
@@ -468,34 +460,33 @@ bool BFloat16Propagation::InstructionIsCandidateForBF16Output(
 
 void BFloat16Propagation::AdjustCalledComputationParameters(
     HloInstruction* hlo) {
-  auto adjust_computation =
-      [this, hlo](HloComputation* computation,
-                  absl::Span<HloInstruction* const> operands) {
-        // Adjust parameters.
-        CHECK_EQ(operands.size(), computation->num_parameters());
-        for (int64 i = 0; i < operands.size(); ++i) {
-          auto parameter = computation->parameter_instruction(i);
-          ShapeUtil::ForEachSubshape(
-              parameter->shape(),
-              [this, i, hlo, &operands, parameter](const Shape& /* subshape */,
-                                                   const ShapeIndex& index) {
-                if (!ShapeUtil::IsLeafIndex(parameter->shape(), index)) {
-                  return;
-                }
-                PrimitiveType operand_type =
-                    OutputTypeAfterChange(operands[i], index);
-                if (OutputTypeAfterChange(parameter, index) == operand_type) {
-                  return;
-                }
-                AddToOrRemoveFromBF16ChangeSet(parameter, index, operand_type);
-                VLOG(2) << "Called computation parameter "
-                        << parameter->ToString() << " at shape index " << index
-                        << " adjusted to "
-                        << (operand_type == BF16 ? "BF16" : "F32")
-                        << " to match operand in HLO " << hlo->ToString();
-              });
-        }
-      };
+  auto adjust_computation = [this, hlo](
+                                HloComputation* computation,
+                                absl::Span<HloInstruction* const> operands) {
+    // Adjust parameters.
+    CHECK_EQ(operands.size(), computation->num_parameters());
+    for (int64_t i = 0; i < operands.size(); ++i) {
+      auto parameter = computation->parameter_instruction(i);
+      ShapeUtil::ForEachSubshape(
+          parameter->shape(),
+          [this, i, hlo, &operands, parameter](const Shape& /* subshape */,
+                                               const ShapeIndex& index) {
+            if (!ShapeUtil::IsLeafIndex(parameter->shape(), index)) {
+              return;
+            }
+            PrimitiveType operand_type =
+                OutputTypeAfterChange(operands[i], index);
+            if (OutputTypeAfterChange(parameter, index) == operand_type) {
+              return;
+            }
+            AddToOrRemoveFromBF16ChangeSet(parameter, index, operand_type);
+            VLOG(2) << "Called computation parameter " << parameter->ToString()
+                    << " at shape index " << index << " adjusted to "
+                    << (operand_type == BF16 ? "BF16" : "F32")
+                    << " to match operand in HLO " << hlo->ToString();
+          });
+    }
+  };
 
   switch (hlo->opcode()) {
     case HloOpcode::kFusion:
@@ -507,7 +498,7 @@ void BFloat16Propagation::AdjustCalledComputationParameters(
       adjust_computation(hlo->while_body(), hlo->operands());
       break;
     case HloOpcode::kConditional:
-      for (int64 i = 0; i < hlo->branch_count(); ++i) {
+      for (int64_t i = 0; i < hlo->branch_count(); ++i) {
         adjust_computation(hlo->branch_computation(i),
                            {hlo->mutable_operand(i + 1)});
       }
@@ -579,80 +570,120 @@ bool BFloat16Propagation::ResolveInconsistencyOfAliasingBuffersHelper(
   auto insts = computation->MakeInstructionPostOrder();
   // Do the adjustment on each instruction in the computation in reverse
   // topological order.
-  for (auto inst_it = insts.rbegin(); inst_it != insts.rend(); ++inst_it) {
-    auto hlo = *inst_it;
-    auto adjust_hlo_output = [this, hlo, &parameter_changed](
-                                 const Shape& /* subshape */,
-                                 const ShapeIndex& index) {
-      auto output_type = OutputTypeAfterChange(hlo, index);
-      if (output_type != F32 && output_type != BF16) {
-        return;
-      }
-      PrimitiveType type = BF16;
-      for (const auto* value : dataflow_->GetValueSet(hlo, index).values()) {
-        auto value_type = ValueTypeAfterChange(value);
-        if (value_type == BF16) {
-          continue;
+  while (true) {
+    bool any_change = false;
+    for (auto inst_it = insts.rbegin(); inst_it != insts.rend(); ++inst_it) {
+      auto hlo = *inst_it;
+      auto adjust_hlo_output = [&](const Shape& /* subshape */,
+                                   const ShapeIndex& index) {
+        auto output_type = OutputTypeAfterChange(hlo, index);
+        VLOG(2) << "output_type is " << ((output_type == BF16) ? "BF16" : "F32")
+                << " for :" << hlo->ToString() << "\n";
+        if (output_type != F32 && output_type != BF16) {
+          return;
         }
-        CHECK_EQ(value_type, F32);
-        type = F32;
-        break;
-      }
-      // It's possible that a user has been changed from BF16 to F32
-      // during this final adjustment pass, so we need to check
-      // AllUsersConsumeBF16() again.
-      if (type == BF16 && !AllUsersConsumeBF16(*hlo, index)) {
-        type = F32;
-      }
-      if (type == F32) {
+        PrimitiveType type = BF16;
         for (const auto* value : dataflow_->GetValueSet(hlo, index).values()) {
-          // We rely on the fact that this adjustment works in reverse
-          // topological order. Adding the value to
-          // values_that_must_be_kept_as_f32_ will ensure the correctness
-          // of the adjustment for HLOs that will be processed later.
-          values_that_must_be_kept_as_f32_.insert(value);
+          auto value_type = ValueTypeAfterChange(value);
+          if (value_type == BF16) {
+            continue;
+          }
+          VLOG(2) << "Adjust to F32 due to aliased dataflow value: "
+                  << value->ToString() << "\n";
+          CHECK_EQ(value_type, F32);
+          type = F32;
+          break;
+        }
+        // In order to find aliases due to in-place operations, use
+        // GetInPlaceInputOutputPairs. Ideally, we'd use HloAliasAnalysis here,
+        // but this code works with HloModules that aren't ready yet to use
+        // HloAliasAnalysis (e.g., their computation graphs may not have been
+        // flattened yet).
+        for (const auto& operand_and_output_index :
+             HloDataflowAnalysis::GetInPlaceInputOutputPairs(hlo)) {
+          if (operand_and_output_index.second == index) {
+            const HloOperandIndex& operand_index =
+                operand_and_output_index.first;
+            for (const auto* value :
+                 dataflow_
+                     ->GetValueSet(hlo->operand(operand_index.operand_number),
+                                   operand_index.operand_index)
+                     .values()) {
+              auto value_type = ValueTypeAfterChange(value);
+              if (value_type == BF16) {
+                continue;
+              }
+              VLOG(2) << "Adjust to F32 due to InputOutPair: "
+                      << value->ToString() << "\n";
+              CHECK_EQ(value_type, F32);
+              type = F32;
+              break;
+            }
+          }
+        }
+
+        // It's possible that a user has been changed from BF16 to F32
+        // during this final adjustment pass, so we need to check
+        // AllUsersConsumeBF16() again.
+        if (type == BF16 && !AllUsersConsumeBF16(*hlo, index)) {
+          VLOG(2) << "Adjust to F32 due to All user consumeBF16 fail\n";
+          type = F32;
+        }
+        if (type == F32) {
+          for (const auto* value :
+               dataflow_->GetValueSet(hlo, index).values()) {
+            // We rely on the fact that this adjustment works in reverse
+            // topological order. Adding the value to
+            // values_that_must_be_kept_as_f32_ will ensure the correctness
+            // of the adjustment for HLOs that will be processed later.
+            values_that_must_be_kept_as_f32_.insert(value);
+          }
+        }
+        if (type != output_type) {
+          any_change = true;
+          AddToOrRemoveFromBF16ChangeSet(hlo, index, type);
+          VLOG(2) << "HloInstruction output at shape index " << index
+                  << " adjusted to " << (type == BF16 ? "BF16" : "F32") << ": "
+                  << hlo->ToString();
+          if (hlo->opcode() == HloOpcode::kParameter) {
+            parameter_changed = true;
+          }
+        }
+      };
+      ShapeUtil::ForEachSubshape(hlo->shape(), adjust_hlo_output);
+      AdjustCalledComputationRoot(hlo);
+      if (hlo->opcode() == HloOpcode::kWhile) {
+        // We need to run on the while body and condition repeatedly until a
+        // fixed point is reached, i.e., the parameters do not change any more.
+        // We may need more than one iteration because the while input and
+        // output alias each other, so changing one input parameter requires
+        // changing the corresponding output element and thus may transitively
+        // require changing another input parameter. A fixed point will be
+        // reached because the parameters can only be changed from BF16 to F32,
+        // not the other way around.
+        absl::flat_hash_set<const HloComputation*> visited_in_while;
+        while (ResolveInconsistencyOfAliasingBuffersHelper(
+                   hlo->while_condition(), &visited_in_while) ||
+               ResolveInconsistencyOfAliasingBuffersHelper(hlo->while_body(),
+                                                           &visited_in_while)) {
+          visited_in_while.clear();
+          ShapeUtil::ForEachSubshape(hlo->shape(), adjust_hlo_output);
+          AdjustCalledComputationRoot(hlo);
+        }
+        visited_computations->insert(visited_in_while.begin(),
+                                     visited_in_while.end());
+      } else if (hlo->opcode() == HloOpcode::kFusion) {
+        ResolveInconsistencyOfAliasingBuffersHelper(
+            hlo->fused_instructions_computation(), visited_computations);
+      } else if (hlo->opcode() == HloOpcode::kConditional) {
+        for (auto* branch : hlo->branch_computations()) {
+          ResolveInconsistencyOfAliasingBuffersHelper(branch,
+                                                      visited_computations);
         }
       }
-      if (type != output_type) {
-        AddToOrRemoveFromBF16ChangeSet(hlo, index, type);
-        VLOG(2) << "HloInstruction output at shape index " << index
-                << " adjusted to " << (type == BF16 ? "BF16" : "F32") << ": "
-                << hlo->ToString();
-        if (hlo->opcode() == HloOpcode::kParameter) {
-          parameter_changed = true;
-        }
-      }
-    };
-    ShapeUtil::ForEachSubshape(hlo->shape(), adjust_hlo_output);
-    AdjustCalledComputationRoot(hlo);
-    if (hlo->opcode() == HloOpcode::kWhile) {
-      // We need to run on the while body and condition repeatedly until a fixed
-      // point is reached, i.e., the parameters do not change any more. We may
-      // need more than one iteration because the while input and output alias
-      // each other, so changing one input parameter requires changing the
-      // corresponding output element and thus may transitively require changing
-      // another input parameter. A fixed point will be reached because the
-      // parameters can only be changed from BF16 to F32, not the other way
-      // around.
-      absl::flat_hash_set<const HloComputation*> visited_in_while;
-      while (ResolveInconsistencyOfAliasingBuffersHelper(hlo->while_condition(),
-                                                         &visited_in_while) ||
-             ResolveInconsistencyOfAliasingBuffersHelper(hlo->while_body(),
-                                                         &visited_in_while)) {
-        visited_in_while.clear();
-        ShapeUtil::ForEachSubshape(hlo->shape(), adjust_hlo_output);
-        AdjustCalledComputationRoot(hlo);
-      }
-      visited_computations->insert(visited_in_while.begin(),
-                                   visited_in_while.end());
-    } else if (hlo->opcode() == HloOpcode::kFusion) {
-      ResolveInconsistencyOfAliasingBuffersHelper(
-          hlo->fused_instructions_computation(), visited_computations);
-    } else if (hlo->opcode() == HloOpcode::kConditional) {
-      for (auto* branch : hlo->branch_computations()) {
-        ResolveInconsistencyOfAliasingBuffersHelper(branch,
-                                                    visited_computations);
-      }
+    }
+    if (!any_change) {
+      break;
     }
   }
   // Now adjust parameters of called computations.
@@ -663,9 +694,10 @@ bool BFloat16Propagation::ResolveInconsistencyOfAliasingBuffersHelper(
 }
 
 void BFloat16Propagation::ResolveInconsistencyOfAliasingBuffers(
-    HloModule* module) {
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
   const auto& computations_topological_order =
-      module->MakeComputationPostOrder();
+      module->MakeComputationPostOrder(execution_threads);
   absl::flat_hash_set<const HloComputation*> resolved;
   for (auto comp_it = computations_topological_order.rbegin();
        comp_it != computations_topological_order.rend(); ++comp_it) {
@@ -676,7 +708,9 @@ void BFloat16Propagation::ResolveInconsistencyOfAliasingBuffers(
   }
 }
 
-Status BFloat16Propagation::ResolveInconsistentFusions(HloModule* module) {
+Status BFloat16Propagation::ResolveInconsistentFusions(
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
   // We could have changed a fusion computation's root shape to have a different
   // precision than the fusion node's output, if the fusion root does not
   // define a buffer (e.g., a tuple). Now we add conversions after such fusion
@@ -702,7 +736,7 @@ Status BFloat16Propagation::ResolveInconsistentFusions(HloModule* module) {
   // (1) a is F32 but tuple is BF16
   // (2) after adding conversion
   // (3) after tuple simplifier and DCE.
-  for (auto computation : module->MakeComputationPostOrder()) {
+  for (auto computation : module->MakeComputationPostOrder(execution_threads)) {
     auto insts = computation->MakeInstructionPostOrder();
     for (auto inst_it = insts.rbegin(); inst_it != insts.rend(); ++inst_it) {
       auto hlo = *inst_it;
@@ -734,10 +768,12 @@ Status BFloat16Propagation::ResolveInconsistentFusions(HloModule* module) {
       fusion_computation->set_root_instruction(copy);
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
-Status BFloat16Propagation::ResolveConvertedConstants(HloModule* module) {
+Status BFloat16Propagation::ResolveConvertedConstants(
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
   // We may have converted some constants from F32 to BF16, so adjust the
   // constant literals in such cases. We do this here instead of when the
   // constant node's is changed because 1) the HloInstruction interface does not
@@ -748,7 +784,7 @@ Status BFloat16Propagation::ResolveConvertedConstants(HloModule* module) {
   // can avoid repeated conversions.
   //
   // TODO(b/73833576): Consider resetting literal in HloInstruction.
-  for (auto computation : module->MakeComputationPostOrder()) {
+  for (auto computation : module->MakeComputationPostOrder(execution_threads)) {
     for (auto hlo : computation->MakeInstructionPostOrder()) {
       if (hlo->opcode() != HloOpcode::kConstant) {
         continue;
@@ -764,11 +800,13 @@ Status BFloat16Propagation::ResolveConvertedConstants(HloModule* module) {
       }
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
-Status BFloat16Propagation::SkipNoopConversions(HloModule* module) {
-  for (auto computation : module->computations()) {
+Status BFloat16Propagation::SkipNoopConversions(
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
+  for (auto computation : module->computations(execution_threads)) {
     for (auto hlo : computation->MakeInstructionPostOrder()) {
       if (hlo->opcode() != HloOpcode::kConvert) {
         continue;
@@ -784,7 +822,7 @@ Status BFloat16Propagation::SkipNoopConversions(HloModule* module) {
       }
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 // The algorithm first does a forward pass (parameters to root) to determine a
@@ -793,7 +831,9 @@ Status BFloat16Propagation::SkipNoopConversions(HloModule* module) {
 // their users. During the backward pass, the potential changes are stored in
 // changes_to_bf16_ which are subject to further adjustments then applied to the
 // HLOs.
-StatusOr<bool> BFloat16Propagation::Run(HloModule* module) {
+StatusOr<bool> BFloat16Propagation::Run(
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
   consider_using_bfloat16_.clear();
   instructions_visited_in_backward_pass_.clear();
   computations_visited_in_backward_pass_.clear();
@@ -802,7 +842,8 @@ StatusOr<bool> BFloat16Propagation::Run(HloModule* module) {
   changes_to_bf16_.clear();
   changed_ = false;
 
-  auto computations_topological_order = module->MakeComputationPostOrder();
+  auto computations_topological_order =
+      module->MakeComputationPostOrder(execution_threads);
 
   // Before running the propagation pass, we insert copies (kConvert to the same
   // type) of F32 inputs to while loops. This prevents other uses of the same
@@ -870,7 +911,7 @@ StatusOr<bool> BFloat16Propagation::Run(HloModule* module) {
   // It's possible that an instruction does not define a buffer, but the
   // defining instruction's shape has changed. So we need to adjust the output
   // shapes of instructions according to the HLO values they refer to.
-  ResolveInconsistencyOfAliasingBuffers(module);
+  ResolveInconsistencyOfAliasingBuffers(module, execution_threads);
 
   // Apply the changes in changes_to_bf16_.
   for (auto& change : changes_to_bf16_) {
@@ -919,13 +960,14 @@ StatusOr<bool> BFloat16Propagation::Run(HloModule* module) {
   // Removes redundant HLOs added by this pass, either when inserting
   // de-aliasing copies to while loop inputs, or later when converting output
   // types.
-  auto clean_up = [this, module]() {
-    TF_RETURN_IF_ERROR(SkipNoopConversions(module));
+  auto clean_up = [this, module, &execution_threads]() {
+    TF_RETURN_IF_ERROR(SkipNoopConversions(module, execution_threads));
     TupleSimplifier tuple_simplifier;
-    TF_RETURN_IF_ERROR(tuple_simplifier.Run(module).status());
+    TF_RETURN_IF_ERROR(
+        tuple_simplifier.Run(module, execution_threads).status());
     HloDCE dce;
-    TF_RETURN_IF_ERROR(dce.Run(module).status());
-    return Status::OK();
+    TF_RETURN_IF_ERROR(dce.Run(module, execution_threads).status());
+    return OkStatus();
   };
 
   if (!changed_) {
@@ -933,8 +975,8 @@ StatusOr<bool> BFloat16Propagation::Run(HloModule* module) {
     return false;
   }
 
-  TF_RETURN_IF_ERROR(ResolveInconsistentFusions(module));
-  TF_RETURN_IF_ERROR(ResolveConvertedConstants(module));
+  TF_RETURN_IF_ERROR(ResolveInconsistentFusions(module, execution_threads));
+  TF_RETURN_IF_ERROR(ResolveConvertedConstants(module, execution_threads));
 
   TF_RETURN_IF_ERROR(clean_up());
   return true;
