@@ -22,7 +22,7 @@ limitations under the License.
 #include "llvm/Transforms/Utils/Cloning.h"
 #include "tensorflow/compiler/xla/service/cpu/vector_support_library.h"
 #include "tensorflow/compiler/xla/service/llvm_ir/math_ops.h"
-#include "tensorflow/core/platform/logging.h"
+#include "tensorflow/tsl/platform/logging.h"
 
 namespace xla {
 namespace cpu {
@@ -83,9 +83,9 @@ void RemoveFunctionFromUsedList(llvm::Module* module, llvm::Function* fn) {
 void RewriteCalls(
     llvm::Module* module, const char* fn_name,
     std::function<llvm::Value*(llvm::IRBuilder<>* b, llvm::Value* input,
-                               int32 vector_width)>
+                               int32_t vector_width)>
         fn_body_generator,
-    int32 vector_width, llvm::FastMathFlags fast_math_flags) {
+    int32_t vector_width, llvm::FastMathFlags fast_math_flags) {
   llvm::Function* fn = module->getFunction(fn_name);
   if (fn == nullptr) {
     // If the function declaration is not present in the module, there can't be
@@ -115,14 +115,15 @@ void RewriteCalls(
 
   // Upcast to vector type if input is a scalar.
   if (vector_width == 1) {
-    llvm::Type* v1_type = llvm::VectorType::get(input->getType(), 1);
+    llvm::Type* v1_type = llvm::VectorType::get(input->getType(), 1, false);
     input = b.CreateInsertElement(llvm::UndefValue::get(v1_type), input,
                                   uint64_t{0});
   }
 
   // Generate the vectorized code.
-  CHECK_EQ(vector_width,
-           llvm::cast<llvm::VectorType>(input->getType())->getNumElements());
+  CHECK_EQ(
+      vector_width,
+      llvm::cast<llvm::FixedVectorType>(input->getType())->getNumElements());
   llvm::Value* result = fn_body_generator(&b, input, vector_width);
 
   // Downcast result to scalar type if necessary.
@@ -154,12 +155,12 @@ void RewriteCalls(
 }
 
 llvm::Value* GenerateVF32Tanh(llvm::IRBuilder<>* b, llvm::Value* input,
-                              int32 /*vector_width*/) {
+                              int32_t /*vector_width*/) {
   return llvm_ir::EmitFastTanh(b, input);
 }
 
 llvm::Value* GenerateVF32Exp(llvm::IRBuilder<>* b, llvm::Value* input,
-                             int32 vector_width) {
+                             int32_t vector_width) {
   VectorSupportLibrary vsl(F32, vector_width, b, "exp_f32");
 
   // This implements the same polynomial approximation as implemented in Cephes.
@@ -264,15 +265,15 @@ llvm::Value* GenerateVF32Exp(llvm::IRBuilder<>* b, llvm::Value* input,
   z = vsl.Add(one, z);
 
   // Convert n' to an i32.  This is safe because we clamped it above.
-  llvm::Value* n_i32 =
-      b->CreateFPToSI(n, llvm::VectorType::get(b->getInt32Ty(), vector_width));
+  llvm::Value* n_i32 = b->CreateFPToSI(
+      n, llvm::VectorType::get(b->getInt32Ty(), vector_width, false));
 
-  auto splat_i32 = [&](int32 v) {
+  auto splat_i32 = [&](int32_t v) {
     return b->CreateVectorSplat(vector_width, b->getInt32(v));
   };
 
   // Creates the value 2^n' if -126 <= n' <= 127 and 0 if n' = -127.
-  const int32 kF32SignificandBits = 23;
+  const int32_t kF32SignificandBits = 23;
   llvm::Value* exp_bias = splat_i32(0x7f);
   llvm::Value* pow2 =
       b->CreateBitCast(b->CreateShl(b->CreateAdd(n_i32, exp_bias),
@@ -284,7 +285,7 @@ llvm::Value* GenerateVF32Exp(llvm::IRBuilder<>* b, llvm::Value* input,
 }
 
 llvm::Value* GenerateVF32Log(llvm::IRBuilder<>* b, llvm::Value* input,
-                             int32 vector_width) {
+                             int32_t vector_width) {
   VectorSupportLibrary vsl(F32, vector_width, b, "log_f32");
 
   const llvm::APFloat half = GetIeeeF32(0.5);
@@ -329,7 +330,7 @@ llvm::Value* GenerateVF32Log(llvm::IRBuilder<>* b, llvm::Value* input,
   llvm::Value* vector_constant_23 =
       b->CreateVectorSplat(vector_width, b->getInt32(23));
   llvm::Type* i32_vector_type =
-      llvm::VectorType::get(b->getInt32Ty(), vector_width);
+      llvm::VectorType::get(b->getInt32Ty(), vector_width, false);
 
   llvm::Value* emm0 = b->CreateLShr(b->CreateBitCast(tmp0, i32_vector_type),
                                     vector_constant_23);
