@@ -15,19 +15,19 @@ limitations under the License.
 
 #include "tensorflow/compiler/xla/service/hlo_lexer.h"
 
-#include <unordered_map>
+#include <cstring>
+#include <limits>
+#include <optional>
+#include <string>
+#include <utility>
 
 #include "absl/base/casts.h"
 #include "absl/strings/ascii.h"
 #include "absl/strings/escaping.h"
 #include "absl/strings/numbers.h"
-#include "absl/strings/str_split.h"
-#include "absl/types/optional.h"
-#include "tensorflow/compiler/xla/shape_util.h"
 #include "tensorflow/compiler/xla/statusor.h"
 #include "tensorflow/compiler/xla/util.h"
-#include "tensorflow/core/lib/strings/numbers.h"
-#include "tensorflow/core/platform/regexp.h"
+#include "tensorflow/tsl/platform/numbers.h"
 
 namespace xla {
 namespace {
@@ -54,7 +54,7 @@ int HloLexer::GetNextChar() {
 }
 
 int HloLexer::PeekCurrentChar() const {
-  if (current_ptr_ == buf_.end()) {
+  if (current_ptr_ == buf_.data() + buf_.size()) {
     return kEOF;
   }
   char current_char = *current_ptr_;
@@ -66,14 +66,14 @@ int HloLexer::PeekCurrentChar() const {
 }
 
 bool HloLexer::CanDereference(const char* ptr) const {
-  return ptr < buf_.end() && ptr >= buf_.begin();
+  return (ptr < buf_.data() + buf_.size()) && ptr >= buf_.data();
 }
 
-absl::string_view HloLexer::StringPieceFromPointers(const char* begin,
-                                                    const char* end) const {
+absl::string_view HloLexer::StringViewFromPointers(const char* begin,
+                                                   const char* end) const {
   CHECK(begin <= end);
-  CHECK(begin == buf_.end() || CanDereference(begin));
-  CHECK(end == buf_.end() || CanDereference(end));
+  CHECK((begin == buf_.data() + buf_.size()) || CanDereference(begin));
+  CHECK((end == buf_.data() + buf_.size()) || CanDereference(end));
   return absl::string_view(begin, end - begin);
 }
 
@@ -127,6 +127,7 @@ TokKind HloLexer::LexToken() {
       case '8':
       case '9':
       case '-':
+      case '?':
         if (current_char == '-' && PeekCurrentChar() == '>') {
           current_ptr_++;
           return TokKind::kArrow;
@@ -148,6 +149,12 @@ TokKind HloLexer::LexToken() {
         return TokKind::kColon;
       case '*':
         return TokKind::kAsterisk;
+      case '#':
+        return TokKind::kOctothorp;
+      case '+':
+        return TokKind::kPlus;
+      case '~':
+        return TokKind::kTilde;
       case '[':
         return TokKind::kLsquare;
       case ']':
@@ -222,6 +229,28 @@ TokKind HloLexer::LexToken() {
   }
 }
 
+std::optional<int64_t> HloLexer::LexNanPayload(absl::string_view& consumable) {
+  static LazyRE2 payload_pattern = {R"(\(0x[0-9a-fA-F]+\))"};
+  if (!RE2::Consume(&consumable, *payload_pattern)) {
+    return std::nullopt;
+  }
+  auto slice = StringViewFromPointers(current_ptr_, consumable.data());
+  current_ptr_ = consumable.data();
+  CHECK(absl::StartsWith(slice, "(0x"));
+  slice.remove_prefix(std::strlen("(0x"));
+  CHECK(absl::EndsWith(slice, ")"));
+  slice.remove_suffix(std::strlen(")"));
+  uint64_t payload_value;
+  if (tsl::strings::HexStringToUint64(slice, &payload_value)) {
+    if (payload_value <= 0 || payload_value > NanPayloadBitMask<double>()) {
+      LOG(ERROR) << "NaN payload out of range: " << payload_value;
+      return std::nullopt;
+    }
+    return payload_value;
+  }
+  return std::nullopt;
+}
+
 // Lex a shape, name, keyword, attribute name, the dim labels pattern, and
 // other identifiers.
 //
@@ -229,7 +258,7 @@ TokKind HloLexer::LexToken() {
 // name     ::= [a-zA-Z_][a-zA-Z0-9_.-]*:
 // keyword  ::= HloModule, ENTRY, ...
 // attribute_name ::= condition, body, dimensions, ...
-// dim_labels_pattern ::= [0-9bf]{2,}_[0-9io]{2,}->[0-9bf]{2,}
+// dim_labels_pattern ::= [0-9bf?]{2,}_[0-9io?]{2,}->[0-9bf?]{2,}
 // identifiers ::= other cases that match [a-zA-Z_][a-zA-Z0-9_.-]*
 TokKind HloLexer::LexIdentifier() {
   while (IsIdentifierChar(PeekCurrentChar())) {
@@ -251,19 +280,34 @@ TokKind HloLexer::LexIdentifier() {
   }
 
   absl::string_view identifier =
-      StringPieceFromPointers(token_state_.token_start, current_ptr_);
+      StringViewFromPointers(token_state_.token_start, current_ptr_);
 
   // Primitive type strings are reserved words. The exception is 'tuple' whose
   // type is represented using nested parentheses without the string 'tuple'.
   if (primitive_util::IsPrimitiveTypeName(identifier)) {
     PrimitiveType primitive_type =
-        primitive_util::StringToPrimitiveType(identifier).ValueOrDie();
+        primitive_util::StringToPrimitiveType(identifier).value();
     if (primitive_type != TUPLE) {
       token_state_.primitive_type_val = primitive_type;
       return TokKind::kPrimitiveType;
     }
   }
 
+  if (identifier == "nan") {
+    std::optional<int64_t> payload;
+    if (PeekCurrentChar() == '(') {
+      absl::string_view consumable =
+          StringViewFromPointers(current_ptr_, buf_.data() + buf_.size());
+      payload = LexNanPayload(consumable);
+      if (!payload.has_value()) {
+        return TokKind::kError;
+      }
+    }
+    token_state_.decimal_val = NanWithSignAndPayload<double>(
+        /*sign=*/false, payload.value_or(QuietNanWithoutPayload<double>()));
+    return TokKind::kDecimal;
+  }
+
   // See if this is a keyword.
 #define KEYWORD(STR)            \
   do {                          \
@@ -275,28 +319,29 @@ TokKind HloLexer::LexIdentifier() {
   KEYWORD(true);
   KEYWORD(false);
   KEYWORD(inf);
-  KEYWORD(nan);
   KEYWORD(HloModule);
   KEYWORD(ENTRY);
   KEYWORD(ROOT);
   KEYWORD(maximal);
   KEYWORD(replicated);
+  KEYWORD(manual);
+  KEYWORD(last_tile_dim_replicate);
 
 #undef KEYWORD
 
   {
-    absl::string_view consumable =
-        StringPieceFromPointers(token_state_.token_start, buf_.end());
+    absl::string_view consumable = StringViewFromPointers(
+        token_state_.token_start, buf_.data() + buf_.size());
     static LazyRE2 dim_labels_pattern = {
-        R"([0-9bf]{2,}_[0-9io]{2,}->[0-9bf]{2,})"};
+        R"([0-9bf?]{2,}_[0-9io?]{2,}->[0-9bf?]{2,})"};
     if (RE2::Consume(&consumable, *dim_labels_pattern)) {
-      current_ptr_ = consumable.begin();
+      current_ptr_ = consumable.data();
       token_state_.str_val.assign(token_state_.token_start, current_ptr_);
       return TokKind::kDimLabels;
     }
   }
 
-  token_state_.str_val = string(identifier);
+  token_state_.str_val = std::string(identifier);
   return TokKind::kIdent;
 }
 
@@ -321,59 +366,58 @@ TokKind HloLexer::LexPercent() {
 //
 // fp with exp ::= [-]?([0-9]+|[0-9]+[.][0-9]*|[0-9]*[.][0-9]+)([eE][+-]?[0-9]+)
 // fp without exp ::= [-]?([0-9]+[.][0-9]*|[0-9]*[.][0-9]+)
-// dim_labels_pattern ::= [0-9bf]{2,}_[0-9io]{2,}->[0-9bf]{2,}
+// dim_labels_pattern ::= [0-9bf?]{2,}_[0-9io?]{2,}->[0-9bf?]{2,}
 // dxd_pattern ::= [0-9]+(x[0-9]+)+
 // pad_pattern ::=
 //   [-]?[0-9]+_[-]?[0-9]+(_[0-9]+)?(x[-]?[0-9]+_[-]?[0-9]+(_[0-9]+)?)*
 // int ::=  [-]?[0-9]+
 // negative inf ::= '-inf'
 TokKind HloLexer::LexNumberOrPattern() {
-  absl::string_view consumable =
-      StringPieceFromPointers(token_state_.token_start, buf_.end());
+  absl::string_view consumable = StringViewFromPointers(
+      token_state_.token_start, buf_.data() + buf_.size());
   static LazyRE2 float_pattern = {
       R"([-]?((\d+|\d+[.]\d*|\d*[.]\d+)([eE][+-]?\d+))|[-]?(\d+[.]\d*|\d*[.]\d+))"};
   if (RE2::Consume(&consumable, *float_pattern)) {
-    current_ptr_ = consumable.begin();
-    CHECK(absl::SimpleAtod(string(token_state_.token_start, current_ptr_),
+    current_ptr_ = consumable.data();
+    CHECK(absl::SimpleAtod(std::string(token_state_.token_start, current_ptr_),
                            &token_state_.decimal_val));
     return TokKind::kDecimal;
   }
 
   static LazyRE2 dim_labels_pattern = {
-      R"([0-9bf]{2,}_[0-9io]{2,}->[0-9bf]{2,})"};
+      R"([0-9bf?]{2,}_[0-9io?]{2,}->[0-9bf?]{2,})"};
   static LazyRE2 dxd_pattern = {R"([0-9]+(x[0-9]+)+)"};
   static LazyRE2 pad_pattern = {
       R"([-]?[0-9]+_[-]?[0-9]+(_[0-9]+)?(x[-]?[0-9]+_[-]?[0-9]+(_[0-9]+)?)*)"};
 
   if (RE2::Consume(&consumable, *dim_labels_pattern)) {
-    current_ptr_ = consumable.begin();
+    current_ptr_ = consumable.data();
     token_state_.str_val.assign(token_state_.token_start, current_ptr_);
     return TokKind::kDimLabels;
   }
 
   if (RE2::Consume(&consumable, *dxd_pattern)) {
-    current_ptr_ = consumable.begin();
+    current_ptr_ = consumable.data();
     token_state_.str_val.assign(token_state_.token_start, current_ptr_);
     return TokKind::kDxD;
   }
 
   if (RE2::Consume(&consumable, *pad_pattern)) {
-    current_ptr_ = consumable.begin();
+    current_ptr_ = consumable.data();
     token_state_.str_val.assign(token_state_.token_start, current_ptr_);
     return TokKind::kPad;
   }
 
   static LazyRE2 int_pattern = {R"([-]?\d+)"};
   if (RE2::Consume(&consumable, *int_pattern)) {
-    current_ptr_ = consumable.begin();
-    auto slice =
-        StringPieceFromPointers(token_state_.token_start, current_ptr_);
+    current_ptr_ = consumable.data();
+    auto slice = StringViewFromPointers(token_state_.token_start, current_ptr_);
     if (absl::SimpleAtoi(slice, &token_state_.int64_val)) {
       return TokKind::kInt;
     }
-    uint64 uint64_val;
+    uint64_t uint64_val;
     if (absl::SimpleAtoi(slice, &uint64_val)) {
-      token_state_.int64_val = absl::bit_cast<int64>(uint64_val);
+      token_state_.int64_val = absl::bit_cast<int64_t>(uint64_val);
       return TokKind::kInt;
     }
     LOG(ERROR) << "Failed to parse int literal: " << slice;
@@ -382,16 +426,32 @@ TokKind HloLexer::LexNumberOrPattern() {
 
   static LazyRE2 neg_inf = {"-inf"};
   if (RE2::Consume(&consumable, *neg_inf)) {
-    current_ptr_ = consumable.begin();
+    current_ptr_ = consumable.data();
     return TokKind::kNegInf;
   }
 
+  static LazyRE2 neg_nan = {"-nan"};
+  if (RE2::Consume(&consumable, *neg_nan)) {
+    current_ptr_ = consumable.data();
+
+    std::optional<int64_t> payload;
+    if (PeekCurrentChar() == '(') {
+      payload = LexNanPayload(consumable);
+      if (!payload.has_value()) {
+        return TokKind::kError;
+      }
+    }
+    token_state_.decimal_val = NanWithSignAndPayload<double>(
+        /*sign=*/true, payload.value_or(QuietNanWithoutPayload<double>()));
+    return TokKind::kDecimal;
+  }
+
   return TokKind::kError;
 }
 
 std::pair<unsigned, unsigned> HloLexer::GetLineAndColumn(LocTy location) const {
   unsigned line_no = 1;
-  const char* start = buf_.begin();
+  const char* start = buf_.data();
   const char* ptr = start;
   if (line_no_cache_.last_query && CanDereference(line_no_cache_.last_query) &&
       line_no_cache_.last_query <= location) {
@@ -399,7 +459,7 @@ std::pair<unsigned, unsigned> HloLexer::GetLineAndColumn(LocTy location) const {
     line_no = line_no_cache_.line_no_of_query;
   }
   for (; ptr != location; ptr++) {
-    CHECK_LT(ptr, buf_.end());
+    CHECK_LT(ptr, buf_.data() + buf_.size());
     if (*ptr == '\n') {
       line_no++;
     }
@@ -408,7 +468,7 @@ std::pair<unsigned, unsigned> HloLexer::GetLineAndColumn(LocTy location) const {
   // Update the line number cache.
   line_no_cache_.last_query = ptr;
   line_no_cache_.line_no_of_query = line_no;
-  size_t line_offset = StringPieceFromPointers(start, ptr).rfind('\n');
+  size_t line_offset = StringViewFromPointers(start, ptr).rfind('\n');
   if (line_offset == absl::string_view::npos) {
     line_offset = 0;
   }
@@ -419,29 +479,30 @@ absl::string_view HloLexer::GetLine(LocTy loc) const {
   if (!CanDereference(loc)) {
     return "LINE OUT OF RANGE";
   }
-  size_t line_start =
-      StringPieceFromPointers(buf_.begin(), loc + 1).rfind('\n');
+  size_t line_start = StringViewFromPointers(buf_.data(), loc + 1).rfind('\n');
   const char* start = line_start == absl::string_view::npos
-                          ? buf_.begin()
-                          : buf_.begin() + line_start + 1;
-  size_t line_end = StringPieceFromPointers(loc, buf_.end()).find('\n');
-  const char* end =
-      line_end == absl::string_view::npos ? buf_.end() : loc + line_end;
+                          ? buf_.data()
+                          : buf_.data() + line_start + 1;
+  size_t line_end =
+      StringViewFromPointers(loc, buf_.data() + buf_.size()).find('\n');
+  const char* end = line_end == absl::string_view::npos
+                        ? buf_.data() + buf_.size()
+                        : loc + line_end;
 
-  return StringPieceFromPointers(start, end);
+  return StringViewFromPointers(start, end);
 }
 
 // Lexes quoted string with escaping characters. If matched, the quoted string
 // will be unescaped and stored to token_state_.str_val.
 TokKind HloLexer::LexString() {
-  absl::string_view consumable =
-      StringPieceFromPointers(token_state_.token_start, buf_.end());
+  absl::string_view consumable = StringViewFromPointers(
+      token_state_.token_start, buf_.data() + buf_.size());
   static LazyRE2 escaping_pattern = {R"("([^"\\]|\\.)*")"};
   if (RE2::Consume(&consumable, *escaping_pattern)) {
-    current_ptr_ = consumable.begin();
+    current_ptr_ = consumable.data();
     absl::string_view raw =
-        StringPieceFromPointers(token_state_.token_start + 1, current_ptr_ - 1);
-    string error;
+        StringViewFromPointers(token_state_.token_start + 1, current_ptr_ - 1);
+    std::string error;
     if (!absl::CUnescape(raw, &token_state_.str_val, &error)) {
       LOG(ERROR) << "Failed unescaping string: " << raw << ". error: " << error;
       return TokKind::kError;
@@ -451,20 +512,66 @@ TokKind HloLexer::LexString() {
   return TokKind::kError;
 }
 
-string TokKindToString(TokKind kind) {
+TokKind HloLexer::LexJsonDict() {
+  // We require that you've already lexed the open curly brace.
+  if (GetKind() != TokKind::kLbrace) return TokKind::kError;
+
+  absl::string_view orig = StringViewFromPointers(token_state_.token_start,
+                                                  buf_.data() + buf_.size());
+  absl::string_view str = orig;
+
+  int64_t object_depth = 0;
+  if (str.empty()) return TokKind::kError;
+
+  if (str.front() != '{') return TokKind::kError;
+  ++object_depth;
+  str = str.substr(1);
+
+  while (!str.empty()) {
+    if (object_depth == 0) break;
+
+    if (str.front() == '"') {
+      static LazyRE2 string_pattern = {R"("([^"\\]|\\.)*")"};
+      if (!RE2::Consume(&str, *string_pattern)) {
+        return TokKind::kError;
+      }
+      continue;
+    }
+
+    if (str.front() == '{') ++object_depth;
+    if (str.front() == '}') --object_depth;
+    str = str.substr(1);
+  }
+  if (object_depth != 0) {
+    return TokKind::kError;
+  }
+  current_ptr_ = str.data();
+  token_state_.current_kind = TokKind::kString;
+  token_state_.str_val =
+      std::string(orig.substr(0, orig.length() - str.length()));
+  return TokKind::kString;
+}
+
+std::string TokKindToString(TokKind kind) {
   switch (kind) {
     case TokKind::kEof:
       return "kEof";
     case TokKind::kError:
       return "kError";
     case TokKind::kEqual:
-      return "kEqaul";
+      return "kEqual";
     case TokKind::kComma:
       return "kComma";
     case TokKind::kColon:
       return "kColon";
     case TokKind::kAsterisk:
       return "kAsterisk";
+    case TokKind::kOctothorp:
+      return "kOctothorp";
+    case TokKind::kPlus:
+      return "kPlus";
+    case TokKind::kTilde:
+      return "kTilde";
     case TokKind::kLsquare:
       return "kLsquare";
     case TokKind::kRsquare:
@@ -495,8 +602,10 @@ string TokKindToString(TokKind kind) {
       return "kw_maximal";
     case TokKind::kw_replicated:
       return "kw_replicated";
-    case TokKind::kw_nan:
-      return "kw_nan";
+    case TokKind::kw_manual:
+      return "kw_manual";
+    case TokKind::kw_last_tile_dim_replicate:
+      return "kw_last_tile_dim_replicate";
     case TokKind::kw_inf:
       return "kw_inf";
     case TokKind::kNegInf:
@@ -526,4 +635,11 @@ string TokKindToString(TokKind kind) {
   }
 }
 
+bool LexesAsJsonDict(absl::string_view str) {
+  HloLexer lexer(str);
+  return lexer.Lex() == TokKind::kLbrace &&
+         lexer.LexJsonDict() == TokKind::kString &&
+         lexer.Lex() == TokKind::kEof;
+}
+
 }  // namespace xla
