@@ -16,27 +16,43 @@ limitations under the License.
 #include "tensorflow/compiler/xla/service/instruction_fusion.h"
 
 #include <algorithm>
+#include <functional>
 #include <list>
 #include <memory>
 #include <numeric>
+#include <optional>
+#include <string>
 #include <utility>
 #include <vector>
 
 #include "absl/algorithm/container.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/flat_hash_set.h"
-#include "absl/memory/memory.h"
 #include "tensorflow/compiler/xla/debug_options_flags.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_instruction.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_opcode.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_reachability.h"
 #include "tensorflow/compiler/xla/map_util.h"
 #include "tensorflow/compiler/xla/service/fusion_queue.h"
-#include "tensorflow/compiler/xla/service/hlo_instruction.h"
-#include "tensorflow/compiler/xla/service/hlo_opcode.h"
-#include "tensorflow/compiler/xla/service/hlo_reachability.h"
+#include "tensorflow/compiler/xla/service/hlo_dataflow_analysis.h"
+#include "tensorflow/compiler/xla/service/hlo_graph_dumper.h"
 #include "tensorflow/compiler/xla/service/pattern_matcher.h"
-#include "tensorflow/core/lib/core/errors.h"
-#include "tensorflow/core/platform/logging.h"
+#include "tensorflow/tsl/platform/errors.h"
+#include "tensorflow/tsl/platform/logging.h"
 
 namespace xla {
+
+#if defined(PLATFORM_GOOGLE)
+FusionDecision::FusionDecision(bool decision,
+                               absl::SourceLocation source_location) {
+  if (!decision) {
+    explanation_ =
+        absl::StrCat("Not fusing: due to ", source_location.file_name(), ":",
+                     source_location.line());
+  }
+}
+#endif  // PLATFORM_GOOGLE
+
 namespace {
 
 // These nodes can always be duplicated into consumers, even if
@@ -51,9 +67,10 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
   // We are always willing to duplicate a widening type-conversion instruction
   // if it means we can fuse the convert into a consumer.  This allows the
   // consumer to read less memory, which is almost always a performance win.
-  return instruction.opcode() == HloOpcode::kConvert &&
-         ShapeUtil::ByteSizeOf(instruction.operand(0)->shape()) <
-             ShapeUtil::ByteSizeOf(instruction.shape());
+  return (instruction.opcode() == HloOpcode::kConvert &&
+          ShapeUtil::ByteSizeOf(instruction.operand(0)->shape()) <
+              ShapeUtil::ByteSizeOf(instruction.shape())) ||
+         instruction.opcode() == HloOpcode::kBroadcast;
 }
 }  // namespace
 
@@ -92,6 +109,7 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kMultiply:
     case HloOpcode::kNegate:
     case HloOpcode::kNot:
+    case HloOpcode::kOptimizationBarrier:
     case HloOpcode::kOr:
     case HloOpcode::kXor:
     case HloOpcode::kOutfeed:
@@ -102,17 +120,19 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kReducePrecision:
     case HloOpcode::kReplicaId:
     case HloOpcode::kReshape:
+    case HloOpcode::kDynamicReshape:
     case HloOpcode::kReverse:
     case HloOpcode::kRoundNearestAfz:
+    case HloOpcode::kRoundNearestEven:
     case HloOpcode::kSelect:
     case HloOpcode::kShiftLeft:
     case HloOpcode::kShiftRightArithmetic:
     case HloOpcode::kShiftRightLogical:
     case HloOpcode::kSlice:
+    case HloOpcode::kStochasticConvert:
     case HloOpcode::kSubtract:
     case HloOpcode::kTranspose:
     case HloOpcode::kTuple:
-    case HloOpcode::kTupleSelect:
       return false;
 
     // Cheap instructions for reals, but expensive for complex.
@@ -120,6 +140,7 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kCos:
     case HloOpcode::kSign:
     case HloOpcode::kSin:
+    case HloOpcode::kTan:
       return ShapeUtil::ElementIsComplex(instruction.shape());
 
     // We say that integer div/mod by a constant is cheap because it gets
@@ -138,6 +159,9 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kAddDependency:
     case HloOpcode::kAfterAll:
     case HloOpcode::kAtan2:
+    case HloOpcode::kAsyncStart:
+    case HloOpcode::kAsyncUpdate:
+    case HloOpcode::kAsyncDone:
     case HloOpcode::kBatchNormGrad:
     case HloOpcode::kBatchNormInference:
     case HloOpcode::kBatchNormTraining:
@@ -146,7 +170,12 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kConditional:
     case HloOpcode::kConvolution:
     case HloOpcode::kAllGather:
+    case HloOpcode::kAllGatherStart:
+    case HloOpcode::kAllGatherDone:
     case HloOpcode::kAllReduce:
+    case HloOpcode::kReduceScatter:
+    case HloOpcode::kAllReduceStart:
+    case HloOpcode::kAllReduceDone:
     case HloOpcode::kAllToAll:
     case HloOpcode::kCollectivePermute:
     case HloOpcode::kCollectivePermuteDone:
@@ -161,6 +190,7 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kGather:
     case HloOpcode::kLog:
     case HloOpcode::kLog1p:
+    case HloOpcode::kLogistic:
     case HloOpcode::kMap:
     case HloOpcode::kParameter:
     case HloOpcode::kPower:
@@ -177,10 +207,10 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
     case HloOpcode::kSend:
     case HloOpcode::kSendDone:
     case HloOpcode::kSort:
+    case HloOpcode::kTopK:
     case HloOpcode::kSqrt:
     case HloOpcode::kCbrt:
     case HloOpcode::kTanh:
-    case HloOpcode::kTrace:
     case HloOpcode::kTriangularSolve:
     case HloOpcode::kWhile:
     case HloOpcode::kGetDimensionSize:
@@ -196,7 +226,7 @@ bool IsAlwaysDuplicable(const HloInstruction& instruction) {
 // We use "has a smaller true rank than the output" as a heuristic
 // for "negligible" memory usage.
 bool InstructionFusion::EffectivelyAtMostUnary(HloInstruction* hlo) {
-  int64 output_rank = 0;
+  int64_t output_rank = 0;
   ShapeUtil::ForEachSubshape(
       hlo->shape(),
       [&output_rank](const Shape& subshape, const ShapeIndex& shape_index) {
@@ -221,6 +251,7 @@ bool InstructionFusion::EffectivelyAtMostUnary(HloInstruction* hlo) {
 bool InstructionFusion::CanFuseOnAllPaths(
     HloInstruction* producer, HloInstruction* consumer,
     const HloInstructionSet& do_not_fuse,
+    const HloReachabilityMap& reachability,
     absl::flat_hash_map<std::pair<HloInstruction*, HloInstruction*>, bool>*
         result_cache) {
   if (consumer == producer) {
@@ -234,11 +265,11 @@ bool InstructionFusion::CanFuseOnAllPaths(
     return cache_it->second;
   }
   bool result = true;
-  for (int64 i = 0, e = consumer->operand_count(); i < e; ++i) {
+  for (int64_t i = 0, e = consumer->operand_count(); i < e; ++i) {
     auto* consumer_operand = consumer->mutable_operand(i);
     // If the operand is not on a path to the producer, it doesn't matter
     // whether it's fusible.
-    if (!reachability_->IsReachable(producer, consumer_operand)) {
+    if (!reachability.IsReachable(producer, consumer_operand)) {
       continue;
     }
     if (do_not_fuse.count(consumer_operand) > 0 || !ShouldFuse(consumer, i)) {
@@ -251,7 +282,7 @@ bool InstructionFusion::CanFuseOnAllPaths(
     // Perform the recursive step: make sure producer can be fused into
     // consumer_operand on all paths.
     if (!CanFuseOnAllPaths(producer, consumer_operand, do_not_fuse,
-                           result_cache)) {
+                           reachability, result_cache)) {
       result = false;
       break;
     }
@@ -262,7 +293,8 @@ bool InstructionFusion::CanFuseOnAllPaths(
 
 InstructionFusion::HloInstructionSet
 InstructionFusion::ComputeGloballyUnfusible(
-    absl::Span<HloInstruction* const> post_order) {
+    absl::Span<HloInstruction* const> post_order,
+    const HloReachabilityMap& reachability) {
   // Forbid fusion of producers that:
   // a) Need to be duplicated, unless they can be fused into all consumers
   //    via all paths.
@@ -291,7 +323,7 @@ InstructionFusion::ComputeGloballyUnfusible(
     // memory traffic. In that case, we do not forbid fusion of the operation
     // here.
     auto total_size = [](const Shape& shape) {
-      int64 size = 0;
+      int64_t size = 0;
       ShapeUtil::ForEachSubshape(
           shape, [&size](const Shape& subshape, const ShapeIndex& shape_index) {
             if (subshape.IsArray()) {
@@ -300,7 +332,7 @@ InstructionFusion::ComputeGloballyUnfusible(
           });
       return size;
     };
-    int64 operands_size = 0;
+    int64_t operands_size = 0;
     for (const HloInstruction* op : producer->unique_operands()) {
       operands_size += total_size(op->shape());
     }
@@ -327,6 +359,7 @@ InstructionFusion::ComputeGloballyUnfusible(
     if (producer->IsFusible() &&
         absl::c_all_of(producer->users(), [&](HloInstruction* consumer) {
           return CanFuseOnAllPaths(producer, consumer, do_not_duplicate,
+                                   reachability,
                                    &can_fuse_on_all_paths_result_cache);
         })) {
       continue;
@@ -356,7 +389,7 @@ class ReversePostOrderFusionQueue : public FusionQueue {
     }
   }
 
-  std::pair<HloInstruction*, std::vector<int64>>
+  std::pair<HloInstruction*, std::vector<int64_t>>
   DequeueNextInstructionAndOperandsToFuseInOrder() override {
     // Instructions are "removed" from the post order by nulling out the element
     // in the vector, so if the pointer is null, continue to the next
@@ -365,7 +398,7 @@ class ReversePostOrderFusionQueue : public FusionQueue {
       post_order_.pop_back();
     }
     if (post_order_.empty()) {
-      return std::pair<HloInstruction*, std::vector<int64>>{nullptr, {}};
+      return std::pair<HloInstruction*, std::vector<int64_t>>{nullptr, {}};
     }
     // We want to iterate in reverse post order, so remove from the back of the
     // vector.
@@ -417,7 +450,7 @@ class ReversePostOrderFusionQueue : public FusionQueue {
     // considered before A.
     //
     // We store the original indices of the operands to pass to ShouldFuse.
-    std::vector<int64> sorted_operand_numbers;
+    std::vector<int64_t> sorted_operand_numbers;
     sorted_operand_numbers.reserve(instruction->operands().size());
     for (int i = 0; i < instruction->operands().size(); ++i) {
       // This will happen if we have two possible instructions to fuse the
@@ -430,7 +463,7 @@ class ReversePostOrderFusionQueue : public FusionQueue {
       }
       sorted_operand_numbers.push_back(i);
     }
-    absl::c_sort(sorted_operand_numbers, [&](int64 i, int64 j) {
+    absl::c_sort(sorted_operand_numbers, [&](int64_t i, int64_t j) {
       // Instructions with higher priority in the queue come first.
       return (FindOrDie(post_order_index_, instruction->mutable_operand(i)) >
               FindOrDie(post_order_index_, instruction->mutable_operand(j)));
@@ -465,44 +498,48 @@ class ReversePostOrderFusionQueue : public FusionQueue {
 
 }  // namespace
 
-std::unique_ptr<FusionQueue> InstructionFusion::GetFusionQueue(
-    HloComputation* computation) {
-  return absl::make_unique<ReversePostOrderFusionQueue>(computation);
+std::vector<HloComputation*> InstructionFusion::GetFusionComputations(
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
+  // Use sorted computations because fusion configuration is order-sensitive.
+  return module->MakeNonfusionComputationsSorted(execution_threads);
 }
 
-StatusOr<bool> InstructionFusion::Run(HloModule* module) {
-  bool changed = false;
-  module_ = module;
-  int64 fuse_count = 0;
-  std::vector<std::vector<bool>>* fusion_config = nullptr;
-  HloModuleConfig module_config;
-  if (config_collection_mode_ != FusionConfigCollection::kOff) {
-    module_config = module->config();
-    fusion_config = module_config.mutable_fusion_config();
-    fusion_config->clear();
-  }
+std::unique_ptr<FusionQueue> InstructionFusion::GetFusionQueue(
+    HloComputation* computation) {
+  return std::make_unique<ReversePostOrderFusionQueue>(computation);
+}
 
-  // Use sorted computations because fusion configuration is order-sensitive.
-  for (auto* computation : module->MakeNonfusionComputationsSorted()) {
+StatusOr<bool> InstructionFusion::Run(
+    HloModule* module,
+    const absl::flat_hash_set<absl::string_view>& execution_threads) {
+  bool changed = false;
+  int64_t fuse_count = 0;
+  std::vector<std::vector<bool>> fusion_config;
+
+  bool dump_fusion =
+      module->config().debug_options().xla_dump_fusion_visualization();
+
+  for (auto* computation : GetFusionComputations(module, execution_threads)) {
     CHECK(!computation->IsFusionComputation());
-    computation_ = computation;
-    reachability_ = HloReachabilityMap::Build(computation_);
+    std::unique_ptr<HloReachabilityMap> reachability =
+        HloReachabilityMap::Build(computation);
 
     HloInstructionSet do_not_duplicate;
     // If we allow duplications, we need to compute which instructions we do not
     // want to duplicate based on a global analysis of the graph.
     if (may_duplicate_) {
-      do_not_duplicate =
-          ComputeGloballyUnfusible(computation_->MakeInstructionPostOrder());
+      do_not_duplicate = ComputeGloballyUnfusible(
+          computation->MakeInstructionPostOrder(), *reachability);
     }
-    auto fusion_queue = GetFusionQueue(computation_);
+    auto fusion_queue = GetFusionQueue(computation);
 
     // Instruction fusion effectively fuses edges in the computation graph
     // (producer instruction -> consumer instruction) so we iterate over all
     // edges. When we fuse an edge, we create a copy of the producer inside the
     // fusion instruction.
     while (true) {
-      auto next_entry =
+      std::pair<HloInstruction*, std::vector<int64_t>> next_entry =
           fusion_queue->DequeueNextInstructionAndOperandsToFuseInOrder();
       HloInstruction* instruction = next_entry.first;
       if (instruction == nullptr) {
@@ -514,11 +551,12 @@ StatusOr<bool> InstructionFusion::Run(HloModule* module) {
         continue;
       }
 
-      VLOG(5) << "Considering fusion of: " << instruction->ToString();
-      std::vector<int64>& sorted_operand_numbers = next_entry.second;
+      std::vector<int64_t>& sorted_operand_numbers = next_entry.second;
 
-      for (int64 i : sorted_operand_numbers) {
+      for (int64_t i : sorted_operand_numbers) {
         HloInstruction* operand = instruction->mutable_operand(i);
+        VLOG(5) << "Considering fusion of: " << instruction->ToString()
+                << " with operand " << operand->name();
 
         if (!operand->IsFusible()) {
           VLOG(3) << "Operand (" << operand->ToString() << ") is not fusible";
@@ -536,26 +574,67 @@ StatusOr<bool> InstructionFusion::Run(HloModule* module) {
         };
 
         HloInstruction* fusion_instruction = nullptr;
+
         // Try "regular" fusion if the operand may be duplicated. Otherwise,
         // perform multi-output fusion, unless this creates a cycle.
-        if (do_not_duplicate.count(operand) == 0 &&
-            ShouldFuse(instruction, i)) {
-          if (consume_fuel()) {
-            fusion_queue->PreFusion(operand, instruction);
-            fusion_instruction = Fuse(operand, instruction);
+        FusionDecision use_regular_fusion(do_not_duplicate.count(operand) == 0,
+                                          "operand can not be duplicated");
+        if (use_regular_fusion) {
+          use_regular_fusion =
+              use_regular_fusion.And(ShouldFuse(instruction, i));
+        }
+
+        if (use_regular_fusion && consume_fuel()) {
+          if (dump_fusion) {
+            DumpPreFusionState(computation, /*consumer=*/instruction,
+                               /*producer=*/operand);
           }
-        } else if (ShouldFuseIntoMultiOutput(instruction, i) &&
-                   !MultiOutputFusionCreatesCycle(operand, instruction)) {
-          if (consume_fuel()) {
+
+          fusion_queue->PreFusion(operand, instruction);
+          fusion_instruction = Fuse(operand, instruction, computation);
+        }
+
+        FusionDecision use_mof;
+        if (!use_regular_fusion) {
+          use_mof = ShouldFuseIntoMultiOutput(instruction, i);
+          if (use_mof) {
+            use_mof = use_mof.And(
+                FusionDecision{!MultiOutputFusionCreatesCycle(
+                                   operand, instruction, *reachability),
+                               "multi-output fusion creates a cycle"});
+          }
+          if (use_mof && consume_fuel()) {
+            if (dump_fusion) {
+              DumpPreFusionState(computation, /*consumer=*/instruction,
+                                 /*producer=*/operand, /*is_mof=*/true);
+            }
+
             fusion_queue->PreFusion(operand, instruction);
-            fusion_instruction = FuseIntoMultiOutput(operand, instruction);
+            fusion_instruction =
+                FuseIntoMultiOutput(operand, instruction, computation);
           }
         }
 
         if (fusion_instruction == nullptr) {
+          FusionDecision fusion_decision = use_regular_fusion.Or(use_mof);
+          CHECK(!fusion_decision.CanFuse());
+          if (dump_fusion) {
+            VLOG(2) << "Not fusing " << operand->ToShortString() << "| into |"
+                    << instruction->ToShortString() << "| as "
+                    << fusion_decision.Explain();
+
+            DumpNotFusingState(computation,
+                               /*consumer=*/instruction,
+                               /*producer=*/operand,
+                               /*decision=*/fusion_decision);
+          }
+
+          fusion_queue->NotFusingInstruction(operand, instruction);
           continue;
         }
 
+        // Saving name to use after the instruction is removed.
+        std::string producer_name(operand->name());
         fusion_queue->OnFusingInstruction(fusion_instruction, operand,
                                           instruction);
         changed = true;
@@ -566,7 +645,11 @@ StatusOr<bool> InstructionFusion::Run(HloModule* module) {
           // Operand is now dead. Remove from queue.
           fusion_queue->RemoveInstruction(operand);
           // Remove from computation.
-          TF_RETURN_IF_ERROR(computation_->RemoveInstruction(operand));
+          TF_RETURN_IF_ERROR(computation->RemoveInstruction(operand));
+        }
+
+        if (dump_fusion) {
+          DumpStateAfterFusion(computation, fusion_instruction, producer_name);
         }
 
         if (fusion_instruction != instruction) {
@@ -580,32 +663,32 @@ StatusOr<bool> InstructionFusion::Run(HloModule* module) {
       const std::vector<bool>* comp_fusion_config =
           fusion_queue->FusionConfiguration();
       if (comp_fusion_config && !comp_fusion_config->empty()) {
-        fusion_config->push_back(*comp_fusion_config);
+        fusion_config.push_back(*comp_fusion_config);
       }
     }
   }
 
   if (config_collection_mode_ != FusionConfigCollection::kOff) {
-    int64 fused_count = 0;
-    for (auto& config_per_computation : *fusion_config) {
-      for (auto edge : config_per_computation) {
-        if (edge) {
-          ++fused_count;
-        }
+    if (VLOG_IS_ON(1)) {
+      int64_t fused_count = 0;
+      for (auto& config_per_computation : fusion_config) {
+        fused_count += std::count(config_per_computation.begin(),
+                                  config_per_computation.end(), true);
       }
+      VLOG(1) << "There are " << fused_count << " fused bits that cause "
+              << fuse_count << " fusion actions.";
     }
-    VLOG(1) << "There are " << fused_count << " fused bits that cause "
-            << fuse_count << " fusion actions.";
-    VLOG(1) << FusionConfigToString(*fusion_config);
-    module->set_config(module_config);
+    *module->config().mutable_fusion_config() = std::move(fusion_config);
   }
+
   VLOG(1) << "Fusion count: " << fuse_count;
 
   return changed;
 }
 
 HloInstruction* InstructionFusion::AddFusionInstruction(
-    HloInstruction* producer, HloInstruction* consumer) {
+    HloInstruction* producer, HloInstruction* consumer,
+    HloComputation* computation) {
   HloInstruction* fusion_instruction;
   auto kind = ChooseKind(producer, consumer);
   if (consumer->opcode() == HloOpcode::kFusion) {
@@ -614,10 +697,13 @@ HloInstruction* InstructionFusion::AddFusionInstruction(
       fusion_instruction->set_fusion_kind(kind);
     }
   } else {
-    fusion_instruction = computation_->AddInstruction(
+    fusion_instruction = computation->AddInstruction(
         HloInstruction::CreateFusion(consumer->shape(), kind, consumer));
-    TF_CHECK_OK(computation_->ReplaceInstruction(consumer, fusion_instruction));
+    TF_CHECK_OK(computation->ReplaceInstruction(consumer, fusion_instruction));
   }
+  fusion_instruction->set_called_computations_execution_thread(
+      computation->execution_thread(),
+      /*skip_async_execution_thread_overwrite=*/false);
   return fusion_instruction;
 }
 
@@ -626,11 +712,36 @@ HloInstruction* InstructionFusion::FuseInstruction(
   return fusion_instruction->FuseInstruction(producer);
 }
 
+void InstructionFusion::UpdateReusedOperandsForFusion(
+    HloInstruction* producer, HloInstruction* fusion_instruction) {
+  // Find or compute the existing fusion reused operands. Note these reflect the
+  // state *before* the current fusion has taken place, although if we have
+  // replaced the consumer with a new single-element fusion, we will compute
+  // the new single-element fusion's reused operands here.
+  absl::flat_hash_set<const HloInstruction*>& fusion_reused_operands =
+      ReusedOperandsOf(fusion_instruction);
+
+  // If the producer is reused, replace it with its operands.
+  if (fusion_reused_operands.erase(producer)) {
+    fusion_reused_operands.insert(producer->operands().begin(),
+                                  producer->operands().end());
+  } else {
+    const absl::flat_hash_set<const HloInstruction*>& producer_reused_operands =
+        ReusedOperandsOf(producer);
+    // Otherwise add the producer's reused operands to the set.
+    fusion_reused_operands.insert(producer_reused_operands.begin(),
+                                  producer_reused_operands.end());
+  }
+}
+
 HloInstruction* InstructionFusion::Fuse(HloInstruction* producer,
-                                        HloInstruction* consumer) {
+                                        HloInstruction* consumer,
+                                        HloComputation* computation) {
   VLOG(2) << "Fusing " << producer->ToString() << " into "
           << consumer->ToString();
-  HloInstruction* fusion_instruction = AddFusionInstruction(producer, consumer);
+  HloInstruction* fusion_instruction =
+      AddFusionInstruction(producer, consumer, computation);
+  UpdateReusedOperandsForFusion(producer, fusion_instruction);
   FuseInstruction(fusion_instruction, producer);
   if (fusion_instruction != producer && fusion_instruction != consumer) {
     VLOG(2) << "       created new fusion: " << fusion_instruction->ToString();
@@ -639,16 +750,20 @@ HloInstruction* InstructionFusion::Fuse(HloInstruction* producer,
 }
 
 HloInstruction* InstructionFusion::FuseIntoMultiOutput(
-    HloInstruction* producer, HloInstruction* consumer) {
+    HloInstruction* producer, HloInstruction* consumer,
+    HloComputation* computation) {
   VLOG(2) << "Multi-output fusing " << producer->ToString() << " into "
           << consumer->ToString();
-  HloInstruction* fusion_instruction = AddFusionInstruction(producer, consumer);
+  HloInstruction* fusion_instruction =
+      AddFusionInstruction(producer, consumer, computation);
+  UpdateReusedOperandsForFusion(producer, fusion_instruction);
   fusion_instruction->FuseInstructionIntoMultiOutput(producer);
   return fusion_instruction;
 }
 
 bool InstructionFusion::MultiOutputFusionCreatesCycle(
-    HloInstruction* producer, HloInstruction* consumer) {
+    HloInstruction* producer, HloInstruction* consumer,
+    const HloReachabilityMap& reachability) {
   absl::flat_hash_set<int> operands;
   for (const HloInstruction* operand : consumer->operands()) {
     if (operand == producer) {
@@ -660,9 +775,8 @@ bool InstructionFusion::MultiOutputFusionCreatesCycle(
     // sure MultiOutputFusion would create a cycle. If not, we need to do a DFS
     // traversal of the computation to verify that this multioutput fusion would
     // not create a cycle.
-    if (reachability_->IsPresent(producer) &&
-        reachability_->IsPresent(operand) &&
-        reachability_->IsReachable(producer, operand)) {
+    if (reachability.IsPresent(producer) && reachability.IsPresent(operand) &&
+        reachability.IsReachable(producer, operand)) {
       return true;
     }
     operands.insert(operand->unique_id());
@@ -687,20 +801,225 @@ bool InstructionFusion::MultiOutputFusionCreatesCycle(
   return false;
 }
 
-bool InstructionFusion::ShouldFuse(HloInstruction* consumer,
-                                   int64 operand_index) {
+namespace {
+
+// Extracts instructions from the fusion that satisfy the filter.
+std::vector<const HloInstruction*> ExtractInstructions(
+    const HloInstruction* hlo, const HloPredicate& filter) {
+  if (filter(hlo)) {
+    return {hlo};
+  }
+  if (hlo->opcode() != HloOpcode::kFusion) {
+    return {};
+  }
+  std::vector<const HloInstruction*> matches;
+  for (HloInstruction* inst :
+       hlo->fused_instructions_computation()->instructions()) {
+    if (filter(inst)) {
+      matches.push_back(inst);
+    }
+  }
+  return matches;
+}
+
+std::vector<const HloInstruction*> ExtractInstructions(
+    const HloInstruction* hlo, HloOpcode opcode) {
+  return ExtractInstructions(hlo, [opcode](const HloInstruction* inst) {
+    return inst->opcode() == opcode;
+  });
+}
+
+// Returns true if fusing a slice or dynamic slice in producer into a dynamic
+// update slice fusion in consumer is safe. It is not safe to fuse the slice and
+// DUS when fusing will cause another non-elementwise op to share operands with
+// the DUS in-place buffer.
+bool IsSafeToFuseSliceIntoDusFusion(const HloInstruction* producer,
+                                    const HloInstruction* consumer,
+                                    const HloInstruction* dus) {
+  // Use a memoization map to avoid exponential runtime.
+  absl::flat_hash_map<const HloInstruction*, bool> nonelementwise_memo;
+  // Recursively check if the instruction or its users (or their users) are
+  // non-elementwise with the exception of the DUS. We have already verified
+  // that the slice and DUS are compatible since their indices match.
+  HloPredicate has_nonelementwise_uses_except_dus =
+      [&](const HloInstruction* instruction) {
+        auto record_and_return = [&](bool val) {
+          nonelementwise_memo[instruction] = val;
+          return val;
+        };
+        auto nonelementwise_memo_it = nonelementwise_memo.find(instruction);
+        if (nonelementwise_memo_it != nonelementwise_memo.end()) {
+          return nonelementwise_memo_it->second;
+        }
+        if (instruction != dus && !instruction->IsElementwise() &&
+            instruction->opcode() != HloOpcode::kParameter) {
+          return record_and_return(true);
+        }
+        return record_and_return(absl::c_any_of(
+            instruction->users(), has_nonelementwise_uses_except_dus));
+      };
+  for (int i = 0; i < consumer->operand_count(); ++i) {
+    if (consumer->operand(i) == producer &&
+        has_nonelementwise_uses_except_dus(consumer->fused_parameter(i))) {
+      VLOG(4) << "Found a different elementwise";
+      return false;
+    }
+  }
+  return true;
+}
+
+}  // namespace
+
+/*static*/ FusionDecision InstructionFusion::ShouldFuseInPlaceOp(
+    const HloInstruction* producer, const HloInstruction* consumer) {
+  // Don't fuse if the producer is a non-elementwise op that has the same
+  // operand as an in-place operand of the consumer. The consumer will modify
+  // the buffer in-place, which will cause producer's operand to change if we
+  // allow them to fuse.
+  std::vector<std::pair<HloOperandIndex, ShapeIndex>>
+      in_place_input_output_pairs =
+          HloDataflowAnalysis::GetInPlaceInputOutputPairs(
+              const_cast<HloInstruction*>(consumer));
+  for (auto& pair : in_place_input_output_pairs) {
+    int operand_number = pair.first.operand_number;
+    VLOG(4) << "in/out pair: " << operand_number << " "
+            << pair.first.operand_index.ToString() << " "
+            << pair.second.ToString();
+    // Check if the consumer also has an additional operand that has the same
+    // value as the in-place buffer. If so, it's unsafe to fuse.
+    for (int i = 0; i < consumer->operand_count(); ++i) {
+      if (i != operand_number &&
+          consumer->operand(operand_number) == consumer->operand(i)) {
+        return "The consumer is an in-place operation that has an additional "
+               "operand that has the same value as the in-place buffer";
+      }
+    }
+    if (!producer->IsElementwise() &&
+        (consumer->operand(operand_number) == producer ||
+         absl::c_find(producer->operands(),
+                      consumer->operand(operand_number)) !=
+             producer->operands().end())) {
+      VLOG(4) << "Found non-elementwise producer that uses the same operand of "
+                 "an in-place consumer";
+      auto get_real_operand = [](const HloInstruction* op,
+                                 const HloInstruction* operand) {
+        if (op->opcode() == HloOpcode::kFusion &&
+            operand->opcode() == HloOpcode::kParameter) {
+          return op->operand(operand->parameter_number());
+        }
+        return operand;
+      };
+
+      auto get_constant_operand =
+          [](const HloInstruction* operand) -> std::optional<int> {
+        if (operand->IsConstant()) {
+          return operand->literal().GetFirstInteger();
+        }
+        return std::nullopt;
+      };
+      // A common special case is a slice or dynamic-slice and a
+      // dynamic-update-slice that use the same indices. This pattern is safe.
+
+      auto producer_nonelementwise_ops =
+          ExtractInstructions(producer, [](const HloInstruction* inst) {
+            return inst->opcode() != HloOpcode::kFusion &&
+                   !inst->IsElementwise() &&
+                   inst->opcode() != HloOpcode::kBitcast &&
+                   inst->opcode() != HloOpcode::kParameter;
+          });
+      if (producer_nonelementwise_ops.size() > 1) {
+        return "Producer fusion has multiple non-elementwise ops, bailing.";
+      }
+      // If the producer has only elementwise ops or bitcasts, we can fuse.
+      if (producer_nonelementwise_ops.empty()) {
+        return {};
+      }
+      auto dus_ops =
+          ExtractInstructions(consumer, HloOpcode::kDynamicUpdateSlice);
+      // Check whether we have a DUS, in which case we should not fuse to make
+      // sure we can compute the fusion in place.
+      // TODO(akuegel): Are there other ops than dynamic update slice where we
+      // have a special emitter if it can be done in-place?
+      if (dus_ops.empty()) {
+        return {};
+      }
+      if (dus_ops.size() > 1) {
+        return "multiple dus ops, bailing.";
+      }
+      auto dus = dus_ops[0];
+      auto producer_nonelementwise = producer_nonelementwise_ops[0];
+      if (producer_nonelementwise->opcode() == HloOpcode::kSlice) {
+        if (producer_nonelementwise->shape() != dus->operand(1)->shape()) {
+          return "Slice op has a different shape than the update shape of the "
+                 "dus op, bailing.";
+        }
+        for (int i = 0; i < dus->shape().rank(); ++i) {
+          const HloInstruction* dus_operand =
+              get_real_operand(consumer, dus->operand(2 + i));
+          auto constant_operand = get_constant_operand(dus_operand);
+          if (!constant_operand ||
+              *constant_operand != producer_nonelementwise->slice_starts(i) ||
+              producer_nonelementwise->slice_strides(i) != 1) {
+            return "DUS and slice index mismatch";
+          }
+        }
+        VLOG(4) << "DUS and slice index match";
+        if (consumer->opcode() == HloOpcode::kFusion &&
+            !IsSafeToFuseSliceIntoDusFusion(producer, consumer, dus)) {
+          return "Fusing slice into DUS will also fuse another non-elementwise "
+                 "op with shared operand as DUS.";
+        }
+        return {};
+      }
+      if (producer_nonelementwise->opcode() == HloOpcode::kDynamicSlice) {
+        if (producer_nonelementwise->shape() != dus->operand(1)->shape()) {
+          return "Dynamic slice op has a different shape than the update shape "
+                 "of the dus op, bailing.";
+        }
+        for (int i = 0; i < dus->shape().rank(); ++i) {
+          const HloInstruction* ds_operand = get_real_operand(
+              producer, producer_nonelementwise->operand(1 + i));
+          const HloInstruction* dus_operand =
+              get_real_operand(consumer, dus->operand(2 + i));
+          auto constant_ds_operand = get_constant_operand(ds_operand);
+          auto constant_dus_operand = get_constant_operand(dus_operand);
+          if (constant_ds_operand != constant_dus_operand ||
+              (!constant_ds_operand && ds_operand != dus_operand)) {
+            return "DUS and DS index mismatch";
+          }
+        }
+        VLOG(4) << "DUS and DS index match";
+        if (consumer->opcode() == HloOpcode::kFusion &&
+            !IsSafeToFuseSliceIntoDusFusion(producer, consumer, dus)) {
+          return "Fusing DS into DUS will also fuse another non-elementwise op "
+                 "with shared operand as DUS.";
+        }
+        return {};
+      }
+      return "unrecognized inplace update non-elementwise output pair";
+    }
+  }
+  return {};
+}
+
+FusionDecision InstructionFusion::ShouldFuse(HloInstruction* consumer,
+                                             int64_t operand_index) {
   HloInstruction* producer = consumer->mutable_operand(operand_index);
 
+  // Don't fuse across a root instruction.
+  if (producer == producer->parent()->root_instruction()) {
+    return "not fusing into the output of the root instruction";
+  }
+
   // Cost condition: don't duplicate expensive instructions.
   if (FusionWouldDuplicate(*producer, *consumer) &&
       (!may_duplicate_ || is_expensive_(*producer)) &&
       !IsAlwaysDuplicable(*producer)) {
-    VLOG(4) << "Stopping: fusion may duplicate operand ("
-            << producer->ToString() << ") , and this is expensive";
-    return false;
+    return may_duplicate_ ? "expensive producer would be duplicated"
+                          : "fusion pass cannot duplicate";
   }
 
-  return true;
+  return ShouldFuseInPlaceOp(producer, consumer);
 }
 
 HloInstruction::FusionKind InstructionFusion::ChooseKind(
@@ -708,4 +1027,73 @@ HloInstruction::FusionKind InstructionFusion::ChooseKind(
   return HloInstruction::FusionKind::kLoop;
 }
 
+absl::flat_hash_set<const HloInstruction*>& InstructionFusion::ReusedOperandsOf(
+    const HloInstruction* instruction) {
+  std::unique_ptr<absl::flat_hash_set<const HloInstruction*>>& reused_operands =
+      reused_fusion_operands_[instruction];
+  if (reused_operands != nullptr) {
+    return *reused_operands;
+  }
+  reused_operands =
+      std::make_unique<absl::flat_hash_set<const HloInstruction*>>();
+
+  for (int64_t i = 0; i < instruction->operand_count(); ++i) {
+    bool reuses = instruction->ReusesOperandElements(i);
+    if (reuses) {
+      // We cache the operand corresponding to the fusion parameter, because the
+      // parameter numbers would be invalidated after the next fusion.
+      reused_operands->insert(instruction->operand(i));
+    }
+  }
+  return *reused_operands;
+}
+
+bool InstructionFusion::ReusesOperandElements(const HloInstruction* consumer,
+                                              int64_t operand_index) {
+  auto operand = consumer->operand(operand_index);
+  return ReusedOperandsOf(consumer).contains(operand);
+}
+
+void InstructionFusion::DumpPreFusionState(HloComputation* computation,
+                                           HloInstruction* consumer,
+                                           HloInstruction* producer,
+                                           bool is_mof) {
+  RegisterFusionState(
+      *computation,
+      absl::StrCat(
+          "About to ", is_mof ? "MOF-fuse" : "fuse", " |", producer->name(),
+          "| into |", consumer->name(),
+          "| inside InstructionFusion with may_duplicate=", may_duplicate_),
+      *consumer, producer);
+}
+
+void InstructionFusion::DumpNotFusingState(HloComputation* computation,
+                                           HloInstruction* consumer,
+                                           HloInstruction* producer,
+                                           FusionDecision decision) {
+  // Readability optimizations: lack of fusion for tuple accesses
+  // generates a lot of noise.
+  if (producer->opcode() == HloOpcode::kGetTupleElement ||
+      consumer->opcode() == HloOpcode::kGetTupleElement) {
+    return;
+  }
+
+  RegisterFusionState(
+      *computation,
+      absl::StrCat("Not fusing |", producer->name(), "| into |",
+                   consumer->name(), "| as ", decision.Explain()),
+      *consumer, producer);
+}
+
+void InstructionFusion::DumpStateAfterFusion(HloComputation* computation,
+                                             HloInstruction* fusion_instruction,
+                                             const std::string& producer_name) {
+  RegisterFusionState(
+      *computation,
+      absl::StrCat(
+          "Fused |", producer_name, "| into |", fusion_instruction->name(),
+          "| inside InstructionFusion with may_duplicate=", may_duplicate_),
+      *fusion_instruction);
+}
+
 }  // namespace xla
