@@ -16,48 +16,37 @@ limitations under the License.
 #include "tensorflow/compiler/xla/service/transpose_folding.h"
 
 #include <memory>
-#include <unordered_set>
 #include <vector>
 
+#include "absl/container/flat_hash_set.h"
+#include "absl/strings/string_view.h"
 #include "tensorflow/compiler/xla/client/xla_builder.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_computation.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_instruction.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_module.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_opcode.h"
+#include "tensorflow/compiler/xla/hlo/utils/hlo_matchers.h"
 #include "tensorflow/compiler/xla/literal.h"
 #include "tensorflow/compiler/xla/service/gpu/ir_emission_utils.h"
-#include "tensorflow/compiler/xla/service/hlo_computation.h"
-#include "tensorflow/compiler/xla/service/hlo_instruction.h"
-#include "tensorflow/compiler/xla/service/hlo_matchers.h"
-#include "tensorflow/compiler/xla/service/hlo_module.h"
-#include "tensorflow/compiler/xla/service/hlo_opcode.h"
 #include "tensorflow/compiler/xla/service/shape_inference.h"
 #include "tensorflow/compiler/xla/shape_util.h"
 #include "tensorflow/compiler/xla/test.h"
 #include "tensorflow/compiler/xla/test_helpers.h"
 #include "tensorflow/compiler/xla/tests/hlo_test_base.h"
 #include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/core/platform/logging.h"
-
-namespace op = xla::testing::opcode_matchers;
+#include "tensorflow/tsl/platform/logging.h"
+#include "tensorflow/tsl/platform/status_matchers.h"
 
 namespace xla {
 namespace {
 
-class TransposeFoldingTest : public HloTestBase {
- protected:
-  void FoldTranspose(HloModule* module) {
-    TransposeFolding transpose_folding(
-        [](const HloInstruction& dot,
-           const TransposeFolding::OperandIndices& candidate_operands) {
-          return candidate_operands;
-        },
-        [](const HloInstruction& convolution,
-           const TransposeFolding::OperandIndices& candidate_operands) {
-          return candidate_operands;
-        });
-    EXPECT_IS_OK(transpose_folding.Run(module).status());
-  }
-};
+namespace op = xla::testing::opcode_matchers;
+using ::tsl::testing::IsOkAndHolds;
+
+using TransposeFoldingTest = HloTestBase;
 
 TEST_F(TransposeFoldingTest, FoldDotTranspose) {
-  string hlo_string = R"(
+  constexpr absl::string_view kHloString = R"(
 HloModule FoldDotTranspose
 
 ENTRY entry_computation {
@@ -67,18 +56,17 @@ ENTRY entry_computation {
   ROOT dot = f32[2,2]{1,0} dot(x, transpose), lhs_contracting_dims={1}, rhs_contracting_dims={0}
 }
 )";
-  TF_ASSERT_OK_AND_ASSIGN(auto module,
-                          ParseAndReturnVerifiedModule(hlo_string));
-
-  FoldTranspose(module.get());
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
 
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
   EXPECT_THAT(module->entry_computation()->root_instruction(),
               op::Dot(op::Parameter(0), op::Parameter(1),
                       /*lhs_contracting_dim=*/1, /*rhs_contracting_dim=*/1));
 }
 
-TEST_F(TransposeFoldingTest, DontFoldTransposeOfBatchDim) {
-  string hlo_string = R"(
+TEST_F(TransposeFoldingTest, DontFoldTransposeOfBatchDimByDefault) {
+  constexpr absl::string_view kHloString = R"(
 HloModule FoldDotTranspose
 
 ENTRY entry_computation {
@@ -88,25 +76,39 @@ ENTRY entry_computation {
   ROOT dot = f32[2] dot(x, transpose), lhs_batch_dims={0}, rhs_batch_dims={0}, lhs_contracting_dims={1}, rhs_contracting_dims={1}
 }
 )";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
 
-  TF_ASSERT_OK_AND_ASSIGN(auto module,
-                          ParseAndReturnVerifiedModule(hlo_string));
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(false));
+}
+
+TEST_F(TransposeFoldingTest, FoldTransposeOfBatchWhenPermitted) {
+  constexpr absl::string_view kHloString = R"(
+HloModule FoldDotTranspose
+
+ENTRY entry_computation {
+  x = f32[5,2,3] parameter(0)
+  y = f32[3,5,4] parameter(1)
+  transpose = f32[5,3,4] transpose(y), dimensions={1,0,2}
+  ROOT dot = f32[5,2,4] dot(x, transpose), lhs_batch_dims={0}, rhs_batch_dims={0}, lhs_contracting_dims={2}, rhs_contracting_dims={1}
+}
+)";
+
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
 
   TransposeFolding transpose_folding(
-      [](const HloInstruction& dot,
-         const TransposeFolding::OperandIndices& candidate_operands) {
-        return candidate_operands;
-      },
-      [](const HloInstruction& convolution,
-         const TransposeFolding::OperandIndices& candidate_operands) {
-        return candidate_operands;
+      /*dot_can_fold_transpose_operand=*/[](const HloInstruction&, int64_t) {
+        return true;
       });
-  TF_ASSERT_OK_AND_ASSIGN(bool changed, transpose_folding.Run(module.get()));
-  EXPECT_FALSE(changed);
+  EXPECT_THAT(transpose_folding.Run(module.get()), IsOkAndHolds(true));
+  EXPECT_THAT(module->entry_computation()->root_instruction(),
+              op::Dot(op::Parameter(0), op::Parameter(1),
+                      /*lhs_contracting_dim=*/2, /*rhs_contracting_dim=*/0));
 }
 
 TEST_F(TransposeFoldingTest, DontFoldTransposeOfRank1Dot) {
-  string hlo_string = R"(
+  constexpr absl::string_view kHloString = R"(
 HloModule FoldDotTranspose
 
 ENTRY entry_computation {
@@ -116,25 +118,31 @@ ENTRY entry_computation {
   ROOT dot = f32[2] dot(x, transpose), lhs_batch_dims={}, rhs_batch_dims={}, lhs_contracting_dims={0}, rhs_contracting_dims={1}
 }
 )";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
 
-  TF_ASSERT_OK_AND_ASSIGN(auto module,
-                          ParseAndReturnVerifiedModule(hlo_string));
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(false));
+}
 
-  TransposeFolding transpose_folding(
-      [](const HloInstruction& dot,
-         const TransposeFolding::OperandIndices& candidate_operands) {
-        return candidate_operands;
-      },
-      [](const HloInstruction& convolution,
-         const TransposeFolding::OperandIndices& candidate_operands) {
-        return candidate_operands;
-      });
-  TF_ASSERT_OK_AND_ASSIGN(bool changed, transpose_folding.Run(module.get()));
-  EXPECT_FALSE(changed);
+TEST_F(TransposeFoldingTest, DontFoldTransposeOfDotWithoutContractingDims) {
+  constexpr absl::string_view kHloString = R"(
+HloModule FoldDotTranspose
+
+ENTRY entry_computation {
+  x = f32[3,4] parameter(0)
+  y = f32[3,4,6,7] parameter(1)
+  transpose = f32[3,4,7,6] transpose(y), dimensions={0,1,3,2}
+  ROOT dot = f32[3,4,7,6] dot(x, transpose), lhs_batch_dims={0,1}, rhs_batch_dims={0,1}, lhs_contracting_dims={}, rhs_contracting_dims={}
+}
+)";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(false));
 }
 
 TEST_F(TransposeFoldingTest, FoldDotTransposeConstant) {
-  string hlo_string = R"(
+  constexpr absl::string_view kHloString = R"(
 HloModule FoldDotTransposeConstant
 
 ENTRY entry_computation {
@@ -145,11 +153,10 @@ ENTRY entry_computation {
   ROOT dot = f32[1,3]{1,0} dot(transpose, transpose.1), lhs_contracting_dims={1}, rhs_contracting_dims={0}
 }
 )";
-  TF_ASSERT_OK_AND_ASSIGN(auto module,
-                          ParseAndReturnVerifiedModule(hlo_string));
-
-  FoldTranspose(module.get());
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
 
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
   EXPECT_THAT(module->entry_computation()->root_instruction(),
               op::Dot(op::Constant(), op::Constant(),
                       /*lhs_contracting_dim=*/0, /*rhs_contracting_dim=*/1));
@@ -187,7 +194,7 @@ TEST_F(TransposeFoldingTest, FuseDotWithConstantOperands) {
 }
 
 TEST_F(TransposeFoldingTest, FoldDotTransposeInCall) {
-  string hlo_string = R"(
+  constexpr absl::string_view kHloString = R"(
 HloModule FoldDotTransposeInCall
 
 callee {
@@ -203,9 +210,10 @@ ENTRY entry_computation {
   ROOT call = f32[2,2]{1,0} call(y, x), to_apply=callee
 }
 )";
-  TF_ASSERT_OK_AND_ASSIGN(auto module,
-                          ParseAndReturnVerifiedModule(hlo_string));
-  FoldTranspose(module.get());
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
 
   const HloComputation* callee = module->GetComputationWithName("callee");
   ASSERT_NE(callee, nullptr);
@@ -240,20 +248,22 @@ TEST_F(TransposeFoldingTest, FoldConvDimSwapTransposeRhs) {
   }
   StatusOr<Shape> conv_shape = ShapeInference::InferConvolveShape(
       x->shape(), transpose_y->shape(), /*feature_group_count=*/1,
-      /*batch_group_count=*/1, window, dnums);
+      /*batch_group_count=*/1, window, dnums,
+      /*preferred_element_type=*/std::nullopt);
   EXPECT_IS_OK(conv_shape);
   HloInstruction* conv = builder.AddInstruction(HloInstruction::CreateConvolve(
-      conv_shape.ValueOrDie(), x, transpose_y,
+      conv_shape.value(), x, transpose_y,
       /*feature_group_count=*/1, /*batch_group_count=*/1, window, dnums,
       DefaultPrecisionConfig(2)));
 
   auto module = CreateNewVerifiedModule("test_module");
   HloComputation* entry_computation =
       module->AddEntryComputation(builder.Build(conv));
-  FoldTranspose(module.get());
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
 
   // Instructions after folding: x, y, and the convolution.
-  std::unordered_set<HloInstruction*> instruction_set(
+  absl::flat_hash_set<HloInstruction*> instruction_set(
       entry_computation->instructions().begin(),
       entry_computation->instructions().end());
   CHECK_EQ(1, instruction_set.erase(x)) << "x is not in entry_computation.";
@@ -296,20 +306,22 @@ TEST_F(TransposeFoldingTest, FoldConvComplexTransposeRhs) {
   }
   StatusOr<Shape> conv_shape = ShapeInference::InferConvolveShape(
       x->shape(), transpose_y->shape(), /*feature_group_count=*/1,
-      /*batch_group_count=*/1, window, dnums);
+      /*batch_group_count=*/1, window, dnums,
+      /*preferred_element_type=*/std::nullopt);
   EXPECT_IS_OK(conv_shape);
   HloInstruction* conv = builder.AddInstruction(HloInstruction::CreateConvolve(
-      conv_shape.ValueOrDie(), x, transpose_y,
+      conv_shape.value(), x, transpose_y,
       /*feature_group_count=*/1, /*batch_group_count=*/1, window, dnums,
       DefaultPrecisionConfig(2)));
 
   auto module = CreateNewVerifiedModule("test_module");
   HloComputation* entry_computation =
       module->AddEntryComputation(builder.Build(conv));
-  FoldTranspose(module.get());
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
 
   // Instructions after folding: x, y, and the convolution.
-  std::unordered_set<HloInstruction*> instruction_set(
+  absl::flat_hash_set<HloInstruction*> instruction_set(
       entry_computation->instructions().begin(),
       entry_computation->instructions().end());
   CHECK_EQ(1, instruction_set.erase(x)) << "x is not in entry_computation.";
@@ -357,20 +369,22 @@ TEST_F(TransposeFoldingTest, FoldConvTransposeLhs) {
   }
   StatusOr<Shape> conv_shape = ShapeInference::InferConvolveShape(
       transpose_x->shape(), y->shape(), /*feature_group_count=*/1,
-      /*batch_group_count=*/1, window, dnums);
+      /*batch_group_count=*/1, window, dnums,
+      /*preferred_element_type=*/std::nullopt);
   EXPECT_IS_OK(conv_shape);
   HloInstruction* conv = builder.AddInstruction(HloInstruction::CreateConvolve(
-      conv_shape.ValueOrDie(), transpose_x, y,
+      conv_shape.value(), transpose_x, y,
       /*feature_group_count=*/1, /*batch_group_count=*/1, window, dnums,
       DefaultPrecisionConfig(2)));
 
   auto module = CreateNewVerifiedModule("test_module");
   HloComputation* entry_computation =
       module->AddEntryComputation(builder.Build(conv));
-  FoldTranspose(module.get());
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
 
   // Instructions after folding: x, y, and the convolution.
-  std::unordered_set<HloInstruction*> instruction_set(
+  absl::flat_hash_set<HloInstruction*> instruction_set(
       entry_computation->instructions().begin(),
       entry_computation->instructions().end());
   EXPECT_EQ(1, instruction_set.erase(x)) << "x is not in entry_computation.";
@@ -424,20 +438,22 @@ TEST_F(TransposeFoldingTest, FoldConvComplexTransposeLhs) {
   }
   StatusOr<Shape> conv_shape = ShapeInference::InferConvolveShape(
       transpose_x->shape(), y->shape(), /*feature_group_count=*/1,
-      /*batch_group_count=*/1, window, dnums);
+      /*batch_group_count=*/1, window, dnums,
+      /*preferred_element_type=*/std::nullopt);
   EXPECT_IS_OK(conv_shape);
   HloInstruction* conv = builder.AddInstruction(HloInstruction::CreateConvolve(
-      conv_shape.ValueOrDie(), transpose_x, y,
+      conv_shape.value(), transpose_x, y,
       /*feature_group_count=*/1, /*batch_group_count=*/1, window, dnums,
       DefaultPrecisionConfig(2)));
 
   auto module = CreateNewVerifiedModule("test_module");
   HloComputation* entry_computation =
       module->AddEntryComputation(builder.Build(conv));
-  FoldTranspose(module.get());
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
 
   // Instructions after folding: x, y, and the convolution.
-  std::unordered_set<HloInstruction*> instruction_set(
+  absl::flat_hash_set<HloInstruction*> instruction_set(
       entry_computation->instructions().begin(),
       entry_computation->instructions().end());
   EXPECT_EQ(1, instruction_set.erase(x)) << "x is not in entry_computation.";
@@ -465,5 +481,83 @@ TEST_F(TransposeFoldingTest, FoldConvComplexTransposeLhs) {
       new_conv->convolution_dimension_numbers().output_spatial_dimensions(1));
 }
 
+TEST_F(TransposeFoldingTest, FoldBatchDotTranspose) {
+  constexpr absl::string_view kHloString = R"(
+HloModule FoldBatchDotTranspose
+
+ENTRY entry_computation {
+  x = f32[7,7,2,3]{3,2,1,0} parameter(0)
+  y = f32[7,7,2,3]{3,2,1,0} parameter(1)
+  transpose = f32[7,7,3,2]{3,2,1,0} transpose(y), dimensions={0,1,3,2}
+  ROOT dot = f32[7,7,2,2]{3,2,1,0} dot(x, transpose), lhs_contracting_dims={3},
+            rhs_contracting_dims={2}, lhs_batch_dims={0,1}, rhs_batch_dims={0,1}
+}
+)";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
+  EXPECT_THAT(module->entry_computation()->root_instruction(),
+              op::Dot(op::Parameter(0), op::Parameter(1),
+                      /*lhs_contracting_dim=*/3, /*rhs_contracting_dim=*/3));
+}
+
+TEST_F(TransposeFoldingTest, NoFoldBatchDotTransposeBatch) {
+  constexpr absl::string_view kHloString = R"(
+HloModule NoFoldBatchDotTransposeBatch
+
+ENTRY entry_computation {
+  x = f32[7,7,2,3]{3,2,1,0} parameter(0)
+  y = f32[7,7,2,3]{3,2,1,0} parameter(1)
+  transpose = f32[7,7,3,2]{3,2,1,0} transpose(y), dimensions={1,0,3,2}
+  ROOT dot = f32[7,7,2,2]{3,2,1,0} dot(x, transpose), lhs_contracting_dims={3},
+            rhs_contracting_dims={2}, lhs_batch_dims={0,1}, rhs_batch_dims={0,1}
+}
+)";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(false));
+}
+
+TEST_F(TransposeFoldingTest, FoldBatchDotTransposeNonContiguousBatch) {
+  constexpr absl::string_view kHloString = R"(
+HloModule FoldBatchDotTransposeNonContiguousBatch
+
+ENTRY entry_computation {
+  x = f32[7,2,7,3]{3,2,1,0} parameter(0)
+  y = f32[7,2,7,3]{3,2,1,0} parameter(1)
+  transpose = f32[7,3,7,2]{3,2,1,0} transpose(y), dimensions={0,3,2,1}
+  ROOT dot = f32[7,7,2,2]{3,2,1,0} dot(x, transpose), lhs_contracting_dims={3},
+            rhs_contracting_dims={1}, lhs_batch_dims={0,2}, rhs_batch_dims={0,2}
+}
+)";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(true));
+  EXPECT_THAT(module->entry_computation()->root_instruction(),
+              op::Dot(op::Parameter(0), op::Parameter(1),
+                      /*lhs_contracting_dim=*/3, /*rhs_contracting_dim=*/3));
+}
+
+TEST_F(TransposeFoldingTest, NoFoldBatchDotTransposeIdentity) {
+  constexpr absl::string_view kHloString = R"(
+HloModule NoFoldBatchDotTransposeIdentity
+
+ENTRY entry_computation {
+  x = f32[7,7,2,3]{3,2,1,0} parameter(0)
+  y = f32[7,7,3,2]{3,2,1,0} parameter(1)
+  transpose = f32[7,7,3,2]{3,2,1,0} transpose(y), dimensions={0,1,2,3}
+  ROOT dot = f32[7,7,2,2]{3,2,1,0} dot(x, transpose), lhs_contracting_dims={3},
+            rhs_contracting_dims={2}, lhs_batch_dims={0,1}, rhs_batch_dims={0,1}
+}
+)";
+  TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
+                          ParseAndReturnVerifiedModule(kHloString));
+
+  EXPECT_THAT(TransposeFolding().Run(module.get()), IsOkAndHolds(false));
+}
+
 }  // namespace
 }  // namespace xla
