@@ -17,16 +17,16 @@ limitations under the License.
 
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_replace.h"
+#include "tensorflow/compiler/xla/hlo/ir/hlo_instruction.h"
+#include "tensorflow/compiler/xla/hlo/utils/hlo_matchers.h"
 #include "tensorflow/compiler/xla/service/algebraic_simplifier.h"
 #include "tensorflow/compiler/xla/service/hlo_cse.h"
 #include "tensorflow/compiler/xla/service/hlo_dce.h"
-#include "tensorflow/compiler/xla/service/hlo_instruction.h"
-#include "tensorflow/compiler/xla/service/hlo_matchers.h"
 #include "tensorflow/compiler/xla/service/hlo_parser.h"
 #include "tensorflow/compiler/xla/service/tuple_simplifier.h"
 #include "tensorflow/compiler/xla/test.h"
 #include "tensorflow/compiler/xla/tests/hlo_test_base.h"
-#include "tensorflow/core/lib/core/status_test_util.h"
+#include "tensorflow/tsl/lib/core/status_test_util.h"
 
 namespace xla {
 namespace {
@@ -37,27 +37,25 @@ namespace op = xla::testing::opcode_matchers;
 // Returns the first kWhile instruction within m's entry computation.
 HloInstruction* FindFirstWhile(HloModule* m) {
   const auto& instrs = m->entry_computation()->instructions();
-  return *absl::c_find_if(instrs, [](const HloInstruction* instr) {
-    return instr->opcode() == HloOpcode::kWhile;
-  });
+  return *absl::c_find_if(instrs, HloPredicateIsOp<HloOpcode::kWhile>);
 }
 
 class WhileLoopSimplifierTest : public HloTestBase {
  protected:
   // Makes an HloModule that contains a loop with `num_iters` iteration.
-  TF_MUST_USE_RESULT std::unique_ptr<VerifiedHloModule>
-  MakeModuleWithSimpleLoop(int num_iters);
+  [[nodiscard]] std::unique_ptr<VerifiedHloModule> MakeModuleWithSimpleLoop(
+      int num_iters);
 
   // Similar to MakeModuleWithSimpleLoop except that the loop bound is passed to
   // the loop-condition through an element of a tuple which is the
   // loop-condition parameter.
-  TF_MUST_USE_RESULT std::unique_ptr<VerifiedHloModule>
+  [[nodiscard]] std::unique_ptr<VerifiedHloModule>
   MakeModuleWithSimpleLoopTupleElementLoopBound(int num_iters);
 };
 
 std::unique_ptr<VerifiedHloModule>
 WhileLoopSimplifierTest::MakeModuleWithSimpleLoop(int num_iters) {
-  string hlo_string_template = R"(
+  std::string hlo_string_template = R"(
   HloModule SimpleLoop
   SimpleLoop.body {
     loop_var.1 = (s32[], s32[3]{0}) parameter(0)
@@ -83,15 +81,15 @@ WhileLoopSimplifierTest::MakeModuleWithSimpleLoop(int num_iters) {
   }
   )";
 
-  string hlo_string = absl::StrReplaceAll(
+  std::string hlo_string = absl::StrReplaceAll(
       hlo_string_template, {{"{{LOOP_BOUND}}", absl::StrCat(42 + num_iters)}});
-  return ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
+  return ParseAndReturnVerifiedModule(hlo_string).value();
 }
 
 std::unique_ptr<VerifiedHloModule>
 WhileLoopSimplifierTest::MakeModuleWithSimpleLoopTupleElementLoopBound(
     int num_iters) {
-  string hlo_string_template = R"(
+  std::string hlo_string_template = R"(
   HloModule SimpleLoopWithIndirectLoopBound
   SimpleLoopWithIndirectLoopBound.body {
     loop_var.1 = (s32[], s32[3]{0}, s32[]) parameter(0)
@@ -121,14 +119,14 @@ WhileLoopSimplifierTest::MakeModuleWithSimpleLoopTupleElementLoopBound(
   }
   )";
 
-  string hlo_string = absl::StrReplaceAll(
+  std::string hlo_string = absl::StrReplaceAll(
       hlo_string_template, {{"{{LOOP_BOUND}}", absl::StrCat(42 + num_iters)}});
-  return ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
+  return ParseAndReturnVerifiedModule(hlo_string).value();
 }
 
 TEST_F(WhileLoopSimplifierTest, LoopWithZeroIterationSimplified) {
   auto m = MakeModuleWithSimpleLoop(/*num_iters=*/0);
-  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   EXPECT_THAT(m->entry_computation()->root_instruction(),
               op::Tuple(op::Constant(), op::Constant()));
 }
@@ -136,14 +134,14 @@ TEST_F(WhileLoopSimplifierTest, LoopWithZeroIterationSimplified) {
 TEST_F(WhileLoopSimplifierTest,
        LoopWithZeroIterationTupleElementLoopBoundSimplified) {
   auto m = MakeModuleWithSimpleLoopTupleElementLoopBound(/*num_iters=*/0);
-  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   EXPECT_THAT(m->entry_computation()->root_instruction(),
               op::Tuple(op::Constant(), op::Constant(), op::Constant()));
 }
 
 TEST_F(WhileLoopSimplifierTest, LoopWithOneIterationSimplified) {
   auto m = MakeModuleWithSimpleLoop(/*num_iters=*/1);
-  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   EXPECT_THAT(m->entry_computation()->root_instruction(),
               op::Tuple(op::Add(), op::Multiply()));
 }
@@ -151,14 +149,14 @@ TEST_F(WhileLoopSimplifierTest, LoopWithOneIterationSimplified) {
 TEST_F(WhileLoopSimplifierTest,
        LoopWithOneIterationTupleELementLoopBoundSimplified) {
   auto m = MakeModuleWithSimpleLoopTupleElementLoopBound(/*num_iters=*/1);
-  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   EXPECT_THAT(m->entry_computation()->root_instruction(),
               op::Tuple(op::Add(), op::Multiply(), op::Constant()));
 }
 
 TEST_F(WhileLoopSimplifierTest, LoopWithTwoIterationsNotSimplified) {
   auto m = MakeModuleWithSimpleLoop(/*num_iters=*/2);
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 TEST_F(WhileLoopSimplifierTest,
@@ -171,7 +169,7 @@ TEST_F(WhileLoopSimplifierTest,
       HloInstruction::CreateConstant(LiteralUtil::CreateR0<bool>(true)));
   TF_ASSERT_OK(true_op->AddControlDependencyTo(
       while_op->while_body()->root_instruction()));
-  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   EXPECT_THAT(computation->root_instruction()->control_predecessors(),
               ElementsAre(op::Constant()))
       << computation->ToString();
@@ -192,7 +190,7 @@ TEST_F(WhileLoopSimplifierTest, LoopWithSendNotSimplified) {
       token,
       /*channel_id=*/0));
   while_body->AddInstruction(HloInstruction::CreateSendDone(send));
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 TEST_F(WhileLoopSimplifierTest, LoopWithRecvNotSimplified) {
@@ -206,7 +204,7 @@ TEST_F(WhileLoopSimplifierTest, LoopWithRecvNotSimplified) {
       HloInstruction::CreateRecv(ShapeUtil::MakeShape(F32, {1}), token,
                                  /*channel_id=*/0));
   while_body->AddInstruction(HloInstruction::CreateRecvDone(recv));
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // We can't simplify loops whose bodies contain infeed or other side-effecting
@@ -220,7 +218,7 @@ TEST_F(WhileLoopSimplifierTest, LoopWithInfeedSimplified) {
   auto token = while_body->AddInstruction(HloInstruction::CreateToken());
   while_body->AddInstruction(HloInstruction::CreateInfeed(
       ShapeUtil::MakeShape(F32, {1}), token, "config"));
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // We don't simplify trip-count-1 loops whose *conditions* contain infeed or
@@ -235,12 +233,12 @@ TEST_F(WhileLoopSimplifierTest, LoopWithInfeedInCondNotSimplified) {
   auto token = while_cond->AddInstruction(HloInstruction::CreateToken());
   while_cond->AddInstruction(HloInstruction::CreateInfeed(
       ShapeUtil::MakeShape(F32, {1}), token, "config"));
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // A non-tuple shaped loop shouldn't be simplified or crash the compiler.
 TEST_F(WhileLoopSimplifierTest, NonTupleShapedLoopNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
  HloModule NonTupleShapedLoop
  NonTupleShapedLoop.body {
    loop_var.1 = s32[] parameter(0)
@@ -260,15 +258,15 @@ TEST_F(WhileLoopSimplifierTest, NonTupleShapedLoopNotSimplified) {
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // A while loop that does nothing else besides swapping tuple elements
 // can't be simplified as the result of the swapping is visible to users of the
 // loop.
 TEST_F(WhileLoopSimplifierTest, LoopSwappingTupleElementsNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule SwappingTupleElements
   SwappingTupleElements.body {
     loop_var = (s32[], s32[]) parameter(0)
@@ -292,8 +290,8 @@ TEST_F(WhileLoopSimplifierTest, LoopSwappingTupleElementsNotSimplified) {
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // Construct a loop where we assign a constant to tuple element 0 in each
@@ -301,7 +299,7 @@ TEST_F(WhileLoopSimplifierTest, LoopSwappingTupleElementsNotSimplified) {
 // value.
 TEST_F(WhileLoopSimplifierTest,
        LoopWithUnusedButModifiedTupleElementNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule UnusedButModifiedTupleElement
   UnusedButModifiedTupleElement.body {
     loop_var = (s32[]) parameter(0)
@@ -321,13 +319,50 @@ TEST_F(WhileLoopSimplifierTest,
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
+}
+
+// Construct a loop where we assign a constant to tuple element 1 in each
+// iteration.  We can eliminate tuple element 1 if it's unused both inside and
+// outside the loop.
+TEST_F(WhileLoopSimplifierTest,
+       LoopWithUnusedOutsideLoopButModifiedTupleElementSimplified) {
+  const std::string hlo_string = R"(
+  HloModule UnusedButModifiedTupleElement
+  UnusedButModifiedTupleElement.body {
+    loop_var = (s32[], s32[]) parameter(0)
+    constant.1 = s32[] constant(1)
+    ROOT tuple = (s32[], s32[]) tuple(s32[] constant.1, constant.1)
+  }
+  UnusedButModifiedTupleElement.cond {
+    param = (s32[], s32[]) parameter(0)
+    gte.cond = s32[] get-tuple-element(param), index=0
+    constant.3 = s32[] constant(1)
+    ROOT lt = pred[] compare(gte.cond, constant.3), direction=LT
+  }
+  ENTRY  UnusedButModifiedTupleElement {
+    constant.2 = s32[] constant(0)
+    tuple.1 = (s32[], s32[]) tuple(constant.2, constant.2)
+    while = (s32[], s32[]) while(tuple.1),
+      condition=UnusedButModifiedTupleElement.cond,
+      body=UnusedButModifiedTupleElement.body
+    ROOT gte = s32[] get-tuple-element(while), index=0
+  }
+  )";
+
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
+  EXPECT_TRUE(TupleSimplifier().Run(m.get()).ok());
+  EXPECT_TRUE(HloDCE().Run(m.get()).ok());
+  auto m_while = AllOf(op::While(), op::Shape("(s32[])"));
+  EXPECT_THAT(m->entry_computation()->root_instruction(),
+              op::GetTupleElement(m_while));
 }
 
 // Nothing to simplify in a while loop whose tuple has 0 elements.
 TEST_F(WhileLoopSimplifierTest, LoopWithEmptyTupleNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule EmptyTuple
   EmptyTuple.body {
     loop_var = () parameter(0)
@@ -344,14 +379,14 @@ TEST_F(WhileLoopSimplifierTest, LoopWithEmptyTupleNotSimplified) {
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // While loop where one tuple element is used twice in the body, and thus can't
 // be simplified away.
 TEST_F(WhileLoopSimplifierTest, LoopWithElemUsedTwiceNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule ElemUsedTwice
   ElemUsedTwice.body {
     param0 = (s32[], s32[]) parameter(0)
@@ -372,15 +407,15 @@ TEST_F(WhileLoopSimplifierTest, LoopWithElemUsedTwiceNotSimplified) {
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 // This while loop has three tuple elements.  Element 0 is unused and should be
 // removed. Element 1 is used by the loop body, and element 2 is used by the
 // loop condition; these two should stay.
 TEST_F(WhileLoopSimplifierTest, RemoveUnusedLoopOperands) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule RemoveUnusedOperands
   RemoveUnusedOperands.body {
     loop_var = (s32[], s32[], s32[]) parameter(0)
@@ -414,8 +449,8 @@ TEST_F(WhileLoopSimplifierTest, RemoveUnusedLoopOperands) {
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
 
   // The original while instruction is still left in the module as a dead
   // instruction, find a while instruction with a different name as the new
@@ -448,7 +483,7 @@ TEST_F(WhileLoopSimplifierTest, RemoveUnusedLoopOperands) {
 // side-effecting instruction.
 TEST_F(WhileLoopSimplifierTest,
        RemoveUnusedLoopOperandsDespiteSideEffectingOps) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule RemoveUnusedOperands
   body {
     loop_var = (s32[]) parameter(0)
@@ -469,8 +504,8 @@ TEST_F(WhileLoopSimplifierTest,
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
 
   // The original while instruction is still left in the module as a dead
   // instruction, find a while instruction with a different name as the new
@@ -486,7 +521,7 @@ TEST_F(WhileLoopSimplifierTest,
 }
 
 TEST_F(WhileLoopSimplifierTest, LoopWithNonTupleBodyShapeNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule BodyHasNonTupleRoot
   BodyHasNonTupleRoot.passthrough {
     ROOT param = (s32[], s32[]) parameter(0)
@@ -503,13 +538,13 @@ TEST_F(WhileLoopSimplifierTest, LoopWithNonTupleBodyShapeNotSimplified) {
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 TEST_F(WhileLoopSimplifierTest,
        LoopWithNonTupleBodyRootInstructionNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule SimpleLoop
   SimpleLoop.body {
     loop_var.1 = (s32[], s32[3]{0}) parameter(0)
@@ -536,12 +571,12 @@ TEST_F(WhileLoopSimplifierTest,
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 TEST_F(WhileLoopSimplifierTest, LoopWithArrayConstantNotSimplified) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule SimpleLoop
   SimpleLoop.body {
     loop_var.1 = (s32[], s32[3]{0}, s32[3]{0}) parameter(0)
@@ -551,7 +586,7 @@ TEST_F(WhileLoopSimplifierTest, LoopWithArrayConstantNotSimplified) {
     get-tuple-element.2 = s32[3]{0} get-tuple-element(loop_var.1), index=1
     get-tuple-element.3 = s32[3]{0} get-tuple-element(loop_var.1), index=2
     add.2 = s32[3]{0} add(get-tuple-element.2, get-tuple-element.3)
-    ROOT tuple = (s32[], s32[3]{0}) tuple(add, add.2, get-tuple-element.3)
+    ROOT tuple = (s32[], s32[3]{0}, s32[3]{0}) tuple(add, add.2, get-tuple-element.3)
   }
   SimpleLoop.condition {
     loop_var.2 = (s32[], s32[3]{0}, s32[3]{0}) parameter(0)
@@ -562,18 +597,18 @@ TEST_F(WhileLoopSimplifierTest, LoopWithArrayConstantNotSimplified) {
   ENTRY SimpleLoop {
     constant.3 = s32[] constant(42)
     constant.4 = s32[3]{0} constant({0, 1, 2})
-    tuple.1 = (s32[], s32[3]{0}) tuple(constant.3, constant.4, constant.4)
+    tuple.1 = (s32[], s32[3]{0}, s32[3]{0}) tuple(constant.3, constant.4, constant.4)
     ROOT while = (s32[], s32[3]{0}, s32[3]{0}) while(tuple.1), condition=
       SimpleLoop.condition, body=SimpleLoop.body
   }
   )";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_FALSE(WhileLoopSimplifier().Run(m.get()).value());
 }
 
 TEST_F(WhileLoopSimplifierTest, FlattenNestedTuple) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule Test
   Body {
     param = ((s32[1]), (s32[2], s32[3], (s32[4]))) parameter(0)
@@ -600,15 +635,14 @@ TEST_F(WhileLoopSimplifierTest, FlattenNestedTuple) {
       condition=Cond, body=Body
   })";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   // DCE away the old loop so there's just one while loop in the module, making
   // it easy to find.
   EXPECT_TRUE(HloDCE().Run(m.get()).ok());
 
   HloInstruction* new_while = FindFirstWhile(m.get());
-  Shape flat_tuple =
-      ParseShape("(s32[1], s32[2], s32[3], s32[4])").ValueOrDie();
+  Shape flat_tuple = ParseShape("(s32[1], s32[2], s32[3], s32[4])").value();
   SCOPED_TRACE(m->ToString());
   EXPECT_TRUE(ShapeUtil::Equal(new_while->shape(), flat_tuple));
   EXPECT_TRUE(ShapeUtil::Equal(
@@ -620,14 +654,14 @@ TEST_F(WhileLoopSimplifierTest, FlattenNestedTuple) {
       flat_tuple));
   EXPECT_TRUE(ShapeUtil::Equal(
       m->entry_computation()->root_instruction()->shape(),
-      ParseShape("((s32[1]), (s32[2], s32[3], (s32[4])))").ValueOrDie()));
+      ParseShape("((s32[1]), (s32[2], s32[3], (s32[4])))").value()));
 }
 
 // Edge-case: All elements of the loop carry are constants which can be removed,
 // leaving us with a nullary loop.  This is a special case, we just replace the
 // loop with its init.
 TEST_F(WhileLoopSimplifierTest, OnlyConstantsInLoopCarry) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule Test
   Body {
     param = (s32[1]) parameter(0)
@@ -644,8 +678,8 @@ TEST_F(WhileLoopSimplifierTest, OnlyConstantsInLoopCarry) {
     ROOT while = (s32[1]) while(init), condition=Cond, body=Body
   })";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   EXPECT_TRUE(HloDCE().Run(m.get()).ok());
   EXPECT_TRUE(TupleSimplifier().Run(m.get()).ok());
   EXPECT_THAT(m->entry_computation()->root_instruction(),
@@ -653,7 +687,7 @@ TEST_F(WhileLoopSimplifierTest, OnlyConstantsInLoopCarry) {
 }
 
 TEST_F(WhileLoopSimplifierTest, RemoveConstantFromLoopCarry) {
-  const string hlo_string = R"(
+  const std::string hlo_string = R"(
   HloModule Test
   Body {
     param = (s32[1], s32[2], s32[3]) parameter(0)
@@ -684,8 +718,8 @@ TEST_F(WhileLoopSimplifierTest, RemoveConstantFromLoopCarry) {
       condition=Cond, body=Body
   })";
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   // DCE away the old loop so there's just one while loop in the module, making
   // it easy to find.
   EXPECT_TRUE(HloDCE().Run(m.get()).ok());
@@ -693,7 +727,7 @@ TEST_F(WhileLoopSimplifierTest, RemoveConstantFromLoopCarry) {
   EXPECT_TRUE(TupleSimplifier().Run(m.get()).ok());
 
   HloInstruction* new_while = FindFirstWhile(m.get());
-  Shape new_while_shape = ParseShape("(s32[1], s32[3])").ValueOrDie();
+  Shape new_while_shape = ParseShape("(s32[1], s32[3])").value();
   EXPECT_TRUE(ShapeUtil::Equal(new_while->shape(), new_while_shape));
   EXPECT_TRUE(ShapeUtil::Equal(
       new_while->while_body()->root_instruction()->shape(), new_while_shape));
@@ -705,7 +739,7 @@ TEST_F(WhileLoopSimplifierTest, RemoveConstantFromLoopCarry) {
       new_while_shape));
   EXPECT_TRUE(
       ShapeUtil::Equal(m->entry_computation()->root_instruction()->shape(),
-                       ParseShape("(s32[1], s32[2], s32[3])").ValueOrDie()));
+                       ParseShape("(s32[1], s32[2], s32[3])").value()));
   EXPECT_THAT(m->entry_computation()->root_instruction(),
               op::Tuple(_, op::Constant(), _));
 }
@@ -744,15 +778,17 @@ const char* const kSimpleMergeInductionVariablesModule = R"(
 
     a1 = TYPE[] get-tuple-element(while), index=0
     b1 = TYPE[] get-tuple-element(while), index=1
-    ROOT sum = TYPE[] add(a1, b1)
+    c1 = TYPE[] get-tuple-element(while), index=2
+    sum = TYPE[] add(a1, b1)
+    ROOT sum.1 = TYPE[] add(sum, c1)
   })";
 
 TEST_F(WhileLoopSimplifierTest, MergeInductionVariables_Simple) {
-  string hlo_string = absl::StrReplaceAll(kSimpleMergeInductionVariablesModule,
-                                          {{"TYPE", "s32"}});
+  std::string hlo_string = absl::StrReplaceAll(
+      kSimpleMergeInductionVariablesModule, {{"TYPE", "s32"}});
 
-  auto m = ParseAndReturnVerifiedModule(hlo_string).ValueOrDie();
-  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).ValueOrDie());
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
   // DCE away the old loop so there's just one while loop in the module, making
   // it easy to find, and run the tuple simplifier to make the resulting HLO
   // easier to check.
@@ -762,8 +798,7 @@ TEST_F(WhileLoopSimplifierTest, MergeInductionVariables_Simple) {
   HloInstruction* new_while = FindFirstWhile(m.get());
   // We should have added a new loop counter for s32[] to the end of the tuple.
   SCOPED_TRACE(m->ToString());
-  Shape new_while_shape =
-      ParseShape("(s32[], s32[], s32[], s32[])").ValueOrDie();
+  Shape new_while_shape = ParseShape("(s32[], s32[], s32[], s32[])").value();
   EXPECT_TRUE(ShapeUtil::Equal(new_while->shape(), new_while_shape));
   EXPECT_TRUE(ShapeUtil::Equal(
       new_while->while_body()->root_instruction()->shape(), new_while_shape));
@@ -786,12 +821,133 @@ TEST_F(WhileLoopSimplifierTest, MergeInductionVariables_Simple) {
 // We shouldn't merge S16 induction variables; we can't create constants of this
 // type because S16 literals are not implemented.
 TEST_F(WhileLoopSimplifierTest, MergeInductionVariables_SkipS16) {
-  string hlo_string = absl::StrReplaceAll(kSimpleMergeInductionVariablesModule,
-                                          {{"TYPE", "s16"}});
-  EXPECT_FALSE(
-      WhileLoopSimplifier()
-          .Run(ParseAndReturnVerifiedModule(hlo_string).ValueOrDie().get())
-          .ValueOrDie());
+  std::string hlo_string = absl::StrReplaceAll(
+      kSimpleMergeInductionVariablesModule, {{"TYPE", "s16"}});
+  EXPECT_FALSE(WhileLoopSimplifier()
+                   .Run(ParseAndReturnVerifiedModule(hlo_string).value().get())
+                   .value());
+}
+
+TEST_F(WhileLoopSimplifierTest, RemoveRepeatedParams) {
+  const std::string hlo_string = R"(
+  HloModule SwappingTupleElements
+
+  SwappingTupleElements.body {
+    loop_var = (s32[], s32[], s32[]) parameter(0)
+    get-tuple-element = s32[] get-tuple-element(loop_var), index=0
+    get-tuple-element.1 = s32[] get-tuple-element(loop_var), index=1
+    get-tuple-element.2 = s32[] get-tuple-element(loop_var), index=2
+    y = s32[] add(get-tuple-element.1, get-tuple-element.2)
+    ROOT tuple = (s32[], s32[], s32[]) tuple(s32[] get-tuple-element, y,
+      s32[] get-tuple-element.2)
+  }
+
+  SwappingTupleElements.always_true {
+   param = (s32[], s32[], s32[]) parameter(0)
+   get-tuple-element = s32[] get-tuple-element(param), index=0
+   get-tuple-element.1 = s32[] get-tuple-element(param), index=1
+   ROOT less-than = pred[] compare(get-tuple-element, get-tuple-element.1), direction=LT
+  }
+
+  ENTRY SwappingTupleElements {
+   x = s32[] parameter(0)
+   y = s32[] parameter(1)
+   tuple.1 = (s32[], s32[], s32[]) tuple(s32[] x, s32[] y, s32[] x)
+   ROOT while = (s32[], s32[], s32[]) while(tuple.1),
+     condition=SwappingTupleElements.always_true,
+     body=SwappingTupleElements.body
+  }
+  )";
+
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  EXPECT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
+  HloInstruction* new_while = FindFirstWhile(m.get());
+  Shape new_while_shape = ParseShape("(s32[], s32[])").value();
+  EXPECT_TRUE(ShapeUtil::Equal(new_while->shape(), new_while_shape));
+  EXPECT_TRUE(ShapeUtil::Equal(
+      new_while->while_body()->root_instruction()->shape(), new_while_shape));
+  EXPECT_TRUE(ShapeUtil::Equal(
+      new_while->while_body()->parameter_instruction(0)->shape(),
+      new_while_shape));
+  EXPECT_TRUE(ShapeUtil::Equal(
+      new_while->while_condition()->parameter_instruction(0)->shape(),
+      new_while_shape));
+}
+
+// A group of elements are inter-dependent, but unused as by the output.
+TEST_F(WhileLoopSimplifierTest, LoopWithUnusedGroupSimplified) {
+  const std::string hlo_string = R"(
+  HloModule LoopWithUnusedGroupSimplified
+  LoopWithUnusedGroupSimplified.body {
+    loop_var = (s32[], s32[], s32[]) parameter(0)
+    constant.1 = s32[] constant(1)
+    gte0 = s32[] get-tuple-element(loop_var), index=1
+    gte1 = s32[] get-tuple-element(loop_var), index=2
+    add = s32[] add(gte0, gte1)
+    ROOT tuple = (s32[], s32[], s32[]) tuple(constant.1, add, add)
+  }
+  LoopWithUnusedGroupSimplified.cond {
+    param = (s32[], s32[], s32[]) parameter(0)
+    gte.cond = s32[] get-tuple-element(param), index=0
+    constant.3 = s32[] constant(1)
+    ROOT lt = pred[] compare(gte.cond, constant.3), direction=LT
+  }
+  ENTRY  LoopWithUnusedGroupSimplified {
+    constant.2 = s32[] constant(0)
+    tuple.1 = (s32[], s32[], s32[]) tuple(constant.2, constant.2, constant.2)
+    while = (s32[], s32[], s32[]) while(tuple.1),
+      condition=LoopWithUnusedGroupSimplified.cond,
+      body=LoopWithUnusedGroupSimplified.body
+    ROOT gte = s32[] get-tuple-element(while), index=0
+  }
+  )";
+
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
+  EXPECT_TRUE(TupleSimplifier().Run(m.get()).ok());
+  EXPECT_TRUE(HloDCE().Run(m.get()).ok());
+  auto m_while = AllOf(op::While(), op::Shape("(s32[])"));
+  EXPECT_THAT(m->entry_computation()->root_instruction(),
+              op::GetTupleElement(m_while));
+}
+
+// An element is not a passthrough, but it's not used by other elements.
+TEST_F(WhileLoopSimplifierTest, LoopWithUnusedNonPassthroughElementSimplified) {
+  const std::string hlo_string = R"(
+  HloModule LoopWithUnusedNonPassthroughElementSimplified
+  LoopWithUnusedNonPassthroughElementSimplified.body {
+    loop_var = (s32[], s32[], s32[]) parameter(0)
+    constant.1 = s32[] constant(1)
+    gte0 = s32[] get-tuple-element(loop_var), index=1
+    gte1 = s32[] get-tuple-element(loop_var), index=2
+    add = s32[] add(gte0, gte1)
+    add2 = s32[] add(gte0, gte0)
+    ROOT tuple = (s32[], s32[], s32[]) tuple(constant.1, add2, add)
+  }
+  LoopWithUnusedNonPassthroughElementSimplified.cond {
+    param = (s32[], s32[], s32[]) parameter(0)
+    gte.cond = s32[] get-tuple-element(param), index=0
+    constant.3 = s32[] constant(1)
+    ROOT lt = pred[] compare(gte.cond, constant.3), direction=LT
+  }
+  ENTRY  LoopWithUnusedNonPassthroughElementSimplified {
+    constant.2 = s32[] constant(0)
+    tuple.1 = (s32[], s32[], s32[]) tuple(constant.2, constant.2, constant.2)
+    while = (s32[], s32[], s32[]) while(tuple.1),
+      condition=LoopWithUnusedNonPassthroughElementSimplified.cond,
+      body=LoopWithUnusedNonPassthroughElementSimplified.body
+    gte2 = s32[] get-tuple-element(while), index=0
+    gte3 = s32[] get-tuple-element(while), index=1
+    ROOT tuple.2 = (s32[], s32[]) tuple(gte2, gte3)
+  }
+  )";
+
+  auto m = ParseAndReturnVerifiedModule(hlo_string).value();
+  ASSERT_TRUE(WhileLoopSimplifier().Run(m.get()).value());
+  EXPECT_TRUE(TupleSimplifier().Run(m.get()).ok());
+  EXPECT_TRUE(HloDCE().Run(m.get()).ok());
+  EXPECT_THAT(m->entry_computation()->root_instruction(),
+              AllOf(op::While(), op::Shape("(s32[], s32[])")));
 }
 
 }  // namespace
