@@ -21,7 +21,6 @@ limitations under the License.
 #include <type_traits>
 #include <vector>
 
-#include "absl/memory/memory.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/span.h"
 #include "tensorflow/compiler/xla/array2d.h"
@@ -34,14 +33,14 @@ limitations under the License.
 #include "tensorflow/compiler/xla/literal.h"
 #include "tensorflow/compiler/xla/literal_util.h"
 #include "tensorflow/compiler/xla/statusor.h"
+#include "tensorflow/compiler/xla/stream_executor/stream_executor.h"
 #include "tensorflow/compiler/xla/tests/literal_test_util.h"
 #include "tensorflow/compiler/xla/tests/manifest_checking_test.h"
 #include "tensorflow/compiler/xla/tests/test_utils.h"
 #include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/core/lib/core/bitmap.h"
-#include "tensorflow/core/platform/stream_executor_no_cuda.h"
-#include "tensorflow/core/platform/test.h"
-#include "tensorflow/core/platform/types.h"
+#include "tensorflow/tsl/lib/core/bitmap.h"
+#include "tensorflow/tsl/platform/float8.h"
+#include "tensorflow/tsl/platform/test.h"
 
 namespace xla {
 
@@ -72,7 +71,7 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
                         const LocalClientOptions& client_options);
 
   // Returns the name of the test currently being run.
-  string TestName() const;
+  std::string TestName() const;
 
   void SetFastMathDisabled(bool disabled) {
     auto* opts = execution_options_.mutable_debug_options();
@@ -81,7 +80,7 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
     opts->set_xla_gpu_enable_fast_min_max(!disabled);
   }
 
-  void SetSeed(uint64 seed) { execution_options_.set_seed(seed); }
+  void SetSeed(uint64_t seed) { execution_options_.set_seed(seed); }
 
   // Provides mutable access to the execution DebugOptions field; this lets
   // tests tweak the options that will be used to compile/run the graph.
@@ -116,8 +115,8 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
 
   // Run a computation and return its value as a string. If an error
   // occurs, then instead return the error as a string.
-  string ExecuteToString(XlaBuilder* builder,
-                         absl::Span<GlobalData* const> arguments);
+  std::string ExecuteToString(XlaBuilder* builder,
+                              absl::Span<GlobalData* const> arguments);
 
   // Convenience methods for building and running a computation, transferring
   // the result, and comparing it to the expected value(s). Methods are
@@ -146,7 +145,7 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // As above, but uses a bitmap to hold the predicate vector to avoid
   // deficiencies of vector<bool>.
   void ComputeAndCompareR1(XlaBuilder* builder,
-                           const tensorflow::core::Bitmap& expected,
+                           const tsl::core::Bitmap& expected,
                            absl::Span<GlobalData* const> arguments);
 
   template <typename NativeT>
@@ -226,7 +225,13 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
                          absl::Span<const Literal> arguments);
   void ComputeAndCompare(XlaBuilder* builder,
                          absl::Span<const Literal> arguments, ErrorSpec error);
-
+  template <typename NativeT>
+  void ComputeAndCompare(XlaBuilder* builder, const Array<NativeT>& expected,
+                         absl::Span<GlobalData* const> arguments);
+  template <typename NativeT>
+  void ComputeAndCompare(XlaBuilder* builder, const Array<NativeT>& expected,
+                         absl::Span<GlobalData* const> arguments,
+                         ErrorSpec error);
   // Create scalar operations for use in reductions.
   XlaComputation CreateScalarRelu();
   XlaComputation CreateScalarMax();
@@ -238,13 +243,13 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // maximum values.
   template <typename NativeT>
   std::vector<NativeT> CreatePseudorandomR1(const int width, NativeT min_value,
-                                            NativeT max_value, uint32 seed);
+                                            NativeT max_value, uint32_t seed);
   template <typename NativeT>
   std::unique_ptr<Array2D<NativeT>> CreatePseudorandomR2(const int rows,
                                                          const int cols,
                                                          NativeT min_value,
                                                          NativeT max_value,
-                                                         uint32 seed);
+                                                         uint32_t seed);
 
   // Creates a (rows x cols) array filled in the following form:
   //
@@ -270,15 +275,15 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // server, then stores into "data_handle" the global handle for that
   // parameter. When the use_bfloat16 flag is set but the literal has F32
   // elements, the literal will be converted to BF16 before being transferred.
-  std::unique_ptr<GlobalData> CreateParameterAndTransferLiteral(
-      int64 parameter_number, const Literal& literal, const string& name,
+  StatusOr<std::unique_ptr<GlobalData>> CreateParameterAndTransferLiteral(
+      int64_t parameter_number, const Literal& literal, const std::string& name,
       XlaBuilder* builder, XlaOp* data_handle);
 
   // As above, but the caller can specify the device that the literal is
   // transferred to. If device_handle is nullptr, the literal will be
   // transferred to the default device.
-  std::unique_ptr<GlobalData> CreateParameterAndTransferLiteral(
-      int64 parameter_number, const Literal& literal, const string& name,
+  StatusOr<std::unique_ptr<GlobalData>> CreateParameterAndTransferLiteral(
+      int64_t parameter_number, const Literal& literal, const std::string& name,
       const DeviceHandle* device_handle, XlaBuilder* builder,
       XlaOp* data_handle);
 
@@ -327,8 +332,8 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // converted to bfloat16.
   template <typename NativeT>
   std::unique_ptr<GlobalData> CreateR0Parameter(NativeT value,
-                                                int64 parameter_number,
-                                                const string& name,
+                                                int64_t parameter_number,
+                                                const std::string& name,
                                                 XlaBuilder* builder,
                                                 XlaOp* data_handle);
 
@@ -342,8 +347,8 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // converted to bfloat16.
   template <typename NativeT>
   std::unique_ptr<GlobalData> CreateR1Parameter(
-      absl::Span<const NativeT> values, int64 parameter_number,
-      const string& name, XlaBuilder* builder, XlaOp* data_handle);
+      absl::Span<const NativeT> values, int64_t parameter_number,
+      const std::string& name, XlaBuilder* builder, XlaOp* data_handle);
 
   // Creates a parameter instruction that wraps the given constant array
   // "array_2d" and then stores it to the global handle for that parameter
@@ -356,8 +361,8 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // converted to bfloat16.
   template <typename NativeT>
   std::unique_ptr<GlobalData> CreateR2Parameter(
-      const Array2D<NativeT>& array_2d, int64 parameter_number,
-      const string& name, XlaBuilder* builder, XlaOp* data_handle);
+      const Array2D<NativeT>& array_2d, int64_t parameter_number,
+      const std::string& name, XlaBuilder* builder, XlaOp* data_handle);
 
   // Creates a parameter instruction that wraps the given constant array
   // "array_3d" and then stores it to the global handle for that parameter
@@ -370,8 +375,8 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // converted to bfloat16.
   template <typename NativeT>
   std::unique_ptr<GlobalData> CreateR3Parameter(
-      const Array3D<NativeT>& array_3d, int64 parameter_number,
-      const string& name, XlaBuilder* builder, XlaOp* data_handle);
+      const Array3D<NativeT>& array_3d, int64_t parameter_number,
+      const std::string& name, XlaBuilder* builder, XlaOp* data_handle);
 
   // Creates a parameter instruction that wraps the given constant array
   // "array_4d" and then stores it to the global handle for that parameter
@@ -384,8 +389,15 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
   // converted to bfloat16.
   template <typename NativeT>
   std::unique_ptr<GlobalData> CreateR4Parameter(
-      const Array4D<NativeT>& array_4d, int64 parameter_number,
-      const string& name, XlaBuilder* builder, XlaOp* data_handle);
+      const Array4D<NativeT>& array_4d, int64_t parameter_number,
+      const std::string& name, XlaBuilder* builder, XlaOp* data_handle);
+
+  template <typename NativeT>
+  std::unique_ptr<GlobalData> CreateParameter(const Array<NativeT>& array_4d,
+                                              int64_t parameter_number,
+                                              const std::string& name,
+                                              XlaBuilder* builder,
+                                              XlaOp* data_handle);
 
   // Getter and setter for the use_bfloat16 flag, which indicates whether to run
   // tests with all float-type input/output converted to bfloat16.
@@ -413,12 +425,14 @@ class ClientLibraryTestBase : public ManifestCheckingTest {
       const xla::XlaComputation& computation, const Literal& expected,
       absl::Span<GlobalData* const> arguments,
       const std::function<void(const Literal& actual,
-                               const string& error_message)>& verify_output);
+                               const std::string& error_message)>&
+          verify_output);
   Status ComputeAndCompareLiteralWithAllInputLayouts(
       const xla::XlaComputation& computation, const Literal& expected,
       absl::Span<GlobalData* const> arguments,
       const std::function<void(const Literal& actual,
-                               const string& error_message)>& verify_output,
+                               const std::string& error_message)>&
+          verify_output,
       const Shape* output_with_layout = nullptr);
 
   // Converts an f32 shape to bf16 if use_bfloat16_ is true.
@@ -449,6 +463,11 @@ void ClientLibraryTestBase::ComputeAndCompareR0(
                     std::is_same<NativeT, double>::value ||
                     std::is_same<NativeT, bfloat16>::value ||
                     std::is_same<NativeT, half>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2fnuz>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fn>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3b11>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fnuz>::value ||
                     std::is_same<NativeT, complex64>::value ||
                     std::is_same<NativeT, complex128>::value,
                 "Float or complex type required when specifying an ErrorSpec");
@@ -474,6 +493,11 @@ void ClientLibraryTestBase::ComputeAndCompareR1(
                     std::is_same<NativeT, double>::value ||
                     std::is_same<NativeT, bfloat16>::value ||
                     std::is_same<NativeT, half>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2fnuz>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fn>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3b11>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fnuz>::value ||
                     std::is_same<NativeT, complex64>::value ||
                     std::is_same<NativeT, complex128>::value,
                 "Float or complex type required when specifying an ErrorSpec");
@@ -500,6 +524,11 @@ void ClientLibraryTestBase::ComputeAndCompareR2(
                     std::is_same<NativeT, double>::value ||
                     std::is_same<NativeT, bfloat16>::value ||
                     std::is_same<NativeT, half>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2fnuz>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fn>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3b11>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fnuz>::value ||
                     std::is_same<NativeT, complex64>::value ||
                     std::is_same<NativeT, complex128>::value,
                 "Float or complex type required when specifying an ErrorSpec");
@@ -527,6 +556,11 @@ void ClientLibraryTestBase::ComputeAndCompareR3(
                     std::is_same<NativeT, double>::value ||
                     std::is_same<NativeT, bfloat16>::value ||
                     std::is_same<NativeT, half>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2fnuz>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fn>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3b11>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fnuz>::value ||
                     std::is_same<NativeT, complex64>::value ||
                     std::is_same<NativeT, complex128>::value,
                 "Float or complex type required when specifying an ErrorSpec");
@@ -554,6 +588,11 @@ void ClientLibraryTestBase::ComputeAndCompareR4(
                     std::is_same<NativeT, double>::value ||
                     std::is_same<NativeT, bfloat16>::value ||
                     std::is_same<NativeT, half>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2fnuz>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fn>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3b11>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fnuz>::value ||
                     std::is_same<NativeT, complex64>::value ||
                     std::is_same<NativeT, complex128>::value,
                 "Float or complex type required when specifying an ErrorSpec");
@@ -563,79 +602,117 @@ void ClientLibraryTestBase::ComputeAndCompareR4(
                                                   arguments, error);
 }
 
+template <typename NativeT>
+void ClientLibraryTestBase::ComputeAndCompare(
+    XlaBuilder* builder, const Array<NativeT>& expected,
+    absl::Span<GlobalData* const> arguments) {
+  Literal expected_literal = LiteralUtil::CreateFromArray<NativeT>(expected);
+  ClientLibraryTestBase::ComputeAndCompareLiteral(builder, expected_literal,
+                                                  arguments);
+}
+
+template <typename NativeT>
+void ClientLibraryTestBase::ComputeAndCompare(
+    XlaBuilder* builder, const Array<NativeT>& expected,
+    absl::Span<GlobalData* const> arguments, ErrorSpec error) {
+  static_assert(std::is_same<NativeT, float>::value ||
+                    std::is_same<NativeT, double>::value ||
+                    std::is_same<NativeT, bfloat16>::value ||
+                    std::is_same<NativeT, half>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2>::value ||
+                    std::is_same<NativeT, tsl::float8_e5m2fnuz>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fn>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3b11>::value ||
+                    std::is_same<NativeT, tsl::float8_e4m3fnuz>::value ||
+                    std::is_same<NativeT, complex64>::value ||
+                    std::is_same<NativeT, complex128>::value,
+                "Float or complex type required when specifying an ErrorSpec");
+  Literal expected_literal = LiteralUtil::CreateFromArray<NativeT>(expected);
+  ClientLibraryTestBase::ComputeAndCompareLiteral(builder, expected_literal,
+                                                  arguments, error);
+}
+
 template <typename NativeT>
 std::unique_ptr<GlobalData> ClientLibraryTestBase::CreateR0Parameter(
-    NativeT value, int64 parameter_number, const string& name,
+    NativeT value, int64_t parameter_number, const std::string& name,
     XlaBuilder* builder, XlaOp* data_handle) {
   Literal literal = LiteralUtil::CreateR0(value);
   if (use_bfloat16_ && literal.shape().element_type() == F32) {
     literal = LiteralUtil::ConvertF32ToBF16(literal);
   }
-  std::unique_ptr<GlobalData> data =
-      client_->TransferToServer(literal).ConsumeValueOrDie();
+  std::unique_ptr<GlobalData> data = client_->TransferToServer(literal).value();
   *data_handle = Parameter(builder, parameter_number, literal.shape(), name);
   return data;
 }
 
 template <typename NativeT>
 std::unique_ptr<GlobalData> ClientLibraryTestBase::CreateR1Parameter(
-    absl::Span<const NativeT> values, int64 parameter_number,
-    const string& name, XlaBuilder* builder, XlaOp* data_handle) {
+    absl::Span<const NativeT> values, int64_t parameter_number,
+    const std::string& name, XlaBuilder* builder, XlaOp* data_handle) {
   Literal literal = LiteralUtil::CreateR1(values);
   if (use_bfloat16_ && literal.shape().element_type() == F32) {
     literal = LiteralUtil::ConvertF32ToBF16(literal);
   }
-  std::unique_ptr<GlobalData> data =
-      client_->TransferToServer(literal).ConsumeValueOrDie();
+  std::unique_ptr<GlobalData> data = client_->TransferToServer(literal).value();
   *data_handle = Parameter(builder, parameter_number, literal.shape(), name);
   return data;
 }
 
 template <typename NativeT>
 std::unique_ptr<GlobalData> ClientLibraryTestBase::CreateR2Parameter(
-    const Array2D<NativeT>& array_2d, int64 parameter_number,
-    const string& name, XlaBuilder* builder, XlaOp* data_handle) {
+    const Array2D<NativeT>& array_2d, int64_t parameter_number,
+    const std::string& name, XlaBuilder* builder, XlaOp* data_handle) {
   Literal literal = LiteralUtil::CreateR2FromArray2D(array_2d);
   if (use_bfloat16_ && literal.shape().element_type() == F32) {
     literal = LiteralUtil::ConvertF32ToBF16(literal);
   }
-  std::unique_ptr<GlobalData> data =
-      client_->TransferToServer(literal).ConsumeValueOrDie();
+  std::unique_ptr<GlobalData> data = client_->TransferToServer(literal).value();
   *data_handle = Parameter(builder, parameter_number, literal.shape(), name);
   return data;
 }
 
 template <typename NativeT>
 std::unique_ptr<GlobalData> ClientLibraryTestBase::CreateR3Parameter(
-    const Array3D<NativeT>& array_3d, int64 parameter_number,
-    const string& name, XlaBuilder* builder, XlaOp* data_handle) {
+    const Array3D<NativeT>& array_3d, int64_t parameter_number,
+    const std::string& name, XlaBuilder* builder, XlaOp* data_handle) {
   Literal literal = LiteralUtil::CreateR3FromArray3D(array_3d);
   if (use_bfloat16_ && literal.shape().element_type() == F32) {
     literal = LiteralUtil::ConvertF32ToBF16(literal);
   }
-  std::unique_ptr<GlobalData> data =
-      client_->TransferToServer(literal).ConsumeValueOrDie();
+  std::unique_ptr<GlobalData> data = client_->TransferToServer(literal).value();
   *data_handle = Parameter(builder, parameter_number, literal.shape(), name);
   return data;
 }
 
 template <typename NativeT>
 std::unique_ptr<GlobalData> ClientLibraryTestBase::CreateR4Parameter(
-    const Array4D<NativeT>& array_4d, int64 parameter_number,
-    const string& name, XlaBuilder* builder, XlaOp* data_handle) {
+    const Array4D<NativeT>& array_4d, int64_t parameter_number,
+    const std::string& name, XlaBuilder* builder, XlaOp* data_handle) {
   Literal literal = LiteralUtil::CreateR4FromArray4D(array_4d);
   if (use_bfloat16_ && literal.shape().element_type() == F32) {
     literal = LiteralUtil::ConvertF32ToBF16(literal);
   }
-  std::unique_ptr<GlobalData> data =
-      client_->TransferToServer(literal).ConsumeValueOrDie();
+  std::unique_ptr<GlobalData> data = client_->TransferToServer(literal).value();
+  *data_handle = Parameter(builder, parameter_number, literal.shape(), name);
+  return data;
+}
+
+template <typename NativeT>
+std::unique_ptr<GlobalData> ClientLibraryTestBase::CreateParameter(
+    const Array<NativeT>& array, int64_t parameter_number,
+    const std::string& name, XlaBuilder* builder, XlaOp* data_handle) {
+  Literal literal = LiteralUtil::CreateFromArray(array);
+  if (use_bfloat16_ && literal.shape().element_type() == F32) {
+    literal = LiteralUtil::ConvertF32ToBF16(literal);
+  }
+  std::unique_ptr<GlobalData> data = client_->TransferToServer(literal).value();
   *data_handle = Parameter(builder, parameter_number, literal.shape(), name);
   return data;
 }
 
 template <typename NativeT>
 std::vector<NativeT> ClientLibraryTestBase::CreatePseudorandomR1(
-    const int width, NativeT min_value, NativeT max_value, uint32 seed) {
+    const int width, NativeT min_value, NativeT max_value, uint32_t seed) {
   std::vector<NativeT> result(width);
   PseudorandomGenerator<NativeT> generator(min_value, max_value, seed);
   for (int i = 0; i < width; ++i) {
@@ -647,8 +724,8 @@ std::vector<NativeT> ClientLibraryTestBase::CreatePseudorandomR1(
 template <typename NativeT>
 std::unique_ptr<Array2D<NativeT>> ClientLibraryTestBase::CreatePseudorandomR2(
     const int rows, const int cols, NativeT min_value, NativeT max_value,
-    uint32 seed) {
-  auto result = absl::make_unique<Array2D<NativeT>>(rows, cols);
+    uint32_t seed) {
+  auto result = std::make_unique<Array2D<NativeT>>(rows, cols);
   PseudorandomGenerator<NativeT> generator(min_value, max_value, seed);
   for (int y = 0; y < rows; ++y) {
     for (int x = 0; x < cols; ++x) {
