@@ -25,16 +25,16 @@ limitations under the License.
 namespace xla {
 namespace {
 
-using absl::nullopt;
+using std::nullopt;
 
 class GatherOperationTest : public HloTestBase {
  protected:
-  void RunTest(const string& hlo_text, Literal* operand,
+  void RunTest(const std::string& hlo_text, Literal* operand,
                Literal* start_indices) {
     RunTest(hlo_text, {operand, start_indices});
   }
 
-  void RunTest(const string& hlo_text, absl::Span<Literal* const> args) {
+  void RunTest(const std::string& hlo_text, absl::Span<Literal* const> args) {
     HloModuleConfig config;
     config.set_debug_options(GetDebugOptionsForTest());
     TF_ASSERT_OK_AND_ASSIGN(std::unique_ptr<HloModule> module,
@@ -44,7 +44,7 @@ class GatherOperationTest : public HloTestBase {
 };
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherV1) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherV1
 
 ENTRY main {
@@ -59,14 +59,14 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({0, 2});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({0, 2});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, BatchDimInMiddle) {
   // Reverse the middle dimension (dim 1).
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDimInMiddle
 
 ENTRY main {
@@ -81,15 +81,15 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR3<int32>({{{1, 2, 3}, {4, 5, 6}},
-                                    {{7, 8, 9}, {10, 11, 12}},
-                                    {{13, 14, 15}, {16, 17, 18}}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({1, 0});
+      LiteralUtil::CreateR3<int32_t>({{{1, 2, 3}, {4, 5, 6}},
+                                      {{7, 8, 9}, {10, 11, 12}},
+                                      {{13, 14, 15}, {16, 17, 18}}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({1, 0});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherV2) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherV2
 
 ENTRY main {
@@ -104,13 +104,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({0, 2});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({0, 2});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherMultipleBatchDims) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherMultipleBatchDims
 
 ENTRY main {
@@ -125,13 +125,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{0, 2}, {2, 1}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{0, 2}, {2, 1}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherNdMultipleBatchDims_0) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherNdMultipleBatchDims
 
 ENTRY main {
@@ -146,14 +146,14 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
   Literal start_indices =
-      LiteralUtil::CreateR3<int32>({{{0, 2}, {2, 1}}, {{1, 2}, {2, 0}}});
+      LiteralUtil::CreateR3<int32_t>({{{0, 2}, {2, 1}}, {{1, 2}, {2, 0}}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherNdMultipleBatchDims_1) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherNdMultipleBatchDims
 
 ENTRY main {
@@ -168,14 +168,14 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
   Literal start_indices =
-      LiteralUtil::CreateR3<int32>({{{0, 2}, {2, 1}}, {{1, 2}, {2, 0}}});
+      LiteralUtil::CreateR3<int32_t>({{{0, 2}, {2, 1}}, {{1, 2}, {2, 0}}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherNd) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherNd
 
 ENTRY main {
@@ -190,15 +190,15 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR3<int32>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
-                                    {{-4, 4}, {-5, 5}, {-6, 6}},  //
-                                    {{-7, 7}, {-8, 8}, {-9, 9}}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{0, 0}, {1, 0}});
+      LiteralUtil::CreateR3<int32_t>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
+                                      {{-4, 4}, {-5, 5}, {-6, 6}},  //
+                                      {{-7, 7}, {-8, 8}, {-9, 9}}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{0, 0}, {1, 0}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, TensorFlowGatherNdNonDefaultIndexVectorDim) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule TensorFlowGatherNd
 
 ENTRY main {
@@ -213,10 +213,10 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR3<int32>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
-                                    {{-4, 4}, {-5, 5}, {-6, 6}},  //
-                                    {{-7, 7}, {-8, 8}, {-9, 9}}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{0, 0}, {1, 0}});
+      LiteralUtil::CreateR3<int32_t>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
+                                      {{-4, 4}, {-5, 5}, {-6, 6}},  //
+                                      {{-7, 7}, {-8, 8}, {-9, 9}}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{0, 0}, {1, 0}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -236,13 +236,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({1, 1});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({1, 1});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, BatchDynamicSlice) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -257,8 +257,8 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{2, 1}, {1, 1}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{2, 1}, {1, 1}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -277,8 +277,8 @@ ENTRY main {
       slice_sizes={1, 0}
 }
 )";
-  Literal operand = LiteralUtil::CreateR2<int32>({{}, {}, {}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({0, 2});
+  Literal operand = LiteralUtil::CreateR2<int32_t>({{}, {}, {}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({0, 2});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -286,7 +286,7 @@ XLA_TEST_F(GatherOperationTest, OutOfBoundsIndex) {
   // Out of bounds indices must not crash, and the indices in range should
   // produce the same values across all backends.
 
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -302,8 +302,8 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>(
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>(
       {{2, 7}, {2, 1}, {1, 1}, {5, 1}, {2147483647, 1}, {1, 2}});
   RunTest(hlo_text, &operand, &start_indices);
 }
@@ -318,7 +318,7 @@ XLA_TEST_F(GatherOperationTest, OutOfBoundsIndex64Bit) {
   // larger than needed to access all values in the input, and the indices
   // produce the same values across all backends.
 
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -334,8 +334,8 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<int64>(
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<int64_t>(
       {{2, 7}, {2, 1}, {1, 1}, {5, 1}, {21474836407, 1}, {1, 2}});
   RunTest(hlo_text, &operand, &start_indices);
 }
@@ -344,7 +344,7 @@ XLA_TEST_F(GatherOperationTest, TooSmallIndex8Bit) {
   // Indices of a type too small to index all locations in gather should not
   // fail.
 
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -359,8 +359,8 @@ ENTRY main {
   ROOT result = s32[6]{0} reshape(gather)
 }
 )";
-  Literal operand = LiteralUtil::MakeIdentityR2<int32>(512);
-  Literal start_indices = LiteralUtil::CreateR2<uint8>(
+  Literal operand = LiteralUtil::MakeIdentityR2<int32_t>(512);
+  Literal start_indices = LiteralUtil::CreateR2<uint8_t>(
       {{2, 7}, {2, 1}, {1, 1}, {5, 1}, {7, 1}, {1, 2}});
   RunTest(hlo_text, &operand, &start_indices);
 }
@@ -371,7 +371,7 @@ XLA_TEST_F(GatherOperationTest, OutOfBoundsUnsignedIndex) {
   // Out of bounds indices must not crash, and the indices in range should
   // produce the same values across all backends.
 
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -387,8 +387,8 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<uint32>(
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<uint32_t>(
       {{2, 7}, {2, 1}, {1, 1}, {5, 1}, {2147483648u, 1}, {1, 2}});
   RunTest(hlo_text, &operand, &start_indices);
 }
@@ -397,7 +397,7 @@ XLA_TEST_F(GatherOperationTest, NegativeIndex) {
   // Negative indices must not crash, and the indices in range should produce
   // the same values across all backends.
 
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -413,14 +413,14 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
   Literal start_indices =
-      LiteralUtil::CreateR2<int32>({{2, -1},
-                                    {2, 1},
-                                    {1, 1},
-                                    {-500, 1},
-                                    {static_cast<int32>(-2147483648), 1},
-                                    {1, 2}});
+      LiteralUtil::CreateR2<int32_t>({{2, -1},
+                                      {2, 1},
+                                      {1, 1},
+                                      {-500, 1},
+                                      {static_cast<int32_t>(-2147483648), 1},
+                                      {1, 2}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -428,7 +428,7 @@ XLA_TEST_F(GatherOperationTest, NegativeIndexIntoUnsignedOperand) {
   // Negative indices must not crash, and the indices in range should produce
   // the same values across all backends.
 
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule BatchDynamicSlice
 
 ENTRY main {
@@ -444,14 +444,14 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<uint32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+      LiteralUtil::CreateR2<uint32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
   Literal start_indices =
-      LiteralUtil::CreateR2<int32>({{2, -1},
-                                    {2, 1},
-                                    {1, 1},
-                                    {-500, 1},
-                                    {static_cast<int32>(-2147483648), 1},
-                                    {1, 2}});
+      LiteralUtil::CreateR2<int32_t>({{2, -1},
+                                      {2, 1},
+                                      {1, 1},
+                                      {-500, 1},
+                                      {static_cast<int32_t>(-2147483648), 1},
+                                      {1, 2}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -470,9 +470,9 @@ ENTRY main {
       slice_sizes={1,3,2}
 }
 )";
-  Literal operand = LiteralUtil::CreateR3<int32>(
+  Literal operand = LiteralUtil::CreateR3<int32_t>(
       {{{1, 2}, {3, 4}, {5, 6}}, {{7, 8}, {9, 10}, {11, 12}}});
-  Literal start_indices = LiteralUtil::CreateR0<int32>(1);
+  Literal start_indices = LiteralUtil::CreateR0<int32_t>(1);
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -491,13 +491,13 @@ ENTRY main {
       slice_sizes={1}
 }
 )";
-  Literal operand = LiteralUtil::CreateR1<int32>({1, 2, 3, 4});
-  Literal start_indices = LiteralUtil::CreateR0<int32>(1);
+  Literal operand = LiteralUtil::CreateR1<int32_t>({1, 2, 3, 4});
+  Literal start_indices = LiteralUtil::CreateR0<int32_t>(1);
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, ZeroSizedResult) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule ZeroSizedResult
 
 ENTRY main {
@@ -512,13 +512,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, FusedTensorFlowGatherV2) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule FusedTensorFlowGatherV2
 
 ENTRY main {
@@ -536,13 +536,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({0, 2});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({0, 2});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, FusedTensorFlowGatherMultipleBatchDims) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule FusedTensorFlowGatherMultipleBatchDims
 
 ENTRY main {
@@ -560,13 +560,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{0, 2}, {2, 1}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{0, 2}, {2, 1}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, FusedTensorFlowGatherNdMultipleBatchDims) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule FusedTensorFlowGatherNdMultipleBatchDims
 
 ENTRY main {
@@ -584,14 +584,14 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
   Literal start_indices =
-      LiteralUtil::CreateR3<int32>({{{0, 2}, {2, 1}}, {{1, 2}, {2, 0}}});
+      LiteralUtil::CreateR3<int32_t>({{{0, 2}, {2, 1}}, {{1, 2}, {2, 0}}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, FusedTensorFlowGatherNd) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule FusedTensorFlowGatherNd
 
 ENTRY main {
@@ -609,16 +609,16 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR3<int32>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
-                                    {{-4, 4}, {-5, 5}, {-6, 6}},  //
-                                    {{-7, 7}, {-8, 8}, {-9, 9}}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{0, 0}, {1, 0}});
+      LiteralUtil::CreateR3<int32_t>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
+                                      {{-4, 4}, {-5, 5}, {-6, 6}},  //
+                                      {{-7, 7}, {-8, 8}, {-9, 9}}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{0, 0}, {1, 0}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest,
            FusedTensorFlowGatherNdNonDefaultIndexVectorDim) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule FusedTensorFlowGatherNd
 
 ENTRY main {
@@ -636,10 +636,10 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR3<int32>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
-                                    {{-4, 4}, {-5, 5}, {-6, 6}},  //
-                                    {{-7, 7}, {-8, 8}, {-9, 9}}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{0, 0}, {1, 0}});
+      LiteralUtil::CreateR3<int32_t>({{{-1, 1}, {-2, 2}, {-3, 3}},  //
+                                      {{-4, 4}, {-5, 5}, {-6, 6}},  //
+                                      {{-7, 7}, {-8, 8}, {-9, 9}}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{0, 0}, {1, 0}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
@@ -662,13 +662,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR1<int32>({1, 1});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({1, 1});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, FusedBatchDynamicSlice) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule FusedBatchDynamicSlice
 
 ENTRY main {
@@ -686,13 +686,13 @@ ENTRY main {
 }
 )";
   Literal operand =
-      LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
-  Literal start_indices = LiteralUtil::CreateR2<int32>({{2, 1}, {1, 1}});
+      LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}});
+  Literal start_indices = LiteralUtil::CreateR2<int32_t>({{2, 1}, {1, 1}});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
 XLA_TEST_F(GatherOperationTest, GatherFromScalar) {
-  const string hlo_text = R"(
+  const std::string hlo_text = R"(
 HloModule GatherFromScalar
 
 ENTRY main {
@@ -707,10 +707,28 @@ ENTRY main {
 }
 )";
   Literal operand = LiteralUtil::CreateR0<float>(1);
-  Literal start_indices = LiteralUtil::CreateR1<int32>({});
+  Literal start_indices = LiteralUtil::CreateR1<int32_t>({});
   RunTest(hlo_text, &operand, &start_indices);
 }
 
+XLA_TEST_F(GatherOperationTest, GatherFromScalarNonZeroIndices) {
+  const std::string hlo_text = R"(
+HloModule GatherFromScalar
+
+ENTRY main {
+  operand = f32[1,1,1] parameter(0)
+  indices = s32[2,3,50] parameter(1)
+  ROOT gather = f32[1,2,50] gather(operand, indices),
+      offset_dims={0},
+      collapsed_slice_dims={0,1},
+      start_index_map={1,0,2},
+      index_vector_dim=1,
+      slice_sizes={1,1,1}
+}
+)";
+  EXPECT_TRUE(RunAndCompare(hlo_text, ErrorSpec{0, 0}));
+}
+
 class GatherClientLibraryTest : public ClientLibraryTestBase {};
 
 // Disabled on interpreter since ExecuteAsyncOnStream is not supported.
@@ -743,14 +761,14 @@ XLA_TEST_F(GatherClientLibraryTest,
   dim_numbers.set_index_vector_dim(1);
   Gather(operand, indices, dim_numbers, {1, 3});
 
-  std::vector<int32> expected = {};
+  std::vector<int32_t> expected = {};
   TF_ASSERT_OK_AND_ASSIGN(
       std::unique_ptr<GlobalData> operand_arg,
       client_->TransferToServer(
-          LiteralUtil::CreateR2<int32>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}})));
+          LiteralUtil::CreateR2<int32_t>({{1, 2, 3}, {4, 5, 6}, {7, 8, 9}})));
   TF_ASSERT_OK_AND_ASSIGN(
       std::unique_ptr<GlobalData> indices_arg,
-      client_->TransferToServer(LiteralUtil::CreateR1<int32>({0, 2})));
+      client_->TransferToServer(LiteralUtil::CreateR1<int32_t>({0, 2})));
   TF_ASSERT_OK_AND_ASSIGN(std::vector<xla::DeviceHandle> devices,
                           client_->GetDeviceHandles(1));
   xla::ExecutionOptions execution_options = CreateDefaultExecutionOptions();
@@ -766,7 +784,8 @@ XLA_TEST_F(GatherClientLibraryTest,
       client_->ExecuteParallel(computation_instances));
   TF_ASSERT_OK_AND_ASSIGN(Literal result_literal,
                           client_->Transfer(*(result_data[0])));
-  LiteralTestUtil::ExpectR2Equal<int32>({{1, 2, 3}, {7, 8, 9}}, result_literal);
+  LiteralTestUtil::ExpectR2Equal<int32_t>({{1, 2, 3}, {7, 8, 9}},
+                                          result_literal);
 }
 }  // namespace
 }  // namespace xla
