@@ -16,19 +16,24 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/graph_constructor.h"
 
 #include <algorithm>
+#include <memory>
+#include <optional>
 #include <set>
+#include <sstream>
 #include <string>
-#include <unordered_map>
-#include <unordered_set>
+#include <utility>
 #include <vector>
 
 #include "absl/algorithm/container.h"
+#include "absl/container/flat_hash_map.h"
 #include "absl/container/flat_hash_set.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 #include "tensorflow/core/common_runtime/shape_refiner.h"
 #include "tensorflow/core/framework/function.h"
 #include "tensorflow/core/framework/function.pb.h"
 #include "tensorflow/core/framework/graph.pb.h"
+#include "tensorflow/core/framework/graph_debug_info.pb.h"
 #include "tensorflow/core/framework/node_def.pb.h"
 #include "tensorflow/core/framework/node_def_util.h"
 #include "tensorflow/core/framework/tensor_shape.pb.h"
@@ -37,6 +42,7 @@ limitations under the License.
 #include "tensorflow/core/framework/versions.pb.h"
 #include "tensorflow/core/graph/algorithm.h"
 #include "tensorflow/core/graph/graph.h"
+#include "tensorflow/core/graph/graph_debug_info_builder.h"
 #include "tensorflow/core/graph/tensor_id.h"
 #include "tensorflow/core/lib/core/errors.h"
 #include "tensorflow/core/lib/gtl/flatmap.h"
@@ -44,6 +50,7 @@ limitations under the License.
 #include "tensorflow/core/lib/gtl/inlined_vector.h"
 #include "tensorflow/core/lib/strings/scanner.h"
 #include "tensorflow/core/lib/strings/str_util.h"
+#include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/platform/macros.h"
 #include "tensorflow/core/public/version.h"
@@ -93,6 +100,10 @@ class GraphConstructor {
     Options(const GraphConstructorOptions& in)  // NOLINT(runtime/explicit)
         : allow_internal_ops(in.allow_internal_ops),
           expect_device_spec(in.expect_device_spec),
+          propagate_device_spec(false),
+          uniquify_names(false),
+          uniquify_prefix(false),
+          skip_mapped_nodes(false),
           importing(false),
           validate_nodes(in.validate_nodes),
           validate_colocation_constraints(false),
@@ -100,6 +111,7 @@ class GraphConstructor {
     Options(const ImportGraphDefOptions& in)  // NOLINT(runtime/explicit)
         : allow_internal_ops(false),
           expect_device_spec(false),
+          propagate_device_spec(in.propagate_device_spec),
           prefix(in.prefix.empty() || str_util::EndsWith(in.prefix, "/")
                      ? in.prefix
                      : in.prefix + "/"),
@@ -118,6 +130,7 @@ class GraphConstructor {
 
     bool allow_internal_ops;
     bool expect_device_spec;
+    bool propagate_device_spec;
 
     string prefix;
     bool uniquify_names;
@@ -153,10 +166,11 @@ class GraphConstructor {
 
   typedef gtl::ArraySlice<const NodeDef*> NodeDefSlice;
 
-  // versions and library may be nullptr
+  // versions, library, and debug_info may be nullptr
   static Status Construct(
       const Options& opts, NodeDefSlice node_defs, const VersionDef* versions,
-      const FunctionDefLibrary* library, Graph* g, ShapeRefiner* refiner,
+      const FunctionDefLibrary* library, const GraphDebugInfo* debug_info,
+      Graph* g, ShapeRefiner* refiner,
       std::vector<std::pair<Node*, int>>* return_tensors,
       std::vector<Node*>* return_nodes,
       std::vector<SafeTensorId>* missing_unused_input_map_keys);
@@ -200,7 +214,7 @@ class GraphConstructor {
     TF_RETURN_IF_ERROR(PopulateMissingUnusedInputMapKeys());
     UpdateUniquifiedColocationNames();
     FixupSourceAndSinkEdges(g_);
-    return Status::OK();
+    return OkStatus();
   }
 
  private:
@@ -215,6 +229,9 @@ class GraphConstructor {
   Status PopulateReturnNodes();
   Status PopulateMissingUnusedInputMapKeys();
 
+  FunctionDefLibraryStackTraces CreateStackTracesForFunctionDefLibrary(
+      const FunctionDefLibrary& library) const;
+
   void Undo();
 
   // Prints cycles in the graph.
@@ -222,7 +239,8 @@ class GraphConstructor {
   // Performs DFS starting at `cur_node` and prints any cycles found.
   void DFS(int cur_node, std::vector<int>* cur_branch,
            std::vector<bool>* is_on_cur_branch,
-           absl::flat_hash_set<int>* unvisited);
+           absl::flat_hash_set<int>* unvisited,
+           const std::vector<absl::string_view>& node_names);
   Status IsNodeFullyMapped(const NodeDef& node_def, bool* is_node_mapped);
   Status ValidateColocationConstraints(const NodeDef& node_def);
   Status MakeNode(NodeDef&& node_def, Node** node);
@@ -285,9 +303,11 @@ class GraphConstructor {
   // Returns the version information for the graph, or nullptr if none is
   // available.
   virtual const VersionDef* versions() const = 0;
-  // Returns the function information for the graph, or nullptr if none is
-  // available.
-  virtual const FunctionDefLibrary* library() const = 0;
+  // Destructively reads the function information for the graph, or nullopt if
+  // none is available.
+  virtual std::optional<FunctionDefLibrary> consume_library() = 0;
+  // Returns the debug info for the graph, or nullptr if none is available.
+  virtual const GraphDebugInfo* debug_info() const = 0;
 
   // From constructor
   const Options opts_;
@@ -297,6 +317,8 @@ class GraphConstructor {
   // A copy of opts_.prefix, possibly uniquified.
   string prefix_;
 
+  StackTracesMap traces_;
+
   ShapeRefiner* refiner_;
 
   // May be null. Not owned.
@@ -323,21 +345,16 @@ class GraphConstructor {
     int gdef_index;
     Node* node;  // nullptr until the NodeDef is converted to a Node.
   };
-  gtl::FlatMap<StringPiece, NodeInfo, StringPieceHasher> gdef_nodes_;
-
-  // Storage for StringPiece keys in gdef_nodes_. Typically, the StringPiece key
-  // will refer to the string stored in `NodeDef::name()`. This intern table is
-  // only used when the original NodeDef's name is changed.
-  std::vector<string> string_intern_table_;
+  absl::flat_hash_map<std::string, NodeInfo> gdef_nodes_;
 
   // Prefixes already used in the GraphDef being imported.
-  gtl::FlatSet<StringPiece, StringPieceHasher> gdef_prefixes_;
+  absl::flat_hash_set<StringPiece> gdef_prefixes_;
 
   // Mapping from node name to the existing node in g_.
-  gtl::FlatMap<StringPiece, Node*, StringPieceHasher> existing_nodes_;
+  absl::flat_hash_map<StringPiece, Node*> existing_nodes_;
 
   // Prefixes already used in the graph.
-  gtl::FlatSet<StringPiece, StringPieceHasher> existing_prefixes_;
+  absl::flat_hash_set<StringPiece> existing_prefixes_;
 
   // Imported node names that have been uniquified. The key is the original
   // name, the value is the new unique name.
@@ -401,7 +418,8 @@ class NodeDefCopyingGraphConstructor : public GraphConstructor {
  public:
   NodeDefCopyingGraphConstructor(
       const Options& opts, NodeDefSlice node_defs, const VersionDef* versions,
-      const FunctionDefLibrary* library, Graph* g, ShapeRefiner* refiner,
+      const FunctionDefLibrary* library, const GraphDebugInfo* debug_info,
+      Graph* g, ShapeRefiner* refiner,
       std::vector<std::pair<Node*, int>>* return_tensors,
       std::vector<Node*>* return_nodes,
       std::vector<SafeTensorId>* missing_unused_input_map_keys)
@@ -409,18 +427,27 @@ class NodeDefCopyingGraphConstructor : public GraphConstructor {
                          missing_unused_input_map_keys),
         node_defs_(node_defs),
         versions_(versions),
-        library_(library) {}
+        library_(library),
+        debug_info_(debug_info) {}
 
  private:
   size_t node_def_count() const override { return node_defs_.size(); }
   const NodeDef& get_node_def(int i) const override { return *node_defs_[i]; }
   NodeDef consume_node_def(int i) override { return *node_defs_[i]; }
   const VersionDef* versions() const override { return versions_; }
-  const FunctionDefLibrary* library() const override { return library_; }
+  std::optional<FunctionDefLibrary> consume_library() override {
+    if (library_ == nullptr) {
+      return std::nullopt;
+    } else {
+      return *library_;
+    }
+  }
+  const GraphDebugInfo* debug_info() const override { return debug_info_; }
 
   const NodeDefSlice node_defs_;
   const VersionDef* const versions_;
   const FunctionDefLibrary* const library_;
+  const GraphDebugInfo* const debug_info_;
 };
 
 // Implementation of GraphConstructor that takes ownership of the input
@@ -450,8 +477,11 @@ class NodeDefMovingGraphConstructor : public GraphConstructor {
     return std::move(*graph_def_.mutable_node(i));
   }
   const VersionDef* versions() const override { return &graph_def_.versions(); }
-  const FunctionDefLibrary* library() const override {
-    return &graph_def_.library();
+  std::optional<FunctionDefLibrary> consume_library() override {
+    return std::move(*graph_def_.mutable_library());
+  }
+  const GraphDebugInfo* debug_info() const override {
+    return &graph_def_.debug_info();
   }
 
   GraphDef graph_def_;
@@ -470,24 +500,25 @@ Status MaybeAppendVersionWarning(const VersionDef* versions,
     return Status(
         import_status.code(),
         absl::StrCat(
-            "Converting GraphDef to Graph has failed. The binary trying to "
-            "import the GraphDef was built when GraphDef version was ",
+            "Converting GraphDef to Graph has failed with an error: '",
+            import_status.message(),
+            "' The binary trying to import the GraphDef was built when "
+            "GraphDef version was ",
             TF_GRAPH_DEF_VERSION,
             ". The GraphDef was produced by a binary built when GraphDef "
             "version was ",
             versions->producer(),
             ". The difference between these versions is larger than "
-            "TensorFlow's forward compatibility guarantee. The following error "
-            "might be due to the binary trying to import the GraphDef being "
-            "too old: ",
-            import_status.error_message()));
+            "TensorFlow's forward compatibility guarantee, and might be the "
+            "root cause for failing to import the GraphDef."));
   }
   return import_status;
 }
 
 /* static */ Status GraphConstructor::Construct(
     const Options& opts, NodeDefSlice node_defs, const VersionDef* versions,
-    const FunctionDefLibrary* library, Graph* g, ShapeRefiner* refiner,
+    const FunctionDefLibrary* library, const GraphDebugInfo* debug_info,
+    Graph* g, ShapeRefiner* refiner,
     std::vector<std::pair<Node*, int>>* return_tensors,
     std::vector<Node*>* return_nodes,
     std::vector<SafeTensorId>* missing_unused_input_map_keys) {
@@ -496,9 +527,9 @@ Status MaybeAppendVersionWarning(const VersionDef* versions,
                                      TF_GRAPH_DEF_VERSION_MIN_PRODUCER,
                                      "GraphDef", "graph"));
   }
-  NodeDefCopyingGraphConstructor c(opts, node_defs, versions, library, g,
-                                   refiner, return_tensors, return_nodes,
-                                   missing_unused_input_map_keys);
+  NodeDefCopyingGraphConstructor c(opts, node_defs, versions, library,
+                                   debug_info, g, refiner, return_tensors,
+                                   return_nodes, missing_unused_input_map_keys);
   Status s = c.TryImport();
   if (!s.ok()) {
     c.Undo();
@@ -565,7 +596,7 @@ bool NodeNameInValues(const std::vector<string>& control_dependencies,
 // Adds any prefixes of `node_name` (not including the full name itself) to
 // `prefixes`.
 void AddPrefixes(StringPiece node_name,
-                 gtl::FlatSet<StringPiece, StringPieceHasher>* prefixes) {
+                 absl::flat_hash_set<StringPiece>* prefixes) {
   size_t idx = -1;
   while ((idx = node_name.find('/', idx + 1)) != StringPiece::npos) {
     prefixes->insert(node_name.substr(0, idx));
@@ -611,7 +642,7 @@ Status GraphConstructor::EnsureNoNameCollisions() {
       prefix_ = strings::StrCat(FindUniqueName(prefix_no_slash), "/");
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::ValidateInputMapAndControlDependencies() {
@@ -638,7 +669,7 @@ Status GraphConstructor::ValidateInputMapAndControlDependencies() {
           "graph");
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::BuildNodeIndex() {
@@ -650,8 +681,7 @@ Status GraphConstructor::BuildNodeIndex() {
           "Node '", node_def.name(),
           "': Node name contains invalid characters");
     }
-    if (!gdef_nodes_
-             .insert(std::make_pair(StringPiece(node_def.name()), NodeInfo(n)))
+    if (!gdef_nodes_.insert(std::make_pair(node_def.name(), NodeInfo(n)))
              .second) {
       return errors::InvalidArgument("Node '", node_def.name(),
                                      "' is not unique");
@@ -683,7 +713,7 @@ Status GraphConstructor::BuildNodeIndex() {
     // Update gdef_prefixes_.
     AddPrefixes(node_def.name(), &gdef_prefixes_);
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::InitFromEdges() {
@@ -707,7 +737,7 @@ Status GraphConstructor::InitFromEdges() {
       // identified by an edge from a NextIteration node to a Merge node. For
       // such Merge nodes, only wait for one non-control input before
       // considering the node ready to process in Convert().
-      int32 num_control_edges = 0;
+      int32_t num_control_edges = 0;
       bool has_loop_back_edge = false;
       for (int i = 0; i < node_def.input_size(); ++i) {
         StringPiece input_name(node_def.input(i));
@@ -750,15 +780,15 @@ Status GraphConstructor::InitFromEdges() {
     }
     pending_count_.push_back(pending_count);
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::ValidateColocationConstraints(
     const NodeDef& node_def) {
   if (!opts_.validate_colocation_constraints || !opts_.importing)
-    return Status::OK();
+    return OkStatus();
   const auto iter = node_def.attr().find(kColocationAttrName);
-  if (iter == node_def.attr().end()) return Status::OK();
+  if (iter == node_def.attr().end()) return OkStatus();
   for (const string& c : iter->second.list().s()) {
     StringPiece s(c);
     if (absl::ConsumePrefix(&s, kColocationGroupPrefix) &&
@@ -768,7 +798,7 @@ Status GraphConstructor::ValidateColocationConstraints(
           "' expects to be colocated with unknown node '", s, "'");
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::MakeNode(NodeDef&& node_def, Node** node) {
@@ -776,21 +806,22 @@ Status GraphConstructor::MakeNode(NodeDef&& node_def, Node** node) {
   Status status;
   *node = g_->AddNode(std::move(node_def), &status);
   if (!status.ok()) return status;
-  if (opts_.expect_device_spec) {
+  if (opts_.expect_device_spec ||
+      (opts_.propagate_device_spec && !(*node)->def().device().empty())) {
     (*node)->set_assigned_device_name((*node)->def().device());
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::ValidateShape(Node* node) {
-  if (!opts_.importing || !opts_.validate_shape) return Status::OK();
+  if (!opts_.importing || !opts_.validate_shape) return OkStatus();
   TF_RETURN_IF_ERROR(refiner_->AddNode(node));
   // For nodes with the _output_shapes attribute, override the shape.
   std::vector<const TensorShapeProto*> shape_attrs;
   const char* kAttrName = "_output_shapes";
   if (!TryGetNodeAttr(node->attrs(), kAttrName, &shape_attrs)) {
     // No _output_shapes attribute, the AddNode call above was sufficient.
-    return Status::OK();
+    return OkStatus();
   }
   auto* ic = refiner_->GetContext(node);
   DCHECK(ic != nullptr)
@@ -817,18 +848,18 @@ Status GraphConstructor::ValidateShape(Node* node) {
     if (!s.ok()) {
       return errors::InvalidArgument("Node '", node->name(), " has an invalid ",
                                      kAttrName, " attribute (shape #", i,
-                                     " error:'", s.error_message(), "'");
+                                     " error:'", s.message(), "'");
     }
     s = refiner_->SetShape(node, i, h);
     if (!s.ok()) {
       return errors::InvalidArgument(
           "Node '", node->name(), "' has an ", kAttrName,
           " attribute inconsistent with the GraphDef for output #", i, ": ",
-          s.error_message());
+          s.message());
     }
   }
   node->ClearAttr(kAttrName);
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::ModifyNodeDefForImport(NodeDef* node_def) {
@@ -839,7 +870,7 @@ Status GraphConstructor::ModifyNodeDefForImport(NodeDef* node_def) {
   if (versions()) {
     TF_RETURN_IF_ERROR(CheckOpDeprecation(*op_def, versions()->producer()));
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 void RemoveInputs(const std::vector<int>& inputs_to_remove, NodeDef* node_def,
@@ -1051,16 +1082,17 @@ Status GraphConstructor::IsNodeFullyMapped(const NodeDef& node_def,
   for (int i = 0; i < op_def->output_arg_size(); ++i) {
     if (opts_.input_map.find({node_def.name(), i}) == opts_.input_map.end()) {
       *is_node_mapped = false;
-      return Status::OK();
+      return OkStatus();
     }
   }
   *is_node_mapped = true;
-  return Status::OK();
+  return OkStatus();
 }
 
 void GraphConstructor::DFS(int cur_node, std::vector<int>* cur_branch,
                            std::vector<bool>* is_on_cur_branch,
-                           absl::flat_hash_set<int>* unvisited) {
+                           absl::flat_hash_set<int>* unvisited,
+                           const std::vector<absl::string_view>& node_names) {
   cur_branch->push_back(cur_node);
   is_on_cur_branch->at(cur_node) = true;
   for (auto next_node : outputs_[cur_node]) {
@@ -1070,12 +1102,14 @@ void GraphConstructor::DFS(int cur_node, std::vector<int>* cur_branch,
             std::find(cur_branch->begin(), cur_branch->end(), next_node);
         LOG(WARNING) << "Cycle detected:";
         while (iter != cur_branch->end()) {
-          LOG(WARNING) << SummarizeNodeDef(get_node_def(*iter));
+          const absl::string_view name = node_names[*iter];
+          DCHECK(!name.empty());
+          LOG(WARNING) << "node id=" << *iter << ", name=" << name;
           ++iter;
         }
         LOG(WARNING) << "End of cycle";
       } else {
-        DFS(next_node, cur_branch, is_on_cur_branch, unvisited);
+        DFS(next_node, cur_branch, is_on_cur_branch, unvisited, node_names);
       }
     }
   }
@@ -1086,10 +1120,20 @@ void GraphConstructor::DFS(int cur_node, std::vector<int>* cur_branch,
 
 void GraphConstructor::PrintCycles() {
   int num_nodes = outputs_.size();
+
+  std::vector<absl::string_view> node_names;
+  node_names.resize(num_nodes);
+  for (const auto& named_node : gdef_nodes_) {
+    DCHECK_GE(named_node.second.gdef_index, 0);
+    DCHECK_LT(named_node.second.gdef_index, num_nodes);
+    node_names[named_node.second.gdef_index] = named_node.first;
+  }
+
   absl::flat_hash_set<int> unvisited;
   for (int i = 0; i < num_nodes; i++) {
     unvisited.insert(i);
   }
+
   while (!unvisited.empty()) {
     int cur_node = *unvisited.begin();
     // Nodes on the current branch of DFS in traversal order. This is used for
@@ -1100,17 +1144,32 @@ void GraphConstructor::PrintCycles() {
     //   (std::find(cur_branch.start(),
     //              cur_branch.end(), i) != cur_branch.end())
     std::vector<bool> is_on_cur_branch(num_nodes, false);
-    DFS(cur_node, &cur_branch, &is_on_cur_branch, &unvisited);
+    DFS(cur_node, &cur_branch, &is_on_cur_branch, &unvisited, node_names);
   }
 }
 
 Status GraphConstructor::Convert() {
+  if (debug_info() != nullptr) {
+    traces_ = LoadTracesFromDebugInfo(*debug_info());
+  }
+
   // Import functions before adding nodes, since imported nodes may refer to
   // functions
-  if (library()) {
-    // TODO(b/135705010): Add rvalue overloads into the function library, to
-    // avoid unnecessarily copying `*library()` here.
-    TF_RETURN_IF_ERROR(g_->AddFunctionLibrary(*library()));
+  if (auto library = consume_library(); library.has_value()) {
+    FunctionDefLibraryStackTraces library_traces;
+    for (const FunctionDef& fdef : library->function()) {
+      const std::string& function_name = fdef.signature().name();
+      StackTracesMap& function_traces = library_traces[function_name];
+      std::string key_suffix = absl::StrCat("@", function_name);
+      for (const auto& [traces_key, stack_trace] : traces_) {
+        if (!absl::EndsWith(traces_key, key_suffix)) continue;
+        std::string node_key =
+            std::string(absl::StripSuffix(traces_key, key_suffix));
+        function_traces[node_key] = stack_trace;
+      }
+    }
+    TF_RETURN_IF_ERROR(
+        g_->AddFunctionLibrary(*std::move(library), library_traces));
   }
 
   std::vector<InputInfo> inputs;
@@ -1138,14 +1197,9 @@ Status GraphConstructor::Convert() {
     input_already_exists.clear();
     input_already_exists.resize(node_def.input_size(), false);
 
-    ssize_t string_intern_table_index = -1;
+    std::string node_name = node_def.name();
 
     if (opts_.importing) {
-      // Intern the original node name, so that we can use a StringPiece of the
-      // name to index gdef_nodes_.
-      string_intern_table_index = string_intern_table_.size();
-      string_intern_table_.push_back(node_def.name());
-
       if (opts_.skip_mapped_nodes) {
         bool is_node_mapped = false;
         TF_RETURN_IF_ERROR(IsNodeFullyMapped(node_def, &is_node_mapped));
@@ -1244,15 +1298,14 @@ Status GraphConstructor::Convert() {
 
     TF_RETURN_IF_ERROR(MakeNode(std::move(node_def), &node));
 
-    if (opts_.importing) {
-      // Use interned original node name so StringPiece remains valid.
-      DCHECK_GE(string_intern_table_index, 0);
-      gdef_nodes_[string_intern_table_[string_intern_table_index]].node = node;
-    } else {
-      DCHECK_EQ(string_intern_table_index, -1);
-      gdef_nodes_[node->name()].node = node;
+    if (node != nullptr) {
+      if (traces_.contains(node_name)) {
+        node->SetStackTrace(traces_[node_name]);
+      }
     }
 
+    gdef_nodes_[node_name].node = node;
+
     // Remove duplicate control inputs before adding edges to the graph. It
     // will allow us to skip expensive duplicates check in 'AddControlEdge'.
     auto first_control = absl::c_find_if(inputs, &InputInfo::IsControlInput);
@@ -1284,7 +1337,7 @@ Status GraphConstructor::Convert() {
   if (processed < node_def_count()) {
     LOG(WARNING) << "IN " << __func__ << " " << (node_def_count() - processed)
                  << " NODES IN A CYCLE";
-    for (int64 i = 0; i < node_def_count(); i++) {
+    for (int64_t i = 0; i < node_def_count(); i++) {
       if (pending_count_[i] != 0) {
         LOG(WARNING) << "PENDING: " << SummarizeNodeDef(get_node_def(i))
                      << " WITH PENDING COUNT = " << pending_count_[i];
@@ -1295,7 +1348,7 @@ Status GraphConstructor::Convert() {
                                    " nodes in a cycle");
   }
 
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::AddBackEdges() {
@@ -1312,15 +1365,15 @@ Status GraphConstructor::AddBackEdges() {
     VLOG(2) << "Add back edge: " << src_node->name() << " -> "
             << e.dst_node->name();
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::UpdateVersionDef() {
-  if (versions() == nullptr) return Status::OK();
+  if (versions() == nullptr) return OkStatus();
 
   if (!opts_.importing) {
     g_->set_versions(*versions());
-    return Status::OK();
+    return OkStatus();
   }
   VersionDef g_versions = g_->versions();
   g_versions.set_producer(
@@ -1338,11 +1391,11 @@ Status GraphConstructor::UpdateVersionDef() {
     }
   }
   g_->set_versions(g_versions);
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::PopulateReturnTensors() {
-  if (opts_.return_tensors.empty()) return Status::OK();
+  if (opts_.return_tensors.empty()) return OkStatus();
   for (const TensorId& id : opts_.return_tensors) {
     auto iter = opts_.input_map.find(id);
     if (iter == opts_.input_map.end()) {
@@ -1369,11 +1422,11 @@ Status GraphConstructor::PopulateReturnTensors() {
       return_tensors_->push_back({node, remapped_id.second});
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::PopulateReturnNodes() {
-  if (opts_.return_nodes.empty()) return Status::OK();
+  if (opts_.return_nodes.empty()) return OkStatus();
   for (StringPiece name : opts_.return_nodes) {
     auto iter = gdef_nodes_.find(name);
     if (iter == gdef_nodes_.end()) {
@@ -1382,11 +1435,11 @@ Status GraphConstructor::PopulateReturnNodes() {
     }
     return_nodes_->push_back(iter->second.node);
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status GraphConstructor::PopulateMissingUnusedInputMapKeys() {
-  if (missing_unused_input_map_keys_ == nullptr) return Status::OK();
+  if (missing_unused_input_map_keys_ == nullptr) return OkStatus();
   for (const auto& input_map_pair : opts_.input_map) {
     TensorId key = input_map_pair.first;
     if (used_input_map_keys_.count(key) > 0) continue;
@@ -1411,7 +1464,7 @@ Status GraphConstructor::PopulateMissingUnusedInputMapKeys() {
       missing_unused_input_map_keys_->push_back(key);
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 void GraphConstructor::Undo() {
@@ -1425,6 +1478,17 @@ void GraphConstructor::Undo() {
 
 Status GraphConstructor::MakeEdge(Node* src, int output_index, Node* dst,
                                   int input_index) {
+  if (output_index >= src->num_outputs()) {
+    return errors::InvalidArgument(
+        "Output ", output_index, " of node ", src->name(),
+        " does not exist. Node only has ", src->num_outputs(), " outputs.");
+  }
+  if (input_index >= dst->num_inputs()) {
+    return errors::InvalidArgument(
+        "Input ", input_index, " of node ", dst->name(),
+        " does not exist. Node only has ", dst->num_inputs(), " inputs.");
+  }
+
   DataType src_out = src->output_type(output_index);
   DataType dst_in = dst->input_type(input_index);
   if (!TypesCompatible(dst_in, src_out)) {
@@ -1434,17 +1498,16 @@ Status GraphConstructor::MakeEdge(Node* src, int output_index, Node* dst,
         " incompatible with expected ", DataTypeString(dst_in), ".");
   }
   g_->AddEdge(src, output_index, dst, input_index);
-  return Status::OK();
+  return OkStatus();
 }
-
 }  // namespace
 
 Status ConvertGraphDefToGraph(const GraphConstructorOptions& opts,
                               const GraphDef& gdef, Graph* g) {
   ShapeRefiner refiner(gdef.versions().producer(), g->op_registry());
   return GraphConstructor::Construct(
-      opts, gdef.node(), &gdef.versions(), &gdef.library(), g, &refiner,
-      /*return_tensors=*/nullptr, /*return_nodes=*/nullptr,
+      opts, gdef.node(), &gdef.versions(), &gdef.library(), &gdef.debug_info(),
+      g, &refiner, /*return_tensors=*/nullptr, /*return_nodes=*/nullptr,
       /*missing_unused_input_map_keys=*/nullptr);
 }
 
@@ -1458,7 +1521,8 @@ Status ConvertGraphDefToGraph(const GraphConstructorOptions& opts,
 }
 
 Status ConvertNodeDefsToGraph(const GraphConstructorOptions& opts,
-                              gtl::ArraySlice<NodeDef> nodes, Graph* g) {
+                              gtl::ArraySlice<NodeDef> nodes, Graph* g,
+                              const GraphDebugInfo* debug_info) {
   ShapeRefiner refiner(TF_GRAPH_DEF_VERSION, g->op_registry());
   // TODO(irving): Copy will go away once NodeInfo exists
   std::vector<const NodeDef*> node_defs;
@@ -1466,8 +1530,9 @@ Status ConvertNodeDefsToGraph(const GraphConstructorOptions& opts,
   for (const auto& n : nodes) {
     node_defs.push_back(&n);
   }
-  return GraphConstructor::Construct(opts, node_defs, nullptr, nullptr, g,
-                                     &refiner, /*return_tensors=*/nullptr,
+  return GraphConstructor::Construct(opts, node_defs, nullptr, nullptr,
+                                     debug_info, g, &refiner,
+                                     /*return_tensors=*/nullptr,
                                      /*return_nodes=*/nullptr,
                                      /*missing_unused_input_map_keys=*/nullptr);
 }
@@ -1535,39 +1600,16 @@ Status ImportGraphDef(const ImportGraphDefOptions& opts, const GraphDef& gdef,
 
   if (results == nullptr) {
     return GraphConstructor::Construct(opts, gdef.node(), &gdef.versions(),
-                                       &gdef.library(), g, refiner, nullptr,
-                                       nullptr, nullptr);
+                                       &gdef.library(), &gdef.debug_info(), g,
+                                       refiner, nullptr, nullptr, nullptr);
   } else {
     return GraphConstructor::Construct(
-        opts, gdef.node(), &gdef.versions(), &gdef.library(), g, refiner,
-        &results->return_tensors, &results->return_nodes,
-        &results->missing_unused_input_map_keys);
+        opts, gdef.node(), &gdef.versions(), &gdef.library(),
+        &gdef.debug_info(), g, refiner, &results->return_tensors,
+        &results->return_nodes, &results->missing_unused_input_map_keys);
   }
 }
 
-void CopyGraph(const Graph& src, Graph* dest) {
-  for (Node* n : dest->nodes()) {
-    CHECK(n->IsSource() || n->IsSink()) << "*dest must be empty";
-  }
-
-  // Copy GraphDef versions
-  dest->set_versions(src.versions());
-
-  // Copy the nodes.
-  // "Node in src" -> "Node in *dest"
-  gtl::FlatMap<const Node*, Node*> node_map;
-  node_map[src.source_node()] = dest->source_node();
-  node_map[src.sink_node()] = dest->sink_node();
-  for (Node* n : src.op_nodes()) {
-    node_map[n] = dest->CopyNode(n);
-  }
-
-  // Copy the edges
-  for (const Edge* e : src.edges()) {
-    Node* src_copy = node_map[e->src()];
-    Node* dst_copy = node_map[e->dst()];
-    dest->AddEdge(src_copy, e->src_output(), dst_copy, e->dst_input());
-  }
-}
+void CopyGraph(const Graph& src, Graph* dest) { dest->Copy(src); }
 
 }  // namespace tensorflow
