@@ -15,6 +15,7 @@ limitations under the License.
 #include "tensorflow/core/common_runtime/ring_alg.h"
 
 #include <stdlib.h>
+
 #include <atomic>
 #include <functional>
 #include <utility>
@@ -47,10 +48,11 @@ limitations under the License.
 // dynamically so that the resulting chunk size does not exceed
 // kMaxChunkSizeBytes, empirically set at 4 MiB.
 constexpr size_t kMaxChunkSizeBytes = (4 * 1024 * 1024);
-// kMaxSubdivsPerDev is used to give an upper bound on the number of
-// subdivisions dynamically generated.  A reasonable value would be a small
+// kMaxSubdivsPerDeviceDefault is used to give an upper bound on the number of
+// subdivisions dynamically generated when user does not provide the parameter
+// through the collectives API. A reasonable value would be a small
 // multiple of the number of NICs adjacent to each device.
-constexpr int kMaxSubdivsPerDevice = 2;
+constexpr int kMaxSubdivsPerDeviceDefault = 2;
 
 namespace tensorflow {
 namespace {
@@ -106,12 +108,26 @@ RingAlg::RingAlg(CollectiveType type, const string& name)
 
 namespace {
 Status GenerateSubdivsInCollectiveParams(CollectiveParams* col_params) {
+  // This function generates subdivision_offsets. Expect it to be empty when
+  // called.
+  DCHECK(col_params->instance.impl_details.subdiv_offsets.empty());
+
+  if (col_params->instance.impl_details.max_subdivs_per_device == -1) {
+    col_params->instance.impl_details.subdiv_offsets = {0};
+    VLOG(2) << "Limiting to 1 subdivision as max_subdivs_per_device == -1";
+    return OkStatus();
+  }
+
   if (col_params->instance.shape.num_elements() == 0) {
     return errors::Internal("shape in CollectiveParams should be non-empty");
   }
   const int kAvgDevPerTask =
       col_params->group.group_size / col_params->group.num_tasks;
-  const int kMaxNumSubdivs = kMaxSubdivsPerDevice * kAvgDevPerTask;
+  const int max_subdivs_per_device =
+      (col_params->instance.impl_details.max_subdivs_per_device > 0)
+          ? col_params->instance.impl_details.max_subdivs_per_device
+          : kMaxSubdivsPerDeviceDefault;
+  const int kMaxNumSubdivs = max_subdivs_per_device * kAvgDevPerTask;
   if (kMaxNumSubdivs <= 0) {
     return errors::Internal("Unexpected kMaxNumSubdivs ", kMaxNumSubdivs,
                             " in ",
@@ -157,13 +173,13 @@ Status GenerateSubdivsInCollectiveParams(CollectiveParams* col_params) {
             << tensor_size << " chunk_size " << chunk_size;
   }
 
-  return Status::OK();
+  return OkStatus();
 }
 }  // namespace
 
 Status RingAlg::InitializeCollectiveParams(CollectiveParams* col_params) {
   const string& device_name =
-      col_params->instance.device_names[col_params->default_rank];
+      col_params->group.members[col_params->default_rank].device.name();
   // Each subdiv permutation is a ring formed by rotating each
   // single-task subsequence of devices by an offset.  This makes most
   // sense when each task has the same number of devices but we can't
@@ -173,16 +189,14 @@ Status RingAlg::InitializeCollectiveParams(CollectiveParams* col_params) {
   // Start by counting the devices in each task.
   // Precondition: device_names must be sorted so that all devices in
   // the same task are adjacent.
-  VLOG(2) << "Sorted task names: "
-          << absl::StrJoin(col_params->instance.task_names, ", ");
   std::vector<int> dev_per_task;
-  const string* prior_task_name = &col_params->instance.task_names[0];
+  const string* prior_task_name = &col_params->group.members[0].task;
   int dev_count = 1;
   for (int di = 1; di < col_params->group.group_size; ++di) {
-    if (col_params->instance.task_names[di] != *prior_task_name) {
+    if (col_params->group.members[di].task != *prior_task_name) {
       dev_per_task.push_back(dev_count);
       dev_count = 1;
-      prior_task_name = &col_params->instance.task_names[di];
+      prior_task_name = &col_params->group.members[di].task;
     } else {
       ++dev_count;
     }
@@ -226,7 +240,8 @@ Status RingAlg::InitializeCollectiveParams(CollectiveParams* col_params) {
         int permuted_di = prior_dev_count + offset_di;
         int rank = static_cast<int>(perm.size());
         perm.push_back(permuted_di);
-        if (col_params->instance.device_names[permuted_di] == device_name) {
+        if (col_params->group.members[permuted_di].device.name() ==
+            device_name) {
           DCHECK_EQ(permuted_di, col_params->default_rank);
           col_params->subdiv_rank[sdi] = rank;
         }
@@ -237,25 +252,27 @@ Status RingAlg::InitializeCollectiveParams(CollectiveParams* col_params) {
   }
 
   VLOG(2) << collective_util::SubdivPermDebugString(*col_params);
-  return Status::OK();
+  return OkStatus();
 }
 
-Status RingAlg::InitializeCollectiveContext(CollectiveContext* col_ctx) {
+Status RingAlg::InitializeCollectiveContext(
+    std::shared_ptr<CollectiveContext> col_ctx) {
   DCHECK(col_ctx->dev_mgr);
   col_ctx_ = col_ctx;
-  col_params_ = &col_ctx->col_params;
+  col_params_ = col_ctx->col_params.get();
   return collective_util::InitializeDeviceAndLocality(
       col_ctx->dev_mgr, col_ctx->device_name, &col_ctx->device,
       &col_ctx->device_locality);
 }
 
 string RingAlg::TensorDebugString(const Tensor& tensor) {
-  const DeviceBase::GpuDeviceInfo* gpu_device_info =
-      col_ctx_->op_ctx->device()->tensorflow_gpu_device_info();
-  if (gpu_device_info) {
+  const DeviceBase::AcceleratorDeviceInfo* accelerator_device_info =
+      col_ctx_->op_ctx->device()->tensorflow_accelerator_device_info();
+  if (accelerator_device_info) {
     Tensor cpu_tensor(tensor.dtype(), tensor.shape());
-    Status st = gpu_device_info->default_context->CopyDeviceTensorToCPUSync(
-        &tensor, "" /*tensor_name*/, col_ctx_->device, &cpu_tensor);
+    Status st =
+        accelerator_device_info->default_context->CopyDeviceTensorToCPUSync(
+            &tensor, "" /*tensor_name*/, col_ctx_->device, &cpu_tensor);
     DCHECK(st.ok());
     return cpu_tensor.SummarizeValue(64);
   } else {
@@ -276,12 +293,17 @@ void RingAlg::StartAbort(const Status& s) {
       status_.Update(s);
     }
   }
-  // If this is the initial entry to abort mode then invoke StartAbort
-  // on the CollectiveExecutor that invoked us.  That should start
-  // cancellation on all of the outstanding CollectiveRemoteAccess
-  // actions.
+  // If this is the initial entry to abort mode and it's not a cancellation,
+  // then invoke StartAbort on the CollectiveExecutor that invoked us.  That
+  // should start cancellation on all of the outstanding CollectiveRemoteAccess
+  // actions. If it's cancellation all pending send/recv should be cancelled as
+  // well and there's then no need to abort.
   if (abort_started) {
-    col_ctx_->col_exec->StartAbort(s);
+    if (col_ctx_->op_ctx->cancellation_manager() == nullptr ||
+        (!col_ctx_->op_ctx->cancellation_manager()->IsCancelled() &&
+         !col_ctx_->op_ctx->cancellation_manager()->IsCancelling())) {
+      col_ctx_->col_exec->StartAbort(s);
+    }
   }
 }
 
@@ -324,8 +346,8 @@ void RingAlg::InitRingField(RingField* rf, int chunk_idx, int subdiv_idx,
                          .subdiv_permutations[subdiv_idx][recv_from_rank];
   int send_dev_idx = col_params_->instance.impl_details
                          .subdiv_permutations[subdiv_idx][send_to_rank];
-  rf->recv_is_remote = !col_params_->task.is_local[rf->recv_dev_idx];
-  rf->send_is_remote = !col_params_->task.is_local[send_dev_idx];
+  rf->recv_is_remote = !col_params_->group.members[rf->recv_dev_idx].is_local;
+  rf->send_is_remote = !col_params_->group.members[send_dev_idx].is_local;
   if (ca_->ChunkBytes(rf->sc_idx) > 0) {
     // In pass 0 we skip Recv when rank = chunk_idx
     rf->do_recv = (rf->chunk_idx != rf->rank);
@@ -382,12 +404,13 @@ void RingAlg::DispatchSend(RingField* rf, const StatusCallback& done) {
   int send_to_rank = (rf->rank + 1) % group_size_;
   int send_to_dev_idx = col_params_->instance.impl_details
                             .subdiv_permutations[rf->subdiv_idx][send_to_rank];
-  col_ctx_->col_exec->PostToPeer(
-      col_params_->instance.device_names[send_to_dev_idx],
-      col_params_->instance.task_names[send_to_dev_idx], send_buf_key,
+  col_ctx_->col_exec->remote_access()->PostToPeer(
+      col_params_->group.members[send_to_dev_idx].device.name(),
+      col_params_->group.members[send_to_dev_idx].task, send_buf_key,
       col_ctx_->device, col_ctx_->op_ctx->op_device_context(),
       col_ctx_->op_ctx->output_alloc_attr(0), &rf->chunk,
-      col_ctx_->device_locality, done);
+      col_ctx_->device_locality, col_ctx_->op_ctx->cancellation_manager(),
+      done);
 }
 
 void RingAlg::DispatchRecv(RingField* rf, const StatusCallback& done) {
@@ -401,13 +424,14 @@ void RingAlg::DispatchRecv(RingField* rf, const StatusCallback& done) {
   Tensor* dst_tensor = (!rf->second_pass && (col_params_->merge_op != nullptr))
                            ? &rf->tmp_chunk
                            : &rf->chunk;
-  col_ctx_->col_exec->RecvFromPeer(
-      col_params_->instance.device_names[rf->recv_dev_idx],
-      col_params_->instance.task_names[rf->recv_dev_idx],
-      col_params_->task.is_local[rf->recv_dev_idx], recv_buf_key,
+  col_ctx_->col_exec->remote_access()->RecvFromPeer(
+      col_params_->group.members[rf->recv_dev_idx].device.name(),
+      col_params_->group.members[rf->recv_dev_idx].task,
+      col_params_->group.members[rf->recv_dev_idx].is_local, recv_buf_key,
       col_ctx_->device, col_ctx_->op_ctx->op_device_context(),
       col_ctx_->op_ctx->output_alloc_attr(0), dst_tensor,
-      col_ctx_->device_locality, rf->subdiv_idx, done);
+      col_ctx_->device_locality, rf->subdiv_idx,
+      col_ctx_->op_ctx->cancellation_manager(), done);
 }
 
 string RingAlg::FieldState() {
