@@ -23,6 +23,7 @@ limitations under the License.
 #include "tensorflow/core/lib/strings/str_util.h"
 #include "tensorflow/core/lib/strings/stringprintf.h"
 #include "tensorflow/core/platform/env.h"
+#include "tensorflow/core/platform/test.h"
 #include "tensorflow/core/platform/tracing.h"
 
 namespace tensorflow {
@@ -44,7 +45,7 @@ class GrpcDebugTest : public ::testing::Test {
   void TearDown() override { TearDownInProcessServer(&server_data_); }
 
   void SetUpInProcessServer(ServerData* server_data,
-                            int64 server_start_delay_micros) {
+                            int64_t server_start_delay_micros) {
     server_data->port = testing::PickUnusedPortOrDie();
     server_data->url = strings::StrCat("grpc://localhost:", server_data->port);
     server_data->server.reset(new test::TestEventListenerImpl());
@@ -65,11 +66,11 @@ class GrpcDebugTest : public ::testing::Test {
 
   void ClearEnabledWatchKeys() { DebugGrpcIO::ClearEnabledWatchKeys(); }
 
-  const int64 GetChannelConnectionTimeoutMicros() {
+  const int64_t GetChannelConnectionTimeoutMicros() {
     return DebugGrpcIO::channel_connection_timeout_micros_;
   }
 
-  void SetChannelConnectionTimeoutMicros(const int64 timeout) {
+  void SetChannelConnectionTimeoutMicros(const int64_t timeout) {
     DebugGrpcIO::channel_connection_timeout_micros_ = timeout;
   }
 
@@ -78,8 +79,8 @@ class GrpcDebugTest : public ::testing::Test {
 
 TEST_F(GrpcDebugTest, ConnectionTimeoutWorks) {
   // Use a short timeout so the test won't take too long.
-  const int64 kOriginalTimeoutMicros = GetChannelConnectionTimeoutMicros();
-  const int64 kShortTimeoutMicros = 500 * 1000;
+  const int64_t kOriginalTimeoutMicros = GetChannelConnectionTimeoutMicros();
+  const int64_t kShortTimeoutMicros = 500 * 1000;
   SetChannelConnectionTimeoutMicros(kShortTimeoutMicros);
   ASSERT_EQ(kShortTimeoutMicros, GetChannelConnectionTimeoutMicros());
 
@@ -98,8 +99,7 @@ TEST_F(GrpcDebugTest, ConnectionTimeoutWorks) {
   const string expected_error_msg = strings::StrCat(
       "Failed to connect to gRPC channel at ", kInvalidGrpcUrl.substr(7),
       " within a timeout of ", kShortTimeoutMicros / 1e6, " s");
-  ASSERT_NE(string::npos,
-            publish_status.error_message().find(expected_error_msg));
+  ASSERT_NE(string::npos, publish_status.message().find(expected_error_msg));
 }
 
 TEST_F(GrpcDebugTest, ConnectionToDelayedStartingServerWorks) {
@@ -153,9 +153,9 @@ TEST_F(GrpcDebugTest, SendDebugTensorWithLargeStringAtIndex0ViaGrpcTest) {
   const Status status = DebugIO::PublishDebugTensor(
       kDebugNodeKey, tensor, Env::Default()->NowMicros(), {server_data_.url});
   ASSERT_FALSE(status.ok());
-  ASSERT_NE(status.error_message().find("string value at index 0 from debug "
-                                        "node foo_tensor:0:DebugIdentity does "
-                                        "not fit gRPC message size limit"),
+  ASSERT_NE(status.message().find("string value at index 0 from debug "
+                                  "node foo_tensor:0:DebugIdentity does "
+                                  "not fit gRPC message size limit"),
             string::npos);
   TF_ASSERT_OK(DebugIO::CloseDebugURL(server_data_.url));
 }
@@ -169,15 +169,15 @@ TEST_F(GrpcDebugTest, SendDebugTensorWithLargeStringAtIndex1ViaGrpcTest) {
   const Status status = DebugIO::PublishDebugTensor(
       kDebugNodeKey, tensor, Env::Default()->NowMicros(), {server_data_.url});
   ASSERT_FALSE(status.ok());
-  ASSERT_NE(status.error_message().find("string value at index 1 from debug "
-                                        "node foo_tensor:0:DebugIdentity does "
-                                        "not fit gRPC message size limit"),
+  ASSERT_NE(status.message().find("string value at index 1 from debug "
+                                  "node foo_tensor:0:DebugIdentity does "
+                                  "not fit gRPC message size limit"),
             string::npos);
   TF_ASSERT_OK(DebugIO::CloseDebugURL(server_data_.url));
 }
 
 TEST_F(GrpcDebugTest, SendMultipleDebugTensorsSynchronizedViaGrpcTest) {
-  const int32 kSends = 4;
+  const int32_t kSends = 4;
 
   // Prepare the tensors to sent.
   std::vector<Tensor> tensors;
