@@ -29,179 +29,16 @@ limitations under the License.
 #include "tensorflow/core/lib/core/status.h"
 #include "tensorflow/core/lib/core/threadpool.h"
 #include "tensorflow/core/lib/strings/strcat.h"
+#include "tensorflow/core/platform/errors.h"
 #include "tensorflow/core/platform/mutex.h"
 #include "tensorflow/core/platform/notification.h"
 #include "tensorflow/core/util/env_var.h"
+#include "tensorflow/tsl/distributed_runtime/rpc/grpc_state.h"
 
 namespace tensorflow {
-
-// Object allocated per active RPC.
-// Manage the state of a single asynchronous RPC request.  If `max_retries`
-// is greater than 0, the request will be retried for any transient failures.
-template <class Response>
-class RPCState : public GrpcClientCQTag {
- public:
-  RPCState(::grpc::GenericStub* stub, ::grpc::CompletionQueue* cq,
-           const ::grpc::string& method, const protobuf::Message& request,
-           Response* response, StatusCallback done, CallOptions* call_opts,
-           thread::ThreadPool* threadpool, int32 max_retries = 0,
-           bool fail_fast = true, const string* target = nullptr)
-      : RPCState(
-            stub, cq, method, request, response, std::move(done), call_opts,
-            threadpool,
-            // 1) If GRPC_FAIL_FAST is specified, fail_fast=$GRPC_FAIL_FAST.
-            // See b/141948186.
-            // 2) Otherwise, if the platform is Google, use the fail_fast from
-            // the caller. See b/140260119.
-            // 3) Otherwise, use fail_fast=false.
-            [fail_fast]() -> bool {
-              bool x;
-#if defined(PLATFORM_GOOGLE)
-              TF_CHECK_OK(ReadBoolFromEnvVar("GRPC_FAIL_FAST", fail_fast, &x));
-#else
-              TF_CHECK_OK(ReadBoolFromEnvVar("GRPC_FAIL_FAST", false, &x));
-#endif  // PLATFORM_GOOGLE
-              return x;
-            }(),
-            /*timeout_in_ms=*/0, max_retries, target) {
-  }
-
-  template <typename Request>
-  RPCState(::grpc::GenericStub* stub, ::grpc::CompletionQueue* cq,
-           const ::grpc::string& method, const Request& request,
-           Response* response, StatusCallback done, CallOptions* call_opts,
-           thread::ThreadPool* threadpool, bool fail_fast, int64 timeout_in_ms,
-           int32 max_retries, const string* target)
-      : call_opts_(call_opts),
-        threadpool_(threadpool),
-        done_(std::move(done)),
-        timeout_in_ms_(timeout_in_ms),
-        max_retries_(max_retries),
-        cq_(cq),
-        stub_(stub),
-        method_(method),
-        fail_fast_(fail_fast),
-        target_(target) {
-    response_ = response;
-    ::grpc::Status s = GrpcMaybeUnparseProto(request, &request_buf_);
-    if (!s.ok()) {
-      LOG(ERROR) << "GrpcMaybeUnparseProto returned with non-ok status: "
-                 << s.error_message();
-      // Skip retry logic if we fail to parse our request.
-      done_(FromGrpcStatus(s));
-      delete this;
-      return;
-    }
-    StartCall();
-  }
-
-  void StartCall() {
-    context_.reset(new ::grpc::ClientContext());
-    context_->set_wait_for_ready(!fail_fast_);
-    if (timeout_in_ms_ > 0) {
-      context_->set_deadline(
-          gpr_time_from_millis(timeout_in_ms_, GPR_TIMESPAN));
-    }
-    if (call_opts_) {
-      call_opts_->SetCancelCallback([this]() { context_->TryCancel(); });
-    }
-
-    VLOG(2) << "Starting call: " << method_;
-
-    call_ = stub_->PrepareUnaryCall(context_.get(), method_, request_buf_, cq_);
-    call_->StartCall();
-    call_->Finish(&response_buf_, &status_, this);
-  }
-
-  void OnCompleted(bool ok) override {
-    if (call_opts_) {
-      call_opts_->ClearCancelCallback();
-    }
-
-    VLOG(2) << "Completed call: " << method_;
-
-    Status s = FromGrpcStatus(status_);
-    if (s.ok() && !ok) {
-      // Since this function is only being used for processing the response
-      // to Finish for client-side unary calls, ok should never be false
-      s.Update(
-          errors::Internal("GRPC status is okay but CompletionQueueStatus is "
-                           "not.  This should never happen."));
-    }
-
-    if (s.ok()) {
-      if (threadpool_) {
-        // Run parse and callback in another thread, returning this
-        // one to service more RPCs.
-        threadpool_->Schedule([this]() { ParseAndCallDone(); });
-      } else {
-        ParseAndCallDone();
-      }
-      return;
-    }
-
-    VLOG(1) << method_ << " returned with non-ok status: " << s
-            << " Retries: " << num_retries_ << " Max: " << max_retries_ << "\n"
-            << context_->debug_error_string();
-    // Retry if we have any attempts left
-    if (++num_retries_ <= max_retries_ &&
-        (errors::IsUnavailable(s) || errors::IsUnknown(s))) {
-      response_buf_.Clear();
-      VLOG(1) << "Retrying call for " << method_ << "Retry: " << num_retries_
-              << " of " << max_retries_;
-      // TODO(b/139945426) Allow user to configure the retry backoff time.
-      StartCall();
-    } else {
-      // Attach additional GRPC error information if any to the final status
-      string error_msg = s.error_message();
-      strings::StrAppend(&error_msg, "\nAdditional GRPC error information");
-      if (target_) {
-        strings::StrAppend(&error_msg, " from remote target ", *target_);
-      }
-      strings::StrAppend(&error_msg, ":\n:", context_->debug_error_string());
-      s = Status(s.code(), error_msg);
-      // Always treat gRPC cancellation as a derived error. This ensures that
-      // other error types are preferred during status aggregation. (gRPC
-      // cancellation messages do not contain the original status message).
-      if (s.code() == tensorflow::error::Code::CANCELLED) {
-        s = StatusGroup::MakeDerived(s);
-      }
-
-      done_(s);
-      delete this;
-    }
-  }
-
-  void ParseAndCallDone() {
-    Status s;
-    if (!GrpcMaybeParseProto(&response_buf_, response_)) {
-      s.Update(errors::Internal("could not parse rpc response"));
-    }
-    done_(s);
-    delete this;
-  }
-
- private:
-  CallOptions* call_opts_;
-  std::unique_ptr<::grpc::ClientContext> context_;
-  thread::ThreadPool* threadpool_;
-  std::unique_ptr<::grpc::GenericClientAsyncResponseReader> call_;
-  Response* response_;
-  ::grpc::ByteBuffer request_buf_;
-  ::grpc::ByteBuffer response_buf_;
-  ::grpc::Status status_;
-  StatusCallback done_;
-  int64 timeout_in_ms_;
-
-  size_t num_retries_ = 0;
-  size_t max_retries_;
-
-  ::grpc::CompletionQueue* cq_;
-  ::grpc::GenericStub* stub_;
-  ::grpc::string method_;
-  bool fail_fast_;
-  const string* target_;
-};
+// NOLINTBEGIN(misc-unused-using-decls)
+using tsl::RPCState;
+// NOLINTEND(misc-unused-using-decls)
 
 // Represents state associated with one streaming RPC call.
 // Similarly to above, we extract the methods of StreamingRPCState that don't
@@ -285,7 +122,7 @@ class Exchange {
   }
 
   // If `status` is success, completes this exchange by parsing the
-  // response_buf_ and invoking cb_ with Status::OK(). Else, invokes the
+  // response_buf_ and invoking cb_ with OkStatus(). Else, invokes the
   // callback with `status`.
   void Complete(Status status);
 
@@ -386,8 +223,9 @@ class StreamingRPCState : public UntypedStreamingRPCState {
  public:
   // Default behavior is to set fail_fast = False and handle timeouts
   // manually.
-  StreamingRPCState(std::unique_ptr<grpc::GenericClientAsyncReaderWriter> call,
-                    const std::shared_ptr<::grpc::ClientContext>& context)
+  StreamingRPCState(
+      std::unique_ptr<::grpc::GenericClientAsyncReaderWriter> call,
+      const std::shared_ptr<::grpc::ClientContext>& context)
       : context_(context), call_(std::move(call)), call_state_(State::kActive) {
     Ref();
     VLOG(3) << "Created new StreamingRPCState " << this;
@@ -410,7 +248,7 @@ class StreamingRPCState : public UntypedStreamingRPCState {
   bool SendNextRequest(const protobuf::Message& request, Response* response,
                        const StatusCallback& done) {
     ::grpc::ByteBuffer request_buf;
-    ::grpc::Status s = GrpcMaybeUnparseProto(request, &request_buf);
+    ::grpc::Status s = tsl::GrpcMaybeUnparseProto(request, &request_buf);
     if (!s.ok()) {
       Status status = FromGrpcStatus(s);
       LOG(ERROR) << "GrpcMaybeUnparseProto returned with non-ok status: "
@@ -494,7 +332,7 @@ class StreamingRPCState : public UntypedStreamingRPCState {
     e = &exchanges_.GetFront();
     mu_.unlock();
 
-    e->Complete(Status::OK());
+    e->Complete(OkStatus());
 
     {
       mutex_lock l(mu_);
@@ -593,7 +431,7 @@ class StreamingRPCState : public UntypedStreamingRPCState {
   // Order of context_ and call_ is important because context_ must outlive
   // call_.
   const std::shared_ptr<const ::grpc::ClientContext> context_;
-  std::unique_ptr<grpc::GenericClientAsyncReaderWriter> call_;
+  std::unique_ptr<::grpc::GenericClientAsyncReaderWriter> call_;
 
   mutable mutex mu_;
   ExchangeQueue exchanges_ TF_GUARDED_BY(mu_);
@@ -679,7 +517,7 @@ class StreamingRPCDispatcher {
     // the channel to become ready.
     context_->set_wait_for_ready(true);
 
-    std::unique_ptr<grpc::GenericClientAsyncReaderWriter> call =
+    std::unique_ptr<::grpc::GenericClientAsyncReaderWriter> call =
         stub_->PrepareCall(context_.get(), method_, cq_);
 
     state_.reset(new StreamingRPCState<Response>(std::move(call), context_));
