@@ -23,8 +23,10 @@ limitations under the License.
 #include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/kernels/bincount_op.h"
 #include "tensorflow/core/kernels/fill_functor.h"
+#include "tensorflow/core/kernels/sparse_utils.h"
 #include "tensorflow/core/lib/core/threadpool.h"
 #include "tensorflow/core/platform/types.h"
+#include "tensorflow/core/util/determinism.h"
 
 namespace tensorflow {
 
@@ -55,7 +57,7 @@ struct BincountFunctor<CPUDevice, Tidx, T, true> {
     // ParallelForWithWorkerId range from 0 to NumThreads() inclusive.
     ThreadPool* thread_pool =
         context->device()->tensorflow_cpu_worker_threads()->workers;
-    const int64 num_threads = thread_pool->NumThreads() + 1;
+    const int64_t num_threads = thread_pool->NumThreads() + 1;
     Tensor partial_bins_t;
     TF_RETURN_IF_ERROR(context->allocate_temp(
         DT_BOOL, TensorShape({num_threads, num_bins}), &partial_bins_t));
@@ -63,8 +65,8 @@ struct BincountFunctor<CPUDevice, Tidx, T, true> {
     partial_bins.setZero();
     thread_pool->ParallelForWithWorkerId(
         arr.size(), 8 /* cost */,
-        [&](int64 start_ind, int64 limit_ind, int64 worker_id) {
-          for (int64 i = start_ind; i < limit_ind; i++) {
+        [&](int64_t start_ind, int64_t limit_ind, int64_t worker_id) {
+          for (int64_t i = start_ind; i < limit_ind; i++) {
             Tidx value = arr(i);
             if (value < num_bins) {
               partial_bins(worker_id, value) = true;
@@ -76,7 +78,7 @@ struct BincountFunctor<CPUDevice, Tidx, T, true> {
     Eigen::array<int, 1> reduce_dim({0});
     output.device(context->eigen_cpu_device()) =
         partial_bins.any(reduce_dim).cast<T>();
-    return Status::OK();
+    return OkStatus();
   }
 };
 
@@ -100,33 +102,66 @@ struct BincountFunctor<CPUDevice, Tidx, T, false> {
     // ParallelForWithWorkerId range from 0 to NumThreads() inclusive.
     ThreadPool* thread_pool =
         context->device()->tensorflow_cpu_worker_threads()->workers;
-    const int64 num_threads = thread_pool->NumThreads() + 1;
-    Tensor partial_bins_t;
-    TF_RETURN_IF_ERROR(context->allocate_temp(
-        DataTypeToEnum<T>::value, TensorShape({num_threads, num_bins}),
-        &partial_bins_t));
-    auto partial_bins = partial_bins_t.matrix<T>();
-    partial_bins.setZero();
-    thread_pool->ParallelForWithWorkerId(
-        arr.size(), 8 /* cost */,
-        [&](int64 start_ind, int64 limit_ind, int64 worker_id) {
-          for (int64 i = start_ind; i < limit_ind; i++) {
-            Tidx value = arr(i);
-            if (value < num_bins) {
-              if (weights.size()) {
-                partial_bins(worker_id, value) += weights(i);
-              } else {
-                // Complex numbers don't support "++".
-                partial_bins(worker_id, value) += T(1);
+    const int64_t num_threads = thread_pool->NumThreads() + 1;
+    const Tidx* arr_data = arr.data();
+    const std::ptrdiff_t arr_size = arr.size();
+    const T* weight_data = weights.data();
+    if (weights.size() && weights.size() != arr_size) {
+      return errors::InvalidArgument(
+          "Input indices and weights must have the same size.");
+    }
+    if (num_threads == 1) {
+      output.setZero();
+      T* output_data = output.data();
+      if (weights.size()) {
+        for (int64_t i = 0; i < arr_size; i++) {
+          const Tidx value = arr_data[i];
+          if (value < num_bins) {
+            output_data[value] += weight_data[i];
+          }
+        }
+      } else {
+        for (int64_t i = 0; i < arr_size; i++) {
+          const Tidx value = arr_data[i];
+          if (value < num_bins) {
+            // Complex numbers don't support "++".
+            output_data[value] += T(1);
+          }
+        }
+      }
+    } else {
+      Tensor partial_bins_t;
+      TF_RETURN_IF_ERROR(context->allocate_temp(
+          DataTypeToEnum<T>::value, TensorShape({num_threads, num_bins}),
+          &partial_bins_t));
+      auto partial_bins = partial_bins_t.matrix<T>();
+      partial_bins.setZero();
+      thread_pool->ParallelForWithWorkerId(
+          arr_size, 8 /* cost */,
+          [&](int64_t start_ind, int64_t limit_ind, int64_t worker_id) {
+            if (weights.size()) {
+              for (int64_t i = start_ind; i < limit_ind; i++) {
+                Tidx value = arr_data[i];
+                if (value < num_bins) {
+                  partial_bins(worker_id, value) += weight_data[i];
+                }
+              }
+            } else {
+              for (int64_t i = start_ind; i < limit_ind; i++) {
+                Tidx value = arr_data[i];
+                if (value < num_bins) {
+                  // Complex numbers don't support "++".
+                  partial_bins(worker_id, value) += T(1);
+                }
               }
             }
-          }
-        });
+          });
 
-    // Sum the partial bins along the 0th axis.
-    Eigen::array<int, 1> reduce_dim({0});
-    output.device(context->eigen_cpu_device()) = partial_bins.sum(reduce_dim);
-    return Status::OK();
+      // Sum the partial bins along the 0th axis.
+      Eigen::array<int, 1> reduce_dim({0});
+      output.device(context->eigen_cpu_device()) = partial_bins.sum(reduce_dim);
+    }
+    return OkStatus();
   }
 };
 
@@ -143,9 +178,9 @@ struct BincountReduceFunctor<CPUDevice, Tidx, T, binary_output> {
         context->device()->tensorflow_cpu_worker_threads()->workers;
     thread_pool->ParallelForWithWorkerId(
         num_rows, 8 /* cost */,
-        [&](int64 start_row, int64 end_row, int64 worker_id) {
-          for (int64 i = start_row; i < end_row; ++i) {
-            for (int64 j = 0; j < num_cols; ++j) {
+        [&](int64_t start_row, int64_t end_row, int64_t worker_id) {
+          for (int64_t i = start_row; i < end_row; ++i) {
+            for (int64_t j = 0; j < num_cols; ++j) {
               Tidx value = in(i, j);
               if (value < num_bins) {
                 if (binary_output) {
@@ -161,7 +196,7 @@ struct BincountReduceFunctor<CPUDevice, Tidx, T, binary_output> {
             }
           }
         });
-    return Status::OK();
+    return OkStatus();
   }
 };
 
@@ -175,21 +210,23 @@ class BincountOp : public OpKernel {
   void Compute(OpKernelContext* ctx) override {
     const Tensor& arr_t = ctx->input(0);
     const Tensor& size_tensor = ctx->input(1);
-    const Tensor& weights_t = ctx->input(2);
-
-    int32 size = size_tensor.scalar<int32>()();
+    OP_REQUIRES(ctx, size_tensor.dims() == 0,
+                errors::InvalidArgument("Shape must be rank 0 but is rank ",
+                                        size_tensor.dims()));
+    int32_t size = size_tensor.scalar<int32_t>()();
     OP_REQUIRES(
         ctx, size >= 0,
         errors::InvalidArgument("size (", size, ") must be non-negative"));
 
-    const auto arr = arr_t.flat<int32>();
+    const Tensor& weights_t = ctx->input(2);
+    const auto arr = arr_t.flat<int32_t>();
     const auto weights = weights_t.flat<T>();
     Tensor* output_t;
     OP_REQUIRES_OK(ctx,
                    ctx->allocate_output(0, TensorShape({size}), &output_t));
     auto output = output_t->flat<T>();
     OP_REQUIRES_OK(ctx,
-                   functor::BincountFunctor<Device, int32, T, false>::Compute(
+                   functor::BincountFunctor<Device, int32_t, T, false>::Compute(
                        ctx, arr, weights, output, size));
   }
 };
@@ -222,13 +259,35 @@ class DenseBincountOp : public OpKernel {
  public:
   explicit DenseBincountOp(OpKernelConstruction* ctx) : OpKernel(ctx) {
     OP_REQUIRES_OK(ctx, ctx->GetAttr("binary_output", &binary_output_));
+    if (std::is_same<Device, GPUDevice>::value) {
+      OP_REQUIRES(
+          ctx, !OpDeterminismRequired(),
+          errors::Unimplemented(
+              "Determinism is not yet supported in GPU implementation of "
+              "DenseBincount."));
+    }
   }
 
   void Compute(OpKernelContext* ctx) override {
     const Tensor& data = ctx->input(0);
+    OP_REQUIRES(ctx, data.dims() <= 2,
+                errors::InvalidArgument(
+                    "Shape must be at most rank 2 but is rank ", data.dims()));
+
     const Tensor& size_t = ctx->input(1);
     const Tensor& weights = ctx->input(2);
 
+    OP_REQUIRES(ctx, size_t.dims() == 0,
+                errors::InvalidArgument("Shape must be rank 0 but is rank ",
+                                        size_t.dims()));
+    OP_REQUIRES(ctx,
+                weights.shape() == data.shape() || weights.NumElements() == 0,
+                errors::InvalidArgument(
+                    "`weights` must be the same shape as `arr` or a length-0 "
+                    "`Tensor`, in which case it acts as all weights equal to "
+                    "1. Received ",
+                    weights.shape().DebugString()));
+
     Tidx size = size_t.scalar<Tidx>()();
     OP_REQUIRES(
         ctx, size >= 0,
@@ -250,10 +309,10 @@ class DenseBincountOp : public OpKernel {
                      ctx, data.flat<Tidx>(), weights.flat<T>(), out, size));
       }
     } else if (data.dims() == 2) {
-      const int64 num_rows = data.dim_size(0);
+      const int64_t num_rows = data.dim_size(0);
       auto weight_matrix =
           (weights.NumElements() == 0)
-              ? weights.shaped<T, 2>(gtl::InlinedVector<int64, 2>(2, 0))
+              ? weights.shaped<T, 2>(gtl::InlinedVector<int64_t, 2>(2, 0))
               : weights.matrix<T>();
       OP_REQUIRES_OK(
           ctx, ctx->allocate_output(0, TensorShape({num_rows, size}), &out_t));
@@ -284,7 +343,7 @@ class DenseBincountOp : public OpKernel {
                           DenseBincountOp<CPUDevice, Tidx, T>);
 #define REGISTER_CPU_KERNELS(T) \
   REGISTER_KERNELS(int32, T);   \
-  REGISTER_KERNELS(int64, T);
+  REGISTER_KERNELS(int64_t, T);
 
 TF_CALL_NUMBER_TYPES(REGISTER_CPU_KERNELS);
 #undef REGISTER_CPU_KERNELS
@@ -301,7 +360,7 @@ TF_CALL_NUMBER_TYPES(REGISTER_CPU_KERNELS);
                           DenseBincountOp<GPUDevice, Tidx, T>);
 #define REGISTER_GPU_KERNELS(T) \
   REGISTER_KERNELS(int32, T);   \
-  REGISTER_KERNELS(int64, T);
+  REGISTER_KERNELS(int64_t, T);
 
 TF_CALL_int32(REGISTER_GPU_KERNELS);
 TF_CALL_float(REGISTER_GPU_KERNELS);
@@ -319,16 +378,23 @@ class SparseBincountOp : public OpKernel {
 
   void Compute(OpKernelContext* ctx) override {
     const Tensor& indices = ctx->input(0);
-    const auto values = ctx->input(1).flat<Tidx>();
+    const Tensor& values = ctx->input(1);
+    const auto values_flat = values.flat<Tidx>();
     const Tensor& dense_shape = ctx->input(2);
     const Tensor& size_t = ctx->input(3);
     const auto weights = ctx->input(4).flat<T>();
-    const int64 weights_size = weights.size();
+    const int64_t weights_size = weights.size();
 
+    OP_REQUIRES(ctx, size_t.dims() == 0,
+                errors::InvalidArgument("Shape must be rank 0 but is rank ",
+                                        size_t.dims()));
     Tidx size = size_t.scalar<Tidx>()();
     OP_REQUIRES(
         ctx, size >= 0,
         errors::InvalidArgument("size (", size, ") must be non-negative"));
+    OP_REQUIRES_OK(ctx, sparse_utils::ValidateSparseTensor<int64_t>(
+                            indices, values, dense_shape,
+                            sparse_utils::IndexValidation::kUnordered));
 
     bool is_1d = dense_shape.NumElements() == 1;
 
@@ -341,23 +407,28 @@ class SparseBincountOp : public OpKernel {
       if (binary_output_) {
         OP_REQUIRES_OK(ctx,
                        functor::BincountFunctor<Device, Tidx, T, true>::Compute(
-                           ctx, values, weights, out, size));
+                           ctx, values_flat, weights, out, size));
       } else {
         OP_REQUIRES_OK(
             ctx, functor::BincountFunctor<Device, Tidx, T, false>::Compute(
-                     ctx, values, weights, out, size));
+                     ctx, values_flat, weights, out, size));
       }
     } else {
-      const auto shape = dense_shape.flat<int64>();
-      const int64 num_rows = shape(0);
+      const auto shape = dense_shape.flat<int64_t>();
+      const int64_t num_rows = shape(0);
       OP_REQUIRES_OK(
           ctx, ctx->allocate_output(0, TensorShape({num_rows, size}), &out_t));
       const auto out = out_t->matrix<T>();
       fill(ctx->eigen_device<Device>(), out_t->flat<T>());
-      const auto indices_mat = indices.matrix<int64>();
-      for (int64 i = 0; i < indices_mat.dimension(0); ++i) {
-        const int64 batch = indices_mat(i, 0);
-        const Tidx bin = values(i);
+      const auto indices_mat = indices.matrix<int64_t>();
+      for (int64_t i = 0; i < indices_mat.dimension(0); ++i) {
+        const int64_t batch = indices_mat(i, 0);
+        const Tidx bin = values_flat(i);
+        OP_REQUIRES(
+            ctx, batch < out.dimension(0),
+            errors::InvalidArgument("Index out of bound. `batch` (", batch,
+                                    ") must be less than the dimension size (",
+                                    out.dimension(0), ")."));
         if (bin < size) {
           if (binary_output_) {
             out(batch, bin) = T(1);
@@ -385,7 +456,7 @@ class SparseBincountOp : public OpKernel {
                           SparseBincountOp<CPUDevice, Tidx, T>);
 #define REGISTER_CPU_KERNELS(T) \
   REGISTER_KERNELS(int32, T);   \
-  REGISTER_KERNELS(int64, T);
+  REGISTER_KERNELS(int64_t, T);
 
 TF_CALL_NUMBER_TYPES(REGISTER_CPU_KERNELS);
 #undef REGISTER_CPU_KERNELS
@@ -399,12 +470,15 @@ class RaggedBincountOp : public OpKernel {
   }
 
   void Compute(OpKernelContext* ctx) override {
-    const auto splits = ctx->input(0).flat<int64>();
+    const auto splits = ctx->input(0).flat<int64_t>();
     const auto values = ctx->input(1).flat<Tidx>();
     const Tensor& size_t = ctx->input(2);
     const auto weights = ctx->input(3).flat<T>();
-    const int64 weights_size = weights.size();
+    const int64_t weights_size = weights.size();
 
+    OP_REQUIRES(ctx, size_t.dims() == 0,
+                errors::InvalidArgument("Shape must be rank 0 but is rank ",
+                                        size_t.dims()));
     Tidx size = size_t.scalar<Tidx>()();
     OP_REQUIRES(
         ctx, size >= 0,
@@ -414,6 +488,18 @@ class RaggedBincountOp : public OpKernel {
     int num_values = values.size();
     int batch_idx = 0;
 
+    OP_REQUIRES(ctx, splits.size() > 0,
+                errors::InvalidArgument("Splits must be non-empty"));
+
+    OP_REQUIRES(ctx, splits(0) == 0,
+                errors::InvalidArgument("Splits must start with 0, not with ",
+                                        splits(0)));
+
+    OP_REQUIRES(ctx, splits(num_rows) == num_values,
+                errors::InvalidArgument(
+                    "Splits must end with the number of values, got ",
+                    splits(num_rows), " instead of ", num_values));
+
     Tensor* out_t;
     OP_REQUIRES_OK(
         ctx, ctx->allocate_output(0, TensorShape({num_rows, size}), &out_t));
@@ -451,7 +537,7 @@ class RaggedBincountOp : public OpKernel {
                           RaggedBincountOp<CPUDevice, Tidx, T>);
 #define REGISTER_CPU_KERNELS(T) \
   REGISTER_KERNELS(int32, T);   \
-  REGISTER_KERNELS(int64, T);
+  REGISTER_KERNELS(int64_t, T);
 
 TF_CALL_NUMBER_TYPES(REGISTER_CPU_KERNELS);
 #undef REGISTER_CPU_KERNELS
