@@ -19,7 +19,11 @@ limitations under the License.
 
 namespace tensorflow {
 namespace functor {
+#if !defined(MLIR_GENERATED_GPU_KERNELS_ENABLED)
 DEFINE_BINARY6(maximum, Eigen::half, float, double, uint8, int16, int64);
+#endif
+
+DEFINE_BINARY1(maximum, bfloat16);
 }  // namespace functor
 }  // namespace tensorflow
 
