@@ -42,31 +42,32 @@ int RowsAndColsArg(int r, int c) { return r * kRows + c; }
 int RowsFromArg(int arg) { return (arg / kRows); }
 int ColsFromArg(int arg) { return (arg % kRows); }
 
-#define BM_UNARY(DEVICE, FUNC, T, TYPE)                              \
-  void BM_##DEVICE##_##FUNC##_##TYPE(int iters, int num) {           \
-    const int64 tot = static_cast<int64>(iters) * num;               \
-    testing::UseRealTime();                                          \
-    testing::ItemsProcessed(tot);                                    \
-    testing::BytesProcessed(tot * sizeof(T));                        \
-    test::Benchmark(#DEVICE, Unary<T>(#FUNC, num, TYPE)).Run(iters); \
-  }                                                                  \
-  BENCHMARK(BM_##DEVICE##_##FUNC##_##TYPE)->Range(4 << 10, 1 << 20);
+#define BM_UNARY(DEVICE, FUNC, T, TYPE)                                    \
+  void BM_##DEVICE##_##FUNC##_##TYPE(::testing::benchmark::State& state) { \
+    const int num = state.range(0);                                        \
+    test::Benchmark(#DEVICE, Unary<T>(#FUNC, num, TYPE),                   \
+                    /*old_benchmark_api*/ false)                           \
+        .Run(state);                                                       \
+    const int64_t tot = static_cast<int64_t>(state.iterations()) * num;    \
+    state.SetItemsProcessed(tot);                                          \
+    state.SetBytesProcessed(tot * sizeof(T));                              \
+  }                                                                        \
+  BENCHMARK(BM_##DEVICE##_##FUNC##_##TYPE)                                 \
+      ->UseRealTime()                                                      \
+      ->Range(4 << 10, 1 << 20);
+
+BM_UNARY(cpu, LeakyRelu, float, DT_FLOAT);
+BM_UNARY(cpu, LeakyRelu, bfloat16, DT_BFLOAT16);
 
 BM_UNARY(cpu, Floor, float, DT_FLOAT);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_UNARY(gpu, Floor, float, DT_FLOAT);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_UNARY(sycl, Floor, float, DT_FLOAT);
-#endif  // TENSORFLOW_USE_SYCL
 
 BM_UNARY(cpu, Floor, double, DT_DOUBLE);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_UNARY(gpu, Floor, double, DT_DOUBLE);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_UNARY(sycl, Floor, double, DT_DOUBLE);
-#endif  // TENSORFLOW_USE_SYCL
 
 BM_UNARY(cpu, Conj, std::complex<float>, DT_COMPLEX64);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
@@ -107,52 +108,46 @@ Graph* BinaryScalar(int num, const string& func) {
   return g;
 }
 
-#define BM_BINARY_SCALAR(DEVICE, FUNC)                             \
-  void BM_##DEVICE##_##FUNC##_scalar(int iters, int num) {         \
-    const int64 tot = static_cast<int64>(iters) * num;             \
-    testing::UseRealTime();                                        \
-    testing::ItemsProcessed(tot);                                  \
-    testing::BytesProcessed(tot * sizeof(float));                  \
-    test::Benchmark(#DEVICE, BinaryScalar(num, #FUNC)).Run(iters); \
-  }                                                                \
-  BENCHMARK(BM_##DEVICE##_##FUNC##_scalar)                         \
-      ->Arg(1 << 12) /* must >= 4096 */                            \
-      ->Arg(1 << 13)                                               \
-      ->Arg(1 << 14)                                               \
-      ->Arg((1 << 15) - (1 << 13))                                 \
-      ->Arg(1 << 15)                                               \
-      ->Arg((1 << 15) + (1 << 14))                                 \
-      ->Arg(1 << 16)                                               \
-      ->Arg((1 << 17) - (1 << 15))                                 \
-      ->Arg(1 << 17)                                               \
-      ->Arg((1 << 17) + (1 << 16))                                 \
-      ->Arg(1 << 18)                                               \
-      ->Arg(1 << 19)                                               \
+#define BM_BINARY_SCALAR(DEVICE, FUNC)                                     \
+  void BM_##DEVICE##_##FUNC##_scalar(::testing::benchmark::State& state) { \
+    const int num = state.range(0);                                        \
+                                                                           \
+    test::Benchmark(#DEVICE, BinaryScalar(num, #FUNC),                     \
+                    /*old_benchmark_api=*/false)                           \
+        .Run(state);                                                       \
+    const int64_t tot = static_cast<int64_t>(state.iterations()) * num;    \
+    state.SetItemsProcessed(tot);                                          \
+    state.SetBytesProcessed(tot * sizeof(float));                          \
+  }                                                                        \
+  BENCHMARK(BM_##DEVICE##_##FUNC##_scalar)                                 \
+      ->Arg(1 << 12) /* must >= 4096 */                                    \
+      ->Arg(1 << 13)                                                       \
+      ->Arg(1 << 14)                                                       \
+      ->Arg((1 << 15) - (1 << 13))                                         \
+      ->Arg(1 << 15)                                                       \
+      ->Arg((1 << 15) + (1 << 14))                                         \
+      ->Arg(1 << 16)                                                       \
+      ->Arg((1 << 17) - (1 << 15))                                         \
+      ->Arg(1 << 17)                                                       \
+      ->Arg((1 << 17) + (1 << 16))                                         \
+      ->Arg(1 << 18)                                                       \
+      ->Arg(1 << 19)                                                       \
       ->Arg(1 << 20);
 
 BM_BINARY_SCALAR(cpu, Less);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BINARY_SCALAR(gpu, Less);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BINARY_SCALAR(sycl, Less);
-#endif  // TENSORFLOW_USE_SYCL
 
 BM_BINARY_SCALAR(cpu, Add);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BINARY_SCALAR(gpu, Add);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BINARY_SCALAR(sycl, Add);
-#endif  // TENSORFLOW_USE_SYCL
 
 BM_BINARY_SCALAR(cpu, DivNoNan);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BINARY_SCALAR(gpu, DivNoNan);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BINARY_SCALAR(sycl, DivNoNan);
-#endif  // TENSORFLOW_USE_SYCL
 
 #undef BM_BINARY_SCALAR
 
@@ -188,17 +183,20 @@ Graph* CubeWithMulSquare(int num) {
   return g;
 }
 
-#define BM_CUBE(DEVICE, Impl)                          \
-  void BM_##DEVICE##_Cube_##Impl(int iters, int num) { \
-    const int64 tot = static_cast<int64>(iters) * num; \
-    testing::UseRealTime();                            \
-    testing::ItemsProcessed(tot);                      \
-    testing::BytesProcessed(tot * sizeof(float));      \
-    test::Benchmark(#DEVICE, Impl(num)).Run(iters);    \
-  }                                                    \
-  BENCHMARK(BM_##DEVICE##_Cube_##Impl)                 \
-      ->Arg(1 << 12) /* must >= 4096 */                \
-      ->Arg(1 << 16)                                   \
+#define BM_CUBE(DEVICE, Impl)                                           \
+  void BM_##DEVICE##_Cube_##Impl(::testing::benchmark::State& state) {  \
+    const int num = state.range(0);                                     \
+                                                                        \
+    test::Benchmark(#DEVICE, Impl(num), /*old_benchmark_api*/ false)    \
+        .Run(state);                                                    \
+    const int64_t tot = static_cast<int64_t>(state.iterations()) * num; \
+    state.SetItemsProcessed(tot);                                       \
+    state.SetBytesProcessed(tot * sizeof(float));                       \
+  }                                                                     \
+  BENCHMARK(BM_##DEVICE##_Cube_##Impl)                                  \
+      ->UseRealTime()                                                   \
+      ->Arg(1 << 12) /* must >= 4096 */                                 \
+      ->Arg(1 << 16)                                                    \
       ->Arg(1 << 20);
 
 BM_CUBE(cpu, CubeWithPow3);
@@ -209,11 +207,6 @@ BM_CUBE(gpu, CubeWithPow3);
 BM_CUBE(gpu, CubeWithTwoMuls);
 BM_CUBE(gpu, CubeWithMulSquare);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_CUBE(sycl, CubeWithPow3);
-BM_CUBE(sycl, CubeWithTwoMuls);
-BM_CUBE(sycl, CubeWithMulSquare);
-#endif  // TENSORFLOW_USE_SYCL
 
 #undef BM_CUBE
 
@@ -231,17 +224,22 @@ Graph* BiasAdd(int rows, int cols, DataType type) {
   return g;
 }
 
-#define BM_BIAS_ADD(DEVICE, C_TYPE, TF_TYPE, R, C)                             \
-  void BM_##DEVICE##_##C_TYPE##_BiasAdd_R##R##_C##C(int iters, int arg) {      \
-    const int rows = RowsFromArg(arg);                                         \
-    const int cols = ColsFromArg(arg);                                         \
-    const int64 tot = static_cast<int64>(iters) * rows * cols;                 \
-    testing::UseRealTime();                                                    \
-    testing::ItemsProcessed(tot);                                              \
-    testing::BytesProcessed(tot * sizeof(C_TYPE));                             \
-    test::Benchmark(#DEVICE, BiasAdd<C_TYPE>(rows, cols, TF_TYPE)).Run(iters); \
-  }                                                                            \
-  BENCHMARK(BM_##DEVICE##_##C_TYPE##_BiasAdd_R##R##_C##C)                      \
+#define BM_BIAS_ADD(DEVICE, C_TYPE, TF_TYPE, R, C)                 \
+  void BM_##DEVICE##_##C_TYPE##_BiasAdd_R##R##_C##C(               \
+      ::testing::benchmark::State& state) {                        \
+    const int arg = state.range(0);                                \
+    const int rows = RowsFromArg(arg);                             \
+    const int cols = ColsFromArg(arg);                             \
+    const int64_t tot =                                            \
+        static_cast<int64_t>(state.iterations()) * rows * cols;    \
+    test::Benchmark(#DEVICE, BiasAdd<C_TYPE>(rows, cols, TF_TYPE), \
+                    /*old_benchmark_api=*/false)                   \
+        .Run(state);                                               \
+    state.SetItemsProcessed(tot);                                  \
+    state.SetBytesProcessed(tot * sizeof(C_TYPE));                 \
+  }                                                                \
+  BENCHMARK(BM_##DEVICE##_##C_TYPE##_BiasAdd_R##R##_C##C)          \
+      ->UseRealTime()                                              \
       ->Arg(RowsAndColsArg(R, C));
 
 #define BM_BIAS_ADD_ALL(DEVICE, C_TYPE, TF_TYPE)   \
@@ -284,16 +282,21 @@ Graph* BiasAddGrad(int rows, int cols, int channels, DataType type,
 
 #define BM_BIAS_ADD_GRAD(DEVICE, FMT, C_TYPE, TF_TYPE, R, C, CH)               \
   void BM_##DEVICE##_##FMT##_##C_TYPE##_BiasAddGrad_R##R##_C##C##_CH##CH(      \
-      int iters, int arg, int channels) {                                      \
+      ::testing::benchmark::State& state) {                                    \
+    const int arg = state.range(0);                                            \
+    const int channels = state.range(1);                                       \
+                                                                               \
     const int rows = RowsFromArg(arg);                                         \
     const int cols = ColsFromArg(arg);                                         \
-    const int64 tot = static_cast<int64>(iters) * rows * cols * channels;      \
-    testing::UseRealTime();                                                    \
-    testing::ItemsProcessed(tot);                                              \
-    testing::BytesProcessed(tot * sizeof(C_TYPE));                             \
-    test::Benchmark(#DEVICE, BiasAddGrad<C_TYPE>(rows, cols, channels,         \
-                                                 TF_TYPE, FORMAT_##FMT))       \
-        .Run(iters);                                                           \
+    test::Benchmark(                                                           \
+        #DEVICE,                                                               \
+        BiasAddGrad<C_TYPE>(rows, cols, channels, TF_TYPE, FORMAT_##FMT),      \
+        /*old_benchmark_api=*/false)                                           \
+        .Run(state);                                                           \
+    const int64_t tot =                                                        \
+        static_cast<int64_t>(state.iterations()) * rows * cols * channels;     \
+    state.SetItemsProcessed(tot);                                              \
+    state.SetBytesProcessed(tot * sizeof(C_TYPE));                             \
   }                                                                            \
   BENCHMARK(BM_##DEVICE##_##FMT##_##C_TYPE##_BiasAddGrad_R##R##_C##C##_CH##CH) \
       ->ArgPair(RowsAndColsArg(R, C), CH);
@@ -346,16 +349,21 @@ Graph* BcastAdd(int rows, int cols, int dim) {
   return g;
 }
 
-#define BM_BCAST_ADD_ROW(DEVICE, R, C)                             \
-  void BM_##DEVICE##_BcastAddRow_R##R##_C##C(int iters, int arg) { \
-    const int rows = RowsFromArg(arg);                             \
-    const int cols = ColsFromArg(arg);                             \
-    const int64 tot = static_cast<int64>(iters) * rows * cols;     \
-    testing::UseRealTime();                                        \
-    testing::ItemsProcessed(tot);                                  \
-    testing::BytesProcessed(tot * sizeof(float));                  \
-    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 0)).Run(iters);  \
-  }                                                                \
+#define BM_BCAST_ADD_ROW(DEVICE, R, C)                          \
+  void BM_##DEVICE##_BcastAddRow_R##R##_C##C(                   \
+      ::testing::benchmark::State& state) {                     \
+    const int arg = state.range(0);                             \
+                                                                \
+    const int rows = RowsFromArg(arg);                          \
+    const int cols = ColsFromArg(arg);                          \
+    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 0),           \
+                    /*old_benchmark_api=*/false)                \
+        .Run(state);                                            \
+    const int64_t tot =                                         \
+        static_cast<int64_t>(state.iterations()) * rows * cols; \
+    state.SetItemsProcessed(tot);                               \
+    state.SetBytesProcessed(tot * sizeof(float));               \
+  }                                                             \
   BENCHMARK(BM_##DEVICE##_BcastAddRow_R##R##_C##C)->Arg(RowsAndColsArg(R, C));
 
 #define BM_BCAST_ADD_ROW_ALL(DEVICE)   \
@@ -367,23 +375,28 @@ BM_BCAST_ADD_ROW_ALL(cpu);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BCAST_ADD_ROW_ALL(gpu);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BCAST_ADD_ROW_ALL(sycl);
-#endif  // TENSORFLOW_USE_SYCL
 #undef BM_BCAST_ADD_ROW_ALL
 #undef BM_BCAST_ADD_ROW
 
-#define BM_BCAST_ADD_COL(DEVICE, R, C)                             \
-  void BM_##DEVICE##_BcastAddCol_R##R##_C##C(int iters, int arg) { \
-    const int rows = RowsFromArg(arg);                             \
-    const int cols = ColsFromArg(arg);                             \
-    const int64 tot = static_cast<int64>(iters) * rows * cols;     \
-    testing::UseRealTime();                                        \
-    testing::ItemsProcessed(tot);                                  \
-    testing::BytesProcessed(tot * sizeof(float));                  \
-    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 1)).Run(iters);  \
-  }                                                                \
-  BENCHMARK(BM_##DEVICE##_BcastAddCol_R##R##_C##C)->Arg(RowsAndColsArg(R, C));
+#define BM_BCAST_ADD_COL(DEVICE, R, C)                          \
+  void BM_##DEVICE##_BcastAddCol_R##R##_C##C(                   \
+      ::testing::benchmark::State& state) {                     \
+    const int arg = state.range(0);                             \
+                                                                \
+    const int rows = RowsFromArg(arg);                          \
+    const int cols = ColsFromArg(arg);                          \
+    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 1),           \
+                    /*old_benchmark_api=*/false)                \
+        .Run(state);                                            \
+    const int64_t tot =                                         \
+        static_cast<int64_t>(state.iterations()) * rows * cols; \
+                                                                \
+    state.SetItemsProcessed(tot);                               \
+    state.SetBytesProcessed(tot * sizeof(float));               \
+  }                                                             \
+  BENCHMARK(BM_##DEVICE##_BcastAddCol_R##R##_C##C)              \
+      ->UseRealTime()                                           \
+      ->Arg(RowsAndColsArg(R, C));
 
 #define BM_BCAST_ADD_COL_ALL(DEVICE)   \
   BM_BCAST_ADD_COL(DEVICE, 512, 2048); \
@@ -394,23 +407,27 @@ BM_BCAST_ADD_COL_ALL(cpu);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BCAST_ADD_COL_ALL(gpu);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BCAST_ADD_COL_ALL(sycl);
-#endif  // TENSORFLOW_USE_SYCL
 #undef BM_BCAST_ADD_COL_ALL
 #undef BM_BCAST_ADD_COL
 
-#define BM_BCAST_ADD_CROSS_RC(DEVICE, R, C)                            \
-  void BM_##DEVICE##_BcastAddCrossRC_R##R##_C##C(int iters, int arg) { \
-    const int rows = RowsFromArg(arg);                                 \
-    const int cols = ColsFromArg(arg);                                 \
-    const int64 tot = static_cast<int64>(iters) * rows * cols;         \
-    testing::UseRealTime();                                            \
-    testing::ItemsProcessed(tot);                                      \
-    testing::BytesProcessed(tot * sizeof(float));                      \
-    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 2)).Run(iters);      \
-  }                                                                    \
-  BENCHMARK(BM_##DEVICE##_BcastAddCrossRC_R##R##_C##C)                 \
+#define BM_BCAST_ADD_CROSS_RC(DEVICE, R, C)                     \
+  void BM_##DEVICE##_BcastAddCrossRC_R##R##_C##C(               \
+      ::testing::benchmark::State& state) {                     \
+    const int arg = state.range(0);                             \
+                                                                \
+    const int rows = RowsFromArg(arg);                          \
+    const int cols = ColsFromArg(arg);                          \
+    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 2),           \
+                    /*old_benchmark_api=*/false)                \
+        .Run(state);                                            \
+    const int64_t tot =                                         \
+        static_cast<int64_t>(state.iterations()) * rows * cols; \
+                                                                \
+    state.SetItemsProcessed(tot);                               \
+    state.SetBytesProcessed(tot * sizeof(float));               \
+  }                                                             \
+  BENCHMARK(BM_##DEVICE##_BcastAddCrossRC_R##R##_C##C)          \
+      ->UseRealTime()                                           \
       ->Arg(RowsAndColsArg(R, C));
 
 #define BM_BCAST_ADD_CROSS_RC_ALL(DEVICE)   \
@@ -422,23 +439,26 @@ BM_BCAST_ADD_CROSS_RC_ALL(cpu);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BCAST_ADD_CROSS_RC_ALL(gpu);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BCAST_ADD_CROSS_RC_ALL(sycl);
-#endif  // TENSORFLOW_USE_SYCL
 #undef BM_BCAST_ADD_CROSS_RC_ALL
 #undef BM_BCAST_ADD_CROSS_RC
 
-#define BM_BCAST_ADD_CROSS_CR(DEVICE, R, C)                            \
-  void BM_##DEVICE##_BcastAddCrossCR_R##R##_C##C(int iters, int arg) { \
-    const int rows = RowsFromArg(arg);                                 \
-    const int cols = ColsFromArg(arg);                                 \
-    const int64 tot = static_cast<int64>(iters) * rows * cols;         \
-    testing::UseRealTime();                                            \
-    testing::ItemsProcessed(tot);                                      \
-    testing::BytesProcessed(tot * sizeof(float));                      \
-    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 3)).Run(iters);      \
-  }                                                                    \
-  BENCHMARK(BM_##DEVICE##_BcastAddCrossCR_R##R##_C##C)                 \
+#define BM_BCAST_ADD_CROSS_CR(DEVICE, R, C)                     \
+  void BM_##DEVICE##_BcastAddCrossCR_R##R##_C##C(               \
+      ::testing::benchmark::State& state) {                     \
+    const int arg = state.range(0);                             \
+                                                                \
+    const int rows = RowsFromArg(arg);                          \
+    const int cols = ColsFromArg(arg);                          \
+    test::Benchmark(#DEVICE, BcastAdd(rows, cols, 3),           \
+                    /*old_benchmark_api*/ false)                \
+        .Run(state);                                            \
+    const int64_t tot =                                         \
+        static_cast<int64_t>(state.iterations()) * rows * cols; \
+    state.SetItemsProcessed(tot);                               \
+    state.SetBytesProcessed(tot * sizeof(float));               \
+  }                                                             \
+  BENCHMARK(BM_##DEVICE##_BcastAddCrossCR_R##R##_C##C)          \
+      ->UseRealTime()                                           \
       ->Arg(RowsAndColsArg(R, C));
 
 #define BM_BCAST_ADD_CROSS_CR_ALL(DEVICE)   \
@@ -450,9 +470,6 @@ BM_BCAST_ADD_CROSS_CR_ALL(cpu);
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 BM_BCAST_ADD_CROSS_CR_ALL(gpu);
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#ifdef TENSORFLOW_USE_SYCL
-BM_BCAST_ADD_CROSS_CR_ALL(sycl);
-#endif  // TENSORFLOW_USE_SYCL
 #undef BM_BCAST_ADD_CROSS_CR_ALL
 #undef BM_BCAST_ADD_CROSS_CR
 
