@@ -25,6 +25,7 @@ limitations under the License.
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/tensor_types.h"
+#include "tensorflow/core/framework/type_traits.h"
 #include "tensorflow/core/framework/variant_op_registry.h"
 #include "tensorflow/core/kernels/cwise_ops_common.h"
 #include "tensorflow/core/kernels/dense_update_functor.h"
@@ -37,8 +38,8 @@ limitations under the License.
 #include "tensorflow/core/platform/threadpool.h"
 
 #if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
-#include "tensorflow/core/kernels/cuda_solvers.h"
-#include "tensorflow/core/kernels/cuda_sparse.h"
+#include "tensorflow/core/util/cuda_sparse.h"
+#include "tensorflow/core/util/gpu_solvers.h"
 #endif
 
 namespace tensorflow {
@@ -53,9 +54,9 @@ namespace tensorflow {
 //
 // Maximum number of shards into which to divide the computation for each CSR
 // Sparse Matrix instance.
-static constexpr int32 kMaxShards = 20;
+static constexpr int32_t kMaxShards = 20;
 // Number of shards allocated to each thread.
-static constexpr int32 kNumShardsPerThread = 3;
+static constexpr int32_t kNumShardsPerThread = 3;
 
 typedef Eigen::ThreadPoolDevice CPUDevice;
 typedef Eigen::GpuDevice GPUDevice;
@@ -93,17 +94,22 @@ class CSRMatMulOp : public OpKernel {
                     "Only one of adjoint_b and transpose_b may be true."));
     OP_REQUIRES_OK(c, c->GetAttr("transpose_output", &transpose_output_));
     OP_REQUIRES_OK(c, c->GetAttr("conjugate_output", &conjugate_output_));
-    conjugate_a_ = adjoint_a;
-    conjugate_b_ = adjoint_b;
     transpose_a_ |= adjoint_a;
     transpose_b_ |= adjoint_b;
+    if (is_complex<T>::value) {
+      conjugate_a_ = adjoint_a;
+      conjugate_b_ = adjoint_b;
+    } else {
+      conjugate_a_ = false;
+      conjugate_b_ = false;
+    }
   }
 
   ~CSRMatMulOp() override {}
 
   Status ValidateInputs(const CSRSparseMatrix& sparse_matrix_a,
                         const Tensor& dense_tensor_b, int* rank,
-                        int64* batch_size) {
+                        int64_t* batch_size) {
     if (sparse_matrix_a.dtype() != dense_tensor_b.dtype()) {
       return errors::InvalidArgument(
           "Input types don't match.  a.dtype == ",
@@ -123,10 +129,10 @@ class CSRMatMulOp : public OpKernel {
                                      sparse_matrix_a.batch_size(), " vs. ",
                                      batch_size, ".");
     }
-    const auto& a_dense_shape = sparse_matrix_a.dense_shape().vec<int64>();
-    const int64 a_inner_dim =
+    const auto& a_dense_shape = sparse_matrix_a.dense_shape().vec<int64_t>();
+    const int64_t a_inner_dim =
         a_dense_shape(this->transpose_a_ ? *rank - 2 : *rank - 1);
-    const int64 b_inner_dim =
+    const int64_t b_inner_dim =
         dense_tensor_b.dim_size(this->transpose_b_ ? *rank - 1 : *rank - 2);
     if (a_inner_dim != b_inner_dim) {
       return errors::InvalidArgument(
@@ -134,7 +140,7 @@ class CSRMatMulOp : public OpKernel {
           TensorShape(a_dense_shape), " vs. ",
           dense_tensor_b.shape().DebugString());
     }
-    return Status::OK();
+    return OkStatus();
   }
 
  public:
@@ -172,15 +178,15 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
     const Tensor& matrix_b = ctx->input(1);
 
     int rank;
-    int64 batch_size;
+    int64_t batch_size;
     OP_REQUIRES_OK(ctx, this->ValidateInputs(*sparse_matrix_a, matrix_b, &rank,
                                              &batch_size));
 
-    const auto dense_shape = sparse_matrix_a->dense_shape().vec<int64>();
-    int64 num_lhs_rows = dense_shape(rank - 2);
-    int64 num_lhs_cols = dense_shape(rank - 1);
-    int64 num_rhs_rows = matrix_b.dim_size(rank - 2);
-    int64 num_rhs_cols = matrix_b.dim_size(rank - 1);
+    const auto dense_shape = sparse_matrix_a->dense_shape().vec<int64_t>();
+    int64_t num_lhs_rows = dense_shape(rank - 2);
+    int64_t num_lhs_cols = dense_shape(rank - 1);
+    int64_t num_rhs_rows = matrix_b.dim_size(rank - 2);
+    int64_t num_rhs_cols = matrix_b.dim_size(rank - 1);
 
     if (this->transpose_a_) {
       std::swap(num_lhs_rows, num_lhs_cols);
@@ -233,13 +239,15 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
   // transpose_output is True, allocates a temporary buffer with the transposed
   // output. 'matmul_result' points to either output or output_transposed, based
   // on whether transpose_output is True.
-  Status AllocateOutput(OpKernelContext* ctx, const int32 rank,
-                        const int64 batch_size, const int64 num_rows,
-                        const int64 num_cols, const bool transpose_output,
+  Status AllocateOutput(OpKernelContext* ctx, const int32_t rank,
+                        const int64_t batch_size, const int64_t num_rows,
+                        const int64_t num_cols, const bool transpose_output,
                         Tensor** output, Tensor* output_transposed,
                         Tensor** matmul_result) {
     TensorShape output_shape;
-    if (rank == 3) output_shape.AddDim(batch_size);
+    if (rank == 3) {
+      TF_RETURN_IF_ERROR(output_shape.AddDimWithStatus(batch_size));
+    }
 
     if (!transpose_output) {
       output_shape.AppendShape({num_rows, num_cols});
@@ -255,26 +263,26 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
       TF_RETURN_IF_ERROR(ctx->allocate_output(0, output_shape, output));
       *matmul_result = output_transposed;
     }
-    return Status::OK();
+    return OkStatus();
   }
 
   // Returns an Eigen::Ref expression of a sparse sub-matrix from the given
   // contiguous segment of rows of the CSR Sparse Matrix.
   Eigen::Ref<const SparseMatrix> GetSparseMatrixRef(
       const CSRSparseMatrix& csr_matrix, const int batch_index,
-      const int64 row_begin, const int64 num_shard_rows,
+      const int64_t row_begin, const int64_t num_shard_rows,
       std::vector<int32>* row_ptrs) {
     // Compute the row pointers of the sparse sub-matrix.
     row_ptrs->resize(num_shard_rows + 1);
-    const int64 row_offset =
+    const int64_t row_offset =
         csr_matrix.row_pointers_vec(batch_index)(row_begin);
-    for (int64 row_idx = 0; row_idx <= num_shard_rows; ++row_idx) {
+    for (int64_t row_idx = 0; row_idx <= num_shard_rows; ++row_idx) {
       row_ptrs->at(row_idx) =
           csr_matrix.row_pointers_vec(batch_index)(row_begin + row_idx) -
           row_offset;
     }
-    const int64 num_cols =
-        csr_matrix.dense_shape().vec<int64>()(csr_matrix.dims() - 1);
+    const int64_t num_cols =
+        csr_matrix.dense_shape().vec<int64_t>()(csr_matrix.dims() - 1);
     return Eigen::Map<const SparseMatrix>(
         num_shard_rows /* num_rows */, num_cols /* num_cols */,
         row_ptrs->at(num_shard_rows) /* total_nnz */, row_ptrs->data(),
@@ -284,28 +292,31 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
 
   // Sparse-Dense Matrix Multiplication between a CSRSparseMatrix (LHS) and a
   // dense Tensor (RHS).
-  void SparseDenseMatMulWithoutTransposedLHS(
-      OpKernelContext* ctx, const int64 batch_size, const int64 num_lhs_rows,
-      const CSRSparseMatrix& lhs, const Tensor& rhs, Tensor* output) {
+  void SparseDenseMatMulWithoutTransposedLHS(OpKernelContext* ctx,
+                                             const int64_t batch_size,
+                                             const int64_t num_lhs_rows,
+                                             const CSRSparseMatrix& lhs,
+                                             const Tensor& rhs,
+                                             Tensor* output) {
     // Parallelize matrix multiplication across batch dimensions and across
     // rows in each batch.
     auto worker_threads = *(ctx->device()->tensorflow_cpu_worker_threads());
-    const int32 num_threads = worker_threads.num_threads;
-    const int64 block_size =
+    const int32_t num_threads = worker_threads.num_threads;
+    const int64_t block_size =
         num_lhs_rows / std::max(kMaxShards, kNumShardsPerThread * num_threads);
-    const int64 num_rhs_rows = rhs.dim_size(rhs.dims() - 2);
-    const int64 num_rhs_cols = rhs.dim_size(rhs.dims() - 1);
+    const int64_t num_rhs_rows = rhs.dim_size(rhs.dims() - 2);
+    const int64_t num_rhs_cols = rhs.dim_size(rhs.dims() - 1);
     worker_threads.workers->ParallelFor(
         batch_size * num_lhs_rows /* total */,
         thread::ThreadPool::SchedulingParams(
             thread::ThreadPool::SchedulingStrategy::
                 kFixedBlockSize /* strategy */,
             absl::nullopt /* cost_per_unit */, block_size),
-        [&](int64 batch_and_row_begin, int64 batch_and_row_end) {
+        [&](int64_t batch_and_row_begin, int64_t batch_and_row_end) {
           HandleBatchAndRowRange(
               num_lhs_rows, batch_and_row_begin, batch_and_row_end,
-              [&](int64 batch_idx, int64 row_begin, int64 row_end) {
-                const int64 num_shard_rows = row_end - row_begin;
+              [&](int64_t batch_idx, int64_t row_begin, int64_t row_end) {
+                const int64_t num_shard_rows = row_end - row_begin;
 
                 // Define an Eigen::SparseMatrix over the row range:
                 // [row_begin, row_end) of the CSR SparseMatrix A.
@@ -333,16 +344,16 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
   // Sparse-Dense Matrix Multiplication assuming the CSRSparseMatrix (LHS) is
   // to be transposed before the operation.
   void SparseDenseMatMulWithTransposedLHS(OpKernelContext* ctx,
-                                          const int64 batch_size,
-                                          const int64 num_lhs_rows,
-                                          const int64 num_lhs_cols,
+                                          const int64_t batch_size,
+                                          const int64_t num_lhs_rows,
+                                          const int64_t num_lhs_cols,
                                           const CSRSparseMatrix& lhs,
                                           const Tensor& rhs, Tensor* output) {
     auto device = ctx->eigen_device<CPUDevice>();
     auto worker_threads = *(ctx->device()->tensorflow_cpu_worker_threads());
-    const int32 num_threads = worker_threads.num_threads;
-    const int64 num_rhs_rows = rhs.dim_size(rhs.dims() - 2);
-    const int64 num_rhs_cols = rhs.dim_size(rhs.dims() - 1);
+    const int32_t num_threads = worker_threads.num_threads;
+    const int64_t num_rhs_rows = rhs.dim_size(rhs.dims() - 2);
+    const int64_t num_rhs_cols = rhs.dim_size(rhs.dims() - 1);
     // Usually, we want to avoid transposing the sparse matrix A since it may be
     // an expensive operation. Instead, we use the identity (A^T B) = (B^T A)^T.
     // We don't actually transpose B or the output because it is more convenient
@@ -364,7 +375,7 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
 
     // Parallelize matrix multiplication across batch dimensions and across
     // columns of A^T in each batch. These correspond to rows of A.
-    const int64 block_size =
+    const int64_t block_size =
         num_lhs_cols / std::max(kMaxShards, kNumShardsPerThread * num_threads);
     worker_threads.workers->ParallelForWithWorkerId(
         batch_size * num_lhs_cols /* total */,
@@ -372,11 +383,11 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
             thread::ThreadPool::SchedulingStrategy::
                 kFixedBlockSize /* strategy */,
             absl::nullopt /* cost_per_unit */, block_size),
-        [&](int64 batch_and_row_begin, int64 batch_and_row_end, int tid) {
+        [&](int64_t batch_and_row_begin, int64_t batch_and_row_end, int tid) {
           HandleBatchAndRowRange(
               num_lhs_cols, batch_and_row_begin, batch_and_row_end,
-              [&](int64 batch_idx, int64 row_begin, int64 row_end) {
-                const int64 num_shard_rows = row_end - row_begin;
+              [&](int64_t batch_idx, int64_t row_begin, int64_t row_end) {
+                const int64_t num_shard_rows = row_end - row_begin;
 
                 // Define a new sparse sub-matrix from the row range
                 // [row_begin, row_end) of the sparse matrix A.
@@ -423,25 +434,25 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
   // fn(batch_idx, row_begin, row_end) for each batch index
   // and the row range [row_begin, row_end) contained in the batch.
   void HandleBatchAndRowRange(
-      const int64 num_rows, const int64 batch_and_row_begin,
-      const int64 batch_and_row_end,
-      const std::function<void(int64, int64, int64)>& fn) {
+      const int64_t num_rows, const int64_t batch_and_row_begin,
+      const int64_t batch_and_row_end,
+      const std::function<void(int64_t, int64_t, int64_t)>& fn) {
     // Obtain the batch indices overlapping with the current shard.
-    const int64 batch_begin = batch_and_row_begin / num_rows;
-    const int64 batch_end_inclusive = batch_and_row_end / num_rows;
+    const int64_t batch_begin = batch_and_row_begin / num_rows;
+    const int64_t batch_end_inclusive = batch_and_row_end / num_rows;
 
-    for (int64 batch_idx = batch_begin; batch_idx <= batch_end_inclusive;
+    for (int64_t batch_idx = batch_begin; batch_idx <= batch_end_inclusive;
          ++batch_idx) {
       // Find the contiguous set of rows which are contained in this shard as
       // well as the current batch. We intersect with interval [batch_idx *
       // num_rows, (batch_idx + 1) * num_rows) which denotes the current batch.
-      const int64 current_batch_row_begin =
+      const int64_t current_batch_row_begin =
           std::max(batch_and_row_begin, batch_idx * num_rows);
-      const int64 current_batch_row_end =
+      const int64_t current_batch_row_end =
           std::min(batch_and_row_end, (batch_idx + 1) * num_rows);
 
-      const int64 row_begin = current_batch_row_begin % num_rows;
-      const int64 num_shard_rows =
+      const int64_t row_begin = current_batch_row_begin % num_rows;
+      const int64_t num_shard_rows =
           current_batch_row_end - current_batch_row_begin;
       // Edge case for when current_batch_row_end is the first index of a new
       // row.
@@ -477,7 +488,7 @@ class CSRMatMulCPUOp : public CSRMatMulOp<CPUDevice, T> {
       TF_RETURN_IF_ERROR(
           DoMatrixTranspose(ctx->eigen_device<CPUDevice>(), input, output));
     }
-    return Status::OK();
+    return OkStatus();
   }
 };
 
@@ -502,38 +513,40 @@ class CSRMatMulGPUOp : public CSRMatMulOp<GPUDevice, T> {
     const Tensor& b_t = ctx->input(1);
 
     int rank;
-    int64 batch_size;
+    int64_t batch_size;
     OP_REQUIRES_OK(ctx,
                    this->ValidateInputs(*a_matrix, b_t, &rank, &batch_size));
 
     const Tensor& a_dense_shape_t = a_matrix->dense_shape();
     TensorShape a_dense_tensor_shape;
-    auto a_dense_shape = a_dense_shape_t.vec<int64>();
+    auto a_dense_shape = a_dense_shape_t.vec<int64_t>();
     OP_REQUIRES_OK(
         ctx, TensorShapeUtils::MakeShape(a_dense_shape, &a_dense_tensor_shape));
 
     const int row_dim = (rank == 2) ? 0 : 1;
-    const int64 a_outer_dim = a_dense_tensor_shape.dim_size(
+    const int64_t a_outer_dim = a_dense_tensor_shape.dim_size(
         this->transpose_a_ ? row_dim + 1 : row_dim);
-    const int64 b_inner_dim =
+    const int64_t b_inner_dim =
         b_t.shape().dim_size(this->transpose_b_ ? row_dim + 1 : row_dim);
-    const int64 b_outer_dim =
+    const int64_t b_outer_dim =
         b_t.dim_size(this->transpose_b_ ? row_dim : row_dim + 1);
-    const int64 b_slice_size = b_inner_dim * b_outer_dim;
+    const int64_t b_slice_size = b_inner_dim * b_outer_dim;
 
     TensorShape c_shape;
-    if (rank == 3) c_shape.AddDim(batch_size);
+    if (rank == 3) {
+      OP_REQUIRES_OK(ctx, c_shape.AddDimWithStatus(batch_size));
+    }
     if (this->transpose_output_) {
-      c_shape.AddDim(b_outer_dim);
-      c_shape.AddDim(a_outer_dim);
+      OP_REQUIRES_OK(ctx, c_shape.AddDimWithStatus(b_outer_dim));
+      OP_REQUIRES_OK(ctx, c_shape.AddDimWithStatus(a_outer_dim));
     } else {
-      c_shape.AddDim(a_outer_dim);
-      c_shape.AddDim(b_outer_dim);
+      OP_REQUIRES_OK(ctx, c_shape.AddDimWithStatus(a_outer_dim));
+      OP_REQUIRES_OK(ctx, c_shape.AddDimWithStatus(b_outer_dim));
     }
 
-    const int64 c_matrix_lhs = c_shape.dim_size(row_dim);
-    const int64 c_matrix_rhs = c_shape.dim_size(row_dim + 1);
-    const int64 c_slice_size = c_matrix_lhs * c_matrix_rhs;
+    const int64_t c_matrix_lhs = c_shape.dim_size(row_dim);
+    const int64_t c_matrix_rhs = c_shape.dim_size(row_dim + 1);
+    const int64_t c_slice_size = c_matrix_lhs * c_matrix_rhs;
     Tensor* c_t;
     OP_REQUIRES_OK(ctx, ctx->allocate_output(0, c_shape, &c_t));
 
@@ -546,8 +559,8 @@ class CSRMatMulGPUOp : public CSRMatMulOp<GPUDevice, T> {
 #endif
     if (use_matrix_vector_multiply) {
       // Call matrix-vector multiply if b is a vector.
-      TTypes<int64>::ConstVec a_dense_shape_comp(a_dense_shape.data() + row_dim,
-                                                 2);
+      TTypes<int64_t>::ConstVec a_dense_shape_comp(
+          a_dense_shape.data() + row_dim, 2);
       Tensor b_conj_t;
       const T* b_base_ptr = b_t.template flat<T>().data();
       bool conjugate_a = this->conjugate_a_;
@@ -629,7 +642,7 @@ class CSRMatMulGPUOp : public CSRMatMulOp<GPUDevice, T> {
       a_input_matrix = &a_matrix_transposed;
     }
 
-    auto a_input_dense_shape = a_input_matrix->dense_shape().vec<int64>();
+    auto a_input_dense_shape = a_input_matrix->dense_shape().vec<int64_t>();
 
     // Possibly transpose b.
     Tensor b_t_input;
@@ -638,10 +651,12 @@ class CSRMatMulGPUOp : public CSRMatMulOp<GPUDevice, T> {
     } else {
       TensorShape b_t_transposed_shape;
       if (rank == 3) {
-        b_t_transposed_shape.AddDim(batch_size);
+        OP_REQUIRES_OK(ctx, b_t_transposed_shape.AddDimWithStatus(batch_size));
       }
-      b_t_transposed_shape.AddDim(b_t.dim_size(row_dim + 1));
-      b_t_transposed_shape.AddDim(b_t.dim_size(row_dim));
+      OP_REQUIRES_OK(ctx, b_t_transposed_shape.AddDimWithStatus(
+                              b_t.dim_size(row_dim + 1)));
+      OP_REQUIRES_OK(
+          ctx, b_t_transposed_shape.AddDimWithStatus(b_t.dim_size(row_dim)));
       OP_REQUIRES_OK(ctx, ctx->allocate_temp(DataTypeToEnum<T>::value,
                                              b_t_transposed_shape, &b_t_input));
       const GPUDevice& d = ctx->eigen_device<GPUDevice>();
@@ -655,7 +670,7 @@ class CSRMatMulGPUOp : public CSRMatMulOp<GPUDevice, T> {
     }
 
     // Dense shape of a batch component of A.
-    TTypes<int64>::ConstVec a_input_dense_shape_comp(
+    TTypes<int64_t>::ConstVec a_input_dense_shape_comp(
         a_input_dense_shape.data() + row_dim, 2);
 
     auto b = b_t_input.flat<T>();
@@ -708,10 +723,8 @@ REGISTER_CPU(complex128)
 
 REGISTER_GPU(float)
 REGISTER_GPU(double)
-#if GOOGLE_CUDA
 REGISTER_GPU(complex64)
 REGISTER_GPU(complex128)
-#endif
 
 #undef REGISTER_GPU
 
@@ -728,14 +741,12 @@ namespace {
 template <typename T>
 struct GPUDataType;
 
-// GPUDataType templates are currently not instantiated in the ROCm flow
-// So leaving out the #elif TENSORFLOW_USE_ROCM blocks for now
-// hipblas library is not (yet) being pulled in via rocm_configure.bzl
-// so cannot reference tyeps from hipblas headers here
 template <>
 struct GPUDataType<Eigen::half> {
 #if GOOGLE_CUDA
   static constexpr cudaDataType_t type = CUDA_R_16F;
+#else
+  static constexpr hipDataType type = HIP_R_16F;
 #endif
 };
 
@@ -743,6 +754,8 @@ template <>
 struct GPUDataType<float> {
 #if GOOGLE_CUDA
   static constexpr cudaDataType_t type = CUDA_R_32F;
+#else
+  static constexpr hipDataType type = HIP_R_32F;
 #endif
 };
 
@@ -750,6 +763,8 @@ template <>
 struct GPUDataType<std::complex<float>> {
 #if GOOGLE_CUDA
   static constexpr cudaDataType_t type = CUDA_C_32F;
+#else
+  static constexpr hipDataType type = HIP_C_32F;
 #endif
 };
 
@@ -757,6 +772,8 @@ template <>
 struct GPUDataType<double> {
 #if GOOGLE_CUDA
   static constexpr cudaDataType_t type = CUDA_R_64F;
+#else
+  static constexpr hipDataType type = HIP_R_64F;
 #endif
 };
 
@@ -764,6 +781,8 @@ template <>
 struct GPUDataType<std::complex<double>> {
 #if GOOGLE_CUDA
   static constexpr cudaDataType_t type = CUDA_C_64F;
+#else
+  static constexpr hipDataType type = HIP_C_64F;
 #endif
 };
 
@@ -835,7 +854,6 @@ class CSRSparseMatrixMatMul<GPUDevice, T> {
       gpusparseSpMatDescr_t matA;
       gpusparseDnMatDescr_t matB, matC;
 
-      // NOTE: the following APIs are not available in ROCM
       TF_RETURN_IF_GPUSPARSE_ERROR(cusparseCreateCsr(
           &matA, m, k, nnz, const_cast<int*>(a.row_ptr.data()),
           const_cast<int*>(a.col_ind.data()), const_cast<T*>(a.values.data()),
@@ -850,24 +868,66 @@ class CSRSparseMatrixMatMul<GPUDevice, T> {
           cusparseCreateDnMat(&matC, m, n, ldc, c.data(), GPUDataType<T>::type,
                               CUSPARSE_ORDER_COL));
 
+#if CUDA_VERSION >= 12000
+      cusparseSpMMAlg_t algo = CUSPARSE_SPMM_ALG_DEFAULT;
+#else
+      cusparseSpMMAlg_t algo = CUSPARSE_MM_ALG_DEFAULT;
+#endif
       size_t bufferSize = 0;
       TF_RETURN_IF_ERROR(cuda_sparse.SpMMBufferSize(
-          transA, transB, &alpha, matA, matB, &beta, matC,
-          CUSPARSE_MM_ALG_DEFAULT, &bufferSize));
+          transA, transB, &alpha, matA, matB, &beta, matC, algo, &bufferSize));
 
       Tensor buffer;
       TF_RETURN_IF_ERROR(ctx->allocate_temp(
-          DT_INT8, TensorShape({static_cast<int64>(bufferSize)}), &buffer));
+          DT_INT8, TensorShape({static_cast<int64_t>(bufferSize)}), &buffer));
       DCHECK(buffer.flat<int8>().data() != nullptr);
 
       TF_RETURN_IF_ERROR(cuda_sparse.SpMM(transA, transB, &alpha, matA, matB,
-                                          &beta, matC, CUSPARSE_MM_ALG_DEFAULT,
+                                          &beta, matC, algo,
                                           buffer.flat<int8>().data()));
 
       TF_RETURN_IF_GPUSPARSE_ERROR(cusparseDestroyDnMat(matB));
       TF_RETURN_IF_GPUSPARSE_ERROR(cusparseDestroyDnMat(matC));
       TF_RETURN_IF_GPUSPARSE_ERROR(cusparseDestroySpMat(matA));
 
+#elif TENSORFLOW_USE_ROCM && TF_ROCM_VERSION >= 40200
+      // Use SPMM
+      const gpusparseOperation_t transB = HIPSPARSE_OPERATION_TRANSPOSE;
+      gpusparseSpMatDescr_t matA;
+      gpusparseDnMatDescr_t matB, matC;
+
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseCreateCsr(
+          &matA, m, k, nnz, const_cast<int*>(a.row_ptr.data()),
+          const_cast<int*>(a.col_ind.data()), const_cast<T*>(a.values.data()),
+          HIPSPARSE_INDEX_32I, HIPSPARSE_INDEX_32I, HIPSPARSE_INDEX_BASE_ZERO,
+          GPUDataType<T>::type));
+
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseCreateDnMat(
+          &matB, n, k, ldb, const_cast<T*>(b.data()), GPUDataType<T>::type,
+          HIPSPARSE_ORDER_COLUMN));
+
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseCreateDnMat(
+          &matC, m, n, ldc, c.data(), GPUDataType<T>::type,
+          HIPSPARSE_ORDER_COLUMN));
+
+      size_t bufferSize = 0;
+      TF_RETURN_IF_ERROR(cuda_sparse.SpMMBufferSize(
+          transA, transB, &alpha, matA, matB, &beta, matC,
+          HIPSPARSE_MM_ALG_DEFAULT, &bufferSize));
+
+      Tensor buffer;
+      TF_RETURN_IF_ERROR(ctx->allocate_temp(
+          DT_INT8, TensorShape({static_cast<int64_t>(bufferSize)}), &buffer));
+      DCHECK(buffer.flat<int8>().data() != nullptr);
+
+      TF_RETURN_IF_ERROR(cuda_sparse.SpMM(transA, transB, &alpha, matA, matB,
+                                          &beta, matC, HIPSPARSE_MM_ALG_DEFAULT,
+                                          buffer.flat<int8>().data()));
+
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseDestroyDnMat(matB));
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseDestroyDnMat(matC));
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseDestroySpMat(matA));
+
 #else
 
 #if GOOGLE_CUDA
@@ -886,11 +946,11 @@ class CSRSparseMatrixMatMul<GPUDevice, T> {
       const gpusparseOperation_t transB = HIPSPARSE_OPERATION_TRANSPOSE;
 
       gpusparseMatDescr_t descrA;
-      TF_RETURN_IF_GPUSPARSE_ERROR(hipsparseCreateMatDescr(&descrA));
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseCreateMatDescr(&descrA));
       TF_RETURN_IF_GPUSPARSE_ERROR(
-          hipsparseSetMatType(descrA, HIPSPARSE_MATRIX_TYPE_GENERAL));
-      TF_RETURN_IF_GPUSPARSE_ERROR(
-          hipsparseSetMatIndexBase(descrA, HIPSPARSE_INDEX_BASE_ZERO));
+          se::wrap::hipsparseSetMatType(descrA, HIPSPARSE_MATRIX_TYPE_GENERAL));
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseSetMatIndexBase(
+          descrA, HIPSPARSE_INDEX_BASE_ZERO));
 #endif  // GOOGLE_CUDA
 
       TF_RETURN_IF_ERROR(
@@ -901,7 +961,7 @@ class CSRSparseMatrixMatMul<GPUDevice, T> {
 #endif  // GOOGLE_CUDA && CUDA_VERSION >= 10020
     }
 
-    return Status::OK();
+    return OkStatus();
   }
 
  private:
@@ -940,11 +1000,11 @@ class CSRSparseMatrixMatVec<GPUDevice, T> {
           cusparseSetMatIndexBase(descrA, CUSPARSE_INDEX_BASE_ZERO));
 #elif TENSORFLOW_USE_ROCM
       gpusparseMatDescr_t descrA;
-      TF_RETURN_IF_GPUSPARSE_ERROR(hipsparseCreateMatDescr(&descrA));
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseCreateMatDescr(&descrA));
       TF_RETURN_IF_GPUSPARSE_ERROR(
-          hipsparseSetMatType(descrA, HIPSPARSE_MATRIX_TYPE_GENERAL));
-      TF_RETURN_IF_GPUSPARSE_ERROR(
-          hipsparseSetMatIndexBase(descrA, HIPSPARSE_INDEX_BASE_ZERO));
+          se::wrap::hipsparseSetMatType(descrA, HIPSPARSE_MATRIX_TYPE_GENERAL));
+      TF_RETURN_IF_GPUSPARSE_ERROR(se::wrap::hipsparseSetMatIndexBase(
+          descrA, HIPSPARSE_INDEX_BASE_ZERO));
 #endif  // GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 
       const int m = a.dense_shape_host(0);
@@ -962,7 +1022,7 @@ class CSRSparseMatrixMatVec<GPUDevice, T> {
 #endif
     }
 
-    return Status::OK();
+    return OkStatus();
   }
 
  private:
