@@ -14,8 +14,10 @@ limitations under the License.
 ==============================================================================*/
 
 #include "tensorflow/core/framework/common_shape_fns.h"
+#include "tensorflow/core/framework/full_type.pb.h"
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/shape_inference.h"
+#include "tensorflow/core/framework/types.pb.h"
 
 namespace tensorflow {
 namespace {
@@ -39,7 +41,7 @@ Status VerifyHandleData(
                                    " but got list with element dtype ",
                                    DataTypeString(list_shape_type.dtype));
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 bool IsValidTensorListHandleData(
@@ -59,6 +61,8 @@ REGISTER_OP("EmptyTensorList")
     .Output("handle: variant")
     .Attr("element_dtype: type")
     .Attr("shape_type: {int32, int64}")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->Scalar());
       DataType element_dtype;
@@ -66,10 +70,11 @@ REGISTER_OP("EmptyTensorList")
       shape_inference::ShapeHandle element_shape;
       TF_RETURN_IF_ERROR(c->MakeShapeFromShapeTensorTreatScalarAsUnknownShape(
           0, &element_shape));
+      const FullTypeDef& ret_types = c->ret_types();
       c->set_output_handle_shapes_and_types(
           0, std::vector<shape_inference::ShapeAndType>{
-                 {element_shape, element_dtype}});
-      return Status::OK();
+                 {element_shape, element_dtype, ret_types.args(0)}});
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListPushBack")
@@ -77,6 +82,8 @@ REGISTER_OP("TensorListPushBack")
     .Input("tensor: element_dtype")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->Scalar());
       DataType element_dtype;
@@ -103,10 +110,11 @@ REGISTER_OP("TensorListPushBack")
             c->Merge(element_shape, list_shape_type.shape, &ignored));
         element_shape = list_shape_type.shape;
       }
+      const FullTypeDef& ret_types = c->ret_types();
       c->set_output_handle_shapes_and_types(
           0, std::vector<shape_inference::ShapeAndType>{
-                 {element_shape, element_dtype}});
-      return Status::OK();
+                 {element_shape, element_dtype, ret_types.args(0)}});
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListPushBackBatch")
@@ -114,6 +122,9 @@ REGISTER_OP("TensorListPushBackBatch")
     .Input("tensor: element_dtype")
     .Output("output_handles: variant")
     .Attr("element_dtype: type")
+    // TODO(mdan): Also support for inferring from an input type as well.
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       shape_inference::ShapeHandle input_handles;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 1, &input_handles));
@@ -150,10 +161,11 @@ REGISTER_OP("TensorListPushBackBatch")
             c->Merge(element_shape, list_shape_type.shape, &ignored));
         element_shape = list_shape_type.shape;
       }
+      const FullTypeDef& ret_types = c->ret_types();
       c->set_output_handle_shapes_and_types(
           0, std::vector<shape_inference::ShapeAndType>{
-                 {element_shape, element_dtype}});
-      return Status::OK();
+                 {element_shape, element_dtype, ret_types.args(0)}});
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListLength")
@@ -179,6 +191,9 @@ REGISTER_OP("TensorListPopBack")
       if (IsValidTensorListHandleData(handle_data)) {
         const shape_inference::ShapeAndType& list_shape_type =
             (*handle_data)[0];
+        if (list_shape_type.type.type_id() != TFT_ARRAY) {
+          return errors::InvalidArgument("Input argument must be a list.");
+        }
         if (list_shape_type.dtype != element_dtype) {
           return errors::InvalidArgument(
               "Trying to read from list with wrong element dtype. List has "
@@ -195,7 +210,7 @@ REGISTER_OP("TensorListPopBack")
       }
       c->set_output(1, tensor_shape);
       c->set_output(0, c->Scalar());
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListStack")
@@ -244,7 +259,7 @@ REGISTER_OP("TensorListStack")
       shape_inference::ShapeHandle result;
       TF_RETURN_IF_ERROR(c->Concatenate(num_elements, element_shape, &result));
       c->set_output(0, result);
-      return Status::OK();
+      return OkStatus();
     });
 
 Status TensorListConcatShapeInference(
@@ -280,7 +295,7 @@ Status TensorListConcatShapeInference(
     c->set_output(0, c->UnknownShape());
   }
   c->set_output(1, c->MakeShape({c->UnknownDim()}));
-  return Status::OK();
+  return OkStatus();
 }
 
 REGISTER_OP("TensorListConcat")
@@ -320,6 +335,8 @@ REGISTER_OP("TensorListSplit")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
     .Attr("shape_type: {int32, int64}")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->Scalar());
       DataType element_dtype;
@@ -342,10 +359,11 @@ REGISTER_OP("TensorListSplit")
       TF_RETURN_IF_ERROR(c->Merge(element_shape_from_tensor_shape,
                                   element_shape,
                                   &element_shape_from_tensor_shape));
+      const FullTypeDef& ret_types = c->ret_types();
       c->set_output_handle_shapes_and_types(
           0, std::vector<shape_inference::ShapeAndType>{
-                 {element_shape, element_dtype}});
-      return Status::OK();
+                 {element_shape, element_dtype, ret_types.args(0)}});
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListFromTensor")
@@ -354,6 +372,9 @@ REGISTER_OP("TensorListFromTensor")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
     .Attr("shape_type: {int32, int64}")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
+    .SetForwardTypeFn(full_type::UnaryContainerCreate(TFT_ARRAY, 0))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->Scalar());
       DataType element_dtype;
@@ -367,10 +388,11 @@ REGISTER_OP("TensorListFromTensor")
           1, &element_shape));
       TF_RETURN_IF_ERROR(c->Merge(tensor_shape_except_first_dim, element_shape,
                                   &tensor_shape_except_first_dim));
+      const FullTypeDef& ret_types = c->ret_types();
       c->set_output_handle_shapes_and_types(
           0, std::vector<shape_inference::ShapeAndType>{
-                 {element_shape, element_dtype}});
-      return Status::OK();
+                 {element_shape, element_dtype, ret_types.args(0)}});
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListElementShape")
@@ -379,12 +401,19 @@ REGISTER_OP("TensorListElementShape")
     .Attr("shape_type: {int32, int64}")
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       auto* handle_data = c->input_handle_shapes_and_types(0);
+      // `TensorListElementShape` returns the scalar -1 if the rank of
+      // element_shape is unknown else returns the shape vector (with possibly
+      // unknown dims).
       if (!IsValidTensorListHandleData(handle_data)) {
-        c->set_output(0, c->Vector(c->UnknownDim()));
-        return Status::OK();
+        c->set_output(0, c->UnknownShape());
+        return OkStatus();
       }
-      c->set_output(0, c->Vector(c->Rank((*handle_data)[0].shape)));
-      return Status::OK();
+      if (c->RankKnown((*handle_data)[0].shape)) {
+        c->set_output(0, c->Vector(c->Rank((*handle_data)[0].shape)));
+      } else {
+        c->set_output(0, c->UnknownShape());
+      }
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListReserve")
@@ -393,6 +422,8 @@ REGISTER_OP("TensorListReserve")
     .Output("handle: variant")
     .Attr("element_dtype: type")
     .Attr("shape_type: {int32, int64}")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       c->set_output(0, c->Scalar());
       shape_inference::ShapeHandle element_shape;
@@ -400,10 +431,11 @@ REGISTER_OP("TensorListReserve")
           0, &element_shape));
       DataType element_dtype;
       TF_RETURN_IF_ERROR(c->GetAttr("element_dtype", &element_dtype));
+      const FullTypeDef& ret_types = c->ret_types();
       c->set_output_handle_shapes_and_types(
           0, std::vector<shape_inference::ShapeAndType>{
-                 {element_shape, element_dtype}});
-      return Status::OK();
+                 {element_shape, element_dtype, ret_types.args(0)}});
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListGetItem")
@@ -434,7 +466,7 @@ REGISTER_OP("TensorListGetItem")
       TF_RETURN_IF_ERROR(
           c->Merge(element_shape, element_shape_input, &element_shape));
       c->set_output(0, element_shape);
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListResize")
@@ -451,7 +483,7 @@ REGISTER_OP("TensorListResize")
       if (IsValidTensorListHandleData(handle_data)) {
         c->set_output_handle_shapes_and_types(0, *handle_data);
       }
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListSetItem")
@@ -460,6 +492,13 @@ REGISTER_OP("TensorListSetItem")
     .Input("item: element_dtype")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
+    .Attr("resize_if_index_out_of_bounds: bool = false")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
+    .SetForwardTypeFn(full_type::UnaryContainerAdd(TFT_ARRAY,
+                                                   /*container_idx=*/0,
+                                                   /*element_idx=*/2,
+                                                   /*homogeneous=*/true))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       DataType element_dtype;
       TF_RETURN_IF_ERROR(c->GetAttr("element_dtype", &element_dtype));
@@ -473,10 +512,12 @@ REGISTER_OP("TensorListSetItem")
             c->Merge(item_shape, list_shape_type.shape, &item_shape));
         c->set_output_handle_shapes_and_types(0, *handle_data);
       } else {
+        const FullTypeDef& ret_types = c->ret_types();
         c->set_output_handle_shapes_and_types(
-            0, {{c->UnknownShape(), element_dtype}});
+            0, std::vector<shape_inference::ShapeAndType>{
+                   {c->UnknownShape(), element_dtype, ret_types.args(0)}});
       }
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListGather")
@@ -509,7 +550,7 @@ REGISTER_OP("TensorListGather")
       shape_inference::ShapeHandle out;
       TF_RETURN_IF_ERROR(c->Concatenate(c->input(1), element_shape, &out));
       c->set_output(0, out);
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListScatter")
@@ -519,16 +560,20 @@ REGISTER_OP("TensorListScatter")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
     .Attr("shape_type: {int32, int64}")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       DataType element_dtype;
       TF_RETURN_IF_ERROR(c->GetAttr("element_dtype", &element_dtype));
       shape_inference::ShapeHandle element_shape;
       TF_RETURN_IF_ERROR(c->MakeShapeFromShapeTensorTreatScalarAsUnknownShape(
           2, &element_shape));
-      c->set_output_handle_shapes_and_types(0,
-                                            {{element_shape, element_dtype}});
+      const FullTypeDef& ret_types = c->ret_types();
+      c->set_output_handle_shapes_and_types(
+          0, std::vector<shape_inference::ShapeAndType>{
+                 {element_shape, element_dtype, ret_types.args(0)}});
       c->set_output(0, c->Scalar());
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListScatterV2")
@@ -539,16 +584,20 @@ REGISTER_OP("TensorListScatterV2")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
     .Attr("shape_type: {int32, int64}")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       DataType element_dtype;
       TF_RETURN_IF_ERROR(c->GetAttr("element_dtype", &element_dtype));
       shape_inference::ShapeHandle element_shape;
       TF_RETURN_IF_ERROR(c->MakeShapeFromShapeTensorTreatScalarAsUnknownShape(
           2, &element_shape));
-      c->set_output_handle_shapes_and_types(0,
-                                            {{element_shape, element_dtype}});
+      const FullTypeDef& ret_types = c->ret_types();
+      c->set_output_handle_shapes_and_types(
+          0, std::vector<shape_inference::ShapeAndType>{
+                 {element_shape, element_dtype, ret_types.args(0)}});
       c->set_output(0, c->Scalar());
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListScatterIntoExistingList")
@@ -557,6 +606,8 @@ REGISTER_OP("TensorListScatterIntoExistingList")
     .Input("indices: int32")
     .Output("output_handle: variant")
     .Attr("element_dtype: type")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       shape_inference::ShapeHandle ignored;
       // Check that tensor is at least a vector.
@@ -573,10 +624,12 @@ REGISTER_OP("TensorListScatterIntoExistingList")
         TF_RETURN_IF_ERROR(VerifyHandleData(c, *handle_data, element_dtype));
         element_shape = GetElementShapeFromHandleData(*handle_data);
       }
-      c->set_output_handle_shapes_and_types(0,
-                                            {{element_shape, element_dtype}});
+      const FullTypeDef& ret_types = c->ret_types();
+      c->set_output_handle_shapes_and_types(
+          0, std::vector<shape_inference::ShapeAndType>{
+                 {element_shape, element_dtype, ret_types.args(0)}});
       c->set_output(0, c->Scalar());
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("TensorListConcatLists")
@@ -584,6 +637,8 @@ REGISTER_OP("TensorListConcatLists")
     .Input("input_b: variant")
     .Attr("element_dtype: type")
     .Output("output: variant")
+    .SetTypeConstructor(full_type::UnaryTensorContainer(TFT_ARRAY,
+                                                        "element_dtype"))
     .SetShapeFn([](shape_inference::InferenceContext* c) {
       auto input_a = c->input(0);
       auto input_b = c->input(1);
@@ -598,9 +653,10 @@ REGISTER_OP("TensorListConcatLists")
       bool handle_data_a_nonempty = handle_data_a && !handle_data_a->empty();
       bool handle_data_b_nonempty = handle_data_b && !handle_data_b->empty();
       if (!(handle_data_a_nonempty || handle_data_b_nonempty)) {
+        const FullTypeDef& ret_types = c->ret_types();
         c->set_output_handle_shapes_and_types(
-            0, {{c->UnknownShape(), element_dtype}});
-        return Status::OK();
+            0, {{c->UnknownShape(), element_dtype, ret_types.args(0)}});
+        return OkStatus();
       }
       shape_inference::ShapeAndType list_shape_type_a =
           handle_data_a_nonempty ? handle_data_a->at(0) : handle_data_b->at(0);
@@ -620,7 +676,7 @@ REGISTER_OP("TensorListConcatLists")
                                   list_shape_type_b.shape,
                                   &list_shape_type_a.shape));
       c->set_output_handle_shapes_and_types(0, {list_shape_type_a});
-      return Status::OK();
+      return OkStatus();
     });
 
 }  // namespace
