@@ -14,7 +14,7 @@ limitations under the License.
 ==============================================================================*/
 
 #include "tensorflow/core/framework/common_shape_fns.h"
-#include "tensorflow/core/framework/dataset_stateful_op_whitelist.h"
+#include "tensorflow/core/framework/dataset_stateful_op_allowlist.h"
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/op_def_builder.h"
 #include "tensorflow/core/framework/shape_inference.h"
@@ -39,7 +39,7 @@ Status TwoElementVectorInputsAndScalarOutputs(InferenceContext* c) {
   for (int i = 0; i < c->num_outputs(); ++i) {
     c->set_output(i, c->Scalar());
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status ScalarAndTwoElementVectorInputsAndScalarOutputs(InferenceContext* c) {
@@ -53,17 +53,17 @@ Status ScalarAndTwoElementVectorInputsAndScalarOutputs(InferenceContext* c) {
   for (int i = 0; i < c->num_outputs(); ++i) {
     c->set_output(i, c->Scalar());
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 Status TwoElementOutput(InferenceContext* c) {
   c->set_output(0, c->Vector(2));
-  return Status::OK();
+  return OkStatus();
 }
 
 Status ScalarOutput(InferenceContext* c) {
   c->set_output(0, c->Scalar());
-  return Status::OK();
+  return OkStatus();
 }
 }  // namespace
 
@@ -84,13 +84,38 @@ REGISTER_OP("LookupTableFind")
       ShapeHandle unused;
       TF_RETURN_IF_ERROR(c->WithRankAtMost(c->input(2), 1, &unused));
       c->set_output(0, c->UnknownShape());
-      return Status::OK();
+      return OkStatus();
     });
 
+Status ValidateTableType(InferenceContext* c,
+                         const ShapeAndType& key_shape_and_type,
+                         const string& key_dtype_attr,
+                         const ShapeAndType& value_shape_and_type,
+                         const string& value_dtype_attr) {
+  DataType key_dtype;
+  TF_RETURN_IF_ERROR(c->GetAttr(key_dtype_attr, &key_dtype));
+  if (key_shape_and_type.dtype != key_dtype) {
+    return errors::InvalidArgument(
+        "Trying to read value with wrong dtype. "
+        "Expected ",
+        DataTypeString(key_shape_and_type.dtype), " got ",
+        DataTypeString(key_dtype));
+  }
+  DataType value_dtype;
+  TF_RETURN_IF_ERROR(c->GetAttr(value_dtype_attr, &value_dtype));
+  if (value_shape_and_type.dtype != value_dtype) {
+    return errors::InvalidArgument(
+        "Trying to read value with wrong dtype. "
+        "Expected ",
+        DataTypeString(value_shape_and_type.dtype), " got ",
+        DataTypeString(value_dtype));
+  }
+  return OkStatus();
+}
+
 Status ValidateTableResourceHandle(InferenceContext* c, ShapeHandle keys,
                                    const string& key_dtype_attr,
                                    const string& value_dtype_attr,
-                                   bool is_lookup,
                                    ShapeAndType* output_shape_and_type) {
   auto* handle_data = c->input_handle_shapes_and_types(0);
   if (handle_data == nullptr || handle_data->size() != 2) {
@@ -99,60 +124,38 @@ Status ValidateTableResourceHandle(InferenceContext* c, ShapeHandle keys,
   } else {
     const ShapeAndType& key_shape_and_type = (*handle_data)[0];
     const ShapeAndType& value_shape_and_type = (*handle_data)[1];
-    DataType key_dtype;
-    TF_RETURN_IF_ERROR(c->GetAttr(key_dtype_attr, &key_dtype));
-    if (key_shape_and_type.dtype != key_dtype) {
-      return errors::InvalidArgument(
-          "Trying to read value with wrong dtype. "
-          "Expected ",
-          DataTypeString(key_shape_and_type.dtype), " got ",
-          DataTypeString(key_dtype));
-    }
-    DataType value_dtype;
-    TF_RETURN_IF_ERROR(c->GetAttr(value_dtype_attr, &value_dtype));
-    if (value_shape_and_type.dtype != value_dtype) {
-      return errors::InvalidArgument(
-          "Trying to read value with wrong dtype. "
-          "Expected ",
-          DataTypeString(value_shape_and_type.dtype), " got ",
-          DataTypeString(value_dtype));
-    }
+    TF_RETURN_IF_ERROR(ValidateTableType(c, key_shape_and_type, key_dtype_attr,
+                                         value_shape_and_type,
+                                         value_dtype_attr));
     output_shape_and_type->dtype = value_shape_and_type.dtype;
-
-    if (is_lookup) {
-      if (c->RankKnown(key_shape_and_type.shape) && c->RankKnown(keys)) {
-        int keys_rank = c->Rank(keys);
-        int key_suffix_rank = c->Rank(key_shape_and_type.shape);
-        if (keys_rank < key_suffix_rank) {
-          return errors::InvalidArgument(
-              "Expected keys to have suffix ",
-              c->DebugString(key_shape_and_type.shape),
-              " but saw shape: ", c->DebugString(keys));
-        }
-        for (int d = 0; d < key_suffix_rank; d++) {
-          // Ensure the suffix of keys match what's in the Table.
-          DimensionHandle dim = c->Dim(key_shape_and_type.shape, d);
-          TF_RETURN_IF_ERROR(
-              c->ReplaceDim(keys, keys_rank - key_suffix_rank + d, dim, &keys));
-        }
-        std::vector<DimensionHandle> keys_prefix_vec;
-        keys_prefix_vec.reserve(keys_rank - key_suffix_rank);
-        for (int d = 0; d < keys_rank - key_suffix_rank; ++d) {
-          keys_prefix_vec.push_back(c->Dim(keys, d));
-        }
-        ShapeHandle keys_prefix = c->MakeShape(keys_prefix_vec);
-        TF_RETURN_IF_ERROR(c->Concatenate(keys_prefix,
-                                          value_shape_and_type.shape,
-                                          &output_shape_and_type->shape));
-      } else {
-        output_shape_and_type->shape = c->UnknownShape();
+    if (c->RankKnown(key_shape_and_type.shape) && c->RankKnown(keys)) {
+      int keys_rank = c->Rank(keys);
+      int key_suffix_rank = c->Rank(key_shape_and_type.shape);
+      if (keys_rank < key_suffix_rank) {
+        return errors::InvalidArgument(
+            "Expected keys to have suffix ",
+            c->DebugString(key_shape_and_type.shape),
+            " but saw shape: ", c->DebugString(keys));
       }
-    } else {
-      TF_RETURN_IF_ERROR(c->Concatenate(keys, value_shape_and_type.shape,
+      for (int d = 0; d < key_suffix_rank; d++) {
+        // Ensure the suffix of keys match what's in the Table.
+        DimensionHandle dim = c->Dim(key_shape_and_type.shape, d);
+        TF_RETURN_IF_ERROR(
+            c->ReplaceDim(keys, keys_rank - key_suffix_rank + d, dim, &keys));
+      }
+      std::vector<DimensionHandle> keys_prefix_vec;
+      keys_prefix_vec.reserve(keys_rank - key_suffix_rank);
+      for (int d = 0; d < keys_rank - key_suffix_rank; ++d) {
+        keys_prefix_vec.push_back(c->Dim(keys, d));
+      }
+      ShapeHandle keys_prefix = c->MakeShape(keys_prefix_vec);
+      TF_RETURN_IF_ERROR(c->Concatenate(keys_prefix, value_shape_and_type.shape,
                                         &output_shape_and_type->shape));
+    } else {
+      output_shape_and_type->shape = c->UnknownShape();
     }
   }
-  return Status::OK();
+  return OkStatus();
 }
 
 REGISTER_OP("LookupTableFindV2")
@@ -166,22 +169,17 @@ REGISTER_OP("LookupTableFindV2")
       ShapeHandle handle;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &handle));
 
-      // Default value must be scalar or vector.
-      ShapeHandle keys;
-      TF_RETURN_IF_ERROR(c->WithRankAtMost(c->input(2), 1, &keys));
-
       ShapeAndType value_shape_and_type;
       TF_RETURN_IF_ERROR(ValidateTableResourceHandle(
           c,
           /*keys=*/c->input(1),
           /*key_dtype_attr=*/"Tin",
-          /*value_dtype_attr=*/"Tout",
-          /*is_lookup=*/true, &value_shape_and_type));
+          /*value_dtype_attr=*/"Tout", &value_shape_and_type));
       c->set_output(0, value_shape_and_type.shape);
 
-      return Status::OK();
+      return OkStatus();
     });
-WHITELIST_STATEFUL_OP_FOR_DATASET_FUNCTIONS("LookupTableFindV2");
+ALLOW_STATEFUL_OP_FOR_DATASET_FUNCTIONS("LookupTableFindV2");
 // TODO(b/72710477): Update this.
 
 REGISTER_OP("LookupTableInsert")
@@ -197,7 +195,7 @@ REGISTER_OP("LookupTableInsert")
       TF_RETURN_IF_ERROR(c->WithValue(c->Dim(handle, 0), 2, &unused_dim));
 
       // TODO(ebrevdo): Validate keys and values shape.
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("LookupTableInsertV2")
@@ -211,7 +209,7 @@ REGISTER_OP("LookupTableInsertV2")
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &handle));
 
       // TODO: Validate keys and values shape.
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("LookupTableRemoveV2")
@@ -224,20 +222,20 @@ REGISTER_OP("LookupTableRemoveV2")
       TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(1), 1, &handle));
 
       // TODO(turboale): Validate keys shape.
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("LookupTableSize")
     .Input("table_handle: Ref(string)")
     .Output("size: int64")
     .SetShapeFn(TwoElementVectorInputsAndScalarOutputs);
-WHITELIST_STATEFUL_OP_FOR_DATASET_FUNCTIONS("LookupTableSize");
+ALLOW_STATEFUL_OP_FOR_DATASET_FUNCTIONS("LookupTableSize");
 
 REGISTER_OP("LookupTableSizeV2")
     .Input("table_handle: resource")
     .Output("size: int64")
     .SetShapeFn(ScalarAndTwoElementVectorInputsAndScalarOutputs);
-WHITELIST_STATEFUL_OP_FOR_DATASET_FUNCTIONS("LookupTableSizeV2");
+ALLOW_STATEFUL_OP_FOR_DATASET_FUNCTIONS("LookupTableSizeV2");
 
 REGISTER_OP("LookupTableExport")
     .Input("table_handle: Ref(string)")
@@ -256,7 +254,7 @@ REGISTER_OP("LookupTableExport")
       ShapeHandle keys = c->Vector(c->Dim(values, 0));
       c->set_output(0, keys);
       c->set_output(1, values);
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("LookupTableExportV2")
@@ -268,17 +266,19 @@ REGISTER_OP("LookupTableExportV2")
     .SetShapeFn([](InferenceContext* c) {
       ShapeHandle handle;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &handle));
-      ShapeHandle keys = c->UnknownShapeOfRank(1);
-      ShapeAndType value_shape_and_type;
-      TF_RETURN_IF_ERROR(ValidateTableResourceHandle(
-          c,
-          /*keys=*/keys,
-          /*key_dtype_attr=*/"Tkeys",
-          /*value_dtype_attr=*/"Tvalues",
-          /*is_lookup=*/false, &value_shape_and_type));
-      c->set_output(0, keys);
-      c->set_output(1, value_shape_and_type.shape);
-      return Status::OK();
+      auto* handle_data = c->input_handle_shapes_and_types(0);
+      if (handle_data != nullptr && handle_data->size() == 2) {
+        const ShapeAndType& key_shape_and_type = (*handle_data)[0];
+        const ShapeAndType& value_shape_and_type = (*handle_data)[1];
+        TF_RETURN_IF_ERROR(ValidateTableType(c, key_shape_and_type,
+                                             /*key_dtype_attr*/ "Tkeys",
+                                             value_shape_and_type,
+                                             /*value_dtype_attr*/ "Tvalues"));
+      }
+      // Different lookup tables have different output shapes.
+      c->set_output(0, c->UnknownShape());
+      c->set_output(1, c->UnknownShape());
+      return OkStatus();
     });
 
 REGISTER_OP("LookupTableImport")
@@ -294,7 +294,7 @@ REGISTER_OP("LookupTableImport")
       TF_RETURN_IF_ERROR(c->WithValue(c->Dim(handle, 0), 2, &unused_dim));
 
       // TODO(ebrevdo): Validate keys and values shape.
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("LookupTableImportV2")
@@ -309,8 +309,11 @@ REGISTER_OP("LookupTableImportV2")
 
       ShapeHandle keys;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &keys));
-      TF_RETURN_IF_ERROR(c->Merge(keys, c->input(2), &keys));
-      return Status::OK();
+      ShapeHandle values;
+      TF_RETURN_IF_ERROR(c->WithRankAtLeast(c->input(2), 1, &values));
+      DimensionHandle unused;
+      TF_RETURN_IF_ERROR(c->Merge(c->Dim(keys, 0), c->Dim(values, 0), &unused));
+      return OkStatus();
     });
 
 Status MutableHashTableShape(InferenceContext* c, const ShapeHandle& key,
@@ -330,7 +333,28 @@ Status MutableHashTableShape(InferenceContext* c, const ShapeHandle& key,
   c->set_output_handle_shapes_and_types(
       0, std::vector<ShapeAndType>{{key_s, key_t}, {value, value_t}});
 
-  return Status::OK();
+  return OkStatus();
+}
+
+Status MutableHashTableShapeFn(InferenceContext* c) {
+  return MutableHashTableShape(c, /*key=*/c->Scalar(),
+                               /*value=*/c->Scalar());
+}
+
+Status MutableHashTableOfTensorsShapeFn(InferenceContext* c) {
+  PartialTensorShape value_p;
+  TF_RETURN_IF_ERROR(c->GetAttr("value_shape", &value_p));
+  ShapeHandle value_s;
+  TF_RETURN_IF_ERROR(c->MakeShapeFromPartialTensorShape(value_p, &value_s));
+  return MutableHashTableShape(c, /*key=*/c->Scalar(), /*value=*/value_s);
+}
+
+Status MutableDenseHashTableShapeFn(InferenceContext* c) {
+  PartialTensorShape value_p;
+  TF_RETURN_IF_ERROR(c->GetAttr("value_shape", &value_p));
+  ShapeHandle value_s;
+  TF_RETURN_IF_ERROR(c->MakeShapeFromPartialTensorShape(value_p, &value_s));
+  return MutableHashTableShape(c, /*key=*/c->input(0), /*value=*/value_s);
 }
 
 REGISTER_OP("HashTable")
@@ -353,6 +377,13 @@ REGISTER_OP("HashTableV2")
     .SetIsStateful()
     .SetShapeFn(ScalarOutput);
 
+REGISTER_OP("AnonymousHashTable")
+    .Output("table_handle: resource")
+    .Attr("key_dtype: type")
+    .Attr("value_dtype: type")
+    .SetIsStateful()
+    .SetShapeFn(ScalarOutput);
+
 REGISTER_OP("MutableHashTable")
     .Output("table_handle: Ref(string)")
     .Attr("container: string = ''")
@@ -371,10 +402,14 @@ REGISTER_OP("MutableHashTableV2")
     .Attr("key_dtype: type")
     .Attr("value_dtype: type")
     .SetIsStateful()
-    .SetShapeFn([](InferenceContext* c) {
-      return MutableHashTableShape(c, /*key=*/c->Scalar(),
-                                   /*value=*/c->Scalar());
-    });
+    .SetShapeFn(MutableHashTableShapeFn);
+
+REGISTER_OP("AnonymousMutableHashTable")
+    .Output("table_handle: resource")
+    .Attr("key_dtype: type")
+    .Attr("value_dtype: type")
+    .SetIsStateful()
+    .SetShapeFn(MutableHashTableShapeFn);
 
 REGISTER_OP("MutableHashTableOfTensors")
     .Output("table_handle: Ref(string)")
@@ -396,13 +431,15 @@ REGISTER_OP("MutableHashTableOfTensorsV2")
     .Attr("value_dtype: type")
     .Attr("value_shape: shape = {}")
     .SetIsStateful()
-    .SetShapeFn([](InferenceContext* c) {
-      PartialTensorShape value_p;
-      TF_RETURN_IF_ERROR(c->GetAttr("value_shape", &value_p));
-      ShapeHandle value_s;
-      TF_RETURN_IF_ERROR(c->MakeShapeFromPartialTensorShape(value_p, &value_s));
-      return MutableHashTableShape(c, /*key=*/c->Scalar(), /*value=*/value_s);
-    });
+    .SetShapeFn(MutableHashTableOfTensorsShapeFn);
+
+REGISTER_OP("AnonymousMutableHashTableOfTensors")
+    .Output("table_handle: resource")
+    .Attr("key_dtype: type")
+    .Attr("value_dtype: type")
+    .Attr("value_shape: shape = {}")
+    .SetIsStateful()
+    .SetShapeFn(MutableHashTableOfTensorsShapeFn);
 
 REGISTER_OP("MutableDenseHashTable")
     .Input("empty_key: key_dtype")
@@ -431,13 +468,19 @@ REGISTER_OP("MutableDenseHashTableV2")
     .Attr("initial_num_buckets: int = 131072")  // 2^17
     .Attr("max_load_factor: float = 0.8")
     .SetIsStateful()
-    .SetShapeFn([](InferenceContext* c) {
-      PartialTensorShape value_p;
-      TF_RETURN_IF_ERROR(c->GetAttr("value_shape", &value_p));
-      ShapeHandle value_s;
-      TF_RETURN_IF_ERROR(c->MakeShapeFromPartialTensorShape(value_p, &value_s));
-      return MutableHashTableShape(c, /*key=*/c->input(0), /*value=*/value_s);
-    });
+    .SetShapeFn(MutableDenseHashTableShapeFn);
+
+REGISTER_OP("AnonymousMutableDenseHashTable")
+    .Input("empty_key: key_dtype")
+    .Input("deleted_key: key_dtype")
+    .Output("table_handle: resource")
+    .Attr("key_dtype: type")
+    .Attr("value_dtype: type")
+    .Attr("value_shape: shape = {}")
+    .Attr("initial_num_buckets: int = 131072")  // 2^17
+    .Attr("max_load_factor: float = 0.8")
+    .SetIsStateful()
+    .SetShapeFn(MutableDenseHashTableShapeFn);
 
 REGISTER_OP("InitializeTable")
     .Input("table_handle: Ref(string)")
@@ -454,7 +497,7 @@ REGISTER_OP("InitializeTable")
       ShapeHandle keys;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &keys));
       TF_RETURN_IF_ERROR(c->Merge(keys, c->input(2), &keys));
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("InitializeTableV2")
@@ -470,7 +513,7 @@ REGISTER_OP("InitializeTableV2")
       ShapeHandle keys;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 1, &keys));
       TF_RETURN_IF_ERROR(c->Merge(keys, c->input(2), &keys));
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("InitializeTableFromTextFile")
@@ -480,6 +523,7 @@ REGISTER_OP("InitializeTableFromTextFile")
     .Attr("value_index: int >= -2")
     .Attr("vocab_size: int >= -1 = -1")
     .Attr("delimiter: string = '\t'")
+    .Attr("offset: int = 0")
     .SetShapeFn([](InferenceContext* c) {
       ShapeHandle handle;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 1, &handle));
@@ -487,7 +531,7 @@ REGISTER_OP("InitializeTableFromTextFile")
       TF_RETURN_IF_ERROR(c->WithValue(c->Dim(handle, 0), 2, &unused_dim));
 
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 0, &handle));
-      return Status::OK();
+      return OkStatus();
     });
 
 REGISTER_OP("InitializeTableFromTextFileV2")
@@ -497,22 +541,13 @@ REGISTER_OP("InitializeTableFromTextFileV2")
     .Attr("value_index: int >= -2")
     .Attr("vocab_size: int >= -1 = -1")
     .Attr("delimiter: string = '\t'")
+    .Attr("offset: int = 0")
     .SetShapeFn([](InferenceContext* c) {
       ShapeHandle handle;
       TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &handle));
 
       TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 0, &handle));
-      return Status::OK();
-    });
-
-REGISTER_OP("InitializeTableFromDataset")
-    .Input("table_handle: resource")
-    .Input("dataset: variant")
-    .SetShapeFn([](InferenceContext* c) {
-      ShapeHandle handle;
-      TF_RETURN_IF_ERROR(c->WithRank(c->input(0), 0, &handle));
-      TF_RETURN_IF_ERROR(c->WithRank(c->input(1), 0, &handle));
-      return Status::OK();
+      return OkStatus();
     });
 
 }  // namespace tensorflow
