@@ -17,13 +17,10 @@ limitations under the License.
 #define TENSORFLOW_CORE_PLATFORM_CLOUD_TIME_UTIL_H_
 
 #include "tensorflow/core/platform/status.h"
+#include "tensorflow/tsl/platform/cloud/time_util.h"
 
 namespace tensorflow {
-
-/// Parses the timestamp in RFC 3339 format and returns it
-/// as nanoseconds since epoch.
-Status ParseRfc3339Time(const string& time, int64* mtime_nsec);
-
+using tsl::ParseRfc3339Time;  // NOLINT(misc-unused-using-decls)
 }  // namespace tensorflow
 
 #endif  // TENSORFLOW_CORE_PLATFORM_CLOUD_TIME_UTIL_H_
