@@ -18,13 +18,15 @@ limitations under the License.
 #include <math.h>
 
 #include <algorithm>
+#include <optional>
+#include <ostream>
+#include <string>
 #include <vector>
 
 #include "google/protobuf/any.pb.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
-#include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
 #include "tensorflow/core/lib/gtl/map_util.h"
 #include "tensorflow/core/platform/logging.h"
@@ -38,17 +40,21 @@ limitations under the License.
 #include "tensorflow/core/profiler/protobuf/steps_db.pb.h"
 #include "tensorflow/core/profiler/utils/diagnostics.h"
 #include "tensorflow/core/profiler/utils/event_span.h"
+#include "tensorflow/core/profiler/utils/hardware_type_utils.h"
 #include "tensorflow/core/profiler/utils/html_utils.h"
 #include "tensorflow/core/profiler/utils/math_utils.h"
-#include "tensorflow/core/profiler/utils/tf_op_utils.h"
-#include "tensorflow/core/profiler/utils/time_utils.h"
-#include "tensorflow/core/util/stats_calculator.h"
+#include "tensorflow/core/profiler/utils/op_metrics_db_utils.h"
+#include "tensorflow/tsl/profiler/utils/format_utils.h"
+#include "tensorflow/tsl/profiler/utils/tf_op_utils.h"
+#include "tensorflow/tsl/util/stats_calculator.h"
 
 namespace tensorflow {
 namespace profiler {
 
 namespace {
 
+using tsl::profiler::OneDigit;
+
 const double kNumPsPerMs = 1000000000.0;
 
 // If the percentage of step time that is due to infeed is less than
@@ -58,6 +64,7 @@ const double kNumPsPerMs = 1000000000.0;
 // input-bound; else if it is considered HIGHLY input-bound.
 constexpr double kModeratelyInfeedBoundThresholdInPercent = 5;
 constexpr double kHighlyInfeedBoundThresholdInPercent = 20;
+
 // If the percentage of step time that is due to outfeed is less than
 // kModeratelyOutfeedBoundThresholdInPercent, it is considered NOT
 // output-bound; else if it is less than
@@ -65,6 +72,7 @@ constexpr double kHighlyInfeedBoundThresholdInPercent = 20;
 // output-bound; else if it is considered HIGHLY output-bound.
 constexpr double kModeratelyOutfeedBoundThresholdInPercent = 5;
 constexpr double kHighlyOutfeedBoundThresholdInPercent = 20;
+
 // If the percentage of step time that is due to kernel launch is less than
 // kModeratelyKernelLaunchBoundThresholdInPercent, it is considered NOT
 // kernel-launch bound; else if it is less than
@@ -72,6 +80,7 @@ constexpr double kHighlyOutfeedBoundThresholdInPercent = 20;
 // kernel-launch bound; else if it is considered HIGHLY kernel-launch bound.
 constexpr double kModeratelyKernelLaunchBoundThresholdInPercent = 3;
 constexpr double kHighlyKernelLaunchBoundThresholdInPercent = 15;
+
 // If the percentage of step time that is due to all other time is less than
 // kModeratelyAllOtherBoundThresholdInPercent, it is considered NOT
 // all-other bound; else if it is less than
@@ -79,6 +88,16 @@ constexpr double kHighlyKernelLaunchBoundThresholdInPercent = 15;
 // all-other bound; else if it is considered HIGHLY all-other bound.
 constexpr double kModeratelyAllOtherBoundThresholdInPercent = 3;
 constexpr double kHighlyAllOtherBoundThresholdInPercent = 15;
+
+// If the percentage of step time that is due to device collectives is less than
+// kModeratelyDeviceCollectivesBoundThresholdInPercent, it is considered NOT
+// device-collectives bound; else if it is less than
+// kHighlyDeviceCollectivesBoundThresholdInPercent, it is considered MODERATELY
+// device-collectives  bound; else if it is considered HIGHLY device-collectives
+// bound.
+constexpr double kModeratelyDeviceCollectivesBoundThresholdInPercent = 3;
+constexpr double kHighlyDeviceCollectivesBoundThresholdInPercent = 15;
+
 // Section number of the host-analysis section in the input-pipeline analysis.
 constexpr int kHostAnalysisSectionNumber = 3;
 // Python-only explanation for "All Others" time.
@@ -92,10 +111,11 @@ const char* kKernelLaunchTfDataContention =
 
 template <class Collection>
 double GetTimeInMs(const Collection& type_ps, EventType event_type) {
-  return PicosToMillis(gtl::FindWithDefault(type_ps, event_type, /*value=*/0));
+  return PicoToMilli(gtl::FindWithDefault(type_ps, event_type, /*value=*/0));
 }
 
-StepSummary GetStepSummaryForSampleStats(const Stat<double>& sample_stats) {
+StepSummary GetStepSummaryForSampleStats(
+    const tsl::Stat<double>& sample_stats) {
   StepSummary step_time_summary;
   double avg, sdv, min, max;
   if (sample_stats.empty()) {
@@ -117,16 +137,17 @@ StepSummary GetStepSummaryForSampleStats(const Stat<double>& sample_stats) {
 
 GenericStepTimeBreakdown ComputeGenericStepTimeBreakdownInMs(
     const InputPipelineAnalysisResult& analysis) {
-  Stat<double> unknown_time_ms;
-  Stat<double> host_wait_input_ms;
-  Stat<double> host_to_device_ms;
-  Stat<double> input_ms;
-  Stat<double> output_ms;
-  Stat<double> device_compute_ms;
-  Stat<double> device_to_device_ms;
-  Stat<double> host_compute_ms;
-  Stat<double> host_prepare_ms;
-  Stat<double> host_compile_ms;
+  tsl::Stat<double> unknown_time_ms;
+  tsl::Stat<double> host_wait_input_ms;
+  tsl::Stat<double> host_to_device_ms;
+  tsl::Stat<double> input_ms;
+  tsl::Stat<double> output_ms;
+  tsl::Stat<double> device_compute_ms;
+  tsl::Stat<double> device_to_device_ms;
+  tsl::Stat<double> device_collectives_ms;
+  tsl::Stat<double> host_compute_ms;
+  tsl::Stat<double> host_prepare_ms;
+  tsl::Stat<double> host_compile_ms;
   GenericStepTimeBreakdown result;
 
   for (const google::protobuf::Any& step_details : analysis.step_details()) {
@@ -145,6 +166,7 @@ GenericStepTimeBreakdown ComputeGenericStepTimeBreakdownInMs(
     output_ms.UpdateStat(details.output_ms());
     device_compute_ms.UpdateStat(details.device_compute_ms());
     device_to_device_ms.UpdateStat(details.device_to_device_ms());
+    device_collectives_ms.UpdateStat(details.device_collectives_ms());
     host_compute_ms.UpdateStat(details.host_compute_ms());
     host_prepare_ms.UpdateStat(details.host_prepare_ms());
     host_compile_ms.UpdateStat(details.host_compile_ms());
@@ -161,6 +183,8 @@ GenericStepTimeBreakdown ComputeGenericStepTimeBreakdownInMs(
       GetStepSummaryForSampleStats(device_compute_ms);
   *result.mutable_device_to_device_ms_summary() =
       GetStepSummaryForSampleStats(device_to_device_ms);
+  *result.mutable_device_collectives_ms_summary() =
+      GetStepSummaryForSampleStats(device_collectives_ms);
   *result.mutable_host_compute_ms_summary() =
       GetStepSummaryForSampleStats(host_compute_ms);
   *result.mutable_host_prepare_ms_summary() =
@@ -173,12 +197,13 @@ GenericStepTimeBreakdown ComputeGenericStepTimeBreakdownInMs(
 InputPipelineAnalysisResult ComputeGenericInputPipelineAnalysisResult(
     const protobuf::RepeatedPtrField<PerCoreStepInfo>& grouped_by_step) {
   InputPipelineAnalysisResult result;
+  result.set_tag(false);
 
   // Computes the summary of step time in ms.
   *result.mutable_step_time_summary() =
       ComputeStepTimeSummaryInMs(grouped_by_step);
 
-  Stat<double> input_summary_stats_in_percent;
+  tsl::Stat<double> input_summary_stats_in_percent;
   for (const auto& coreid_stepinfo_map : grouped_by_step) {
     // Iterates over each step.
     const auto* ptr = gtl::FindOrNull(coreid_stepinfo_map.step_info_per_core(),
@@ -192,7 +217,12 @@ InputPipelineAnalysisResult ComputeGenericInputPipelineAnalysisResult(
     // Adds the details for a new step.
     PerGenericStepDetails details;
     details.set_step_number(step_info.step_num());
-    details.set_step_time_ms(PicosToMillis(step_info.duration_ps()));
+    if (step_info.step_name().empty()) {
+      details.set_step_name(absl::StrCat(step_info.step_num()));
+    } else {
+      details.set_step_name(step_info.step_name());
+    }
+    details.set_step_time_ms(PicoToMilli(step_info.duration_ps()));
     GenericStepBreakdown generic;
     bool success = step_info.step_breakdown().UnpackTo(&generic);
     if (!success && !step_info.step_breakdown().type_url().empty()) {
@@ -210,10 +240,10 @@ InputPipelineAnalysisResult ComputeGenericInputPipelineAnalysisResult(
                                   GetTimeInMs(type_ps, DEVICE_COMPUTE_32));
     details.set_device_to_device_ms(GetTimeInMs(type_ps, DEVICE_TO_DEVICE) +
                                     GetTimeInMs(type_ps, DEVICE_WAIT_DEVICE));
+    details.set_device_collectives_ms(GetTimeInMs(type_ps, DEVICE_COLLECTIVES));
     details.set_host_compute_ms(GetTimeInMs(type_ps, HOST_COMPUTE));
     details.set_host_prepare_ms(GetTimeInMs(type_ps, HOST_PREPARE));
     details.set_host_compile_ms(GetTimeInMs(type_ps, HOST_COMPILE));
-
     result.add_step_details()->PackFrom(details);
 
     const double input_percent_of_step_time =
@@ -260,19 +290,21 @@ std::string InputOpCategoryString(InputOpCategory category) {
 inline bool IsInputOp(absl::string_view category) {
   // Do not include "IteratorGetNext*" here, because IteratorGetNext is an Op
   // that experiences the install stall, not an Op that causes the input stall.
-  return IsInfeedEnqueueOp(category) || IsDatasetOp(category) ||
-         IsMemcpyHToDOp(category);
+  return tsl::profiler::IsInfeedEnqueueOp(category) ||
+         tsl::profiler::IsDatasetOp(category) ||
+         tsl::profiler::IsMemcpyHToDOp(category);
 }
 
 // TODO(ckluk):
 //   Confirm with the tf.data team if the classification below is correct.
 InputOpCategory CategorizeInputOp(absl::string_view name,
                                   absl::string_view category) {
-  if (IsInfeedEnqueueOp(category) || IsMemcpyHToDOp(category)) {
+  if (tsl::profiler::IsInfeedEnqueueOp(category) ||
+      tsl::profiler::IsMemcpyHToDOp(category)) {
     // Ops for sending input from host to device.
     return InputOpCategory::kEnqueue;
   }
-  DCHECK(IsDatasetOp(category));
+  DCHECK(tsl::profiler::IsDatasetOp(category));
   if (absl::EndsWith(name, "::TFRecord") ||
       absl::EndsWith(name, "::TextLine") ||
       absl::EndsWith(name, "::FixedLengthRecord") ||
@@ -318,8 +350,8 @@ InputOpDetails ConvertOpMetricsToInputOpDetails(const OpMetrics& op_metrics,
   InputOpDetails details;
   details.set_op_name(op_metrics.name());
   details.set_count(op_metrics.occurrences());
-  details.set_time_in_ms(PicosToMillis(op_metrics.time_ps()));
-  details.set_self_time_in_ms(PicosToMillis(op_metrics.self_time_ps()));
+  details.set_time_in_ms(PicoToMilli(op_metrics.time_ps()));
+  details.set_self_time_in_ms(PicoToMilli(op_metrics.self_time_ps()));
   details.set_time_in_percent(
       100.0 * SafeDivide(op_metrics.time_ps(), input_op_time_ps));
   details.set_self_time_in_percent(
@@ -332,16 +364,13 @@ InputOpDetails ConvertOpMetricsToInputOpDetails(const OpMetrics& op_metrics,
 double RatioOfHostToDeviceTimeToStepTime(
     const OpMetricsDb& host_tf_metrics_db,
     const InputPipelineAnalysisResult& input_pipeline_analysis) {
-  if (host_tf_metrics_db.total_host_infeed_enq_start_timestamp_ps_diff() > 0) {
-    // For TPU execution that uses infeed.
-    //    We use total_host_infeed_enq_start_timestamp_ps_diff_ to approximate
-    //    the total host step time.
-    return std::min(
-        1.0, SafeDivide(host_tf_metrics_db.total_host_infeed_enq_duration_ps(),
-                        host_tf_metrics_db
-                            .total_host_infeed_enq_start_timestamp_ps_diff()));
+  // For TPU execution that uses infeed.
+  std::optional<double> host_infeed_enqueue_ratio =
+      HostInfeedEnqueueRatio(host_tf_metrics_db);
+  if (host_infeed_enqueue_ratio.has_value()) {
+    return host_infeed_enqueue_ratio.value();
   }
-  // For GPU and TPU execution that doesn't use infeed.
+  // For GPU and TPU execution that do not use infeed.
   double avg_step_time_ms =
       input_pipeline_analysis.step_time_summary().average();
   if (avg_step_time_ms > 0) {
@@ -351,21 +380,42 @@ double RatioOfHostToDeviceTimeToStepTime(
             &generic_breakdown)) {
       double avg_host_to_device_time_ms =
           generic_breakdown.host_to_device_ms_summary().average();
-      return std::min(1.0,
-                      SafeDivide(avg_host_to_device_time_ms, avg_step_time_ms));
+      return SafeDivide(avg_host_to_device_time_ms, avg_step_time_ms);
     }
   }
   return 0.0;
 }
 
+void DeviceCollectivesAnalysis(double device_collectives_percent,
+                               std::string* device_collectives_classification,
+                               std::string* device_collectives_statement) {
+  if (device_collectives_percent >=
+      kHighlyDeviceCollectivesBoundThresholdInPercent) {
+    *device_collectives_classification = "high";
+    *device_collectives_statement =
+        absl::StrCat(OneDigit(device_collectives_percent),
+                     " % of the total step time sampled is spent on 'Device "
+                     "Collective Communication'.");
+  } else if (device_collectives_percent >=
+             kModeratelyDeviceCollectivesBoundThresholdInPercent) {
+    *device_collectives_classification = "moderate";
+    *device_collectives_statement =
+        absl::StrCat(OneDigit(device_collectives_percent),
+                     " % of the total step time sampled is spent on 'Device "
+                     "Collective Communication'.");
+  } else {
+    *device_collectives_classification = "no";
+    *device_collectives_statement = "";
+  }
+}
+
 void KernelLaunchAnalysis(bool tfdata_used, double kernel_launch_percent,
                           std::string* kernel_launch_classification,
                           std::string* kernel_launch_statement) {
-  std::string percent_str = absl::StrFormat("%.1lf", kernel_launch_percent);
   if (kernel_launch_percent >= kHighlyKernelLaunchBoundThresholdInPercent) {
     *kernel_launch_classification = "high";
     *kernel_launch_statement = absl::StrCat(
-        percent_str,
+        OneDigit(kernel_launch_percent),
         " % of the total step time sampled is spent on 'Kernel Launch'.");
     if (tfdata_used) {
       absl::StrAppend(kernel_launch_statement, kKernelLaunchTfDataContention);
@@ -374,7 +424,7 @@ void KernelLaunchAnalysis(bool tfdata_used, double kernel_launch_percent,
              kModeratelyKernelLaunchBoundThresholdInPercent) {
     *kernel_launch_classification = "moderate";
     *kernel_launch_statement = absl::StrCat(
-        percent_str,
+        OneDigit(kernel_launch_percent),
         " % of the total step time sampled is spent on 'Kernel Launch'.");
     if (tfdata_used) {
       absl::StrAppend(kernel_launch_statement, kKernelLaunchTfDataContention);
@@ -393,15 +443,14 @@ void AllOtherAnalysis(bool all_other_reported, double all_other_percent,
     *all_other_statement = "";
     return;
   }
-  std::string percent_str = absl::StrFormat("%.1lf", all_other_percent);
   if (all_other_percent >= kHighlyAllOtherBoundThresholdInPercent) {
     *all_other_classification = "high";
     *all_other_statement =
-        absl::StrCat(percent_str, kAllOthersPythonExplanation);
+        absl::StrCat(OneDigit(all_other_percent), kAllOthersPythonExplanation);
   } else if (all_other_percent >= kModeratelyAllOtherBoundThresholdInPercent) {
     *all_other_classification = "moderate";
     *all_other_statement =
-        absl::StrCat(percent_str, kAllOthersPythonExplanation);
+        absl::StrCat(OneDigit(all_other_percent), kAllOthersPythonExplanation);
   } else {
     *all_other_classification = "no";
     *all_other_statement = "";
@@ -444,7 +493,7 @@ void GenerateHostResult(const OpMetricsDb& host_tf_metrics_db,
     *result->add_input_op_details() = ConvertOpMetricsToInputOpDetails(
         *op_metrics, input_op_metrics.input_op_time_ps, category);
     aggregated_input_op_times_us[category] +=
-        PicosToMicros(op_metrics->self_time_ps());
+        PicoToMicro(op_metrics->self_time_ps());
   }
 
   double enqueue_time_us =
@@ -454,8 +503,8 @@ void GenerateHostResult(const OpMetricsDb& host_tf_metrics_db,
       aggregated_input_op_times_us[InputOpCategory::kAdvancedFileRead] +
       aggregated_input_op_times_us[InputOpCategory::kPreprocessing];
 
-  double ratio = RatioOfHostToDeviceTimeToStepTime(host_tf_metrics_db, *result);
-  DCHECK_LE(ratio, 1.0);
+  double ratio = std::min(
+      1.0, RatioOfHostToDeviceTimeToStepTime(host_tf_metrics_db, *result));
   DCHECK_GE(ratio, 0.0);
   double non_enqueue_time_us = (ratio != 0.0)
                                    ? (enqueue_time_us * (1.0 - ratio) / ratio)
@@ -534,7 +583,7 @@ InputPipelineAnalysisRecommendation GenerateRecommendation() {
 
 StepSummary ComputeStepTimeSummaryInMs(
     const protobuf::RepeatedPtrField<PerCoreStepInfo>& grouped_by_step) {
-  Stat<double> total_step_stats_in_ms;
+  tsl::Stat<double> total_step_stats_in_ms;
   // iterates over each step.
   for (const auto& coreid_stepinfo_map : grouped_by_step) {
     double max_per_step_stats_in_ms = 0.0;
@@ -553,18 +602,23 @@ StepSummary ComputeStepTimeSummaryInMs(
 }
 
 InputPipelineAnalysisResult ConvertOpStatsToInputPipelineAnalysis(
-    const OpStats& op_stats, const HardwareType& hardware_type) {
+    const OpStats& op_stats) {
   InputPipelineAnalysisResult result =
       ComputeGenericInputPipelineAnalysisResult(
           op_stats.step_db().step_sequence());
   PopulateStepDiagnostics(op_stats, result.mutable_diagnostics());
-  result.set_hardware_type(HardwareType_Name(hardware_type));
+  result.set_hardware_type(HardwareType_Name(
+      ParseHardwareType(op_stats.run_environment().device_type())));
   GenerateHostResult(op_stats.host_op_metrics_db(), &result);
 
   InputPipelineAnalysisRecommendation recommendation = GenerateRecommendation();
   BottleneckAnalysis bottleneck_analysis = ComputeBottleneckAnalysis(
       result.input_time_breakdown(), result.step_details());
-  result.set_overall_input_percent(bottleneck_analysis.input_percent());
+  result.set_input_percent(bottleneck_analysis.input_percent());
+  result.set_output_percent(bottleneck_analysis.output_percent());
+  result.set_idle_percent(bottleneck_analysis.idle_percent());
+  result.set_compute_percent(bottleneck_analysis.compute_percent());
+
   recommendation.mutable_bottleneck_analysis()->PackFrom(bottleneck_analysis);
   *recommendation.mutable_summary_next_step() =
       GetSummaryNextStep(bottleneck_analysis.input_classification(),
@@ -578,18 +632,18 @@ bool InputAnalysis(double input_percent, double all_other_percent,
                    std::string* input_classification,
                    std::string* input_statement) {
   absl::string_view non_input_time = "other time";
-  std::string infeed_percent_str = absl::StrFormat("%.1lf", input_percent);
   if (input_percent >= kHighlyInfeedBoundThresholdInPercent) {
     *input_classification = "host";
     *input_statement = absl::StrCat(
-        "Your program is HIGHLY input-bound because ", infeed_percent_str,
+        "Your program is HIGHLY input-bound because ", OneDigit(input_percent),
         "% of the total step time sampled is waiting for input. Therefore, you "
         "should first focus on reducing the input time.");
     return false;
   } else if (input_percent >= kModeratelyInfeedBoundThresholdInPercent) {
     *input_classification = "both";
     *input_statement = absl::StrCat(
-        "Your program is MODERATELY input-bound because ", infeed_percent_str,
+        "Your program is MODERATELY input-bound because ",
+        OneDigit(input_percent),
         "% of the total step time sampled is waiting for input. Therefore, "
         "you would need to reduce both the input time and ",
         non_input_time, ".");
@@ -598,41 +652,40 @@ bool InputAnalysis(double input_percent, double all_other_percent,
     // Input analysis says it is not input-bound, but "All-Other" time
     // is significant. It could still be input-bound (or Python overhead).
     *input_classification = "both";
-    std::string all_other_percent_str =
-        absl::StrFormat("%.1lf", all_other_percent);
     *input_statement = absl::StrCat(
         "Your program is POTENTIALLY input-bound because ",
-        all_other_percent_str,
+        OneDigit(all_other_percent),
         "% of the total step time sampled is spent on 'All Others' time (which "
         "could be due to I/O or Python execution or both).");
     return true;
   } else {
     // Defintely not input-bound.
     *input_classification = "device";
-    *input_statement = absl::StrCat(
-        "Your program is NOT input-bound because only ", infeed_percent_str,
-        "% of the total step time sampled is waiting for "
-        "input. Therefore, you should focus on "
-        "reducing ",
-        non_input_time, ".");
+    *input_statement =
+        absl::StrCat("Your program is NOT input-bound because only ",
+                     OneDigit(input_percent),
+                     "% of the total step time sampled is waiting for "
+                     "input. Therefore, you should focus on "
+                     "reducing ",
+                     non_input_time, ".");
     return false;
   }
 }
 
 void OutputAnalysis(double output_percent, std::string* output_classification,
                     std::string* output_statement) {
-  string tc_outfeed_percent_str = absl::StrFormat("%.1lf", output_percent);
   if (output_percent >= kHighlyOutfeedBoundThresholdInPercent) {
     *output_classification = "host";
     *output_statement = absl::StrCat(
-        "Your program is HIGHLY output-bound because ", tc_outfeed_percent_str,
+        "Your program is HIGHLY output-bound because ",
+        OneDigit(output_percent),
         "% of the total step time sampled is spent on output. Therefore, you "
         "should first focus on reducing the output time.");
   } else if (output_percent >= kModeratelyOutfeedBoundThresholdInPercent) {
     *output_classification = "both";
     *output_statement = absl::StrCat(
         "Your program is MODERATELY output-bound because ",
-        tc_outfeed_percent_str,
+        OneDigit(output_percent),
         "% of the total step time sampled is spent on output. Therefore, "
         "you would need to reduce both the output time and other time.");
   } else {
@@ -651,7 +704,9 @@ BottleneckAnalysis ComputeBottleneckAnalysis(
   double total_host_compute_ms = 0;
   double total_host_prepare_ms = 0;
   double total_host_compile_ms = 0;
+  double total_device_compute_ms = 0;
   double total_device_to_device_ms = 0;
+  double total_device_collectives_ms = 0;
   double total_unknown_ms = 0;
 
   for (const google::protobuf::Any& step_details : any_step_details) {
@@ -667,7 +722,9 @@ BottleneckAnalysis ComputeBottleneckAnalysis(
         details.host_wait_input_ms() + details.host_to_device_ms();
     total_output_ms += details.output_ms();
     total_host_prepare_ms += details.host_prepare_ms();
+    total_device_compute_ms += details.device_compute_ms();
     total_device_to_device_ms += details.device_to_device_ms();
+    total_device_collectives_ms += details.device_collectives_ms();
     total_host_compute_ms += details.host_compute_ms();
     total_host_compile_ms += details.host_compile_ms();
     total_unknown_ms += details.unknown_time_ms();
@@ -683,18 +740,37 @@ BottleneckAnalysis ComputeBottleneckAnalysis(
     analysis.set_kernel_launch_statement("");
     analysis.set_all_other_classification("no");
     analysis.set_all_other_statement("");
+    analysis.set_device_collectives_classification("no");
+    analysis.set_device_collectives_statement("");
     return analysis;
   }
   double input_percent = 100.0 * total_input_ms / total_step_time_ms;
+  double output_percent = 100.0 * total_output_ms / total_step_time_ms;
+  double compute_percent = 100.0 * total_device_compute_ms / total_step_time_ms;
+  double device_collectives_percent =
+      100.0 * total_device_collectives_ms / total_step_time_ms;
+
+  // idle_percent includes host_prepare (i.e. kernel launch, device-to-device,
+  // host compute, host compile, and unknown.
+  double idle_percent =
+      std::max(0.0, 100.0 - input_percent - output_percent - compute_percent -
+                        device_collectives_percent);
   double kernel_launch_percent =
       100.0 * total_host_prepare_ms / total_step_time_ms;
   double all_other_percent = 100.0 * total_unknown_ms / total_step_time_ms;
+
   std::string input_classification;
   std::string input_statement;
   bool all_other_reported =
       InputAnalysis(input_percent, all_other_percent, &input_classification,
                     &input_statement);
 
+  std::string device_collectives_classification;
+  std::string device_collectives_statement;
+  DeviceCollectivesAnalysis(device_collectives_percent,
+                            &device_collectives_classification,
+                            &device_collectives_statement);
+
   std::string kernel_launch_classification;
   std::string kernel_launch_statement;
   KernelLaunchAnalysis(TfDataInUse(input_time_breakdown), kernel_launch_percent,
@@ -707,12 +783,20 @@ BottleneckAnalysis ComputeBottleneckAnalysis(
 
   BottleneckAnalysis analysis;
   analysis.set_input_percent(input_percent);
+  analysis.set_output_percent(output_percent);
+  analysis.set_idle_percent(idle_percent);
+  analysis.set_compute_percent(compute_percent);
+
   analysis.set_input_classification(input_classification);
   analysis.set_input_statement(input_statement);
   analysis.set_kernel_launch_classification(kernel_launch_classification);
   analysis.set_kernel_launch_statement(kernel_launch_statement);
   analysis.set_all_other_classification(all_other_classification);
   analysis.set_all_other_statement(all_other_statement);
+  analysis.set_device_collectives_classification(
+      device_collectives_classification);
+  analysis.set_device_collectives_statement(device_collectives_statement);
+
   return analysis;
 }
 
