@@ -33,7 +33,6 @@ limitations under the License.
 
 #include "third_party/eigen3/unsupported/Eigen/CXX11/Tensor"
 #if GOOGLE_CUDA
-#include "third_party/gpus/cuda/include/cuComplex.h"
 #include "third_party/gpus/cuda/include/cuda.h"
 #else
 #include "rocm/include/hip/hip_complex.h"
@@ -43,8 +42,6 @@ limitations under the License.
 #include "tensorflow/core/util/gpu_cuda_alias.h"
 
 #if GOOGLE_CUDA
-using gpuFloatComplex = cuFloatComplex;
-using gpuDoubleComplex = cuDoubleComplex;
 using gpuStream_t = cudaStream_t;
 using gpuEvent_t = cudaEvent_t;
 #define gpuEventRecord cudaEventRecord
@@ -56,8 +53,6 @@ using gpuEvent_t = cudaEvent_t;
 #define gpuDeviceSynchronize cudaDeviceSynchronize
 #define gpuFree cudaFree
 #elif TENSORFLOW_USE_ROCM
-using gpuFloatComplex = hipFloatComplex;
-using gpuDoubleComplex = hipDoubleComplex;
 using gpuStream_t = hipStream_t;
 using gpuEvent_t = hipEvent_t;
 using cudaError = int;
@@ -163,8 +158,9 @@ using CudaGridRange = GpuGridRange<T...>;
 // Usage: for(int i : GpuGridRangeX(count)) { visit(i); }
 template <typename T>
 __device__ detail::GpuGridRange<T> GpuGridRangeX(T count) {
-  return detail::GpuGridRange<T>(blockIdx.x * blockDim.x + threadIdx.x,
-                                 gridDim.x * blockDim.x, count);
+  return detail::GpuGridRange<T>(
+      /*begin=*/blockIdx.x * blockDim.x + threadIdx.x,
+      /*delta=*/gridDim.x * blockDim.x, /*end=*/count);
 }
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuGridRangeX, CudaGridRangeX);
 
@@ -172,8 +168,9 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuGridRangeX, CudaGridRangeX);
 // Usage: for(int i : GpuGridRangeY(count)) { visit(i); }
 template <typename T>
 __device__ detail::GpuGridRange<T> GpuGridRangeY(T count) {
-  return detail::GpuGridRange<T>(blockIdx.y * blockDim.y + threadIdx.y,
-                                 gridDim.y * blockDim.y, count);
+  return detail::GpuGridRange<T>(
+      /*begin=*/blockIdx.y * blockDim.y + threadIdx.y,
+      /*delta=*/gridDim.y * blockDim.y, /*end=*/count);
 }
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuGridRangeY, CudaGridRangeY);
 
@@ -181,8 +178,9 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuGridRangeY, CudaGridRangeY);
 // Usage: for(int i : GpuGridRangeZ(count)) { visit(i); }
 template <typename T>
 __device__ detail::GpuGridRange<T> GpuGridRangeZ(T count) {
-  return detail::GpuGridRange<T>(blockIdx.z * blockDim.z + threadIdx.z,
-                                 gridDim.z * blockDim.z, count);
+  return detail::GpuGridRange<T>(
+      /*begin=*/blockIdx.z * blockDim.z + threadIdx.z,
+      /*delta=*/gridDim.z * blockDim.z, /*end=*/count);
 }
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuGridRangeZ, CudaGridRangeZ);
 
@@ -223,15 +221,11 @@ namespace detail {
 __device__ inline bool GpuValidateShuffleSyncMask(unsigned mask,
                                                   unsigned src_lane) {
   unsigned src_dst_mask = 1u << GpuLaneId() | 1u << src_lane;
-#if CUDA_VERSION >= 9000
-  unsigned src_lane_mask = __shfl_sync(mask, mask, src_lane);
-#else
 #if GOOGLE_CUDA
-  unsigned src_lane_mask = __shfl(mask, src_lane);
-#elif TENSORFLOW_USE_ROCM
+  unsigned src_lane_mask = __shfl_sync(mask, mask, src_lane);
+#else  // TENSORFLOW_USE_ROCM
   unsigned src_lane_mask =
       __shfl(static_cast<int>(mask), static_cast<int>(src_lane));
-#endif
 #endif
   return (src_dst_mask & ~mask) == 0 && src_lane_mask == mask;
 }
@@ -294,7 +288,7 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuShuffleXorGetSrcLane,
 // Wrapper for __syncwarp. No-op for CUDA 8 and earlier.
 __device__ inline void GpuSyncWarp(unsigned mask = kCudaWarpAll) {
   assert(mask & 1u << GpuLaneId());
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   __syncwarp(mask);
 #endif
 }
@@ -304,9 +298,9 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuSyncWarp, CudaSyncWarp);
 // convergence, see comment above for details.
 __device__ inline unsigned GpuBallotSync(unsigned mask, int pred) {
   assert(mask & 1u << GpuLaneId());
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   return __ballot_sync(mask, pred);
-#else
+#else  // TENSORFLOW_USE_ROCM
   return __ballot(pred) & mask;  // Apply mask to match __ballot_sync's spec.
 #endif
 }
@@ -316,9 +310,9 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuBallotSync, CudaBallotSync);
 // convergence, see comment above for details.
 __device__ inline int GpuAnySync(unsigned mask, int pred) {
   assert(mask & 1u << GpuLaneId());
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   return __any_sync(mask, pred);
-#else
+#else  // TENSORFLOW_USE_ROCM
   return __any(pred);
 #endif
 }
@@ -328,9 +322,9 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuAnySync, CudaAnySync);
 // convergence, see comment above for details.
 __device__ inline int GpuAllSync(unsigned mask, int pred) {
   assert(mask & 1u << GpuLaneId());
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   return __all_sync(mask, pred);
-#else
+#else  // TENSORFLOW_USE_ROCM
   return __all(pred);
 #endif
 }
@@ -344,9 +338,9 @@ __device__ T GpuShuffleSync(unsigned mask, T value, int src_lane,
   assert(!(width & width - 1));
   assert(detail::GpuValidateShuffleSyncMask(
       mask, detail::GpuShuffleGetSrcLane(src_lane, width)));
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   return __shfl_sync(mask, value, src_lane, width);
-#else
+#else  // TENSORFLOW_USE_ROCM
   return __shfl(value, src_lane, width);
 #endif
 }
@@ -383,9 +377,9 @@ __device__ inline T GpuShuffleUpSync(unsigned mask, T value, unsigned delta,
   assert(!(width & width - 1));
   assert(detail::GpuValidateShuffleSyncMask(
       mask, detail::GpuShuffleUpGetSrcLane(delta, width)));
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   return __shfl_up_sync(mask, value, delta, width);
-#else
+#else  // TENSORFLOW_USE_ROCM
   return __shfl_up(value, delta, width);
 #endif
 }
@@ -423,9 +417,9 @@ __device__ inline T GpuShuffleDownSync(unsigned mask, T value, unsigned delta,
   assert(!(width & width - 1));
   assert(detail::GpuValidateShuffleSyncMask(
       mask, detail::GpuShuffleDownGetSrcLane(delta, width)));
-#if CUDA_VERSION >= 9000
+#if GOOGLE_CUDA
   return __shfl_down_sync(mask, value, delta, width);
-#else
+#else  // TENSORFLOW_USE_ROCM
   return __shfl_down(value, delta, width);
 #endif
 }
@@ -464,11 +458,7 @@ __device__ T GpuShuffleXorSync(unsigned mask, T value, int lane_mask,
   assert(detail::GpuValidateShuffleSyncMask(
       mask, detail::GpuShuffleXorGetSrcLane(lane_mask, width)));
 #if GOOGLE_CUDA
-#if CUDA_VERSION >= 9000
   return __shfl_xor_sync(mask, value, lane_mask, width);
-#else
-  return __shfl_xor(value, lane_mask, width);
-#endif
 #elif TENSORFLOW_USE_ROCM
   // ROCM TODO: check if HIP should be changed to cope with more types
   return __shfl_xor(static_cast<int>(value), lane_mask, width);
@@ -565,14 +555,14 @@ __global__ void SetZero(const int count, T* __restrict__ ptr) {
 }
 
 // Helper to set all tensor entries to a specific value.
-template <typename T>
-__global__ void SetToValue(const int count, T* __restrict__ ptr, T value) {
+template <typename T, typename Tvalue = T>
+__global__ void SetToValue(const int count, T* __restrict__ ptr, Tvalue value) {
   // Check that the grid is one dimensional and index doesn't overflow.
   assert(blockDim.y == 1);
   assert(blockDim.z == 1);
   assert(blockDim.x * gridDim.x / blockDim.x == gridDim.x);
   for (int i : GpuGridRangeX(count)) {
-    ptr[i] = value;
+    ptr[i] = static_cast<T>(value);
   }
 }
 
@@ -606,7 +596,7 @@ __device__ double GpuAtomicCasHelper(double* ptr, F accumulate) {
   // HIP has a bug in the implementation of __longlong_as_double
   // So workaround it by using reinterpret_cast<double*>.
   uint64_t result =
-      GpuAtomicCasHelper(reinterpret_cast<tensorflow::uint64*>(ptr),
+      GpuAtomicCasHelper(reinterpret_cast<unsigned long long*>(ptr),
                          [accumulate](tensorflow::uint64 a) {
                            return __double_as_longlong(
                                accumulate(*(reinterpret_cast<double*>(&a))));
@@ -614,7 +604,7 @@ __device__ double GpuAtomicCasHelper(double* ptr, F accumulate) {
   return *(reinterpret_cast<double*>(&result));
 #else
   return __longlong_as_double(GpuAtomicCasHelper(
-      reinterpret_cast<tensorflow::uint64*>(ptr),
+      reinterpret_cast<unsigned long long*>(ptr),
       [accumulate](tensorflow::uint64 a) {
         return __double_as_longlong(accumulate(__longlong_as_double(a)));
       }));
@@ -638,7 +628,6 @@ __device__ Eigen::half GpuAtomicCasHelper(Eigen::half* ptr, F accumulate) {
 #if defined(__BYTE_ORDER__) && defined(__ORDER_LITTLE_ENDIAN__)
   static_assert(__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__, "Not little endian");
 #endif
-  namespace half_impl = Eigen::half_impl;
   intptr_t intptr = reinterpret_cast<intptr_t>(ptr);
   assert(!(intptr & 0x1));  // should be 2-aligned.
   if (intptr & 0x2) {
@@ -646,22 +635,38 @@ __device__ Eigen::half GpuAtomicCasHelper(Eigen::half* ptr, F accumulate) {
     uint32* address = reinterpret_cast<uint32*>(intptr - 2);
     uint32 result = GpuAtomicCasHelper(address, [accumulate](uint32 arg) {
       unsigned short high = static_cast<unsigned short>(arg >> 16);
-      Eigen::half acc = accumulate(half_impl::raw_uint16_to_half(high));
-      return (static_cast<uint32>(acc.x) << 16) | (arg & 0xffff);
+      Eigen::half acc = accumulate(Eigen::numext::bit_cast<Eigen::half>(high));
+      return (static_cast<uint32>(Eigen::numext::bit_cast<uint16>(acc)) << 16) |
+             (arg & 0xffff);
     });
-    return half_impl::raw_uint16_to_half(static_cast<uint16>(result >> 16));
+    return Eigen::numext::bit_cast<Eigen::half>(
+        static_cast<uint16>(result >> 16));
   } else {
     // The half is in the first part of the uint32 (lower 16 bits).
     uint32* address = reinterpret_cast<uint32*>(intptr);
     uint32 result = GpuAtomicCasHelper(address, [accumulate](uint32 arg) {
       unsigned short low = static_cast<unsigned short>(arg & 0xffff);
-      Eigen::half acc = accumulate(half_impl::raw_uint16_to_half(low));
-      return (arg & 0xffff0000) | static_cast<uint32>(acc.x);
+      Eigen::half acc = accumulate(Eigen::numext::bit_cast<Eigen::half>(low));
+      return (arg & 0xffff0000) |
+             static_cast<uint32>(Eigen::numext::bit_cast<uint16>(acc));
     });
-    return half_impl::raw_uint16_to_half(static_cast<uint16>(result & 0xffff));
+    return Eigen::numext::bit_cast<Eigen::half>(
+        static_cast<uint16>(result & 0xffff));
   }
 }
 
+template <typename F>
+__device__ Eigen::bfloat16 GpuAtomicCasHelper(Eigen::bfloat16* ptr,
+                                              F accumulate) {
+  Eigen::half ret = detail::GpuAtomicCasHelper(
+      reinterpret_cast<Eigen::half*>(ptr), [accumulate](Eigen::half a) {
+        Eigen::bfloat16 acc =
+            accumulate(Eigen::numext::bit_cast<Eigen::bfloat16>(a));
+        return Eigen::numext::bit_cast<Eigen::half>(acc);
+      });
+  return Eigen::numext::bit_cast<Eigen::bfloat16>(ret);
+}
+
 template <typename F>
 __device__ long long GpuAtomicCasHelper(long long* ptr, F accumulate) {
   return static_cast<long long>(
@@ -676,6 +681,38 @@ template <typename From, typename To>
 using ToTypeIfConvertible =
     typename std::enable_if<std::is_convertible<From, To>::value, To>::type;
 
+template <typename T>
+struct CudaSupportedTypeImpl {
+  using type = T;
+};
+
+template <>
+struct CudaSupportedTypeImpl<long long> {
+  using type = unsigned long long;
+};
+
+template <>
+struct CudaSupportedTypeImpl<unsigned long> {
+  using type =
+      typename std::conditional<sizeof(unsigned long) == sizeof(unsigned int),
+                                unsigned int, unsigned long long>::type;
+};
+
+template <>
+struct CudaSupportedTypeImpl<long> {
+  // This cast should be safe since module-2 addition should work fine. However,
+  // signed overflow is not handled correctly since it's undefined behavior.
+  using type = typename CudaSupportedTypeImpl<unsigned long>::type;
+};
+
+template <typename T>
+using CudaSupportedType = typename CudaSupportedTypeImpl<T>::type;
+
+template <typename T>
+__device__ CudaSupportedType<T>* ToCudaSupportedPtr(T* ptr) {
+  return reinterpret_cast<CudaSupportedType<T>*>(ptr);
+}
+
 }  // namespace detail
 
 // CUDA provides atomic ops, but not for all types.  We provide wrappers
@@ -683,13 +720,7 @@ using ToTypeIfConvertible =
 
 template <typename T, typename U>
 __device__ detail::ToTypeIfConvertible<U, T> GpuAtomicAdd(T* ptr, U value) {
-  return atomicAdd(ptr, value);
-}
-
-__device__ inline int64 GpuAtomicAdd(int64* ptr, int64 value) {
-  // This cast should be safe since module-2 addition should work fine. However,
-  // signed overflow is not handled correctly since it's undefined behavior.
-  return atomicAdd(reinterpret_cast<uint64*>(ptr), static_cast<uint64>(value));
+  return atomicAdd(detail::ToCudaSupportedPtr(ptr), value);
 }
 
 __device__ inline Eigen::half GpuAtomicAdd(Eigen::half* ptr,
@@ -698,6 +729,12 @@ __device__ inline Eigen::half GpuAtomicAdd(Eigen::half* ptr,
       ptr, [value](Eigen::half a) { return a + value; });
 }
 
+__device__ inline Eigen::bfloat16 GpuAtomicAdd(Eigen::bfloat16* ptr,
+                                               Eigen::bfloat16 value) {
+  return detail::GpuAtomicCasHelper(
+      ptr, [value](Eigen::bfloat16 a) { return a + value; });
+}
+
 #if (__CUDA_ARCH__ < 600) || TENSORFLOW_USE_ROCM
 __device__ inline double GpuAtomicAdd(double* ptr, double value) {
   return detail::GpuAtomicCasHelper(ptr,
@@ -712,8 +749,7 @@ __device__ inline double GpuAtomicAdd(double* ptr, double value) {
 // components individually. The operation as a whole is not atomic, but we can
 // safely treat the components independently for the purpose of accumulating.
 
-// ROCM TODO support GpuAtomicAdd for std::complex<>
-#if GOOGLE_CUDA
+#if GOOGLE_CUDA || TENSORFLOW_USE_ROCM
 __device__ inline std::complex<float> GpuAtomicAdd(std::complex<float>* ptr,
                                                    std::complex<float> value) {
   auto ptr_scalar = reinterpret_cast<float*>(ptr);
@@ -745,14 +781,13 @@ __device__ inline double GpuAtomicSub(double* ptr, double value) {
   return GpuAtomicAdd(ptr, -value);
 }
 
-__device__ inline tensorflow::int64 GpuAtomicSub(tensorflow::int64* ptr,
-                                                 tensorflow::int64 value) {
+__device__ inline int64_t GpuAtomicSub(int64_t* ptr, int64_t value) {
   return GpuAtomicAdd(ptr, -value);
 }
 
 __device__ inline tensorflow::uint64 GpuAtomicSub(tensorflow::uint64* ptr,
                                                   tensorflow::uint64 value) {
-  return GpuAtomicAdd(ptr, -static_cast<tensorflow::int64>(value));
+  return GpuAtomicAdd(ptr, -static_cast<int64_t>(value));
 }
 
 __device__ inline Eigen::half GpuAtomicSub(Eigen::half* ptr,
@@ -760,12 +795,19 @@ __device__ inline Eigen::half GpuAtomicSub(Eigen::half* ptr,
   return detail::GpuAtomicCasHelper(
       ptr, [value](Eigen::half a) { return a - value; });
 }
+
+__device__ inline Eigen::bfloat16 GpuAtomicSub(Eigen::bfloat16* ptr,
+                                               Eigen::bfloat16 value) {
+  return detail::GpuAtomicCasHelper(
+      ptr, [value](Eigen::bfloat16 a) { return a - value; });
+}
+
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuAtomicSub, CudaAtomicSub);
 
 // GpuAtomicMax
 template <typename T, typename U>
 __device__ detail::ToTypeIfConvertible<U, T> GpuAtomicMax(T* ptr, U value) {
-  return atomicMax(ptr, value);
+  return atomicMax(detail::ToCudaSupportedPtr(ptr), value);
 }
 
 #if TENSORFLOW_USE_ROCM
@@ -793,11 +835,6 @@ __device__ inline double GpuAtomicMax(double* ptr, double value) {
       ptr, [value](double a) { return fmax(a, value); });
 }
 
-__device__ inline long long GpuAtomicMax(long long* ptr, long long value) {
-  return detail::GpuAtomicCasHelper(
-      ptr, [value](long long a) { return max(a, value); });
-}
-
 #else
 
 __device__ inline float GpuAtomicMax(float* ptr, float value) {
@@ -818,16 +855,24 @@ __device__ inline Eigen::half GpuAtomicMax(Eigen::half* ptr,
       ptr, [value](Eigen::half a) { return max(a, value); });
 }
 
-#if __CUDA_ARCH__ < 320
+__device__ inline Eigen::bfloat16 GpuAtomicMax(Eigen::bfloat16* ptr,
+                                               Eigen::bfloat16 value) {
+  return detail::GpuAtomicCasHelper(
+      ptr, [value](Eigen::bfloat16 a) { return max(a, value); });
+}
+
+#if TENSORFLOW_USE_ROCM || (__CUDA_ARCH__ < 320)
 __device__ inline tensorflow::uint64 GpuAtomicMax(tensorflow::uint64* ptr,
                                                   tensorflow::uint64 value) {
   return detail::GpuAtomicCasHelper(
-      ptr, [value](tensorflow::uint64 a) { return max(a, value); });
+      detail::ToCudaSupportedPtr(ptr),
+      [value](tensorflow::uint64 a) { return max(a, value); });
 }
 
-__device__ inline int64 GpuAtomicMax(int64* ptr, int64 value) {
-  return detail::GpuAtomicCasHelper(ptr,
-                                    [value](int64 a) { return max(a, value); });
+__device__ inline int64_t GpuAtomicMax(int64_t* ptr, int64_t value) {
+  return detail::GpuAtomicCasHelper(
+      detail::ToCudaSupportedPtr(ptr),
+      [value](int64_t a) { return max(a, value); });
 }
 #endif
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuAtomicMax, CudaAtomicMax);
@@ -835,7 +880,7 @@ CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuAtomicMax, CudaAtomicMax);
 // GpuAtomicMin
 template <typename T, typename U>
 __device__ detail::ToTypeIfConvertible<U, T> GpuAtomicMin(T* ptr, U value) {
-  return atomicMin(ptr, value);
+  return atomicMin(detail::ToCudaSupportedPtr(ptr), value);
 }
 
 #if TENSORFLOW_USE_ROCM
@@ -863,11 +908,6 @@ __device__ inline double GpuAtomicMin(double* ptr, double value) {
       ptr, [value](double a) { return fmin(a, value); });
 }
 
-__device__ inline long long GpuAtomicMin(long long* ptr, long long value) {
-  return detail::GpuAtomicCasHelper(
-      ptr, [value](long long a) { return min(a, value); });
-}
-
 #else
 
 __device__ inline float GpuAtomicMin(float* ptr, float value) {
@@ -888,16 +928,24 @@ __device__ inline Eigen::half GpuAtomicMin(Eigen::half* ptr,
       ptr, [value](Eigen::half a) { return min(a, value); });
 }
 
-#if __CUDA_ARCH__ < 320
+__device__ inline Eigen::bfloat16 GpuAtomicMin(Eigen::bfloat16* ptr,
+                                               Eigen::bfloat16 value) {
+  return detail::GpuAtomicCasHelper(
+      ptr, [value](Eigen::bfloat16 a) { return min(a, value); });
+}
+
+#if TENSORFLOW_USE_ROCM || (__CUDA_ARCH__ < 320)
 __device__ inline tensorflow::uint64 GpuAtomicMin(tensorflow::uint64* ptr,
                                                   tensorflow::uint64 value) {
   return detail::GpuAtomicCasHelper(
-      ptr, [value](tensorflow::uint64 a) { return min(a, value); });
+      detail::ToCudaSupportedPtr(ptr),
+      [value](tensorflow::uint64 a) { return min(a, value); });
 }
 
-__device__ inline int64 GpuAtomicMin(int64* ptr, int64 value) {
-  return detail::GpuAtomicCasHelper(ptr,
-                                    [value](int64 a) { return min(a, value); });
+__device__ inline int64_t GpuAtomicMin(int64_t* ptr, int64_t value) {
+  return detail::GpuAtomicCasHelper(
+      detail::ToCudaSupportedPtr(ptr),
+      [value](int64_t a) { return min(a, value); });
 }
 #endif
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuAtomicMin, CudaAtomicMin);
@@ -916,66 +964,17 @@ __device__ detail::ToTypeIfConvertible<U, T> GpuAtomicDiv(T* ptr, U value) {
 }
 CREATE_CUDA_DEVICE_FUNCTION_ALIAS(GpuAtomicDiv, CudaAtomicDiv);
 
-// Operator overloads for complex numbers.
-#if GOOGLE_CUDA
-__device__ inline std::complex<float> operator+(const std::complex<float>& a,
-                                                const std::complex<float>& b) {
-  auto result = cuCaddf(make_cuComplex(a.real(), a.imag()),
-                        make_cuComplex(b.real(), b.imag()));
-  return std::complex<float>(result.x, result.y);
-}
-
-__device__ inline std::complex<float> operator-(const std::complex<float>& a,
-                                                const std::complex<float>& b) {
-  auto result = cuCsubf(make_cuComplex(a.real(), a.imag()),
-                        make_cuComplex(b.real(), b.imag()));
-  return std::complex<float>(result.x, result.y);
-}
-
-__device__ inline std::complex<float> operator*(const std::complex<float>& a,
-                                                const std::complex<float>& b) {
-  auto result = cuCmulf(make_cuComplex(a.real(), a.imag()),
-                        make_cuComplex(b.real(), b.imag()));
-  return std::complex<float>(result.x, result.y);
-}
-
-__device__ inline std::complex<float> operator/(const std::complex<float>& a,
-                                                const std::complex<float>& b) {
-  auto result = cuCdivf(make_cuComplex(a.real(), a.imag()),
-                        make_cuComplex(b.real(), b.imag()));
-  return std::complex<float>(result.x, result.y);
-}
-
-__device__ inline std::complex<double> operator+(
-    const std::complex<double>& a, const std::complex<double>& b) {
-  auto result = cuCadd(make_cuDoubleComplex(a.real(), a.imag()),
-                       make_cuDoubleComplex(b.real(), b.imag()));
-  return std::complex<double>(result.x, result.y);
-}
-
-__device__ inline std::complex<double> operator-(
-    const std::complex<double>& a, const std::complex<double>& b) {
-  auto result = cuCsub(make_cuDoubleComplex(a.real(), a.imag()),
-                       make_cuDoubleComplex(b.real(), b.imag()));
-  return std::complex<double>(result.x, result.y);
-}
-
-__device__ inline std::complex<double> operator*(
-    const std::complex<double>& a, const std::complex<double>& b) {
-  auto result = cuCmul(make_cuDoubleComplex(a.real(), a.imag()),
-                       make_cuDoubleComplex(b.real(), b.imag()));
-  return std::complex<double>(result.x, result.y);
-}
-
-__device__ inline std::complex<double> operator/(
-    const std::complex<double>& a, const std::complex<double>& b) {
-  auto result = cuCdiv(make_cuDoubleComplex(a.real(), a.imag()),
-                       make_cuDoubleComplex(b.real(), b.imag()));
-  return std::complex<double>(result.x, result.y);
-}
+// Import all specialized std::complex device operators in namespace tensorflow.
+#if GOOGLE_CUDA && defined(EIGEN_USING_STD_COMPLEX_OPERATORS)
+EIGEN_USING_STD_COMPLEX_OPERATORS
 #endif  // GOOGLE_CUDA
 
 namespace functor {
+// Import all specialized std::complex device operators in namespace functor.
+#if GOOGLE_CUDA && defined(EIGEN_USING_STD_COMPLEX_OPERATORS)
+EIGEN_USING_STD_COMPLEX_OPERATORS
+#endif  // GOOGLE_CUDA
+
 // ROCm hcc(clang) has severe difficulties dealing with std::complex directly
 // due to a header issue. This template assists in casting std::complex into the
 // corresponding internal ROCm types.
