@@ -13,10 +13,6 @@
 # limitations under the License.
 # ==============================================================================
 """Cuda op Python library."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os.path
 
 import tensorflow as tf
