@@ -15,19 +15,23 @@ limitations under the License.
 
 #include "tensorflow/lite/delegates/xnnpack/binary_elementwise_tester.h"
 
+#include <algorithm>
 #include <array>
 #include <cstdint>
 #include <functional>
+#include <memory>
 #include <numeric>
 #include <random>
 #include <vector>
 
 #include <gtest/gtest.h>
-#include <fp16.h>
+#include "fp16.h"  // from @FP16
 #include "flatbuffers/flatbuffers.h"  // from @flatbuffers
+#include "tensorflow/lite/core/kernels/register.h"
+#include "tensorflow/lite/core/model.h"
+#include "tensorflow/lite/delegates/xnnpack/test_util.h"
 #include "tensorflow/lite/interpreter.h"
-#include "tensorflow/lite/kernels/register.h"
-#include "tensorflow/lite/model.h"
+#include "tensorflow/lite/schema/schema_conversion_utils.h"
 #include "tensorflow/lite/schema/schema_generated.h"
 #include "tensorflow/lite/version.h"
 
@@ -63,8 +67,14 @@ void BinaryElementwiseTester::Test(tflite::BuiltinOperator binary_op,
   if (Input1Static()) {
     ASSERT_FALSE(Input2Static());
   }
-  if (FP16Weights()) {
+  if (FP16Weights() || INT8Weights() || INT8ChannelWiseWeights()) {
     ASSERT_TRUE(Input1Static() || Input2Static());
+    if (INT8ChannelWiseWeights() && Input1Static()) {
+      ASSERT_FALSE(Input1Shape().empty());
+    }
+    if (INT8ChannelWiseWeights() && Input2Static()) {
+      ASSERT_FALSE(Input2Shape().empty());
+    }
   }
 
   std::random_device random_device;
@@ -91,12 +101,16 @@ void BinaryElementwiseTester::Test(tflite::BuiltinOperator binary_op,
 
   std::unique_ptr<Interpreter> delegate_interpreter;
   ASSERT_EQ(
-      InterpreterBuilder(model, ::tflite::ops::builtin::BuiltinOpResolver())(
+      InterpreterBuilder(
+          model,
+          ::tflite::ops::builtin::BuiltinOpResolverWithoutDefaultDelegates())(
           &delegate_interpreter),
       kTfLiteOk);
   std::unique_ptr<Interpreter> default_interpreter;
   ASSERT_EQ(
-      InterpreterBuilder(model, ::tflite::ops::builtin::BuiltinOpResolver())(
+      InterpreterBuilder(
+          model,
+          ::tflite::ops::builtin::BuiltinOpResolverWithoutDefaultDelegates())(
           &default_interpreter),
       kTfLiteOk);
 
@@ -120,40 +134,36 @@ void BinaryElementwiseTester::Test(tflite::BuiltinOperator binary_op,
   ASSERT_EQ(delegate_interpreter->ModifyGraphWithDelegate(delegate), kTfLiteOk);
 
   if (!Input1Static()) {
-    float* default_input1_data = default_interpreter->typed_tensor<float>(
-        default_interpreter->inputs()[0]);
-    std::generate(default_input1_data,
-                  default_input1_data + ComputeSize(Input1Shape()),
-                  std::ref(input1_rng));
+    float* default_input1_data =
+        default_interpreter->typed_input_tensor<float>(0);
+    std::generate_n(default_input1_data, ComputeSize(Input1Shape()),
+                    std::ref(input1_rng));
 
-    float* xnnpack_input1_data = delegate_interpreter->typed_tensor<float>(
-        delegate_interpreter->inputs()[0]);
-    std::copy(default_input1_data,
-              default_input1_data + ComputeSize(Input1Shape()),
-              xnnpack_input1_data);
+    float* xnnpack_input1_data =
+        delegate_interpreter->typed_input_tensor<float>(0);
+    std::copy_n(default_input1_data, ComputeSize(Input1Shape()),
+                xnnpack_input1_data);
   }
 
   if (!Input2Static()) {
-    float* default_input2_data = default_interpreter->typed_tensor<float>(
-        default_interpreter->inputs()[Input1Static() ? 0 : 1]);
-    std::generate(default_input2_data,
-                  default_input2_data + ComputeSize(Input2Shape()),
-                  std::ref(input2_rng));
+    float* default_input2_data =
+        default_interpreter->typed_input_tensor<float>(Input1Static() ? 0 : 1);
+    std::generate_n(default_input2_data, ComputeSize(Input2Shape()),
+                    std::ref(input2_rng));
 
-    float* xnnpack_input2_data = delegate_interpreter->typed_tensor<float>(
-        delegate_interpreter->inputs()[Input1Static() ? 0 : 1]);
-    std::copy(default_input2_data,
-              default_input2_data + ComputeSize(Input2Shape()),
-              xnnpack_input2_data);
+    float* xnnpack_input2_data =
+        delegate_interpreter->typed_input_tensor<float>(Input1Static() ? 0 : 1);
+    std::copy_n(default_input2_data, ComputeSize(Input2Shape()),
+                xnnpack_input2_data);
   }
 
   ASSERT_EQ(default_interpreter->Invoke(), kTfLiteOk);
   ASSERT_EQ(delegate_interpreter->Invoke(), kTfLiteOk);
 
-  float* default_output_data = default_interpreter->typed_tensor<float>(
-      default_interpreter->outputs()[0]);
-  float* xnnpack_output_data = delegate_interpreter->typed_tensor<float>(
-      delegate_interpreter->outputs()[0]);
+  float* default_output_data =
+      default_interpreter->typed_output_tensor<float>(0);
+  float* xnnpack_output_data =
+      delegate_interpreter->typed_output_tensor<float>(0);
 
   for (size_t i = 0; i < ComputeSize(OutputShape()); i++) {
     ASSERT_NEAR(default_output_data[i], xnnpack_output_data[i],
@@ -186,7 +196,7 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
   flatbuffers::FlatBufferBuilder builder;
   std::vector<flatbuffers::Offset<OperatorCode>> operator_codes{
       {CreateOperatorCode(builder, binary_op)}};
-  if (FP16Weights()) {
+  if (FP16Weights() || INT8Weights() || INT8ChannelWiseWeights()) {
     operator_codes.emplace_back(
         CreateOperatorCode(builder, BuiltinOperator_DEQUANTIZE));
   } else if (SparseWeights()) {
@@ -198,6 +208,9 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
       CreateBuffer(builder, builder.CreateVector({})),
   }};
 
+  std::vector<float> input1_scales;
+  std::vector<int64_t> input1_zero_points;
+  int32_t input1_quantized_dimension = 0;
   int32_t input1_buffer = 0;
   if (Input1Static()) {
     if (FP16Weights()) {
@@ -213,16 +226,51 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
       std::vector<float> input1_data(ComputeSize(Input1Shape()));
       std::generate(input1_data.begin(), input1_data.end(), input1_rng);
 
-      if (!SparseWeights()) {
-        input1_buffer = buffers.size();
+      if (INT8Weights()) {
+        std::vector<int8_t> quantized_input1_data(input1_data.size());
+        input1_scales.resize(1, GetInt8QuantizationScale(input1_data));
+        input1_zero_points.resize(1, 0);
+        std::transform(input1_data.begin(), input1_data.end(),
+                       quantized_input1_data.begin(),
+                       std::bind(QuantizeInt8, std::placeholders::_1, 0,
+                                 input1_scales[0]));
+        buffers.push_back(CreateBuffer(
+            builder,
+            builder.CreateVector(
+                reinterpret_cast<const uint8_t*>(quantized_input1_data.data()),
+                sizeof(int8_t) * quantized_input1_data.size())));
+      } else if (INT8ChannelWiseWeights()) {
+        std::vector<int8_t> quantized_input1_data(input1_data.size());
+        input1_quantized_dimension =
+            static_cast<int32_t>(Input1Shape().size()) - 1;
+        const int32_t num_scales = Input1Shape()[input1_quantized_dimension];
+        input1_scales = GetInt8QuantizationScalePerChannel(
+            input1_data.data(), input1_quantized_dimension, Input1Shape());
+        input1_zero_points.resize(num_scales, 0);
+        QuantizeInt8PerChannel(input1_scales.data(), input1_zero_points.data(),
+                               input1_quantized_dimension, input1_data.data(),
+                               quantized_input1_data.data(), Input1Shape());
+        buffers.push_back(CreateBuffer(
+            builder,
+            builder.CreateVector(
+                reinterpret_cast<const uint8_t*>(quantized_input1_data.data()),
+                sizeof(int8_t) * quantized_input1_data.size())));
+      } else {
+        if (!SparseWeights()) {
+          input1_buffer = buffers.size();
+        }
+
+        buffers.push_back(CreateBuffer(
+            builder, builder.CreateVector(
+                         reinterpret_cast<const uint8_t*>(input1_data.data()),
+                         sizeof(float) * input1_data.size())));
       }
-      buffers.push_back(CreateBuffer(
-          builder, builder.CreateVector(
-                       reinterpret_cast<const uint8_t*>(input1_data.data()),
-                       sizeof(float) * input1_data.size())));
     }
   }
 
+  std::vector<float> input2_scales;
+  std::vector<int64_t> input2_zero_points;
+  int32_t input2_quantized_dimension = 0;
   int32_t input2_buffer = 0;
   if (Input2Static()) {
     if (FP16Weights()) {
@@ -238,13 +286,45 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
       std::vector<float> input2_data(ComputeSize(Input2Shape()));
       std::generate(input2_data.begin(), input2_data.end(), input2_rng);
 
-      if (!SparseWeights()) {
-        input2_buffer = buffers.size();
+      if (INT8Weights()) {
+        std::vector<int8_t> quantized_input2_data(input2_data.size());
+        input2_scales.resize(1, GetInt8QuantizationScale(input2_data));
+        input2_zero_points.resize(1, 0);
+        std::transform(input2_data.begin(), input2_data.end(),
+                       quantized_input2_data.begin(),
+                       std::bind(QuantizeInt8, std::placeholders::_1, 0,
+                                 input2_scales[0]));
+        buffers.push_back(CreateBuffer(
+            builder,
+            builder.CreateVector(
+                reinterpret_cast<const uint8_t*>(quantized_input2_data.data()),
+                sizeof(int8_t) * quantized_input2_data.size())));
+      } else if (INT8ChannelWiseWeights()) {
+        std::vector<int8_t> quantized_input2_data(input2_data.size());
+        input2_quantized_dimension =
+            static_cast<int32_t>(Input2Shape().size()) - 1;
+        const int32_t num_scales = Input1Shape()[input2_quantized_dimension];
+        input2_scales = GetInt8QuantizationScalePerChannel(
+            input2_data.data(), input2_quantized_dimension, Input2Shape());
+        input2_zero_points.resize(num_scales, 0);
+        QuantizeInt8PerChannel(input2_scales.data(), input2_zero_points.data(),
+                               input2_quantized_dimension, input2_data.data(),
+                               quantized_input2_data.data(), Input2Shape());
+        buffers.push_back(CreateBuffer(
+            builder,
+            builder.CreateVector(
+                reinterpret_cast<const uint8_t*>(quantized_input2_data.data()),
+                sizeof(int8_t) * quantized_input2_data.size())));
+      } else {
+        if (!SparseWeights()) {
+          input2_buffer = buffers.size();
+        }
+
+        buffers.push_back(CreateBuffer(
+            builder, builder.CreateVector(
+                         reinterpret_cast<const uint8_t*>(input2_data.data()),
+                         sizeof(float) * input2_data.size())));
       }
-      buffers.push_back(CreateBuffer(
-          builder, builder.CreateVector(
-                       reinterpret_cast<const uint8_t*>(input2_data.data()),
-                       sizeof(float) * input2_data.size())));
     }
   }
 
@@ -257,6 +337,18 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
                      builder.CreateVector<int32_t>(Input1Shape().data(),
                                                    Input1Shape().size()),
                      TensorType_FLOAT16, 1));
+  } else if ((INT8Weights() || INT8ChannelWiseWeights()) && Input1Static()) {
+    tensors.emplace_back(
+        CreateTensor(builder,
+                     builder.CreateVector<int32_t>(Input1Shape().data(),
+                                                   Input1Shape().size()),
+                     TensorType_INT8, 1, 0,
+                     CreateQuantizationParameters(
+                         builder, /*min=*/0, /*max=*/0,
+                         builder.CreateVector<float>(input1_scales),
+                         builder.CreateVector<int64_t>(input1_zero_points),
+                         /*details_type=*/QuantizationDetails_NONE,
+                         /*details=*/0, input1_quantized_dimension)));
   } else if (SparseWeights() && Input1Static()) {
     int dims_count = Input1Shape().size();
     std::vector<flatbuffers::Offset<DimensionMetadata>> dim_metadata(
@@ -283,6 +375,17 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
                      builder.CreateVector<int32_t>(Input2Shape().data(),
                                                    Input2Shape().size()),
                      TensorType_FLOAT16, 1));
+  } else if ((INT8Weights() || INT8ChannelWiseWeights()) && Input2Static()) {
+    tensors.emplace_back(CreateTensor(
+        builder,
+        builder.CreateVector<int32_t>(Input2Shape().data(),
+                                      Input2Shape().size()),
+        TensorType_INT8, 1, 0,
+        CreateQuantizationParameters(
+            builder, /*min=*/0, /*max=*/0,
+            builder.CreateVector<float>(input2_scales),
+            builder.CreateVector<int64_t>(input2_zero_points),
+            QuantizationDetails_NONE, 0, input2_quantized_dimension)));
   } else if (SparseWeights() && Input2Static()) {
     int dims_count = Input2Shape().size();
     std::vector<flatbuffers::Offset<DimensionMetadata>> dim_metadata(
@@ -303,7 +406,7 @@ std::vector<char> BinaryElementwiseTester::CreateTfLiteModel(
         TensorType_FLOAT32, /*buffer=*/1, /*name=*/0, /*quantization=*/0,
         /*is_variable=*/false, /*sparsity=*/sparsity_param));
   }
-  if (FP16Weights()) {
+  if (FP16Weights() || INT8Weights() || INT8ChannelWiseWeights()) {
     const std::array<int32_t, 1> dequantize_inputs{{0}};
     const std::array<int32_t, 1> dequantize_outputs{{Input1Static() ? 1 : 2}};
     operators.emplace_back(CreateOperator(
