@@ -94,7 +94,9 @@ class ComparisonOpModel : public SingleOpModel {
                      CreateLessEqualOptions(builder_).Union());
         break;
       }
-      default: { FAIL() << "We shouldn't get here."; }
+      default: {
+        FAIL() << "We shouldn't get here.";
+      }
     }
   }
 };
@@ -104,7 +106,7 @@ TEST(ComparisonsTest, EqualBool) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<bool>(model.input1(), {true, false, true, false});
   model.PopulateTensor<bool>(model.input2(), {true, true, false, false});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -115,7 +117,7 @@ TEST(ComparisonsTest, EqualFloat) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<float>(model.input1(), {0.1, 0.9, 0.7, 0.3});
   model.PopulateTensor<float>(model.input2(), {0.1, 0.2, 0.6, 0.5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -126,7 +128,18 @@ TEST(ComparisonsTest, EqualInt) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, true, false));
+  EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
+}
+
+TEST(ComparisonsTest, EqualInt16) {
+  ComparisonOpModel model({1, 1, 1, 4}, {1, 1, 1, 4}, TensorType_INT16,
+                          BuiltinOperator_EQUAL);
+  model.PopulateTensor<int16_t>(model.input1(), {-1, 9, 7, 3});
+  model.PopulateTensor<int16_t>(model.input2(), {1, 2, 7, 5});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -140,7 +153,7 @@ TEST(ComparisonsTest, EqualString) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<std::string>(model.input1(), {"A", "B", "C", "D"});
   model.PopulateTensor<std::string>(model.input2(), {"A", "C", "B", "D"});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4, 1));
@@ -151,7 +164,7 @@ TEST(ComparisonsTest, EqualBroadcast) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {7});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -162,7 +175,7 @@ TEST(ComparisonsTest, EqualBroadcastTwoD) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3, 2, 4, 2, 8});
   model.PopulateTensor<int>(model.input2(), {7, 1, 2, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, false, false, false,
                                              false, true, false));
@@ -177,7 +190,7 @@ TEST(ComparisonsTest, EqualBroadcastString) {
                           BuiltinOperator_EQUAL);
   model.PopulateTensor<std::string>(model.input1(), {"A", "B", "A", "B"});
   model.PopulateTensor<std::string>(model.input2(), {"A"});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -188,7 +201,7 @@ TEST(ComparisonsTest, NotEqualBool) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<bool>(model.input1(), {true, false, true, false});
   model.PopulateTensor<bool>(model.input2(), {true, true, false, false});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -199,7 +212,7 @@ TEST(ComparisonsTest, NotEqualFloat) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<float>(model.input1(), {0.1, 0.9, 0.7, 0.3});
   model.PopulateTensor<float>(model.input2(), {0.1, 0.2, 0.6, 0.5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -210,7 +223,7 @@ TEST(ComparisonsTest, NotEqualInt) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, true, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -224,7 +237,7 @@ TEST(ComparisonsTest, NotEqualString) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<std::string>(model.input1(), {"A", "B", "C", "D"});
   model.PopulateTensor<std::string>(model.input2(), {"A", "C", "B", "D"});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 1, 4));
@@ -235,7 +248,7 @@ TEST(ComparisonsTest, NotEqualBroadcast) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {7});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, true, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -246,7 +259,7 @@ TEST(ComparisonsTest, NotEqualBroadcastTwoD) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3, 2, 4, 2, 8});
   model.PopulateTensor<int>(model.input2(), {7, 1, 2, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAre(true, true, true, true, true, true, false, true));
@@ -261,7 +274,7 @@ TEST(ComparisonsTest, NotEqualBroadcastString) {
                           BuiltinOperator_NOT_EQUAL);
   model.PopulateTensor<std::string>(model.input1(), {"A", "B", "A", "B"});
   model.PopulateTensor<std::string>(model.input2(), {"A"});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -272,7 +285,7 @@ TEST(ComparisonsTest, GreaterFloat) {
                           BuiltinOperator_GREATER);
   model.PopulateTensor<float>(model.input1(), {0.1, 0.9, 0.7, 0.3});
   model.PopulateTensor<float>(model.input2(), {0.1, 0.2, 0.6, 0.5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -283,7 +296,7 @@ TEST(ComparisonsTest, GreaterInt) {
                           BuiltinOperator_GREATER);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, false, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -294,7 +307,7 @@ TEST(ComparisonsTest, GreaterBroadcast) {
                           BuiltinOperator_GREATER);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {7});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, false, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -305,7 +318,7 @@ TEST(ComparisonsTest, GreaterBroadcastTwoD) {
                           BuiltinOperator_GREATER);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3, 2, 4, 2, 8});
   model.PopulateTensor<int>(model.input2(), {7, 1, 2, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAre(false, true, true, false, false, true, false, true));
@@ -317,7 +330,7 @@ TEST(ComparisonsTest, GreaterEqualFloat) {
                           BuiltinOperator_GREATER_EQUAL);
   model.PopulateTensor<float>(model.input1(), {0.1, 0.9, 0.7, 0.3});
   model.PopulateTensor<float>(model.input2(), {0.1, 0.2, 0.6, 0.5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, true, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -328,7 +341,18 @@ TEST(ComparisonsTest, GreaterEqualInt) {
                           BuiltinOperator_GREATER_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
+  EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
+}
+
+TEST(ComparisonsTest, GreaterEqualInt16) {
+  ComparisonOpModel model({1, 1, 1, 4}, {1, 1, 1, 4}, TensorType_INT16,
+                          BuiltinOperator_GREATER_EQUAL);
+  model.PopulateTensor<int16_t>(model.input1(), {-1, 9, 7, 3});
+  model.PopulateTensor<int16_t>(model.input2(), {1, 2, 7, 5});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -339,7 +363,7 @@ TEST(ComparisonsTest, GreaterEqualBroadcast) {
                           BuiltinOperator_GREATER_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {7});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -350,20 +374,19 @@ TEST(ComparisonsTest, GreaterEqualBroadcastTwoD) {
                           BuiltinOperator_GREATER_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3, 2, 4, 2, 8});
   model.PopulateTensor<int>(model.input2(), {7, 1, 2, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAre(false, true, true, false, false, true, true, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 2, 4));
 }
 
-
 TEST(ComparisonsTest, LessFloat) {
   ComparisonOpModel model({1, 1, 1, 4}, {1, 1, 1, 4}, TensorType_FLOAT32,
                           BuiltinOperator_LESS);
   model.PopulateTensor<float>(model.input1(), {0.1, 0.9, 0.7, 0.3});
   model.PopulateTensor<float>(model.input2(), {0.1, 0.2, 0.6, 0.5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -374,7 +397,18 @@ TEST(ComparisonsTest, LessInt) {
                           BuiltinOperator_LESS);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {1, 2, 6, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
+  EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
+}
+
+TEST(ComparisonsTest, LessInt16) {
+  ComparisonOpModel model({1, 1, 1, 4}, {1, 1, 1, 4}, TensorType_INT16,
+                          BuiltinOperator_LESS);
+  model.PopulateTensor<int16_t>(model.input1(), {-1, 9, 7, 3});
+  model.PopulateTensor<int16_t>(model.input2(), {1, 2, 6, 5});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -385,7 +419,7 @@ TEST(ComparisonsTest, LessBroadcast) {
                           BuiltinOperator_LESS);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {7});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -396,7 +430,7 @@ TEST(ComparisonsTest, LessBroadcastTwoD) {
                           BuiltinOperator_LESS);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3, 2, 4, 6, 8});
   model.PopulateTensor<int>(model.input2(), {7, 1, 2, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAre(true, false, false, true, true, false, false, false));
@@ -408,7 +442,7 @@ TEST(ComparisonsTest, LessEqualFloat) {
                           BuiltinOperator_LESS_EQUAL);
   model.PopulateTensor<float>(model.input1(), {0.1, 0.9, 0.7, 0.3});
   model.PopulateTensor<float>(model.input2(), {0.1, 0.2, 0.6, 0.5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -419,7 +453,7 @@ TEST(ComparisonsTest, LessEqualInt) {
                           BuiltinOperator_LESS_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, true, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -430,7 +464,7 @@ TEST(ComparisonsTest, LessEqualBroadcast) {
                           BuiltinOperator_LESS_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3});
   model.PopulateTensor<int>(model.input2(), {7});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, true, true));
   EXPECT_THAT(model.GetOutputShape(), ElementsAre(1, 1, 1, 4));
@@ -441,7 +475,7 @@ TEST(ComparisonsTest, LessEqualBroadcastTwoD) {
                           BuiltinOperator_LESS_EQUAL);
   model.PopulateTensor<int>(model.input1(), {-1, 9, 7, 3, 2, 4, 2, 8});
   model.PopulateTensor<int>(model.input2(), {7, 1, 2, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAre(true, false, false, true, true, false, true, false));
@@ -456,7 +490,7 @@ TEST(QuantizedComparisonsTest, EqualUInt8Quantized) {
                           TensorType_UINT8, BuiltinOperator_EQUAL);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1, 9, 7, 3});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, true, false));
 }
@@ -469,7 +503,7 @@ TEST(QuantizedComparisonsTest, EqualInt8Quantized) {
                           TensorType_INT8, BuiltinOperator_EQUAL);
   model.QuantizeAndPopulate<int8_t>(model.input1(), {1, -9, 7, 3});
   model.QuantizeAndPopulate<int8_t>(model.input2(), {-1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, true, false));
 }
@@ -482,7 +516,7 @@ TEST(QuantizedComparisonsTest, NotEqualUInt8Quantized) {
                           TensorType_UINT8, BuiltinOperator_NOT_EQUAL);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1, 9, 7, 3});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1, 2, 7, 0});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, false, true));
 }
@@ -495,7 +529,7 @@ TEST(QuantizedComparisonsTest, NotEqualInt8Quantized) {
                           TensorType_INT8, BuiltinOperator_NOT_EQUAL);
   model.QuantizeAndPopulate<int8_t>(model.input1(), {1, -9, 7, 3});
   model.QuantizeAndPopulate<int8_t>(model.input2(), {1, 2, 7, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, false, true));
 }
@@ -508,7 +542,7 @@ TEST(ComparisonsTest, GreaterQuantized) {
                           TensorType_UINT8, BuiltinOperator_GREATER);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1, 9, 7, 3});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1, 2, 6, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
 }
@@ -519,7 +553,7 @@ TEST(ComparisonsTest, GreaterQuantizedSmallRange) {
                           TensorType_UINT8, BuiltinOperator_GREATER);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1.0, 0.5, 0.35, 0.1});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1.01, 0.25, 0.3, 0.4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, true, true, false));
 }
@@ -532,7 +566,7 @@ TEST(ComparisonsTest, GreaterEqualQuantized) {
                           TensorType_UINT8, BuiltinOperator_GREATER_EQUAL);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1, 9, 7, 3});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1, 2, 6, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, true, true, false));
 }
@@ -545,7 +579,7 @@ TEST(ComparisonsTest, LessQuantized) {
                           TensorType_UINT8, BuiltinOperator_LESS);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1, 9, 7, 3});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1, 2, 6, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(false, false, false, true));
 }
@@ -558,7 +592,7 @@ TEST(ComparisonsTest, LessEqualQuantized) {
                           TensorType_UINT8, BuiltinOperator_LESS_EQUAL);
   model.QuantizeAndPopulate<uint8_t>(model.input1(), {1, 9, 7, 3});
   model.QuantizeAndPopulate<uint8_t>(model.input2(), {1, 2, 6, 5});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(), ElementsAre(true, false, false, true));
 }
@@ -574,7 +608,7 @@ TEST(ComparisonsTest, QuantizedEqualWithBroadcast) {
                             TensorType_UINT8, BuiltinOperator_EQUAL);
     model.QuantizeAndPopulate<uint8_t>(model.input1(), {20, 2, 7, 8, 11, 20});
     model.QuantizeAndPopulate<uint8_t>(model.input2(), {2});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(false, true, false, false, false, false))
         << "With shape number " << i;
@@ -592,7 +626,7 @@ TEST(ComparisonsTest, QuantizedUInt8NotEqualWithBroadcast) {
                             TensorType_UINT8, BuiltinOperator_NOT_EQUAL);
     model.QuantizeAndPopulate<uint8_t>(model.input1(), {20, 2, 7, 8, 11, 20});
     model.QuantizeAndPopulate<uint8_t>(model.input2(), {2});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(true, false, true, true, true, true))
         << "With shape number " << i;
@@ -610,7 +644,7 @@ TEST(ComparisonsTest, QuantizedInt8NotEqualWithBroadcast) {
                             BuiltinOperator_NOT_EQUAL);
     model.QuantizeAndPopulate<int8_t>(model.input1(), {-20, 2, 7, -8, 11, 20});
     model.QuantizeAndPopulate<int8_t>(model.input2(), {2});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(true, false, true, true, true, true))
         << "With shape number " << i;
@@ -628,7 +662,7 @@ TEST(ComparisonsTest, QuantizedUInt8GreaterWithBroadcast) {
                             TensorType_UINT8, BuiltinOperator_GREATER);
     model.QuantizeAndPopulate<uint8_t>(model.input1(), {20, 2, 7, 8, 11, 20});
     model.QuantizeAndPopulate<uint8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(true, false, false, false, true, true))
         << "With shape number " << i;
@@ -646,7 +680,27 @@ TEST(ComparisonsTest, QuantizedInt8GreaterWithBroadcast) {
                             BuiltinOperator_GREATER);
     model.QuantizeAndPopulate<int8_t>(model.input1(), {20, -2, -71, 8, 11, 20});
     model.QuantizeAndPopulate<int8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
+    EXPECT_THAT(model.GetOutput(),
+                ElementsAre(true, false, false, false, true, true))
+        << "With shape number " << i;
+  }
+}
+
+TEST(ComparisonsTest,
+     QuantizedInt8GreaterWithBroadcastMultiplierGreaterThanOne) {
+  const float kMin = -127.f;
+  const float kMax = 127.f;
+  std::vector<std::vector<int>> test_shapes = {
+      {6}, {2, 3}, {2, 1, 3}, {1, 3, 1, 2}};
+  for (int i = 0; i < test_shapes.size(); ++i) {
+    ComparisonOpModel model({TensorType_INT8, test_shapes[i], kMin, kMax},
+                            {TensorType_INT8, {}, kMin, kMax}, TensorType_INT8,
+                            BuiltinOperator_GREATER);
+    model.QuantizeAndPopulate<int8_t>(model.input1(),
+                                      {572, -2, -71, 8, 11, 20});
+    model.QuantizeAndPopulate<int8_t>(model.input2(), {8});
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(true, false, false, false, true, true))
         << "With shape number " << i;
@@ -664,7 +718,7 @@ TEST(ComparisonsTest, QuantizedUInt8GreaterEqualWithBroadcast) {
                             TensorType_UINT8, BuiltinOperator_GREATER_EQUAL);
     model.QuantizeAndPopulate<uint8_t>(model.input1(), {20, 2, 7, 8, 11, 20});
     model.QuantizeAndPopulate<uint8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(true, false, false, true, true, true))
         << "With shape number " << i;
@@ -682,7 +736,7 @@ TEST(ComparisonsTest, QuantizedInt8GreaterEqualWithBroadcast) {
                             BuiltinOperator_GREATER_EQUAL);
     model.QuantizeAndPopulate<int8_t>(model.input1(), {20, -2, -71, 8, 11, 20});
     model.QuantizeAndPopulate<int8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(true, false, false, true, true, true))
         << "With shape number " << i;
@@ -700,7 +754,7 @@ TEST(ComparisonsTest, QuantizedUInt8LessWithBroadcast) {
                             TensorType_UINT8, BuiltinOperator_LESS);
     model.QuantizeAndPopulate<uint8_t>(model.input1(), {20, 2, 7, 8, 11, 20});
     model.QuantizeAndPopulate<uint8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(false, true, true, false, false, false))
         << "With shape number " << i;
@@ -718,7 +772,7 @@ TEST(ComparisonsTest, QuantizedInt8LessWithBroadcast) {
                             BuiltinOperator_LESS);
     model.QuantizeAndPopulate<int8_t>(model.input1(), {20, -2, -71, 8, 11, 20});
     model.QuantizeAndPopulate<int8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(false, true, true, false, false, false))
         << "With shape number " << i;
@@ -736,7 +790,7 @@ TEST(ComparisonsTest, QuantizedUInt8LessEqualWithBroadcast) {
                             TensorType_UINT8, BuiltinOperator_LESS_EQUAL);
     model.QuantizeAndPopulate<uint8_t>(model.input1(), {20, 2, 7, 8, 11, 20});
     model.QuantizeAndPopulate<uint8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(false, true, true, true, false, false))
         << "With shape number " << i;
@@ -754,7 +808,7 @@ TEST(ComparisonsTest, QuantizedInt8LessEqualWithBroadcast) {
                             BuiltinOperator_LESS_EQUAL);
     model.QuantizeAndPopulate<int8_t>(model.input1(), {20, -2, -71, 8, 11, 20});
     model.QuantizeAndPopulate<int8_t>(model.input2(), {8});
-    model.Invoke();
+    ASSERT_EQ(model.Invoke(), kTfLiteOk);
     EXPECT_THAT(model.GetOutput(),
                 ElementsAre(false, true, true, true, false, false))
         << "With shape number " << i;
