@@ -26,7 +26,7 @@ limitations under the License.
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "absl/memory/memory.h"
-#include "tensorflow/lite/interpreter.h"
+#include "tensorflow/lite/core/interpreter.h"
 #include "tensorflow/lite/kernels/test_util.h"
 #include "tensorflow/lite/schema/schema_generated.h"
 #include "tensorflow/lite/string_type.h"
@@ -60,6 +60,7 @@ class BaseTransposeConvOpModel : public SingleOpModel {
                            std::initializer_list<InputType> filter_data,
                            const TensorData& input, const TensorData& output,
                            Padding padding, int stride_w, int stride_h,
+                           tflite::ActivationFunctionType fused_activation,
                            TestType test_type, int version = 1) {
     // Just to be confusing, transpose_conv has an _input_ named "output_shape"
     // that sets the shape of the output tensor of the op :). It must always be
@@ -75,11 +76,12 @@ class BaseTransposeConvOpModel : public SingleOpModel {
 
     output_ = AddOutput(output);
 
-    SetBuiltinOp(
-        BuiltinOperator_TRANSPOSE_CONV, BuiltinOptions_TransposeConvOptions,
-        CreateTransposeConvOptions(builder_, padding, stride_w, stride_h)
-            .Union());
-    resolver_ = absl::make_unique<SingleOpResolver>(
+    SetBuiltinOp(BuiltinOperator_TRANSPOSE_CONV,
+                 BuiltinOptions_TransposeConvOptions,
+                 CreateTransposeConvOptions(builder_, padding, stride_w,
+                                            stride_h, fused_activation)
+                     .Union());
+    resolver_ = std::make_unique<SingleOpResolver>(
         BuiltinOperator_TRANSPOSE_CONV, registration, version);
     BuildInterpreter(
         {GetShape(output_shape_), GetShape(filter_), GetShape(input_)});
@@ -146,9 +148,10 @@ TEST_P(TransposeConvOpTest, SimpleTest) {
   TransposeConvOpModel model(
       GetRegistration(), {1, 4, 4, 1}, {TensorType_FLOAT32, {1, 3, 3, 1}},
       {1, 2, 3, 4, 5, 6, 7, 8, 9}, {TensorType_FLOAT32, {1, 4, 4, 1}},
-      {TensorType_FLOAT32, {}}, Padding_SAME, 1, 1, GetTestType());
+      {TensorType_FLOAT32, {}}, Padding_SAME, 1, 1, ActivationFunctionType_NONE,
+      GetTestType());
   model.SetInput({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAreArray({29, 62, 83, 75, 99, 192, 237, 198, 207, 372,
@@ -157,6 +160,34 @@ TEST_P(TransposeConvOpTest, SimpleTest) {
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
 }
 
+// Test case:
+// output = tf.nn.conv2d_backprop_input(
+//     tf.constant([ 1, 4, 4, 1 ]),
+//     tf.constant(np.arange(1, 10), shape=[ 3, 3, 1, 1 ], dtype=tf.float32),
+//     tf.constant(np.arange(1, 17), shape=[ 1, 4, 4, 1 ], dtype=tf.float32),
+//     [1, 1, 1, 1 ],
+//     "SAME")
+TEST_P(TransposeConvOpTest, fusedRELUTest) {
+  // NNAPI can not support for the new kernel behaviors at the moment.
+  if (SingleOpModel::GetForceUseNnapi()) {
+    return;
+  }
+
+  TransposeConvOpModel model(
+      GetRegistration(), {1, 4, 4, 1}, {TensorType_FLOAT32, {1, 3, 3, 1}},
+      {1, 2, 3, 4, 5, 6, 7, 8, 9}, {TensorType_FLOAT32, {1, 4, 4, 1}},
+      {TensorType_FLOAT32, {}}, Padding_SAME, 1, 1, ActivationFunctionType_RELU,
+      GetTestType());
+  model.SetInput({1, 2, -3, -4, 5, 6, -7, -8, 9, 10, -11, -12, 13, 14, 15, 16});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetOutput(),
+              ElementsAreArray({29, 24, 0, 0, 99, 72, 0, 0, 207, 186, 0, 0, 263,
+                                292, 141, 0}));
+  // GetOutputShape() should always be same as model.SetOutputShape(...);
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
 // Test case:
 // filter = tf.constant(np.arange(1, 19),
 //                      shape=[ 3, 3, 1, 2 ],
@@ -174,11 +205,11 @@ TEST_P(TransposeConvOpTest, TwoFiltersTest) {
       GetRegistration(), {1, 4, 4, 1}, {TensorType_FLOAT32, {1, 3, 3, 2}},
       {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18},
       {TensorType_FLOAT32, {1, 4, 4, 2}}, {TensorType_FLOAT32, {}},
-      Padding_SAME, 1, 1, GetTestType());
+      Padding_SAME, 1, 1, ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,
                   12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
                   23, 24, 25, 26, 27, 28, 29, 30, 31, 32});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAreArray({184, 412, 568, 528, 678, 1347, 1689, 1434, 1494,
@@ -203,11 +234,11 @@ TEST_P(TransposeConvOpTest, PaddingValidTest) {
       GetRegistration(), {1, 6, 6, 1}, {TensorType_FLOAT32, {1, 3, 3, 2}},
       {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18},
       {TensorType_FLOAT32, {1, 4, 4, 2}}, {TensorType_FLOAT32, {}},
-      Padding_VALID, 1, 1, GetTestType());
+      Padding_VALID, 1, 1, ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,
                   12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
                   23, 24, 25, 26, 27, 28, 29, 30, 31, 32});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetOutput(),
@@ -232,9 +263,10 @@ TEST_P(TransposeConvOpTest, StrideValidTest) {
   TransposeConvOpModel model(
       GetRegistration(), {1, 5, 5, 1}, {TensorType_FLOAT32, {1, 3, 3, 1}},
       {1, 2, 3, 4, 5, 6, 7, 8, 9}, {TensorType_FLOAT32, {1, 2, 2, 1}},
-      {TensorType_FLOAT32, {}}, Padding_VALID, 2, 2, GetTestType());
+      {TensorType_FLOAT32, {}}, Padding_VALID, 2, 2,
+      ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1, 2, 3, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetOutput(),
@@ -258,9 +290,9 @@ TEST_P(TransposeConvOpTest, MultiChannelTest) {
       GetRegistration(), {1, 5, 5, 2}, {TensorType_FLOAT32, {2, 3, 3, 1}},
       {1, 3, 5, 7, 9, 11, 13, 15, 17, 2, 4, 6, 8, 10, 12, 14, 16, 18},
       {TensorType_FLOAT32, {1, 2, 2, 1}}, {TensorType_FLOAT32, {}},
-      Padding_VALID, 2, 2, GetTestType());
+      Padding_VALID, 2, 2, ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1, 2, 3, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetOutput(),
@@ -287,9 +319,10 @@ TEST_P(TransposeConvOpTest, AccuracyTest) {
   TransposeConvOpModel model(
       GetRegistration(), {1, 3, 4, 1}, {TensorType_FLOAT32, {1, 3, 3, 1}},
       {9, 5, 6, 9, 8, 5, 3, 1, 4}, {TensorType_FLOAT32, {1, 1, 2, 1}},
-      {TensorType_FLOAT32, {}}, Padding_SAME, 3, 3, GetTestType());
+      {TensorType_FLOAT32, {}}, Padding_SAME, 3, 3, ActivationFunctionType_NONE,
+      GetTestType());
   model.SetInput({323, 521});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetOutput(),
               ElementsAreArray(
@@ -298,6 +331,37 @@ TEST_P(TransposeConvOpTest, AccuracyTest) {
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 3, 4, 1}));
 }
 
+// Test case:
+// filter = tf.constant(np.random.randint(1, 10, size=9),
+//                      shape=[ 3, 3, 1, 1 ],
+//                      dtype=tf.float32)
+// output = tf.nn.conv2d_backprop_input(
+//     tf.constant([ 1, 3, 4, 1 ]),
+//     filter,
+//     tf.constant([323, 521], shape=[ 1, 1, 2, 1], dtype=tf.float32),
+//     [1, 3, 3, 1 ],
+//     "SAME")
+// And filter value is derived by:
+// filter = tf.reshape(tf.transpose(filter, perm=[3, 0, 1, 2]), shape=[-1])
+TEST_P(TransposeConvOpTest, AccuracyWithFusedActivationTest) {
+  // NNAPI can not support for the new kernel behaviors at the moment.
+  if (SingleOpModel::GetForceUseNnapi()) {
+    return;
+  }
+  TransposeConvOpModel model(
+      GetRegistration(), {1, 3, 4, 1}, {TensorType_FLOAT32, {1, 3, 3, 1}},
+      {9, 5, 6, 9, 8, 5, 3, 1, 4}, {TensorType_FLOAT32, {1, 1, 2, 1}},
+      {TensorType_FLOAT32, {}}, Padding_SAME, 3, 3, ActivationFunctionType_RELU,
+      GetTestType());
+  model.SetInput({323, -521});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetOutput(),
+              ElementsAreArray(ArrayFloatNear(
+                  {1615, 1938, 0, 0, 2584, 1615, 0, 0, 323, 1292, 0, 0})));
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 3, 4, 1}));
+}
+
 class QuantizedTransposeConvOpModel : public BaseTransposeConvOpModel<uint8_t> {
  public:
   using BaseTransposeConvOpModel::BaseTransposeConvOpModel;
@@ -314,22 +378,49 @@ TEST_P(TransposeConvOpTest, SimpleTestQuantized) {
                                                 139, 141, 143, 145};
   QuantizedTransposeConvOpModel model(
       GetRegistration(), {1, 4, 4, 1},
-      {TensorType_UINT8, {1, 3, 3, 1}, -63.5, 64}, filter_data,
-      {TensorType_UINT8, {1, 4, 4, 1}, -63.5, 64},
-      {TensorType_UINT8, {}, -508, 512}, Padding_SAME, 1, 1, GetTestType());
+      {TensorType_UINT8, {1, 3, 3, 1}, -63, 64}, filter_data,
+      {TensorType_UINT8, {1, 4, 4, 1}, -63, 64},
+      {TensorType_UINT8, {}, -508, 512}, Padding_SAME, 1, 1,
+      ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetDequantizedOutput(),
-      ElementsAreArray(ArrayFloatNear({28, 64, 84, 76, 100, 192, 236, 200, 208,
-                                       372, 416, 332, 264, 448, 484, 364},
+      ElementsAreArray(ArrayFloatNear({36, 72, 96, 84, 116, 216, 268, 220, 232,
+                                       412, 464, 360, 284, 480, 512, 388},
                                       1e-5)));
 
   // GetOutputShape() should always be same as model.SetOutputShape(...);
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
 }
 
+TEST_P(TransposeConvOpTest, SimpleTestWithFusedActivationQuantized) {
+  // NNAPI can not support for the new kernel behaviors at the moment.
+  if (SingleOpModel::GetForceUseNnapi()) {
+    return;
+  }
+
+  // Float would be {1, 2, 3, 4, 5, 6, 7, 8, 9}
+  std::initializer_list<uint8_t> filter_data = {129, 131, 133, 135, 137,
+                                                139, 141, 143, 145};
+  QuantizedTransposeConvOpModel model(
+      GetRegistration(), {1, 4, 4, 1},
+      {TensorType_UINT8, {1, 3, 3, 1}, -63, 64}, filter_data,
+      {TensorType_UINT8, {1, 4, 4, 1}, -63, 64},
+      {TensorType_UINT8, {}, -508, 512}, Padding_SAME, 1, 1,
+      ActivationFunctionType_RELU, GetTestType());
+  model.SetInput({1, 2, -3, -4, 5, 6, -7, -8, 9, 10, -11, -12, 13, 14, 15, 16});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetDequantizedOutput(),
+              ElementsAreArray(ArrayFloatNear({36, 24, 0, 0, 116, 76, 0, 0, 232,
+                                               212, 0, 0, 284, 316, 156, 0})));
+
+  // GetOutputShape() should always be same as model.SetOutputShape(...);
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
 TEST_P(TransposeConvOpTest, TwoFiltersTestQuantized) {
   // Float would be {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
   // 18}
@@ -338,18 +429,19 @@ TEST_P(TransposeConvOpTest, TwoFiltersTestQuantized) {
                                                 153, 155, 157, 159, 161, 163};
   QuantizedTransposeConvOpModel model(
       GetRegistration(), {1, 4, 4, 1},
-      {TensorType_UINT8, {1, 3, 3, 2}, -63.5, 64}, filter_data,
-      {TensorType_UINT8, {1, 4, 4, 2}, -63.5, 64},
-      {TensorType_UINT8, {}, -4064, 4096}, Padding_SAME, 1, 1, GetTestType());
+      {TensorType_UINT8, {1, 3, 3, 2}, -63, 64}, filter_data,
+      {TensorType_UINT8, {1, 4, 4, 2}, -63, 64},
+      {TensorType_UINT8, {}, -4064, 4096}, Padding_SAME, 1, 1,
+      ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,
                   12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
                   23, 24, 25, 26, 27, 28, 29, 30, 31, 32});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetDequantizedOutput(),
               ElementsAreArray(ArrayFloatNear(
-                  {192, 416, 576, 544, 672, 1344, 1696, 1440, 1504, 2720, 3072,
-                   2432, 1984, 3360, 3648, 2752},
+                  {224, 448, 608, 576, 736, 1440, 1792, 1504, 1600, 2880, 3232,
+                   2560, 2048, 3456, 3776, 2848},
                   1e-5)));
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
 }
@@ -362,20 +454,21 @@ TEST_P(TransposeConvOpTest, PaddingValidTestQuantized) {
                                                 153, 155, 157, 159, 161, 163};
   QuantizedTransposeConvOpModel model(
       GetRegistration(), {1, 6, 6, 1},
-      {TensorType_UINT8, {1, 3, 3, 2}, -63.5, 64}, filter_data,
-      {TensorType_UINT8, {1, 4, 4, 2}, -63.5, 64},
-      {TensorType_UINT8, {}, -4064, 4096}, Padding_VALID, 1, 1, GetTestType());
+      {TensorType_UINT8, {1, 3, 3, 2}, -63, 64}, filter_data,
+      {TensorType_UINT8, {1, 4, 4, 2}, -63, 64},
+      {TensorType_UINT8, {}, -4064, 4096}, Padding_VALID, 1, 1,
+      ActivationFunctionType_NONE, GetTestType());
   model.SetInput({1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,
                   12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
                   23, 24, 25, 26, 27, 28, 29, 30, 31, 32});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetDequantizedOutput(),
               ElementsAreArray(ArrayFloatNear(
-                  {0,    32,   64,   96,   128,  96,   64,   192,  416,
-                   576,  544,  352,  224,  672,  1344, 1696, 1440, 864,
-                   608,  1504, 2720, 3072, 2432, 1440, 864,  1984, 3360,
-                   3648, 2752, 1536, 704,  1536, 2528, 2720, 2016, 1088},
+                  {0,    32,   64,   128,  128,  96,   64,   224,  448,
+                   608,  576,  352,  256,  736,  1440, 1792, 1504, 928,
+                   640,  1600, 2880, 3232, 2560, 1504, 896,  2048, 3456,
+                   3776, 2848, 1600, 704,  1568, 2592, 2784, 2048, 1120},
                   1e-5)));
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 6, 6, 1}));
 }
@@ -397,20 +490,26 @@ class PerChannelQuantizedTransposeConvOpModel
 
 TEST_P(TransposeConvOpTest, SimpleTestQuantizedPerChannelSingleChannel) {
   const std::initializer_list<float> filter_data = {1, 2, 3, 4, 5, 6, 7, 8, 9};
+  const std::initializer_list<int8_t> const_filter_data = {14, 28, 42,  56, 71,
+                                                           85, 99, 113, 127};
   PerChannelQuantizedTransposeConvOpModel model(
       GetRegistration(), {1, 4, 4, 1},
       {TensorType_INT8, {1, 3, 3, 1}, 0, 0, 0, 0, true, {9.0 / 127}, {0}, 0},
-      {}, {TensorType_INT8, {1, 4, 4, 1}, 0, 0, 16.0 / 255, -128},
-      {TensorType_INT8, {}, 0, 0, 2, -128}, Padding_SAME, 1, 1, GetTestType(),
+      const_filter_data,
+      {TensorType_INT8, {1, 4, 4, 1}, 0, 0, 16.0 / 127, -128},
+      {TensorType_INT8, {}, 0, 0, 2, -128}, Padding_SAME, 1, 1,
+      ActivationFunctionType_NONE, GetTestType(),
       /* version */ 2);
   model.SetInput({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16});
-  model.SetFilter(filter_data);
-  model.Invoke();
+  if (GetTestType() == TestType::kDynamic) {
+    model.SetFilter(filter_data);
+  }
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetDequantizedOutput(),
-      ElementsAreArray(ArrayFloatNear({28, 62, 82, 76, 98, 192, 238, 198, 206,
-                                       372, 416, 330, 262, 446, 486, 366},
+      ElementsAreArray(ArrayFloatNear({30, 62, 84, 76, 100, 192, 238, 198, 206,
+                                       372, 416, 330, 262, 446, 484, 366},
                                       1e-5)));
 
   // GetOutputShape() should always be same as model.SetOutputShape(...);
@@ -421,6 +520,9 @@ TEST_P(TransposeConvOpTest, SimpleTestQuantizedPerChannelSingleChannel) {
 TEST_P(TransposeConvOpTest, TestQuantizedPerChannelMultiChannel) {
   const std::initializer_list<float> filter_data = {
       1, 3, 5, 7, 9, 11, 13, 15, 17, 2, 4, 6, 8, 10, 12, 14, 16, 18};
+  const std::initializer_list<int8_t> const_filter_data = {
+      7,  22, 37, 52, 67, 82, 97, 112, 127,
+      14, 28, 42, 56, 71, 85, 99, 113, 127};
   PerChannelQuantizedTransposeConvOpModel model(
       GetRegistration(), {1, 5, 5, 2},
       {TensorType_INT8,
@@ -433,12 +535,56 @@ TEST_P(TransposeConvOpTest, TestQuantizedPerChannelMultiChannel) {
        {17.0 / 127, 18.0 / 127},
        {0, 0},
        0},
-      {}, {TensorType_INT8, {1, 2, 2, 1}, 0, 0, 4.0 / 255, -128},
-      {TensorType_INT8, {}, 0, 0, 1, -128}, Padding_VALID, 2, 2, GetTestType(),
+      const_filter_data, {TensorType_INT8, {1, 2, 2, 1}, 0, 0, 4.0 / 255, -128},
+      {TensorType_INT8, {}, 0, 0, 1, -128}, Padding_VALID, 2, 2,
+      ActivationFunctionType_NONE, GetTestType(),
       /* version */ 2);
   model.SetInput({1, 2, 3, 4});
-  model.SetFilter(filter_data);
-  model.Invoke();
+  if (GetTestType() == TestType::kDynamic) {
+    model.SetFilter(filter_data);
+  }
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(
+      model.GetDequantizedOutput(),
+      ElementsAreArray(ArrayFloatNear(
+          {1,  2,  3,  4,  7,  10, 6,  8,  10, 12, 7,   8,   9,  10, 25, 28, 18,
+           20, 22, 24, 16, 20, 24, 28, 62, 72, 42, 48,  54,  60, 21, 24, 27, 30,
+           61, 68, 36, 40, 44, 48, 39, 42, 45, 48, 103, 110, 60, 64, 68, 72},
+          1e-5)));
+
+  // GetOutputShape() should always be same as model.SetOutputShape(...);
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 5, 5, 2}));
+}
+
+// Test data copied from the float multi-channel test above.
+TEST_P(TransposeConvOpTest, TestQuantizedPerTensorMultiChannel) {
+  const std::initializer_list<float> filter_data = {
+      1, 3, 5, 7, 9, 11, 13, 15, 17, 2, 4, 6, 8, 10, 12, 14, 16, 18};
+  const std::initializer_list<int8_t> const_filter_data = {
+      7,  21, 35, 49, 64, 78, 92, 106, 120,
+      14, 28, 42, 56, 71, 85, 99, 113, 127};
+  PerChannelQuantizedTransposeConvOpModel model(
+      GetRegistration(), {1, 5, 5, 2},
+      {TensorType_INT8,
+       {2, 3, 3, 1},
+       0,
+       0,
+       0,
+       0,
+       true,
+       {18.0 / 127, 18.0 / 127},
+       {0, 0},
+       0},
+      const_filter_data, {TensorType_INT8, {1, 2, 2, 1}, 0, 0, 4.0 / 255, -128},
+      {TensorType_INT8, {}, 0, 0, 1, -128}, Padding_VALID, 2, 2,
+      ActivationFunctionType_NONE, GetTestType(),
+      /* version */ 2);
+  model.SetInput({1, 2, 3, 4});
+  if (GetTestType() == TestType::kDynamic) {
+    model.SetFilter(filter_data);
+  }
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetDequantizedOutput(),
@@ -505,7 +651,8 @@ TEST_P(TransposeConvOpTest, SimpleTestQuantizedPerChannel16x8) {
        /*scale=*/1.0,
        /*zero_point=*/0},
       /*padding=*/Padding_SAME,
-      /*stride_w=*/1, /*stride_h=*/1, GetTestType());
+      /*stride_w=*/1, /*stride_h=*/1,
+      /*fused_activation_function=*/ActivationFunctionType_NONE, GetTestType());
   model.SetInput({
       // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
       3, 2,    // batch = 0, y = 0, x = 0
@@ -516,7 +663,7 @@ TEST_P(TransposeConvOpTest, SimpleTestQuantizedPerChannel16x8) {
       -3, -4,  // batch = 0, y = 1, x = 2
   });
   model.SetFilter(filter_data);
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(model.GetDequantizedOutput(),
               ElementsAreArray(ArrayFloatNear(
@@ -526,6 +673,72 @@ TEST_P(TransposeConvOpTest, SimpleTestQuantizedPerChannel16x8) {
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 2, 3, 2}));
 }
 
+TEST_P(TransposeConvOpTest,
+       SimpleTestWithFusedActivationQuantizedPerChannel16x8) {
+  // NNAPI can not support for the new kernel behaviors at the moment.
+  if (SingleOpModel::GetForceUseNnapi()) {
+    return;
+  }
+
+  const std::initializer_list<float> filter_data = {
+      // [2 * 2 * 2 * 2] as [output_channel, y, x, input_channel]
+      1, 2,  // out channel = 0, y = 0, x = 0
+      3, 4,  // out channel = 0, y = 0, x = 1
+      3, 4,  // out channel = 0, y = 1, x = 0
+      5, 6,  // out channel = 0, y = 1, x = 1
+      7, 8,  // out channel = 1, y = 0, x = 0
+      5, 6,  // out channel = 1, y = 0, x = 1
+      3, 4,  // out channel = 1, y = 1, x = 0
+      1, 2,  // out channel = 1, y = 1, x = 1
+  };
+  PerChannelQuantizedTransposeConvOpModel16x8 model(
+      GetRegistration(),
+      /*output_shape_data=*/{1, 2, 3, 2},
+      /*filter=*/
+      {TensorType_INT8,
+       /*shape=*/{2, 2, 2, 2},
+       /*min=*/-64, /*max=*/64,
+       /*scale=*/0, /*zero_point=*/0,
+       /*per_channel_quantization=*/true,
+       /*per_channel_quantization_scales=*/{7.0 / 127, 8.0 / 127},
+       /*per_channel_quantization_offsets=*/{0, 0},
+       /*channel_index=*/0},
+      /*filter_data=*/{},
+      /*input=*/
+      {TensorType_INT16,
+       /*shape=*/{1, 2, 3, 2},
+       /*min=*/0, /*max=*/0,
+       /*scale=*/4.0 / 127,
+       /*zero_point=*/0},
+      /*output=*/
+      {TensorType_INT16,
+       /*shape=*/{},
+       /*min=*/0, /*max=*/0,
+       /*scale=*/1.0,
+       /*zero_point=*/0},
+      /*padding=*/Padding_SAME,
+      /*stride_w=*/1, /*stride_h=*/1,
+      /*fused_activation_function=*/ActivationFunctionType_RELU, GetTestType());
+  model.SetInput({
+      // [1 * 2 * 3 * 2] as [batch, y, x, input_channel]
+      3, 2,    // batch = 0, y = 0, x = 0
+      1, -1,   // batch = 0, y = 0, x = 1
+      -2, -3,  // batch = 0, y = 0, x = 2
+      4, 3,    // batch = 0, y = 1, x = 0
+      2, -2,   // batch = 0, y = 1, x = 1
+      -3, -4,  // batch = 0, y = 1, x = 2
+  });
+  model.SetFilter(filter_data);
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetDequantizedOutput(),
+              ElementsAreArray(ArrayFloatNear(
+                  {7, 37, 16, 26, 0, 0, 27, 69, 48, 42, 0, 0}, 1e-5)));
+
+  // GetOutputShape() should always be same as model.SetOutputShape(...);
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 2, 3, 2}));
+}
+
 template <typename InputType>
 class BaseTransposeConvBiasOpModel : public SingleOpModel {
  public:
@@ -535,8 +748,9 @@ class BaseTransposeConvBiasOpModel : public SingleOpModel {
                                std::initializer_list<InputType> filter_data,
                                const TensorData& input,
                                const TensorData& output, Padding padding,
-                               int stride_w, int stride_h, TestType test_type,
-                               int version = 3) {
+                               int stride_w, int stride_h,
+                               tflite::ActivationFunctionType fused_activation,
+                               TestType test_type, int version = 3) {
     if (test_type == TestType::kDynamic) {
       output_shape_ = AddInput({TensorType_INT32, {4}});
       filter_ = AddInput(filter);
@@ -580,11 +794,12 @@ class BaseTransposeConvBiasOpModel : public SingleOpModel {
 
     output_ = AddOutput(output);
 
-    SetBuiltinOp(
-        BuiltinOperator_TRANSPOSE_CONV, BuiltinOptions_TransposeConvOptions,
-        CreateTransposeConvOptions(builder_, padding, stride_w, stride_h)
-            .Union());
-    resolver_ = absl::make_unique<SingleOpResolver>(
+    SetBuiltinOp(BuiltinOperator_TRANSPOSE_CONV,
+                 BuiltinOptions_TransposeConvOptions,
+                 CreateTransposeConvOptions(builder_, padding, stride_w,
+                                            stride_h, fused_activation)
+                     .Union());
+    resolver_ = std::make_unique<SingleOpResolver>(
         BuiltinOperator_TRANSPOSE_CONV, registration, version);
     BuildInterpreter({GetShape(output_shape_), GetShape(filter_),
                       GetShape(input_), GetShape(bias_)});
@@ -645,10 +860,6 @@ class TransposeConvOpBiasModel : public BaseTransposeConvBiasOpModel<float> {
 // model.layers[1].set_weights([filter_data, bias_data])
 // output = model.predict(input_data)
 TEST_P(TransposeConvOpTest, MultiChannelBiasTest) {
-  // TODO(b/138722124): Enable these tests on NNAPI.
-  if (SingleOpModel::GetForceUseNnapi()) {
-    return;
-  }
   TransposeConvOpBiasModel model(
       GetRegistration(), /*output_shape=*/{1, 5, 5, 2},
       /*filter=*/{TensorType_FLOAT32, {2, 3, 3, 1}},
@@ -656,10 +867,12 @@ TEST_P(TransposeConvOpTest, MultiChannelBiasTest) {
       {1, 3, 5, 7, 9, 11, 13, 15, 17, 2, 4, 6, 8, 10, 12, 14, 16, 18},
       /*input=*/{TensorType_FLOAT32, {1, 2, 2, 1}},
       /*output=*/{TensorType_FLOAT32, {}}, Padding_VALID,
-      /*stride_w=*/2, /*stride_h=*/2, GetTestType(), /* version */ 3);
+      /*stride_w=*/2, /*stride_h=*/2,
+      /*fused_activation_function=*/ActivationFunctionType_NONE, GetTestType(),
+      /* version */ 3);
   model.SetInput({1, 2, 3, 4});
   model.SetBias({3, 4});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetOutput(),
@@ -670,6 +883,47 @@ TEST_P(TransposeConvOpTest, MultiChannelBiasTest) {
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 5, 5, 2}));
 }
 
+// Test case:
+// input_data = np.arange(1, 5).reshape(1,2,2,1).astype(np.float32)
+// filter_data = np.arange(1, 19).reshape(3,3,2,1).astype(np.float32)
+// bias_data = np.array([3,4])
+// input = tf.keras.layers.Input(shape=(2, 2, 1))
+// output = tf.keras.layers.Convolution2DTranspose(filters=2,
+//                                                 kernel_size=[3, 3],
+//                                                 strides=[2, 2],
+//                                                 padding="valid")(input)
+// model = tf.keras.models.Model(input, output)
+// model.layers[1].set_weights([filter_data, bias_data])
+// output = model.predict(input_data)
+TEST_P(TransposeConvOpTest, MultiChannelBiasWithFusedActivationTest) {
+  // NNAPI can not support for the new kernel behaviors at the moment.
+  if (SingleOpModel::GetForceUseNnapi()) {
+    return;
+  }
+
+  TransposeConvOpBiasModel model(
+      GetRegistration(), /*output_shape=*/{1, 5, 5, 2},
+      /*filter=*/{TensorType_FLOAT32, {2, 3, 3, 1}},
+      /*filter_data=*/
+      {1, 3, 5, 7, 9, 11, 13, 15, 17, 2, 4, 6, 8, 10, 12, 14, 16, 18},
+      /*input=*/{TensorType_FLOAT32, {1, 2, 2, 1}},
+      /*output=*/{TensorType_FLOAT32, {}}, Padding_VALID,
+      /*stride_w=*/2, /*stride_h=*/2,
+      /*fused_activation_function=*/ActivationFunctionType_RELU, GetTestType(),
+      /* version */ 3);
+  model.SetInput({1, 2, -3, 4});
+  model.SetBias({3, 4});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(
+      model.GetOutput(),
+      ElementsAreArray({4,  6,  6,  8,  10, 14, 9,  12, 13, 16, 10, 12, 12,
+                        14, 28, 32, 21, 24, 25, 28, 13, 12, 9,  8,  35, 40,
+                        45, 52, 57, 64, 0,  0,  0,  0,  0,  0,  39, 44, 47,
+                        52, 0,  0,  0,  0,  4,  6,  63, 68, 71, 76}));
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 5, 5, 2}));
+}
+
 class QuantizedTransposeConvBiasOpModel
     : public BaseTransposeConvBiasOpModel<uint8_t> {
  public:
@@ -682,10 +936,6 @@ class QuantizedTransposeConvBiasOpModel
 };
 
 TEST_P(TransposeConvOpTest, SimpleBiasTestQuantized) {
-  // TODO(b/138722124): Enable these tests on NNAPI.
-  if (SingleOpModel::GetForceUseNnapi()) {
-    return;
-  }
   // Float would be {1, 2, 3, 4, 5, 6, 7, 8, 9}
   std::initializer_list<uint8_t> filter_data = {129, 131, 133, 135, 137,
                                                 139, 141, 143, 145};
@@ -693,11 +943,11 @@ TEST_P(TransposeConvOpTest, SimpleBiasTestQuantized) {
       GetRegistration(), {1, 4, 4, 1},
       {TensorType_UINT8, {1, 3, 3, 1}, -63.5, 64}, filter_data,
       {TensorType_UINT8, {1, 4, 4, 1}, -63.5, 64},
-      {TensorType_UINT8, {}, -508, 512}, Padding_SAME, 1, 1, GetTestType(),
-      /* version */ 3);
+      {TensorType_UINT8, {}, -508, 512}, Padding_SAME, 1, 1,
+      ActivationFunctionType_NONE, GetTestType(), /* version */ 3);
   model.SetInput({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16});
   model.SetBias({1});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetDequantizedOutput(),
@@ -709,6 +959,35 @@ TEST_P(TransposeConvOpTest, SimpleBiasTestQuantized) {
   EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
 }
 
+TEST_P(TransposeConvOpTest, SimpleBiasWithFusedActivationTestQuantized) {
+  // NNAPI can not support for the new kernel behaviors at the moment.
+  if (SingleOpModel::GetForceUseNnapi()) {
+    return;
+  }
+
+  // Float would be {1, 2, 3, 4, 5, 6, 7, 8, 9}
+  std::initializer_list<uint8_t> filter_data = {129, 131, 133, 135, 137,
+                                                139, 141, 143, 145};
+  QuantizedTransposeConvBiasOpModel model(
+      GetRegistration(), {1, 4, 4, 1},
+      {TensorType_UINT8, {1, 3, 3, 1}, -63.5, 64}, filter_data,
+      {TensorType_UINT8, {1, 4, 4, 1}, -63.5, 64},
+      {TensorType_UINT8, {}, -508, 512}, Padding_SAME, 1, 1,
+      ActivationFunctionType_RELU, GetTestType(), /* version */ 3);
+  model.SetInput(
+      {1, 2, -3, -4, 5, 6, -7, -8, 9, 10, -11, -12, 13, 14, -15, -16});
+  model.SetBias({1});
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
+
+  EXPECT_THAT(model.GetDequantizedOutput(),
+              ElementsAreArray(ArrayFloatNear(
+                  {32, 24, 0, 0, 100, 72, 0, 0, 208, 156, 0, 0, 264, 172, 0, 0},
+                  1e-5)));
+
+  // GetOutputShape() should always be same as model.SetOutputShape(...);
+  EXPECT_THAT(model.GetOutputShape(), ElementsAreArray({1, 4, 4, 1}));
+}
+
 class PerChannelQuantizedTransposeConvBiasOpModel
     : public BaseTransposeConvBiasOpModel<int8_t> {
  public:
@@ -729,22 +1008,22 @@ class PerChannelQuantizedTransposeConvBiasOpModel
 };
 
 TEST_P(TransposeConvOpTest, SimpleBiasTestQuantizedPerChannelSingleChannel) {
-  // TODO(b/138722124): Enable these tests on NNAPI.
-  if (SingleOpModel::GetForceUseNnapi()) {
-    return;
-  }
-
   const std::initializer_list<float> filter_data = {1, 2, 3, 4, 5, 6, 7, 8, 9};
+  const std::initializer_list<int8_t> const_filter_data = {14, 28, 42,  56, 71,
+                                                           85, 99, 113, 127};
   PerChannelQuantizedTransposeConvBiasOpModel model(
       GetRegistration(), {1, 4, 4, 1},
       {TensorType_INT8, {1, 3, 3, 1}, 0, 0, 0, 0, true, {9.0 / 127}, {0}, 0},
-      {}, {TensorType_INT8, {1, 4, 4, 1}, 0, 0, 16.0 / 255, -128},
-      {TensorType_INT8, {}, 0, 0, 2, -128}, Padding_SAME, 1, 1, GetTestType(),
-      /* version */ 3);
+      const_filter_data,
+      {TensorType_INT8, {1, 4, 4, 1}, 0, 0, 16.0 / 255, -128},
+      {TensorType_INT8, {}, 0, 0, 2, -128}, Padding_SAME, 1, 1,
+      ActivationFunctionType_NONE, GetTestType(), /* version */ 3);
   model.SetInput({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16});
-  model.SetFilter(filter_data);
+  if (GetTestType() == TestType::kDynamic) {
+    model.SetFilter(filter_data);
+  }
   model.SetBias({1});
-  model.Invoke();
+  ASSERT_EQ(model.Invoke(), kTfLiteOk);
 
   EXPECT_THAT(
       model.GetDequantizedOutput(),
