@@ -1,4 +1,4 @@
-/* Copyright 2017 The TensorFlow Authors. All Rights Reserved.
+/* Copyright 2020 The TensorFlow Authors. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -12,30 +12,12 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
-/// Deserialization infrastructure for tflite. Provides functionality
-/// to go from a serialized tflite model in flatbuffer format to an
-/// interpreter.
-///
-#ifndef TENSORFLOW_LITE_MODEL_H_
-#define TENSORFLOW_LITE_MODEL_H_
+#ifndef TENSORFLOW_LITE_CORE_SHIMS_CC_MODEL_H_
+#define TENSORFLOW_LITE_CORE_SHIMS_CC_MODEL_H_
 
+/// For documentation, see third_party/tensorflow/lite/core/model.h.
+
+#include "tensorflow/lite/interpreter_builder.h"
 #include "tensorflow/lite/model_builder.h"
 
-#if TFLITE_EXPERIMENTAL_RUNTIME_EAGER
-#include "tensorflow/lite/experimental/tf_runtime/lib/eager_model.h"
-#else
-#include "tensorflow/lite/interpreter_builder.h"
-#endif
-
-namespace tflite {
-
-#if TFLITE_EXPERIMENTAL_RUNTIME_EAGER
-using InterpreterBuilder = tflrt::EagerTfLiteInterpreterBuilderAPI;
-using Interpreter = tflrt::EagerInterpreter;
-#else
-using InterpreterBuilder = impl::InterpreterBuilder;
-#endif
-
-}  // namespace tflite
-
-#endif  // TENSORFLOW_LITE_MODEL_H_
+#endif  // TENSORFLOW_LITE_CORE_SHIMS_CC_MODEL_H_
