@@ -19,11 +19,18 @@ limitations under the License.
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <memory>
+
 #include "tensorflow/lite/nnapi/NeuralNetworksTypes.h"
 
 struct NnApi {
   bool nnapi_exists;
   int32_t android_sdk_version;
+  // NNAPI feature level should be used when deciding which NNAPI feature to
+  // use, as feature levels after Android API level 31 have no association with
+  // API level because the NNAPI specification can be updated between Android
+  // API releases.
+  int64_t nnapi_runtime_feature_level;
 
   /**
    * Creates a shared memory object from a file descriptor.
@@ -272,6 +279,7 @@ struct NnApi {
    *
    * See {@link ANeuralNetworksModel} for information on multithreaded usage.
    *
+   * @return ANEURALNETWORKS_NO_ERROR if successful.
    */
   int (*ANeuralNetworksModel_identifyInputsAndOutputs)(
       ANeuralNetworksModel* model, uint32_t inputCount, const uint32_t* inputs,
@@ -298,6 +306,8 @@ struct NnApi {
    * Available since API level 28.
    *
    * See {@link ANeuralNetworksModel} for information on multithreaded usage.
+   *
+   * @return ANEURALNETWORKS_NO_ERROR if successful.
    */
   int (*ANeuralNetworksModel_relaxComputationFloat32toFloat16)(
       ANeuralNetworksModel* model, bool allow);
@@ -1313,7 +1323,7 @@ struct NnApi {
    */
   int (*ANeuralNetworksMemoryDesc_addInputRole)(
       ANeuralNetworksMemoryDesc* desc,
-      const ANeuralNetworksCompilation* compilation, int32_t index,
+      const ANeuralNetworksCompilation* compilation, uint32_t index,
       float frequency);
 
   /**
@@ -1530,6 +1540,570 @@ struct NnApi {
    */
   int (*ANeuralNetworksMemory_copy)(const ANeuralNetworksMemory* src,
                                     const ANeuralNetworksMemory* dst);
+
+  /**
+   * Create a {@link ANeuralNetworksEvent} from a sync_fence file descriptor.
+   *
+   * The newly created ANeuralNetworksEvent does not take ownership of the
+   * provided sync_fence_fd, it will instead dup the provided sync_fence_fd and
+   * own the duplicate.
+   *
+   * @param sync_fence_fd The sync_fence file descriptor.
+   * @param event The newly created object or NULL if unsuccessful.
+   *
+   * @return ANEURALNETWORKS_NO_ERROR if successful.
+   *
+   * Available since API level 30.
+   */
+  int (*ANeuralNetworksEvent_createFromSyncFenceFd)(
+      int sync_fence_fd, ANeuralNetworksEvent** event);
+
+  /**
+   * Get sync_fence file descriptor from the event.
+   *
+   * If the ANeuralNetworksEvent is not backed by a sync fence, the
+   * sync_fence_fd will be set to -1, and ANEURALNETWORKS_BAD_DATA will be
+   * returned.
+   *
+   * See {@link ANeuralNetworksEvent_createFromSyncFenceFd} and
+   * {@link ANeuralNetworksExecution_startComputeWithDependencies} to see how to
+   * create an event backed by a sync fence.
+   *
+   * The user takes ownership of the returned fd, and must close the returned
+   * file descriptor when it is no longer needed.
+   *
+   * @param event An event that is backed by a sync fence.
+   * @param sync_fence_fd The sync_fence file descriptor. The file descriptor
+   * will be set to -1 if there is an error.
+   *
+   * @return ANEURALNETWORKS_NO_ERROR if successful.
+   *
+   * Available since API level 30.
+   */
+  int (*ANeuralNetworksEvent_getSyncFenceFd)(const ANeuralNetworksEvent* event,
+                                             int* sync_fence_fd);
+
+  /**
+   * Schedule asynchronous evaluation of the execution with dependencies.
+   *
+   * The execution will wait for all the depending events to be signaled before
+   * starting the evaluation. Once the execution has completed and the outputs
+   * are ready to be consumed, the returned event will be signaled. Depending on
+   * which devices are handling the execution, the event could be backed by a
+   * sync fence. Use {@link ANeuralNetworksEvent_wait} to wait for that event.
+   *
+   * ANeuralNetworksEvent_wait must be called to recurperate the resources used
+   * by the execution.
+   *
+   * If parts of the execution are scheduled on devices that do not support
+   * fenced execution, the function call may wait for such parts to finish
+   * before returning.
+   *
+   * The function will return an error if any of the events in dependencies is
+   * already in a bad state. After the execution is scheduled, if any of the
+   * events in dependencies does not complete normally, the execution will fail,
+   * and {@link ANeuralNetworksEvent_wait} on the returned event will return an
+   * error.
+   *
+   * The function will return an error if any of the execution outputs has a
+   * tensor operand type that is not fully specified.
+   *
+   * The function can be passed a timeout duration in nanoseconds. This timeout
+   * duration acts as a hint to drivers in the same way that the timeout
+   * durations in {@link ANeuralNetworksCompilation_setTimeout} and {@link
+   * ANeuralNetworksExecution_setTimeout} act as hints to drivers. The duration
+   * begins when all waitFor sync fences have been signaled, and can be used
+   * together with {@link ANeuralNetworksExecution_setTimeout} which specifies
+   * the maximum timeout duration beginning at the call to
+   * {@link ANeuralNetworksExecution_startComputeWithDependencies}.
+   * If the duration is non-zero, the {@link ANeuralNetworksExecution} must have
+   * been created from an {@link ANeuralNetworksCompilation} which in turn was
+   * created from
+   * {@link ANeuralNetworksCompilation_createForDevices} with numDevices = 1,
+   * otherwise this function will fail with ANEURALNETWORKS_BAD_DATA. If either
+   * the timeout duration from {@link ANeuralNetworksExecution_setTimeout} or
+   * the timeout duration passed to this call is exceeded, the execution may be
+   * aborted, in which case {@link ANEURALNETWORKS_MISSED_DEADLINE_*} will be
+   * returned through {@link
+   * ANeuralNetworksExecution_startComputeWithDependencies} or {@link
+   * ANeuralNetworksEvent_wait} on the event object. If the device has a feature
+   * level reported by {@link ANeuralNetworksDevice_getFeatureLevel} that is
+   * lower than 30, then the timeout duration hints will be ignored.
+   *
+   * If this execution contains a {@link ANEURALNETWORKS_WHILE} operation, and
+   * the condition model does not output false within the loop timeout duration,
+   * then execution will be aborted and {@link
+   * ANEURALNETWORKS_MISSED_DEADLINE_*} will be returned through {@link
+   * ANeuralNetworksEvent_wait} on the event object.
+   *
+   * See {@link ANeuralNetworksExecution} for information on multithreaded
+   * usage.
+   *
+   * See {@link ANeuralNetworksExecution_compute} for synchronous execution.
+   * See {@link ANeuralNetworksExecution_burstCompute} for burst synchronous
+   * execution. See {@link ANeuralNetworksExecution_startCompute} for regular
+   * asynchronous execution.
+   *
+   * @param execution The execution to be scheduled and executed.
+   * @param dependencies A set of depending events. The actual evaluation will
+   * not start until all the events are signaled.
+   * @param num_dependencies The number of events in the dependencies set.
+   * @param duration The maximum amount of time in nanoseconds that is expected
+   * to be spent executing the model after all dependencies are signaled. If set
+   * to 0, the timeout duration is considered infinite.
+   * @param event The event that will be signaled on completion. event is set to
+   *              NULL if there's an error.
+   *
+   * @return ANEURALNETWORKS_NO_ERROR if the evaluation is successfully
+   * scheduled.
+   *
+   * Available since API level 30.
+   */
+  int (*ANeuralNetworksExecution_startComputeWithDependencies)(
+      ANeuralNetworksExecution* execution,
+      const ANeuralNetworksEvent* const* dependencies,
+      uint32_t num_dependencies, uint64_t duration,
+      ANeuralNetworksEvent** event);
+
+  /**
+   * Specifies whether the {@link ANeuralNetworksExecution} is able to accept
+   * padded input and output buffers and memory objects.
+   *
+   * By default, the input and output buffers and memory objects of {@link
+   * ANeuralNetworksExecution} do not allow padding.
+   *
+   * Setting the execution to accept padded input and output buffers and memory
+   * objects enables the length argument of {@link
+   * ANeuralNetworksExecution_setInput},
+   * {@link ANeuralNetworksExecution_setInputFromMemory}, {@link
+   * ANeuralNetworksExecution_setOutput}, and {@link
+   * ANeuralNetworksExecution_setOutputFromMemory} to be greater than the raw
+   * size of the operand (i.e. the size of an element multiplied by the number
+   * of elements). The extra bytes at the end of the buffer or memory region may
+   * be used by the driver to access data in chunks, for efficiency.
+   *
+   * This method must not be called after {@link
+   * ANeuralNetworksExecution_setInput},
+   * {@link ANeuralNetworksExecution_setInputFromMemory}, {@link
+   * ANeuralNetworksExecution_setOutput}, or {@link
+   * ANeuralNetworksExecution_setOutputFromMemory}.
+   *
+   * See {@link ANeuralNetworksExecution} for information on multithreaded
+   * usage.
+   *
+   * @param execution The execution to be modified.
+   * @param enable 'true' if the execution is to be able to accept padded input
+   * and output buffers and memory objects, 'false' if not.
+   *
+   * @return ANEURALNETWORKS_NO_ERROR if successful.
+   *         ANEURALNETWORKS_UNEXPECTED_NULL if execution is NULL.
+   *         ANEURALNETWORKS_BAD_STATE if {@link
+   * ANeuralNetworksExecution_setInput},
+   *         {@link ANeuralNetworksExecution_setInputFromMemory},
+   *         {@link ANeuralNetworksExecution_setOutput}, or
+   *         {@link ANeuralNetworksExecution_setOutputFromMemory} has been
+   * called on the execution.
+   *
+   * Available since API level 31.
+   */
+  int (*ANeuralNetworksExecution_enableInputAndOutputPadding)(
+      ANeuralNetworksExecution* execution, bool enable);
+
+  /**
+   * Specifies whether the {@link ANeuralNetworksExecution} can be reused for
+   * multiple computations.
+   *
+   * By default, the {@link ANeuralNetworksExecution} is not reusable.
+   *
+   * Setting the execution to be reusable enables multiple computations to be
+   * scheduled and evaluated on the same execution sequentially, either by means
+   * of
+   * {@link ANeuralNetworksExecution_burstCompute}, {@link
+   * ANeuralNetworksExecution_compute},
+   * {@link ANeuralNetworksExecution_startCompute} or
+   * {@link ANeuralNetworksExecution_startComputeWithDependencies}.
+   *
+   * This function may only be invoked when the execution is in the preparation
+   * state.
+   *
+   * See {@link ANeuralNetworksExecution} for information on execution states
+   * and multithreaded usage.
+   *
+   * @param execution The execution to be modified.
+   * @param reusable 'true' if the execution is to be reusable, 'false' if not.
+   *
+   * @return ANEURALNETWORKS_NO_ERROR if successful.
+   *         ANEURALNETWORKS_UNEXPECTED_NULL if execution is NULL.
+   *         ANEURALNETWORKS_BAD_STATE if the execution is not in the
+   * preparation state.
+   *
+   * Available since API level 31.
+   */
+  int (*ANeuralNetworksExecution_setReusable)(
+      ANeuralNetworksExecution* execution, bool reusable);
+
+  /**
+   * Get the NNAPI runtime feature level.
+   *
+   * Since API level 31 (NNAPI feature level 5), the NNAPI runtime
+   * (libneuralnetworks.so) and its API specification can be updated between
+   * Android API releases.
+   *
+   * On Android devices with API level 31 and newer, for NNAPI runtime feature
+   * discovery, the NNAPI runtime feature level must be used instead of the
+   * Android device API level.
+   *
+   * On Android devices with API level 30 and older, the Android API level of
+   * the Android device must be used for NNAPI runtime feature discovery. Enum
+   * values in
+   * {@link FeatureLevelCode} from feature level 1 to 5 have their corresponding
+   * Android API levels listed in their documentation, and each such enum value
+   * equals the corresponding API level. This allows using the Android API level
+   * as the feature level. This mapping between enum value and Android API level
+   * does not exist for feature levels after NNAPI feature level 5 and API
+   * levels after S (31).
+   *
+   * Example usage:
+   * int device_api_level = android_get_device_api_level();
+   * int64_t runtime_feature_level = (device_api_level < __ANDROID_API_S__) ?
+   *                                  device_api_level :
+   * ANeuralNetworks_getRuntimeFeatureLevel();
+   *
+   * Runtime feature level is closely related to NNAPI device feature level
+   * ({@link ANeuralNetworksDevice_getFeatureLevel}), which indicates an NNAPI
+   * device feature level (the most advanced NNAPI specification and features
+   * that the driver implements). This function expresses NNAPI runtime feature
+   * level, which indicates the most advanced NNAPI specification and features
+   * the runtime implements. An NNAPI device feature level is always less than
+   * or equal to the runtime feature level.
+   *
+   * This function returns a {@link FeatureLevelCode} enum value,
+   * which is the NNAPI specification version that this NNAPI runtime
+   * implements. It is NOT an Android API level.
+   *
+   * Available since NNAPI feature level 5.
+   */
+  int64_t (*ANeuralNetworks_getRuntimeFeatureLevel)();
+
+  /**
+   * Gets the ID that identifies a single session of client interacting with
+   * NNAPI runtime.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Session info id.
+   */
+  int32_t (*SL_ANeuralNetworksDiagnosticCompilationInfo_getSessionId)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets NNAPI version.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return NNAPI version.
+   */
+  int64_t (*SL_ANeuralNetworksDiagnosticCompilationInfo_getNnApiVersion)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets the hash of the model architecture (without weights).
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Model hash.
+   */
+  const uint8_t* (
+      *SL_ANeuralNetworksDiagnosticCompilationInfo_getModelArchHash)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets the device IDs as a comma-concatenated string.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Device ID.
+   */
+  const char* (*SL_ANeuralNetworksDiagnosticCompilationInfo_getDeviceIds)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets the error code.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Error code.
+   */
+  int32_t (*SL_ANeuralNetworksDiagnosticCompilationInfo_getErrorCode)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets the type of tensors used for inputs.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Input data class.
+   */
+  ANeuralNetworksDiagnosticDataClass (
+      *SL_ANeuralNetworksDiagnosticCompilationInfo_getInputDataClass)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets the type of tensors used for outputs.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Output data class.
+   */
+  ANeuralNetworksDiagnosticDataClass (
+      *SL_ANeuralNetworksDiagnosticCompilationInfo_getOutputDataClass)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets how many nanoseconds elapsed when compiling the model.
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Time to compile the model in nanoseconds. UINT64_MAX indicates that
+   * timing information is not available.
+   */
+  uint64_t (
+      *SL_ANeuralNetworksDiagnosticCompilationInfo_getCompilationTimeNanos)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Is caching enabled?
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Whether caching is enabled.
+   */
+  bool (*SL_ANeuralNetworksDiagnosticCompilationInfo_isCachingEnabled)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Is control flow used?
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Whether control flow was used.
+   */
+  bool (*SL_ANeuralNetworksDiagnosticCompilationInfo_isControlFlowUsed)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Are dynamic tensors used?
+   *
+   * @param diagnosticCompilationInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Whether dynamic tensors were used.
+   */
+  bool (*SL_ANeuralNetworksDiagnosticCompilationInfo_areDynamicTensorsUsed)(
+      const ANeuralNetworksDiagnosticCompilationInfo*
+          diagnosticCompilationInfo);
+
+  /**
+   * Gets the ID that identifies a single session of client interacting with
+   * NNAPI runtime.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Session info id.
+   */
+  int32_t (*SL_ANeuralNetworksDiagnosticExecutionInfo_getSessionId)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets NNAPI version.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return NNAPI version.
+   */
+  int64_t (*SL_ANeuralNetworksDiagnosticExecutionInfo_getNnApiVersion)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the hash of the model architecture (without weights).
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Model hash.
+   */
+  const uint8_t* (*SL_ANeuralNetworksDiagnosticExecutionInfo_getModelArchHash)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the device IDs as a comma-concatenated string.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Device ID.
+   */
+  const char* (*SL_ANeuralNetworksDiagnosticExecutionInfo_getDeviceIds)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the execution mode.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Execution mode.
+   */
+  ANeuralNetworksDiagnosticExecutionMode (
+      *SL_ANeuralNetworksDiagnosticExecutionInfo_getExecutionMode)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the input data class.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Input data class.
+   */
+  ANeuralNetworksDiagnosticDataClass (
+      *SL_ANeuralNetworksDiagnosticExecutionInfo_getInputDataClass)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the output data class.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Output data class.
+   */
+  ANeuralNetworksDiagnosticDataClass (
+      *SL_ANeuralNetworksDiagnosticExecutionInfo_getOutputDataClass)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the error code.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Error code.
+   */
+  uint32_t (*SL_ANeuralNetworksDiagnosticExecutionInfo_getErrorCode)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the time taken to execute from runtime, including runtime/ipc
+   * overhead.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Time taken to execute as measured by the runtime in nanoseconds.
+   * UINT64_MAX indicates that timing information is not available.
+   */
+  uint64_t (
+      *SL_ANeuralNetworksDiagnosticExecutionInfo_getRuntimeExecutionTimeNanos)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the time taken to execute in the driver, excluding runtime/ipc
+   * overhead.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Time taken to execute on the driver in nanoseconds. UINT64_MAX
+   * indicates that timing information is not available.
+   */
+  uint64_t (
+      *SL_ANeuralNetworksDiagnosticExecutionInfo_getDriverExecutionTimeNanos)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Gets the time taken to execute on the hardware, excluding driver overhead.
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Time taken to execute on the hardware in nanoseconds. UINT64_MAX
+   * indicates that timing information is not available.
+   */
+  uint64_t (
+      *SL_ANeuralNetworksDiagnosticExecutionInfo_getHardwareExecutionTimeNanos)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Is caching enabled?
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Whether caching is enabled.
+   */
+  bool (*SL_ANeuralNetworksDiagnosticExecutionInfo_isCachingEnabled)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Is control flow used?
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Whether control flow was used.
+   */
+  bool (*SL_ANeuralNetworksDiagnosticExecutionInfo_isControlFlowUsed)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Are dynamic tensors used?
+   *
+   * @param diagnosticExecutionInfo The NNAPI diagnostic compilation info
+   * object.
+   * @return Whether dynamic tensors were used.
+   */
+  bool (*SL_ANeuralNetworksDiagnosticExecutionInfo_areDynamicTensorsUsed)(
+      const ANeuralNetworksDiagnosticExecutionInfo* diagnosticExecutionInfo);
+
+  /**
+   * Sets the callbacks to be called when compilations or executions finish.
+   *
+   * Example usage:
+   *
+   * // Callback to be invoked whenever a compilation has completed.
+   * void compilationCallback(void* context,
+   * ANeuralNetworksDiagnosticCompilationInfo* info) {
+   *     // The context object can be used to store state without the use of a
+   * global variable. ExampleLoggerObject* logger =
+   * static_cast<ExampleLoggerObject*>(context);
+   *
+   *     // Calls to getters to get the details...
+   *     const int32_t sessionId =
+   * ANeuralNetworksDiagnosticCompilationInfo_getSessionId(info);
+   *
+   *     ...
+   *
+   *     logger->write(...);
+   * }
+   *
+   * void executionCallback(void* context,
+   * ANeuralNetworksDiagnosticExecutionInfo* info) {
+   *      ...
+   * }
+   *
+   * ExampleLoggerObject exampleLoggerObject;
+   * ANeuralNetworksDiagnostic_registerCallbacks(&compilationCallback,
+   * &executionCallback, static_cast<void*>(&exampleLoggerObject));
+   *
+   * @param compilationCallback The compilation callback to set.
+   * @param executionCallback The execution callback to set.
+   * @param callbackContext The context to be passed to the callbacks when they
+   * are invoked. The context object may be used by multiple threads
+   * simulatenously, so it must be thread-safe.
+   */
+  void (*SL_ANeuralNetworksDiagnostic_registerCallbacks)(
+      ANeuralNetworksDiagnosticCompilationFinishedCallback compilationCallback,
+      ANeuralNetworksDiagnosticExecutionFinishedCallback executionCallback,
+      void* callbackContext);
 };
 
 /**
@@ -1539,4 +2113,22 @@ struct NnApi {
  */
 const NnApi* NnApiImplementation();
 
+// Forward declaration for CreateNnApiFromSupportLibrary below.
+struct NnApiSLDriverImplFL5;
+
+/**
+ * Allocate a new NnApi structure instance and fill it with function pointers
+ * from NnApiSLDriverImplFL5 instance. Functions that are not present in the
+ * support library are assigned null pointers.
+ *
+ * The NN API Support Library Driver must support at least NNAPI Feature Level 5
+ * (introduced in SDK level 31), but this might point to a compatible struct
+ * that also supports a higher NNAPI Feature Level. These cases can be
+ * distinguished by examining the base.implFeatureLevel field, which should be
+ * set to the supported feature level (which must be >=
+ * ANEURALNETWORKS_FEATURE_LEVEL_5).
+ */
+std::unique_ptr<const NnApi> CreateNnApiFromSupportLibrary(
+    const NnApiSLDriverImplFL5* nnapi_support_library_driver);
+
 #endif  // TENSORFLOW_LITE_NNAPI_NNAPI_IMPLEMENTATION_H_
