@@ -13,7 +13,9 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 #include "tensorflow/lite/toco/import_tensorflow.h"
-#include "tensorflow/lite/toco/toco_port.h"
+
+#include <memory>
+#include <string>
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -25,6 +27,7 @@ limitations under the License.
 #include "tensorflow/core/framework/tensor_shape.pb.h"
 #include "tensorflow/core/lib/core/status.h"
 #include "tensorflow/lite/testing/util.h"
+#include "tensorflow/lite/toco/toco_port.h"
 
 namespace toco {
 
@@ -37,6 +40,8 @@ using tensorflow::DT_INT64;
 using tensorflow::DT_INVALID;
 using tensorflow::DT_QUINT8;
 using tensorflow::DT_STRING;
+using tensorflow::DT_UINT16;
+using tensorflow::DT_UINT32;
 using tensorflow::NodeDef;
 using tensorflow::Status;
 using ::testing::ElementsAre;
@@ -127,6 +132,11 @@ void BuildConstNode(std::initializer_list<int64_t> shape,
         t.add_int_val(i % std::numeric_limits<int>::max() + 1);
       }
       break;
+    case DT_UINT32:
+      for (int64_t i = 0; i < num_elements; ++i) {
+        t.add_int_val(i % std::numeric_limits<uint32_t>::max() + 1);
+      }
+      break;
     case DT_QUINT8:
       for (int64_t i = 0; i < num_elements; ++i) {
         t.add_int_val(i % std::numeric_limits<uint8_t>::max() + 1);
@@ -137,6 +147,11 @@ void BuildConstNode(std::initializer_list<int64_t> shape,
         t.add_int64_val(i + 1);
       }
       break;
+    case DT_UINT16:
+      for (int64_t i = 0; i < num_elements; ++i) {
+        t.add_int_val(i % std::numeric_limits<uint16_t>::max() + 1);
+      }
+      break;
     case DT_STRING:
       break;
     case DT_BOOL:
@@ -231,7 +246,7 @@ TEST_P(ShapeImportTest, ShapeElementIsNegative) {
   BuildConstNode({1, -2, 10}, GetParam(), 0, &node);
   auto status = ImportNode(node);
   EXPECT_EQ(
-      status.error_message(),
+      status.message(),
       "Tensor shape should not include negative values\n\t (while processing "
       "node 'Node1')");
 }
@@ -267,7 +282,7 @@ TEST_P(ShapeImportTest, ShapeElementTooLarge) {
   NodeDef node;
   BuildConstNode({3000000000}, GetParam(), 0, &node);
   auto status = ImportNode(node);
-  EXPECT_EQ(status.error_message(),
+  EXPECT_EQ(status.message(),
             "Shape element overflows\n\t (while processing node 'Node1')");
 }
 
@@ -275,21 +290,10 @@ TEST_P(ShapeImportTest, ShapeTooLarge) {
   NodeDef node;
   BuildConstNode({1000000, 2000000, 2000000, 2000000}, GetParam(), 0, &node);
   auto status = ImportNode(node);
-  EXPECT_EQ(status.error_message(),
+  EXPECT_EQ(status.message(),
             "Tensor shape is too large\n\t (while processing node 'Node1')");
 }
 
-TEST_P(ShapeImportTest, ValidShapeButZeroElements) {
-  NodeDef node;
-  BuildConstNode({1, 2, 2, 2}, GetParam(), 0, &node);
-  auto status = ImportNode(node);
-  EXPECT_THAT(status.error_message(),
-              ::testing::MatchesRegex(
-                  "Neither input_content .0. nor .*_val .0. have the right "
-                  "dimensions .8. for this .* tensor\n\t .while processing "
-                  "node 'Node1'."));
-}
-
 std::vector<tensorflow::DataType> TestTypes() {
   return {DT_FLOAT, DT_INT32, DT_INT64, DT_BOOL, DT_QUINT8, DT_COMPLEX64};
 }
@@ -303,7 +307,7 @@ class ContentImportTest : public ::testing::Test {
   std::vector<DataType<T>> ImportAndGetData(const NodeDef& node) {
     Model model;
     auto status = ImportNode(node, &model);
-    CHECK(status.ok()) << status.error_message();
+    CHECK(status.ok()) << status.message();
     const auto& array = model.GetArray("Node1");
     return array.GetBuffer<T>().data;
   }
@@ -332,6 +336,8 @@ TEST_F(ContentImportTest, Int32) {
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 2, 3, 4, 5, 5));
   RemoveTrailingElements(&node, 4);
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 1, 1, 1, 1, 1));
+  RemoveTrailingElements(&node, 1);
+  EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(0, 0, 0, 0, 0, 0));
 }
 
 TEST_F(ContentImportTest, Int64) {
@@ -345,6 +351,8 @@ TEST_F(ContentImportTest, Int64) {
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 2, 3, 4, 5, 5));
   RemoveTrailingElements(&node, 4);
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 1, 1, 1, 1, 1));
+  RemoveTrailingElements(&node, 1);
+  EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(0, 0, 0, 0, 0, 0));
 }
 
 TEST_F(ContentImportTest, Quint8) {
@@ -358,6 +366,8 @@ TEST_F(ContentImportTest, Quint8) {
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 2, 3, 4, 5, 5));
   RemoveTrailingElements(&node, 4);
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 1, 1, 1, 1, 1));
+  RemoveTrailingElements(&node, 1);
+  EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(0, 0, 0, 0, 0, 0));
 }
 
 TEST_F(ContentImportTest, Bool) {
@@ -371,6 +381,8 @@ TEST_F(ContentImportTest, Bool) {
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 0, 1, 0, 1, 1));
   RemoveTrailingElements(&node, 4);
   EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(1, 1, 1, 1, 1, 1));
+  RemoveTrailingElements(&node, 1);
+  EXPECT_THAT(ImportAndGetData<kType>(node), ElementsAre(0, 0, 0, 0, 0, 0));
 }
 
 TEST_F(ContentImportTest, Float) {
@@ -387,6 +399,9 @@ TEST_F(ContentImportTest, Float) {
   RemoveTrailingElements(&node, 4);
   EXPECT_THAT(ImportAndGetData<kType>(node),
               ElementsAre(1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000));
+  RemoveTrailingElements(&node, 1);
+  EXPECT_THAT(ImportAndGetData<kType>(node),
+              ElementsAre(0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000));
 }
 
 TEST_F(ContentImportTest, Complex64) {
@@ -414,6 +429,13 @@ TEST_F(ContentImportTest, Complex64) {
       ElementsAre(std::complex<float>(1.0000, -1.0000), cplx(1.0000, -1.0000),
                   cplx(1.0000, -1.0000), cplx(1.0000, -1.0000),
                   cplx(1.0000, -1.0000), cplx(1.0000, -1.0000)));
+
+  RemoveTrailingElements(&node, 1);
+  EXPECT_THAT(
+      ImportAndGetData<kType>(node),
+      ElementsAre(std::complex<float>(0.0000, 0.0000), cplx(0.0000, 0.0000),
+                  cplx(0.0000, 0.0000), cplx(0.0000, 0.0000),
+                  cplx(0.0000, 0.0000), cplx(0.0000, 0.0000)));
 }
 
 std::vector<std::pair<tensorflow::DataType, ArrayDataType>> UnaryTestTypes() {
@@ -428,7 +450,7 @@ class TensorContentTest : public ::testing::Test {
   std::vector<DataType<T>> ImportAndGetData(const NodeDef& node) {
     Model model;
     auto status = ImportNode(node, &model);
-    CHECK(status.ok()) << status.error_message();
+    CHECK(status.ok()) << status.message();
     const auto& nodearray = model.GetArray("Node1");
     return nodearray.GetBuffer<T>().data;
   }
@@ -444,7 +466,7 @@ class TensorContentTest : public ::testing::Test {
     SetAttrValue(dtype, &dtype_attr);
     (*node->mutable_attr())["dtype"] = dtype_attr;
 
-    auto allocated_content = absl::make_unique<T[]>(num_elements);
+    auto allocated_content = std::make_unique<T[]>(num_elements);
 
     // An attribute describing the content of this const node.
     tensorflow::TensorProto t;
