@@ -17,6 +17,7 @@ limitations under the License.
 #include <cstdlib>
 #include <memory>
 #include <set>
+#include <string>
 
 #include "absl/memory/memory.h"
 #include "absl/strings/str_join.h"
@@ -353,8 +354,7 @@ tensorflow::Status TransformWithStatus(const TocoFlags& toco_flags,
     // that didn't either have a min/max specified or get one set via
     // HardcodeMinMax or PropagateFakeQuantNumBits. This may require running
     // HardcodeMinMax to move changes through the graph as we make changes.
-    auto propagate_default_min_max =
-        absl::make_unique<PropagateDefaultMinMax>();
+    auto propagate_default_min_max = std::make_unique<PropagateDefaultMinMax>();
     bool has_default_ranges_flag = (toco_flags.has_default_ranges_min() &&
                                     toco_flags.has_default_ranges_max());
     if (has_default_ranges_flag) {
@@ -451,13 +451,13 @@ tensorflow::Status TransformWithStatus(const TocoFlags& toco_flags,
   CheckFinalDataTypesSatisfied(*model);
 
   // Estimate and log the number of arithmetic ops
-  int64 ops_count = 0;
+  int64_t ops_count = 0;
   if (EstimateArithmeticOpsCount(*model, &ops_count)) {
     LOG(INFO) << "Estimated count of arithmetic ops: " << ops_count
               << " ops, equivalently " << ops_count / 2 << " MACs";
   }
   model->ops_count = ops_count;
-  int64 params_count = 0;
+  int64_t params_count = 0;
 
   // Compute and log the number of parameters
   for (const auto& array_pair : model->GetArrayMap()) {
@@ -469,7 +469,7 @@ tensorflow::Status TransformWithStatus(const TocoFlags& toco_flags,
     params_count += RequiredBufferSizeForShape(array.shape());
   }
   LOG(INFO) << "Number of parameters: " << params_count;
-  return tensorflow::Status::OK();
+  return ::tensorflow::OkStatus();
 }
 
 tensorflow::Status Export(const TocoFlags& toco_flags, const Model& model,
@@ -486,7 +486,8 @@ tensorflow::Status Export(const TocoFlags& toco_flags, const Model& model,
           toco_flags.force_select_tf_ops() || toco_flags.enable_select_tf_ops();
       params.allow_custom_ops = allow_custom_ops;
       params.allow_dynamic_tensors = toco_flags.allow_dynamic_tensors();
-
+      params.disable_per_channel =
+          toco_flags.disable_per_channel_quantization();
       if (toco_flags.post_training_quantize()) {
         if (toco_flags.quantize_to_float16()) {
           params.quantize_weights = tflite::QuantizedBufferType::FLOAT16;
@@ -496,7 +497,7 @@ tensorflow::Status Export(const TocoFlags& toco_flags, const Model& model,
       }
       auto status = toco::tflite::Export(model, output_file_contents, params);
       if (!status.ok()) {
-        LOG(ERROR) << status.error_message();
+        LOG(ERROR) << status.message();
       }
       return status;
     } break;
