@@ -14,9 +14,11 @@ limitations under the License.
 ==============================================================================*/
 #include "tensorflow/lite/toco/tooling_util.h"
 
+#include <algorithm>
 #include <functional>
 #include <iterator>
 #include <set>
+#include <string>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
@@ -582,7 +584,7 @@ void DumpGraphvizVideoFrame(const Model& model) {
             dump_options.dump_graphviz,
             toco::port::StringF("toco_video_%05d.dot", dump_id)),
         graphviz_dump, port::file::Defaults());
-    QCHECK(result.ok()) << result.error_message();
+    QCHECK(result.ok()) << result.message();
     dump_id++;
   }
 }
@@ -602,7 +604,7 @@ void LogDump(int log_level, const std::string& message, const Model& model) {
             absl::StrCat("toco_", absl::StrReplaceAll(message, {{" ", "_"}}),
                          ".dot")),
         graphviz_dump, port::file::Defaults());
-    QCHECK(result.ok()) << result.error_message();
+    QCHECK(result.ok()) << result.message();
   }
 
   if (!VLOG_IS_ON(log_level)) {
@@ -1074,7 +1076,7 @@ void CheckEachArray(const Model& model) {
     // Check name.  Either "name_with_suffix_8", "name_with_port:3", but not
     // "name_with_both:3_8".
     const std::string& name = array_entry.first;
-    auto colon_pos = name.find_first_of(":");
+    auto colon_pos = name.find_first_of(':');
     if (colon_pos != std::string::npos) {
       CHECK_EQ(name.substr(colon_pos + 1).find_first_not_of("0123456789"),
                std::string::npos)
@@ -1769,6 +1771,8 @@ int ElementSize(ArrayDataType data_type) {
       return 8;
     case ArrayDataType::kComplex64:
       return 8;
+    case ArrayDataType::kComplex128:
+      return 16;
     case ArrayDataType::kFloat64:
       return 8;
 
@@ -1893,7 +1897,7 @@ std::string CreateInt32Array(Model* model, const std::string& param_name,
 }
 
 bool EstimateArithmeticOpsCount(const Model& model, const Operator& op,
-                                int64* result) {
+                                int64_t* result) {
   switch (op.type) {
     case OperatorType::kFullyConnected:
     case OperatorType::kConv:
@@ -1903,11 +1907,11 @@ bool EstimateArithmeticOpsCount(const Model& model, const Operator& op,
       if (!output_array.has_shape() || !weights_array.has_shape()) {
         return false;
       }
-      int64 cols = 1;
+      int64_t cols = 1;
       for (int i = 0; i < output_array.shape().dimensions_count() - 1; i++) {
         cols *= output_array.shape().dims(i);
       }
-      const int64 cost_per_col =
+      const int64_t cost_per_col =
           2 * RequiredBufferSizeForShape(weights_array.shape());
       *result = cost_per_col * cols;
       if (op.inputs.size() > 2) {
@@ -1952,7 +1956,7 @@ bool EstimateArithmeticOpsCount(const Model& model, const Operator& op,
         return false;
       }
       // AddN cost is roughly the same cost as N-1 Adds.
-      const int64 num_adds = op.inputs.size() - 1;
+      const int64_t num_adds = op.inputs.size() - 1;
       *result = num_adds * RequiredBufferSizeForShape(output_array.shape());
       break;
     }
@@ -1999,7 +2003,7 @@ bool EstimateArithmeticOpsCount(const Model& model, const Operator& op,
       }
       // The sum of squares requires (kheight*kwidth) multiply-adds,
       // and then there is the sqrt which we ballpark at 32 ops.
-      const int64 cost_per_val = 2 * maxpool->kheight * maxpool->kwidth + 32;
+      const int64_t cost_per_val = 2 * maxpool->kheight * maxpool->kwidth + 32;
       *result = RequiredBufferSizeForShape(output_array.shape()) * cost_per_val;
       break;
     }
@@ -2021,10 +2025,10 @@ bool EstimateArithmeticOpsCount(const Model& model, const Operator& op,
   return true;
 }
 
-bool EstimateArithmeticOpsCount(const Model& model, int64* result) {
-  int64 total = 0;
+bool EstimateArithmeticOpsCount(const Model& model, int64_t* result) {
+  int64_t total = 0;
   for (const auto& op : model.operators) {
-    int64 num_ops;
+    int64_t num_ops;
     if (!EstimateArithmeticOpsCount(model, *op, &num_ops)) {
       return false;
     }
@@ -2034,9 +2038,9 @@ bool EstimateArithmeticOpsCount(const Model& model, int64* result) {
   return true;
 }
 
-std::string FormattedNumber(int64 x) {
-  const int64 million = 1000000;
-  const int64 billion = 1000000000;
+std::string FormattedNumber(int64_t x) {
+  const int64_t million = 1000000;
+  const int64_t billion = 1000000000;
   if (x < 10000) {
     return toco::port::StringF("%d ", x);
   } else if (x < billion) {
@@ -2297,26 +2301,39 @@ ArrayDataType ConvertIODataTypeToArrayDataType(IODataType type) {
   switch (type) {
     case FLOAT:
       return ArrayDataType::kFloat;
+    case UINT8:
     case QUANTIZED_UINT8:
       return ArrayDataType::kUint8;
     case INT8:
+    case QUANTIZED_INT8:
       return ArrayDataType::kInt8;
+    case INT16:
     case QUANTIZED_INT16:
       return ArrayDataType::kInt16;
+    case UINT16:
+      return ArrayDataType::kUint16;
     case INT32:
       return ArrayDataType::kInt32;
+    case UINT32:
+      return ArrayDataType::kUint32;
     case INT64:
       return ArrayDataType::kInt64;
+    case UINT64:
+      return ArrayDataType::kUint64;
     case BOOL:
       return ArrayDataType::kBool;
     case STRING:
       return ArrayDataType::kString;
     case COMPLEX64:
       return ArrayDataType::kComplex64;
+    case COMPLEX128:
+      return ArrayDataType::kComplex128;
     case FLOAT16:
       return ArrayDataType::kFloat16;
     case FLOAT64:
       return ArrayDataType::kFloat64;
+    case RESOURCE:
+    case VARIANT:
     default:
       return ArrayDataType::kNone;
   }
