@@ -18,12 +18,12 @@ limitations under the License.
 
 // This could serve as the main function for all eval tools.
 int main(int argc, char* argv[]) {
-  auto task_executor = tflite::evaluation::CreateTaskExecutor(&argc, argv);
+  auto task_executor = tflite::evaluation::CreateTaskExecutor();
   if (task_executor == nullptr) {
     TFLITE_LOG(ERROR) << "Could not create the task evaluation!";
     return EXIT_FAILURE;
   }
-  const auto metrics = task_executor->Run();
+  const auto metrics = task_executor->Run(&argc, argv);
   if (!metrics.has_value()) {
     TFLITE_LOG(ERROR) << "Could not run the task evaluation!";
     return EXIT_FAILURE;
