@@ -19,27 +19,31 @@ limitations under the License.
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
+#include <utility>
 #include <vector>
 
+#include "absl/container/flat_hash_map.h"
 #include "absl/memory/memory.h"
-#include "tensorflow/lite/c/common.h"
 #include "tensorflow/lite/core/api/error_reporter.h"
 #include "tensorflow/lite/core/api/op_resolver.h"
-#include "tensorflow/lite/interpreter.h"
+#include "tensorflow/lite/core/c/common.h"
+#include "tensorflow/lite/core/interpreter.h"
+#include "tensorflow/lite/core/kernels/register.h"
+#include "tensorflow/lite/core/model.h"
 #include "tensorflow/lite/kernels/kernel_util.h"
-#include "tensorflow/lite/kernels/register.h"
-#include "tensorflow/lite/model.h"
+#include "tensorflow/lite/minimal_logging.h"
 #include "tensorflow/lite/op_resolver.h"
 #include "tensorflow/lite/schema/schema_generated.h"
+#include "tensorflow/lite/schema/schema_utils.h"
 #include "tensorflow/lite/stderr_reporter.h"
 #include "tensorflow/lite/string_util.h"
 #include "tensorflow/lite/tools/optimize/calibration/builtin_logging_ops/lstm.h"
 #include "tensorflow/lite/tools/optimize/calibration/calibration_common.h"
 #include "tensorflow/lite/tools/optimize/calibration/calibration_logger.h"
 #include "tensorflow/lite/tools/optimize/calibration/calibration_reader.h"
+#include "tensorflow/lite/tools/optimize/calibration/custom_logging_ops/lstm.h"
 #include "tensorflow/lite/tools/optimize/calibration/logging_op.h"
 #include "tensorflow/lite/tools/optimize/calibration/logging_op_resolver.h"
-#include "tensorflow/lite/tools/optimize/calibration/node_info_delegate.h"
 
 namespace tflite {
 namespace optimize {
@@ -63,7 +67,7 @@ class Calibrator {
       : node_ptr_opinfo_map_(node_ptr_opinfo_map),
         logging_op_resolver_(std::move(logging_op_resolver)),
         error_reporter_(error_reporter) {
-    logger_ = absl::make_unique<Logger>();
+    logger_ = std::make_unique<Logger>();
   }
 
   // Returns the wrapped kernel invoke function |TfLiteRegistration.invoke|.
@@ -80,6 +84,15 @@ class Calibrator {
     return node_ptr_opinfo_map_.at(node);
   }
 
+  std::vector<const TfLiteNode*> GetNodesUnderCalibration() {
+    std::vector<const TfLiteNode*> nodes;
+    nodes.reserve(node_ptr_opinfo_map_.size());
+    for (const auto& entry : node_ptr_opinfo_map_) {
+      nodes.push_back(entry.first);
+    }
+    return nodes;
+  }
+
  private:
   std::unordered_map<const TfLiteNode*, OperatorInfo> node_ptr_opinfo_map_;
   std::unique_ptr<LoggingOpResolver> logging_op_resolver_;
@@ -127,16 +140,21 @@ class GlobalCalibratorRegistry {
  public:
   // Get the |Calibrator| associated with given context, returns null if no
   // calibrator is associated with the given context.
-  Calibrator* GetCalibrator(const TfLiteContext* context) const {
-    if (calibrator_registry_.find(context) == calibrator_registry_.cend()) {
+  Calibrator* GetCalibrator(const TfLiteNode* node) const {
+    if (node_to_calibrator_.find(node) == node_to_calibrator_.cend()) {
       return nullptr;
     }
-    return calibrator_registry_.at(context).get();
+    return node_to_calibrator_.at(node);
   }
 
   // Removes the association between calibrator and context.
   // Note: This deletes the calibrator as well.
   void RemoveCalibrator(const TfLiteContext* context) {
+    Calibrator* calibrator = calibrator_registry_.at(context).get();
+    auto nodes = calibrator->GetNodesUnderCalibration();
+    for (auto node : nodes) {
+      node_to_calibrator_.erase(node);
+    }
     calibrator_registry_.erase(context);
   }
 
@@ -153,16 +171,20 @@ class GlobalCalibratorRegistry {
           "Failed to create calibrator, context already registered.");
       return kTfLiteError;
     }
-    auto calibrator = absl::make_unique<Calibrator>(
+    auto calibrator = std::make_unique<Calibrator>(
         node_to_opinfo, std::move(logging_op_resolver), reporter);
     calibrator_registry_[context] = std::move(calibrator);
     *calibrator_ptr = calibrator_registry_.at(context).get();
+    for (const auto& entry : node_to_opinfo) {
+      node_to_calibrator_[entry.first] = *calibrator_ptr;
+    }
     return kTfLiteOk;
   }
 
  private:
-  std::unordered_map<const TfLiteContext*, std::unique_ptr<Calibrator>>
+  absl::flat_hash_map<const TfLiteContext*, std::unique_ptr<Calibrator>>
       calibrator_registry_;
+  absl::flat_hash_map<const TfLiteNode*, Calibrator*> node_to_calibrator_;
 };
 
 GlobalCalibratorRegistry* GetCalibratorRegistry() {
@@ -174,22 +196,30 @@ GlobalCalibratorRegistry* GetCalibratorRegistry() {
 // TODO(jianlijianli): extend this to support multiple recipe for the same
 // model.
 logging_kernel_func_ptr GetLoggingEvalFunc(TfLiteContext* context,
-                                           TfLiteNode* node) {
-  const int lstm_number_input = 24;
-  if (node->inputs->size == lstm_number_input) {
-    // LSTM Op.
-    return tflite::optimize::calibration::builtin::lstm_logging_kernel;
+                                           TfLiteNode* node,
+                                           int builtin_op_code) {
+  switch (builtin_op_code) {
+    case BuiltinOperator_LSTM: {
+      if (node->intermediates->size == 12) {
+        return tflite::optimize::calibration::custom::lstm_logging_kernel;
+      }
+      return tflite::optimize::calibration::builtin::lstm_logging_kernel;
+    }
+    case BuiltinOperator_UNIDIRECTIONAL_SEQUENCE_LSTM:
+      return tflite::optimize::calibration::builtin::
+          unidirectional_sequence_lstm_logging_kernel;
+    default:
+      return nullptr;
   }
-  return nullptr;
 }
 
 // A wrapper implementation for |TfLiteRegistration.invoke| that logs inputs,
 // invokes the wrapped implementation and then logs the outputs.
 TfLiteStatus LoggingEval(TfLiteContext* context, TfLiteNode* node) {
-  Calibrator* calibrator = GetCalibratorRegistry()->GetCalibrator(context);
+  Calibrator* calibrator = GetCalibratorRegistry()->GetCalibrator(node);
 
   if (!calibrator) {
-    context->ReportError(context, "No calibrator found for context.");
+    TF_LITE_KERNEL_LOG(context, "No calibrator found for context.");
     return kTfLiteError;
   }
 
@@ -200,16 +230,19 @@ TfLiteStatus LoggingEval(TfLiteContext* context, TfLiteNode* node) {
 
   for (int i : op_info.loggable_inputs) {
     auto tensor = context->tensors[i];
-    TF_LITE_ENSURE_STATUS(logger->LogTensorValue(
-        i, tensor.data.f, tensor.bytes / sizeof(float), error_reporter));
+    TF_LITE_ENSURE_STATUS(
+        logger->LogTensorValue(op_info.subgraph_index, i, tensor.data.f,
+                               tensor.bytes / sizeof(float), error_reporter));
   }
-  auto kernel_invoke_intermediate = GetLoggingEvalFunc(context, node);
-  TfLiteStatus status;
+  auto builtin_op_code = calibrator->GetOpInfo(node).builtin_op_code;
+  auto kernel_invoke_intermediate =
+      GetLoggingEvalFunc(context, node, builtin_op_code);
   if (kernel_invoke_intermediate == nullptr) {
-    status = kernel_invoke(context, node);
+    TF_LITE_ENSURE_STATUS(kernel_invoke(context, node));
   } else {
-    status = kernel_invoke_intermediate(context, node, calibrator->GetLogger(),
-                                        error_reporter);
+    TF_LITE_ENSURE_STATUS(
+        kernel_invoke_intermediate(context, op_info.subgraph_index, node,
+                                   calibrator->GetLogger(), error_reporter));
   }
 
   // TODO(shashishekhar): An intermediate tensor in graph will get logged twice
@@ -221,17 +254,19 @@ TfLiteStatus LoggingEval(TfLiteContext* context, TfLiteNode* node) {
   // cell.
   for (int i : op_info.loggable_inputs) {
     auto tensor = context->tensors[i];
-    TF_LITE_ENSURE_STATUS(logger->LogTensorValue(
-        i, tensor.data.f, tensor.bytes / sizeof(float), error_reporter));
+    TF_LITE_ENSURE_STATUS(
+        logger->LogTensorValue(op_info.subgraph_index, i, tensor.data.f,
+                               tensor.bytes / sizeof(float), error_reporter));
   }
 
   for (int i : op_info.loggable_outputs) {
     auto tensor = context->tensors[i];
-    TF_LITE_ENSURE_STATUS(logger->LogTensorValue(
-        i, tensor.data.f, tensor.bytes / sizeof(float), error_reporter));
+    TF_LITE_ENSURE_STATUS(
+        logger->LogTensorValue(op_info.subgraph_index, i, tensor.data.f,
+                               tensor.bytes / sizeof(float), error_reporter));
   }
 
-  return status;
+  return kTfLiteOk;
 }
 
 // Returns the loggable tensors. Not all inputs and outputs need to be logged.
@@ -264,21 +299,29 @@ std::vector<int> GetLoggableTensorIndices(
 // This is done by querying the TfLiteContext for node and registrations using
 // the |NodeInfoDelegateObserver|.
 TfLiteStatus GetNodeOpInfoMapAndContext(
-    const std::unordered_map<int, OperatorInfo>& node_to_opinfo,
+    const absl::flat_hash_map<std::tuple<int, int>, OperatorInfo>&
+        node_to_opinfo,
     tflite::Interpreter* const interpreter,
     std::unordered_map<const TfLiteNode*, OperatorInfo>* node_ptr_opinfo_map,
-    const TfLiteContext** context) {
-  NodeInfoDelegateObserver delegate_observer(node_to_opinfo,
-                                             node_ptr_opinfo_map);
-  NodeInfoDelegateParams delegate_params;
-  delegate_params.delegate_observer = &delegate_observer;
-  TfLiteDelegate logging_delegate = CreateNodeInfoDelegate(&delegate_params);
+    TfLiteContext** context) {
+  *context = interpreter->primary_subgraph().context();
 
-  auto modify_status = interpreter->ModifyGraphWithDelegate(&logging_delegate);
-  if (modify_status != kTfLiteOk) {
-    return kTfLiteError;
+  // Since we only consider the primary subgraph while populating
+  // node_to_opinfo, do the same here.
+  // Because Flex delegate can merge multiple op nodes into one Delegate node if
+  // they are located in a row, the size of the execution plan can be lesser
+  // than the size of the graph's op nodes.
+  TF_LITE_ENSURE(*context,
+                 interpreter->execution_plan().size() <= node_to_opinfo.size());
+  for (const auto& entry : node_to_opinfo) {
+    auto op_info = entry.second;
+    int subgraph_index, op_index;
+    std::tie(subgraph_index, op_index) = entry.first;
+    const auto* node_and_reg =
+        interpreter->node_and_registration(subgraph_index, op_index);
+    op_info.registration = &node_and_reg->second;
+    node_ptr_opinfo_map->insert({&node_and_reg->first, op_info});
   }
-  *context = delegate_observer.GetContext();
   return kTfLiteOk;
 }
 
@@ -286,7 +329,7 @@ string GetOpName(const tflite::OperatorCode& opcode) {
   if (opcode.custom_code() != nullptr) {
     return opcode.custom_code()->str();
   }
-  return tflite::EnumNamesBuiltinOperator()[opcode.builtin_code()];
+  return tflite::EnumNamesBuiltinOperator()[GetBuiltinCode(&opcode)];
 }
 
 // A |CalibrationReader| that owns the Calibrator.
@@ -301,6 +344,30 @@ class Reader : public CalibrationReader {
   const TfLiteContext* context_;
 };
 
+bool HasInputs(BuiltinOperator code) {
+  switch (code) {
+    case BuiltinOperator_CALL_ONCE:
+    case BuiltinOperator_VAR_HANDLE:
+    // Custom ops, including Flex ops, might not have inputs.
+    case BuiltinOperator_CUSTOM:
+      return false;
+    default:
+      return true;
+  }
+}
+
+bool HasOutputs(BuiltinOperator code) {
+  switch (code) {
+    case BuiltinOperator_ASSIGN_VARIABLE:
+    case BuiltinOperator_CALL_ONCE:
+    // Custom ops, including Flex ops, might not have outputs.
+    case BuiltinOperator_CUSTOM:
+      return false;
+    default:
+      return true;
+  }
+}
+
 }  // namespace
 
 TfLiteStatus BuildLoggingInterpreter(
@@ -308,13 +375,15 @@ TfLiteStatus BuildLoggingInterpreter(
     std::unique_ptr<Interpreter>* interpreter,
     std::unique_ptr<CalibrationReader>* calibration_reader) {
   return BuildLoggingInterpreter(model.GetModel(), model.error_reporter(),
-                                 op_resolver, interpreter, calibration_reader);
+                                 op_resolver, interpreter, calibration_reader,
+                                 model.allocation());
 }
 
 TfLiteStatus BuildLoggingInterpreter(
     const tflite::Model* tflite_model, ErrorReporter* error_reporter,
     const OpResolver& op_resolver, std::unique_ptr<Interpreter>* interpreter,
-    std::unique_ptr<CalibrationReader>* calibration_reader) {
+    std::unique_ptr<CalibrationReader>* calibration_reader,
+    const Allocation* allocation) {
   if (error_reporter == nullptr) {
     // Make sure error_reporter is valid.
     error_reporter = DefaultErrorReporter();
@@ -322,66 +391,79 @@ TfLiteStatus BuildLoggingInterpreter(
   auto subgraphs = tflite_model->subgraphs();
   auto tensor_buffers = tflite_model->buffers();
 
-  if (subgraphs->size() != 1) {
-    error_reporter->Report(
-        "Only models with a single subgraph are supported, model had %d "
-        "subgraphs",
-        subgraphs->size());
-    return kTfLiteError;
-  }
-
   // Populate the node index to operator info map.
   // We want to collect this information so we can use it during runtime to
   // log details of which inputs and outputs.
   // At runtime TFLite kernel invoke functions can only look into their
   // own node in the graph (TFLiteNode*) and some limited context information.
-  auto primary_subgraph = subgraphs->Get(0);
-  auto operator_codes = tflite_model->operator_codes();
-  auto operators = primary_subgraph->operators();
-  auto tensors = primary_subgraph->tensors();
-  std::unordered_map<int, OperatorInfo> node_to_opinfo;
+  absl::flat_hash_map<std::tuple<int, int>, OperatorInfo> node_to_opinfo;
   BuiltinOpsSet builtin_op_and_versions;
   CustomOpsSet custom_op_and_versions;
 
-  for (size_t i = 0; i < operators->size(); i++) {
-    OperatorInfo op_info;
-    op_info.node_index = i;
-    auto op = operators->Get(i);
-    auto operator_code = operator_codes->Get(op->opcode_index());
-    op_info.builtin_op_code = operator_code->builtin_code();
-    op_info.name = GetOpName(*operator_code);
-    op_info.is_custom_op = operator_code->custom_code() != nullptr;
-    op_info.version = operator_code->version();
+  for (size_t subgraph_index = 0; subgraph_index < subgraphs->size();
+       subgraph_index++) {
+    auto subgraph = subgraphs->Get(subgraph_index);
+    auto operator_codes = tflite_model->operator_codes();
+    auto operators = subgraph->operators();
+    auto tensors = subgraph->tensors();
+    if (!operators) {
+      continue;
+    }
 
-    auto op_inputs = op->inputs();
-    auto op_outputs = op->outputs();
-    op_info.inputs = std::vector<int>(op_inputs->begin(), op_inputs->end());
-    op_info.outputs = std::vector<int>(op_outputs->begin(), op_outputs->end());
-    op_info.loggable_inputs =
-        GetLoggableTensorIndices(op_info.inputs, tensors, tensor_buffers);
-    op_info.loggable_outputs =
-        GetLoggableTensorIndices(op_info.outputs, tensors, tensor_buffers);
-    if (op_info.is_custom_op) {
-      op_info.registration =
-          op_resolver.FindOp(op_info.name.c_str(), operator_code->version());
-      custom_op_and_versions.insert(
-          {op_info.name.c_str(), operator_code->version()});
-    } else {
-      op_info.registration = op_resolver.FindOp(operator_code->builtin_code(),
-                                                operator_code->version());
-      builtin_op_and_versions.insert(
-          {op_info.builtin_op_code, operator_code->version()});
+    for (size_t i = 0; i < operators->size(); i++) {
+      OperatorInfo op_info;
+      op_info.subgraph_index = subgraph_index;
+      op_info.node_index = i;
+      auto op = operators->Get(i);
+      auto operator_code = operator_codes->Get(op->opcode_index());
+      op_info.builtin_op_code = GetBuiltinCode(operator_code);
+      op_info.name = GetOpName(*operator_code);
+      op_info.is_custom_op = operator_code->custom_code() != nullptr;
+      op_info.version = operator_code->version();
+
+      auto op_inputs = op->inputs();
+      auto op_outputs = op->outputs();
+      if (op_inputs) {
+        op_info.inputs = std::vector<int>(op_inputs->begin(), op_inputs->end());
+      } else if (HasInputs(op_info.builtin_op_code)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "Op %s missing inputs",
+                   op_info.name.c_str());
+      }
+      if (op_outputs) {
+        op_info.outputs =
+            std::vector<int>(op_outputs->begin(), op_outputs->end());
+      } else if (HasOutputs(op_info.builtin_op_code)) {
+        TFLITE_LOG(TFLITE_LOG_WARNING, "Op %s missing outputs",
+                   op_info.name.c_str());
+      }
+      op_info.loggable_inputs =
+          GetLoggableTensorIndices(op_info.inputs, tensors, tensor_buffers);
+      op_info.loggable_outputs =
+          GetLoggableTensorIndices(op_info.outputs, tensors, tensor_buffers);
+      if (op_info.is_custom_op) {
+        op_info.registration =
+            op_resolver.FindOp(op_info.name.c_str(), operator_code->version());
+        custom_op_and_versions.insert(
+            {op_info.name.c_str(), operator_code->version()});
+      } else {
+        op_info.registration = op_resolver.FindOp(GetBuiltinCode(operator_code),
+                                                  operator_code->version());
+        builtin_op_and_versions.insert(
+            {op_info.builtin_op_code, operator_code->version()});
+      }
+      std::tuple<int, int> key{subgraph_index, i};
+      node_to_opinfo[key] = op_info;
     }
-    node_to_opinfo[i] = op_info;
   }
 
   // Prepare the logging op resolver to use |LoggingEval| for kernel
   // invocations.
-  auto logging_op_resolver = absl::make_unique<LoggingOpResolver>(
+  auto logging_op_resolver = std::make_unique<LoggingOpResolver>(
       builtin_op_and_versions, custom_op_and_versions, op_resolver, LoggingEval,
       error_reporter);
-  tflite::InterpreterBuilder(tflite_model, *logging_op_resolver,
-                             error_reporter)(interpreter);
+  tflite::InterpreterBuilder(tflite_model, *logging_op_resolver, error_reporter,
+                             /*options_experimental=*/nullptr,
+                             allocation)(interpreter);
 
   if (!(*interpreter)) {
     error_reporter->Report("Failed to construct interpreter");
@@ -391,9 +473,9 @@ TfLiteStatus BuildLoggingInterpreter(
   // Compute the mapping between runtime and static graph structure, i.e.
   // (TfLiteContext, TfLiteNode) -> OperatorInfo
   std::unordered_map<const TfLiteNode*, OperatorInfo> node_ptr_opinfo_map;
-  const TfLiteContext* context = nullptr;
-  GetNodeOpInfoMapAndContext(node_to_opinfo, interpreter->get(),
-                             &node_ptr_opinfo_map, &context);
+  TfLiteContext* context = nullptr;
+  TF_LITE_ENSURE_STATUS(GetNodeOpInfoMapAndContext(
+      node_to_opinfo, interpreter->get(), &node_ptr_opinfo_map, &context));
 
   Calibrator* calibrator = nullptr;
   // Register a calibrator object for the context. This can be accessed
