@@ -15,7 +15,7 @@ limitations under the License.
 #ifndef TENSORFLOW_LITE_TOOLS_OPTIMIZE_CALIBRATION_LOGGING_OP_H_
 #define TENSORFLOW_LITE_TOOLS_OPTIMIZE_CALIBRATION_LOGGING_OP_H_
 
-#include "tensorflow/lite/c/common.h"
+#include "tensorflow/lite/core/c/common.h"
 #include "tensorflow/lite/tools/optimize/calibration/calibration_logger.h"
 
 namespace tflite {
@@ -23,6 +23,7 @@ namespace optimize {
 namespace calibration {
 
 typedef TfLiteStatus (*logging_kernel_func_ptr)(TfLiteContext* context,
+                                                const int subgraph_index,
                                                 TfLiteNode* node,
                                                 Logger* logger,
                                                 ErrorReporter* error_reporter);
