@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2017 The TensorFlow Authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,14 +14,12 @@
 # ==============================================================================
 """Tests for control_flow module."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import collections
 
 import numpy as np
 
+from tensorflow.python.autograph.converters import break_statements
+from tensorflow.python.autograph.converters import continue_statements
 from tensorflow.python.autograph.converters import control_flow
 from tensorflow.python.autograph.core import converter_testing
 from tensorflow.python.eager import def_function
@@ -34,33 +31,33 @@ from tensorflow.python.framework import tensor_util
 from tensorflow.python.platform import test
 from tensorflow.python.util import nest
 
-# TODO(mdan): These tests are not isolated - they also test the operators.
+
+for_unaffected_global = None
+for_mixed_globals_nonglobals = None
+for_test_global_local = None
 
 
 class ControlFlowTestBase(converter_testing.TestCase):
 
   def assertValuesEqual(self, actual, expected):
     values = nest.map_structure(
-        lambda x: self.evaluate(x) if tensor_util.is_tensor(x) else x,
+        lambda x: self.evaluate(x) if tensor_util.is_tf_type(x) else x,
         actual)
     self.assertAllEqual(values, expected)
 
-  def assertTransformedResult(self, test_fn, inputs, expected, symbols=None):
+  def assertTransformedResult(self, f, inputs, expected):
     if not isinstance(inputs, tuple):
       inputs = (inputs,)
-    if not symbols:
-      symbols = {}
-    with self.converted(test_fn, control_flow, symbols,
-                        (constant_op.constant,)) as result:
-      returns = result.test_fn(*inputs)
-      self.assertValuesEqual(returns, expected)
+    tr = self.transform(f, control_flow)
+    returns = tr(*inputs)
+    self.assertValuesEqual(returns, expected)
 
 
 class NestedControlFlowTest(ControlFlowTestBase):
 
   def test_basic(self):
 
-    def test_fn(n):
+    def f(n):
       i = 0
       j = 0
       s = 0
@@ -73,7 +70,26 @@ class NestedControlFlowTest(ControlFlowTestBase):
         j = 0
       return s, i, j, n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5),
+    self.assertTransformedResult(f, constant_op.constant(5),
+                                 (25, 5, 0, 5))
+
+  def test_mixed_globals_nonglobals(self):
+
+    def f(n):
+      global for_mixed_globals_nonglobals
+      i = 0
+      j = 0
+      for_mixed_globals_nonglobals = 0
+      while i < n:
+        while j < i:
+          j += 3
+        u = i + j  # 'u' is not defined within the inner loop
+        for_mixed_globals_nonglobals += u
+        i += 1
+        j = 0
+      return for_mixed_globals_nonglobals, i, j, n
+
+    self.assertTransformedResult(f, constant_op.constant(5),
                                  (25, 5, 0, 5))
 
   def test_composite_state_complex(self):
@@ -88,7 +104,7 @@ class NestedControlFlowTest(ControlFlowTestBase):
       def __init__(self, y):
         self.y = y
 
-    def test_fn(n):
+    def f(n):
       tc = TestClassX(TestClassY({'z': TestClassX(n)}))
       if n > 0:
         while n > 0:
@@ -97,19 +113,17 @@ class NestedControlFlowTest(ControlFlowTestBase):
           n -= 1
       return n, tc
 
-    with self.converted(test_fn, control_flow, {
-        'TestClassX': TestClassX,
-        'TestClassY': TestClassY,
-    }) as result:
-      n, tc = result.test_fn(constant_op.constant(5))
-      self.assertValuesEqual((n, tc.x.y['z'].x), (0, 6))
+    tr = self.transform(f, control_flow)
+
+    n, tc = tr(constant_op.constant(5))
+    self.assertValuesEqual((n, tc.x.y['z'].x), (0, 6))
 
 
 class WhileStatementTest(ControlFlowTestBase):
 
   def test_basic(self):
 
-    def test_fn(n):
+    def f(n):
       i = 0
       s = 0
       while i < n:
@@ -117,16 +131,16 @@ class WhileStatementTest(ControlFlowTestBase):
         i += 1
       return s, i, n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5), (10, 5, 5))
+    self.assertTransformedResult(f, constant_op.constant(5), (10, 5, 5))
 
   def test_single_output(self):
 
-    def test_fn(n):
+    def f(n):
       while n > 0:
         n -= 1
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5), 0)
+    self.assertTransformedResult(f, constant_op.constant(5), 0)
 
   def test_composite_state_attr(self):
 
@@ -135,19 +149,18 @@ class WhileStatementTest(ControlFlowTestBase):
       def __init__(self):
         self.x = constant_op.constant(3)
 
-    def test_fn(n):
+    def f(n):
       tc = TestClass()
       while n > 0:
         tc.x += 1
         n -= 1
       return n
 
-    self.assertTransformedResult(
-        test_fn, constant_op.constant(5), 0, symbols={'TestClass': TestClass})
+    self.assertTransformedResult(f, constant_op.constant(5), 0)
 
   def test_composite_state_slice(self):
 
-    def test_fn(n):
+    def f(n):
       d = {'a': n}
       k = 'a'
       while n > 0:
@@ -155,25 +168,25 @@ class WhileStatementTest(ControlFlowTestBase):
         n -= 1
       return d[k], n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5), (10, 0))
+    self.assertTransformedResult(f, constant_op.constant(5), (10, 0))
 
   def test_composite_state_literal_slice(self):
 
-    def test_fn(n):
+    def f(n):
       d = {'a': n}
       while n > 0:
         d['a'] += 1
         n -= 1
       return d['a'], n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5), (10, 0))
+    self.assertTransformedResult(f, constant_op.constant(5), (10, 0))
 
   def test_composite_state_attr_initialized_in_loop(self):
 
     class TestClass(object):
       pass
 
-    def test_fn(n, x):
+    def f(n, x):
       tc = TestClass()
       while n < 5:
         if n == 0:
@@ -183,19 +196,15 @@ class WhileStatementTest(ControlFlowTestBase):
         n += 1
       return tc.subattr
 
-    self.assertTransformedResult(
-        test_fn, (0, constant_op.constant(10)),
-        14,
-        symbols={'TestClass': TestClass})
-    with self.converted(
-        test_fn, control_flow, {'TestClass': TestClass}) as result:
-      with self.assertRaisesRegex(
-          ValueError, "'tc.subattr' must be defined before the loop"):
-        result.test_fn(constant_op.constant(0), 0)
+    self.assertTransformedResult(f, (0, constant_op.constant(10)), 14)
+    tr = self.transform(f, control_flow)
+    with self.assertRaisesRegex(
+        ValueError, "'tc.subattr' must be defined before the loop"):
+      tr(constant_op.constant(0), 0)
 
   def test_composite_state_slice_initialized_in_loop(self):
 
-    def test_fn(n, x):
+    def f(n, x):
       d = {}
       k = 'subkey'
       while n < 5:
@@ -206,16 +215,16 @@ class WhileStatementTest(ControlFlowTestBase):
         n += 1
       return d
 
-    self.assertTransformedResult(test_fn, (0, constant_op.constant(10)),
+    self.assertTransformedResult(f, (0, constant_op.constant(10)),
                                  {'subkey': 14})
-    with self.converted(test_fn, control_flow, {}) as result:
-      with self.assertRaisesRegex(
-          ValueError, r"'d\[k\]' must be defined before the loop"):
-        result.test_fn(constant_op.constant(0), 0)
+    tr = self.transform(f, control_flow)
+    with self.assertRaisesRegex(
+        ValueError, r"'d\[k\]' must be defined before the loop"):
+      tr(constant_op.constant(0), 0)
 
   def test_composite_state_literal_slice_initialized_in_loop(self):
 
-    def test_fn(n, x):
+    def f(n, x):
       d = {}
       while n < 5:
         if n == 0:
@@ -225,16 +234,16 @@ class WhileStatementTest(ControlFlowTestBase):
         n += 1
       return d
 
-    self.assertTransformedResult(test_fn, (0, constant_op.constant(10)),
+    self.assertTransformedResult(f, (0, constant_op.constant(10)),
                                  {'subkey': 14})
-    with self.converted(test_fn, control_flow, {}) as result:
-      with self.assertRaisesRegex(
-          ValueError, r"'d\['subkey'\]' must be defined before the loop"):
-        result.test_fn(constant_op.constant(0), 0)
+    tr = self.transform(f, control_flow)
+    with self.assertRaisesRegex(
+        ValueError, r"'d\['subkey'\]' must be defined before the loop"):
+      tr(constant_op.constant(0), 0)
 
   def test_composite_state_slice_aliased_to_local(self):
 
-    def test_fn(n, x):
+    def f(n, x):
       d = {}
       while n < 5:
         k = 'subkey'
@@ -242,15 +251,15 @@ class WhileStatementTest(ControlFlowTestBase):
         n += 1
       return d
 
-    self.assertTransformedResult(test_fn, (0, constant_op.constant(10)),
+    self.assertTransformedResult(f, (0, constant_op.constant(10)),
                                  {'subkey': 11})
-    with self.converted(test_fn, control_flow, {}) as result:
-      # TODO(b/136999953): Better error message.
-      # Note that this error happens at execution time.
-      with self.assertRaises(errors.InaccessibleTensorError):
-        graph_fn = def_function.function(result.test_fn, autograph=False)
-        self.evaluate(
-            graph_fn(constant_op.constant(0), constant_op.constant(5)))
+    tr = self.transform(f, control_flow)
+    # TODO(b/136999953): Better error message.
+    # Note that this error happens at execution time.
+    with self.assertRaises(errors.InaccessibleTensorError):
+      graph_fn = def_function.function(tr, autograph=False)
+      self.evaluate(
+          graph_fn(constant_op.constant(0), constant_op.constant(5)))
 
   def test_local_composite_attr(self):
 
@@ -259,19 +268,18 @@ class WhileStatementTest(ControlFlowTestBase):
       def __init__(self):
         self.x = constant_op.constant(3)
 
-    def test_fn(n):
+    def f(n):
       while n > 0:
         tc = TestClass()
         tc.x = tc.x
         n -= 1
       return n
 
-    self.assertTransformedResult(
-        test_fn, constant_op.constant(5), 0, symbols={'TestClass': TestClass})
+    self.assertTransformedResult(f, constant_op.constant(5), 0)
 
   def test_local_composite_slice(self):
 
-    def test_fn(n):
+    def f(n):
       while n > 0:
         d = {'x': n}
         k = 'x'
@@ -279,26 +287,26 @@ class WhileStatementTest(ControlFlowTestBase):
         n -= 1
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5), 0, {})
+    self.assertTransformedResult(f, constant_op.constant(5), 0)
 
   def test_local_composite_literal_slice(self):
 
-    def test_fn(n):
+    def f(n):
       while n > 0:
         d = {'x': n}
         d['x'] = d['x']
         n -= 1
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(5), 0, {})
+    self.assertTransformedResult(f, constant_op.constant(5), 0)
 
   def test_non_tensor_state(self):
 
-    # This class is ok to be in a tf.while_loop's state.
+    # This class is ok to be in a tf.while's state.
     class TestClass(collections.namedtuple('TestClass', ('x'))):
       pass
 
-    def test_fn(n):
+    def f(n):
       tc = TestClass([constant_op.constant(0)])
       while n > 0:
         tc = TestClass([constant_op.constant(3)])
@@ -306,9 +314,7 @@ class WhileStatementTest(ControlFlowTestBase):
         n -= 1
       return tc.x[0]
 
-    ns = {'TestClass': TestClass, 'constant_op': constant_op}
-    self.assertTransformedResult(
-        test_fn, constant_op.constant(5), 4, symbols=ns)
+    self.assertTransformedResult(f, constant_op.constant(5), 4)
 
   def test_non_tensor_state_illegal_type(self):
 
@@ -317,20 +323,20 @@ class WhileStatementTest(ControlFlowTestBase):
       def __init__(self):
         self.x = [constant_op.constant(3)]
 
-    def test_fn(n):
+    def f(n):
       while n > 0:
         tc = TestClass()
         tc.x[0] = tc.x[0] + 1
         n -= 1
       return tc.x[0]
 
-    with self.converted(
-        test_fn, control_flow, {'TestClass': TestClass}) as result:
-      # The tested function would require `tc` to become part of the while loop
-      # state, but TensorFlow doesn't support classes at the moment.
-      with self.assertRaisesRegexp(
-          ValueError, 'tc.*must be defined before the loop'):
-        result.test_fn(constant_op.constant(5))
+    tr = self.transform(f, control_flow)
+
+    # The tested function would require `tc` to become part of the while loop
+    # state, but TensorFlow doesn't support classes at the moment.
+    with self.assertRaisesRegex(
+        ValueError, 'tc.*must be defined before the loop'):
+      tr(constant_op.constant(5))
 
   def test_dispatches_by_cond_only(self):
 
@@ -343,27 +349,27 @@ class WhileStatementTest(ControlFlowTestBase):
       def __add__(self, other):
         return TensorIncompatibleNumeric(self.val + other)
 
-    def test_fn(n, s):
+    def f(n, s):
       while n > 0:
         n -= 1
         s += n
       return s
 
-    self.assertTransformedResult(test_fn, (constant_op.constant(5), 0), 10)
-    with self.converted(test_fn, control_flow, {}) as result:
-      # n alone controls the staging. When the loop is not staged, Python
-      # knows how to add the two objects. But when staged, tf.while_loop will
-      # not know how to deal with the TensorIncompatibleNumeric object.
-      self.assertEqual(result.test_fn(5, TensorIncompatibleNumeric(0)).val, 10)
-      with self.assertRaises(TypeError):
-        result.test_fn(constant_op.constant(5), TensorIncompatibleNumeric(0))
+    self.assertTransformedResult(f, (constant_op.constant(5), 0), 10)
+    tr = self.transform(f, control_flow)
+    # n alone controls the staging. When the loop is not staged, Python
+    # knows how to add the two objects. But when staged, tf.while will
+    # not know how to deal with the TensorIncompatibleNumeric object.
+    self.assertEqual(tr(5, TensorIncompatibleNumeric(0)).val, 10)
+    with self.assertRaises(TypeError):
+      tr(constant_op.constant(5), TensorIncompatibleNumeric(0))
 
 
 class IfStatementTest(ControlFlowTestBase):
 
   def test_basic(self):
 
-    def test_fn(n):
+    def f(n):
       a = 0
       b = 0
       if n > 0:
@@ -372,20 +378,20 @@ class IfStatementTest(ControlFlowTestBase):
         b = 2 * n
       return a, b
 
-    self.assertTransformedResult(test_fn, constant_op.constant(1), (-1, 0))
-    self.assertTransformedResult(test_fn, constant_op.constant(-1), (0, -2))
+    self.assertTransformedResult(f, constant_op.constant(1), (-1, 0))
+    self.assertTransformedResult(f, constant_op.constant(-1), (0, -2))
 
   def test_sparse_tensor(self):
 
-    def test_fn(cond, a):
+    def f(cond, a):
       if cond:
         a = -a
       return a
 
     st = sparse_tensor.SparseTensor(
         indices=((0,),), values=(0,), dense_shape=(1,))
-    self.assertTransformedResult(test_fn, (st, constant_op.constant(1)), -1)
-    self.assertTransformedResult(test_fn, (None, constant_op.constant(1)), 1)
+    self.assertTransformedResult(f, (st, constant_op.constant(1)), -1)
+    self.assertTransformedResult(f, (None, constant_op.constant(1)), 1)
 
   def test_complex_outputs(self):
 
@@ -395,7 +401,7 @@ class IfStatementTest(ControlFlowTestBase):
         self.a = a
         self.b = b
 
-    def test_fn(n, obj):
+    def f(n, obj):
       obj.a = 0
       obj.b = 0
       if n > 0:
@@ -404,94 +410,111 @@ class IfStatementTest(ControlFlowTestBase):
         obj.b = 2 * n
       return obj
 
-    with self.converted(test_fn, control_flow, {}) as result:
-      res_obj = result.test_fn(constant_op.constant(1), TestClass(0, 0))
-      self.assertValuesEqual((res_obj.a, res_obj.b), (-1, 0))
-      res_obj = result.test_fn(constant_op.constant(-1), TestClass(0, 0))
-      self.assertValuesEqual((res_obj.a, res_obj.b), (0, -2))
+    tr = self.transform(f, control_flow)
+
+    res_obj = tr(constant_op.constant(1), TestClass(0, 0))
+    self.assertValuesEqual((res_obj.a, res_obj.b), (-1, 0))
+    res_obj = tr(constant_op.constant(-1), TestClass(0, 0))
+    self.assertValuesEqual((res_obj.a, res_obj.b), (0, -2))
 
   def test_single_output(self):
 
-    def test_fn(n):
+    def f(n):
       if n > 0:
         n = -n
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(1), -1)
+    self.assertTransformedResult(f, constant_op.constant(1), -1)
 
   def test_unbalanced(self):
 
-    def test_fn(n):
+    def f(n):
       if n > 0:
         n = 3
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(2), 3)
-    self.assertTransformedResult(test_fn, constant_op.constant(-3), -3)
+    self.assertTransformedResult(f, constant_op.constant(2), 3)
+    self.assertTransformedResult(f, constant_op.constant(-3), -3)
 
   def test_unbalanced_raising(self):
 
-    def test_fn(n):
+    def f(n):
       if n > 0:
         n = n + 1
         raise ValueError()
       return n
 
-    self.assertTransformedResult(test_fn, -3, -3)
+    self.assertTransformedResult(f, -3, -3)
 
-    with self.converted(test_fn, control_flow, {}) as result:
-      with self.assertRaises(ValueError):
-        result.test_fn(1)
+    tr = self.transform(f, control_flow)
+
+    with self.assertRaises(ValueError):
+      tr(1)
 
   def test_local_var(self):
 
-    def test_fn(n):
+    def f(n):
       if n > 0:
         b = 4
         n = b + 1
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(1), 5)
-    self.assertTransformedResult(test_fn, constant_op.constant(-1), -1)
+    self.assertTransformedResult(f, constant_op.constant(1), 5)
+    self.assertTransformedResult(f, constant_op.constant(-1), -1)
 
   def test_local_remains_local(self):
 
-    def test_fn(n):
+    def f(n):
       if n > 0:
         b = 4
         n = b + 1
       return n
 
-    self.assertTransformedResult(test_fn, constant_op.constant(1), 5)
-    self.assertTransformedResult(test_fn, constant_op.constant(-1), -1)
+    self.assertTransformedResult(f, constant_op.constant(1), 5)
+    self.assertTransformedResult(f, constant_op.constant(-1), -1)
+
+  def test_global_local(self):
+
+    def f(n):
+      if n > 0:
+        global for_test_global_local
+        if for_test_global_local is None:
+          for_test_global_local = 1
+        else:
+          for_test_global_local += 1
+        n += for_test_global_local
+      return n
+
+    tr = self.transform(f, control_flow)
+    assert for_test_global_local is None
+    self.assertEqual(tr(1), 2)
+    self.assertEqual(for_test_global_local, 1)
 
   def test_no_outputs(self):
 
-    def test_fn(n):
+    def f(n):
       if n > 0:
         b = 4  # pylint:disable=unused-variable
       return n
 
-    # Without side effect guards, the if statement will stage a cond,
-    # but that will be pruned at execution.
-    self.assertTransformedResult(test_fn, constant_op.constant(1), 1)
-    self.assertTransformedResult(test_fn, constant_op.constant(-1), -1)
+    self.assertTransformedResult(f, constant_op.constant(1), 1)
+    self.assertTransformedResult(f, constant_op.constant(-1), -1)
 
   def test_created_outputs(self):
 
-    def test_fn(i):
+    def f(i):
       if i == 0:
         result = i - 1
       else:
         result = i + 1
       return result
 
-    self.assertTransformedResult(test_fn, 0, -1)
-    self.assertTransformedResult(test_fn, 1, 2)
+    self.assertTransformedResult(f, 0, -1)
+    self.assertTransformedResult(f, 1, 2)
 
   def test_created_loop_local_outputs(self):
 
-    def test_fn(n, x):
+    def f(n, x):
       for i in n:
         if i == 0:
           result = i - 1
@@ -501,11 +524,11 @@ class IfStatementTest(ControlFlowTestBase):
           x += 1
       return x
 
-    self.assertTransformedResult(test_fn, (range(5), 10), 14)
+    self.assertTransformedResult(f, (range(5), 10), 14)
 
   def test_created_loop_variable(self):
 
-    def test_fn(n, x):
+    def f(n, x):
       for i in n:
         if i == 0:
           result = i - 1
@@ -514,22 +537,26 @@ class IfStatementTest(ControlFlowTestBase):
             x += 1
       return x
 
-    self.assertTransformedResult(test_fn, (range(5), 10), 14)
+    self.assertTransformedResult(f, (range(5), 10), 14)
 
   def test_unaffected_global(self):
 
-    def test_fn(i):
-      global g  # pylint:disable=global-variable-undefined
-      if i == 0:
-        g = i - 1
-      return g
+    global for_unaffected_global
+    for_unaffected_global = 3
 
-    self.assertTransformedResult(test_fn, 1, 3, symbols={'g': 3})
-    self.assertTransformedResult(test_fn, 0, -1, symbols={'g': 3})
+    def f(i):
+      global for_unaffected_global
+      if i == 0:
+        for_unaffected_global = i - 1
+      return for_unaffected_global
+
+    self.assertTransformedResult(f, 1, 3)
+    self.assertTransformedResult(f, 0, -1)
+    self.assertEqual(for_unaffected_global, -1)
 
   def test_unaffected_nonlocal(self):
 
-    def test_fn(i):
+    def f(i):
       def inner_fn():
         nonlocal n
         if i == 0:
@@ -539,12 +566,12 @@ class IfStatementTest(ControlFlowTestBase):
       inner_fn()
       return n
 
-    self.assertTransformedResult(test_fn, 1, 3)
-    self.assertTransformedResult(test_fn, 0, -1)
+    self.assertTransformedResult(f, 1, 3)
+    self.assertTransformedResult(f, 0, -1)
 
   def test_output_defined_in_prior_except(self):
 
-    def test_fn(i):
+    def f(i):
       try:
         raise ValueError()
       except ValueError:
@@ -553,8 +580,8 @@ class IfStatementTest(ControlFlowTestBase):
         x = i - 1
       return x
 
-    self.assertTransformedResult(test_fn, 1, 1)
-    self.assertTransformedResult(test_fn, 0, -1)
+    self.assertTransformedResult(f, 1, 1)
+    self.assertTransformedResult(f, 0, -1)
 
   def test_unbalanced_multiple_composites(self):
 
@@ -564,7 +591,7 @@ class IfStatementTest(ControlFlowTestBase):
         self.b = 2
         self.c = 3
 
-    def test_fn(x, condition):
+    def f(x, condition):
 
       z = 5
       if condition:
@@ -574,9 +601,9 @@ class IfStatementTest(ControlFlowTestBase):
 
       return x.b, x.c, z
 
-    self.assertTransformedResult(test_fn, (Foo(), constant_op.constant(True)),
+    self.assertTransformedResult(f, (Foo(), constant_op.constant(True)),
                                  (7, 11, 13))
-    self.assertTransformedResult(test_fn, (Foo(), constant_op.constant(False)),
+    self.assertTransformedResult(f, (Foo(), constant_op.constant(False)),
                                  (2, 3, 5))
 
   def test_unbalanced_composite(self):
@@ -586,7 +613,7 @@ class IfStatementTest(ControlFlowTestBase):
       def __init__(self):
         self.b = 2
 
-    def test_fn(x, condition):
+    def f(x, condition):
 
       z = 5
       if condition:
@@ -595,9 +622,9 @@ class IfStatementTest(ControlFlowTestBase):
 
       return x.b, z
 
-    self.assertTransformedResult(test_fn, (Foo(), constant_op.constant(True)),
+    self.assertTransformedResult(f, (Foo(), constant_op.constant(True)),
                                  (7, 13))
-    self.assertTransformedResult(test_fn, (Foo(), constant_op.constant(False)),
+    self.assertTransformedResult(f, (Foo(), constant_op.constant(False)),
                                  (2, 5))
 
 
@@ -605,7 +632,7 @@ class ForStatementTest(ControlFlowTestBase):
 
   def test_basic(self):
 
-    def test_fn(l):
+    def f(l):
       s1 = 0
       s2 = 0
       for e in l:
@@ -613,21 +640,21 @@ class ForStatementTest(ControlFlowTestBase):
         s2 += e * e
       return s1, s2
 
-    self.assertTransformedResult(test_fn, constant_op.constant([1, 3]), (4, 10))
+    self.assertTransformedResult(f, constant_op.constant([1, 3]), (4, 10))
     empty_vector = constant_op.constant([], shape=(0,), dtype=dtypes.int32)
-    self.assertTransformedResult(test_fn, empty_vector, (0, 0))
+    self.assertTransformedResult(f, empty_vector, (0, 0))
 
   def test_single_output(self):
 
-    def test_fn(l):
+    def f(l):
       s = 0
       for e in l:
         s += e
       return s
 
-    self.assertTransformedResult(test_fn, constant_op.constant([1, 3]), 4)
+    self.assertTransformedResult(f, constant_op.constant([1, 3]), 4)
     empty_vector = constant_op.constant([], shape=(0,), dtype=dtypes.int32)
-    self.assertTransformedResult(test_fn, empty_vector, 0)
+    self.assertTransformedResult(f, empty_vector, 0)
 
   def test_iterated_expression(self):
 
@@ -637,26 +664,23 @@ class ForStatementTest(ControlFlowTestBase):
       eval_count[0] += 1
       return x
 
-    def test_fn(n):
+    def f(n):
       s = 0
       for e in count_evals(range(n)):
         s += e
       return s
 
-    ns = {'count_evals': count_evals}
-    node, ctx = self.prepare(test_fn, ns)
-    node = control_flow.transform(node, ctx)
+    tr = self.transform(f, control_flow)
 
-    with self.compiled(node, ns) as result:
-      self.assertEqual(result.test_fn(5), 10)
-      self.assertEqual(eval_count[0], 1)
+    self.assertEqual(tr(5), 10)
+    self.assertEqual(eval_count[0], 1)
 
   def test_composite_state_initialized_in_loop(self):
 
     class TestClass(object):
       pass
 
-    def test_fn(n, x):
+    def f(n, x):
       tc = TestClass()
       for i in n:
         if i == 0:
@@ -665,37 +689,97 @@ class ForStatementTest(ControlFlowTestBase):
           tc.x = tc.x + i
       return tc.x
 
-    self.assertTransformedResult(
-        test_fn, (range(5), constant_op.constant(10)),
-        20,
-        symbols={'TestClass': TestClass})
-    with self.converted(
-        test_fn, control_flow, {'TestClass': TestClass}) as result:
-      with self.assertRaisesRegex(
-          ValueError, "'tc.x' must be defined before the loop"):
-        result.test_fn(constant_op.constant(list(range(5))), 0)
+    self.assertTransformedResult(f, (range(5), constant_op.constant(10)), 20)
+    tr = self.transform(f, control_flow)
+
+    with self.assertRaisesRegex(
+        ValueError, "'tc.x' must be defined before the loop"):
+      tr(constant_op.constant(list(range(5))), 0)
 
   def test_tuple_unpacking(self):
-    def test_fn(x_list):
-      z = tf.constant(0)  # pylint:disable=undefined-variable
+
+    def f(x_list):
+      z = constant_op.constant(0)  # pylint:disable=undefined-variable
       for i, x in enumerate(x_list):
         z = z + x + i
       return z
 
-    self.assertTransformedResult(test_fn, [3, 3], 7)
+    self.assertTransformedResult(f, [3, 3], 7)
 
   def test_with_comprehension_in_body(self):
 
-    def test_fn(l, n):
+    def f(l, n):
       s = constant_op.constant(list(range(n)))
       for _ in l:
         s += constant_op.constant([a for a in range(n)])
       return s
 
-    self.assertTransformedResult(
-        test_fn, (constant_op.constant([1, 2, 3]), 5),
-        np.array(range(5)) * 4,
-        symbols={'constant_op': constant_op})
+    self.assertTransformedResult(f, (constant_op.constant([1, 2, 3]), 5),
+                                 np.array(range(5)) * 4)
+
+
+class AdvancedControlFlowTest(ControlFlowTestBase):
+
+  def assertTransformedEquivalent(self, f, *inputs):
+    tr = self.transform(
+        f, (break_statements, continue_statements, control_flow))
+    self.assertEqual(f(*inputs), tr(*inputs))
+
+  def test_while_with_else(self):
+
+    def f(x):
+      while x > 2:
+        x /= 2
+      else:
+        x += 1
+      return x
+
+    self.assertTransformedEquivalent(f, 4)
+    self.assertTransformedEquivalent(f, 2)
+
+  def test_while_with_else_and_break(self):
+
+    def f(cond1):
+      x = 8
+      while x > 2:
+        x /= 2
+        if cond1:
+          break
+      else:
+        x += 1
+      return x
+
+    self.assertTransformedEquivalent(f, True)
+    self.assertTransformedEquivalent(f, False)
+
+  def test_for_with_else(self):
+
+    def f(l):
+      res = 0
+      for x in l:
+        res += x
+      else:
+        res += 1
+      return res
+
+    self.assertTransformedEquivalent(f, [])
+    self.assertTransformedEquivalent(f, [1, 2])
+
+  def test_for_with_else_and_break(self):
+
+    def f(flag):
+      l = [1, 2, 3]
+      res = 0
+      for x in l:
+        res += x
+        if flag:
+          break
+      else:
+        res += 1
+      return res
+
+    self.assertTransformedEquivalent(f, True)
+    self.assertTransformedEquivalent(f, False)
 
 
 if __name__ == '__main__':
