@@ -14,12 +14,7 @@
 # ==============================================================================
 """Tests for activity module."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import gast
-import six
 
 from tensorflow.python.autograph.pyct import anno
 from tensorflow.python.autograph.pyct import naming
@@ -69,11 +64,25 @@ class ScopeTest(test.TestCase):
 
     self.assertMissing(QN('bar'), scope)
 
-    scope.modified.add(QN('bar'))
+  def test_merge_from(self):
+    scope = activity.Scope(None)
+    other = activity.Scope(None)
+
+    for col in (scope.modified, scope.read, scope.bound, scope.deleted):
+      col.add(QN('foo'))
+
+    for col in (other.modified, other.read, other.bound, other.deleted):
+      col.add(QN('foo'))
+      col.add(QN('bar'))
+
     scope.merge_from(other)
 
-    self.assertWriteOnly(QN('bar'), scope)
-    self.assertMissing(QN('bar'), other)
+    self.assertReadWrite(QN('foo'), scope)
+    self.assertReadWrite(QN('bar'), scope)
+    self.assertIn(QN('foo'), scope.bound)
+    self.assertIn(QN('bar'), scope.bound)
+    self.assertIn(QN('foo'), scope.deleted)
+    self.assertIn(QN('bar'), scope.deleted)
 
   def test_copy_of(self):
     scope = activity.Scope(None)
@@ -715,10 +724,7 @@ class ActivityAnalyzerTest(ActivityAnalyzerTestBase):
     node, _ = self._parse_and_analyze(test_fn)
     fn_node = node
     body_scope = anno.getanno(fn_node, NodeAnno.BODY_SCOPE)
-    if six.PY2:
-      self.assertScopeIs(body_scope, ('a',), ('b', 'c'))
-    else:
-      self.assertScopeIs(body_scope, ('a',), ('b',))
+    self.assertScopeIs(body_scope, ('a',), ('b',))
 
   def test_comprehension_targets_are_isolated_in_augassign(self):
 
@@ -728,10 +734,7 @@ class ActivityAnalyzerTest(ActivityAnalyzerTestBase):
     node, _ = self._parse_and_analyze(test_fn)
     fn_node = node
     body_scope = anno.getanno(fn_node, NodeAnno.BODY_SCOPE)
-    if six.PY2:
-      self.assertScopeIs(body_scope, ('a', 'b'), ('b', 'c'))
-    else:
-      self.assertScopeIs(body_scope, ('a', 'b'), ('b',))
+    self.assertScopeIs(body_scope, ('a', 'b'), ('b',))
 
   def test_comprehension_generator_order(self):
 
@@ -759,6 +762,79 @@ class ActivityAnalyzerTest(ActivityAnalyzerTestBase):
     global_a_scope = anno.getanno(fn_node.body[0], anno.Static.SCOPE)
     self.assertScopeIs(global_a_scope, ('global_a',), ())
 
+  def test_nonlocal_symbol(self):
+    nonlocal_a = 3
+    nonlocal_b = 13
+
+    def test_fn(c):
+      nonlocal nonlocal_a
+      nonlocal nonlocal_b
+      nonlocal_a = nonlocal_b + c
+
+    node, _ = self._parse_and_analyze(test_fn)
+    fn_node = node
+    body_scope = anno.getanno(fn_node, NodeAnno.BODY_SCOPE)
+    self.assertScopeIs(
+        body_scope, ('nonlocal_a', 'nonlocal_b', 'c'), ('nonlocal_a',))
+    nonlocal_a_scope = anno.getanno(fn_node.body[0], anno.Static.SCOPE)
+    self.assertScopeIs(nonlocal_a_scope, ('nonlocal_a',), ())
+
+  def test_annotated_assign(self):
+    b = int
+
+    def test_fn(c):
+      a: b = c
+      return a
+
+    node, _ = self._parse_and_analyze(test_fn)
+    fn_node = node
+
+    body_scope = anno.getanno(fn_node, NodeAnno.BODY_SCOPE)
+    self.assertScopeIs(body_scope, ('b', 'c', 'a'), ('a',))
+    self.assertSymbolSetsAre(('b',), body_scope.annotations, 'annotations')
+
+    ann_assign_scope = anno.getanno(fn_node.body[0], anno.Static.SCOPE)
+    self.assertScopeIs(ann_assign_scope, ('b', 'c'), ('a',))
+    self.assertSymbolSetsAre(
+        ('b',), ann_assign_scope.annotations, 'annotations')
+
+  def test_pure_definition(self):
+    b = int
+
+    def test_fn():
+      a: b
+      return a
+
+    node, _ = self._parse_and_analyze(test_fn)
+    fn_node = node
+
+    body_scope = anno.getanno(fn_node, NodeAnno.BODY_SCOPE)
+    self.assertScopeIs(body_scope, ('b', 'a'), ('a',))
+    self.assertSymbolSetsAre(('b',), body_scope.annotations, 'annotations')
+
+    ann_assign_scope = anno.getanno(fn_node.body[0], anno.Static.SCOPE)
+    self.assertScopeIs(ann_assign_scope, ('b',), ('a',))
+    self.assertSymbolSetsAre(
+        ('b',), ann_assign_scope.annotations, 'annotations')
+
+  def test_function_def_annotations(self):
+    b = int
+    c = int
+
+    def test_fn(a: b) -> c:
+      return a
+
+    node, _ = self._parse_and_analyze(test_fn)
+    fn_node = node
+
+    fn_scope = anno.getanno(fn_node, anno.Static.SCOPE)
+    self.assertScopeIs(fn_scope, ('b', 'c'), ('test_fn',))
+    self.assertSymbolSetsAre(('b', 'c'), fn_scope.annotations, 'annotations')
+
+    body_scope = anno.getanno(fn_node, NodeAnno.BODY_SCOPE)
+    self.assertScopeIs(body_scope, ('a',), ())
+    self.assertSymbolSetsAre((), body_scope.annotations, 'annotations')
+
   def test_class_definition_basic(self):
 
     def test_fn(a, b):
