@@ -93,7 +93,8 @@ RunOptions* kEmptyRunOptions() {
 class SessionLogger {
  public:
   SessionLogger() {
-    std::string log_name = getenv("TF_REPLAY_LOG_FILE");
+    const char* log_file_env = getenv("TF_REPLAY_LOG_FILE");
+    std::string log_name = log_file_env ? std::string(log_file_env) : ".";
     LOG(INFO) << "Constructing new session logger for " << log_name;
     TF_CHECK_OK(
         Env::Default()->RecursivelyCreateDir(string(io::Dirname(log_name))));
@@ -146,7 +147,7 @@ class SessionLogger {
     // Build an index from fetch tensor name to first index in
     // output_tensor_names.
     std::unordered_map<string, int> output_name_to_offset;
-    for (int i = 0; i < output_tensor_names.size(); ++i) {
+    for (int i = 0, end = output_tensor_names.size(); i < end; ++i) {
       const string& name = output_tensor_names[i];
       if (output_name_to_offset.insert(std::make_pair(name, i)).second) {
         req->add_fetch(name);
@@ -393,7 +394,7 @@ SessionRef::~SessionRef() = default;
 Status SessionRef::CheckNotClosed() {
   mutex_lock l(run_lock_);
   if (session_ == nullptr) return errors::Cancelled("Session has been closed.");
-  return ::tensorflow::Status::OK();
+  return OkStatus();
 }
 
 // If logging is active, log the start and end time of the operation along with
@@ -479,7 +480,7 @@ Status SessionRef::ReleaseCallable(CallableHandle handle) {
     mutex_lock l(run_lock_);
     if (session_ == nullptr) {
       // Session already closed. Do nothing.
-      return Status::OK();
+      return OkStatus();
     }
   }
   LOG_AND_RUN_OPERATION(ReleaseCallable, handle);
