@@ -14,10 +14,6 @@
 # ==============================================================================
 """Tests for `tf.data.experimental.CsvDataset`."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import gzip
 import os
 import zlib
@@ -26,6 +22,7 @@ from absl.testing import parameterized
 
 from tensorflow.python.data.experimental.ops import error_ops
 from tensorflow.python.data.experimental.ops import readers
+from tensorflow.python.data.kernel_tests import checkpoint_test_base
 from tensorflow.python.data.kernel_tests import test_base
 from tensorflow.python.data.ops import readers as core_readers
 from tensorflow.python.eager import context
@@ -74,29 +71,6 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, **kwargs)
     self.assertDatasetsEqual(dataset_actual, dataset_expected)
 
-  def _verify_output_or_err(self,
-                            dataset,
-                            expected_output=None,
-                            expected_err_re=None):
-    if expected_err_re is None:
-      # Verify that output is expected, without errors
-      nxt = self.getNext(dataset)
-      expected_output = [[
-          v.encode('utf-8') if isinstance(v, str) else v for v in op
-      ] for op in expected_output]
-      for value in expected_output:
-        op = self.evaluate(nxt())
-        self.assertAllEqual(op, value)
-      with self.assertRaises(errors.OutOfRangeError):
-        self.evaluate(nxt())
-    else:
-      nxt = self.getNext(dataset)
-      while True:
-        try:
-          self.evaluate(nxt())
-        except errors.OutOfRangeError:
-          break
-
   def _test_dataset(
       self,
       inputs,
@@ -113,37 +87,42 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
       # Verify that OpError is produced as expected
       with self.assertRaisesOpError(expected_err_re):
         dataset = readers.CsvDataset(filenames, **kwargs)
-        self._verify_output_or_err(dataset, expected_output, expected_err_re)
+        self.getDatasetOutput(dataset)
     else:
       dataset = readers.CsvDataset(filenames, **kwargs)
-      self._verify_output_or_err(dataset, expected_output, expected_err_re)
+      expected_output = [
+          tuple(v.encode('utf-8') if isinstance(v, str) else v
+                for v in op)
+          for op in expected_output
+      ]
+      self.assertDatasetProduces(dataset, expected_output)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_requiredFields(self):
+  def testRequiredFields(self):
     record_defaults = [[]] * 4
     inputs = [['1,2,3,4']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_int(self):
+  def testInt(self):
     record_defaults = [[0]] * 4
     inputs = [['1,2,3,4', '5,6,7,8']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_float(self):
+  def testFloat(self):
     record_defaults = [[0.0]] * 4
     inputs = [['1.0,2.1,3.2,4.3', '5.4,6.5,7.6,8.7']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_string(self):
+  def testString(self):
     record_defaults = [['']] * 4
     inputs = [['1.0,2.1,hello,4.3', '5.4,6.5,goodbye,8.7']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withEmptyFields(self):
+  def testWithEmptyFields(self):
     record_defaults = [[0]] * 4
     inputs = [[',,,', '1,1,1,', ',2,2,2']]
     self._test_dataset(
@@ -151,7 +130,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errWithUnquotedQuotes(self):
+  def testErrWithUnquotedQuotes(self):
     record_defaults = [['']] * 3
     inputs = [['1,2"3,4']]
     self._test_dataset(
@@ -160,7 +139,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errWithUnescapedQuotes(self):
+  def testErrWithUnescapedQuotes(self):
     record_defaults = [['']] * 3
     inputs = [['"a"b","c","d"']]
     self._test_dataset(
@@ -170,32 +149,32 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_ignoreErrWithUnescapedQuotes(self):
+  def testIgnoreErrWithUnescapedQuotes(self):
     record_defaults = [['']] * 3
     inputs = [['1,"2"3",4', '1,"2"3",4",5,5', 'a,b,"c"d"', 'e,f,g']]
     filenames = self._setup_files(inputs)
     dataset = readers.CsvDataset(filenames, record_defaults=record_defaults)
     dataset = dataset.apply(error_ops.ignore_errors())
-    self._verify_output_or_err(dataset, [['e', 'f', 'g']])
+    self.assertDatasetProduces(dataset, [(b'e', b'f', b'g')])
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_ignoreErrWithUnquotedQuotes(self):
+  def testIgnoreErrWithUnquotedQuotes(self):
     record_defaults = [['']] * 3
     inputs = [['1,2"3,4', 'a,b,c"d', '9,8"7,6,5', 'e,f,g']]
     filenames = self._setup_files(inputs)
     dataset = readers.CsvDataset(filenames, record_defaults=record_defaults)
     dataset = dataset.apply(error_ops.ignore_errors())
-    self._verify_output_or_err(dataset, [['e', 'f', 'g']])
+    self.assertDatasetProduces(dataset, [(b'e', b'f', b'g')])
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withNoQuoteDelimAndUnquotedQuotes(self):
+  def testWithNoQuoteDelimAndUnquotedQuotes(self):
     record_defaults = [['']] * 3
     inputs = [['1,2"3,4']]
     self._test_by_comparison(
         inputs, record_defaults=record_defaults, use_quote_delim=False)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_mixedTypes(self):
+  def testMixedTypes(self):
     record_defaults = [
         constant_op.constant([], dtype=dtypes.int32),
         constant_op.constant([], dtype=dtypes.float32),
@@ -206,35 +185,35 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withUseQuoteDelimFalse(self):
+  def testWithUseQuoteDelimFalse(self):
     record_defaults = [['']] * 4
     inputs = [['1,2,"3,4"', '"5,6",7,8']]
     self._test_by_comparison(
         inputs, record_defaults=record_defaults, use_quote_delim=False)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withFieldDelim(self):
+  def testWithFieldDelim(self):
     record_defaults = [[0]] * 4
     inputs = [['1:2:3:4', '5:6:7:8']]
     self._test_by_comparison(
         inputs, record_defaults=record_defaults, field_delim=':')
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withNaValue(self):
+  def testWithNaValue(self):
     record_defaults = [[0]] * 4
     inputs = [['1,NA,3,4', 'NA,6,7,8']]
     self._test_by_comparison(
         inputs, record_defaults=record_defaults, na_value='NA')
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withSelectCols(self):
+  def testWithSelectCols(self):
     record_defaults = [['']] * 2
     inputs = [['1,2,3,4', '"5","6","7","8"']]
     self._test_by_comparison(
         inputs, record_defaults=record_defaults, select_cols=[1, 2])
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withSelectColsTooHigh(self):
+  def testWithSelectColsTooHigh(self):
     record_defaults = [[0]] * 2
     inputs = [['1,2,3,4', '5,6,7,8']]
     self._test_dataset(
@@ -244,27 +223,27 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         select_cols=[3, 4])
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withOneCol(self):
+  def testWithOneCol(self):
     record_defaults = [['NA']]
     inputs = [['0', '', '2']]
     self._test_dataset(
         inputs, [['0'], ['NA'], ['2']], record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withMultipleFiles(self):
+  def testWithMultipleFiles(self):
     record_defaults = [[0]] * 4
     inputs = [['1,2,3,4', '5,6,7,8'], ['5,6,7,8']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withLeadingAndTrailingSpaces(self):
+  def testWithLeadingAndTrailingSpaces(self):
     record_defaults = [[0.0]] * 4
     inputs = [['0, 1, 2, 3']]
     expected = [[0.0, 1.0, 2.0, 3.0]]
     self._test_dataset(inputs, expected, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errorWithMissingDefault(self):
+  def testErrorWithMissingDefault(self):
     record_defaults = [[]] * 2
     inputs = [['0,']]
     self._test_dataset(
@@ -273,7 +252,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errorWithFewerDefaultsThanFields(self):
+  def testErrorWithFewerDefaultsThanFields(self):
     record_defaults = [[0.0]] * 2
     inputs = [['0,1,2,3']]
     self._test_dataset(
@@ -282,7 +261,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errorWithMoreDefaultsThanFields(self):
+  def testErrorWithMoreDefaultsThanFields(self):
     record_defaults = [[0.0]] * 5
     inputs = [['0,1,2,3']]
     self._test_dataset(
@@ -291,7 +270,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withHeader(self):
+  def testWithHeader(self):
     record_defaults = [[0]] * 2
     inputs = [['col1,col2', '1,2']]
     expected = [[1, 2]]
@@ -303,7 +282,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
     )
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withHeaderAndNoRecords(self):
+  def testWithHeaderAndNoRecords(self):
     record_defaults = [[0]] * 2
     inputs = [['col1,col2']]
     expected = []
@@ -315,7 +294,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
     )
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errorWithHeaderEmptyFile(self):
+  def testErrorWithHeaderEmptyFile(self):
     record_defaults = [[0]] * 2
     inputs = [[]]
     expected_err_re = "Can't read header of file"
@@ -327,14 +306,14 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
     )
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withEmptyFile(self):
+  def testWithEmptyFile(self):
     record_defaults = [['']] * 2
     inputs = [['']]  # Empty file
     self._test_dataset(
         inputs, expected_output=[], record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errorWithEmptyRecord(self):
+  def testErrorWithEmptyRecord(self):
     record_defaults = [['']] * 2
     inputs = [['', '1,2']]  # First record is empty
     self._test_dataset(
@@ -343,7 +322,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withChainedOps(self):
+  def testWithChainedOps(self):
     # Testing that one dataset can create multiple iterators fine.
     # `repeat` creates multiple iterators from the same C++ Dataset.
     record_defaults = [[0]] * 4
@@ -355,7 +334,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         ds_expected.repeat(5).prefetch(1))
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withTypeDefaults(self):
+  def testWithTypeDefaults(self):
     # Testing using dtypes as record_defaults for required fields
     record_defaults = [dtypes.float32, [0.0]]
     inputs = [['1.0,2.0', '3.0,4.0']]
@@ -365,38 +344,23 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults,
     )
 
-  @combinations.generate(test_base.default_test_combinations())
-  def testMakeCsvDataset_fieldOrder(self):
-    data = [[
-        '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19',
-        '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19'
-    ]]
-    file_path = self._setup_files(data)
-
-    ds = readers.make_csv_dataset(
-        file_path, batch_size=1, shuffle=False, num_epochs=1)
-    nxt = self.getNext(ds)
-
-    result = list(self.evaluate(nxt()).values())
-
-    self.assertEqual(result, sorted(result))
-
 ## The following tests exercise parsing logic for quoted fields
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withQuoted(self):
+  def testWithQuoted(self):
     record_defaults = [['']] * 4
     inputs = [['"a","b","c :)","d"', '"e","f","g :(","h"']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
 
-  def testCsvDataset_withOneColAndQuotes(self):
+  @combinations.generate(test_base.default_test_combinations())
+  def testWithOneColAndQuotes(self):
     record_defaults = [['']]
     inputs = [['"0"', '"1"', '"2"']]
     self._test_dataset(
         inputs, [['0'], ['1'], ['2']], record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withNewLine(self):
+  def testWithNewLine(self):
     # In this case, we expect it to behave differently from
     # TextLineDataset->map(decode_csv) since that flow has bugs
     record_defaults = [['']] * 4
@@ -405,7 +369,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
     self._test_dataset(inputs, expected, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withNewLineInUnselectedCol(self):
+  def testWithNewLineInUnselectedCol(self):
     record_defaults = [['']]
     inputs = [['1,"2\n3",4', '5,6,7']]
     self._test_dataset(
@@ -414,8 +378,49 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults,
         select_cols=[0])
 
+  @combinations.generate(test_base.v2_only_combinations())
+  def testWithExcludeCol(self):
+    record_defaults = [['']]
+    inputs = [['1,2,3', '5,6,7']]
+    self._test_dataset(
+        inputs,
+        expected_output=[['1'], ['5']],
+        record_defaults=record_defaults,
+        exclude_cols=[1, 2])
+
+  @combinations.generate(test_base.v2_only_combinations())
+  def testWithSelectandExcludeCol(self):
+    record_defaults = [['']]
+    inputs = [['1,2,3', '5,6,7']]
+    self._test_dataset(
+        inputs,
+        expected_err_re='Either select_cols or exclude_cols should be empty',
+        record_defaults=record_defaults,
+        select_cols=[0],
+        exclude_cols=[1, 2])
+
+  @combinations.generate(test_base.v2_only_combinations())
+  def testWithExcludeColandRecordDefaultsTooLow(self):
+    record_defaults = [['']]
+    inputs = [['1,2,3', '5,6,7']]
+    self._test_dataset(
+        inputs,
+        expected_err_re='Expect 1 fields but have more in record',
+        record_defaults=record_defaults,
+        exclude_cols=[0])
+
+  @combinations.generate(test_base.v2_only_combinations())
+  def testWithExcludeColandRecordDefaultsTooHigh(self):
+    record_defaults = [['']] * 3
+    inputs = [['1,2,3', '5,6,7']]
+    self._test_dataset(
+        inputs,
+        expected_err_re='Expect 3 fields but have 2 in record',
+        record_defaults=record_defaults,
+        exclude_cols=[0])
+
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withMultipleNewLines(self):
+  def testWithMultipleNewLines(self):
     # In this case, we expect it to behave differently from
     # TextLineDataset->map(decode_csv) since that flow has bugs
     record_defaults = [['']] * 4
@@ -424,7 +429,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
     self._test_dataset(inputs, expected, record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_errorWithTerminateMidRecord(self):
+  def testErrorWithTerminateMidRecord(self):
     record_defaults = [['']] * 4
     inputs = [['a,b,c,"a']]
     self._test_dataset(
@@ -434,7 +439,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withEscapedQuotes(self):
+  def testWithEscapedQuotes(self):
     record_defaults = [['']] * 4
     inputs = [['1.0,2.1,"she said: ""hello""",4.3', '5.4,6.5,goodbye,8.7']]
     self._test_by_comparison(inputs, record_defaults=record_defaults)
@@ -444,7 +449,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
 ## and different types of line breaks
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withInvalidBufferSize(self):
+  def testWithInvalidBufferSize(self):
     record_defaults = [['']] * 4
     inputs = [['a,b,c,d']]
     self._test_dataset(
@@ -471,7 +476,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
           buffer_size=i)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withLF(self):
+  def testWithLF(self):
     record_defaults = [['NA']] * 3
     inputs = [['abc,def,ghi', '0,1,2', ',,']]
     expected = [['abc', 'def', 'ghi'], ['0', '1', '2'], ['NA', 'NA', 'NA']]
@@ -479,7 +484,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, expected, linebreak='\n', record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withCR(self):
+  def testWithCR(self):
     # Test that when the line separator is '\r', parsing works with all buffer
     # sizes
     record_defaults = [['NA']] * 3
@@ -489,7 +494,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, expected, linebreak='\r', record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withCRLF(self):
+  def testWithCRLF(self):
     # Test that when the line separator is '\r\n', parsing works with all buffer
     # sizes
     record_defaults = [['NA']] * 3
@@ -499,7 +504,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, expected, linebreak='\r\n', record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withBufferSizeAndQuoted(self):
+  def testWithBufferSizeAndQuoted(self):
     record_defaults = [['NA']] * 3
     inputs = [['"\n\n\n","\r\r\r","abc"', '"0","1","2"', '"","",""']]
     expected = [['\n\n\n', '\r\r\r', 'abc'], ['0', '1', '2'],
@@ -508,7 +513,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, expected, linebreak='\n', record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withCRAndQuoted(self):
+  def testWithCRAndQuoted(self):
     # Test that when the line separator is '\r', parsing works with all buffer
     # sizes
     record_defaults = [['NA']] * 3
@@ -519,7 +524,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, expected, linebreak='\r', record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withCRLFAndQuoted(self):
+  def testWithCRLFAndQuoted(self):
     # Test that when the line separator is '\r\n', parsing works with all buffer
     # sizes
     record_defaults = [['NA']] * 3
@@ -530,7 +535,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         inputs, expected, linebreak='\r\n', record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withGzipCompressionType(self):
+  def testWithGzipCompressionType(self):
     record_defaults = [['NA']] * 3
     inputs = [['"\n\n\n","\r\r\r","abc"', '"0","1","2"', '"","",""']]
     expected = [['\n\n\n', '\r\r\r', 'abc'], ['0', '1', '2'],
@@ -543,7 +548,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withZlibCompressionType(self):
+  def testWithZlibCompressionType(self):
     record_defaults = [['NA']] * 3
     inputs = [['"\n\n\n","\r\r\r","abc"', '"0","1","2"', '"","",""']]
     expected = [['\n\n\n', '\r\r\r', 'abc'], ['0', '1', '2'],
@@ -556,7 +561,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_withScalarDefaults(self):
+  def testWithScalarDefaults(self):
     record_defaults = [constant_op.constant(0, dtype=dtypes.int64)] * 4
     inputs = [[',,,', '1,1,1,', ',2,2,2']]
     self._test_dataset(
@@ -564,7 +569,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         record_defaults=record_defaults)
 
   @combinations.generate(test_base.default_test_combinations())
-  def testCsvDataset_with2DDefaults(self):
+  def testWith2DDefaults(self):
     record_defaults = [constant_op.constant([[0]], dtype=dtypes.int64)] * 4
     inputs = [[',,,', '1,1,1,', ',2,2,2']]
 
@@ -580,7 +585,7 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
           inputs, [[0, 0, 0, 0], [1, 1, 1, 0], [0, 2, 2, 2]],
           record_defaults=record_defaults)
 
-  def testCsvDataset_immutableParams(self):
+  def testImmutableParams(self):
     inputs = [['a,b,c', '1,2,3', '4,5,6']]
     filenames = self._setup_files(inputs)
     select_cols = ['a', 'c']
@@ -588,5 +593,52 @@ class CsvDatasetTest(test_base.DatasetTestBase, parameterized.TestCase):
         filenames, batch_size=1, select_columns=select_cols)
     self.assertAllEqual(select_cols, ['a', 'c'])
 
+
+class CsvDatasetCheckpointTest(checkpoint_test_base.CheckpointTestBase,
+                               parameterized.TestCase):
+
+  def setUp(self):
+    super(CsvDatasetCheckpointTest, self).setUp()
+    self._num_cols = 7
+    self._num_rows = 10
+    self._num_epochs = 14
+    self._num_outputs = self._num_rows * self._num_epochs
+
+    inputs = [
+        ','.join(str(self._num_cols * j + i)
+                 for i in range(self._num_cols))
+        for j in range(self._num_rows)
+    ]
+    contents = '\n'.join(inputs).encode('utf-8')
+
+    self._filename = os.path.join(self.get_temp_dir(), 'file.csv')
+    self._compressed = os.path.join(self.get_temp_dir(),
+                                    'comp.csv')  # GZip compressed
+
+    with open(self._filename, 'wb') as f:
+      f.write(contents)
+    with gzip.GzipFile(self._compressed, 'wb') as f:
+      f.write(contents)
+
+  def ds_func(self, **kwargs):
+    compression_type = kwargs.get('compression_type', None)
+    if compression_type == 'GZIP':
+      filename = self._compressed
+    elif compression_type is None:
+      filename = self._filename
+    else:
+      raise ValueError('Invalid compression type:', compression_type)
+
+    return readers.CsvDataset(filename, **kwargs).repeat(self._num_epochs)
+
+  @combinations.generate(
+      combinations.times(test_base.default_test_combinations(),
+                         checkpoint_test_base.default_test_combinations()))
+  def testCore(self, verify_fn):
+    defs = [[0]] * self._num_cols
+    verify_fn(self, lambda: self.ds_func(record_defaults=defs, buffer_size=2),
+              self._num_outputs)
+
+
 if __name__ == '__main__':
   test.main()
