@@ -13,10 +13,6 @@
 # limitations under the License.
 # ==============================================================================
 """Python wrappers for reader Datasets."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import collections
 import csv
 import functools
@@ -28,6 +24,8 @@ from tensorflow.python import tf2
 from tensorflow.python.data.experimental.ops import error_ops
 from tensorflow.python.data.experimental.ops import parsing_ops
 from tensorflow.python.data.ops import dataset_ops
+from tensorflow.python.data.ops import map_op
+from tensorflow.python.data.ops import options as options_lib
 from tensorflow.python.data.ops import readers as core_readers
 from tensorflow.python.data.util import convert
 from tensorflow.python.data.util import nest
@@ -116,14 +114,17 @@ def _next_csv_row(filenames, num_cols, field_delim, use_quote_delim, header,
           f,
           delimiter=field_delim,
           quoting=csv.QUOTE_MINIMAL if use_quote_delim else csv.QUOTE_NONE)
+      row_num = 1
       if header:
         next(rdr)  # Skip header lines
+        row_num += 1
 
       for csv_row in rdr:
         if len(csv_row) != num_cols:
           raise ValueError(
-              "Problem inferring types: CSV row has different number of fields "
-              "than expected.")
+              f"Problem inferring types: CSV row {row_num} has {len(csv_row)} "
+              f"number of fields. Expected: {num_cols}.")
+        row_num += 1
         yield csv_row
 
 
@@ -164,18 +165,19 @@ def _infer_column_names(filenames, field_delim, use_quote_delim, file_io_fn):
     try:
       column_names = next(csv.reader(f, **csv_kwargs))
     except StopIteration:
-      raise ValueError(("Received StopIteration when reading the header line "
-                        "of %s.  Empty file?") % filenames[0])
+      raise ValueError("Failed when reading the header line of "
+                       f"{filenames[0]}. Is it an empty file?")
 
   for name in filenames[1:]:
     with file_io_fn(name) as f:
       try:
         if next(csv.reader(f, **csv_kwargs)) != column_names:
           raise ValueError(
-              "Files have different column names in the header row.")
+              "All input CSV files should have the same column names in the "
+              f"header row. File {name} has different column names.")
       except StopIteration:
-        raise ValueError(("Received StopIteration when reading the header line "
-                          "of %s.  Empty file?") % filenames[0])
+        raise ValueError("Failed when reading the header line of "
+                         f"{name}. Is it an empty file?")
   return column_names
 
 
@@ -190,22 +192,26 @@ def _get_sorted_col_indices(select_columns, column_names):
     if isinstance(v, int):
       if v < 0 or v >= num_cols:
         raise ValueError(
-            "Column index %d specified in select_columns out of valid range." %
-            v)
+            f"Column index {v} specified in `select_columns` should be > 0 "
+            f" and <= {num_cols}, which is the number of columns.")
       results.append(v)
     # Otherwise, check that it's a valid column name and convert to the
     # the relevant column index.
     elif v not in names_to_indices:
       raise ValueError(
-          "Value '%s' specified in select_columns not a valid column index or "
-          "name." % v)
+          f"Column {v} specified in `select_columns` must be of one of the "
+          f"columns: {names_to_indices.keys()}.")
     else:
       results.append(names_to_indices[v])
 
   # Sort and ensure there are no duplicates
   results = sorted(set(results))
   if len(results) != len(select_columns):
-    raise ValueError("select_columns contains duplicate columns")
+    sorted_names = sorted(results)
+    duplicate_columns = set([a for a, b in zip(
+        sorted_names[:-1], sorted_names[1:]) if a == b])
+    raise ValueError("The `select_columns` argument contains duplicate "
+                     f"columns: {duplicate_columns}.")
   return results
 
 
@@ -329,7 +335,7 @@ def make_csv_dataset_v2(
     use_quote_delim=True,
     na_value="",
     header=True,
-    num_epochs=None,
+    num_epochs=None,  # TODO(aaudibert): Change default to 1 when graduating.
     shuffle=True,
     shuffle_buffer_size=10000,
     shuffle_seed=None,
@@ -339,14 +345,57 @@ def make_csv_dataset_v2(
     num_rows_for_inference=100,
     compression_type=None,
     ignore_errors=False,
+    encoding="utf-8",
 ):
   """Reads CSV files into a dataset.
 
-  Reads CSV files into a dataset, where each element is a (features, labels)
-  tuple that corresponds to a batch of CSV rows. The features dictionary
-  maps feature column names to `Tensor`s containing the corresponding
+  Reads CSV files into a dataset, where each element of the dataset is a
+  (features, labels) tuple that corresponds to a batch of CSV rows. The features
+  dictionary maps feature column names to `Tensor`s containing the corresponding
   feature data, and labels is a `Tensor` containing the batch's label data.
 
+  By default, the first rows of the CSV files are expected to be headers listing
+  the column names. If the first rows are not headers, set `header=False` and
+  provide the column names with the `column_names` argument.
+
+  By default, the dataset is repeated indefinitely, reshuffling the order each
+  time. This behavior can be modified by setting the `num_epochs` and `shuffle`
+  arguments.
+
+  For example, suppose you have a CSV file containing
+
+  | Feature_A | Feature_B |
+  | --------- | --------- |
+  | 1         | "a"       |
+  | 2         | "b"       |
+  | 3         | "c"       |
+  | 4         | "d"       |
+
+  ```
+  # No label column specified
+  dataset = tf.data.experimental.make_csv_dataset(filename, batch_size=2)
+  iterator = dataset.as_numpy_iterator()
+  print(dict(next(iterator)))
+  # prints a dictionary of batched features:
+  # OrderedDict([('Feature_A', array([1, 4], dtype=int32)),
+  #              ('Feature_B', array([b'a', b'd'], dtype=object))])
+  ```
+
+  ```
+  # Set Feature_B as label column
+  dataset = tf.data.experimental.make_csv_dataset(
+      filename, batch_size=2, label_name="Feature_B")
+  iterator = dataset.as_numpy_iterator()
+  print(next(iterator))
+  # prints (features, labels) tuple:
+  # (OrderedDict([('Feature_A', array([1, 2], dtype=int32))]),
+  #  array([b'a', b'b'], dtype=object))
+  ```
+
+  See the
+  [Load CSV data guide](https://www.tensorflow.org/tutorials/load_data/csv) for
+  more examples of using `make_csv_dataset` to read CSV data.
+
   Args:
     file_pattern: List of files or patterns of file paths containing CSV
       records. See `tf.io.gfile.glob` for pattern rules.
@@ -416,6 +465,7 @@ def make_csv_dataset_v2(
       such as malformed data or empty lines, and moves on to the next valid
       CSV record. Otherwise, the dataset raises an error and stops processing
       when encountering any invalid records. Defaults to `False`.
+    encoding: Encoding to use when reading. Defaults to `UTF-8`.
 
   Returns:
     A dataset, where each element is a (features, labels) tuple that corresponds
@@ -442,28 +492,40 @@ def make_csv_dataset_v2(
   # Clean arguments; figure out column names and defaults
   if column_names is None or column_defaults is None:
     # Find out which io function to open the file
-    file_io_fn = lambda filename: file_io.FileIO(filename, "r")
+    file_io_fn = lambda filename: file_io.FileIO(  # pylint: disable=g-long-lambda
+        filename, "r", encoding=encoding)
     if compression_type is not None:
       compression_type_value = tensor_util.constant_value(compression_type)
       if compression_type_value is None:
-        raise ValueError("Received unknown compression_type")
+        raise ValueError(
+            f"Received unknown `compression_type` {compression_type}. "
+            "Expected: GZIP, ZLIB or "" (empty string).")
       if compression_type_value == "GZIP":
-        file_io_fn = lambda filename: gzip.open(filename, "rt")
+        file_io_fn = lambda filename: gzip.open(  # pylint: disable=g-long-lambda
+            filename, "rt", encoding=encoding)
       elif compression_type_value == "ZLIB":
         raise ValueError(
-            "compression_type (%s) is not supported for probing columns" %
-            compression_type)
+            f"`compression_type` {compression_type} is not supported for "
+            "probing columns.")
       elif compression_type_value != "":
-        raise ValueError("compression_type (%s) is not supported" %
-                         compression_type)
+        raise ValueError(
+            f"Received unknown `compression_type` {compression_type}. "
+            "Expected: GZIP, ZLIB or "
+            " (empty string).")
   if column_names is None:
     if not header:
-      raise ValueError("Cannot infer column names without a header line.")
+      raise ValueError("Expected `column_names` or `header` arguments. Neither "
+                       "is provided.")
     # If column names are not provided, infer from the header lines
     column_names = _infer_column_names(filenames, field_delim, use_quote_delim,
                                        file_io_fn)
   if len(column_names) != len(set(column_names)):
-    raise ValueError("Cannot have duplicate column names.")
+    sorted_names = sorted(column_names)
+    duplicate_columns = set([a for a, b in zip(
+        sorted_names[:-1], sorted_names[1:]) if a == b])
+    raise ValueError(
+        "Either `column_names` argument or CSV header row contains duplicate "
+        f"column names: {duplicate_columns}.")
 
   if select_columns is not None:
     select_columns = _get_sorted_col_indices(select_columns, column_names)
@@ -471,7 +533,7 @@ def make_csv_dataset_v2(
   if column_defaults is not None:
     column_defaults = [
         constant_op.constant([], dtype=x)
-        if not tensor_util.is_tensor(x) and x in _ACCEPTABLE_CSV_TYPES else x
+        if not tensor_util.is_tf_type(x) and x in _ACCEPTABLE_CSV_TYPES else x
         for x in column_defaults
     ]
   else:
@@ -485,15 +547,16 @@ def make_csv_dataset_v2(
 
   if select_columns is not None and len(column_defaults) != len(select_columns):
     raise ValueError(
-        "If specified, column_defaults and select_columns must have same "
-        "length."
-    )
+        "If specified, `column_defaults` and `select_columns` must have the "
+        f"same length: `column_defaults` has length {len(column_defaults)}, "
+        f"`select_columns` has length {len(select_columns)}.")
   if select_columns is not None and len(column_names) > len(select_columns):
     # Pick the relevant subset of column names
     column_names = [column_names[i] for i in select_columns]
 
   if label_name is not None and label_name not in column_names:
-    raise ValueError("`label_name` provided must be one of the columns.")
+    raise ValueError("`label_name` provided must be one of the columns: "
+                     f"{column_names}. Received: {label_name}.")
 
   def filename_to_dataset(filename):
     dataset = CsvDataset(
@@ -529,8 +592,8 @@ def make_csv_dataset_v2(
   if num_parallel_reads == dataset_ops.AUTOTUNE:
     dataset = dataset.interleave(
         filename_to_dataset, num_parallel_calls=num_parallel_reads)
-    options = dataset_ops.Options()
-    options.experimental_deterministic = not sloppy
+    options = options_lib.Options()
+    options.deterministic = not sloppy
     dataset = dataset.with_options(options)
   else:
     # Read files sequentially (if num_parallel_reads=1) or in parallel
@@ -557,7 +620,7 @@ def make_csv_dataset_v2(
   # indefinitely, and all batches will be full-sized.
   dataset = dataset.batch(batch_size=batch_size,
                           drop_remainder=num_epochs is None)
-  dataset = dataset_ops.MapDataset(
+  dataset = map_op._MapDataset(  # pylint: disable=protected-access
       dataset, map_fn, use_inter_op_parallelism=False)
   dataset = dataset.prefetch(prefetch_buffer_size)
 
@@ -586,13 +649,16 @@ def make_csv_dataset_v1(
     num_rows_for_inference=100,
     compression_type=None,
     ignore_errors=False,
+    encoding="utf-8",
 ):  # pylint: disable=missing-docstring
-  return dataset_ops.DatasetV1Adapter(make_csv_dataset_v2(
-      file_pattern, batch_size, column_names, column_defaults, label_name,
-      select_columns, field_delim, use_quote_delim, na_value, header,
-      num_epochs, shuffle, shuffle_buffer_size, shuffle_seed,
-      prefetch_buffer_size, num_parallel_reads, sloppy, num_rows_for_inference,
-      compression_type, ignore_errors))
+  return dataset_ops.DatasetV1Adapter(
+      make_csv_dataset_v2(file_pattern, batch_size, column_names,
+                          column_defaults, label_name, select_columns,
+                          field_delim, use_quote_delim, na_value, header,
+                          num_epochs, shuffle, shuffle_buffer_size,
+                          shuffle_seed, prefetch_buffer_size,
+                          num_parallel_reads, sloppy, num_rows_for_inference,
+                          compression_type, ignore_errors, encoding))
 make_csv_dataset_v1.__doc__ = make_csv_dataset_v2.__doc__
 
 
@@ -601,8 +667,166 @@ _DEFAULT_READER_BUFFER_SIZE_BYTES = 4 * 1024 * 1024  # 4 MB
 
 @tf_export("data.experimental.CsvDataset", v1=[])
 class CsvDatasetV2(dataset_ops.DatasetSource):
+  r"""A Dataset comprising lines from one or more CSV files.
+
+  The `tf.data.experimental.CsvDataset` class provides a minimal CSV Dataset
+  interface. There is also a richer `tf.data.experimental.make_csv_dataset`
+  function which provides additional convenience features such as column header
+  parsing, column type-inference, automatic shuffling, and file interleaving.
+
+  The elements of this dataset correspond to records from the file(s).
+  RFC 4180 format is expected for CSV files
+  (https://tools.ietf.org/html/rfc4180)
+  Note that we allow leading and trailing spaces for int or float fields.
+
+  For example, suppose we have a file 'my_file0.csv' with four CSV columns of
+  different data types:
+
+  >>> with open('/tmp/my_file0.csv', 'w') as f:
+  ...   f.write('abcdefg,4.28E10,5.55E6,12\n')
+  ...   f.write('hijklmn,-5.3E14,,2\n')
+
+  We can construct a CsvDataset from it as follows:
+
+  >>> dataset = tf.data.experimental.CsvDataset(
+  ...   "/tmp/my_file0.csv",
+  ...   [tf.float32,  # Required field, use dtype or empty tensor
+  ...    tf.constant([0.0], dtype=tf.float32),  # Optional field, default to 0.0
+  ...    tf.int32,  # Required field, use dtype or empty tensor
+  ...   ],
+  ...   select_cols=[1,2,3]  # Only parse last three columns
+  ... )
+
+  The expected output of its iterations is:
+
+  >>> for element in dataset.as_numpy_iterator():
+  ...   print(element)
+  (4.28e10, 5.55e6, 12)
+  (-5.3e14, 0.0, 2)
+
+  See
+  https://www.tensorflow.org/tutorials/load_data/csv#tfdataexperimentalcsvdataset
+  for more in-depth example usage.
+  """
+
+  def __init__(self,
+               filenames,
+               record_defaults,
+               compression_type=None,
+               buffer_size=None,
+               header=False,
+               field_delim=",",
+               use_quote_delim=True,
+               na_value="",
+               select_cols=None,
+               exclude_cols=None):
+    """Creates a `CsvDataset` by reading and decoding CSV files.
+
+    Args:
+      filenames: A `tf.string` tensor containing one or more filenames.
+      record_defaults: A list of default values for the CSV fields. Each item in
+        the list is either a valid CSV `DType` (float32, float64, int32, int64,
+        string), or a `Tensor` object with one of the above types. One per
+        column of CSV data, with either a scalar `Tensor` default value for the
+        column if it is optional, or `DType` or empty `Tensor` if required. If
+        both this and `select_columns` are specified, these must have the same
+        lengths, and `column_defaults` is assumed to be sorted in order of
+        increasing column index. If both this and 'exclude_cols' are specified,
+        the sum of lengths of record_defaults and exclude_cols should equal
+        the total number of columns in the CSV file.
+      compression_type: (Optional.) A `tf.string` scalar evaluating to one of
+        `""` (no compression), `"ZLIB"`, or `"GZIP"`. Defaults to no
+        compression.
+      buffer_size: (Optional.) A `tf.int64` scalar denoting the number of bytes
+        to buffer while reading files. Defaults to 4MB.
+      header: (Optional.) A `tf.bool` scalar indicating whether the CSV file(s)
+        have header line(s) that should be skipped when parsing. Defaults to
+        `False`.
+      field_delim: (Optional.) A `tf.string` scalar containing the delimiter
+        character that separates fields in a record. Defaults to `","`.
+      use_quote_delim: (Optional.) A `tf.bool` scalar. If `False`, treats
+        double quotation marks as regular characters inside of string fields
+        (ignoring RFC 4180, Section 2, Bullet 5). Defaults to `True`.
+      na_value: (Optional.) A `tf.string` scalar indicating a value that will
+        be treated as NA/NaN.
+      select_cols: (Optional.) A sorted list of column indices to select from
+        the input data. If specified, only this subset of columns will be
+        parsed. Defaults to parsing all columns. At most one of `select_cols`
+        and `exclude_cols` can be specified.
+      exclude_cols: (Optional.) A sorted list of column indices to exclude from
+        the input data. If specified, only the complement of this set of column
+        will be parsed. Defaults to parsing all columns. At most one of
+        `select_cols` and `exclude_cols` can be specified.
+
+    Raises:
+       InvalidArgumentError: If exclude_cols is not None and
+           len(exclude_cols) + len(record_defaults) does not match the total
+           number of columns in the file(s)
+
+
+    """
+    self._filenames = ops.convert_to_tensor(
+        filenames, dtype=dtypes.string, name="filenames")
+    self._compression_type = convert.optional_param_to_tensor(
+        "compression_type",
+        compression_type,
+        argument_default="",
+        argument_dtype=dtypes.string)
+    record_defaults = [
+        constant_op.constant([], dtype=x)
+        if not tensor_util.is_tf_type(x) and x in _ACCEPTABLE_CSV_TYPES else x
+        for x in record_defaults
+    ]
+    self._record_defaults = ops.convert_n_to_tensor(
+        record_defaults, name="record_defaults")
+    self._buffer_size = convert.optional_param_to_tensor(
+        "buffer_size", buffer_size, _DEFAULT_READER_BUFFER_SIZE_BYTES)
+    self._header = ops.convert_to_tensor(
+        header, dtype=dtypes.bool, name="header")
+    self._field_delim = ops.convert_to_tensor(
+        field_delim, dtype=dtypes.string, name="field_delim")
+    self._use_quote_delim = ops.convert_to_tensor(
+        use_quote_delim, dtype=dtypes.bool, name="use_quote_delim")
+    self._na_value = ops.convert_to_tensor(
+        na_value, dtype=dtypes.string, name="na_value")
+    self._select_cols = convert.optional_param_to_tensor(
+        "select_cols",
+        select_cols,
+        argument_default=[],
+        argument_dtype=dtypes.int64,
+    )
+    self._exclude_cols = convert.optional_param_to_tensor(
+        "exclude_cols",
+        exclude_cols,
+        argument_default=[],
+        argument_dtype=dtypes.int64,
+    )
+    self._element_spec = tuple(
+        tensor_spec.TensorSpec([], d.dtype) for d in self._record_defaults)
+    variant_tensor = gen_experimental_dataset_ops.csv_dataset_v2(
+        filenames=self._filenames,
+        record_defaults=self._record_defaults,
+        buffer_size=self._buffer_size,
+        header=self._header,
+        output_shapes=self._flat_shapes,
+        field_delim=self._field_delim,
+        use_quote_delim=self._use_quote_delim,
+        na_value=self._na_value,
+        select_cols=self._select_cols,
+        exclude_cols=self._exclude_cols,
+        compression_type=self._compression_type)
+    super(CsvDatasetV2, self).__init__(variant_tensor)
+
+  @property
+  def element_spec(self):
+    return self._element_spec
+
+
+@tf_export(v1=["data.experimental.CsvDataset"])
+class CsvDatasetV1(dataset_ops.DatasetV1Adapter):
   """A Dataset comprising lines from one or more CSV files."""
 
+  @functools.wraps(CsvDatasetV2.__init__, ("__module__", "__name__"))
   def __init__(self,
                filenames,
                record_defaults,
@@ -660,7 +884,9 @@ class CsvDatasetV2(dataset_ops.DatasetSource):
         column if it is optional, or `DType` or empty `Tensor` if required. If
         both this and `select_columns` are specified, these must have the same
         lengths, and `column_defaults` is assumed to be sorted in order of
-        increasing column index.
+        increasing column index. If both this and 'exclude_cols' are specified,
+        the sum of lengths of record_defaults and exclude_cols should equal the
+        total number of columns in the CSV file.
       compression_type: (Optional.) A `tf.string` scalar evaluating to one of
         `""` (no compression), `"ZLIB"`, or `"GZIP"`. Defaults to no
         compression.
@@ -671,80 +897,16 @@ class CsvDatasetV2(dataset_ops.DatasetSource):
         `False`.
       field_delim: (Optional.) A `tf.string` scalar containing the delimiter
         character that separates fields in a record. Defaults to `","`.
-      use_quote_delim: (Optional.) A `tf.bool` scalar. If `False`, treats
-        double quotation marks as regular characters inside of string fields
-        (ignoring RFC 4180, Section 2, Bullet 5). Defaults to `True`.
-      na_value: (Optional.) A `tf.string` scalar indicating a value that will
-        be treated as NA/NaN.
+      use_quote_delim: (Optional.) A `tf.bool` scalar. If `False`, treats double
+        quotation marks as regular characters inside of string fields (ignoring
+        RFC 4180, Section 2, Bullet 5). Defaults to `True`.
+      na_value: (Optional.) A `tf.string` scalar indicating a value that will be
+        treated as NA/NaN.
       select_cols: (Optional.) A sorted list of column indices to select from
         the input data. If specified, only this subset of columns will be
-        parsed. Defaults to parsing all columns.
+        parsed. Defaults to parsing all columns. At most one of `select_cols`
+        and `exclude_cols` can be specified.
     """
-    self._filenames = ops.convert_to_tensor(
-        filenames, dtype=dtypes.string, name="filenames")
-    self._compression_type = convert.optional_param_to_tensor(
-        "compression_type",
-        compression_type,
-        argument_default="",
-        argument_dtype=dtypes.string)
-    record_defaults = [
-        constant_op.constant([], dtype=x)
-        if not tensor_util.is_tensor(x) and x in _ACCEPTABLE_CSV_TYPES else x
-        for x in record_defaults
-    ]
-    self._record_defaults = ops.convert_n_to_tensor(
-        record_defaults, name="record_defaults")
-    self._buffer_size = convert.optional_param_to_tensor(
-        "buffer_size", buffer_size, _DEFAULT_READER_BUFFER_SIZE_BYTES)
-    self._header = ops.convert_to_tensor(
-        header, dtype=dtypes.bool, name="header")
-    self._field_delim = ops.convert_to_tensor(
-        field_delim, dtype=dtypes.string, name="field_delim")
-    self._use_quote_delim = ops.convert_to_tensor(
-        use_quote_delim, dtype=dtypes.bool, name="use_quote_delim")
-    self._na_value = ops.convert_to_tensor(
-        na_value, dtype=dtypes.string, name="na_value")
-    self._select_cols = convert.optional_param_to_tensor(
-        "select_cols",
-        select_cols,
-        argument_default=[],
-        argument_dtype=dtypes.int64,
-    )
-    self._element_spec = tuple(
-        tensor_spec.TensorSpec([], d.dtype) for d in self._record_defaults)
-    variant_tensor = gen_experimental_dataset_ops.csv_dataset(
-        filenames=self._filenames,
-        record_defaults=self._record_defaults,
-        buffer_size=self._buffer_size,
-        header=self._header,
-        output_shapes=self._flat_shapes,
-        field_delim=self._field_delim,
-        use_quote_delim=self._use_quote_delim,
-        na_value=self._na_value,
-        select_cols=self._select_cols,
-        compression_type=self._compression_type)
-    super(CsvDatasetV2, self).__init__(variant_tensor)
-
-  @property
-  def element_spec(self):
-    return self._element_spec
-
-
-@tf_export(v1=["data.experimental.CsvDataset"])
-class CsvDatasetV1(dataset_ops.DatasetV1Adapter):
-  """A Dataset comprising lines from one or more CSV files."""
-
-  @functools.wraps(CsvDatasetV2.__init__)
-  def __init__(self,
-               filenames,
-               record_defaults,
-               compression_type=None,
-               buffer_size=None,
-               header=False,
-               field_delim=",",
-               use_quote_delim=True,
-               na_value="",
-               select_cols=None):
     wrapped = CsvDatasetV2(filenames, record_defaults, compression_type,
                            buffer_size, header, field_delim, use_quote_delim,
                            na_value, select_cols)
@@ -885,8 +1047,8 @@ def make_batched_features_dataset_v2(file_pattern,
     dataset = dataset.interleave(
         lambda filename: reader(filename, *reader_args),
         num_parallel_calls=reader_num_threads)
-    options = dataset_ops.Options()
-    options.experimental_deterministic = not sloppy_ordering
+    options = options_lib.Options()
+    options.deterministic = not sloppy_ordering
     dataset = dataset.with_options(options)
   else:
     # Read files sequentially (if reader_num_threads=1) or in parallel
@@ -905,7 +1067,7 @@ def make_batched_features_dataset_v2(file_pattern,
   # Extract values if the `Example` tensors are stored as key-value tuples.
   if dataset_ops.get_legacy_output_types(dataset) == (
       dtypes.string, dtypes.string):
-    dataset = dataset_ops.MapDataset(
+    dataset = map_op._MapDataset(  # pylint: disable=protected-access
         dataset, lambda _, v: v, use_inter_op_parallelism=False)
 
   # Apply dataset repeat and shuffle transformations.
@@ -927,8 +1089,8 @@ def make_batched_features_dataset_v2(file_pattern,
   if label_key:
     if label_key not in features:
       raise ValueError(
-          "The `label_key` provided (%r) must be one of the `features` keys." %
-          label_key)
+          f"The `label_key` provided ({label_key}) must be one of the "
+          f"`features` keys: {features.keys()}.")
     dataset = dataset.map(lambda x: (x, x.pop(label_key)))
 
   dataset = dataset.prefetch(prefetch_buffer_size)
@@ -975,7 +1137,7 @@ def _get_file_names(file_pattern, shuffle):
   """
   if isinstance(file_pattern, list):
     if not file_pattern:
-      raise ValueError("File pattern is empty.")
+      raise ValueError("Argument `file_pattern` should not be empty.")
     file_names = []
     for entry in file_pattern:
       file_names.extend(gfile.Glob(entry))
@@ -983,7 +1145,7 @@ def _get_file_names(file_pattern, shuffle):
     file_names = list(gfile.Glob(file_pattern))
 
   if not file_names:
-    raise ValueError("No files match %s." % file_pattern)
+    raise ValueError(f"No files match `file_pattern` {file_pattern}.")
 
   # Sort files so it will be deterministic for unit tests.
   if not shuffle:
@@ -993,23 +1155,24 @@ def _get_file_names(file_pattern, shuffle):
 
 @tf_export("data.experimental.SqlDataset", v1=[])
 class SqlDatasetV2(dataset_ops.DatasetSource):
-  """A `Dataset` consisting of the results from a SQL query."""
+  """A `Dataset` consisting of the results from a SQL query.
+
+  `SqlDataset` allows a user to read data from the result set of a SQL query.
+  For example:
+
+  ```python
+  dataset = tf.data.experimental.SqlDataset("sqlite", "/foo/bar.sqlite3",
+                                            "SELECT name, age FROM people",
+                                            (tf.string, tf.int32))
+  # Prints the rows of the result set of the above query.
+  for element in dataset:
+    print(element)
+  ```
+  """
 
   def __init__(self, driver_name, data_source_name, query, output_types):
     """Creates a `SqlDataset`.
 
-    `SqlDataset` allows a user to read data from the result set of a SQL query.
-    For example:
-
-    ```python
-    dataset = tf.data.experimental.SqlDataset("sqlite", "/foo/bar.sqlite3",
-                                              "SELECT name, age FROM people",
-                                              (tf.string, tf.int32))
-    # Prints the rows of the result set of the above query.
-    for element in dataset:
-      print(element)
-    ```
-
     Args:
       driver_name: A 0-D `tf.string` tensor containing the database type.
         Currently, the only supported value is 'sqlite'.
