@@ -13,10 +13,6 @@
 # limitations under the License.
 # ==============================================================================
 """Tests for `tf.data.Iterator`."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import warnings
 
 from absl.testing import parameterized
@@ -27,6 +23,7 @@ from tensorflow.core.protobuf import config_pb2
 from tensorflow.python.client import session
 from tensorflow.python.data.kernel_tests import test_base
 from tensorflow.python.data.ops import dataset_ops
+from tensorflow.python.data.ops import from_generator_op
 from tensorflow.python.data.ops import iterator_ops
 from tensorflow.python.data.util import structure
 from tensorflow.python.eager import context
@@ -38,7 +35,7 @@ from tensorflow.python.framework import errors
 from tensorflow.python.framework import function
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import sparse_tensor
-from tensorflow.python.framework import tensor_spec
+from tensorflow.python.framework import tensor
 from tensorflow.python.framework import test_util
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import data_flow_ops
@@ -53,6 +50,7 @@ from tensorflow.python.training import server_lib
 from tensorflow.python.util import compat
 
 
+@test_util.with_eager_op_as_function
 class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
 
   @combinations.generate(test_base.graph_only_combinations())
@@ -72,9 +70,12 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
     dataset = (
         dataset_ops.Dataset.from_tensor_slices([0.0, 1.0, 2.0])
         .map(lambda x: x + var))
-    with self.assertRaisesRegexp(
-        ValueError, r"`Dataset.make_one_shot_iterator\(\)` does not support "
-        "datasets that capture stateful objects.+myvar"):
+    with self.assertRaisesRegex(
+        ValueError, r"A likely cause of this error is that the dataset for "
+        r"which you are calling `make_one_shot_iterator\(\)` captures a "
+        r"stateful object, such as a `tf.Variable` or "
+        r"`tf.lookup.StaticHashTable`, which is not supported. Use "
+        r"`make_initializable_iterator\(\)` instead."):
       dataset_ops.make_one_shot_iterator(dataset)
 
   @combinations.generate(test_base.graph_only_combinations())
@@ -213,17 +214,17 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
     next_element = iterator.get_next()
 
     with self.cached_session() as sess:
-      with self.assertRaisesRegexp(errors.InvalidArgumentError, ""):
+      with self.assertRaisesRegex(errors.InvalidArgumentError, ""):
         sess.run(next_element)
 
       # Test that subsequent attempts to use the iterator also fail.
-      with self.assertRaisesRegexp(errors.InvalidArgumentError, ""):
+      with self.assertRaisesRegex(errors.InvalidArgumentError, ""):
         sess.run(next_element)
 
     with self.cached_session() as sess:
 
       def consumer_thread():
-        with self.assertRaisesRegexp(errors.InvalidArgumentError, ""):
+        with self.assertRaisesRegex(errors.InvalidArgumentError, ""):
           sess.run(next_element)
 
       num_threads = 8
@@ -235,6 +236,13 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
       for t in threads:
         t.join()
 
+  @combinations.generate(test_base.default_test_combinations())
+  def testOneShotIteratorEmptyDataset(self):
+    dataset = dataset_ops.Dataset.range(0)
+    iterator = dataset_ops.make_one_shot_iterator(dataset)
+    with self.assertRaises(errors.OutOfRangeError):
+      self.evaluate(iterator.get_next())
+
   @combinations.generate(test_base.graph_only_combinations())
   def testSimpleSharedResource(self):
     components = (np.array(1, dtype=np.int64),
@@ -293,8 +301,8 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
     get_next = iterator.get_next()
 
     with self.cached_session() as sess:
-      with self.assertRaisesRegexp(errors.FailedPreconditionError,
-                                   "iterator has not been initialized"):
+      with self.assertRaisesRegex(errors.FailedPreconditionError,
+                                  "iterator has not been initialized"):
         sess.run(get_next)
 
   @combinations.generate(test_base.graph_only_combinations())
@@ -381,14 +389,18 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
                                                      dtypes.float64))
 
     # Incompatible structure.
-    with self.assertRaises(ValueError):
+    with self.assertRaisesRegex(
+        ValueError, "The two structures don't have the same nested structure."):
       iterator.make_initializer(
           dataset_ops.Dataset.from_tensors(((constant_op.constant(
               [1, 2, 3], dtype=dtypes.int64),), (constant_op.constant(
                   [4., 5., 6., 7.], dtype=dtypes.float64),))))
 
     # Incompatible types.
-    with self.assertRaises(TypeError):
+    with self.assertRaisesRegex(
+        TypeError,
+        r"Expected output types \(tf.int64, tf.float64\) but got dataset with "
+        r"output types \(tf.int32, tf.float32\)."):
       iterator.make_initializer(
           dataset_ops.Dataset.from_tensors(
               (constant_op.constant([1, 2, 3], dtype=dtypes.int32),
@@ -397,12 +409,27 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
     # Incompatible shapes.
     iterator = iterator_ops.Iterator.from_structure(
         (dtypes.int64, dtypes.float64), ([None], []))
-    with self.assertRaises(TypeError):
+    with self.assertRaisesRegex(
+        TypeError,
+        r"Expected output shapes compatible with .* but got dataset with "
+        r"output shapes.*"):
       iterator.make_initializer(
           dataset_ops.Dataset.from_tensors(
               (constant_op.constant([1, 2, 3], dtype=dtypes.int64),
                constant_op.constant([4., 5., 6., 7.], dtype=dtypes.float64))))
 
+  @combinations.generate(test_base.default_test_combinations())
+  def testReinitializableIteratorEmptyDataset(self):
+    dataset = dataset_ops.Dataset.range(0)
+    iterator = iterator_ops.Iterator.from_structure(
+        dataset_ops.get_legacy_output_types(dataset), [])
+    init_op = iterator.make_initializer(dataset)
+
+    with self.cached_session() as sess:
+      sess.run(init_op)
+      with self.assertRaises(errors.OutOfRangeError):
+        sess.run(iterator.get_next())
+
   @combinations.generate(test_base.graph_only_combinations())
   def testIteratorStringHandle(self):
     dataset_3 = dataset_ops.Dataset.from_tensor_slices([1, 2, 3])
@@ -544,7 +571,7 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
                   structure_iterator.string_handle())
 
     # Assert that getting the (default) string handle creates no ops.
-    self.assertEqual(created_ops, len(ops.get_default_graph().get_operations()))
+    self.assertLen(ops.get_default_graph().get_operations(), created_ops)
 
     # Specifying an explicit name will create a new op.
     handle_with_name = one_shot_iterator.string_handle(name="foo")
@@ -772,7 +799,7 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
     with warnings.catch_warnings(record=True) as w:
       for _ in range(100):
         iterator.get_next()
-    self.assertEqual(100 - iterator_ops.GET_NEXT_CALL_WARNING_THRESHOLD, len(w))
+    self.assertLen(w, 100 - iterator_ops.GET_NEXT_CALL_WARNING_THRESHOLD)
     for warning in w:
       self.assertIn(
           iterator_ops.GET_NEXT_CALL_WARNING_MESSAGE, str(warning.message))
@@ -781,9 +808,8 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
       combinations.times(
           test_base.default_test_combinations(),
           combinations.combine(
-              expected_element_structure=tensor_spec.TensorSpec([],
-                                                                dtypes.float32),
-              expected_output_classes=ops.Tensor,
+              expected_element_structure=tensor.TensorSpec([], dtypes.float32),
+              expected_output_classes=tensor.Tensor,
               expected_output_types=dtypes.float32,
               expected_output_shapes=[[]])))
   def testTensorIteratorStructure(self, expected_element_structure,
@@ -845,13 +871,13 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
           combinations.combine(
               expected_element_structure={
                   "a":
-                      tensor_spec.TensorSpec([], dtypes.float32),
-                  "b": (tensor_spec.TensorSpec([1], dtypes.string),
-                        tensor_spec.TensorSpec([], dtypes.string))
+                      tensor.TensorSpec([], dtypes.float32),
+                  "b": (tensor.TensorSpec([1], dtypes.string),
+                        tensor.TensorSpec([], dtypes.string))
               },
               expected_output_classes={
-                  "a": ops.Tensor,
-                  "b": (ops.Tensor, ops.Tensor)
+                  "a": tensor.Tensor,
+                  "b": (tensor.Tensor, tensor.Tensor)
               },
               expected_output_types={
                   "a": dtypes.float32,
@@ -886,7 +912,7 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
     self.assertEqual(expected_output_shapes,
                      dataset_ops.get_legacy_output_shapes(iterator))
 
-  @combinations.generate(test_base.default_test_combinations())
+  @combinations.generate(test_base.graph_only_combinations())
   def testIteratorGetNextName(self):
     with ops.Graph().as_default():
       iterator = dataset_ops.make_one_shot_iterator(
@@ -946,7 +972,10 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
 
     @def_function.function
     def fn():
-      dataset = dataset_ops._GeneratorDataset(1, init_fn, next_fn, finalize_fn)
+      output_signature = tensor.TensorSpec((), dtypes.int64)
+      dataset = from_generator_op._GeneratorDataset(1, init_fn, next_fn,
+                                                    finalize_fn,
+                                                    output_signature)
       iterator = iter(dataset)
       next(iterator)
 
@@ -955,6 +984,33 @@ class IteratorTest(test_base.DatasetTestBase, parameterized.TestCase):
 
     self.assertEqual(queue.size().numpy(), 2)
 
+  @combinations.generate(test_base.default_test_combinations())
+  def testNoInitializer(self):
+    dataset = dataset_ops.Dataset.range(10)
+    iterator = iterator_ops.Iterator.from_structure(
+        dataset_ops.get_legacy_output_types(dataset), [])
+    with self.assertRaisesRegex(
+        ValueError, "The iterator does not have an initializer."):
+      _ = iterator.initializer
+
+  @combinations.generate(test_base.default_test_combinations())
+  def testtestMissingInput(self):
+    with self.assertRaisesRegex(
+        ValueError,
+        "When `dataset` is not provided, both `components` and `element_spec` "
+        "must be specified."):
+      iterator_ops.OwnedIterator(dataset=None)
+
+  @combinations.generate(test_base.eager_only_combinations())
+  def testExtraElementSpecInput(self):
+    dataset = dataset_ops.Dataset.range(1000)
+    with self.assertRaisesRegex(
+        ValueError,
+        "When `dataset` is provided, `element_spec` and `components` must "
+        "not be specified."):
+      iterator_ops.OwnedIterator(
+          dataset, element_spec=dataset.element_spec)
+
   @combinations.generate(test_base.eager_only_combinations())
   def testLimitedRetracing(self):
     trace_count = [0]
