@@ -13,10 +13,6 @@
 # limitations under the License.
 # ==============================================================================
 """Unit tests for local command-line-interface debug wrapper session."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
 import tempfile
 
@@ -43,6 +39,7 @@ from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import resource_variable_ops  # pylint: disable=unused-import
 from tensorflow.python.ops import sparse_ops
 from tensorflow.python.ops import state_ops
+from tensorflow.python.ops import variable_v1
 from tensorflow.python.ops import variables
 from tensorflow.python.platform import googletest
 from tensorflow.python.training import monitored_session
@@ -73,7 +70,7 @@ class LocalCLIDebuggerWrapperSessionForTest(
     """
 
     local_cli_wrapper.LocalCLIDebugWrapperSession.__init__(
-        self, sess, dump_root=dump_root, log_usage=False)
+        self, sess, dump_root=dump_root)
 
     self._command_sequence = command_sequence
     self._command_pointer = 0
@@ -116,7 +113,7 @@ class LocalCLIDebuggerWrapperSessionForTest(
             config_file_path=os.path.join(tempfile.mkdtemp(), ".tfdbg_config")))
     self._register_this_run_info(readline_cli)
 
-    while True:
+    while self._command_pointer < len(self._command_sequence):
       command = self._command_sequence[self._command_pointer]
       self._command_pointer += 1
 
@@ -136,10 +133,10 @@ class LocalCLIDebuggerWrapperSessionForTest(
 class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
 
   def setUp(self):
-    self._tmp_dir = tempfile.mktemp()
+    self._tmp_dir = tempfile.mkdtemp()
 
-    self.v = variables.VariableV1(10.0, name="v")
-    self.w = variables.VariableV1(21.0, name="w")
+    self.v = variable_v1.VariableV1(10.0, name="v")
+    self.w = variable_v1.VariableV1(21.0, name="w")
     self.delta = constant_op.constant(1.0, name="delta")
     self.inc_v = state_ops.assign_add(self.v, self.delta, name="inc_v")
 
@@ -175,35 +172,25 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
       file_io.delete_recursively(self._tmp_dir)
 
   def testConstructWrapper(self):
-    local_cli_wrapper.LocalCLIDebugWrapperSession(
-        session.Session(), log_usage=False)
-
-  def testConstructWrapperWithExistingEmptyDumpRoot(self):
-    os.mkdir(self._tmp_dir)
-    self.assertTrue(os.path.isdir(self._tmp_dir))
-
-    local_cli_wrapper.LocalCLIDebugWrapperSession(
-        session.Session(), dump_root=self._tmp_dir, log_usage=False)
+    local_cli_wrapper.LocalCLIDebugWrapperSession(session.Session())
 
   def testConstructWrapperWithExistingNonEmptyDumpRoot(self):
-    os.mkdir(self._tmp_dir)
     dir_path = os.path.join(self._tmp_dir, "foo")
     os.mkdir(dir_path)
     self.assertTrue(os.path.isdir(dir_path))
 
-    with self.assertRaisesRegexp(
+    with self.assertRaisesRegex(
         ValueError, "dump_root path points to a non-empty directory"):
       local_cli_wrapper.LocalCLIDebugWrapperSession(
-          session.Session(), dump_root=self._tmp_dir, log_usage=False)
+          session.Session(), dump_root=self._tmp_dir)
 
   def testConstructWrapperWithExistingFileDumpRoot(self):
-    os.mkdir(self._tmp_dir)
     file_path = os.path.join(self._tmp_dir, "foo")
     open(file_path, "a").close()  # Create the file
     self.assertTrue(os.path.isfile(file_path))
-    with self.assertRaisesRegexp(ValueError, "dump_root path points to a file"):
+    with self.assertRaisesRegex(ValueError, "dump_root path points to a file"):
       local_cli_wrapper.LocalCLIDebugWrapperSession(
-          session.Session(), dump_root=file_path, log_usage=False)
+          session.Session(), dump_root=file_path)
 
   def testRunsUnderDebugMode(self):
     wrapped_sess = LocalCLIDebuggerWrapperSessionForTest(
@@ -364,7 +351,7 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
   def testDebuggingMakeCallableTensorRunnerWorks(self):
     wrapped_sess = LocalCLIDebuggerWrapperSessionForTest(
         [["run"], ["run"]], self.sess, dump_root=self._tmp_dir)
-    v = variables.VariableV1(42)
+    v = variable_v1.VariableV1(42)
     tensor_runner = wrapped_sess.make_callable(v)
     self.sess.run(v.initializer)
 
@@ -388,7 +375,7 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
   def testDebuggingMakeCallableOperationRunnerWorks(self):
     wrapped_sess = LocalCLIDebuggerWrapperSessionForTest(
         [["run"], ["run"]], self.sess, dump_root=self._tmp_dir)
-    v = variables.VariableV1(10.0)
+    v = variable_v1.VariableV1(10.0)
     inc_v = state_ops.assign_add(v, 1.0)
     op_runner = wrapped_sess.make_callable(inc_v.op)
     self.sess.run(v.initializer)
@@ -409,7 +396,7 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
     self.assertEqual(1, len(wrapped_sess.observers["debug_dumps"]))
 
   def testDebuggingMakeCallableFromOptionsWithZeroFeedWorks(self):
-    variable_1 = variables.VariableV1(
+    variable_1 = variable_v1.VariableV1(
         10.5, dtype=dtypes.float32, name="variable_1")
     a = math_ops.add(variable_1, variable_1, "callable_a")
     math_ops.add(a, a, "callable_b")
@@ -488,7 +475,7 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
       self.assertIn("callable_b", node_names)
 
   def testDebugMakeCallableFromOptionsWithCustomOptionsAndMetadataWorks(self):
-    variable_1 = variables.VariableV1(
+    variable_1 = variable_v1.VariableV1(
         10.5, dtype=dtypes.float32, name="variable_1")
     a = math_ops.add(variable_1, variable_1, "callable_a")
     math_ops.add(a, a, "callable_b")
@@ -533,16 +520,6 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
     tf_error = wrapped_sess.observers["tf_errors"][0]
     self.assertEqual("y", tf_error.op.name)
 
-  def testRuntimeErrorBeforeGraphExecutionIsRaised(self):
-    # Use an impossible device name to cause an error before graph execution.
-    with ops.device("/device:GPU:1337"):
-      w = variables.VariableV1([1.0] * 10, name="w")
-
-    wrapped_sess = LocalCLIDebuggerWrapperSessionForTest(
-        [["run"]], self.sess, dump_root=self._tmp_dir)
-    with self.assertRaisesRegexp(errors.OpError, r".*[Dd]evice.*1337.*"):
-      wrapped_sess.run(w)
-
   def testRunTillFilterPassesShouldLaunchCLIAtCorrectRun(self):
     wrapped_sess = LocalCLIDebuggerWrapperSessionForTest(
         [["run", "-f", "v_greater_than_twelve"],
@@ -811,7 +788,7 @@ class LocalCLIDebugWrapperSessionTest(test_util.TensorFlowTestCase):
   def testCallingShouldStopMethodOnNonWrappedNonMonitoredSessionErrors(self):
     wrapped_sess = LocalCLIDebuggerWrapperSessionForTest(
         [["run"], ["run"]], self.sess)
-    with self.assertRaisesRegexp(
+    with self.assertRaisesRegex(
         ValueError,
         r"The wrapped session .* does not have a method .*should_stop.*"):
       wrapped_sess.should_stop()
