@@ -14,27 +14,22 @@
 # ==============================================================================
 """Test DistributionStrategy, ReplicaContext, and supporting APIs."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 from absl.testing import parameterized
-import numpy as np
 
 from tensorflow.python.autograph.core import converter_testing
 from tensorflow.python.data.ops import dataset_ops
 from tensorflow.python.distribute import combinations
 from tensorflow.python.distribute import distribute_lib
-from tensorflow.python.distribute import distribution_strategy_context as ds_context
 from tensorflow.python.distribute import input_lib
 from tensorflow.python.distribute import reduce_util
-from tensorflow.python.distribute.cluster_resolver import SimpleClusterResolver
+from tensorflow.python.distribute.cluster_resolver import cluster_resolver as cluster_resolver_lib
+from tensorflow.python.distribute.v1 import input_lib as input_lib_v1
 from tensorflow.python.eager import context
 from tensorflow.python.eager import def_function
 from tensorflow.python.framework import constant_op
-from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.ops import variable_scope
+from tensorflow.python.ops import variable_v1
 from tensorflow.python.ops import variables
 from tensorflow.python.platform import test
 from tensorflow.python.training import server_lib
@@ -75,8 +70,7 @@ class _TestExtended(distribute_lib.StrategyExtendedV1):
 
   def _call_for_each_replica(self, fn, args, kwargs):
     with _TestReplicaContext(
-        self._container_strategy(),
-        replica_id_in_sync_group=constant_op.constant(0, dtypes.int32)):
+        self._container_strategy(), replica_id_in_sync_group=0):
       return fn(*args, **kwargs)
 
   def _create_variable(self, next_creator, **kwargs):
@@ -87,25 +81,20 @@ class _TestExtended(distribute_lib.StrategyExtendedV1):
       self,
       input_fn,
       replication_mode=distribute_lib.InputReplicationMode.PER_WORKER):
-    return input_lib.InputFunctionIterator(input_fn, self._input_workers,
-                                           [distribute_lib.InputContext()],
-                                           self._container_strategy())
+    return input_lib_v1.InputFunctionIterator(input_fn, self._input_workers,
+                                              [distribute_lib.InputContext()],
+                                              self._container_strategy())
 
-  def _experimental_distribute_datasets_from_function(self, dataset_fn,
-                                                      options):
+  def _distribute_datasets_from_function(self, dataset_fn, options):
     return dataset_fn(distribute_lib.InputContext())
 
   def _local_results(self, value):
     return (value,)
 
-  def _reduce_to(self, reduce_op, value, destinations, experimental_hints):
-    del reduce_op, destinations, experimental_hints
+  def _reduce_to(self, reduce_op, value, destinations, options):
+    del reduce_op, destinations, options
     return value
 
-  def _experimental_make_numpy_dataset(self, numpy_input, session):
-    del session
-    return dataset_ops.DatasetV2.from_tensor_slices(numpy_input)
-
   def _experimental_run_steps_on_iterator(self, fn, iterator, iterations,
                                           initial_loop_values=None):
     # TODO(tomhennigan) This is missing many things (e.g. ctx.run_op).
@@ -127,14 +116,18 @@ class _TestExtended(distribute_lib.StrategyExtendedV1):
     else:
       return nest.map_structure(self._unwrap, result)
 
+  def _get_local_replica_id(self, replica_id_in_sync_group):
+    return replica_id_in_sync_group
+
 
 def _assert_in_default_state(t):
-  t.assertIs(ds_context._get_default_replica_context(),
-             ds_context.get_replica_context())
-  t.assertIs(None, ds_context.get_cross_replica_context())
-  t.assertFalse(ds_context.in_cross_replica_context())
-  t.assertIs(ds_context._get_default_strategy(), ds_context.get_strategy())
-  t.assertFalse(ds_context.has_strategy())
+  t.assertIs(distribute_lib._get_default_replica_context(),
+             distribute_lib.get_replica_context())
+  t.assertIs(None, distribute_lib.get_cross_replica_context())
+  t.assertFalse(distribute_lib.in_cross_replica_context())
+  t.assertIs(
+      distribute_lib._get_default_strategy(), distribute_lib.get_strategy())
+  t.assertFalse(distribute_lib.has_strategy())
 
 
 def _run_in_and_out_of_scope(unbound_test_method):
@@ -150,7 +143,7 @@ def _run_in_and_out_of_scope(unbound_test_method):
     # When run under a different strategy the test method should fail.
     another_strategy = _TestStrategy()
     msg = "Mixing different .*Strategy objects"
-    with test_case.assertRaisesRegexp(RuntimeError, msg):
+    with test_case.assertRaisesRegex(RuntimeError, msg):
       with another_strategy.scope():
         unbound_test_method(test_case, dist)
   return wrapper
@@ -163,18 +156,18 @@ class TestStrategyTest(test.TestCase):
     dist = _TestStrategy()
 
     def run_fn():
-      replica_context = ds_context.get_replica_context()
-      self.assertTrue(replica_context is not None)
-      self.assertIs(None, ds_context.get_cross_replica_context())
-      self.assertFalse(ds_context.in_cross_replica_context())
-      self.assertTrue(ds_context.has_strategy())
-      self.assertIs(dist, ds_context.get_strategy())
+      replica_context = distribute_lib.get_replica_context()
+      self.assertIsNotNone(replica_context)
+      self.assertIs(None, distribute_lib.get_cross_replica_context())
+      self.assertFalse(distribute_lib.in_cross_replica_context())
+      self.assertTrue(distribute_lib.has_strategy())
+      self.assertIs(dist, distribute_lib.get_strategy())
       self.assertEqual("foo", replica_context.merge_call(None, test_arg="foo"))
       expected_value = _get_test_variable(
           "bar", variable_scope.VariableSynchronization.AUTO,
           variable_scope.VariableAggregation.NONE)
       self.assertDictEqual(expected_value,
-                           variable_scope.variable(1.0, name="bar"))
+                           variable_v1.VariableV1(1.0, name="bar"))
 
     dist.extended.call_for_each_replica(run_fn)
     with dist.scope():
@@ -185,16 +178,16 @@ class TestStrategyTest(test.TestCase):
     _assert_in_default_state(self)
     dist = _TestStrategy()
     with dist.scope():
-      self.assertIs(None, ds_context.get_replica_context())
-      self.assertIs(dist, ds_context.get_cross_replica_context())
-      self.assertTrue(ds_context.in_cross_replica_context())
-      self.assertTrue(ds_context.has_strategy())
-      self.assertIs(dist, ds_context.get_strategy())
+      self.assertIs(None, distribute_lib.get_replica_context())
+      self.assertIs(dist, distribute_lib.get_cross_replica_context())
+      self.assertTrue(distribute_lib.in_cross_replica_context())
+      self.assertTrue(distribute_lib.has_strategy())
+      self.assertIs(dist, distribute_lib.get_strategy())
       expected_value = _get_test_variable(
           "baz", variable_scope.VariableSynchronization.AUTO,
           variable_scope.VariableAggregation.NONE)
       self.assertDictEqual(expected_value,
-                           variable_scope.variable(1.0, name="baz"))
+                           variable_v1.VariableV1(1.0, name="baz"))
     _assert_in_default_state(self)
 
   def testScopeDeviceNestingError(self):
@@ -204,9 +197,9 @@ class TestStrategyTest(test.TestCase):
     dist.extended._default_device = "/device:GPU:0"
     scope = dist.scope()
     scope.__enter__()
-    self.assertIs(dist, ds_context.get_strategy())
+    self.assertIs(dist, distribute_lib.get_strategy())
     with ops.device("/device:CPU:0"):
-      with self.assertRaisesRegexp(RuntimeError, "Device scope nesting error"):
+      with self.assertRaisesRegex(RuntimeError, "Device scope nesting error"):
         scope.__exit__(None, None, None)
     scope.__exit__(None, None, None)
     _assert_in_default_state(self)
@@ -220,10 +213,10 @@ class TestStrategyTest(test.TestCase):
     dist = _TestStrategy()
     scope = dist.scope()
     scope.__enter__()
-    self.assertIs(dist, ds_context.get_strategy())
+    self.assertIs(dist, distribute_lib.get_strategy())
     with variable_scope.variable_creator_scope(creator):
-      with self.assertRaisesRegexp(RuntimeError,
-                                   "Variable creator scope nesting error"):
+      with self.assertRaisesRegex(RuntimeError,
+                                  "Variable creator scope nesting error"):
         scope.__exit__(None, None, None)
     scope.__exit__(None, None, None)
     _assert_in_default_state(self)
@@ -237,10 +230,10 @@ class TestStrategyTest(test.TestCase):
       dist = _TestStrategy()
       scope = dist.scope()
       scope.__enter__()
-      self.assertIs(dist, ds_context.get_strategy())
+      self.assertIs(dist, distribute_lib.get_strategy())
       with variable_scope.variable_scope("AA"):
-        with self.assertRaisesRegexp(RuntimeError,
-                                     "Variable scope nesting error"):
+        with self.assertRaisesRegex(RuntimeError,
+                                    "Variable scope nesting error"):
           scope.__exit__(None, None, None)
     _assert_in_default_state(self)
 
@@ -253,7 +246,7 @@ class TestStrategyTest(test.TestCase):
           variable_scope.VariableAggregation.MEAN)
       self.assertDictEqual(
           expected_value,
-          variable_scope.variable(
+          variable_v1.VariableV1(
               1.0,
               name="baz",
               synchronization=variable_scope.VariableSynchronization.ON_WRITE,
@@ -264,38 +257,38 @@ class TestStrategyTest(test.TestCase):
     _assert_in_default_state(self)
     dist = _TestStrategy()
     dist2 = _TestStrategy()
-    ds_context.experimental_set_strategy(dist)
-    self.assertIs(None, ds_context.get_replica_context())
-    self.assertIs(dist, ds_context.get_cross_replica_context())
-    self.assertTrue(ds_context.in_cross_replica_context())
-    self.assertTrue(ds_context.has_strategy())
-    self.assertIs(dist, ds_context.get_strategy())
+    distribute_lib.experimental_set_strategy(dist)
+    self.assertIs(None, distribute_lib.get_replica_context())
+    self.assertIs(dist, distribute_lib.get_cross_replica_context())
+    self.assertTrue(distribute_lib.in_cross_replica_context())
+    self.assertTrue(distribute_lib.has_strategy())
+    self.assertIs(dist, distribute_lib.get_strategy())
     expected_value = _get_test_variable(
         "baz", variable_scope.VariableSynchronization.AUTO,
         variable_scope.VariableAggregation.NONE)
     self.assertDictEqual(expected_value,
-                         variable_scope.variable(1.0, name="baz"))
-    ds_context.experimental_set_strategy(dist2)
-    self.assertIs(dist2, ds_context.get_strategy())
-    ds_context.experimental_set_strategy(None)
+                         variable_v1.VariableV1(1.0, name="baz"))
+    distribute_lib.experimental_set_strategy(dist2)
+    self.assertIs(dist2, distribute_lib.get_strategy())
+    distribute_lib.experimental_set_strategy(None)
     _assert_in_default_state(self)
 
   def testSetStrategyInScope(self):
     _assert_in_default_state(self)
     dist = _TestStrategy()
     with dist.scope():
-      with self.assertRaisesRegexp(
+      with self.assertRaisesRegex(
           RuntimeError,
           "Must not be called inside a `tf.distribute.Strategy` scope"):
-        ds_context.experimental_set_strategy(_TestStrategy())
-      with self.assertRaisesRegexp(
+        distribute_lib.experimental_set_strategy(_TestStrategy())
+      with self.assertRaisesRegex(
           RuntimeError,
           "Must not be called inside a `tf.distribute.Strategy` scope"):
-        ds_context.experimental_set_strategy(dist)
-      with self.assertRaisesRegexp(
+        distribute_lib.experimental_set_strategy(dist)
+      with self.assertRaisesRegex(
           RuntimeError,
           "Must not be called inside a `tf.distribute.Strategy` scope"):
-        ds_context.experimental_set_strategy(None)
+        distribute_lib.experimental_set_strategy(None)
     _assert_in_default_state(self)
 
   def testSameScopeNesting(self):
@@ -303,24 +296,23 @@ class TestStrategyTest(test.TestCase):
     dist = _TestStrategy()
     scope_a = dist.scope()
     with scope_a:
-      self.assertIs(dist, ds_context.get_strategy())
+      self.assertIs(dist, distribute_lib.get_strategy())
       scope_b = dist.scope()
       with scope_b:
-        self.assertIs(dist, ds_context.get_strategy())
+        self.assertIs(dist, distribute_lib.get_strategy())
         with scope_a:
-          self.assertIs(dist, ds_context.get_strategy())
-        self.assertIs(dist, ds_context.get_strategy())
-      self.assertIs(dist, ds_context.get_strategy())
+          self.assertIs(dist, distribute_lib.get_strategy())
+        self.assertIs(dist, distribute_lib.get_strategy())
+      self.assertIs(dist, distribute_lib.get_strategy())
       dist2 = _TestStrategy()
       scope2 = dist2.scope()
-      with self.assertRaisesRegexp(
-          RuntimeError,
-          "Mixing different tf.distribute.Strategy objects"):
+      with self.assertRaisesRegex(
+          RuntimeError, "Mixing different tf.distribute.Strategy objects"):
         with scope2:
           pass
     _assert_in_default_state(self)
     with scope_b:
-      self.assertIs(dist, ds_context.get_strategy())
+      self.assertIs(dist, distribute_lib.get_strategy())
     _assert_in_default_state(self)
 
   @_run_in_and_out_of_scope
@@ -348,11 +340,24 @@ class TestStrategyTest(test.TestCase):
       self.assertEqual(self.evaluate(y), self.evaluate(y_r))
 
   @_run_in_and_out_of_scope
-  def testExperimentalMakeNumpyDataset(self, dist):
-    numpy_input = np.ones([10], dtype=np.float32)
-    dataset = dist.experimental_make_numpy_dataset(numpy_input)
-    self.assertEqual(
-        self.evaluate(dataset.reduce(0., lambda a, b: a + b)), 10.)
+  def testReduceMeanAxis(self, dist):
+    x = constant_op.constant([[1., 2.], [3., 4.]])
+    x_r = dist.reduce(reduce_util.ReduceOp.MEAN, x, axis=None)
+    self.assertAllEqual(self.evaluate(x), self.evaluate(x_r))
+    x_r = dist.reduce(reduce_util.ReduceOp.MEAN, x, axis=0)
+    self.assertAllEqual([2., 3.], self.evaluate(x_r))
+    x_r = dist.reduce(reduce_util.ReduceOp.MEAN, x, axis=(0, 1))
+    self.assertEqual(2.5, self.evaluate(x_r))
+
+  @_run_in_and_out_of_scope
+  def testReduceSumAxis(self, dist):
+    x = constant_op.constant([[1., 2.], [3., 4.]])
+    x_r = dist.reduce(reduce_util.ReduceOp.SUM, x, axis=None)
+    self.assertAllEqual(self.evaluate(x), self.evaluate(x_r))
+    x_r = dist.reduce(reduce_util.ReduceOp.SUM, x, axis=0)
+    self.assertAllEqual([4., 6.], self.evaluate(x_r))
+    x_r = dist.reduce(reduce_util.ReduceOp.SUM, x, axis=(0, 1))
+    self.assertEqual(10., self.evaluate(x_r))
 
   @_run_in_and_out_of_scope
   def testExperimentalRunStepsOnIterator(self, dist):
@@ -431,7 +436,8 @@ class TestStrategyTest(test.TestCase):
         "ps": ["ps0:2222", "ps1:2222"],
         "worker": ["worker0:2222", "worker1:2222", "worker2:2222"]
     })
-    cluster_resolver = SimpleClusterResolver(base_cluster_spec)
+    cluster_resolver = cluster_resolver_lib.SimpleClusterResolver(
+        base_cluster_spec)
     dist.extended._cluster_resolver = cluster_resolver
     self.assertIs(dist.cluster_resolver, cluster_resolver)
 
@@ -456,16 +462,16 @@ class DefaultDistributionStrategyTest(test.TestCase, parameterized.TestCase):
     _assert_in_default_state(self)
 
     def merge_fn(dist, s):
-      self.assertIs(ds_context._get_default_strategy(), dist)
-      self.assertIs(None, ds_context.get_replica_context())
-      self.assertIs(dist, ds_context.get_cross_replica_context())
-      self.assertTrue(ds_context.in_cross_replica_context())
-      self.assertIs(dist, ds_context.get_strategy())
-      self.assertFalse(ds_context.has_strategy())
+      self.assertIs(distribute_lib._get_default_strategy(), dist)
+      self.assertIs(None, distribute_lib.get_replica_context())
+      self.assertIs(dist, distribute_lib.get_cross_replica_context())
+      self.assertTrue(distribute_lib.in_cross_replica_context())
+      self.assertIs(dist, distribute_lib.get_strategy())
+      self.assertFalse(distribute_lib.has_strategy())
       return "foo_" + s
 
-    replica_ctx = ds_context.get_replica_context()
-    self.assertIs(ds_context._get_default_replica_context(), replica_ctx)
+    replica_ctx = distribute_lib.get_replica_context()
+    self.assertIs(distribute_lib._get_default_replica_context(), replica_ctx)
     self.assertEqual("foo_bar", replica_ctx.merge_call(merge_fn, args=("bar",)))
     _assert_in_default_state(self)
 
@@ -478,7 +484,7 @@ class DefaultDistributionStrategyTest(test.TestCase, parameterized.TestCase):
 
     @def_function.function  # AutoGraph is default-on only within tf.function
     def test_fn():
-      replica_ctx = ds_context.get_replica_context()
+      replica_ctx = distribute_lib.get_replica_context()
       replica_ctx.merge_call(merge_fn, args=("bar",))
 
     test_fn()
@@ -488,34 +494,34 @@ class DefaultDistributionStrategyTest(test.TestCase, parameterized.TestCase):
 
     test_strategy = _TestStrategy2()
     with test_strategy.scope():
-      variable_scope.variable(1.0, name="before")
+      variable_v1.VariableV1(1.0, name="before")
 
-    default_strategy = ds_context._get_default_strategy()
+    default_strategy = distribute_lib._get_default_strategy()
     scope = default_strategy.scope()
     with scope:
       _assert_in_default_state(self)
 
       with test_strategy.scope():
-        with self.assertRaisesRegexp(
+        with self.assertRaisesRegex(
             RuntimeError, "Mixing different tf.distribute.Strategy objects"):
-          variable_scope.variable(1.0, name="error")
+          variable_v1.VariableV1(1.0, name="error")
 
       with scope:
         _assert_in_default_state(self)
 
         with test_strategy.scope():
-          with self.assertRaisesRegexp(
+          with self.assertRaisesRegex(
               RuntimeError, "Mixing different tf.distribute.Strategy objects"):
-            variable_scope.variable(1.0, name="also_error")
+            variable_v1.VariableV1(1.0, name="also_error")
 
       _assert_in_default_state(self)
 
     _assert_in_default_state(self)
     with test_strategy.scope():
-      variable_scope.variable(1.0, name="after")
+      variable_v1.VariableV1(1.0, name="after")
 
   def testExperimentalRunV2(self):
-    default_strategy = ds_context._get_default_strategy()
+    default_strategy = distribute_lib._get_default_strategy()
     dataset = dataset_ops.Dataset.range(10).batch(2)
     iterator = default_strategy.extended._make_dataset_iterator(dataset)
     next_val = iterator.get_next()
@@ -528,7 +534,7 @@ class DefaultDistributionStrategyTest(test.TestCase, parameterized.TestCase):
 
   @combinations.generate(combinations.combine(mode=["graph", "eager"]))
   def testDistributedDatasets(self):
-    default_strategy = ds_context._get_default_strategy()
+    default_strategy = distribute_lib._get_default_strategy()
     if context.executing_eagerly():
       dataset_fn = lambda _: dataset_ops.DatasetV2.range(10).batch(2)
       dist_dataset = default_strategy.experimental_distribute_dataset(
@@ -545,21 +551,31 @@ class DefaultDistributionStrategyTest(test.TestCase, parameterized.TestCase):
 
   @combinations.generate(combinations.combine(mode=["graph", "eager"]))
   def testDistributedDatasetsFromFunction(self):
-    default_strategy = ds_context._get_default_strategy()
+    default_strategy = distribute_lib._get_default_strategy()
     if context.executing_eagerly():
       dataset_fn = lambda _: dataset_ops.DatasetV2.range(10).batch(2)
       dist_dataset_from_func = \
-          default_strategy.experimental_distribute_datasets_from_function(
+          default_strategy.distribute_datasets_from_function(
               dataset_fn)
       next_val = next(iter(dist_dataset_from_func))
       self.assertAllEqual([0, 1], self.evaluate(next_val))
     else:
       dataset_fn = lambda _: dataset_ops.DatasetV2.range(10).batch(2)
       dist_dataset_from_func = \
-        default_strategy.experimental_distribute_datasets_from_function(
+        default_strategy.distribute_datasets_from_function(
             dataset_fn)
       dataset_ops.make_initializable_iterator(dist_dataset_from_func)
 
+  @combinations.generate(combinations.combine(tf_api_version=1))
+  def testV1(self):
+    self.assertIsInstance(
+        distribute_lib.get_strategy(), distribute_lib.StrategyV1)
+
+  @combinations.generate(combinations.combine(tf_api_version=2))
+  def testV2(self):
+    self.assertIsInstance(
+        distribute_lib.get_strategy(), distribute_lib.Strategy)
+
 
 class InputContextTest(test.TestCase):
 
