@@ -12,19 +12,26 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
+// Must be included first
+// clang-format off
+#include "tensorflow/tsl/python/lib/core/numpy.h" //NOLINT
+// clang-format on
 
 #include "tensorflow/python/eager/pywrap_tensor.h"
 
-#include <stdlib.h>
-#include <string.h>
+#include <stdlib.h>  // NOLINT
+#include <string.h>  // NOLINT
 
-#include <cmath>
+#include <cmath>  // NOLINT
 
 #include "structmember.h"  // NOLINT // For PyMemberDef
-#include "pybind11/pybind11.h"
+#include "pybind11/pybind11.h"  // from @pybind11
 #include "tensorflow/c/c_api.h"
 #include "tensorflow/c/eager/c_api.h"
 #include "tensorflow/c/eager/c_api_internal.h"
+#include "tensorflow/c/eager/tfe_context_internal.h"
+#include "tensorflow/c/eager/tfe_tensorhandle_internal.h"
+#include "tensorflow/c/safe_ptr.h"
 #include "tensorflow/c/tf_status.h"
 #include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/framework/types.pb.h"
@@ -33,16 +40,54 @@ limitations under the License.
 #include "tensorflow/python/eager/pywrap_tfe.h"
 #include "tensorflow/python/lib/core/ndarray_tensor.h"
 #include "tensorflow/python/lib/core/ndarray_tensor_bridge.h"
-#include "tensorflow/python/lib/core/numpy.h"
 #include "tensorflow/python/lib/core/py_exception_registry.h"
 #include "tensorflow/python/lib/core/py_seq_tensor.h"
-#include "tensorflow/python/lib/core/safe_ptr.h"
+#include "tensorflow/python/lib/core/pybind11_status.h"
+#include "tensorflow/python/lib/core/safe_pyobject_ptr.h"
 
 // forward declare
 struct EagerTensor;
+namespace tensorflow {
 
+// Convert a TFE_TensorHandle to a Python numpy.ndarray object.
+// The two may share underlying storage so changes to one may reflect in the
+// other.
+PyObject* TFE_TensorHandleToNumpy(TFE_TensorHandle* handle, TF_Status* status) {
+  if (TFE_TensorHandleDataType(handle) == TF_RESOURCE) {
+    TF_SetStatus(status, TF_INVALID_ARGUMENT,
+                 "Cannot convert a Tensor of dtype resource to a NumPy array.");
+    return nullptr;
+  }
+
+  if (TFE_TensorHandleDataType(handle) == TF_VARIANT) {
+    TF_SetStatus(status, TF_INVALID_ARGUMENT,
+                 "Cannot convert a Tensor of dtype variant to a NumPy array.");
+    return nullptr;
+  }
+  tensorflow::Safe_TF_TensorPtr tensor = nullptr;
+  Py_BEGIN_ALLOW_THREADS;
+  tensor = tensorflow::make_safe(TFE_TensorHandleResolve(handle, status));
+  Py_END_ALLOW_THREADS;
+  if (!status->status.ok()) {
+    return nullptr;
+  }
+
+  PyObject* ret = nullptr;
+  auto cppstatus =
+      tensorflow::TF_TensorToMaybeAliasedPyArray(std::move(tensor), &ret);
+  tensorflow::Set_TF_Status_from_Status(status, cppstatus);
+  if (!status->status.ok()) {
+    Py_XDECREF(ret);
+    return nullptr;
+  }
+  CHECK_NE(ret, nullptr);
+  return ret;
+}
+}  // namespace tensorflow
 namespace {
 
+using tensorflow::TFE_TensorHandleToNumpy;
+
 // An instance of _EagerTensorProfiler that will receive callbacks about
 // events on eager tensors. This is set by TFE_Py_InitEagerTensor, if at all.
 PyObject* eager_tensor_profiler = nullptr;
@@ -87,35 +132,6 @@ TFE_Context* GetContextHandle(PyObject* py_context) {
   return ctx;
 }
 
-// Convert a TFE_TensorHandle to a Python numpy.ndarray object.
-// The two may share underlying storage so changes to one may reflect in the
-// other.
-PyObject* TFE_TensorHandleToNumpy(TFE_TensorHandle* handle, TF_Status* status) {
-  if (TFE_TensorHandleDataType(handle) == TF_RESOURCE) {
-    TF_SetStatus(status, TF_INVALID_ARGUMENT,
-                 "Cannot convert a Tensor of dtype resource to a NumPy array.");
-    return nullptr;
-  }
-
-  tensorflow::Safe_TF_TensorPtr tensor = nullptr;
-  Py_BEGIN_ALLOW_THREADS;
-  tensor = tensorflow::make_safe(TFE_TensorHandleResolve(handle, status));
-  Py_END_ALLOW_THREADS;
-  if (!status->status.ok()) {
-    return nullptr;
-  }
-
-  PyObject* ret = nullptr;
-  auto cppstatus =
-      tensorflow::TF_TensorToMaybeAliasedPyArray(std::move(tensor), &ret);
-  tensorflow::Set_TF_Status_from_Status(status, cppstatus);
-  if (!status->status.ok()) {
-    Py_XDECREF(ret);
-    return nullptr;
-  }
-  CHECK_NE(ret, nullptr);
-  return ret;
-}
 
 // Helper function to convert `v` to a tensorflow::DataType and store it in
 // `*out`. Returns true on success, false otherwise.
@@ -180,13 +196,9 @@ int ConvertDeviceName(PyObject* obj, const char** dst) {
   return 1;
 }
 
-void RaiseExceptionTypeFromTFStatus(TF_Status* status) {
-  TF_Code code = TF_GetCode(status);
-  PyObject* exception = tensorflow::PyExceptionRegistry::Lookup(code);
-  PyErr_SetObject(exception,
-                  pybind11::make_tuple(pybind11::none(), pybind11::none(),
-                                       TF_Message(status))
-                      .ptr());
+void RaiseExceptionTypeFromTFStatus(TF_Status* tf_status) {
+  auto status = tensorflow::StatusFromTF_Status(tf_status);
+  tsl::SetRegisteredErrFromStatus(status);
 }
 
 }  // namespace
@@ -255,6 +267,28 @@ TFE_TensorHandle* EagerCast(TFE_Context* ctx, TFE_TensorHandle* handle,
 #undef RETURN_ERROR
 }
 
+Safe_TFE_TensorHandlePtr EagerConst(TFE_Context* ctx, TFE_TensorHandle* handle,
+                                    const char* device_name,
+                                    TF_Status* out_status) {
+  const char* op_name = "_EagerConst";
+  std::unique_ptr<TFE_Op, decltype(&TFE_DeleteOp)> op(
+      TFE_NewOp(ctx, op_name, out_status), TFE_DeleteOp);
+  if (!out_status->status.ok()) return nullptr;
+  TFE_OpSetDevice(op.get(), device_name, out_status);
+  if (!out_status->status.ok()) return nullptr;
+  TFE_OpAddInput(op.get(), handle, out_status);
+  if (!out_status->status.ok()) return nullptr;
+  TFE_OpSetAttrType(op.get(), "T", TFE_TensorHandleDataType(handle));
+  TFE_TensorHandle* output = nullptr;
+  int num_outputs = 1;
+  TFE_Execute(op.get(), &output, &num_outputs, out_status);
+  Safe_TFE_TensorHandlePtr result(output);
+  if (!out_status->status.ok() || num_outputs != 1) {
+    return nullptr;
+  }
+  return result;
+}
+
 TFE_TensorHandle* ConvertToEagerTensorUncached(TFE_Context* ctx,
                                                PyObject* value,
                                                tensorflow::DataType dtype,
@@ -303,19 +337,53 @@ TFE_TensorHandle* ConvertToEagerTensorUncached(TFE_Context* ctx,
     }
   }
 
-  // We always generate CPU:0 tensors, but we may need to change the device
-  // slightly, as for example from /job:localhost/... to /job:worker/...
-  //
-  // Note that this is a shallow copy and will share the underlying buffer,
-  // because we are copying to the same device.
-  if (device_name != nullptr &&
-      strstr(device_name, "/device:CPU:0") != nullptr) {
-    handle = make_safe(TFE_TensorHandleCopyToDevice(handle.get(), ctx,
-                                                    device_name, status.get()));
-    const TF_Code code = TF_GetCode(status.get());
-    if (code != TF_OK) {
-      RaiseExceptionTypeFromTFStatus(status.get());
-      return nullptr;
+  // We always initially generate CPU:0 tensors. Copy to the current device.
+  if (device_name != nullptr) {
+    if (strstr(device_name, "/device:CPU:0") != nullptr) {
+      // We always generate CPU:0 tensors, but we may need to change the device
+      // slightly, as for example from /job:localhost/... to /job:worker/...
+      //
+      // Note that this is a shallow copy and will share the underlying buffer,
+      // because we are copying to the same device.
+      handle = make_safe(TFE_TensorHandleCopyToDevice(
+          handle.get(), ctx, device_name, status.get()));
+      const TF_Code code = TF_GetCode(status.get());
+      if (code != TF_OK) {
+        RaiseExceptionTypeFromTFStatus(status.get());
+        return nullptr;
+      }
+    } else {
+      /*Copy the constant to the current device. Identity is sometimes
+        overloaded to allow copies like this, but using a different op allows
+        devices to support constant creation without allowing copies via
+        identity ops.
+
+        Note that running this _EagerConst op limits mirroring of cached Python
+        literals somewhat. Mirroring of constants themselves works:
+
+        with tf.device("GPU:0"):
+          tf.constant(1.)  # Cached on CPU:0, mirrored to GPU:0
+        with tf.device("GPU:1"):
+          tf.constant(1.)  # Cache hit for the CPU version, new mirror to GPU:1.
+        with tf.device("GPU:1"):
+          tf.constant(1.)  # Cache hit for the CPU version, cached mirror
+
+        But mirrors for the output of `tf.constant` are not shared just because
+        there was a cache hit for the input literal, because of _EagerConst:
+
+        x = tf.constant(2.)  # Cached on CPU:0
+        with tf.device("GPU:1"):
+          tf.identity(x)  # `x` now mirrored to GPU:1
+        y = tf.constant(2.)  # Cache hit for CPU version
+        with tf.device("GPU:1"):
+          tf.identity(y)  # `y` now mirrored on GPU:1 (new copy!)*/
+      handle =
+          tensorflow::EagerConst(ctx, handle.get(), device_name, status.get());
+      const TF_Code code = TF_GetCode(status.get());
+      if (code != TF_OK) {
+        RaiseExceptionTypeFromTFStatus(status.get());
+        return nullptr;
+      }
     }
   }
 
@@ -331,12 +399,12 @@ TFE_TensorHandle* ConvertToEagerTensor(TFE_Context* ctx, PyObject* value,
   // TODO(slebedev): also cache singleton NumPy arrays and scalars?
   if (PyArray_IsPythonNumber(value)) {
     auto* cache = TFE_TensorHandleCache::Get();
-    TFE_TensorHandle* handle = cache->Lookup(value, dtype, device_name);
+    TFE_TensorHandle* handle = cache->Lookup(value, dtype, ctx, device_name);
     if (handle != nullptr) return handle;
     handle = ConvertToEagerTensorUncached(ctx, value, dtype, device_name);
     if (handle == nullptr) return nullptr;
     if (!PyFloat_Check(value) || std::isfinite(PyFloat_AS_DOUBLE(value))) {
-      cache->Insert(value, dtype, device_name, handle);
+      cache->Insert(value, dtype, ctx, device_name, handle);
     }
     return handle;
   } else {
@@ -439,7 +507,7 @@ int EagerTensor_init(EagerTensor* self, PyObject* args, PyObject* kwds) {
   self->handle_data = Py_None;
   Py_INCREF(Py_None);
   self->tensor_shape = Py_None;
-  self->status.status = tensorflow::Status::OK();
+  self->status.status = ::tensorflow::OkStatus();
   self->dict = nullptr;
   self->weakreflist = nullptr;
   self->context = nullptr;
@@ -486,7 +554,11 @@ void EagerTensor_dealloc(EagerTensor* self) {
   // create it ourselves.
   Py_CLEAR(self->dict);
   if (self->handle != nullptr) {
+    // Destructor may call arbitrary functions that end up calling into
+    // Python from another thread.
+    Py_BEGIN_ALLOW_THREADS;
     TFE_DeleteTensorHandle(self->handle);
+    Py_END_ALLOW_THREADS;
     self->handle = nullptr;
   }
 
@@ -519,14 +591,23 @@ static PyObject* EagerTensor_shape_tuple(EagerTensor* self) {
   if (code != TF_OK) {
     RaiseExceptionTypeFromTFStatus(&self->status);
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   }
   PyObject* shape = PyTuple_New(n);
   if (PyErr_Occurred()) return nullptr;
   for (int i = 0; i < n; ++i) {
-    PyObject* dim =
-        PyLong_FromLongLong(TFE_TensorHandleDim(handle, i, &self->status));
+    int64_t dim_c_value = TFE_TensorHandleDim(handle, i, &self->status);
+    PyObject* dim;
+    // The C++ convention is -1 for unknown/variable axis lengths. Translate
+    // that to the Python "None" convention. Unknown axis lengths are unusual
+    // for eager tensors.
+    if (dim_c_value < 0) {
+      Py_IncRef(Py_None);
+      dim = Py_None;
+    } else {
+      dim = PyLong_FromLongLong(dim_c_value);
+    }
     code = TF_GetCode(&self->status);
     if (code != TF_OK || dim == nullptr ||
         PyTuple_SetItem(shape, i, dim) != 0) {
@@ -536,7 +617,7 @@ static PyObject* EagerTensor_shape_tuple(EagerTensor* self) {
         PyErr_SetString(PyExc_RuntimeError, "Error while creating shape");
       }
       // Cleanup self->status before returning.
-      self->status.status = tensorflow::Status::OK();
+      self->status.status = ::tensorflow::OkStatus();
       Py_DECREF(shape);
       if (dim != nullptr) Py_DECREF(dim);
       return nullptr;
@@ -548,9 +629,9 @@ static PyObject* EagerTensor_shape_tuple(EagerTensor* self) {
 // Getter for `_rank`.
 static PyObject* EagerTensor_rank(EagerTensor* self) {
   int num_dims = TFE_TensorHandleNumDims(self->handle, &self->status);
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, nullptr)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status, nullptr)) {
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   }
 #if PY_MAJOR_VERSION < 3
@@ -564,9 +645,9 @@ static PyObject* EagerTensor_rank(EagerTensor* self) {
 static PyObject* EagerTensor_num_elements(EagerTensor* self) {
   auto handle = self->handle;
   int n = TFE_TensorHandleNumElements(handle, &self->status);
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, nullptr)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status, nullptr)) {
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   }
   return PyLong_FromLongLong(n);
@@ -614,9 +695,10 @@ static PyObject* EagerTensor_copy_to_device(EagerTensor* self, PyObject* args,
   TFE_TensorHandle* handle = TFE_TensorHandleCopyToDevice(
       self->handle, GetContextHandle(self->context), device_name,
       &self->status);
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, PyExc_RuntimeError)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status,
+                                                  PyExc_RuntimeError)) {
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   }
 
@@ -630,22 +712,49 @@ static PyObject* EagerTensor_copy_to_device(EagerTensor* self, PyObject* args,
 // Note that if `self` is not on CPU, we raise an Exception.
 static PyObject* EagerTensor_numpy_internal(EagerTensor* self) {
   auto* py_array = TFE_TensorHandleToNumpy(self->handle, &self->status);
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, nullptr)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status, nullptr)) {
     Py_XDECREF(py_array);
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   } else {
     return PyArray_Return(reinterpret_cast<PyArrayObject*>(py_array));
   }
 }
 
+// Function `_prefer_custom_summarizer`.
+//
+// A hint that callers should prefer `SummarizeValue` to resolving this handle
+// and formatting the tensor.
+static PyObject* EagerTensor_prefer_custom_summarizer(EagerTensor* self) {
+  if (tensorflow::unwrap(self->handle)->PreferCustomSummarizer()) {
+    Py_RETURN_TRUE;
+  } else {
+    Py_RETURN_FALSE;
+  }
+}
+
+// Function `_summarize_value`.
+//
+// Returns a string PyObject which summarizes the value of this tensor. It does
+// not include a shape or dtype.
+static PyObject* EagerTensor_summarize_value(EagerTensor* self) {
+  std::string summary;
+  tensorflow::Status status =
+      tensorflow::unwrap(self->handle)->SummarizeValue(summary);
+  if (MaybeRaiseExceptionFromStatus(status, nullptr)) {
+    return nullptr;
+  }
+  return PyUnicode_FromString(summary.c_str());
+}
+
 // Getter `device`.
 static PyObject* EagerTensor_device(EagerTensor* self) {
   const char* device = TFE_TensorHandleDeviceName(self->handle, &self->status);
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, PyExc_ValueError)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status,
+                                                  PyExc_ValueError)) {
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   }
 #if PY_MAJOR_VERSION >= 3
@@ -659,9 +768,10 @@ static PyObject* EagerTensor_device(EagerTensor* self) {
 static PyObject* EagerTensor_backing_device(EagerTensor* self) {
   const char* device =
       TFE_TensorHandleBackingDeviceName(self->handle, &self->status);
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, PyExc_ValueError)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status,
+                                                  PyExc_ValueError)) {
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return nullptr;
   }
 #if PY_MAJOR_VERSION >= 3
@@ -720,6 +830,11 @@ static PyMethodDef EagerTensor_methods[] = {
      PyDoc_STR("Copies the tensor to the desired device.")},
     {"_num_elements", (PyCFunction)EagerTensor_num_elements, METH_NOARGS,
      PyDoc_STR("Number of elements in the tensor.")},
+    {"_prefer_custom_summarizer",
+     (PyCFunction)EagerTensor_prefer_custom_summarizer, METH_NOARGS,
+     PyDoc_STR("Indicates whether _numpy_internal loses information.")},
+    {"_summarize_value", (PyCFunction)EagerTensor_summarize_value, METH_NOARGS,
+     PyDoc_STR("A string which summarizes the value of this tensor.")},
     {nullptr, nullptr},
 };
 
@@ -735,9 +850,10 @@ static int EagerTensor_getbuffer(EagerTensor* self, Py_buffer* view,
   // implementation.
   auto py_array = tensorflow::make_safe(
       TFE_TensorHandleToNumpy(self->handle, &self->status));
-  if (MaybeRaiseExceptionFromTFStatus(&self->status, PyExc_BufferError)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&self->status,
+                                                  PyExc_BufferError)) {
     // Cleanup self->status before returning.
-    self->status.status = tensorflow::Status::OK();
+    self->status.status = ::tensorflow::OkStatus();
     return -1;
   }
   if (PyObject_GetBuffer(py_array.get(), view, flags) < 0) {
@@ -860,7 +976,7 @@ PyObject* EagerTensorFromHandle(TFE_TensorHandle* handle,
     Py_INCREF(Py_None);
     t->tensor_shape = Py_None;
     t->handle = handle;
-    t->status.status = tensorflow::Status::OK();
+    t->status.status = ::tensorflow::OkStatus();
     t->weakreflist = nullptr;
     PyObject* py_context = GetPyEagerContext();
     if (py_context == nullptr) {
@@ -877,7 +993,7 @@ PyObject* EagerTensorFromHandle(TFE_TensorHandle* handle,
   return reinterpret_cast<PyObject*>(t);
 }
 
-tensorflow::int64 PyEagerTensor_ID(const PyObject* tensor) {
+int64_t PyEagerTensor_ID(const PyObject* tensor) {
   DCHECK(EagerTensor_CheckExact(tensor));
   return reinterpret_cast<const EagerTensor*>(tensor)->id;
 }
@@ -888,16 +1004,16 @@ tensorflow::DataType PyEagerTensor_Dtype(const PyObject* tensor) {
       reinterpret_cast<const EagerTensor*>(tensor)->handle));
 }
 
-tensorflow::int64 PyEagerTensor_NumElements(PyObject* tensor) {
+int64_t PyEagerTensor_NumElements(PyObject* tensor) {
   DCHECK(EagerTensor_CheckExact(tensor));
   EagerTensor* as_c_eager_tensor = reinterpret_cast<EagerTensor*>(tensor);
-  tensorflow::int64 result = TFE_TensorHandleNumElements(
-      as_c_eager_tensor->handle, &as_c_eager_tensor->status);
+  int64_t result = TFE_TensorHandleNumElements(as_c_eager_tensor->handle,
+                                               &as_c_eager_tensor->status);
 
-  if (MaybeRaiseExceptionFromTFStatus(&as_c_eager_tensor->status,
-                                      PyExc_ValueError)) {
+  if (tensorflow::MaybeRaiseExceptionFromTFStatus(&as_c_eager_tensor->status,
+                                                  PyExc_ValueError)) {
     // Cleanup status before returning.
-    as_c_eager_tensor->status.status = tensorflow::Status::OK();
+    as_c_eager_tensor->status.status = ::tensorflow::OkStatus();
     return -1;
   }
 
@@ -1026,49 +1142,73 @@ PyObject* TFE_Py_TensorShapeSlice(PyObject* tensors, int slice_dim) {
     return nullptr;
   }
 
+  PyObject* py_context = GetPyEagerContext();
+  if (py_context == nullptr) {
+    PyErr_SetString(PyExc_RuntimeError, tensorflow::strings::StrCat(
+                                            "Cannot create EagerTensor when "
+                                            "EagerContext is not valid")
+                                            .c_str());
+    return nullptr;
+  }
+
+  TFE_Context* ctx = GetContextHandle(py_context);
+
   Py_ssize_t num_tensors = PySequence_Fast_GET_SIZE(tensors);
   PyObject** tensors_array = PySequence_Fast_ITEMS(tensors);
   int64_t num_tensors_int = static_cast<int64_t>(num_tensors);
-  auto tensor = tensorflow::make_safe(TF_AllocateTensor(
-      TF_INT32, &num_tensors_int, /*num_dims=*/1, /*len=*/4 * num_tensors_int));
-  int32_t* data = reinterpret_cast<int32_t*>(TF_TensorData(tensor.get()));
-  auto status = tensorflow::make_safe(TF_NewStatus());
-  for (Py_ssize_t i = 0; i < num_tensors; ++i) {
-    PyObject* tensor_obj = tensors_array[i];
-    if (!EagerTensor_CheckExact(tensor_obj)) {
-      PyErr_SetString(PyExc_TypeError,
-                      tensorflow::strings::StrCat(
-                          "Expected a list of EagerTensors but "
-                          "element ",
-                          i, " has type \"", Py_TYPE(tensor_obj)->tp_name, "\"")
-                          .c_str());
-      return nullptr;
-    }
 
-    EagerTensor* t = reinterpret_cast<EagerTensor*>(tensor_obj);
-    TFE_TensorHandle* handle = t->handle;
-    int num_dims = TFE_TensorHandleNumDims(handle, status.get());
-    if (MaybeRaiseExceptionFromTFStatus(status.get(), PyExc_ValueError)) {
-      return nullptr;
+  auto status = tensorflow::make_safe(TF_NewStatus());
+
+  // Create an empty tensor.
+  auto* tensor = tensorflow::unwrap(ctx)->CreateTensor(
+      tensorflow::DT_INT32, /*dim_sizes=*/{num_tensors_int});
+
+  if (num_tensors_int > 0) {
+    int32_t* data = reinterpret_cast<int32_t*>(tensor->Data());
+
+    // Fill the tensor with dims.
+    for (Py_ssize_t i = 0; i < num_tensors; ++i) {
+      PyObject* tensor_obj = tensors_array[i];
+      if (!EagerTensor_CheckExact(tensor_obj)) {
+        PyErr_SetString(
+            PyExc_TypeError,
+            tensorflow::strings::StrCat("Expected a list of EagerTensors but "
+                                        "element ",
+                                        i, " has type \"",
+                                        Py_TYPE(tensor_obj)->tp_name, "\"")
+                .c_str());
+        return nullptr;
+      }
+
+      EagerTensor* t = reinterpret_cast<EagerTensor*>(tensor_obj);
+      TFE_TensorHandle* handle = t->handle;
+      int num_dims = TFE_TensorHandleNumDims(handle, status.get());
+      if (tensorflow::MaybeRaiseExceptionFromTFStatus(status.get(),
+                                                      PyExc_ValueError)) {
+        return nullptr;
+      }
+      if (slice_dim >= num_dims) {
+        PyErr_SetString(
+            PyExc_IndexError,
+            tensorflow::strings::StrCat("Slice dimension (", slice_dim,
+                                        ") must be smaller than rank of all "
+                                        "tensors, but tensor at index ",
+                                        i, " has rank ", num_dims)
+                .c_str());
+        return nullptr;
+      }
+      int64_t dim = TFE_TensorHandleDim(handle, slice_dim, status.get());
+      if (tensorflow::MaybeRaiseExceptionFromTFStatus(status.get(),
+                                                      PyExc_ValueError)) {
+        return nullptr;
+      }
+      data[i] = dim;
     }
-    if (slice_dim >= num_dims) {
-      PyErr_SetString(
-          PyExc_IndexError,
-          tensorflow::strings::StrCat("Slice dimension (", slice_dim,
-                                      ") must be smaller than rank of all "
-                                      "tensors, but tensor at index ",
-                                      i, " has rank ", num_dims)
-              .c_str());
-      return nullptr;
-    }
-    int64_t dim = TFE_TensorHandleDim(handle, slice_dim, status.get());
-    if (MaybeRaiseExceptionFromTFStatus(status.get(), PyExc_ValueError)) {
-      return nullptr;
-    }
-    data[i] = dim;
   }
 
-  TFE_TensorHandle* handle = TFE_NewTensorHandle(tensor.get(), status.get());
+  TFE_TensorHandle* handle =
+      tensorflow::wrap(tensorflow::unwrap(ctx)->CreateLocalHandle(tensor));
+
   if (!status->status.ok()) {
     PyErr_SetString(
         PyExc_RuntimeError,
@@ -1107,7 +1247,7 @@ PyObject* TFE_Py_TensorShapeOnDevice(PyObject* tensor) {
   int rank = TFE_TensorDebugInfoOnDeviceNumDims(debug_info);
   PyObject* shape = PyTuple_New(rank);
   for (int i = 0; i < rank; ++i) {
-    tensorflow::int64 dim_size = TFE_TensorDebugInfoOnDeviceDim(debug_info, i);
+    int64_t dim_size = TFE_TensorDebugInfoOnDeviceDim(debug_info, i);
     PyTuple_SET_ITEM(shape, i, PyLong_FromLongLong(dim_size));
   }
   TFE_DeleteTensorDebugInfo(debug_info);
