@@ -14,22 +14,20 @@
 # ==============================================================================
 """Tests for convert_to_constants.py."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
 import re
 
 import numpy as np
 
 from google.protobuf import text_format
+from tensorflow.core.framework import attr_value_pb2
 from tensorflow.core.framework import function_pb2
 from tensorflow.core.framework import graph_pb2
 from tensorflow.core.framework import node_def_pb2
 from tensorflow.core.framework import op_def_pb2
 from tensorflow.core.protobuf import config_pb2
 from tensorflow.core.protobuf import meta_graph_pb2
+from tensorflow.core.protobuf import saved_model_pb2
 from tensorflow.python.client import session as session_lib
 from tensorflow.python.eager import def_function
 from tensorflow.python.framework import constant_op
@@ -41,23 +39,32 @@ from tensorflow.python.framework import ops
 from tensorflow.python.framework import tensor_spec
 from tensorflow.python.framework import test_util
 from tensorflow.python.grappler import tf_optimizer
+from tensorflow.python.lib.io import file_io
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import cond
 from tensorflow.python.ops import cond_v2
-from tensorflow.python.ops import control_flow_ops
+from tensorflow.python.ops import control_flow_case
+from tensorflow.python.ops import control_flow_switch_case
 from tensorflow.python.ops import control_flow_v2_toggles
 from tensorflow.python.ops import gen_math_ops
 from tensorflow.python.ops import math_ops
+from tensorflow.python.ops import ref_variable
 from tensorflow.python.ops import rnn
 from tensorflow.python.ops import rnn_cell_impl
 from tensorflow.python.ops import variable_scope
+from tensorflow.python.ops import variable_v1
 from tensorflow.python.ops import variables
+from tensorflow.python.ops import while_loop
 from tensorflow.python.ops import while_v2
 from tensorflow.python.platform import test
+from tensorflow.python.saved_model import constants
+from tensorflow.python.saved_model import loader_impl
 from tensorflow.python.saved_model import simple_save
 from tensorflow.python.saved_model.load import load
 from tensorflow.python.saved_model.save import save
+from tensorflow.python.trackable import autotrackable
 from tensorflow.python.training.saver import export_meta_graph
-from tensorflow.python.training.tracking import tracking
+from tensorflow.python.util import compat
 from tensorflow.python.util import nest
 
 
@@ -142,43 +149,45 @@ class _GraphMerger(object):
     return merged
 
 
+def has_stateful_partitioned_call_op(graph_def):
+  """Determines if a StatefulPartitionedCall op exists in the graph."""
+  for node in graph_def.node:
+    if node.op == "StatefulPartitionedCall":
+      return True
+  return False
+
+
+def get_num_variables(graph_def):
+  """Returns the number of ReadVariableOp in the graph."""
+  return sum(node.op == "ReadVariableOp" for node in graph_def.node)
+
+
 class VariablesToConstantsTest(test.TestCase):
 
-  def _freezeModel(self, model):
-    """Freezes the model.
+  def _freezeModel(self, func):
+    """Freezes the function.
 
     Args:
-      model: Function.
+      func: Function.
 
     Returns:
       root: AutoTrackable object with original ConcreteFunction.
       output_func: frozen ConcreteFunction.
     """
-    root = tracking.AutoTrackable()
-    root.f = model
+    root = autotrackable.AutoTrackable()
+    root.f = func
     input_func = root.f.get_concrete_function()
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(
         input_func, lower_control_flow=False)
     return root, output_func
 
-  def _hasStatefulPartitionedCallOp(self, graph_def):
-    """Determines if a StatefulPartitionedCall op exists in the graph."""
-    for node in graph_def.node:
-      if node.op == "StatefulPartitionedCall":
-        return True
-    return False
-
-  def _getNumVariables(self, graph_def):
-    """Returns the number of ReadVariableOp in the graph."""
-    return sum(node.op == "ReadVariableOp" for node in graph_def.node)
-
   def _testConvertedFunction(self, obj, func, converted_concrete_func,
                              input_data):
     # Ensure the converted graph has no variables and no function calls.
     constant_graph_def = converted_concrete_func.graph.as_graph_def()
-    self.assertEqual(0, self._getNumVariables(constant_graph_def))
-    self.assertFalse(self._hasStatefulPartitionedCallOp(constant_graph_def))
+    self.assertEqual(0, get_num_variables(constant_graph_def))
+    self.assertFalse(has_stateful_partitioned_call_op(constant_graph_def))
 
     # Check that the converted ConcreteFunction produces the same result as the
     # original Function.
@@ -195,7 +204,7 @@ class VariablesToConstantsTest(test.TestCase):
 
     # Save the converted ConcreteFunction as a signature.
     save_dir = os.path.join(self.get_temp_dir(), "frozen_saved_model")
-    root = tracking.AutoTrackable()
+    root = autotrackable.AutoTrackable()
     root.f = converted_concrete_func
     save(root, save_dir, {"mykey": converted_concrete_func})
 
@@ -207,9 +216,9 @@ class VariablesToConstantsTest(test.TestCase):
 
   @test_util.run_v2_only
   def testConstSavedModel(self):
-    """Test a basic model with functions to make sure functions are inlined."""
+    """Test a basic model with constants while saving/loading the SavedModel."""
     input_data = {"x": constant_op.constant(1., shape=[1])}
-    root = tracking.AutoTrackable()
+    root = autotrackable.AutoTrackable()
     root.f = def_function.function(lambda x: 2. * x)
     to_save = root.f.get_concrete_function(input_data["x"])
 
@@ -219,7 +228,7 @@ class VariablesToConstantsTest(test.TestCase):
     input_func = saved_model.signatures["serving_default"]
 
     variable_graph_def = input_func.graph.as_graph_def()
-    self.assertEqual(0, self._getNumVariables(variable_graph_def))
+    self.assertEqual(0, get_num_variables(variable_graph_def))
     self.assertTrue(variable_graph_def.library.function)
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(
@@ -230,14 +239,14 @@ class VariablesToConstantsTest(test.TestCase):
   def testVariableModel(self):
     """Test a basic model with Variables."""
     input_data = {"x": constant_op.constant(1., shape=[1])}
-    root = tracking.AutoTrackable()
+    root = autotrackable.AutoTrackable()
     root.v1 = variables.Variable(3.)
     root.v2 = variables.Variable(2.)
     root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
     input_func = root.f.get_concrete_function(input_data["x"])
 
     variable_graph_def = input_func.graph.as_graph_def()
-    self.assertEqual(2, self._getNumVariables(variable_graph_def))
+    self.assertEqual(2, get_num_variables(variable_graph_def))
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(
         input_func)
@@ -247,14 +256,14 @@ class VariablesToConstantsTest(test.TestCase):
   def testScalarModel(self):
     """Test a basic model with Variables."""
     input_data = {"x": constant_op.constant(1., shape=[])}
-    root = tracking.AutoTrackable()
+    root = autotrackable.AutoTrackable()
     root.v1 = variables.Variable(3.)
     root.v2 = variables.Variable(2.)
     root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
     input_func = root.f.get_concrete_function(input_data["x"])
 
     variable_graph_def = input_func.graph.as_graph_def()
-    self.assertEqual(2, self._getNumVariables(variable_graph_def))
+    self.assertEqual(2, get_num_variables(variable_graph_def))
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(
         input_func)
@@ -264,7 +273,7 @@ class VariablesToConstantsTest(test.TestCase):
   def testVariableSavedModel(self):
     """Test a basic model with Variables with saving/loading the SavedModel."""
     input_data = {"x": constant_op.constant(1., shape=[1])}
-    root = tracking.AutoTrackable()
+    root = autotrackable.AutoTrackable()
     root.v1 = variables.Variable(3.)
     root.v2 = variables.Variable(2.)
     root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
@@ -276,7 +285,7 @@ class VariablesToConstantsTest(test.TestCase):
     input_func = saved_model.signatures["serving_default"]
 
     variable_graph_def = input_func.graph.as_graph_def()
-    self.assertTrue(self._hasStatefulPartitionedCallOp(variable_graph_def))
+    self.assertTrue(has_stateful_partitioned_call_op(variable_graph_def))
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(
         input_func)
@@ -284,9 +293,9 @@ class VariablesToConstantsTest(test.TestCase):
 
   @test_util.run_v2_only
   def testMultiFunctionModel(self):
-    """Test a basic model with Variables."""
+    """Test a basic model with multiple tf.functions."""
 
-    class BasicModel(tracking.AutoTrackable):
+    class BasicModel(autotrackable.AutoTrackable):
 
       def __init__(self):
         self.y = None
@@ -309,7 +318,7 @@ class VariablesToConstantsTest(test.TestCase):
     input_func = root.add.get_concrete_function(input_data["x"])
 
     variable_graph_def = input_func.graph.as_graph_def()
-    self.assertEqual(1, self._getNumVariables(variable_graph_def))
+    self.assertEqual(1, get_num_variables(variable_graph_def))
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(
         input_func)
@@ -320,9 +329,9 @@ class VariablesToConstantsTest(test.TestCase):
     with export_graph.as_default():
       start = array_ops.placeholder(
           shape=[1, 1], dtype=dtypes.float32, name="start")
-      distractor = variables.RefVariable(-1., name="distractor")
-      v = variables.RefVariable(3., name="v")
-      local_variable = variables.VariableV1(
+      distractor = ref_variable.RefVariable(-1., name="distractor")
+      v = ref_variable.RefVariable(3., name="v")
+      local_variable = variable_v1.VariableV1(
           1.,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           trainable=False,
@@ -350,7 +359,7 @@ class VariablesToConstantsTest(test.TestCase):
     fn = imported.signatures["serving_default"]
 
     output_func = convert_to_constants.convert_variables_to_constants_v2(fn)
-    root = tracking.AutoTrackable()
+    root = autotrackable.AutoTrackable()
     self._testConvertedFunction(root, fn, output_func, input_data)
 
   @test_util.run_v2_only
@@ -374,7 +383,7 @@ class VariablesToConstantsTest(test.TestCase):
         tensor_spec.TensorSpec(shape=(), dtype=dtypes.bool)
     ])
     def model(x, b):
-      return control_flow_ops.cond(
+      return cond.cond(
           b, true_fn=lambda: true_fn(x), false_fn=lambda: false_fn(x))
 
     root, output_func = self._freezeModel(model)
@@ -441,7 +450,7 @@ class VariablesToConstantsTest(test.TestCase):
         tensor_spec.TensorSpec(shape=[2, 2], dtype=dtypes.float32)
     ])
     def model(x):
-      return control_flow_ops.while_loop(condition, body, [x])
+      return while_loop.while_loop(condition, body, [x])
 
     root, output_func = self._freezeModel(model)
 
@@ -487,6 +496,7 @@ class VariablesToConstantsTest(test.TestCase):
     self._testConvertedFunction(root, root.f, output_func, input_data)
 
   @test_util.run_v2_only
+  @test_util.disable_tfrt("b/180451239")
   def testSwitchCase(self):
     """Test a switch_case statement."""
     input_data = {
@@ -514,13 +524,392 @@ class VariablesToConstantsTest(test.TestCase):
         tensor_spec.TensorSpec(shape=[10, 3], dtype=dtypes.float32),
     ])
     def model(i, x):
-      return control_flow_ops.switch_case(i, [
-          lambda: branch0(x), lambda: branch1(x), lambda: branch2(x)])
+      return control_flow_switch_case.switch_case(
+          i, [lambda: branch0(x), lambda: branch1(x), lambda: branch2(x)])
 
     root, output_func = self._freezeModel(model)
     self._testConvertedFunction(root, root.f, output_func, input_data)
 
 
+class ConvertVariablesToConstantsV2SessionTest(test.TestCase):
+
+  def _freezeModel(self, func):
+    """Freezes the function.
+
+    Args:
+      func: Function.
+
+    Returns:
+      root: AutoTrackable object with original ConcreteFunction.
+      output_func: frozen ConcreteFunction.
+    """
+    root = autotrackable.AutoTrackable()
+    root.f = func
+    input_func = root.f.get_concrete_function()
+
+    output_func = convert_to_constants.convert_var_to_const_function_in_v1(
+        input_func, lower_control_flow=False)
+    return root, output_func
+
+  def _testConvertedFunction(self, sess, obj, func, converted_concrete_func,
+                             input_data):
+    # Ensure the converted graph has no variables and no function calls.
+    constant_graph_def = converted_concrete_func.graph.as_graph_def()
+    self.assertEqual(0, get_num_variables(constant_graph_def))
+    self.assertFalse(has_stateful_partitioned_call_op(constant_graph_def))
+
+    # Check that the converted ConcreteFunction produces the same result as the
+    # original Function.
+    expected_value = nest.flatten(func(**input_data))
+    actual_value = nest.flatten(converted_concrete_func(**input_data))
+
+    for expected, actual in zip(expected_value, actual_value):
+      np.testing.assert_almost_equal(sess.run(expected), sess.run(actual))
+
+    # Ensure the shape is retained.
+    for tensor in converted_concrete_func.inputs:
+      actual_shape = input_data[tensor.name.split(":")[0]].shape
+      self.assertEqual(tensor.shape, actual_shape)
+
+    # Save the converted ConcreteFunction as a signature.
+    save_dir = os.path.join(self.get_temp_dir(), "frozen_saved_model")
+    root = autotrackable.AutoTrackable()
+    root.f = converted_concrete_func
+    save(root, save_dir, {"mykey": converted_concrete_func})
+
+    # Load it back and make sure it works.
+    loaded_obj = load(save_dir)
+    actual_value = nest.flatten(loaded_obj.signatures["mykey"](**input_data))
+    for expected, actual in zip(expected_value, actual_value):
+      np.testing.assert_almost_equal(sess.run(expected), sess.run(actual))
+
+  def testRaiseErrorInEagerMode(self):
+    """Test the raised exception in Eager mode."""
+    input_data = {"x": constant_op.constant(1., shape=[1])}
+    root = autotrackable.AutoTrackable()
+    root.v1 = variables.Variable(3.)
+    root.v2 = variables.Variable(2.)
+    root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
+    input_func = root.f.get_concrete_function(input_data["x"])
+
+    with self.assertRaisesRegex(RuntimeError,
+                                "must be carried out in a Session"):
+      convert_to_constants.convert_var_to_const_function_in_v1(
+          input_func)
+
+  def testConvertVariables(self):
+    """Test a basic model with Variables."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant(1., shape=[1])}
+        root = autotrackable.AutoTrackable()
+        root.v1 = variables.Variable(3.)
+        root.v2 = variables.Variable(2.)
+        root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
+        input_func = root.f.get_concrete_function(input_data["x"])
+
+        variable_graph_def = input_func.graph.as_graph_def()
+        self.assertEqual(2, get_num_variables(variable_graph_def))
+
+        output_func = convert_to_constants.convert_var_to_const_function_in_v1(
+            input_func)
+
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testConvertVariablesWithAssignments(self):
+    """Test a basic model with Variables and assignment ops."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant(1., shape=[1])}
+        root = autotrackable.AutoTrackable()
+        root.v1 = variables.Variable(3.)
+        root.v2 = variables.Variable(2.)
+        root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
+        input_func = root.f.get_concrete_function(input_data["x"])
+
+        variable_graph_def = input_func.graph.as_graph_def()
+        self.assertEqual(2, get_num_variables(variable_graph_def))
+
+        assign_op_1 = root.v1.assign(1.5)
+        assign_op_2 = root.v2.assign(3.0)
+        assign_op_3 = root.v1.assign(4.0)
+        ops.get_default_graph().add_to_collection(
+            convert_to_constants.VAR_ASSIGN_COLLECTION, assign_op_1)
+        ops.get_default_graph().add_to_collection(
+            convert_to_constants.VAR_ASSIGN_COLLECTION, assign_op_2)
+        ops.get_default_graph().add_to_collection(
+            convert_to_constants.VAR_ASSIGN_COLLECTION, assign_op_3)
+
+        output_func = convert_to_constants.convert_var_to_const_function_in_v1(
+            input_func)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testConstSavedModel(self):
+    """Test a basic model with constants while saving/loading the SavedModel."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant(1., shape=[1])}
+        root = autotrackable.AutoTrackable()
+        root.f = def_function.function(lambda x: 2. * x)
+        to_save = root.f.get_concrete_function(input_data["x"])
+
+        save_dir = os.path.join(self.get_temp_dir(), "saved_model")
+        save(root, save_dir, to_save)
+        saved_model = load(save_dir)
+        input_func = saved_model.signatures["serving_default"]
+
+        variable_graph_def = input_func.graph.as_graph_def()
+        self.assertEqual(0, get_num_variables(variable_graph_def))
+        self.assertTrue(variable_graph_def.library.function)
+
+        output_func = convert_to_constants.convert_var_to_const_function_in_v1(
+            input_func)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testVariableSavedModel(self):
+    """Test a basic model with Variables with saving/loading the SavedModel."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant(1., shape=[1])}
+        root = autotrackable.AutoTrackable()
+        root.v1 = variables.Variable(3.)
+        root.v2 = variables.Variable(2.)
+        root.f = def_function.function(lambda x: root.v1 * root.v2 * x)
+        to_save = root.f.get_concrete_function(input_data["x"])
+        sess.run(variables.global_variables_initializer())
+
+        save_dir = os.path.join(self.get_temp_dir(), "saved_model")
+        save(root, save_dir, to_save)
+        saved_model = load(save_dir)
+        input_func = saved_model.signatures["serving_default"]
+
+        variable_graph_def = input_func.graph.as_graph_def()
+        self.assertTrue(has_stateful_partitioned_call_op(variable_graph_def))
+
+        output_func = convert_to_constants.convert_var_to_const_function_in_v1(
+            input_func)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testMultiFunctionModel(self):
+    """Test a basic model with multiple tf.functions."""
+
+    class BasicModel(autotrackable.AutoTrackable):
+
+      def __init__(self):
+        self.y = None
+        self.z = None
+
+      @def_function.function
+      def add(self, x):
+        if self.y is None:
+          self.y = variables.Variable(2.)
+        return x + self.y
+
+      @def_function.function
+      def sub(self, x):
+        if self.z is None:
+          self.z = variables.Variable(3.)
+        return x - self.z
+
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant(1., shape=[1])}
+        root = BasicModel()
+        input_func = root.add.get_concrete_function(input_data["x"])
+
+        variable_graph_def = input_func.graph.as_graph_def()
+        self.assertEqual(1, get_num_variables(variable_graph_def))
+
+        output_func = convert_to_constants.convert_var_to_const_function_in_v1(
+            input_func)
+        self._testConvertedFunction(sess, root, root.add, output_func,
+                                    input_data)
+
+  def testIf(self):
+    """Test a model with the If op."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {
+            "x": constant_op.constant([1., 2.], shape=[1, 2]),
+            "b": constant_op.constant(True)
+        }
+
+        weights = variables.Variable([[0.1, 0.2], [0.3, 0.4]],
+                                     dtype=dtypes.float32)
+
+        def true_fn(x):
+          return math_ops.matmul(x, weights)
+
+        def false_fn(x):
+          return math_ops.add(x, weights)
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=[1, 2], dtype=dtypes.float32),
+            tensor_spec.TensorSpec(shape=(), dtype=dtypes.bool)
+        ])
+        def model(x, b):
+          return cond.cond(
+              b, true_fn=lambda: true_fn(x), false_fn=lambda: false_fn(x))
+
+        root, output_func = self._freezeModel(model)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testStatelessIf(self):
+    """Test a model with the StatelessIf op."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"b": constant_op.constant(True)}
+
+        x = constant_op.constant([1., 2.], shape=[1, 2], name="x")
+
+        def true_fn():
+          return x
+
+        def false_fn():
+          return x + 2
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=(), dtype=dtypes.bool)
+        ])
+        def model(b):
+          return cond_v2.cond_v2(b, true_fn, false_fn)
+
+        root, output_func = self._freezeModel(model)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testStaticRnn(self):
+    """Test a StaticRnn containing If ops."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {
+            "x":
+                constant_op.constant(
+                    np.array(
+                        np.random.random_sample((3, 10)), dtype=np.float32))
+        }
+
+        cell = rnn_cell_impl.LSTMCell(10)
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=[3, 10], dtype=dtypes.float32)
+        ])
+        def model(x):
+          seq = array_ops.split(x, 3, 0)
+          return rnn.static_rnn(
+              cell, seq, dtype=dtypes.float32, sequence_length=[1])
+
+        root, output_func = self._freezeModel(model)
+
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testWhile(self):
+    """Test a While loop."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant([1., 2., 3., 4.], shape=[2, 2])}
+
+        weights = variables.Variable([[0.1, 0.2], [0.3, 0.4]],
+                                     dtype=dtypes.float32)
+
+        def condition(x):
+          return math_ops.reduce_sum(x) < 100
+
+        def body(x):
+          return math_ops.add(x, weights)
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=[2, 2], dtype=dtypes.float32)
+        ])
+        def model(x):
+          return while_loop.while_loop(condition, body, [x])
+
+        root, output_func = self._freezeModel(model)
+
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testStatelessWhile(self):
+    """Test a StatelessWhile loop."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {"x": constant_op.constant(2.)}
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=(), dtype=dtypes.float32)
+        ])
+        def model(x):
+          return while_v2.while_loop(
+              lambda v: v < 4.,
+              lambda v: v * v, [x],
+              return_same_structure=False,
+              name="while_1")  # x**2
+
+        root, output_func = self._freezeModel(model)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  def testDynamicRnn(self):
+    """Test a DynamicRnn containing While loops."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {
+            "x":
+                constant_op.constant(
+                    np.array(
+                        np.random.random_sample((3, 10, 10)), dtype=np.float32))
+        }
+
+        cell = rnn_cell_impl.LSTMCell(10)
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=[3, 10, 10], dtype=dtypes.float32)
+        ])
+        def model(x):
+          return rnn.dynamic_rnn(cell, x, dtype=dtypes.float32)
+
+        root, output_func = self._freezeModel(model)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+  @test_util.disable_tfrt("b/180451239")
+  def testSwitchCase(self):
+    """Test a switch_case statement."""
+    with ops.Graph().as_default():
+      with session_lib.Session() as sess:
+        input_data = {
+            "i":
+                constant_op.constant(np.random.randint(0, 3, dtype=np.int32)),
+            "x":
+                constant_op.constant(
+                    np.asarray(
+                        np.random.random_sample((10, 3)), dtype=np.float32)),
+        }
+
+        w0 = variables.Variable(
+            np.random.random_sample((3, 4)), dtype=np.float32)
+        w1 = variables.Variable(
+            np.random.random_sample((3, 4)), dtype=np.float32)
+        w2 = variables.Variable(np.random.random_sample((4,)), dtype=np.float32)
+
+        def branch0(x):
+          return math_ops.matmul(x, w0)
+
+        def branch1(x):
+          return math_ops.matmul(x, w1)
+
+        def branch2(x):
+          x = array_ops.pad(x, [[0, 0], [0, 1]])
+          return x + w2
+
+        @def_function.function(input_signature=[
+            tensor_spec.TensorSpec(shape=[], dtype=dtypes.int32),
+            tensor_spec.TensorSpec(shape=[10, 3], dtype=dtypes.float32),
+        ])
+        def model(i, x):
+          return control_flow_switch_case.switch_case(
+              i, [lambda: branch0(x), lambda: branch1(x), lambda: branch2(x)])
+
+        root, output_func = self._freezeModel(model)
+        self._testConvertedFunction(sess, root, root.f, output_func, input_data)
+
+
 class ConvertVariablesToConstantsSessionTest(test.TestCase):
 
   def _assertGraphContains(self, graph, subgraph):
@@ -594,7 +983,7 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
         output = self.evaluate(output_node)
         self.assertNear(2.0, output, 0.00001)
 
-  def test_resource_variable_can_be_written_after_blacklisting(self):
+  def test_resource_variable_can_be_written_after_denylisting(self):
     with ops.Graph().as_default():
       with variable_scope.variable_scope("", use_resource=True):
         variable_node = variable_scope.get_variable(
@@ -614,17 +1003,17 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
 
           # Test variable name black list. This should result in the variable
           # not being a const.  Furthermore, the paths that read from and assign
-          # to the blacklisted variable should continue to be valid.
-          constant_graph_def_with_blacklist = (
+          # to the denylisted variable should continue to be valid.
+          constant_graph_def_with_denylist = (
               convert_to_constants
               .convert_variables_to_constants_from_session_graph(
                   session=sess,
                   graph_def=variable_graph_def,
                   output_node_names=["output_node", initializer_name],
-                  variable_names_blacklist=set(["variable_node"])))
+                  variable_names_denylist=set(["variable_node"])))
 
           variable_node = None
-          for node in constant_graph_def_with_blacklist.node:
+          for node in constant_graph_def_with_denylist.node:
             if node.name == "variable_node":
               variable_node = node
           self.assertIsNotNone(variable_node)
@@ -634,7 +1023,7 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
     # variable is not, and that the graph can be executed and update the
     # variable can be updated with each execution.
     with ops.Graph().as_default():
-      _ = importer.import_graph_def(constant_graph_def_with_blacklist, name="")
+      _ = importer.import_graph_def(constant_graph_def_with_denylist, name="")
       with session_lib.Session() as sess:
         output_node = sess.graph.get_tensor_by_name("output_node:0")
         self.evaluate(sess.graph.get_operation_by_name(initializer_name))
@@ -713,8 +1102,8 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
         y = variable_scope.get_variable("var_y", initializer=2.0)
         f1 = lambda: variable_scope.get_variable("var_f1", initializer=17.0)
         f2 = lambda: variable_scope.get_variable("var_f2", initializer=23.0)
-        cond_node = control_flow_ops.case([(gen_math_ops.less(x, y), f1)],
-                                          default=f2)
+        cond_node = control_flow_case.case([(gen_math_ops.less(x, y), f1)],
+                                           default=f2)
         _ = math_ops.multiply(cond_node, 2.0, name="output_node")
 
         with session_lib.Session() as sess:
@@ -798,7 +1187,7 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
             .convert_variables_to_constants_from_session_graph(
                 sess,
                 variable_graph_def, ["out"],
-                variable_names_blacklist=["y"]))
+                variable_names_denylist=["y"]))
         self._assertGraphContains(
             constant_graph_def, """
             node {
@@ -840,7 +1229,7 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
             .convert_variables_to_constants_from_session_graph(
                 sess,
                 variable_graph_def, ["out"],
-                variable_names_blacklist=["y"]))
+                variable_names_denylist=["y"]))
         self._assertGraphContains(
             constant_graph_def, """
             node {
@@ -911,8 +1300,8 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
         control_flow_v2_toggles.disable_control_flow_v2()
         x = variable_scope.get_variable("x", initializer=1.0)
         y = variable_scope.get_variable("y", initializer=2.0)
-        _ = control_flow_ops.case([(gen_math_ops.less(x, y), lambda: x)],
-                                  default=lambda: y)
+        _ = control_flow_case.case([(gen_math_ops.less(x, y), lambda: x)],
+                                   default=lambda: y)
       with session_lib.Session() as sess:
         sess.run(variables.global_variables_initializer())
         variable_graph_def = sess.graph.as_graph_def()
@@ -962,8 +1351,8 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
         a = variable_scope.get_variable("a", initializer=2.0)
         x = variable_scope.get_variable("x", initializer=1.0)
         y = variable_scope.get_variable("y", initializer=2.0)
-        _ = control_flow_ops.case([(gen_math_ops.less(x, y), lambda: a)],
-                                  default=lambda: y)
+        _ = control_flow_case.case([(gen_math_ops.less(x, y), lambda: a)],
+                                   default=lambda: y)
         control_flow_v2_toggles.disable_control_flow_v2()
       with session_lib.Session() as sess:
         sess.run(variables.global_variables_initializer())
@@ -1024,8 +1413,8 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
         control_flow_v2_toggles.enable_control_flow_v2()
         x = variable_scope.get_variable("x", initializer=1.0)
         y = variable_scope.get_variable("y", initializer=2.0)
-        _ = control_flow_ops.case([(gen_math_ops.less(x, y), lambda: x)],
-                                  default=lambda: y)
+        _ = control_flow_case.case([(gen_math_ops.less(x, y), lambda: x)],
+                                   default=lambda: y)
         control_flow_v2_toggles.disable_control_flow_v2()
       with session_lib.Session() as sess:
         sess.run(variables.global_variables_initializer())
@@ -1072,9 +1461,9 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
         y = variable_scope.get_variable("y", initializer=2.0)
         z = variable_scope.get_variable("z", initializer=3.0)
         # pylint: disable=g-long-lambda
-        _ = control_flow_ops.case(
+        _ = control_flow_case.case(
             [(gen_math_ops.less(x, y), lambda: x)],
-            default=lambda: control_flow_ops.case(
+            default=lambda: control_flow_case.case(
                 [(gen_math_ops.less(z, y), lambda: z)], default=lambda: y))
         # pylint: enable=g-long-lambda
         control_flow_v2_toggles.disable_control_flow_v2()
@@ -1086,7 +1475,7 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
             .convert_variables_to_constants_from_session_graph(
                 sess,
                 variable_graph_def, ["case/cond"],
-                variable_names_blacklist=["y"]))
+                variable_names_denylist=["y"]))
         self._assertGraphContains(
             constant_graph_def, """
             node {name: "x" op: "Const"}
@@ -1180,6 +1569,67 @@ class ConvertVariablesToConstantsSessionTest(test.TestCase):
                 node_def {name: "ReadVariableOp" op: "Identity"
                   input: "readvariableop_z"}}}""")
 
+  def _addNoinlineAttributeToFunction(self, saved_model_dir, func_name):
+    saved_model_proto = loader_impl.parse_saved_model(saved_model_dir)
+    new_saved_model = saved_model_pb2.SavedModel()
+    new_saved_model.CopyFrom(saved_model_proto)
+    new_meta_graph_def = new_saved_model.meta_graphs[0]
+    prefix_len = len("__inference_")
+    for func_def in new_meta_graph_def.graph_def.library.function:
+      func_name_without_prefix = func_def.signature.name[prefix_len:]
+      if func_name_without_prefix.startswith(func_name):
+        func_def.attr["_noinline"].CopyFrom(attr_value_pb2.AttrValue(b=True))
+    old_saved_model_file = os.path.join(saved_model_dir,
+                                        constants.SAVED_MODEL_FILENAME_PB)
+    if os.path.exists(old_saved_model_file):
+      os.remove(old_saved_model_file)
+    path = os.path.join(
+        compat.as_bytes(saved_model_dir),
+        compat.as_bytes(constants.SAVED_MODEL_FILENAME_PB))
+    file_io.write_string_to_file(
+        path, new_saved_model.SerializeToString(deterministic=True))
+
+  @test_util.run_v2_only
+  def testVariableModelWithFunctionAndFunctionInliningDisabled(self):
+    """Test a model with Variables and disable function inlining."""
+
+    class BasicModel:
+
+      def __init__(self):
+        self.v1 = None
+        self.v2 = variables.Variable(2.)
+
+      @def_function.function(input_signature=[
+          tensor_spec.TensorSpec(shape=[1], dtype=dtypes.float32)
+      ])
+      def add_all(self, x):
+        if self.v1 is None:
+          self.v1 = variables.Variable(3.)
+        return x + self.v1 + self.v2
+
+      def run(self, x):
+        y = self.add_all(x)
+        return y
+
+    save_dir = os.path.join(self.get_temp_dir(), "frozen_saved_model")
+    with ops.Graph().as_default():
+      model = BasicModel()
+      a = array_ops.placeholder(dtypes.float32, shape=[1])
+      b = model.run(a)
+      with session_lib.Session() as sess:
+        sess.run(variables.global_variables_initializer())
+        simple_save.simple_save(sess, save_dir, {"myinput": a}, {"myoutput": b})
+
+    # Add _noinline to the SavedModel.
+    self._addNoinlineAttributeToFunction(
+        saved_model_dir=save_dir, func_name="add_all")
+
+    saved_model = load(save_dir)
+    func = saved_model.signatures["serving_default"]
+    frozen_func = convert_to_constants.convert_variables_to_constants_v2(func)
+    constant_graph_def = frozen_func.graph.as_graph_def()
+    self._ensure_no_variables_in_graph(constant_graph_def)
+
 
 if __name__ == "__main__":
   test.main()
