@@ -13,16 +13,20 @@
 # limitations under the License.
 # ==============================================================================
 """Helper classes for tensor shape inference."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
-import six
+import functools
+import operator
+from typing import Optional, Sequence, Type
 
 from tensorflow.core.framework import tensor_shape_pb2
+from tensorflow.core.function import trace_type
+from tensorflow.core.protobuf import struct_pb2
 from tensorflow.python import tf2
 from tensorflow.python.eager import monitoring
+from tensorflow.python.platform import tf_logging as logging
+from tensorflow.python.saved_model import nested_structure_coder
+from tensorflow.python.types import trace
 from tensorflow.python.util.tf_export import tf_export
+from tensorflow.tools.docs import doc_controls
 
 _TENSORSHAPE_V2_OVERRIDE = None
 
@@ -80,6 +84,7 @@ def enable_v2_tensorshape():
   """
   global _TENSORSHAPE_V2_OVERRIDE  # pylint: disable=invalid-name
   _TENSORSHAPE_V2_OVERRIDE = True
+  logging.vlog(1, "Enabling v2 tensorshape")
   _api_usage_gauge.get_cell().set(True)
 
 
@@ -91,6 +96,7 @@ def disable_v2_tensorshape():
   """
   global _TENSORSHAPE_V2_OVERRIDE  # pylint: disable=invalid-name
   _TENSORSHAPE_V2_OVERRIDE = False
+  logging.vlog(1, "Disabling v2 tensorshape")
   _api_usage_gauge.get_cell().set(False)
 
 
@@ -116,7 +122,7 @@ def dimension_value(dimension):
   value = tensor_shape[i]  # Warning: this will return the dim value in V2!
   ```
 
-  Arguments:
+  Args:
     dimension: Either a `Dimension` instance, an integer, or None.
 
   Returns:
@@ -162,7 +168,7 @@ def dimension_at_index(shape, index):
   # instantiated on the fly.
   ```
 
-  Arguments:
+  Args:
     shape: A TensorShape instance.
     index: An integer index.
 
@@ -178,7 +184,17 @@ def dimension_at_index(shape, index):
 
 @tf_export(v1=["Dimension"])
 class Dimension(object):
-  """Represents the value of one dimension in a TensorShape."""
+  """Represents the value of one dimension in a TensorShape.
+
+  @compatibility(TF2)
+  In TF2, members of a `TensorShape` object are integers. The `Dimension` class
+  is not part of TF2's data model.
+
+  Please refer to the [TensorShape section of the migration guide]
+  (https://www.tensorflow.org/guide/migrate/index#tensorshape) on common code
+  patterns adapting Dimension objects to a TF2 syntax.
+  @end_compatibility
+  """
 
   __slots__ = ["_value"]
 
@@ -198,10 +214,10 @@ class Dimension(object):
         # TODO(b/143206389): Remove once we fully migrate to 3.X.
         self._value = int(value.__index__())
       except AttributeError:
-        six.raise_from(
-            TypeError("Dimension value must be integer or None or have "
-                      "an __index__ method, got value '{0!r}' with type '{1!r}'"
-                      .format(value, type(value))), None)
+        raise TypeError(
+            "Dimension value must be integer or None or have "
+            "an __index__ method, got value '{0!r}' with type '{1!r}'".format(
+                value, type(value))) from None
       if self._value < 0:
         raise ValueError("Dimension %d must be >= 0" % self._value)
 
@@ -232,6 +248,10 @@ class Dimension(object):
       return None
     return self._value != other.value
 
+  def __bool__(self):
+    """Equivalent to `bool(self.value)`."""
+    return bool(self._value)
+
   def __int__(self):
     return self._value
 
@@ -249,6 +269,7 @@ class Dimension(object):
     """The value of this dimension, or None if it is unknown."""
     return self._value
 
+  # TODO(b/225058047): Reconsider semantics.
   def is_compatible_with(self, other):
     """Returns true if `other` is compatible with this Dimension.
 
@@ -721,11 +742,16 @@ def as_dimension(value):
 
 
 @tf_export("TensorShape")
-class TensorShape(object):
+class TensorShape(trace.TraceType, trace_type.Serializable):
   """Represents the shape of a `Tensor`.
 
-  A `TensorShape` represents a possibly-partial shape specification for a
-  `Tensor`. It may be one of the following:
+  >>> t = tf.constant([[1,2,3],[4,5,6]])
+  >>> t.shape
+  TensorShape([2, 3])
+
+  `TensorShape` is the *static* shape representation of a Tensor.
+  During eager execution a Tensor always has a fully specified shape but
+  when tracing a `tf.function` it may be one of the following:
 
   * *Fully-known shape:* has a known number of dimensions and a known size
     for each dimension. e.g. `TensorShape([16, 256])`
@@ -734,12 +760,57 @@ class TensorShape(object):
   * *Unknown shape:* has an unknown number of dimensions, and an unknown
     size in all dimensions. e.g. `TensorShape(None)`
 
+  During function tracing `t.shape` will return a `TensorShape` object
+  representing the shape of Tensor as it is known during tracing.
+  This static representation will be partially defined in cases where the
+  exact shape depends on the values within the tensors. To get the
+  *dynamic* representation, please use `tf.shape(t)`
+  which will return Tensor representing the fully defined shape of `t`.
+  This way, you can express logic that manipulates the shapes of tensors by
+  building other tensors that depend on the dynamic shape of `t`.
+
+  Note: `tf.RaggedTensor.shape` also returns a `tf.TensorShape`,
+  the lengths of any ragged dimensions are unknown (`None`).
+
+  For example, this function prints the `TensorShape' (`t.shape`), when you
+  trace the function, and returns a tensor `tf.shape(t)` for given input `t`:
+
+  >>> @tf.function
+  ... def get_dynamic_shape(t):
+  ...   print("tracing...")
+  ...   print(f"static shape is {t.shape}")
+  ...   return tf.shape(t)
+
+  Just calling the function traces it with a fully-specified static shape:
+
+  >>> result = get_dynamic_shape(tf.constant([[1, 1, 1], [0, 0, 0]]))
+  tracing...
+  static shape is (2, 3)
+  >>> result.numpy()
+  array([2, 3], dtype=int32)
+
+  But `tf.function` can also trace the function with a partially specified
+  (or even unspecified) shape:
+
+  >>> cf1 = get_dynamic_shape.get_concrete_function(tf.TensorSpec(
+  ...                                               shape=[None, 2]))
+  tracing...
+  static shape is (None, 2)
+  >>> cf1(tf.constant([[1., 0],[1, 0],[1, 0]])).numpy()
+  array([3, 2], dtype=int32)
+
+  >>> cf2 = get_dynamic_shape.get_concrete_function(tf.TensorSpec(shape=None))
+  tracing...
+  static shape is <unknown>
+  >>> cf2(tf.constant([[[[[1., 0]]]]])).numpy()
+  array([1, 1, 1, 1, 2], dtype=int32)
+
   If a tensor is produced by an operation of type `"Foo"`, its shape
   may be inferred if there is a registered shape function for
   `"Foo"`. See [Shape
-  functions](https://tensorflow.org/extend/adding_an_op#shape_functions_in_c)
+  functions](https://www.tensorflow.org/guide/create_op#shape_functions_in_c)
   for details of shape functions and how to register them. Alternatively,
-  the shape may be set explicitly using `tf.Tensor.set_shape`.
+  you may set the shape explicitly using `tf.Tensor.ensure_shape`.
   """
   __slots__ = ["_dims"]
 
@@ -753,39 +824,38 @@ class TensorShape(object):
       TypeError: If dims cannot be converted to a list of dimensions.
     """
     if isinstance(dims, (tuple, list)):  # Most common case.
-      self._dims = [Dimension(d) for d in dims]
+      self._dims = tuple(as_dimension(d).value for d in dims)
     elif dims is None:
       self._dims = None
     elif isinstance(dims, tensor_shape_pb2.TensorShapeProto):
       if dims.unknown_rank:
         self._dims = None
       else:
-        self._dims = [
+        self._dims = tuple(
             # Protos store variable-size dimensions as -1
-            as_dimension(dim.size if dim.size != -1 else None)
+            dim.size if dim.size != -1 else None
             for dim in dims.dim
-        ]
+            )
     elif isinstance(dims, TensorShape):
-      self._dims = dims.dims
+      self._dims = dims._dims
     else:
       try:
         dims_iter = iter(dims)
       except TypeError:
         # Treat as a singleton dimension
-        self._dims = [as_dimension(dims)]
+        self._dims = (as_dimension(dims).value,)
       else:
         self._dims = []
         for d in dims_iter:
           try:
-            self._dims.append(as_dimension(d))
+            self._dims.append(as_dimension(d).value)
           except TypeError as e:
-            six.raise_from(
-                TypeError(
-                    "Failed to convert '{0!r}' to a shape: '{1!r}'"
-                    "could not be converted to a dimension. A shape should "
-                    "either be single dimension (e.g. 10), or an iterable of "
-                    "dimensions (e.g. [1, 10, None])."
-                    .format(dims, d)), e)
+            raise TypeError(
+                "Failed to convert '{0!r}' to a shape: '{1!r}'"
+                "could not be converted to a dimension. A shape should "
+                "either be single dimension (e.g. 10), or an iterable of "
+                "dimensions (e.g. [1, 10, None]).".format(dims, d)) from e
+        self._dims = tuple(self._dims)
 
   @property
   def _v2_behavior(self):
@@ -796,25 +866,25 @@ class TensorShape(object):
   def __repr__(self):
     if self._v2_behavior:
       if self._dims is not None:
-        return "TensorShape(%r)" % [dim.value for dim in self._dims]
+        return f"TensorShape({list(self._dims)})"
       else:
         return "TensorShape(None)"
     else:
-      return "TensorShape(%r)" % self._dims
+      return f"TensorShape({self.dims})"
 
   def __str__(self):
     if self.rank is None:
       return "<unknown>"
     elif self.rank == 1:
       if self._v2_behavior:
-        return "(%s,)" % self._dims[0].value
-      else:
         return "(%s,)" % self._dims[0]
+      else:
+        return "(%s,)" % self.dims[0]
     else:
       if self._v2_behavior:
-        return "(%s)" % ", ".join(str(d.value) for d in self._dims)
-      else:
         return "(%s)" % ", ".join(str(d) for d in self._dims)
+      else:
+        return "(%s)" % ", ".join(str(d) for d in self.dims)
 
   @property
   def rank(self):
@@ -833,7 +903,9 @@ class TensorShape(object):
       A list containing `tf.compat.v1.Dimension`s, or None if the shape is
       unspecified.
     """
-    return self._dims
+    if self._dims is None:
+      return None
+    return [as_dimension(d) for d in self._dims]
 
   @property
   def ndims(self):
@@ -859,9 +931,9 @@ class TensorShape(object):
       raise ValueError("Cannot iterate over a shape with unknown rank.")
     else:
       if self._v2_behavior:
-        return iter(d.value for d in self._dims)
-      else:
         return iter(d for d in self._dims)
+      else:
+        return iter(d for d in self.dims)
 
   def __getitem__(self, key):
     """Returns the value of a dimension or a shape, depending on the key.
@@ -884,9 +956,9 @@ class TensorShape(object):
         return TensorShape(self._dims[key])
       else:
         if self._v2_behavior:
-          return self._dims[key].value
-        else:
           return self._dims[key]
+        else:
+          return self.dims[key]
     else:
       if isinstance(key, slice):
         start = key.start if key.start is not None else 0
@@ -916,18 +988,35 @@ class TensorShape(object):
   def num_elements(self):
     """Returns the total number of elements, or none for incomplete shapes."""
     if self.is_fully_defined():
-      size = 1
-      for dim in self._dims:
-        size *= dim.value
-      return size
+      return functools.reduce(operator.mul, self.as_list(), 1)
     else:
       return None
 
   def merge_with(self, other):
     """Returns a `TensorShape` combining the information in `self` and `other`.
 
-    The dimensions in `self` and `other` are merged elementwise,
-    according to the rules defined for `Dimension.merge_with()`.
+    The dimensions in `self` and `other` are merged element-wise,
+    according to the rules below:
+
+    ```python
+    Dimension(n).merge_with(Dimension(None)) == Dimension(n)
+    Dimension(None).merge_with(Dimension(n)) == Dimension(n)
+    Dimension(None).merge_with(Dimension(None)) == Dimension(None)
+    # raises ValueError for n != m
+    Dimension(n).merge_with(Dimension(m))
+    ```
+    >> ts = tf.TensorShape([1,2])
+    >> ot1 = tf.TensorShape([1,2])
+    >> ts.merge_with(ot).as_list()
+    [1,2]
+
+    >> ot2 = tf.TensorShape([1,None])
+    >> ts.merge_with(ot2).as_list()
+    [1,2]
+
+    >> ot3 = tf.TensorShape([None, None])
+    >> ot3.merge_with(ot2).as_list()
+    [1, None]
 
     Args:
       other: Another `TensorShape`.
@@ -940,21 +1029,22 @@ class TensorShape(object):
       ValueError: If `self` and `other` are not compatible.
     """
     other = as_shape(other)
-    if self._dims is None:
+    if self.dims is None:
       return other
+    if other.dims is None:
+      return self
     else:
       try:
         self.assert_same_rank(other)
-        new_dims = []
-        for i, dim in enumerate(self._dims):
-          new_dims.append(dim.merge_with(other[i]))
+        new_dims = [
+            dim.merge_with(other_dim)
+            for dim, other_dim in zip(self.dims, other.dims)
+        ]
         return TensorShape(new_dims)
       except ValueError:
         raise ValueError("Shapes %s and %s are not compatible" % (self, other))
 
   def __add__(self, other):
-    if not isinstance(other, TensorShape):
-      other = TensorShape(other)
     return self.concatenate(other)
 
   def __radd__(self, other):
@@ -980,10 +1070,10 @@ class TensorShape(object):
     # TODO(mrry): Handle the case where we concatenate a known shape with a
     # completely unknown shape, so that we can use the partial information.
     other = as_shape(other)
-    if self._dims is None or other.dims is None:
+    if self.dims is None or other.dims is None:
       return unknown_shape()
     else:
-      return TensorShape(self._dims + other.dims)
+      return TensorShape(self.dims + other.dims)
 
   def assert_same_rank(self, other):
     """Raises an exception if `self` and `other` do not have compatible ranks.
@@ -1071,6 +1161,143 @@ class TensorShape(object):
     else:
       return self
 
+  def is_subtype_of(self, other: trace.TraceType) -> bool:
+    """Returns True iff `self` is subtype of `other`.
+
+    Shape A is a subtype of shape B if shape B can successfully represent it:
+
+    * A `TensorShape` of any rank is a subtype of `TensorShape(None)`.
+
+    *  TensorShapes of equal ranks are covariant, i.e.
+      `TensorShape([A1, A2, ..])` is a subtype of
+      `TensorShape([B1, B2, ..])` iff An is a subtype of Bn.
+
+      An is subtype of Bn iff An == Bn or Bn is None.
+
+    * TensorShapes of different defined ranks have no subtyping relation.
+
+    The subtyping relation is reflexive and transitive, but not symmetric.
+
+    Some examples:
+    * `TensorShape([32, 784])` is a subtype of `TensorShape(None)`, and
+      `TensorShape([4, 4])` is also a subtype of `TensorShape(None)` but
+      `TensorShape([32, 784])` and `TensorShape([4, 4])` are not subtypes of
+      each other.
+
+    * All two-dimensional shapes are subtypes of `TensorShape([None, None])`,
+      such as `TensorShape([32, 784])`. There is no subtype relationship with,
+      for example, `TensorShape([None])` or `TensorShape([None, None, None])`.
+
+    * `TensorShape([32, None])` is also a subtype of `TensorShape([None, None])`
+      and `TensorShape(None)`. It is not a subtype of, for example,
+      `TensorShape([32])`, `TensorShape([32, None, 1])`,
+      `TensorShape([64, None])` or `TensorShape([None, 32])`.
+
+    * `TensorShape([32, 784])` is a subtype of itself, and also
+      `TensorShape([32, None])`, `TensorShape([None, 784])`,
+      `TensorShape([None, None])` and `TensorShape(None)`.
+      It has no subtype relation with, for example, `TensorShape([32, 1, 784])`
+      or `TensorShape([None])`.
+
+    Args:
+      other: Another `TensorShape`.
+
+    Returns:
+      True iff `self` is subtype of `other`.
+
+    """
+    if not isinstance(other, TensorShape):
+      return False
+
+    # All Tensors are subtypes of a Tensor with no shape.
+    if other.rank is None:
+      return True
+
+    # Tensor with a defined shape can only be subtype of another with a defined
+    # shape if they have the same number of dimensions.
+    if self.rank != other.rank:
+      return False
+
+    # A Tensor is a subtype if each corresponding dimension is a subtype.
+    return all(o is None or s == o for s, o in zip(self._dims, other._dims))  # pylint: disable=protected-access
+
+  def most_specific_common_supertype(
+      self, others: Sequence[trace.TraceType]) -> Optional["TensorShape"]:
+    """Returns the most specific supertype `TensorShape` of self and others.
+
+    * `TensorShape([None, 1])` is the most specific `TensorShape` supertyping
+      both `TensorShape([2, 1])` and `TensorShape([5, 1])`. Note that
+      `TensorShape(None)` is also a supertype but it is not "most specific".
+
+    * `TensorShape([1, 2, 3])` is the most specific `TensorShape` supertyping
+      both `TensorShape([1, 2, 3])` and `TensorShape([1, 2, 3]`). There are
+      other less specific TensorShapes that supertype above mentioned
+      TensorShapes, e.g. `TensorShape([1, 2, None])`, `TensorShape(None)`.
+
+     * `TensorShape([None, None])` is the most specific `TensorShape`
+       supertyping both `TensorShape([2, None])` and `TensorShape([None, 3])`.
+       As always, `TensorShape(None)` is also a supertype but not the most
+       specific one.
+
+     * `TensorShape(None`) is the only `TensorShape` supertyping both
+       `TensorShape([1, 2, 3])` and `TensorShape([1, 2])`. In general, any two
+       shapes that have different ranks will only have `TensorShape(None)`
+       as a common supertype.
+
+     * `TensorShape(None)` is the only `TensorShape` supertyping both
+       `TensorShape([1, 2, 3])` and `TensorShape(None)`. In general, the common
+       supertype of any shape with `TensorShape(None)` is `TensorShape(None)`.
+
+    Args:
+      others: Sequence of `TensorShape`.
+
+    Returns:
+      A `TensorShape` which is the most specific supertype shape of `self`
+      and `others`. None if it does not exist.
+    """
+    if any(not isinstance(other, TensorShape) for other in others):
+      return None
+
+    # A Rankless TensorShape is already a global supertype so we return another
+    # instance of it.
+    if self.rank is None:
+      return unknown_shape()
+
+    # A Rankless TensorShape is the most specific supertype for shapes whose
+    # ranks do not match.
+    if any(other.dims is None or self.rank != other.rank for other in others):
+      return unknown_shape()
+
+    # Retain the integer dimension if it is the same across all others, else
+    # use an undefined dimension.
+    dims = [
+        dim if all(dim == other._dims[i]
+                   for other in others) else None
+        for i, dim in enumerate(self._dims)
+    ]
+    return TensorShape(dims)
+
+  @doc_controls.do_not_doc_inheritable
+  def placeholder_value(self, placeholder_context):
+    raise NotImplementedError("A graph placeholder is not currently supported"
+                              "for an object of type: TensorShape.")
+
+  @classmethod
+  def experimental_type_proto(cls) -> Type[tensor_shape_pb2.TensorShapeProto]:
+    """Returns the type of proto associated with TensorShape serialization."""
+    return tensor_shape_pb2.TensorShapeProto
+
+  @classmethod
+  def experimental_from_proto(
+      cls, proto: tensor_shape_pb2.TensorShapeProto) -> "TensorShape":
+    """Returns a TensorShape instance based on the serialized proto."""
+    return TensorShape(proto)
+
+  def experimental_as_proto(self) -> tensor_shape_pb2.TensorShapeProto:
+    """Returns a proto representation of the TensorShape instance."""
+    return self.as_proto()
+
+  # TODO(b/216206374): Consider deprecation at TraceType release.
   def is_compatible_with(self, other):
     """Returns True iff `self` is compatible with `other`.
 
@@ -1110,10 +1337,10 @@ class TensorShape(object):
 
     """
     other = as_shape(other)
-    if self._dims is not None and other.dims is not None:
+    if self.dims is not None and other.dims is not None:
       if self.rank != other.rank:
         return False
-      for x_dim, y_dim in zip(self._dims, other.dims):
+      for x_dim, y_dim in zip(self.dims, other.dims):
         if not x_dim.is_compatible_with(y_dim):
           return False
     return True
@@ -1154,19 +1381,19 @@ class TensorShape(object):
     """
 
     other = as_shape(other)
-    if self._dims is None or other.dims is None or self.rank != other.rank:
+    if self.dims is None or other.dims is None or self.rank != other.rank:
       return unknown_shape()
 
-    dims = [(Dimension(None))] * self.rank
-    for i, (d1, d2) in enumerate(zip(self._dims, other.dims)):
-      if d1 is not None and d2 is not None and d1 == d2:
-        dims[i] = d1
+    dims = [
+        d1 if d1 is not None and d2 is not None and d1 == d2 else None
+        for d1, d2 in zip(self.dims, other.dims)
+    ]
     return TensorShape(dims)
 
   def is_fully_defined(self):
     """Returns True iff `self` is fully defined in every dimension."""
     return (self._dims is not None and
-            all(dim.value is not None for dim in self._dims))
+            all(dim is not None for dim in self._dims))
 
   def assert_is_fully_defined(self):
     """Raises an exception if `self` is not fully defined in every dimension.
@@ -1188,7 +1415,7 @@ class TensorShape(object):
     """
     if self._dims is None:
       raise ValueError("as_list() is not defined on an unknown TensorShape.")
-    return [dim.value for dim in self._dims]
+    return list(self._dims)
 
   def as_proto(self):
     """Returns this shape as a `TensorShapeProto`."""
@@ -1197,35 +1424,99 @@ class TensorShape(object):
     else:
       return tensor_shape_pb2.TensorShapeProto(dim=[
           tensor_shape_pb2.TensorShapeProto.Dim(
-              size=-1 if d.value is None else d.value) for d in self._dims
+              size=-1 if d is None else d) for d in self._dims
       ])
 
   def __eq__(self, other):
-    """Returns True if `self` is equivalent to `other`."""
-    try:
-      other = as_shape(other)
-    except TypeError:
-      return NotImplemented
-    return self._dims == other.dims
+    """Returns True if `self` is equivalent to `other`.
+
+    It first tries to convert `other` to `TensorShape`. `TypeError` is thrown
+    when the conversion fails. Otherwise, it compares each element in the
+    TensorShape dimensions.
+
+    * Two *Fully known* shapes, return True iff each element is equal.
+    >>> t_a = tf.TensorShape([1,2])
+    >>> a = [1, 2]
+    >>> t_b = tf.TensorShape([1,2])
+    >>> t_c = tf.TensorShape([1,2,3])
+    >>> t_a.__eq__(a)
+    True
+    >>> t_a.__eq__(t_b)
+    True
+    >>> t_a.__eq__(t_c)
+    False
+
+    * Two *Partially-known* shapes, return True iff each element is equal.
+    >>> p_a = tf.TensorShape([1,None])
+    >>> p_b = tf.TensorShape([1,None])
+    >>> p_c = tf.TensorShape([2,None])
+    >>> p_a.__eq__(p_b)
+    True
+    >>> t_a.__eq__(p_a)
+    False
+    >>> p_a.__eq__(p_c)
+    False
+
+    * Two *Unknown shape*, return True.
+    >>> unk_a = tf.TensorShape(None)
+    >>> unk_b = tf.TensorShape(None)
+    >>> unk_a.__eq__(unk_b)
+    True
+    >>> unk_a.__eq__(t_a)
+    False
+
+    Args:
+      other: A `TensorShape` or type that can be converted to `TensorShape`.
+
+    Returns:
+      True if the dimensions are all equal.
+
+    Raises:
+      TypeError if `other` can not be converted to `TensorShape`.
+    """
 
-  def __ne__(self, other):
-    """Returns True if `self` is known to be different from `other`."""
     try:
       other = as_shape(other)
     except TypeError:
       return NotImplemented
-    if self.rank is None or other.rank is None:
-      raise ValueError("The inequality of unknown TensorShapes is undefined.")
-    if self.rank != other.rank:
-      return True
-    return self._dims != other.dims
+
+    return self._dims == other._dims
+
+  def __hash__(self):
+    return hash(self._dims)
 
   def __reduce__(self):
-    return TensorShape, (self._dims,)
+    return TensorShape, (self.dims,)
 
   def __concat__(self, other):
     return self.concatenate(other)
 
+trace_type.register_serializable(TensorShape)
+
+
+class _TensorShapeCodec:
+  """Codec for `TensorShape`."""
+
+  def can_encode(self, pyobj):
+    return isinstance(pyobj, TensorShape)
+
+  def do_encode(self, tensor_shape_value, encode_fn):
+    del encode_fn
+    encoded_tensor_shape = struct_pb2.StructuredValue()
+    encoded_tensor_shape.tensor_shape_value.CopyFrom(
+        tensor_shape_value.as_proto())
+    return encoded_tensor_shape
+
+  def can_decode(self, value):
+    return value.HasField("tensor_shape_value")
+
+  def do_decode(self, value, decode_fn):
+    del decode_fn
+    return TensorShape(value.tensor_shape_value)
+
+
+nested_structure_coder.register_codec(_TensorShapeCodec())
+
 
 def as_shape(shape):
   """Converts the given object to a TensorShape."""
