@@ -14,11 +14,8 @@
 # ==============================================================================
 """Tests for Grappler AutoMixedPrecision."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
+import re
 
 from absl.testing import parameterized
 import numpy as np
@@ -37,7 +34,6 @@ from tensorflow.python.framework import random_seed
 from tensorflow.python.framework import test_util
 from tensorflow.python.layers import layers
 from tensorflow.python.ops import array_ops
-from tensorflow.python.ops import control_flow_ops
 from tensorflow.python.ops import init_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import nn
@@ -45,10 +41,13 @@ from tensorflow.python.ops import nn_impl
 from tensorflow.python.ops import random_ops
 from tensorflow.python.ops import tensor_array_ops
 from tensorflow.python.ops import variables
+from tensorflow.python.ops import while_loop
 from tensorflow.python.ops.losses import losses
+from tensorflow.python.platform import sysconfig as sysconfig_lib
 from tensorflow.python.platform import test
 from tensorflow.python.training import adam
 from tensorflow.python.training import gradient_descent
+from tensorflow.python.util import _pywrap_utils
 
 
 def _input(shape):
@@ -59,8 +58,8 @@ def _input(shape):
 def _weight(shape):
   """Generates a weight of a given shape."""
   # Note that the lambda is needed to allow construction inside loops.
-  return variables.Variable(
-      lambda: init_ops.glorot_uniform_initializer(seed=0)(shape))
+  return variables.Variable(lambda: init_ops.glorot_uniform_initializer(seed=0)
+                            (shape))
 
 
 def _bias(shape):
@@ -138,12 +137,17 @@ def _conv_pool(x):
   return h_pool2
 
 
+def _depthwise_conv2d(x, w):
+  """Returns a 2d depthwise convolution layer with full stride."""
+  return nn.depthwise_conv2d(x, w, strides=[1, 1, 1, 1], padding='SAME')
+
+
 def _simple_loop(x, functor):
   """Simple loop whose body is provided by the functor."""
   init = (constant_op.constant(0), x)
   c = lambda i, j: i < 4
   b = lambda i, j: (i + 1, functor(j))
-  ij = control_flow_ops.while_loop(c, b, init)
+  ij = while_loop.while_loop(c, b, init)
   return ij
 
 
@@ -152,7 +156,7 @@ def _loop_vars_intertwined(x0, y0, functor_x, functor_y):
   c = lambda i, j, x, y: j < 4
   b = lambda i, j, x, y: (j + 1, i + 1, functor_y(y), functor_x(x))
   init = (constant_op.constant(0), constant_op.constant(0), x0, y0)
-  ijzw = control_flow_ops.while_loop(c, b, init)
+  ijzw = while_loop.while_loop(c, b, init)
   return ijzw
 
 
@@ -194,21 +198,21 @@ def _recurrent_lstm(c, h):
     ta_x = ta_x.write(
         i, constant_op.constant(0.1, shape=[8, 4], dtype=dtypes.float32))
   init = (constant_op.constant(0), c, h, ta_x)
-  r = control_flow_ops.while_loop(cond, body, init)
+  r = while_loop.while_loop(cond, body, init)
   return r
 
 
 def _make_node_with_color(color, input_tensor, name=None):
   """Returns a node representative of the specified list type."""
   color = color.lower()
-  if color == 'w':  # White node
+  if color == 'w':  # Allow node
     weights = _weight(input_tensor.get_shape().as_list())
     return math_ops.matmul(input_tensor, weights, name=name)
-  if color == 'g':  # Gray node
+  if color == 'g':  # Infer node
     return math_ops.add(input_tensor, 0.1, name=name)
   if color == 'c':  # Clear node
     return nn.relu(input_tensor, name=name)
-  if color == 'b':  # Black node
+  if color == 'b':  # Deny node
     return math_ops.pow(math_ops.pow(input_tensor, 2.), 0.5, name=name)
   raise ValueError('Invalid node color: ' + str(color))
 
@@ -242,7 +246,7 @@ def _get_config(auto_mixed_precision_mode):
   if auto_mixed_precision_mode == 'cuda':
     rewrite_config.auto_mixed_precision = rewriter_config_pb2.RewriterConfig.ON
   elif auto_mixed_precision_mode == 'mkl':
-    rewrite_config.auto_mixed_precision_mkl = (
+    rewrite_config.auto_mixed_precision_onednn_bfloat16 = (
         rewriter_config_pb2.RewriterConfig.ON)
   else:
     assert auto_mixed_precision_mode is None
@@ -254,16 +258,24 @@ def _get_config(auto_mixed_precision_mode):
   return config
 
 
+def _get_device(auto_mixed_precision_mode):
+  """Returns the device to run on. If mode is mkl, run on CPU"""
+  if auto_mixed_precision_mode == 'mkl':
+    return '/cpu:0'
+  else:
+    return ''
+
+
 def _is_cast_to_fp16(node_name):
-  return node_name.endswith('-CastToFp16-AutoMixedPrecision')
+  return re.match('.*-CastToFp16-[0-9]-AutoMixedPrecision$', node_name)
 
 
 def _is_cast_to_bf16(node_name):
-  return node_name.endswith('-CastToBf16-AutoMixedPrecision')
+  return re.match('.*-CastToBf16-[0-9]-AutoMixedPrecision$', node_name)
 
 
 def _is_cast_to_fp32(node_name):
-  return node_name.endswith('-CastToFp32-AutoMixedPrecision')
+  return re.match('.*-CastToFp32-[0-9]-AutoMixedPrecision$', node_name)
 
 
 def _count_casts(mode, nodes):
@@ -365,14 +377,18 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
       self.skipTest('No GPU is available')
     if mode == 'mkl' and not test_util.IsMklEnabled():
       self.skipTest('MKL is not enabled')
+    # Test will fail on machines without AVX512f, e.g., Broadwell
+    isAVX512f = _pywrap_utils.IsBF16SupportedByOneDNNOnThisCPU()
+    if mode == 'mkl' and not isAVX512f:
+      self.skipTest('Skipping test due to non-AVX512f machine')
 
   def _run_simple_loop_test(self, mode, inp, body, out):
     """Runs a test of a simple loop.
 
     The loop has different node colors in different sections of the graph. The
     arguments must be strings where each character represents the color of a
-    node in that section of the graph: w = white, g = gray, c = clear,
-    b = black. CAPITALIZED characters indicate that the node is expected to be
+    node in that section of the graph: w = allow, g = infer, c = clear,
+    b = deny. CAPITALIZED characters indicate that the node is expected to be
     changed to DT_HALF during graph optimization.
 
     inp -> loop [ body ] -> out.
@@ -387,19 +403,19 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
         output nodes.
     """
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    expected_types = []
-    for section in [inp, body, out]:
-      section_expected_types = []
-      for color in section:
-        if color.isupper():
-          expected_type = self._lower_precision_dtype(mode).as_datatype_enum
-        else:
-          expected_type = types_pb2.DT_FLOAT
-        section_expected_types.append(expected_type)
-      expected_types.append(section_expected_types)
-
-    a = _build_simple_loop_graph(inp, body, out)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      expected_types = []
+      for section in [inp, body, out]:
+        section_expected_types = []
+        for color in section:
+          if color.isupper():
+            expected_type = self._lower_precision_dtype(mode).as_datatype_enum
+          else:
+            expected_type = types_pb2.DT_FLOAT
+          section_expected_types.append(expected_type)
+        expected_types.append(section_expected_types)
+      a = _build_simple_loop_graph(inp, body, out)
     output_val_ref, output_val, cost_graph = self._run(mode, a)
     node_map = _build_node_map(cost_graph.node)
 
@@ -426,10 +442,11 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
   def test_conv_bn(self, mode):
     """Test graph with convolution followed by batch norm."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([2, 8, 8, 1])
-    x = _conv_bn(x)
-    output = _conv_bn(x)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 1])
+      x = _conv_bn(x)
+      output = _conv_bn(x)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -460,10 +477,11 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     if mode == 'cuda':
       # TODO(reedwm): enable these tests when cuDNN is upgraded to >= 7.6.2.
       self.skipTest('Test case should be skipped when cuDNN < 7.6.2')
-    random_seed.set_random_seed(0)
-    x = _input([2, 8, 8, 8, 1])
-    x = _conv3d_bn(x)
-    output = _conv3d_bn(x)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 8, 1])
+      x = _conv3d_bn(x)
+      output = _conv3d_bn(x)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -485,14 +503,15 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     if mode == 'cuda':
       # TODO(reedwm): enable these tests when cuDNN is upgraded to >= 7.6.2.
       self.skipTest('Test case should be skipped when cuDNN < 7.6.2')
-    random_seed.set_random_seed(0)
-    x = _input([2, 8, 8, 8, 1])
-    f = _weight([3, 3, 3, 1, 6])
-    y = _conv3d(x, f)
-    y = array_ops.identity(y)
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(y, [x, f])
-    output = (y, g)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 8, 1])
+      f = _weight([3, 3, 3, 1, 6])
+      y = _conv3d(x, f)
+      y = array_ops.identity(y)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x, f])
+      output = (y, g)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -506,24 +525,23 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     tol = 5e-2 if mode == 'mkl' else 1e-3
     self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
 
-  # TODO(reedwm): Fix and enable this test with MKL. Currently this crashes with
-  # MKL
-  @parameterized.parameters(['cuda'])
+  @parameterized.parameters(['cuda', 'mkl'])
   @test_util.run_deprecated_v1
   @test_util.disable_xla('This test does not pass with XLA')
   def test_conv_bn_dropout(self, mode):
     """Test dropout precision of convolution batch norm graph."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([2, 8, 8, 1])
-    y = _conv_bn(x)
-    y = nn.dropout(y, rate=0.5)
-    y = math_ops.add(y, 1, name='addition')
-    y = _conv_bn(y)
-    y = array_ops.identity(y)
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(y, [x])
-    output = (y, g)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 1])
+      y = _conv_bn(x)
+      y = nn.dropout(y, rate=0.5)
+      y = math_ops.add(y, 1, name='addition')
+      y = _conv_bn(y)
+      y = array_ops.identity(y)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x])
+      output = (y, g)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -539,6 +557,7 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     # The default tolerance (1e-3) results in a tiny fraction (<1%) of
     # miscompares on ROCm platform, and hence the tolerance bump
     tol = 2e-3 if test.is_built_with_rocm else 1e-3
+    tol = 5e-2 if mode == 'mkl' else tol
     self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
 
   # TODO(reedwm): Fix and enable this test with MKL. Currently this crashes with
@@ -549,9 +568,10 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
   def test_conv_pool(self, mode):
     """Test graph with convolution followed by pooling."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([2, 8, 8, 1])
-    output = _conv_pool(x)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 1])
+      output = _conv_pool(x)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -566,18 +586,57 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     tol = 5e-3 if mode == 'mkl' else 1e-3
     self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
 
+  # TODO(benbarsdell): This test has not been tried with MKL.
+  @parameterized.parameters(['cuda'])
+  @test_util.run_deprecated_v1
+  @test_util.disable_xla('This test does not pass with XLA')
+  def test_depthwise_conv2d(self, mode):
+    """Test grad ops with depthwise convolution2d graph."""
+    self._maybe_skip(mode)
+    cudnn_version_str = sysconfig_lib.get_build_info().get(
+        'cudnn_version', '0.0')
+    cudnn_version = tuple([int(x) for x in cudnn_version_str.split('.')])
+    if cudnn_version < (8,):
+      # Depthwise conv2d ops are only enabled in auto_mixed_precision as of
+      # cuDNN v8.
+      self.skipTest('cuDNN version >= 8 required')
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 1])
+      f = _weight([3, 3, 1, 4])
+      y = _depthwise_conv2d(x, f)
+      y = array_ops.identity(y)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x, f])
+      output = (y, g)
+
+    output_val_ref, output_val, cost_graph = self._run(mode, output)
+    node_map = _build_node_map(cost_graph.node)
+    self._assert_output_f16(mode, node_map, 'depthwise')
+    self._assert_output_f16(
+        mode, node_map,
+        'gradients/depthwise_grad/DepthwiseConv2dNativeBackpropInput')
+    self._assert_output_f16(
+        mode, node_map,
+        'gradients/depthwise_grad/DepthwiseConv2dNativeBackpropFilter')
+
+    output_val_ref, output_val, cost_graph = self._run(mode, output)
+    tol = 2e-3
+    self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
+
   @parameterized.parameters(['cuda', 'mkl'])
   @test_util.run_v1_only('b/138749235')
   @test_util.disable_xla('This test does not pass with XLA')
   def test_simple_loop(self, mode):
     """Test graph with while loop."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([8, 8])
-    y = _simple_loop(x, _matmul_act)[1]
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(y, [x])
-    output = (y, g)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([8, 8])
+      y = _simple_loop(x, _matmul_act)[1]
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x])
+      output = (y, g)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -593,13 +652,14 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
   def test_loop_with_vars_intertwined(self, mode):
     """Test graph with intertwined while loops."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([8, 8])
-    _, _, k, l = _loop_vars_intertwined(
-        array_ops.ones(array_ops.shape(x)), x, _matmul_act, _matmul_act)
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(k, [x])
-    output = (k, l, g)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([8, 8])
+      _, _, k, l = _loop_vars_intertwined(
+          array_ops.ones(array_ops.shape(x)), x, _matmul_act, _matmul_act)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(k, [x])
+      output = (k, l, g)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -617,17 +677,18 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
   def test_multi_paths(self, mode):
     """Test graph with multiple paths."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([2, 8, 8, 3])
-    x1, x2, x3 = array_ops.split(x, num_or_size_splits=3, axis=3)
-    y1 = _conv_pool(x1)
-    y2 = _conv_pool(x2)
-    y3 = _conv_pool(x3)
-    y = array_ops.concat([y1, y2, y3], axis=3)
-    y = array_ops.identity(y)
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(y, [x])
-    output = (y, g)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([2, 8, 8, 3])
+      x1, x2, x3 = array_ops.split(x, num_or_size_splits=3, axis=3)
+      y1 = _conv_pool(x1)
+      y2 = _conv_pool(x2)
+      y3 = _conv_pool(x3)
+      y = array_ops.concat([y1, y2, y3], axis=3)
+      y = array_ops.identity(y)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x])
+      output = (y, g)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -638,7 +699,8 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
       self._assert_output_f16(mode, node_map, 'Relu' + suffix)
       self._assert_output_f16(mode, node_map, 'MaxPool' + suffix)
     self._assert_output_f16(mode, node_map, 'concat')
-    self.assertAllClose(output_val_ref, output_val, atol=1e-3, rtol=1e-3)
+    atol = 1e-2 if test.is_built_with_rocm() else 1e-3
+    self.assertAllClose(output_val_ref, output_val, atol=atol, rtol=1e-3)
 
   @parameterized.parameters(['cuda', 'mkl'])
   @test_util.run_deprecated_v1
@@ -646,14 +708,15 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
   def test_multi_paths_2(self, mode):
     """Test graph with multiple paths."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([8, 8])
-    y1 = _matmul_act(x)
-    y2 = _matmul_act(x)
-    y = y1 + y2 + x
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(y, [x])
-    output = (g, y)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([8, 8])
+      y1 = _matmul_act(x)
+      y2 = _matmul_act(x)
+      y = y1 + y2 + x
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x])
+      output = (g, y)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -668,7 +731,7 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
       # Bump up the tolerance for the ROCm platform
       # The default tolerance (1e-3) results in a tiny fraction (<1%) of
       # miscompares on ROCm platform, and hence the tolerance bump
-      tol = 2e-3
+      tol = 1e-2
     else:
       tol = 1e-3
     self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
@@ -679,13 +742,14 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
   def test_recurrent_lstm(self, mode):
     """Test graph with recurrent lstm."""
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    init_c = _input([8, 4])
-    init_h = _input([8, 4])
-    _, _, h, _ = _recurrent_lstm(init_c, init_h)
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(h, [init_c, init_h])
-    output = (h, g)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      init_c = _input([8, 4])
+      init_h = _input([8, 4])
+      _, _, h, _ = _recurrent_lstm(init_c, init_h)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(h, [init_c, init_h])
+      output = (h, g)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
@@ -761,20 +825,22 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
       mode: Either 'cuda' or 'mkl'.
     """
     self._maybe_skip(mode)
-    random_seed.set_random_seed(0)
-    x = _input([8, 8])
-    y = _matmul_act(x)
-    y = _example_noninlined_funcdef(y)
-    optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
-    g = optimizer.compute_gradients(y, [x])
-    output = (g, y)
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(0)
+      x = _input([8, 8])
+      y = _matmul_act(x)
+      y = _example_noninlined_funcdef(y)
+      optimizer = gradient_descent.GradientDescentOptimizer(learning_rate=0.01)
+      g = optimizer.compute_gradients(y, [x])
+      output = (g, y)
 
     output_val_ref, output_val, cost_graph = self._run(mode, output)
     node_map = _build_node_map(cost_graph.node)
 
     self._assert_output_f16(mode, node_map, 'MatMul')
     tol = 1e-2 if mode == 'mkl' else 1e-3
-    self.assertAllClose(output_val_ref, output_val, atol=tol, rtol=tol)
+    atol = 1e-2 if test.is_built_with_rocm() else tol
+    self.assertAllClose(output_val_ref, output_val, atol=atol, rtol=tol)
 
   @parameterized.parameters(['cuda', 'mkl'])
   @test_util.run_deprecated_v1
@@ -793,31 +859,32 @@ class AutoMixedPrecisionTest(test.TestCase, parameterized.TestCase):
     if tf2.enabled():
       # This test tests non-resource variables, which are only used in TF1.
       self.skipTest('TensorFlow 1 required')
-    random_seed.set_random_seed(1234)
-    np.random.seed(1234)
-    num_iter, bs, nchan, nclass = 100, 64, 32, 100
+    with ops.device(_get_device(mode)):
+      random_seed.set_random_seed(1234)
+      np.random.seed(1234)
+      num_iter, bs, nchan, nclass = 100, 64, 32, 100
 
-    data = np.random.normal(size=(bs * num_iter, nchan)).astype(np.float32)
-    labels = np.random.randint(nclass, size=(bs * num_iter,))
-    ds = dataset_ops.Dataset.from_tensor_slices((data, labels))
-    ds = ds.batch(bs).prefetch(3)
-    it = ds.make_one_shot_iterator()
+      data = np.random.normal(size=(bs * num_iter, nchan)).astype(np.float32)
+      labels = np.random.randint(nclass, size=(bs * num_iter,))
+      ds = dataset_ops.Dataset.from_tensor_slices((data, labels))
+      ds = ds.batch(bs).prefetch(3)
+      it = ds.make_one_shot_iterator()
 
-    def body(_, i):
-      i += 1
-      x, yt = it.get_next()
-      dense = layers.Dense(nclass)
-      y = dense(x)
-      loss = losses.sparse_softmax_cross_entropy(yt, y)
-      opt = adam.AdamOptimizer()
-      train_op = opt.minimize(loss, var_list=dense.trainable_weights)
-      with ops.control_dependencies([train_op]):
-        loss = array_ops.identity(loss)
-      return loss, i
+      def body(_, i):
+        i += 1
+        x, yt = it.get_next()
+        dense = layers.Dense(nclass)
+        y = dense(x)
+        loss = losses.sparse_softmax_cross_entropy(yt, y)
+        opt = adam.AdamOptimizer()
+        train_op = opt.minimize(loss, var_list=dense.trainable_weights)
+        with ops.control_dependencies([train_op]):
+          loss = array_ops.identity(loss)
+        return loss, i
 
-    begin, end = constant_op.constant(0), constant_op.constant(num_iter)
-    loss, _ = control_flow_ops.while_loop(lambda loss, i: math_ops.less(i, end),
-                                          body, [0.0, begin])
+      begin, end = constant_op.constant(0), constant_op.constant(num_iter)
+      loss, _ = while_loop.while_loop(lambda loss, i: math_ops.less(i, end),
+                                      body, [0.0, begin])
 
     output_val_ref, output_val, cost_graph = self._run(mode, loss)
     node_map = _build_node_map(cost_graph.node)
