@@ -14,23 +14,20 @@
 # ==============================================================================
 """Adam optimizer implementation."""
 # pylint: disable=g-classes-have-attributes
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
 
 from tensorflow.python.eager import def_function
+from tensorflow.python.framework import indexed_slices
 from tensorflow.python.framework import ops
+from tensorflow.python.framework import tensor_conversion
 from tensorflow.python.keras import backend_config
 from tensorflow.python.keras.optimizer_v2 import optimizer_v2
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import control_flow_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import state_ops
-from tensorflow.python.training import training_ops
-from tensorflow.python.util.tf_export import keras_export
+from tensorflow.python.training import gen_training_ops
 
 
-@keras_export('keras.optimizers.Adam')
 class Adam(optimizer_v2.OptimizerV2):
   r"""Optimizer that implements the Adam algorithm.
 
@@ -144,13 +141,17 @@ class Adam(optimizer_v2.OptimizerV2):
     apply_state[(var_device, var_dtype)].update(
         dict(
             lr=lr,
-            epsilon=ops.convert_to_tensor_v2(self.epsilon, var_dtype),
+            epsilon=tensor_conversion.convert_to_tensor_v2_with_dispatch(
+                self.epsilon, var_dtype
+            ),
             beta_1_t=beta_1_t,
             beta_1_power=beta_1_power,
             one_minus_beta_1_t=1 - beta_1_t,
             beta_2_t=beta_2_t,
             beta_2_power=beta_2_power,
-            one_minus_beta_2_t=1 - beta_2_t))
+            one_minus_beta_2_t=1 - beta_2_t,
+        )
+    )
 
   def set_weights(self, weights):
     params = self.weights
@@ -171,32 +172,32 @@ class Adam(optimizer_v2.OptimizerV2):
     v = self.get_slot(var, 'v')
 
     if not self.amsgrad:
-      return training_ops.resource_apply_adam(
-          var.handle,
-          m.handle,
-          v.handle,
-          coefficients['beta_1_power'],
-          coefficients['beta_2_power'],
-          coefficients['lr_t'],
-          coefficients['beta_1_t'],
-          coefficients['beta_2_t'],
-          coefficients['epsilon'],
-          grad,
+      return gen_training_ops.ResourceApplyAdam(
+          var=var.handle,
+          m=m.handle,
+          v=v.handle,
+          beta1_power=coefficients['beta_1_power'],
+          beta2_power=coefficients['beta_2_power'],
+          lr=coefficients['lr_t'],
+          beta1=coefficients['beta_1_t'],
+          beta2=coefficients['beta_2_t'],
+          epsilon=coefficients['epsilon'],
+          grad=grad,
           use_locking=self._use_locking)
     else:
       vhat = self.get_slot(var, 'vhat')
-      return training_ops.resource_apply_adam_with_amsgrad(
-          var.handle,
-          m.handle,
-          v.handle,
-          vhat.handle,
-          coefficients['beta_1_power'],
-          coefficients['beta_2_power'],
-          coefficients['lr_t'],
-          coefficients['beta_1_t'],
-          coefficients['beta_2_t'],
-          coefficients['epsilon'],
-          grad,
+      return gen_training_ops.ResourceApplyAdamWithAmsgrad(
+          var=var.handle,
+          m=m.handle,
+          v=v.handle,
+          vhat=vhat.handle,
+          beta1_power=coefficients['beta_1_power'],
+          beta2_power=coefficients['beta_2_power'],
+          lr=coefficients['lr_t'],
+          beta1=coefficients['beta_1_t'],
+          beta2=coefficients['beta_2_t'],
+          epsilon=coefficients['epsilon'],
+          grad=grad,
           use_locking=self._use_locking)
 
   def _resource_apply_sparse(self, grad, var, indices, apply_state=None):
@@ -243,7 +244,7 @@ class Adam(optimizer_v2.OptimizerV2):
     config = super(Adam, self).get_config()
     config.update({
         'learning_rate': self._serialize_hyperparameter('learning_rate'),
-        'decay': self._serialize_hyperparameter('decay'),
+        'decay': self._initial_decay,
         'beta_1': self._serialize_hyperparameter('beta_1'),
         'beta_2': self._serialize_hyperparameter('beta_2'),
         'epsilon': self.epsilon,
@@ -396,13 +397,17 @@ class NonFusedAdam(optimizer_v2.OptimizerV2):
     apply_state[(var_device, var_dtype)].update(
         dict(
             lr=lr,
-            epsilon=ops.convert_to_tensor_v2(self.epsilon, var_dtype),
+            epsilon=tensor_conversion.convert_to_tensor_v2_with_dispatch(
+                self.epsilon, var_dtype
+            ),
             beta_1_t=beta_1_t,
             beta_1_power=beta_1_power,
             one_minus_beta_1_t=1 - beta_1_t,
             beta_2_t=beta_2_t,
             beta_2_power=beta_2_power,
-            one_minus_beta_2_t=1 - beta_2_t))
+            one_minus_beta_2_t=1 - beta_2_t,
+        )
+    )
 
   def set_weights(self, weights):
     params = self.weights
@@ -414,7 +419,7 @@ class NonFusedAdam(optimizer_v2.OptimizerV2):
       weights = weights[:len(params)]
     super(NonFusedAdam, self).set_weights(weights)
 
-  @def_function.function(experimental_compile=True)
+  @def_function.function(jit_compile=True)
   def _resource_apply_dense(self, grad, var, apply_state=None):
     var_device, var_dtype = var.device, var.dtype.base_dtype
     coefficients = ((apply_state or {}).get((var_device, var_dtype)) or
@@ -435,7 +440,7 @@ class NonFusedAdam(optimizer_v2.OptimizerV2):
     var.assign_sub(
         (m * alpha) / (math_ops.sqrt(v) - coefficients['epsilon']))
 
-  @def_function.function(experimental_compile=True)
+  @def_function.function(jit_compile=True)
   def _resource_apply_sparse(self, grad, var, indices, apply_state=None):
     var_device, var_dtype = var.device, var.dtype.base_dtype
     coefficients = ((apply_state or {}).get((var_device, var_dtype)) or
@@ -445,13 +450,13 @@ class NonFusedAdam(optimizer_v2.OptimizerV2):
     m = self.get_slot(var, 'm')
     m_scaled_g_values = grad * coefficients['one_minus_beta_1_t']
     m.assign(m * coefficients['beta_1_t'])
-    m.scatter_add(ops.IndexedSlices(m_scaled_g_values, indices))
+    m.scatter_add(indexed_slices.IndexedSlices(m_scaled_g_values, indices))
 
     # v_t = beta2 * v + (1 - beta2) * (g_t * g_t)
     v = self.get_slot(var, 'v')
     v_scaled_g_values = (grad * grad) * coefficients['one_minus_beta_2_t']
     v.assign(v * coefficients['beta_2_t'])
-    v.scatter_add(ops.IndexedSlices(v_scaled_g_values, indices))
+    v.scatter_add(indexed_slices.IndexedSlices(v_scaled_g_values, indices))
 
     if not self.amsgrad:
       var.assign_sub(coefficients['lr'] * m /
@@ -466,7 +471,7 @@ class NonFusedAdam(optimizer_v2.OptimizerV2):
     config = super(NonFusedAdam, self).get_config()
     config.update({
         'learning_rate': self._serialize_hyperparameter('learning_rate'),
-        'decay': self._serialize_hyperparameter('decay'),
+        'decay': self._initial_decay,
         'beta_1': self._serialize_hyperparameter('beta_1'),
         'beta_2': self._serialize_hyperparameter('beta_2'),
         'epsilon': self.epsilon,
