@@ -14,10 +14,6 @@
 # ==============================================================================
 """Tests for tf_record.TFRecordWriter and tf_record.tf_record_iterator."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import gzip
 import os
 import random
@@ -268,7 +264,7 @@ class TFRecordWriterTest(TFCompressionTestCase):
     records = [random_record, repeated_record, random_record]
 
     tests = [
-        ("compression_level", 2, -1),  # Lower compression is worse.
+        ("compression_level", 2, "LE"),  # Lower compression is worse or equal.
         ("compression_level", 6, 0),  # Default compression_level is equal.
         ("flush_mode", zlib.Z_FULL_FLUSH, 1),  # A few less bytes.
         ("flush_mode", zlib.Z_NO_FLUSH, 0),  # NO_FLUSH is the default.
@@ -276,7 +272,7 @@ class TFRecordWriterTest(TFCompressionTestCase):
         ("output_buffer_size", 4096, 0),  # Increases time not size.
         ("window_bits", 8, -1),  # Smaller than default window increases size.
         ("compression_strategy", zlib.Z_HUFFMAN_ONLY, -1),  # Worse.
-        ("compression_strategy", zlib.Z_FILTERED, -1),  # Worse.
+        ("compression_strategy", zlib.Z_FILTERED, "LE"),  # Worse or equal.
     ]
 
     compression_type = tf_record.TFRecordCompressionType.ZLIB
@@ -285,10 +281,16 @@ class TFRecordWriterTest(TFCompressionTestCase):
       options_b = tf_record.TFRecordOptions(
           compression_type=compression_type, **{prop: value})
       delta = self._CompressionSizeDelta(records, options_a, options_b)
-      self.assertTrue(
-          delta == 0 if delta_sign == 0 else delta // delta_sign > 0,
-          "Setting {} = {}, file was {} smaller didn't match sign of {}".format(
-              prop, value, delta, delta_sign))
+      if delta_sign == "LE":
+        self.assertLessEqual(
+            delta, 0,
+            "Setting {} = {}, file was {} smaller didn't match sign of {}"
+            .format(prop, value, delta, delta_sign))
+      else:
+        self.assertTrue(
+            delta == 0 if delta_sign == 0 else delta // delta_sign > 0,
+            "Setting {} = {}, file was {} smaller didn't match sign of {}"
+            .format(prop, value, delta, delta_sign))
 
 
 class TFRecordWriterZlibTest(TFCompressionTestCase):
@@ -474,6 +476,58 @@ class TFRecordIteratorTest(TFCompressionTestCase):
       with self.assertRaises(StopIteration):
         next(iterator)
 
+  def testReadReplacedFile_preservesReadOffset_afterReopen(self):
+    """Verify that tf_record_iterator allows reopening at the same read offset.
+
+    In some cases, data will be logically "appended" to a file by replacing the
+    entire file with a new version that includes the additional data. For
+    example, this can happen with certain GCS implementations (since GCS has no
+    true append operation), or when using rsync without the `--inplace` option
+    to transfer snapshots of a growing file. Since the iterator retains a handle
+    to a stale version of the file, it won't return any of the new data.
+
+    To force this to happen, callers can check for a replaced file (e.g. via a
+    stat call that reflects an increased file size) and opt to close and reopen
+    the iterator. When iteration is next attempted, this should result in
+    reading from the newly opened file, while preserving the read offset. This
+    behavior is required by TensorBoard.
+    """
+    def write_records_to_file(filename, records):
+      writer = tf_record.TFRecordWriter(filename)
+      for record in records:
+        writer.write(record)
+      writer.close()
+
+    fn = os.path.join(self.get_temp_dir(), "orig_file")
+    write_records_to_file(fn, [b"one", b"two"])
+    iterator = tf_record.tf_record_iterator(fn)
+    self.assertEqual(b"one", next(iterator))
+    self.assertEqual(b"two", next(iterator))
+    # Iterating at EOF results in StopIteration repeatedly.
+    with self.assertRaises(StopIteration):
+      next(iterator)
+    with self.assertRaises(StopIteration):
+      next(iterator)
+    # Add a new record to the end of the file by overwriting it.
+    fn2 = os.path.join(self.get_temp_dir(), "new_file")
+    write_records_to_file(fn2, [b"one", b"two", b"three"])
+    # Windows disallows replacing files while in use, so close iterator early.
+    if os.name == "nt":
+      iterator.close()
+    os.replace(fn2, fn)
+    # Iterating at EOF still results in StopIteration; new data is not shown.
+    with self.assertRaises(StopIteration):
+      next(iterator)
+    with self.assertRaises(StopIteration):
+      next(iterator)
+    # Retrying after close and reopen successfully returns the new record,
+    # preserving the prior read offset.
+    iterator.close()
+    iterator.reopen()
+    self.assertEqual(b"three", next(iterator))
+    with self.assertRaises(StopIteration):
+      next(iterator)
+
 
 class TFRecordRandomReaderTest(TFCompressionTestCase):
 
@@ -491,7 +545,7 @@ class TFRecordRandomReaderTest(TFCompressionTestCase):
       self.assertEqual(record, records[i])
       offsets.append(offset)
     # Reading off the bound should lead to error.
-    with self.assertRaisesRegexp(IndexError, r"Out of range.*offset"):
+    with self.assertRaisesRegex(IndexError, r"Out of range.*offset"):
       reader.read(offset)
     # Do a pass of backward reading.
     for i in range(self._num_records - 1, 0, -1):
@@ -503,8 +557,7 @@ class TFRecordRandomReaderTest(TFCompressionTestCase):
     records = [self._Record(0, i) for i in range(self._num_records)]
     fn = self._WriteRecordsToFile(records, "uncompressed_records")
     reader = tf_record.tf_record_random_reader(fn)
-    with self.assertRaisesRegexp(
-        errors_impl.DataLossError, r"corrupted record"):
+    with self.assertRaisesRegex(errors_impl.DataLossError, r"corrupted record"):
       reader.read(1)  # 1 is guaranteed to be an invalid offset.
 
   def testClosingRandomReaderCausesErrorsForFurtherReading(self):
@@ -512,8 +565,7 @@ class TFRecordRandomReaderTest(TFCompressionTestCase):
     fn = self._WriteRecordsToFile(records, "uncompressed_records")
     reader = tf_record.tf_record_random_reader(fn)
     reader.close()
-    with self.assertRaisesRegexp(
-        errors_impl.FailedPreconditionError, r"closed"):
+    with self.assertRaisesRegex(errors_impl.FailedPreconditionError, r"closed"):
       reader.read(0)
 
 
