@@ -14,16 +14,11 @@
 # ==============================================================================
 """Tests for `tf.Module`."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import abc
 import collections
 import itertools
 
 from absl.testing import parameterized
-import six
 
 from tensorflow.python import tf2
 from tensorflow.python.distribute import ps_values
@@ -31,8 +26,10 @@ from tensorflow.python.distribute import tpu_values
 from tensorflow.python.distribute import values as distributed_values
 from tensorflow.python.eager import context
 from tensorflow.python.eager import def_function
+from tensorflow.python.framework import composite_tensor
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import test_util
+from tensorflow.python.framework import type_spec
 from tensorflow.python.module import module
 from tensorflow.python.ops import variables
 from tensorflow.python.platform import test
@@ -109,7 +106,7 @@ class TestModuleNaming(test_util.TensorFlowTestCase):
 
   def test_invalid_name(self):
     msg = ".* is not a valid module name"
-    with self.assertRaisesRegexp(ValueError, msg):
+    with self.assertRaisesRegex(ValueError, msg):
       module.Module(name="$Foo")
 
   @test_util.run_in_graph_and_eager_modes
@@ -170,7 +167,7 @@ class TestModuleNaming(test_util.TensorFlowTestCase):
 
       def __getattr__(self, name):
         scope_names.append((name, get_name_scope()))
-        return super(GetAttrModule, self).__getattr__(name)
+        return super().__getattr__(name)
 
     mod = GetAttrModule()
     with self.assertRaises(AttributeError):
@@ -184,7 +181,7 @@ class TestModuleNaming(test_util.TensorFlowTestCase):
 
       def __getattribute__(self, name):
         scope_names.append((name, get_name_scope()))
-        return super(GetAttributeModule, self).__getattribute__(name)
+        return super().__getattribute__(name)
 
     mod = GetAttributeModule()
     with self.assertRaises(AttributeError):
@@ -260,6 +257,37 @@ class VariableTrackingTest(test_util.TensorFlowTestCase):
     m.c = aggregating
     self.assertEqual(m.variables, (mirrored, tpu, aggregating))
 
+  def test_composite_variable(self):
+
+    class Spec(type_spec.TypeSpec):
+
+      value_type = property(lambda self: CompositeVariable)
+
+      def _component_specs(self):
+        pass
+
+      def _serialize(self):
+        pass
+
+      def _to_components(self, value):
+        return value._variables
+
+      def _from_components(self, variable_list):
+        return CompositeVariable(variable_list)
+
+    class CompositeVariable(composite_tensor.CompositeTensor):
+
+      def __init__(self, variable_list):
+        self._variables = variable_list
+
+      @property
+      def _type_spec(self):
+        return Spec()
+
+    m = module.Module()
+    m.a = CompositeVariable([variables.Variable(1.), variables.Variable(2.)])
+    self.assertEqual(list(m.variables), list(m.a._variables))
+
 
 class ModuleTrackingTest(test_util.TensorFlowTestCase):
 
@@ -302,8 +330,8 @@ class ForwardMethodsTest(test_util.TensorFlowTestCase):
 class AbcTest(test_util.TensorFlowTestCase):
 
   def testAbstract(self):
-    msg = "Can't instantiate .* abstract methods"
-    with self.assertRaisesRegexp(TypeError, msg):
+    msg = "Can't instantiate.*abstract"
+    with self.assertRaisesRegex(TypeError, msg):
       AbstractModule()  # pylint: disable=abstract-class-instantiated
 
   def testConcrete(self):
@@ -328,7 +356,7 @@ class ErrorModule(module.Module):
 
   def __init__(self, call_super, raise_in_constructor=True):
     if call_super:
-      super(ErrorModule, self).__init__()
+      super().__init__()
     if raise_in_constructor:
       raise ErrorModuleError("Deliberate error!")
 
@@ -339,7 +367,7 @@ class ErrorModule(module.Module):
 class RecursiveModule(module.Module):
 
   def __init__(self, depth, trainable=True):
-    super(RecursiveModule, self).__init__(name="badger")
+    super().__init__(name="badger")
     with self.name_scope:
       self.child = None
       if depth > 1:
@@ -347,8 +375,7 @@ class RecursiveModule(module.Module):
       self.w = variables.Variable(1.0, trainable=trainable, name="mushroom")
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractModule(module.Module):
+class AbstractModule(module.Module, metaclass=abc.ABCMeta):
 
   @abc.abstractmethod
   def __call__(self, x):
@@ -365,7 +392,7 @@ class ConcreteModule(AbstractModule):
 class TreeModule(module.Module):
 
   def __init__(self, name=None):
-    super(TreeModule, self).__init__(name=name)
+    super().__init__(name=name)
     self._leaves = []
 
   @module.Module.with_name_scope
@@ -423,7 +450,7 @@ class ModuleWithFunctionAnnotatedCall(module.Module):
 class PropertyModule(module.Module):
 
   def __init__(self):
-    super(PropertyModule, self).__init__()
+    super().__init__()
     self._setter_scope_name = None
 
   @property
@@ -504,14 +531,30 @@ class FlattenTest(parameterized.TestCase, test_util.TensorFlowTestCase):
                       ("decoder", "w", 0, 0, "k"): mod.decoder.w[0][0]["k"],
                       ("decoder", "w", 0, 1, "k"): mod.decoder.w[0][1]["k"]},)
 
-  def test_raises_error_with_path(self):
-    if six.PY2:
-      class NonOrderable(object):
-        __lt__ = None
+  def test_cycles_with_path(self):
+    mod = module.Module()
+    mod.w = variables.Variable(1.)
+    mod.encoder = module.Module()
+    mod.encoder.w = [({"k": mod.w}, {"k": mod.w})]
+    mod.decoder = mod.encoder
 
-      non_orderable = NonOrderable
-    else:
-      non_orderable = object
+    # This introduces two cycles: on mod.encoder.mod and mod.decoder.mod.
+    mod.decoder.mod = mod
+
+    state_dict = dict(
+        mod._flatten(with_path=True, predicate=module._is_variable))
+
+    self.assertEqual(state_dict,
+                     {("w",): mod.w,
+                      ("encoder", "mod", "w"): mod.encoder.mod.w,
+                      ("decoder", "mod", "w"): mod.decoder.mod.w,
+                      ("encoder", "w", 0, 0, "k"): mod.encoder.w[0][0]["k"],
+                      ("encoder", "w", 0, 1, "k"): mod.encoder.w[0][1]["k"],
+                      ("decoder", "w", 0, 0, "k"): mod.decoder.w[0][0]["k"],
+                      ("decoder", "w", 0, 1, "k"): mod.decoder.w[0][1]["k"]},)
+
+  def test_raises_error_with_path(self):
+    non_orderable = object
 
     m = module.Module()
     m.layers = {non_orderable(): None, non_orderable(): None}
@@ -523,7 +566,7 @@ class FlattenTest(parameterized.TestCase, test_util.TensorFlowTestCase):
 class LayerModule(module.Module):
 
   def __init__(self):
-    super(LayerModule, self).__init__()
+    super().__init__()
     self._trainable_variables = [
         variables.Variable(1., name="a"),
         variables.Variable(2., name="b"),
@@ -546,7 +589,7 @@ class LayerModule(module.Module):
             attribute_traversal_key=key_function))
 
 
-class MemberType(object):
+class MemberType:
   """A simple type to search for."""
   pass
 
@@ -554,7 +597,7 @@ class MemberType(object):
 class SimpleModule(module.Module):
 
   def __init__(self, create_child=True, container_type=list):
-    super(SimpleModule, self).__init__()
+    super().__init__()
     self.z = MemberType()
     self.a = container_type([MemberType(), MemberType()])
     if create_child:
