@@ -14,10 +14,6 @@
 # ==============================================================================
 """Tests for Collective Operations."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import time
 
 from tensorflow.core.protobuf import config_pb2
@@ -33,15 +29,20 @@ from tensorflow.python.framework import ops
 from tensorflow.python.framework import test_util
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import collective_ops
-from tensorflow.python.ops import control_flow_ops
 from tensorflow.python.ops import math_ops
+from tensorflow.python.ops import variable_v1
 from tensorflow.python.ops import variables
+from tensorflow.python.ops import while_loop
 from tensorflow.python.platform import test
 from tensorflow.python.platform import tf_logging as logging
 
 
 class CollectiveOpTest(test.TestCase):
 
+  def setUp(self):
+    context._reset_context()  # pylint: disable=protected-access
+    super(CollectiveOpTest, self).setUp()
+
   def _testCollectiveReduce(self,
                             inputs,
                             expected,
@@ -162,49 +163,6 @@ class CollectiveOpTest(test.TestCase):
     elapsed = time.time() - start_time
     self.assertAllGreaterEqual(elapsed, timeout)
 
-  @test_util.run_v2_only
-  def testCollectiveTimeoutV2(self):
-    context._reset_context()
-    timeout = 4.5
-    cpus = config.list_physical_devices('CPU')
-    self.assertEqual(len(cpus), 1)
-    config.set_logical_device_configuration(cpus[0], [
-        context.LogicalDeviceConfiguration(),
-        context.LogicalDeviceConfiguration()
-    ])
-    context.ensure_initialized()
-
-    @def_function.function
-    def run_all_reduce(group_size, reported_group_size=None):
-      group_key = 20
-      instance_key = 30
-      tensor = [1, 2, 3, 4]
-      results = []
-      if reported_group_size is None:
-        reported_group_size = group_size
-      for i in range(group_size):
-        with ops.device('/CPU:{}'.format(i)):
-          input_data = constant_op.constant(tensor)
-          collective_op = collective_ops.all_reduce(
-              input_data,
-              group_size=reported_group_size,
-              group_key=group_key,
-              instance_key=instance_key,
-              merge_op='Add',
-              final_op='Id',
-              timeout=timeout)
-          results.append(collective_op)
-      return results
-
-    run_all_reduce(2, 2)
-
-    start_time = time.time()
-    with self.assertRaisesRegex(errors.DeadlineExceededError,
-                                'Collective has timed out during execution'):
-      run_all_reduce(1, 2)
-    elapsed = time.time() - start_time
-    self.assertAllGreaterEqual(elapsed, timeout)
-
   def testNcclHintFallbackToRingReduce(self):
     """Tests that setting `communication_hint=nccl` works on non-GPU builds."""
     if kernels.get_registered_kernels_for_op('NcclAllReduce'):
@@ -236,9 +194,9 @@ class CollectiveOpTest(test.TestCase):
       for device in devices:
         with ops.device(device):
           loop_vars.append(
-              [variables.VariableV1((1 << i) * 1.) for i in range(num_vars)])
+              [variable_v1.VariableV1((1 << i) * 1.) for i in range(num_vars)])
       # This variable controls number of iterations.
-      loop_vars.append(variables.VariableV1(0.))
+      loop_vars.append(variable_v1.VariableV1(0.))
       def loop_body(dev0_tensors, dev1_tensors, loop_tensor):
         return_ops = []
         for i in range(len(devices)):
@@ -263,8 +221,7 @@ class CollectiveOpTest(test.TestCase):
       # Run until last variable exceeds number of iterations.
       loop_cond = lambda d0, d1, i: math_ops.less(i, num_iterations)
       sess.run(variables.global_variables_initializer())
-      results = sess.run(control_flow_ops.while_loop(loop_cond, loop_body,
-                                                     loop_vars))
+      results = sess.run(while_loop.while_loop(loop_cond, loop_body, loop_vars))
       self.assertEqual(results[:-1], [
           [((1 << (num_iterations + v)) * 1.) for v in range(num_vars)]
           for _ in range(group_size)])
@@ -301,7 +258,7 @@ class CollectiveOpTest(test.TestCase):
             constant = constant_op.constant(0.)
             cond = lambda i: math_ops.less(i, 10.)
             body = lambda i: math_ops.add(i, 1.)
-            input0 = control_flow_ops.while_loop(cond, body, [constant])
+            input0 = while_loop.while_loop(cond, body, [constant])
             input1 = math_ops.add(constant, 5)
             colred0 = collective_ops.all_reduce(input0, group_size, group_key,
                                                 instance_key0, 'Add', 'Id')
@@ -426,8 +383,8 @@ class CollectiveOpTest(test.TestCase):
         run_options = config_pb2.RunOptions()
         run_options.experimental.collective_graph_key = 1
         sess.run([c0, c1], options=run_options)
-        with self.assertRaisesRegexp(errors.InvalidArgumentError,
-                                     'Shape mismatch'):
+        with self.assertRaisesRegex(errors.InvalidArgumentError,
+                                    'Shape mismatch'):
           sess.run([c0, c2], options=run_options)
 
   def testCollectiveGatherShapeMismatchAcrossDevices(self):
@@ -447,8 +404,8 @@ class CollectiveOpTest(test.TestCase):
           c1 = collective_ops.all_gather(in1, 2, group_key, instance_key)
         run_options = config_pb2.RunOptions()
         run_options.experimental.collective_graph_key = 1
-        with self.assertRaisesRegexp(errors.InvalidArgumentError,
-                                     'Shape mismatch'):
+        with self.assertRaisesRegex(errors.InvalidArgumentError,
+                                    'Shape mismatch'):
           sess.run([c0, c1], options=run_options)
 
   def testCollectiveGatherPolymorphicShape(self):
@@ -483,6 +440,8 @@ class CollectiveOpTest(test.TestCase):
     self.assertAllClose(results_[1], expected_output_, rtol=1e-5, atol=1e-5)
 
   @test_util.run_v2_only
+  @test_util.disable_tfrt(
+      'b/177270918: TFRT has dead lock when executing collective ops.')
   def testCollectiveGroupSizeMismatch(self):
     cpus = config.list_physical_devices('CPU')
     self.assertEqual(len(cpus), 1)
@@ -492,6 +451,20 @@ class CollectiveOpTest(test.TestCase):
     ])
     context.ensure_initialized()
 
+  @test_util.run_v2_only
+  def testCollectiveGatherShapeCheckFailure(self):
+    with self.assertRaisesRegex(errors.InvalidArgumentError,
+                                'input should have rank > 0'):
+      collective_ops.gen_collective_ops.CollectiveGather(
+          input=1,
+          group_size=1,
+          group_key=1,
+          instance_key=1,
+          shape=(3, 3, 3),
+          communication_hint='auto',
+          timeout_seconds=0,
+          name='')
+
     @def_function.function
     def run_all_reduce():
       group_key = 10
@@ -510,13 +483,12 @@ class CollectiveOpTest(test.TestCase):
             merge_op='Add', final_op='Id')
       return c0, c1
 
-    with self.assertRaisesRegexp(errors.InternalError,
-                                 'but that group has size'):
+    with self.assertRaisesRegex(errors.InternalError,
+                                'but that group has size'):
       run_all_reduce()
 
   @test_util.run_v2_only
   def testCollectiveTensorsHaveNoDeviceSpecified(self):
-    context._reset_context()
     cpus = config.list_physical_devices('CPU')
     self.assertEqual(len(cpus), 1)
     config.set_logical_device_configuration(cpus[0], [
@@ -555,22 +527,6 @@ class CollectiveOpTest(test.TestCase):
     result = fn([in0, in1])
     self.assertAllClose(result, [2, 2])
 
-  @test_util.run_v2_only
-  def testCollectiveGroupSizeOne(self):
-    group_size = 1
-    group_key = 100
-    instance_key = 100
-    in_value = [1, 2, 3, 4]
-    in_tensor = constant_op.constant(in_value)
-
-    reduced_tensor = collective_ops.all_reduce(
-        in_tensor, group_size, group_key, instance_key, 'Add', 'Id')
-    self.assertAllEqual(in_value, reduced_tensor.numpy())
-
-    gathered_tensor = collective_ops.all_gather(
-        in_tensor, group_size, group_key, instance_key)
-    self.assertAllEqual(in_value, gathered_tensor.numpy())
-
   def testConstantWithScopedAllocator(self):
     group_size = 2
     group_key = 1
@@ -607,41 +563,6 @@ class CollectiveOpTest(test.TestCase):
         results = sess.run(run_ops)
     self.assertEqual(results, [3., 3., 3., 3.])
 
-  @test_util.run_v2_only
-  def testMultipleGroups(self):
-    context._reset_context()
-    cpus = config.list_physical_devices('CPU')
-    self.assertEqual(len(cpus), 1)
-    config.set_logical_device_configuration(cpus[0], [
-        context.LogicalDeviceConfiguration(),
-        context.LogicalDeviceConfiguration(),
-        context.LogicalDeviceConfiguration()
-    ])
-    context.ensure_initialized()
-    num_elements = 4
-
-    @def_function.function
-    def run_all_reduce(group_size, group_key):
-      instance_key = group_key
-      input_value = [group_key for i in range(num_elements)]
-      collectives = []
-      for device_idx in range(group_size):
-        with ops.device('/CPU:{}'.format(device_idx)):
-          input_tensor = constant_op.constant(input_value)
-          collectives.append(collective_ops.all_reduce(
-              input_tensor, group_size, group_key, instance_key, merge_op='Add',
-              final_op='Id'))
-      return collectives
-
-    def run_and_assert(group_size, group_key):
-      for reduced_tensor in run_all_reduce(group_size, group_key):
-        self.assertAllEqual(
-            [group_key * group_size for i in range(num_elements)],
-            reduced_tensor.numpy())
-
-    run_and_assert(group_size=2, group_key=1)
-    run_and_assert(group_size=3, group_key=2)
-
 
 if __name__ == '__main__':
   test.main()
