@@ -14,15 +14,13 @@
 # ==============================================================================
 """Create a Block Diagonal operator from one or more `LinearOperators`."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 from tensorflow.python.framework import common_shapes
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
+from tensorflow.python.framework import tensor_conversion
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import array_ops_stack
 from tensorflow.python.ops import check_ops
 from tensorflow.python.ops import control_flow_ops
 from tensorflow.python.ops.linalg import linear_operator
@@ -34,29 +32,27 @@ __all__ = ["LinearOperatorBlockDiag"]
 
 
 @tf_export("linalg.LinearOperatorBlockDiag")
+@linear_operator.make_composite_tensor
 class LinearOperatorBlockDiag(linear_operator.LinearOperator):
   """Combines one or more `LinearOperators` in to a Block Diagonal matrix.
 
   This operator combines one or more linear operators `[op1,...,opJ]`,
-  building a new `LinearOperator`, whose underlying matrix representation is
-  square and has each operator `opi` on the main diagonal, and zero's elsewhere.
+  building a new `LinearOperator`, whose underlying matrix representation
+  has each operator `opi` on the main diagonal, and zero's elsewhere.
 
   #### Shape compatibility
 
-  If `opj` acts like a [batch] square matrix `Aj`, then `op_combined` acts like
-  the [batch] square matrix formed by having each matrix `Aj` on the main
+  If `opj` acts like a [batch] matrix `Aj`, then `op_combined` acts like
+  the [batch] matrix formed by having each matrix `Aj` on the main
   diagonal.
 
-  Each `opj` is required to represent a square matrix, and hence will have
-  shape `batch_shape_j + [M_j, M_j]`.
+  Each `opj` is required to represent a matrix, and hence will have
+  shape `batch_shape_j + [M_j, N_j]`.
 
-  If `opj` has shape `batch_shape_j + [M_j, M_j]`, then the combined operator
-  has shape `broadcast_batch_shape + [sum M_j, sum M_j]`, where
+  If `opj` has shape `batch_shape_j + [M_j, N_j]`, then the combined operator
+  has shape `broadcast_batch_shape + [sum M_j, sum N_j]`, where
   `broadcast_batch_shape` is the mutual broadcast of `batch_shape_j`,
   `j = 1,...,J`, assuming the intermediate batch shapes broadcast.
-  Even if the combined shape is well defined, the combined operator's
-  methods may fail due to lack of broadcasting ability in the defining
-  operators' methods.
 
   Arguments to `matmul`, `matvec`, `solve`, and `solvevec` may either be single
   `Tensor`s or lists of `Tensor`s that are interpreted as blocks. The `j`th
@@ -64,6 +60,10 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
   `opj` for the given method. If a list of blocks is input, then a list of
   blocks is returned as well.
 
+  When the `opj` are not guaranteed to be square, this operator's methods might
+  fail due to the combined operator not being square and/or lack of efficient
+  methods.
+
   ```python
   # Create a 4 x 4 linear operator combined of two 2 x 2 operators.
   operator_1 = LinearOperatorFullMatrix([[1., 2.], [3., 4.]])
@@ -88,6 +88,23 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
   operator.matmul(x)
   ==> tf.concat([operator_1.matmul(x1), operator_2.matmul(x2)])
 
+  # Create a 5 x 4 linear operator combining three blocks.
+  operator_1 = LinearOperatorFullMatrix([[1.], [3.]])
+  operator_2 = LinearOperatorFullMatrix([[1., 6.]])
+  operator_3 = LinearOperatorFullMatrix([[2.], [7.]])
+  operator = LinearOperatorBlockDiag([operator_1, operator_2, operator_3])
+
+  operator.to_dense()
+  ==> [[1., 0., 0., 0.],
+       [3., 0., 0., 0.],
+       [0., 1., 6., 0.],
+       [0., 0., 0., 2.]]
+       [0., 0., 0., 7.]]
+
+  operator.shape
+  ==> [5, 4]
+
+
   # Create a [2, 3] batch of 4 x 4 linear operators.
   matrix_44 = tf.random.normal(shape=[2, 3, 4, 4])
   operator_44 = LinearOperatorFullMatrix(matrix)
@@ -163,6 +180,15 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
       TypeError:  If all operators do not have the same `dtype`.
       ValueError:  If `operators` is empty or are non-square.
     """
+    parameters = dict(
+        operators=operators,
+        is_non_singular=is_non_singular,
+        is_self_adjoint=is_self_adjoint,
+        is_positive_definite=is_positive_definite,
+        is_square=is_square,
+        name=name
+    )
+
     # Validate operators.
     check_ops.assert_proper_iterable(operators)
     operators = list(operators)
@@ -204,31 +230,19 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
             "positive definite.")
       is_positive_definite = True
 
-    if not (is_square and all(operator.is_square for operator in operators)):
-      raise ValueError(
-          "Can only represent a block diagonal of square matrices.")
-
-    # Initialization.
-    graph_parents = []
-    for operator in operators:
-      graph_parents.extend(operator.graph_parents)
-
     if name is None:
       # Using ds to mean direct sum.
       name = "_ds_".join(operator.name for operator in operators)
-    with ops.name_scope(name, values=graph_parents):
+    with ops.name_scope(name):
       super(LinearOperatorBlockDiag, self).__init__(
           dtype=dtype,
-          graph_parents=None,
           is_non_singular=is_non_singular,
           is_self_adjoint=is_self_adjoint,
           is_positive_definite=is_positive_definite,
-          is_square=True,
+          is_square=is_square,
+          parameters=parameters,
           name=name)
 
-    # TODO(b/143910018) Remove graph_parents in V3.
-    self._set_graph_parents(graph_parents)
-
   @property
   def operators(self):
     return self._operators
@@ -249,7 +263,7 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
     # Get final matrix shape.
     domain_dimension = sum(self._block_domain_dimensions())
     range_dimension = sum(self._block_range_dimensions())
-    matrix_shape = tensor_shape.TensorShape([domain_dimension, range_dimension])
+    matrix_shape = tensor_shape.TensorShape([range_dimension, domain_dimension])
 
     # Get broadcast batch shape.
     # broadcast_shape checks for compatibility.
@@ -263,12 +277,13 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
   def _shape_tensor(self):
     # Avoid messy broadcasting if possible.
     if self.shape.is_fully_defined():
-      return ops.convert_to_tensor(
-          self.shape.as_list(), dtype=dtypes.int32, name="shape")
+      return tensor_conversion.convert_to_tensor_v2_with_dispatch(
+          self.shape.as_list(), dtype=dtypes.int32, name="shape"
+      )
 
     domain_dimension = sum(self._block_domain_dimension_tensors())
     range_dimension = sum(self._block_range_dimension_tensors())
-    matrix_shape = array_ops.stack([domain_dimension, range_dimension])
+    matrix_shape = array_ops_stack.stack([range_dimension, domain_dimension])
 
     # Dummy Tensor of zeros.  Will never be materialized.
     zeros = array_ops.zeros(shape=self.operators[0].batch_shape_tensor())
@@ -278,6 +293,9 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
 
     return array_ops.concat((batch_shape, matrix_shape), 0)
 
+  # TODO(b/188080761): Add a more efficient implementation of `cond` that
+  # constructs the condition number from the blockwise singular values.
+
   def matmul(self, x, adjoint=False, adjoint_arg=False, name="matmul"):
     """Transform [batch] matrix `x` with left multiplication:  `x --> Ax`.
 
@@ -309,33 +327,53 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
         as `self`, or if `x` is blockwise, a list of `Tensor`s with shapes that
         concatenate to `[..., M, R]`.
     """
+    def _check_operators_agree(r, l, message):
+      if (r.range_dimension is not None and
+          l.domain_dimension is not None and
+          r.range_dimension != l.domain_dimension):
+        raise ValueError(message)
+
     if isinstance(x, linear_operator.LinearOperator):
       left_operator = self.adjoint() if adjoint else self
       right_operator = x.adjoint() if adjoint_arg else x
 
-      if (right_operator.range_dimension is not None and
-          left_operator.domain_dimension is not None and
-          right_operator.range_dimension != left_operator.domain_dimension):
-        raise ValueError(
-            "Operators are incompatible. Expected `x` to have dimension"
-            " {} but got {}.".format(
-                left_operator.domain_dimension, right_operator.range_dimension))
-      with self._name_scope(name):
+      _check_operators_agree(
+          right_operator, left_operator,
+          "Operators are incompatible. Expected `x` to have dimension"
+          " {} but got {}.".format(
+              left_operator.domain_dimension, right_operator.range_dimension))
+
+      # We can efficiently multiply BlockDiag LinearOperators if the number of
+      # blocks agree.
+      if isinstance(x, LinearOperatorBlockDiag):
+        if len(left_operator.operators) != len(right_operator.operators):
+          raise ValueError(
+              "Can not efficiently multiply two `LinearOperatorBlockDiag`s "
+              "together when number of blocks differ.")
+
+        for o1, o2 in zip(left_operator.operators, right_operator.operators):
+          _check_operators_agree(
+              o2, o1,
+              "Blocks are incompatible. Expected `x` to have dimension"
+              " {} but got {}.".format(
+                  o1.domain_dimension, o2.range_dimension))
+
+      with self._name_scope(name):  # pylint: disable=not-callable
         return linear_operator_algebra.matmul(left_operator, right_operator)
 
-    with self._name_scope(name):
+    with self._name_scope(name):  # pylint: disable=not-callable
       arg_dim = -1 if adjoint_arg else -2
       block_dimensions = (self._block_range_dimensions() if adjoint
                           else self._block_domain_dimensions())
       if linear_operator_util.arg_is_blockwise(block_dimensions, x, arg_dim):
         for i, block in enumerate(x):
           if not isinstance(block, linear_operator.LinearOperator):
-            block = ops.convert_to_tensor(block)
+            block = tensor_conversion.convert_to_tensor_v2_with_dispatch(block)
             self._check_input_dtype(block)
             block_dimensions[i].assert_is_compatible_with(block.shape[arg_dim])
             x[i] = block
       else:
-        x = ops.convert_to_tensor(x, name="x")
+        x = tensor_conversion.convert_to_tensor_v2_with_dispatch(x, name="x")
         self._check_input_dtype(x)
         op_dimension = (self.range_dimension if adjoint
                         else self.domain_dimension)
@@ -346,17 +384,19 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
     arg_dim = -1 if adjoint_arg else -2
     block_dimensions = (self._block_range_dimensions() if adjoint
                         else self._block_domain_dimensions())
+    block_dimensions_fn = (
+        self._block_range_dimension_tensors if adjoint
+        else self._block_domain_dimension_tensors)
     blockwise_arg = linear_operator_util.arg_is_blockwise(
         block_dimensions, x, arg_dim)
     if blockwise_arg:
       split_x = x
+
     else:
       split_dim = -1 if adjoint_arg else -2
       # Split input by rows normally, and otherwise columns.
       split_x = linear_operator_util.split_arg_into_blocks(
-          self._block_domain_dimensions(),
-          self._block_domain_dimension_tensors,
-          x, axis=split_dim)
+          block_dimensions, block_dimensions_fn, x, axis=split_dim)
 
     result_list = []
     for index, operator in enumerate(self.operators):
@@ -398,13 +438,13 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
     Returns:
       A `Tensor` with shape `[..., M]` and same `dtype` as `self`.
     """
-    with self._name_scope(name):
+    with self._name_scope(name):  # pylint: disable=not-callable
       block_dimensions = (self._block_range_dimensions() if adjoint
                           else self._block_domain_dimensions())
       if linear_operator_util.arg_is_blockwise(block_dimensions, x, -1):
         for i, block in enumerate(x):
           if not isinstance(block, linear_operator.LinearOperator):
-            block = ops.convert_to_tensor(block)
+            block = tensor_conversion.convert_to_tensor_v2_with_dispatch(block)
             self._check_input_dtype(block)
             block_dimensions[i].assert_is_compatible_with(block.shape[-1])
             x[i] = block
@@ -412,7 +452,7 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
         y_mat = self.matmul(x_mat, adjoint=adjoint)
         return [array_ops.squeeze(y, axis=-1) for y in y_mat]
 
-      x = ops.convert_to_tensor(x, name="x")
+      x = tensor_conversion.convert_to_tensor_v2_with_dispatch(x, name="x")
       self._check_input_dtype(x)
       op_dimension = (self.range_dimension if adjoint
                       else self.domain_dimension)
@@ -483,21 +523,42 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
       raise NotImplementedError(
           "Exact solve not implemented for an operator that is expected to "
           "not be square.")
+
+    def _check_operators_agree(r, l, message):
+      if (r.range_dimension is not None and
+          l.domain_dimension is not None and
+          r.range_dimension != l.domain_dimension):
+        raise ValueError(message)
+
     if isinstance(rhs, linear_operator.LinearOperator):
       left_operator = self.adjoint() if adjoint else self
       right_operator = rhs.adjoint() if adjoint_arg else rhs
 
-      if (right_operator.range_dimension is not None and
-          left_operator.domain_dimension is not None and
-          right_operator.range_dimension != left_operator.domain_dimension):
-        raise ValueError(
-            "Operators are incompatible. Expected `rhs` to have dimension"
-            " {} but got {}.".format(
-                left_operator.domain_dimension, right_operator.range_dimension))
-      with self._name_scope(name):
+      _check_operators_agree(
+          right_operator, left_operator,
+          "Operators are incompatible. Expected `x` to have dimension"
+          " {} but got {}.".format(
+              left_operator.domain_dimension, right_operator.range_dimension))
+
+      # We can efficiently solve BlockDiag LinearOperators if the number of
+      # blocks agree.
+      if isinstance(right_operator, LinearOperatorBlockDiag):
+        if len(left_operator.operators) != len(right_operator.operators):
+          raise ValueError(
+              "Can not efficiently solve `LinearOperatorBlockDiag` when "
+              "number of blocks differ.")
+
+        for o1, o2 in zip(left_operator.operators, right_operator.operators):
+          _check_operators_agree(
+              o2, o1,
+              "Blocks are incompatible. Expected `x` to have dimension"
+              " {} but got {}.".format(
+                  o1.domain_dimension, o2.range_dimension))
+
+      with self._name_scope(name):  # pylint: disable=not-callable
         return linear_operator_algebra.solve(left_operator, right_operator)
 
-    with self._name_scope(name):
+    with self._name_scope(name):  # pylint: disable=not-callable
       block_dimensions = (self._block_domain_dimensions() if adjoint
                           else self._block_range_dimensions())
       arg_dim = -1 if adjoint_arg else -2
@@ -508,12 +569,14 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
         split_rhs = rhs
         for i, block in enumerate(split_rhs):
           if not isinstance(block, linear_operator.LinearOperator):
-            block = ops.convert_to_tensor(block)
+            block = tensor_conversion.convert_to_tensor_v2_with_dispatch(block)
             self._check_input_dtype(block)
             block_dimensions[i].assert_is_compatible_with(block.shape[arg_dim])
             split_rhs[i] = block
       else:
-        rhs = ops.convert_to_tensor(rhs, name="rhs")
+        rhs = tensor_conversion.convert_to_tensor_v2_with_dispatch(
+            rhs, name="rhs"
+        )
         self._check_input_dtype(rhs)
         op_dimension = (self.domain_dimension if adjoint
                         else self.range_dimension)
@@ -577,13 +640,13 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
     Raises:
       NotImplementedError:  If `self.is_non_singular` or `is_square` is False.
     """
-    with self._name_scope(name):
+    with self._name_scope(name):  # pylint: disable=not-callable
       block_dimensions = (self._block_domain_dimensions() if adjoint
                           else self._block_range_dimensions())
       if linear_operator_util.arg_is_blockwise(block_dimensions, rhs, -1):
         for i, block in enumerate(rhs):
           if not isinstance(block, linear_operator.LinearOperator):
-            block = ops.convert_to_tensor(block)
+            block = tensor_conversion.convert_to_tensor_v2_with_dispatch(block)
             self._check_input_dtype(block)
             block_dimensions[i].assert_is_compatible_with(block.shape[-1])
             rhs[i] = block
@@ -591,7 +654,9 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
         solution_mat = self.solve(rhs_mat, adjoint=adjoint)
         return [array_ops.squeeze(x, axis=-1) for x in solution_mat]
 
-      rhs = ops.convert_to_tensor(rhs, name="rhs")
+      rhs = tensor_conversion.convert_to_tensor_v2_with_dispatch(
+          rhs, name="rhs"
+      )
       self._check_input_dtype(rhs)
       op_dimension = (self.domain_dimension if adjoint
                       else self.range_dimension)
@@ -601,6 +666,10 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
       return array_ops.squeeze(solution_mat, axis=-1)
 
   def _diag_part(self):
+    if not all(operator.is_square for operator in self.operators):
+      raise NotImplementedError(
+          "`diag_part` not implemented for an operator whose blocks are not "
+          "square.")
     diag_list = []
     for operator in self.operators:
       # Extend the axis for broadcasting.
@@ -610,6 +679,10 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
     return array_ops.squeeze(diagonal, axis=-1)
 
   def _trace(self):
+    if not all(operator.is_square for operator in self.operators):
+      raise NotImplementedError(
+          "`trace` not implemented for an operator whose blocks are not "
+          "square.")
     result = self.operators[0].trace()
     for operator in self.operators[1:]:
       result += operator.trace()
@@ -655,6 +728,10 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
         operator.assert_positive_definite() for operator in self.operators])
 
   def _eigvals(self):
+    if not all(operator.is_square for operator in self.operators):
+      raise NotImplementedError(
+          "`eigvals` not implemented for an operator whose blocks are not "
+          "square.")
     eig_list = []
     for operator in self.operators:
       # Extend the axis for broadcasting.
@@ -662,3 +739,11 @@ class LinearOperatorBlockDiag(linear_operator.LinearOperator):
     eig_list = linear_operator_util.broadcast_matrix_batch_dims(eig_list)
     eigs = array_ops.concat(eig_list, axis=-2)
     return array_ops.squeeze(eigs, axis=-1)
+
+  @property
+  def _composite_tensor_fields(self):
+    return ("operators",)
+
+  @property
+  def _experimental_parameter_ndims_to_matrix_ndims(self):
+    return {"operators": [0] * len(self.operators)}
