@@ -14,16 +14,13 @@
 # ==============================================================================
 """Tests for vectorization of math kernels."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 from absl.testing import parameterized
 
 from tensorflow.python.eager import backprop
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops as framework_ops
+from tensorflow.python.framework import tensor
 from tensorflow.python.framework import test_util
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import clip_ops
@@ -33,6 +30,7 @@ from tensorflow.python.ops import nn
 from tensorflow.python.ops import random_ops
 from tensorflow.python.ops import special_math_ops
 from tensorflow.python.ops import tensor_array_grad  # pylint: disable=unused-import
+from tensorflow.python.ops.parallel_for import control_flow_ops as pfor_control_flow_ops
 from tensorflow.python.ops.parallel_for.test_util import PForTestCase
 from tensorflow.python.platform import test
 
@@ -152,8 +150,8 @@ class MathTest(PForTestCase, parameterized.TestCase):
 
   def test_binary_cwise_ops(self):
     # Enable tensor equality to test `equal` and `not_equal` ops below.
-    default_equality = framework_ops.Tensor._USE_EQUALITY
-    framework_ops.enable_tensor_equality()
+    default_equality = tensor.Tensor._USE_EQUALITY
+    tensor.enable_tensor_equality()
     try:
       logical_ops = [
           math_ops.logical_and, math_ops.logical_or, math_ops.logical_xor
@@ -196,12 +194,15 @@ class MathTest(PForTestCase, parameterized.TestCase):
           math_ops.subtract,
           math_ops.truncate_mod,
           safe_polygamma,
-          safe_zeta,
       ]
       # FloorDiv fails on XLA due floor's discontinuities exacerbating small
       # division differences.
       if not test_util.is_xla_enabled():
         float_ops += [math_ops.floor_div]
+        # TODO(b/168912036): Re-enable once GPU + XLA issues for Zeta are
+        # resolved.
+        if not test_util.is_gpu_available():
+          float_ops += [safe_zeta]
       for op in logical_ops + float_ops:
         x = random_ops.random_uniform([7, 3, 5])
         y = random_ops.random_uniform([3, 5])
@@ -225,7 +226,7 @@ class MathTest(PForTestCase, parameterized.TestCase):
         self._test_loop_fn(loop_fn, 3)
     finally:
       if not default_equality:
-        framework_ops.disable_tensor_equality()
+        tensor.disable_tensor_equality()
 
   def test_approximate_equal(self):
     x = random_ops.random_uniform([3, 5])
@@ -238,6 +239,21 @@ class MathTest(PForTestCase, parameterized.TestCase):
 
     self._test_loop_fn(loop_fn, 3)
 
+  def test_abs_complex(self):
+    r = pfor_control_flow_ops.vectorized_map(
+        math_ops.abs, math_ops.cast([0, -1], dtype=dtypes.complex128))
+    self.assertAllEqual(self.evaluate(r), [0, 1])
+
+  def test_colocate_with(self):
+    a = random_ops.random_uniform([3, 5])
+
+    def loop_fn(i):
+      x1 = array_ops.gather(a, i)
+      with framework_ops.colocate_with(x1):
+        return x1 * 2
+
+    self._test_loop_fn(loop_fn, 3)
+
   def test_addn(self):
     x = random_ops.random_uniform([2, 3, 5])
     y = random_ops.random_uniform([3, 5])
@@ -261,6 +277,9 @@ class MathTest(PForTestCase, parameterized.TestCase):
 
     self._test_loop_fn(loop_fn, 4)
 
+  @test_util.run_without_tensor_float_32(
+      "Calls matmul in parallel for-loop and compares result to calling matmul "
+      "in sequential for-loop")
   def test_matmul(self):
     for tr_a in (True, False):
       for tr_b in (True, False):
@@ -341,7 +360,7 @@ class MathTest(PForTestCase, parameterized.TestCase):
         math_ops.reduce_min,
         math_ops.reduce_mean,
     ]:
-      for axis in ([1], None, [0, 2]):
+      for axis in ([1], None, [0, 2], constant_op.constant([1], dtypes.int64)):
         for keepdims in (True, False):
 
           # pylint: disable=cell-var-from-loop
@@ -356,7 +375,7 @@ class MathTest(PForTestCase, parameterized.TestCase):
   def test_boolean_reduction(self):
     x = random_ops.random_uniform([2, 3, 4, 5]) > 0.5
     for op in [math_ops.reduce_any, math_ops.reduce_all]:
-      for axis in ([1], None, [0, 2]):
+      for axis in ([1], None, [0, 2], constant_op.constant([1], dtypes.int64)):
         for keepdims in (True, False):
 
           # pylint: disable=cell-var-from-loop
@@ -402,7 +421,7 @@ class MathTest(PForTestCase, parameterized.TestCase):
 
   def test_cum_sum(self):
     x = random_ops.random_uniform([2, 3, 4, 5])
-    for axis in (1, -2):
+    for axis in (1, -2, constant_op.constant(1, dtypes.int64)):
       for exclusive in (True, False):
         for reverse in (True, False):
 
@@ -418,7 +437,7 @@ class MathTest(PForTestCase, parameterized.TestCase):
 
   def test_cum_prod(self):
     x = random_ops.random_uniform([2, 3, 4, 5])
-    for axis in (1, -2):
+    for axis in (1, -2, constant_op.constant(1, dtypes.int64)):
       for exclusive in (True, False):
         for reverse in (True, False):
 
@@ -539,7 +558,8 @@ class MathTest(PForTestCase, parameterized.TestCase):
 
     self._test_loop_fn(loop_fn, 3)
 
-  @parameterized.parameters(math_ops.sparse_segment_mean_grad,
+  @parameterized.parameters(math_ops.sparse_segment_sum_grad,
+                            math_ops.sparse_segment_mean_grad,
                             math_ops.sparse_segment_sqrt_n_grad)
   def test_sparse_segment_grad(self, op_func):
     grad = random_ops.random_uniform([3, 3, 2])
@@ -678,15 +698,15 @@ class LinalgTest(PForTestCase):
       self._test_loop_fn(loop_fn, 3)
 
   def test_matrix_inverse(self):
-    x = (random_ops.random_uniform([3, 4, 2, 2]) +
-         10 * linalg_ops.eye(2))  # Ensure well-conditioned.
+    x = (random_ops.random_uniform([3, 4, 2, 2]) + 10 * linalg_ops.eye(2)
+        )  # Ensure well-conditioned.
 
     for adjoint in (True, False):
 
       # pylint: disable=cell-var-from-loop
       def loop_fn(i):
-        return linalg_ops.matrix_inverse(array_ops.gather(x, i),
-                                         adjoint=adjoint)
+        return linalg_ops.matrix_inverse(
+            array_ops.gather(x, i), adjoint=adjoint)
 
       # pylint: enable=cell-var-from-loop
       self._test_loop_fn(loop_fn, 2)
@@ -697,8 +717,8 @@ class LinalgTest(PForTestCase):
         for stack_b in (True, False):
           shape_a = (2, 4, 3, 3) if stack_a else (4, 3, 3)
           shape_b = (2, 4, 3, 5) if stack_b else (4, 3, 5)
-          x = (random_ops.random_uniform(shape_a) +
-               10 * linalg_ops.eye(3))  # Ensure well-conditioned.
+          x = (random_ops.random_uniform(shape_a) + 10 * linalg_ops.eye(3)
+              )  # Ensure well-conditioned.
           y = random_ops.random_uniform(shape_b)
 
           # pylint: disable=cell-var-from-loop
@@ -745,6 +765,9 @@ class LinalgTest(PForTestCase):
 
     self._test_loop_fn(loop_fn, 2)
 
+  @test_util.run_without_tensor_float_32(
+      "Calls einsum in parallel for-loop and compares result to calling einsum "
+      "in sequential for-loop")
   def test_einsum(self):
     b = 10
     x_series = random_ops.random_uniform([b, 9, 9])
@@ -755,12 +778,13 @@ class LinalgTest(PForTestCase):
       y = array_ops.gather(y_series, 0)  # invariant.
       x_i = array_ops.gather(x_series, i)
       y_i = array_ops.gather(y_series, i)
+      z0 = special_math_ops.einsum("ab->b", x_i)
       z1 = special_math_ops.einsum("ab,bc->ac", x_i, y)
       z2 = special_math_ops.einsum("ab,bc->ac", x, y_i)
       z3 = special_math_ops.einsum("ab,bc->ac", x, y)
       z4 = special_math_ops.einsum("ab,bc->ac", x_i, y_i)
       z5 = special_math_ops.einsum("cd,ce->de", y_i, x_i)  # Includes transpose.
-      outputs = [z1, z2, z3, z4, z5]
+      outputs = [z0, z1, z2, z3, z4, z5]
       return outputs
 
     self._test_loop_fn(loop_fn, b)
