@@ -14,30 +14,36 @@
 # ==============================================================================
 """Array operations for RaggedTensors."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from typing import Optional
+from typing import Union
 
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import sparse_tensor
+from tensorflow.python.framework import tensor as tensor_lib
+from tensorflow.python.framework import tensor_shape
 from tensorflow.python.framework import tensor_util
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import array_ops_stack
 from tensorflow.python.ops import check_ops
+from tensorflow.python.ops import control_flow_ops
+from tensorflow.python.ops import data_flow_ops
 from tensorflow.python.ops import gen_ragged_array_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import sort_ops
+from tensorflow.python.ops.ragged import dynamic_ragged_shape
 from tensorflow.python.ops.ragged import ragged_functional_ops
 from tensorflow.python.ops.ragged import ragged_math_ops
 from tensorflow.python.ops.ragged import ragged_tensor
 from tensorflow.python.ops.ragged import ragged_util
 from tensorflow.python.ops.ragged import segment_id_ops
+from tensorflow.python.types import core as core_types
 from tensorflow.python.util import dispatch
 from tensorflow.python.util.tf_export import tf_export
 
-#===============================================================================
+# ===============================================================================
 # Masking
-#===============================================================================
+# ===============================================================================
 
 
 @tf_export('ragged.boolean_mask')
@@ -205,10 +211,11 @@ def boolean_mask(data, mask, name=None):
       return masked_values
 
 
-#===============================================================================
+# ===============================================================================
 # Tiling
-#===============================================================================
-def tile(input, multiples, name=None):  # pylint: disable=redefined-builtin
+# ===============================================================================
+@dispatch.dispatch_for_api(array_ops.tile)
+def tile(input: ragged_tensor.Ragged, multiples, name=None):  # pylint: disable=redefined-builtin
   """Constructs a `RaggedTensor` by tiling a given `RaggedTensor`.
 
   The values of `input` are replicated `multiples[i]` times along the
@@ -376,12 +383,13 @@ def _tile_ragged_splits(rt_input, multiples, const_multiples=None):
   return result_splits
 
 
-#===============================================================================
+# ===============================================================================
 # Reshaping
-#===============================================================================
+# ===============================================================================
 
 
-def expand_dims(input, axis, name=None):  # pylint: disable=redefined-builtin
+@dispatch.dispatch_for_api(array_ops.expand_dims_v2)
+def expand_dims(input: ragged_tensor.Ragged, axis, name=None):  # pylint: disable=redefined-builtin
   """Inserts a dimension with shape 1 into a potentially ragged tensor's shape.
 
   Given a potentially ragged tenor `input`, this operation inserts a
@@ -447,15 +455,30 @@ def expand_dims(input, axis, name=None):  # pylint: disable=redefined-builtin
       return ragged_tensor.RaggedTensor.from_uniform_row_length(
           input, uniform_row_length=1, nrows=input.nrows(), validate=False)
     else:
-      return input.with_values(expand_dims(input.values, axis - 1))
+      if ragged_tensor.is_ragged(input.values):
+        return input.with_values(expand_dims(input.values, axis - 1))
+      else:
+        return input.with_values(array_ops.expand_dims(input.values, axis - 1))
 
 
-#===============================================================================
+@dispatch.dispatch_for_api(array_ops.expand_dims)
+def _ragged_expand_dims_v1(
+    input: ragged_tensor.Ragged,  # pylint: disable=redefined-builtin
+    axis=None,
+    name=None,
+    dim=None):
+  if dim is not None:
+    axis = dim
+  return expand_dims(input=input, axis=axis, name=name)
+
+
+# ===============================================================================
 # RaggedTensor Size
-#===============================================================================
+# ===============================================================================
 
 
-def size(input, out_type=dtypes.int32, name=None):  # pylint: disable=redefined-builtin
+@dispatch.dispatch_for_api(array_ops.size_v2)
+def size(input: ragged_tensor.Ragged, out_type=dtypes.int32, name=None):  # pylint: disable=redefined-builtin
   """Returns the size of a potentially ragged tensor.
 
   The size of a ragged tensor is the size of its inner values.
@@ -479,10 +502,19 @@ def size(input, out_type=dtypes.int32, name=None):  # pylint: disable=redefined-
     return array_ops.size(input, out_type=out_type, name=name)
 
 
-#===============================================================================
+@dispatch.dispatch_for_api(array_ops.size)
+def _ragged_size_v1(
+    input: ragged_tensor.Ragged,  # pylint: disable=redefined-builtin
+    name=None,
+    out_type=dtypes.int32):
+  return size(input=input, out_type=out_type, name=name)
+
+
+# ===============================================================================
 # ragged.rank
-#===============================================================================
-def rank(input, name=None):  # pylint: disable=redefined-builtin
+# ===============================================================================
+@dispatch.dispatch_for_api(array_ops.rank)
+def rank(input: ragged_tensor.Ragged, name=None):  # pylint: disable=redefined-builtin
   """Returns the rank of a RaggedTensor.
 
   Returns a 0-D `int32` `Tensor` representing the rank of `input`.
@@ -508,10 +540,11 @@ def rank(input, name=None):  # pylint: disable=redefined-builtin
     return input.ragged_rank + array_ops.rank(input.flat_values)
 
 
-#===============================================================================
+# ===============================================================================
 # ragged.one_hot
-#===============================================================================
-def ragged_one_hot(indices,
+# ===============================================================================
+@dispatch.dispatch_for_api(array_ops.one_hot)
+def ragged_one_hot(indices: ragged_tensor.Ragged,
                    depth,
                    on_value=None,
                    off_value=None,
@@ -536,9 +569,9 @@ def ragged_one_hot(indices,
                           dtype, name))
 
 
-#===============================================================================
+# ===============================================================================
 # ragged.stack_dynamic_partitions
-#===============================================================================
+# ===============================================================================
 @tf_export('ragged.stack_dynamic_partitions')
 @dispatch.add_dispatch_support
 def stack_dynamic_partitions(data, partitions, num_partitions, name=None):
@@ -603,8 +636,8 @@ def stack_dynamic_partitions(data, partitions, num_partitions, name=None):
       # If partitions is a scalar, then just create a RaggedTensor containing
       # that single the complete `data` value in the specified row.
       return ragged_tensor.RaggedTensor.from_value_rowids(
-          values=array_ops.stack([data]),
-          value_rowids=array_ops.stack([partitions]),
+          values=array_ops_stack.stack([data]),
+          value_rowids=array_ops_stack.stack([partitions]),
           nrows=num_partitions,
           validate=False)
 
@@ -615,11 +648,14 @@ def stack_dynamic_partitions(data, partitions, num_partitions, name=None):
       permutation = sort_ops.argsort(partitions, stable=True)
       value_rowids = array_ops.gather(partitions, permutation)
       values = array_ops.gather(data, permutation)
-      check = check_ops.assert_less(
-          value_rowids[-1:],
-          num_partitions,
-          message='partitions must be less than num_partitions')
-      with ops.control_dependencies([check]):
+      checks = [
+          check_ops.assert_less(
+              value_rowids[-1:], num_partitions,
+              message='partitions must be less than num_partitions'),
+          check_ops.assert_non_negative(
+              partitions, message='partitions must be non-negative.')
+      ]
+      with ops.control_dependencies(checks):
         return ragged_tensor.RaggedTensor.from_value_rowids(
             values, value_rowids, nrows=num_partitions, validate=False)
 
@@ -642,10 +678,11 @@ def stack_dynamic_partitions(data, partitions, num_partitions, name=None):
                                         num_partitions)
 
 
-#===============================================================================
+# ===============================================================================
 # Reverse
-#===============================================================================
-def reverse(tensor, axis, name=None):
+# ===============================================================================
+@dispatch.dispatch_for_api(array_ops.reverse)
+def reverse(tensor: ragged_tensor.Ragged, axis, name=None):
   """Reverses a RaggedTensor along the specified axes.
 
   #### Example:
@@ -668,7 +705,7 @@ def reverse(tensor, axis, name=None):
                     'when reversing axes in a ragged tensor')
 
   with ops.name_scope(name, 'Reverse', [tensor, axis]):
-    if isinstance(axis, ops.Tensor):
+    if isinstance(axis, tensor_lib.Tensor):
       axis = tensor_util.constant_value(axis)
       if axis is None:
         raise TypeError(type_error_msg)
@@ -696,9 +733,9 @@ def reverse(tensor, axis, name=None):
     return tensor[tuple(slices)]
 
 
-#===============================================================================
+# ===============================================================================
 # Cross
-#===============================================================================
+# ===============================================================================
 
 
 @tf_export('ragged.cross')
@@ -835,3 +872,429 @@ def _cross_internal(inputs,
 
     return ragged_tensor.RaggedTensor.from_row_splits(
         values_out, splits_out, validate=False)
+
+
+def fill_empty_rows(ragged_input, default_value, name=None):
+  """Fills empty rows in the input `RaggedTensor` with rank 2 with a default
+
+  value.
+
+  This op adds entries with the specified `default_value` for any row in the
+  input that does not already have a value.
+
+  The op also returns an indicator vector such that
+
+      empty_row_indicator[i] = True iff row i was an empty row.
+
+  Args:
+    ragged_input: A `RaggedTensor` with rank 2.
+    default_value: The value to fill for empty rows, with the same type as
+      `ragged_input.`
+    name: A name prefix for the returned tensors (optional)
+
+  Returns:
+    ragged_ordered_output: A `RaggedTensor`with all empty rows filled in with
+      `default_value`.
+    empty_row_indicator: A bool vector indicating whether each input row was
+      empty.
+
+  Raises:
+    TypeError: If `ragged_input` is not a `RaggedTensor`.
+  """
+  with ops.name_scope(name, 'RaggedFillEmptyRows', [ragged_input]):
+    if not isinstance(ragged_input, ragged_tensor.RaggedTensor):
+      raise TypeError(
+          'ragged_input must be RaggedTensor,             got'
+          f' {type(ragged_input)}'
+      )
+    default_value = ops.convert_to_tensor(
+        default_value, dtype=ragged_input.dtype
+    )
+    (
+        output_value_rowids,
+        output_values,
+        empty_row_indicator,
+        unused_reverse_index_map,
+    ) = gen_ragged_array_ops.ragged_fill_empty_rows(
+        value_rowids=ragged_input.value_rowids(),
+        values=ragged_input.values,
+        nrows=ragged_input.nrows(),
+        default_value=default_value,
+    )
+    return (
+        ragged_tensor.RaggedTensor.from_value_rowids(
+            values=output_values,
+            value_rowids=output_value_rowids,
+            validate=False,
+        ),
+        empty_row_indicator,
+    )
+
+
+@ops.RegisterGradient('RaggedFillEmptyRows')
+def _ragged_fill_empty_rows_grad(
+    op,
+    unused_grad_output_indices,
+    output_grad_values,
+    unused_grad_empty_row_indicator,
+    unused_grad_reverse_index_map,
+):
+  """Gradients for RaggedFillEmptyRows."""
+  reverse_index_map = op.outputs[3]
+
+  d_values, d_default_value = gen_ragged_array_ops.ragged_fill_empty_rows_grad(
+      reverse_index_map=reverse_index_map, grad_values=output_grad_values
+  )
+
+  # d_value_rowids, d_values, d_nrows, d_default_value.
+  return [None, d_values, None, d_default_value]
+
+
+# ===============================================================================
+# dynamic_partition
+# ===============================================================================
+@dispatch.dispatch_for_api(data_flow_ops.dynamic_partition)
+def dynamic_partition(data: ragged_tensor.RaggedOrDense,
+                      partitions: ragged_tensor.RaggedOrDense,
+                      num_partitions,
+                      name=None):
+  """RaggedTensor dispatch override for tf.dynamic_partition."""
+  if not isinstance(num_partitions, int) or num_partitions < 0:
+    raise TypeError('num_partitions must be a non-negative integer')
+  result = stack_dynamic_partitions(data, partitions, num_partitions, name)
+  return [result[i] for i in range(num_partitions)]
+
+
+# ===============================================================================
+# split
+# ===============================================================================
+@dispatch.dispatch_for_api(array_ops.split)
+def split(value: ragged_tensor.Ragged,
+          num_or_size_splits,
+          axis=0,
+          num=None,
+          name=None):
+  """Splits a RaggedTensor `value` into a list of sub RaggedTensors.
+
+  If `num_or_size_splits` is an `int`,  then it splits `value` along the
+  dimension `axis` into `num_or_size_splits` smaller RaggedTensors. This
+  requires that `value.shape[axis]` is divisible by `num_or_size_splits`.
+
+  If `num_or_size_splits` is a 1-D Tensor (or list), then `value` is split into
+  `len(num_or_size_splits)` elements. The shape of the `i`-th element has the
+  same size as the `value` except along dimension `axis` where the size is
+  `num_or_size_splits[i]`.
+
+  Splits along a ragged dimension is not allowed.
+
+  For example:
+
+  >>> rt = tf.RaggedTensor.from_row_lengths(
+  ...      np.arange(6 * 3).reshape(6, 3), row_lengths=[1, 2, 2, 1])
+  >>> rt.shape
+  TensorShape([4, None, 3])
+  >>>
+  >>> rt1, rt2 = tf.split(rt, 2)  # uniform splits
+  >>> rt1.shape
+  TensorShape([2, None, 3])
+  >>> rt2.shape
+  TensorShape([2, None, 3])
+  >>>
+  >>> rt3, rt4, rt5 = tf.split(rt, [1, 2, 1])  # ragged splits
+  >>> rt3.shape
+  TensorShape([1, None, 3])
+  >>> rt4.shape
+  TensorShape([2, None, 3])
+  >>> rt5.shape
+  TensorShape([1, None, 3])
+  >>>
+  >>> rt6, rt7 = tf.split(rt, [1, 2], axis=2)  # splits along axis 2
+  >>> rt6.shape
+  TensorShape([4, None, 1])
+  >>> rt7.shape
+  TensorShape([4, None, 2])
+
+  Args:
+    value: The `RaggedTensor` to split.
+    num_or_size_splits: Either an `int` indicating the number of splits
+      along `axis` or a 1-D integer `Tensor` or Python list containing the sizes
+      of each output tensor along `axis`. If a Python int, then it must evenly
+      divide `value.shape[axis]`; otherwise the sum of sizes along the split
+      axis must match that of the `value`.
+    axis: An `int` or scalar `int32` `Tensor`. The dimension along which
+      to split. Must be in the range `[-rank(value), rank(value))`. Defaults to
+      0.
+    num: An `int` used to specify the number of outputs when
+      `num_or_size_splits` is a 1-D list or `Tensor` and its length is
+      statically unknown, e.g., specifying `tf.TensorSepc(None)` with
+      the `input_signature` argument of `tf.function` (optional).
+    name: A name for the operation (optional).
+
+  Returns:
+    if `num_or_size_splits` is an `int` returns a list of `num_or_size_splits`
+    `RaggedTensor` objects; if `num_or_size_splits` is a 1-D Tensor returns
+    `num_or_size_splits.get_shape[0]` `RaggedTensor` objects resulting from
+    splitting `value`.
+
+  Raises:
+    ValueError: If the dimension `axis` of `value` is a ragged dimension.
+    ValueError: If `num` is unspecified and cannot be inferred.
+    ValueError: If `num` is specified but doesn't match the length of
+      `num_or_size_splits`.
+    ValueError: If `num_or_size_splits` is an `int` and less than 1.
+    TypeError: If `num_or_size_splits` is not an `int` or 1-D
+      list or 1-D `Tensor`.
+    InvalidArgumentError: If the `axis` of `value` cannot be exactly splitted
+      by `num_or_size_splits`.
+    InvalidArgumentError: If `num_or_size_splits` is contains negative integers.
+    InvalidArgumentError: If `num_or_size_splits`'s static shape is unknown and
+      its dynamic shape is inconsistent `num`.
+    InvalidArgumentError: If `num_or_size_splits`'s static rank is unknown and
+      `axis` is a negative integer.
+  """
+  with ops.name_scope(name, 'RaggedSplit'):
+    value = ragged_tensor.convert_to_tensor_or_ragged_tensor(
+        value, name='value')
+    if isinstance(num_or_size_splits, int) and num_or_size_splits == 1:
+      return [value]
+
+    # static assert
+    check_ops.assert_integer_v2(
+        num_or_size_splits,
+        message=('`num_or_size_splits` must be an `int` or 1-D list or '
+                 '`Tensor` of integers.'))
+    value_shape = dynamic_ragged_shape.DynamicRaggedShape.from_tensor(value)
+    axis = array_ops.get_positive_axis(axis, value_shape.rank)
+    try:
+      dim_size = value_shape[axis]
+    except ValueError:
+      raise ValueError('Cannot split a ragged dimension. Got `value` with '
+                       f'shape {value_shape} and `axis` {axis}.')
+    if isinstance(num_or_size_splits, int):
+      # Uniform split
+      num_splits = num_or_size_splits
+      if num_splits < 1:
+        raise ValueError('`num_or_size_splits` must be >=1 if it is an `int`.'
+                         f'Received {num_or_size_splits}.')
+      split_length = math_ops.floordiv(dim_size, num_splits)
+      split_lengths = array_ops.repeat(split_length, num_splits)
+    else:
+      # Ragged split
+      num_splits = None
+      split_lengths = ops.convert_to_tensor(num_or_size_splits)
+      if split_lengths.shape.ndims is not None:
+        if split_lengths.shape.ndims != 1:
+          raise TypeError('`num_or_size_splits` must be an `int` or 1-D list '
+                          f'or `Tensor`. Received {num_or_size_splits}.')
+        num_splits = tensor_shape.dimension_value(split_lengths.shape[0])
+
+      if num_splits is None:
+        if num is None:
+          raise ValueError('`num` must be specified as an `int` when the '
+                           'size of `num_or_size_split` is statically '
+                           f'unknown. Received `num`: {num} and '
+                           f'`num_or_size_split`: {num_or_size_splits}.')
+        num_splits = num
+      else:
+        if num is not None and num != num_splits:
+          raise ValueError('`num` does not match the size of '
+                           f'`num_or_size_split`. Received `num`: {num} and '
+                           f'size of `num_or_size_split`: {num_splits}.')
+
+    splits = array_ops.concat([[0], math_ops.cumsum(split_lengths)], axis=0)
+    checks = []
+    checks.append(
+        check_ops.assert_non_negative_v2(
+            num_or_size_splits,
+            message='`num_or_size_splits` must be non-negative.'))
+    checks.append(
+        check_ops.assert_equal_v2(
+            num_splits,
+            array_ops.shape(split_lengths)[0],
+            message='`num` is inconsistent with `num_or_size_split.shape[0]`.'))
+    checks.append(
+        check_ops.assert_equal_v2(
+            math_ops.cast(dim_size, splits.dtype),
+            splits[-1],
+            message=('Cannot exactly split the `axis` dimension of `value` '
+                     'with the given `num_or_size_split`.')))
+    splits = control_flow_ops.with_dependencies(checks, splits)
+    splited_rts = []
+    slices = [slice(None)] * (axis + 1)
+    for i in range(num_splits):
+      slices[-1] = slice(splits[i], splits[i + 1])
+      splited_rts.append(value[tuple(slices)])
+    return splited_rts
+
+
+# ===============================================================================
+# RaggedTensor shape operations
+# ===============================================================================
+
+
+@dispatch.dispatch_for_api(array_ops.reshape)
+def ragged_reshape(
+    tensor: ragged_tensor.RaggedOrDense,
+    shape: dynamic_ragged_shape.DenseOrRaggedShape
+) -> Union[ragged_tensor.RaggedTensor, tensor_lib.Tensor]:
+  """Reshapes a tensor or ragged tensor."""
+  tensor = ragged_tensor.convert_to_tensor_or_ragged_tensor(
+      tensor, name='tensor')
+  if isinstance(tensor, ragged_tensor.RaggedTensor):
+    tensor = tensor.values
+
+  if isinstance(shape, dynamic_ragged_shape.DynamicRaggedShape):
+    flat_values = array_ops.reshape(tensor, shape.inner_shape)
+    return ragged_tensor.RaggedTensor._from_nested_row_partitions(  # pylint: disable=protected-access
+        flat_values,
+        shape.row_partitions,
+        validate=False)
+  else:
+    shape = ops.convert_to_tensor(shape, name='shape')
+    return array_ops.reshape(tensor, shape)
+
+
+@dispatch.dispatch_for_api(array_ops.broadcast_to)
+def broadcast_to(
+    input: ragged_tensor.RaggedOrDense,  # pylint: disable=redefined-builtin
+    shape: dynamic_ragged_shape.DynamicRaggedShape
+) -> Union[ragged_tensor.RaggedTensor, tensor_lib.Tensor]:
+  """Broadcasts a potentially ragged tensor to a ragged shape.
+
+  Tiles `input` as necessary to match the given shape.
+
+  Behavior is undefined if `input` is not broadcast-compatible with `shape`.
+
+  Args:
+    input: The potentially ragged tensor to broadcast.
+    shape: A `DynamicRaggedShape`
+
+  Returns:
+    A potentially ragged tensor whose values are taken from
+    `input`, and whose shape matches `shape`.
+  """
+  return dynamic_ragged_shape.broadcast_to(input, shape)
+
+
+# Note: default value for out_type needs to be int32, to match the
+# default for tf.shape's out_type parameter.
+@dispatch.dispatch_for_api(array_ops.shape)
+def ragged_shape(
+    input: ragged_tensor.Ragged,  # pylint: disable=redefined-builtin
+    name: Optional[str] = None,
+    out_type=dtypes.int32) -> dynamic_ragged_shape.DynamicRaggedShape:
+  """Returns the shape of a RaggedTensor.
+
+  Args:
+    input: A `RaggedTensor`
+    name: A name for the operation (optional).
+    out_type: dtype used to encode the shape.
+
+  Returns:
+    A `tf.experimental.DynamicRaggedShape`
+  """
+  with ops.name_scope(name, 'RaggedShape', [input]):
+    return dynamic_ragged_shape.DynamicRaggedShape.from_tensor(input, out_type)
+
+
+@dispatch.dispatch_for_api(array_ops.broadcast_dynamic_shape)
+def broadcast_dynamic_shape(
+    shape_x: dynamic_ragged_shape.DenseOrRaggedShape,
+    shape_y: dynamic_ragged_shape.DenseOrRaggedShape
+) -> dynamic_ragged_shape.DynamicRaggedShape:
+  """Returns the shape formed by broadcasting two shapes to be compatible.
+
+  1. If shape_x and shape_y both have row_partitions, then fail if their dtypes
+     don't match.
+  2. If neither has row_partitions and they have different dtypes,
+     go with int64.
+  3. If one has row_partitions, go with that dtype.
+
+  Args:
+    shape_x: A `DynamicRaggedShape`
+    shape_y: A `DynamicRaggedShape`
+
+  Returns:
+    A `DynamicRaggedShape`.
+  Raises:
+    ValueError: If `shape_x` and `shape_y` are not broadcast-compatible.
+  """
+  if not isinstance(shape_x, dynamic_ragged_shape.DynamicRaggedShape):
+    shape_x = dynamic_ragged_shape.DynamicRaggedShape([], shape_x)
+  if not isinstance(shape_y, dynamic_ragged_shape.DynamicRaggedShape):
+    shape_y = dynamic_ragged_shape.DynamicRaggedShape([], shape_y)
+  return dynamic_ragged_shape.broadcast_dynamic_shape(shape_x, shape_y)
+
+
+@dispatch.dispatch_for_api(array_ops.ones)
+def ones(
+    shape: dynamic_ragged_shape.DynamicRaggedShape,
+    dtype=dtypes.float32,
+    name=None,
+    layout=None,
+) -> ragged_tensor.RaggedOrDense:
+  """Returns ones shaped like x."""
+  if layout is not None and not layout.is_fully_replicated():
+    raise ValueError(
+        f'RaggedTensor only allows replicated layout. got {layout}'
+    )
+  flat_values = array_ops.ones(
+      shape.inner_shape, dtype=dtype, name=name, layout=layout
+  )
+  return shape._add_row_partitions(flat_values)  # pylint: disable=protected-access
+
+
+@dispatch.dispatch_for_api(array_ops.zeros)
+def zeros(
+    shape: dynamic_ragged_shape.DynamicRaggedShape,
+    dtype=dtypes.float32,
+    name=None,
+    layout=None,
+) -> ragged_tensor.RaggedOrDense:
+  """Returns ones shaped like x."""
+  if layout is not None and not layout.is_fully_replicated():
+    raise ValueError(
+        f'RaggedTensor only allows replicated layout. got {layout}'
+    )
+  flat_values = array_ops.zeros(
+      shape.inner_shape, dtype=dtype, name=name, layout=layout
+  )
+  return shape._add_row_partitions(flat_values)  # pylint: disable=protected-access
+
+
+@dispatch.dispatch_for_api(array_ops.fill)
+def fill(
+    dims: dynamic_ragged_shape.DynamicRaggedShape,
+    value: core_types.TensorLike,
+    name: Optional[str] = None,
+    layout=None,
+) -> ragged_tensor.RaggedOrDense:
+  """Creates a tensor with shape `dims` and fills it with `value`."""
+  if layout is not None and not layout.is_fully_replicated():
+    raise ValueError(
+        f'RaggedTensor only allows replicated layout. got {layout}'
+    )
+  flat_values = array_ops.fill(
+      dims.inner_shape, value, name=name, layout=layout
+  )
+  return dims._add_row_partitions(flat_values)  # pylint: disable=protected-access
+
+
+# ===============================================================================
+# bitcast
+# ===============================================================================
+@dispatch.dispatch_for_api(array_ops.bitcast)
+def bitcast(
+    input: ragged_tensor.RaggedOrDense,  # pylint: disable=redefined-builtin
+    type,  # pylint: disable=redefined-builtin
+    name=None) -> ragged_tensor.RaggedOrDense:
+  """RaggedTensor dispatch override for tf.bitcast."""
+  type = dtypes.as_dtype(type)
+  with ops.name_scope(name, 'Bitcast', [input]):
+    input = ragged_tensor.convert_to_tensor_or_ragged_tensor(
+        input, name='input')
+    if (input.dtype.size < type.size and input.flat_values.shape.rank < 2):
+      raise ValueError('`input.flat_values` is required to have rank >= 2 when '
+                       'input.dtype.size < type.size. Actual rank: '
+                       f'{input.flat_values.shape.rank}')
+    return input.with_flat_values(array_ops.bitcast(input.flat_values, type))
