@@ -13,10 +13,6 @@
 # limitations under the License.
 # ===================================================================
 """TPU Feature Column Library."""
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import math
 
 from tensorflow.python.feature_column import feature_column as fc
@@ -27,6 +23,7 @@ from tensorflow.python.ops import init_ops
 from tensorflow.python.ops import variable_scope
 from tensorflow.python.tpu import tpu
 from tensorflow.python.tpu import tpu_function
+from tensorflow.python.tpu import tpu_replication
 # pylint: disable=protected-access
 
 
@@ -36,13 +33,13 @@ _SUPPORTED_SEQUENCE_COLUMNS = (fc._SequenceCategoricalColumn,
 
 
 # For V2 columns, we support anything that inherits from CategoricalColumn
-# other than those in the blacklist. User-provided columns that inherit from
+# other than those in the denylist. User-provided columns that inherit from
 # CategoricalColumn may or may not be compatible; it is up to the user to
 # manage TPU compatibility for custom columns.
 _SUPPORTED_CATEGORICAL_COLUMNS_V2 = (fc_lib.CategoricalColumn,)
-_BLACKLISTED_CATEGORICAL_COLUMNS_V2 = (fc_lib.HashedCategoricalColumn,
-                                       fc_lib.BucketizedColumn,
-                                       fc_lib.CrossedColumn)
+_DENYLISTED_CATEGORICAL_COLUMNS_V2 = (fc_lib.HashedCategoricalColumn,
+                                      fc_lib.BucketizedColumn,
+                                      fc_lib.CrossedColumn)
 _SUPPORTED_CATEGORICAL_COLUMNS = (fc._IdentityCategoricalColumn,
                                   fc._VocabularyFileCategoricalColumn,
                                   fc._VocabularyListCategoricalColumn,
@@ -106,14 +103,14 @@ def embedding_column(categorical_column,
     ValueError: if `initializer` is specified but not callable.
     TypeError: if categorical_column is not a supported type.
   """
-  if isinstance(categorical_column, _BLACKLISTED_CATEGORICAL_COLUMNS_V2):
+  if isinstance(categorical_column, _DENYLISTED_CATEGORICAL_COLUMNS_V2):
     raise TypeError('categorical_column for tpu '
-                    ' embedding_column was blacklisted type %s' %
-                    type(categorical_column))
+                    ' embedding_column was '
+                    f'denylisted type {type(categorical_column)}')
   if not isinstance(categorical_column, _SUPPORTED_CATEGORICAL_COLUMNS):
     raise TypeError(
         'categorical_column for tpu '
-        ' embedding_column must be type %s, got %s.' % (' or '.join([
+        ' embedding_column must be type {}, got {}.'.format(' or '.join([
             cc.__name__ for cc in _SUPPORTED_CATEGORICAL_COLUMNS
         ]), type(categorical_column)))
   if (dimension is None) or (dimension < 1):
@@ -223,16 +220,17 @@ def shared_embedding_columns(categorical_columns,
       or 0 for a sequence column.
   """
   for categorical_column in categorical_columns:
-    if isinstance(categorical_column, _BLACKLISTED_CATEGORICAL_COLUMNS_V2):
+    if isinstance(categorical_column, _DENYLISTED_CATEGORICAL_COLUMNS_V2):
       raise TypeError('categorical_column for tpu '
-                      ' embedding_column was blacklisted type %s' %
-                      type(categorical_column))
+                      ' embedding_column was denylisted type '
+                      f'{type(categorical_column)}')
     if not isinstance(categorical_column, _SUPPORTED_CATEGORICAL_COLUMNS):
       raise TypeError(
           'categorical_column for tpu '
-          ' shared_embedding_columns must be type %s, got %s.' % (' or '.join([
-              cc.__name__ for cc in _SUPPORTED_CATEGORICAL_COLUMNS
-          ]), type(categorical_column)))
+          ' shared_embedding_columns must be type {}, got {}.'.format(
+              ' or '.join(
+                  [cc.__name__ for cc in _SUPPORTED_CATEGORICAL_COLUMNS]),
+              type(categorical_column)))
 
   if not max_sequence_lengths:
     max_sequence_lengths = [0] * len(categorical_columns)
@@ -372,10 +370,13 @@ class _TPUEmbeddingColumn(_TPUBaseEmbeddingColumn, fc._EmbeddingColumn):
               trainable=True,
               max_sequence_length=0,
               learning_rate_fn=None,
-              use_safe_embedding_lookup=True):
+              use_safe_embedding_lookup=True,
+              bypass_scope_validation=False):
     # Note, args ckpt_to_load_from, tensor_name_in_ckpt, max_norm and trainable
     # are not supported on TPU. They are solely for matching the signature of
     # __new__ of parent class fc._EmbeddingColumn.
+    del bypass_scope_validation
+    # pylint: disable=redundant-keyword-arg
     return fc._EmbeddingColumn.__new__(
         cls,
         categorical_column,
@@ -399,13 +400,18 @@ class _TPUEmbeddingColumn(_TPUBaseEmbeddingColumn, fc._EmbeddingColumn):
                trainable=True,
                max_sequence_length=0,
                learning_rate_fn=None,
-               use_safe_embedding_lookup=True):
+               use_safe_embedding_lookup=True,
+               bypass_scope_validation=False):
     _TPUBaseEmbeddingColumn.__init__(
         self,
         categorical_column,
         max_sequence_length=max_sequence_length,
         learning_rate_fn=learning_rate_fn)
     self._key = None
+    # If true, scope validation is skipped to allow the same column to be used
+    # in multiple variable scopes. By default, this is False, and we expect a
+    # 1:1 mapping between feature columns and scopes.
+    self._bypass_scope_validation = bypass_scope_validation
 
   def get_combiner(self):
     return self.combiner
@@ -448,7 +454,8 @@ class _TPUEmbeddingColumn(_TPUBaseEmbeddingColumn, fc._EmbeddingColumn):
       def host_computation():
         return fc._EmbeddingColumn._get_dense_tensor(
             self, inputs, weight_collections, trainable)
-      return tpu.outside_compilation(host_computation)
+
+      return tpu_replication.outside_compilation(host_computation)
 
     if _is_running_on_cpu():
       return fc._EmbeddingColumn._get_dense_tensor(
@@ -459,8 +466,10 @@ class _TPUEmbeddingColumn(_TPUBaseEmbeddingColumn, fc._EmbeddingColumn):
     tensor = inputs.get(self.get_feature_key_name())
 
     # Add to collection for _create_tpu_embedding_variables_and_ops
-    _record_variable_scope_and_name(self.get_embedding_var_name(),
-                                    'embedding_weights')
+    _record_variable_scope_and_name(
+        self.get_embedding_var_name(),
+        'embedding_weights',
+        bypass_scope_validation=self._bypass_scope_validation)
 
     return tensor
 
@@ -470,7 +479,8 @@ class _TPUEmbeddingColumn(_TPUBaseEmbeddingColumn, fc._EmbeddingColumn):
       def host_computation():
         return fc._EmbeddingColumn._get_sequence_dense_tensor(
             self, inputs, weight_collections, trainable)
-      return tpu.outside_compilation(host_computation)
+
+      return tpu_replication.outside_compilation(host_computation)
 
     if _is_running_on_cpu():
       return fc._EmbeddingColumn._get_sequence_dense_tensor(
@@ -484,8 +494,10 @@ class _TPUEmbeddingColumn(_TPUBaseEmbeddingColumn, fc._EmbeddingColumn):
     tensor_lengths = array_ops.squeeze(tensor_lengths, -1)
 
     # Add to collection for _create_tpu_embedding_variables_and_ops
-    _record_variable_scope_and_name(self.get_embedding_var_name(),
-                                    'embedding_weights')
+    _record_variable_scope_and_name(
+        self.get_embedding_var_name(),
+        'embedding_weights',
+        bypass_scope_validation=self._bypass_scope_validation)
 
     return fc._SequenceDenseColumn.TensorSequenceLengthPair(
         dense_tensor=tensor, sequence_length=tensor_lengths)
@@ -583,7 +595,8 @@ class _TPUSharedEmbeddingColumn(_TPUBaseEmbeddingColumn,
       def host_computation():
         return fc._SharedEmbeddingColumn._get_dense_tensor(
             self, inputs, weight_collections, trainable)
-      return tpu.outside_compilation(host_computation)
+
+      return tpu_replication.outside_compilation(host_computation)
 
     if _is_running_on_cpu():
       return fc._SharedEmbeddingColumn._get_dense_tensor(
@@ -606,7 +619,8 @@ class _TPUSharedEmbeddingColumn(_TPUBaseEmbeddingColumn,
       def host_computation():
         return fc._SharedEmbeddingColumn._get_sequence_dense_tensor(
             self, inputs, weight_collections, trainable)
-      return tpu.outside_compilation(host_computation)
+
+      return tpu_replication.outside_compilation(host_computation)
 
     if _is_running_on_cpu():
       return fc._SharedEmbeddingColumn._get_sequence_dense_tensor(
@@ -627,7 +641,8 @@ class _TPUSharedEmbeddingColumn(_TPUBaseEmbeddingColumn,
 
 def _record_variable_scope_and_name(embedding_var_name,
                                     embedding_var_name_in_fc,
-                                    is_shared_embedding=False):
+                                    is_shared_embedding=False,
+                                    bypass_scope_validation=False):
   """Add embedding variable name and scope to collection."""
   g = ops.get_default_graph()
   collection = g.get_collection_ref(_TPU_FC_TO_SCOPE)
@@ -640,8 +655,8 @@ def _record_variable_scope_and_name(embedding_var_name,
   captured_scope_name = captured_scope.name
 
   if embedding_var_name in var_def_dict:
-    if (var_def_dict[embedding_var_name][0] != captured_scope_name
-        and not is_shared_embedding):
+    if (var_def_dict[embedding_var_name][0] != captured_scope_name and
+        not is_shared_embedding and not bypass_scope_validation):
       raise ValueError(
           'For embedding var name {}, the variable scope name is different, '
           'got {}; expected {}'.format(embedding_var_name,
@@ -702,7 +717,7 @@ def split_sequence_columns(feature_columns):
     if not isinstance(column, (_TPUEmbeddingColumn, _TPUSharedEmbeddingColumn)):
       raise TypeError(
           'column must be a _TPUEmbeddingColumn or  _TPUSharedEmbeddingColumn '
-          'but got %s instead.' % (type(column)))
+          f'but got {type(column)} instead.')
     if column.is_sequence_column():
       sequence_columns.append(column)
     else:
