@@ -14,12 +14,8 @@
 # ==============================================================================
 """Tests for SessionManager."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
-
+from tensorflow.python.checkpoint import checkpoint_management
 from tensorflow.python.client import session as session_lib
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import errors
@@ -27,11 +23,12 @@ from tensorflow.python.framework import errors_impl
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import test_util
 from tensorflow.python.ops import array_ops
-from tensorflow.python.ops import control_flow_ops
+from tensorflow.python.ops import variable_scope
+from tensorflow.python.ops import variable_v1
 from tensorflow.python.ops import variables
+from tensorflow.python.ops import while_loop
 from tensorflow.python.platform import gfile
 from tensorflow.python.platform import test
-from tensorflow.python.training import checkpoint_management
 from tensorflow.python.training import saver as saver_lib
 from tensorflow.python.training import server_lib
 from tensorflow.python.training import session_manager
@@ -39,9 +36,14 @@ from tensorflow.python.training import session_manager
 
 class SessionManagerTest(test.TestCase):
 
+  @classmethod
+  def setUpClass(cls):
+    super(SessionManagerTest, cls).setUpClass()
+    variable_scope.disable_resource_variables()
+
   def testPrepareSessionSucceeds(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1([1.0, 2.0, 3.0], name="v")
+      v = variable_v1.VariableV1([1.0, 2.0, 3.0], name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       sess = sm.prepare_session(
@@ -51,7 +53,7 @@ class SessionManagerTest(test.TestCase):
   def testPrepareSessionSucceedsWithInitFeedDict(self):
     with ops.Graph().as_default():
       p = array_ops.placeholder(dtypes.float32, shape=(3,))
-      v = variables.VariableV1(p, name="v")
+      v = variable_v1.VariableV1(p, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       sess = sm.prepare_session(
@@ -62,7 +64,7 @@ class SessionManagerTest(test.TestCase):
 
   def testPrepareSessionSucceedsWithInitFn(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1([125], name="v")
+      v = variable_v1.VariableV1([125], name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       sess = sm.prepare_session(
@@ -72,8 +74,8 @@ class SessionManagerTest(test.TestCase):
   def testPrepareSessionSucceedsWithLocalInitFeedDict(self):
     with ops.Graph().as_default():
       p = array_ops.placeholder(dtypes.float32, shape=(3,))
-      v = variables.VariableV1(p, name="v",
-                               collections=[ops.GraphKeys.LOCAL_VARIABLES])
+      v = variable_v1.VariableV1(
+          p, name="v", collections=[ops.GraphKeys.LOCAL_VARIABLES])
       sm = session_manager.SessionManager(
           local_init_op=v.initializer,
           local_init_feed_dict={p: [1.0, 2.0, 3.0]},
@@ -81,7 +83,6 @@ class SessionManagerTest(test.TestCase):
       sess = sm.prepare_session("")
       self.assertAllClose([1.0, 2.0, 3.0], sess.run(v))
 
-  @test_util.run_v1_only("b/120545219")
   def testPrepareSessionFails(self):
     checkpoint_dir = os.path.join(self.get_temp_dir(), "prepare_session")
     checkpoint_dir2 = os.path.join(self.get_temp_dir(), "prepare_session2")
@@ -93,7 +94,7 @@ class SessionManagerTest(test.TestCase):
     gfile.MakeDirs(checkpoint_dir)
 
     with ops.Graph().as_default():
-      v = variables.VariableV1([1.0, 2.0, 3.0], name="v")
+      v = variable_v1.VariableV1([1.0, 2.0, 3.0], name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       saver = saver_lib.Saver({"v": v})
@@ -111,14 +112,14 @@ class SessionManagerTest(test.TestCase):
       # Renames the checkpoint directory.
       os.rename(checkpoint_dir, checkpoint_dir2)
       gfile.MakeDirs(checkpoint_dir)
-      v = variables.VariableV1([6.0, 7.0, 8.0], name="v")
+      v = variable_v1.VariableV1([6.0, 7.0, 8.0], name="v")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
       session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       saver = saver_lib.Saver({"v": v})
       # This should fail as there's no checkpoint within 2 seconds.
-      with self.assertRaisesRegexp(
+      with self.assertRaisesRegex(
           RuntimeError, "no init_op or init_fn or local_init_op was given"):
         sess = sm.prepare_session(
             "",
@@ -140,7 +141,7 @@ class SessionManagerTest(test.TestCase):
           max_wait_secs=2)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
 
   def _test_recovered_variable(self,
@@ -148,9 +149,9 @@ class SessionManagerTest(test.TestCase):
                                checkpoint_filename_with_path=None):
     # Create a new Graph and SessionManager and recover from a checkpoint.
     with ops.Graph().as_default():
-      v = variables.VariableV1(2, name="v")
+      v = variable_v1.VariableV1(2, name="v")
       with session_lib.Session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       saver = saver_lib.Saver({"v": v})
@@ -162,11 +163,10 @@ class SessionManagerTest(test.TestCase):
       self.assertTrue(initialized)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
 
-  @test_util.run_v1_only("b/120545219")
   def testRecoverSession(self):
     # Create a checkpoint.
     checkpoint_dir = os.path.join(self.get_temp_dir(), "recover_session")
@@ -177,7 +177,7 @@ class SessionManagerTest(test.TestCase):
     gfile.MakeDirs(checkpoint_dir)
 
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
+      v = variable_v1.VariableV1(1, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       saver = saver_lib.Saver({"v": v})
@@ -185,7 +185,7 @@ class SessionManagerTest(test.TestCase):
           "", saver=saver, checkpoint_dir=checkpoint_dir)
       self.assertFalse(initialized)
       sess.run(v.initializer)
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
       saver.save(sess, os.path.join(checkpoint_dir,
                                     "recover_session_checkpoint"))
     self._test_recovered_variable(checkpoint_dir=checkpoint_dir)
@@ -199,10 +199,9 @@ class SessionManagerTest(test.TestCase):
           checkpoint_filename_with_path=checkpoint_management.latest_checkpoint(
               checkpoint_dir))
 
-  @test_util.run_v1_only("b/120545219")
   def testWaitForSessionReturnsNoneAfterTimeout(self):
     with ops.Graph().as_default():
-      variables.VariableV1(1, name="v")
+      variable_v1.VariableV1(1, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           recovery_wait_secs=1)
@@ -214,7 +213,7 @@ class SessionManagerTest(test.TestCase):
   def testInitWithNoneLocalInitOpError(self):
     # Creating a SessionManager with a None local_init_op but
     # non-None ready_for_local_init_op raises ValueError
-    with self.assertRaisesRegexp(
+    with self.assertRaisesRegex(
         ValueError, "If you pass a ready_for_local_init_op "
         "you must also pass a local_init_op "):
       session_manager.SessionManager(
@@ -222,7 +221,6 @@ class SessionManagerTest(test.TestCase):
               variables.global_variables()),
           local_init_op=None)
 
-  @test_util.run_v1_only("b/120545219")
   def testRecoverSessionWithReadyForLocalInitOp(self):
     # Create a checkpoint.
     checkpoint_dir = os.path.join(self.get_temp_dir(),
@@ -234,7 +232,7 @@ class SessionManagerTest(test.TestCase):
     gfile.MakeDirs(checkpoint_dir)
 
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
+      v = variable_v1.VariableV1(1, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       saver = saver_lib.Saver({"v": v})
@@ -242,20 +240,20 @@ class SessionManagerTest(test.TestCase):
           "", saver=saver, checkpoint_dir=checkpoint_dir)
       self.assertFalse(initialized)
       sess.run(v.initializer)
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
       saver.save(sess, os.path.join(checkpoint_dir,
                                     "recover_session_checkpoint"))
     # Create a new Graph and SessionManager and recover.
     with ops.Graph().as_default():
-      v = variables.VariableV1(2, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(2, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=variables.report_uninitialized_variables(
@@ -267,16 +265,15 @@ class SessionManagerTest(test.TestCase):
       self.assertTrue(initialized)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(v))
-      self.assertEquals(1, sess.run(w))
+      self.assertEqual(1, sess.run(v))
+      self.assertEqual(1, sess.run(w))
 
-  @test_util.run_v1_only("b/120545219")
   def testRecoverSessionWithReadyForLocalInitOpFailsToReadyLocal(self):
     # We use ready_for_local_init_op=report_uninitialized_variables(),
     # which causes recover_session to not run local_init_op, and to return
@@ -293,7 +290,7 @@ class SessionManagerTest(test.TestCase):
     gfile.MakeDirs(checkpoint_dir)
 
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
+      v = variable_v1.VariableV1(1, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       saver = saver_lib.Saver({"v": v})
@@ -301,20 +298,20 @@ class SessionManagerTest(test.TestCase):
           "", saver=saver, checkpoint_dir=checkpoint_dir)
       self.assertFalse(initialized)
       sess.run(v.initializer)
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
       saver.save(sess, os.path.join(checkpoint_dir,
                                     "recover_session_checkpoint"))
     # Create a new Graph and SessionManager and recover.
     with ops.Graph().as_default():
-      v = variables.VariableV1(2, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(2, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=variables.report_uninitialized_variables(),
@@ -325,28 +322,27 @@ class SessionManagerTest(test.TestCase):
       self.assertFalse(initialized)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
       self.assertEqual(
           False,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
 
-  @test_util.run_v1_only("b/120545219")
   def testRecoverSessionNoChkptStillRunsLocalInitOp(self):
     # This test checks for backwards compatibility.
     # In particular, we continue to ensure that recover_session will execute
     # local_init_op exactly once, regardless of whether the session was
     # successfully recovered.
     with ops.Graph().as_default():
-      w = variables.VariableV1(
+      w = variable_v1.VariableV1(
           1,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=None,
@@ -358,11 +354,10 @@ class SessionManagerTest(test.TestCase):
       self.assertFalse(initialized)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(w))
+      self.assertEqual(1, sess.run(w))
 
-  @test_util.run_v1_only("b/120545219")
   def testRecoverSessionFailsStillRunsLocalInitOp(self):
     # Create a checkpoint.
     checkpoint_dir = os.path.join(
@@ -376,15 +371,15 @@ class SessionManagerTest(test.TestCase):
 
     # Create a new Graph and SessionManager and recover.
     with ops.Graph().as_default():
-      v = variables.VariableV1(2, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(2, name="v")
+      w = variable_v1.VariableV1(
           1,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=None,
@@ -398,20 +393,19 @@ class SessionManagerTest(test.TestCase):
       self.assertFalse(initialized)
       self.assertEqual(
           False,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(w))
+      self.assertEqual(1, sess.run(w))
 
-  @test_util.run_v1_only("b/120545219")
   def testWaitForSessionLocalInit(self):
     server = server_lib.Server.create_local_server()
     with ops.Graph().as_default() as graph:
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
@@ -430,19 +424,19 @@ class SessionManagerTest(test.TestCase):
       sess = sm.wait_for_session(server.target, max_wait_secs=3)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(v))
-      self.assertEquals(1, sess.run(w))
+      self.assertEqual(1, sess.run(v))
+      self.assertEqual(1, sess.run(w))
 
   def testWaitForSessionWithReadyForLocalInitOpFailsToReadyLocal(self):
     with ops.Graph().as_default() as graph:
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
@@ -458,11 +452,11 @@ class SessionManagerTest(test.TestCase):
         # because of overly restrictive ready_for_local_init_op
         sm.wait_for_session("", max_wait_secs=3)
 
-  @test_util.run_v1_only("b/120545219")
+  @test_util.run_v1_only("Requires TF V1 variable behavior.")
   def testWaitForSessionInsufficientReadyForLocalInitCheck(self):
     with ops.Graph().as_default() as graph:
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
@@ -472,28 +466,27 @@ class SessionManagerTest(test.TestCase):
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=None,
           local_init_op=w.initializer)
-    with self.assertRaisesRegexp(errors_impl.DeadlineExceededError,
-                                 "Session was not ready after waiting.*"):
+    with self.assertRaisesRegex(errors_impl.DeadlineExceededError,
+                                "Session was not ready after waiting.*"):
       sm.wait_for_session("", max_wait_secs=3)
 
-  @test_util.run_v1_only("b/120545219")
   def testPrepareSessionWithReadyForLocalInitOp(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
-      x = variables.VariableV1(
+      x = variable_v1.VariableV1(
           3 * v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="x")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
-        self.assertEqual(False, variables.is_variable_initialized(x).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(x).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=variables.report_uninitialized_variables(
@@ -502,187 +495,194 @@ class SessionManagerTest(test.TestCase):
       sess = sm2.prepare_session("", init_op=v.initializer)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("x:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(v))
-      self.assertEquals(1, sess.run(w))
-      self.assertEquals(3, sess.run(x))
+      self.assertEqual(1, sess.run(v))
+      self.assertEqual(1, sess.run(w))
+      self.assertEqual(3, sess.run(x))
 
-  @test_util.run_v1_only("b/120545219")
+  @test_util.run_v1_only("Requires TF V1 variable behavior.")
   def testPrepareSessionWithPartialInitOp(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
-      x = variables.VariableV1(
+      x = variable_v1.VariableV1(
           3 * v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="x")
       # TODO(b/70206927): Use ResourceVariables once they are handled properly.
-      v_res = variables.VariableV1(1, name="v_res")
-      w_res = variables.VariableV1(
+      v_res = variable_v1.VariableV1(1, name="v_res")
+      w_res = variable_v1.VariableV1(
           v_res,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w_res")
-      x_res = variables.VariableV1(
+      x_res = variable_v1.VariableV1(
           3 * v_res,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="x_res")
 
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
-        self.assertEqual(False, variables.is_variable_initialized(x).eval())
-        self.assertEqual(False, variables.is_variable_initialized(v_res).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w_res).eval())
-        self.assertEqual(False, variables.is_variable_initialized(x_res).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(x).eval())
+        self.assertEqual(False,
+                         variable_v1.is_variable_initialized(v_res).eval())
+        self.assertEqual(False,
+                         variable_v1.is_variable_initialized(w_res).eval())
+        self.assertEqual(False,
+                         variable_v1.is_variable_initialized(x_res).eval())
       sm2 = session_manager.SessionManager(local_init_op=[
           w.initializer, x.initializer, w_res.initializer, x_res.initializer
       ])
       sess = sm2.prepare_session("", init_op=None)
       self.assertEqual(
           False,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("x:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(w))
-      self.assertEquals(3, sess.run(x))
+      self.assertEqual(1, sess.run(w))
+      self.assertEqual(3, sess.run(x))
       self.assertEqual(
           False,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v_res:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("w_res:0")).eval(session=sess))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("x_res:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(w_res))
-      self.assertEquals(3, sess.run(x_res))
+      self.assertEqual(1, sess.run(w_res))
+      self.assertEqual(3, sess.run(x_res))
 
-  @test_util.run_v1_only("b/120545219")
   def testPrepareSessionWithCyclicInitializer(self):
     # Regression test. Previously Variable._build_initializer_expr would enter
     # into an infinite recursion when the variable's initial_value involved
     # cyclic dependencies.
     with ops.Graph().as_default():
-      i = control_flow_ops.while_loop(lambda i: i < 1, lambda i: i + 1, [0])
-      v = variables.VariableV1(array_ops.identity(i), name="v")
+      i = while_loop.while_loop(lambda i: i < 1, lambda i: i + 1, [0])
+      v = variable_v1.VariableV1(array_ops.identity(i), name="v")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
       sm = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
       sess = sm.prepare_session("", init_op=v.initializer)
       self.assertEqual(1, sess.run(v))
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
 
   def testPrepareSessionDidNotInitLocalVariable(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
-      with self.assertRaisesRegexp(
-          RuntimeError, "Init operations did not make model ready.*"):
+      with self.assertRaisesRegex(RuntimeError,
+                                  "Init operations did not make model ready.*"):
         sm2.prepare_session("", init_op=v.initializer)
 
   def testPrepareSessionDidNotInitLocalVariableList(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables())
-      with self.assertRaisesRegexp(RuntimeError,
-                                   "Init operations did not make model ready"):
+      with self.assertRaisesRegex(RuntimeError,
+                                  "Init operations did not make model ready"):
         sm2.prepare_session("", init_op=[v.initializer])
 
   def testPrepareSessionWithReadyNotReadyForLocal(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=variables.report_uninitialized_variables(
               variables.global_variables()),
           local_init_op=w.initializer)
-      with self.assertRaisesRegexp(
+      with self.assertRaisesRegex(
           RuntimeError,
           "Init operations did not make model ready for local_init"):
         sm2.prepare_session("", init_op=None)
 
-  @test_util.run_v1_only("b/120545219")
+  @test_util.run_v1_only("Requires TF V1 variable behavior.")
   def testPrepareSessionWithInsufficientReadyForLocalInitCheck(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
-      w = variables.VariableV1(
+      v = variable_v1.VariableV1(1, name="v")
+      w = variable_v1.VariableV1(
           v,
           trainable=False,
           collections=[ops.GraphKeys.LOCAL_VARIABLES],
           name="w")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
-        self.assertEqual(False, variables.is_variable_initialized(w).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(w).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.report_uninitialized_variables(),
           ready_for_local_init_op=None,
           local_init_op=w.initializer)
-    with self.assertRaisesRegexp(RuntimeError,
-                                 "Init operations did not make model ready.*"):
+    with self.assertRaisesRegex(RuntimeError,
+                                "Init operations did not make model ready.*"):
       sm2.prepare_session("", init_op=None)
 
 
 class ObsoleteSessionManagerTest(test.TestCase):
 
+  @classmethod
+  def setUpClass(cls):
+    super(ObsoleteSessionManagerTest, cls).setUpClass()
+    variable_scope.disable_resource_variables()
+
   def testPrepareSessionSucceeds(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1([1.0, 2.0, 3.0], name="v")
+      v = variable_v1.VariableV1([1.0, 2.0, 3.0], name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       sess = sm.prepare_session(
@@ -692,7 +692,7 @@ class ObsoleteSessionManagerTest(test.TestCase):
   def testPrepareSessionSucceedsWithInitFeedDict(self):
     with ops.Graph().as_default():
       p = array_ops.placeholder(dtypes.float32, shape=(3,))
-      v = variables.VariableV1(p, name="v")
+      v = variable_v1.VariableV1(p, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       sess = sm.prepare_session(
@@ -703,14 +703,13 @@ class ObsoleteSessionManagerTest(test.TestCase):
 
   def testPrepareSessionSucceedsWithInitFn(self):
     with ops.Graph().as_default():
-      v = variables.VariableV1([125], name="v")
+      v = variable_v1.VariableV1([125], name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       sess = sm.prepare_session(
           "", init_fn=lambda sess: sess.run(v.initializer))
       self.assertAllClose([125], sess.run(v))
 
-  @test_util.run_v1_only("b/120545219")
   def testPrepareSessionFails(self):
     checkpoint_dir = os.path.join(self.get_temp_dir(), "prepare_session")
     checkpoint_dir2 = os.path.join(self.get_temp_dir(), "prepare_session2")
@@ -722,7 +721,7 @@ class ObsoleteSessionManagerTest(test.TestCase):
     gfile.MakeDirs(checkpoint_dir)
 
     with ops.Graph().as_default():
-      v = variables.VariableV1([1.0, 2.0, 3.0], name="v")
+      v = variable_v1.VariableV1([1.0, 2.0, 3.0], name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       saver = saver_lib.Saver({"v": v})
@@ -740,14 +739,14 @@ class ObsoleteSessionManagerTest(test.TestCase):
       # Renames the checkpoint directory.
       os.rename(checkpoint_dir, checkpoint_dir2)
       gfile.MakeDirs(checkpoint_dir)
-      v = variables.VariableV1([6.0, 7.0, 8.0], name="v")
+      v = variable_v1.VariableV1([6.0, 7.0, 8.0], name="v")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
       session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       saver = saver_lib.Saver({"v": v})
       # This should fail as there's no checkpoint within 2 seconds.
-      with self.assertRaisesRegexp(
+      with self.assertRaisesRegex(
           RuntimeError, "no init_op or init_fn or local_init_op was given"):
         sess = sm.prepare_session(
             "",
@@ -769,10 +768,9 @@ class ObsoleteSessionManagerTest(test.TestCase):
           max_wait_secs=2)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
 
-  @test_util.run_v1_only("b/120545219")
   def testRecoverSession(self):
     # Create a checkpoint.
     checkpoint_dir = os.path.join(self.get_temp_dir(), "recover_session")
@@ -783,7 +781,7 @@ class ObsoleteSessionManagerTest(test.TestCase):
     gfile.MakeDirs(checkpoint_dir)
 
     with ops.Graph().as_default():
-      v = variables.VariableV1(1, name="v")
+      v = variable_v1.VariableV1(1, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       saver = saver_lib.Saver({"v": v})
@@ -791,14 +789,14 @@ class ObsoleteSessionManagerTest(test.TestCase):
           "", saver=saver, checkpoint_dir=checkpoint_dir)
       self.assertFalse(initialized)
       sess.run(v.initializer)
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
       saver.save(sess, os.path.join(checkpoint_dir,
                                     "recover_session_checkpoint"))
     # Create a new Graph and SessionManager and recover.
     with ops.Graph().as_default():
-      v = variables.VariableV1(2, name="v")
+      v = variable_v1.VariableV1(2, name="v")
       with self.cached_session():
-        self.assertEqual(False, variables.is_variable_initialized(v).eval())
+        self.assertEqual(False, variable_v1.is_variable_initialized(v).eval())
       sm2 = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized())
       saver = saver_lib.Saver({"v": v})
@@ -807,14 +805,13 @@ class ObsoleteSessionManagerTest(test.TestCase):
       self.assertTrue(initialized)
       self.assertEqual(
           True,
-          variables.is_variable_initialized(
+          variable_v1.is_variable_initialized(
               sess.graph.get_tensor_by_name("v:0")).eval(session=sess))
-      self.assertEquals(1, sess.run(v))
+      self.assertEqual(1, sess.run(v))
 
-  @test_util.run_v1_only("b/120545219")
   def testWaitForSessionReturnsNoneAfterTimeout(self):
     with ops.Graph().as_default():
-      variables.VariableV1(1, name="v")
+      variable_v1.VariableV1(1, name="v")
       sm = session_manager.SessionManager(
           ready_op=variables.assert_variables_initialized(),
           recovery_wait_secs=1)
